#line 2 "scan.c"
/*-------------------------------------------------------------------------
 *
 * scan.l
 *	  lexical scanner for PostgreSQL
 *
 * NOTE NOTE NOTE:
 *
 * The rules in this file must be kept in sync with src/fe_utils/psqlscan.l!
 *
 * The rules are designed so that the scanner never has to backtrack,
 * in the sense that there is always a rule that can match the input
 * consumed so far (the rule action may internally throw back some input
 * with yyless(), however).  As explained in the flex manual, this makes
 * for a useful speed increase --- about a third faster than a plain -CF
 * lexer, in simple testing.  The extra complexity is mostly in the rules
 * for handling float numbers and continued string literals.  If you change
 * the lexical rules, verify that you haven't broken the no-backtrack
 * property by running flex with the "-b" option and checking that the
 * resulting "lex.backup" file says that no backing up is needed.  (As of
 * Postgres 9.2, this check is made automatically by the Makefile.)
 *
 *
 * Portions Copyright (c) 1996-2017, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * IDENTIFICATION
 *	  src/backend/parser/scan.l
 *
 *-------------------------------------------------------------------------
 */
#include "postgres.h"

#include <ctype.h>
#include <unistd.h>

#include "parser/gramparse.h"
#include "parser/parser.h"		/* only needed for GUC variables */
#include "parser/scansup.h"
#include "mb/pg_wchar.h"

#ifdef _PG_ORCL_
#include "utils/guc.h"
#endif

#line 47 "scan.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 6
#define YY_FLEX_SUBMINOR_VERSION 1
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

/* TODO: this is always defined, so inline it */
#define yyconst const

#if defined(__GNUC__) && __GNUC__ >= 3
#define yynoreturn __attribute__((__noreturn__))
#else
#define yynoreturn
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* An opaque pointer. */
#ifndef YY_TYPEDEF_YY_SCANNER_T
#define YY_TYPEDEF_YY_SCANNER_T
typedef void* yyscan_t;
#endif

/* For convenience, these vars (plus the bison vars far below)
   are macros in the reentrant scanner. */
#define yyin yyg->yyin_r
#define yyout yyg->yyout_r
#define yyextra yyg->yyextra_r
#define yyleng yyg->yyleng_r
#define yytext yyg->yytext_r
#define yylineno (YY_CURRENT_BUFFER_LVALUE->yy_bs_lineno)
#define yycolumn (YY_CURRENT_BUFFER_LVALUE->yy_bs_column)
#define yy_flex_debug yyg->yy_flex_debug_r

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yyg->yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yyg->yy_start - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE core_yyrestart(yyin ,yyscanner )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    #define YY_LINENO_REWIND_TO(ptr)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        yy_size_t yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = yyg->yy_hold_char; \
		YY_RESTORE_YY_MORE_OFFSET \
		yyg->yy_c_buf_p = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, yyg->yytext_ptr , yyscanner )

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	int yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via core_yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( yyg->yy_buffer_stack \
                          ? yyg->yy_buffer_stack[yyg->yy_buffer_stack_top] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE yyg->yy_buffer_stack[yyg->yy_buffer_stack_top]

void core_yyrestart (FILE *input_file ,yyscan_t yyscanner );
void core_yy_switch_to_buffer (YY_BUFFER_STATE new_buffer ,yyscan_t yyscanner );
YY_BUFFER_STATE core_yy_create_buffer (FILE *file,int size ,yyscan_t yyscanner );
void core_yy_delete_buffer (YY_BUFFER_STATE b ,yyscan_t yyscanner );
void core_yy_flush_buffer (YY_BUFFER_STATE b ,yyscan_t yyscanner );
void core_yypush_buffer_state (YY_BUFFER_STATE new_buffer ,yyscan_t yyscanner );
void core_yypop_buffer_state (yyscan_t yyscanner );

static void core_yyensure_buffer_stack (yyscan_t yyscanner );
static void core_yy_load_buffer_state (yyscan_t yyscanner );
static void core_yy_init_buffer (YY_BUFFER_STATE b,FILE *file ,yyscan_t yyscanner );

#define YY_FLUSH_BUFFER core_yy_flush_buffer(YY_CURRENT_BUFFER ,yyscanner)

YY_BUFFER_STATE core_yy_scan_buffer (char *base,yy_size_t size ,yyscan_t yyscanner );
YY_BUFFER_STATE core_yy_scan_string (yyconst char *yy_str ,yyscan_t yyscanner );
YY_BUFFER_STATE core_yy_scan_bytes (yyconst char *bytes,int len ,yyscan_t yyscanner );

void *core_yyalloc (yy_size_t ,yyscan_t yyscanner );
void *core_yyrealloc (void *,yy_size_t ,yyscan_t yyscanner );
void core_yyfree (void * ,yyscan_t yyscanner );

#define yy_new_buffer core_yy_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        core_yyensure_buffer_stack (yyscanner); \
		YY_CURRENT_BUFFER_LVALUE =    \
            core_yy_create_buffer(yyin,YY_BUF_SIZE ,yyscanner); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        core_yyensure_buffer_stack (yyscanner); \
		YY_CURRENT_BUFFER_LVALUE =    \
            core_yy_create_buffer(yyin,YY_BUF_SIZE ,yyscanner); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

#define core_yywrap(yyscanner) (/*CONSTCOND*/1)
#define YY_SKIP_YYWRAP

typedef unsigned char YY_CHAR;

typedef yyconst struct yy_trans_info *yy_state_type;

#define yytext_ptr yytext_r

static yy_state_type yy_get_previous_state (yyscan_t yyscanner );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  ,yyscan_t yyscanner);
static int yy_get_next_buffer (yyscan_t yyscanner );
static void yynoreturn yy_fatal_error (yyconst char* msg ,yyscan_t yyscanner );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yyg->yytext_ptr = yy_bp; \
	yyleng = (int) (yy_cp - yy_bp); \
	yyg->yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
	yyg->yy_c_buf_p = yy_cp;

#define YY_NUM_RULES 80
#define YY_END_OF_BUFFER 81
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst struct yy_trans_info yy_transition[37330] =
    {
 {   0,   0 }, {   0,37074 }, {   0,   0 }, {   0,37072 }, {   1,6708 },
 {   2,6708 }, {   3,6708 }, {   4,6708 }, {   5,6708 }, {   6,6708 },
 {   7,6708 }, {   8,6708 }, {   9,6710 }, {  10,6715 }, {  11,6708 },
 {  12,6710 }, {  13,6710 }, {  14,6708 }, {  15,6708 }, {  16,6708 },
 {  17,6708 }, {  18,6708 }, {  19,6708 }, {  20,6708 }, {  21,6708 },
 {  22,6708 }, {  23,6708 }, {  24,6708 }, {  25,6708 }, {  26,6708 },
 {  27,6708 }, {  28,6708 }, {  29,6708 }, {  30,6708 }, {  31,6708 },
 {  32,6710 }, {  33,6717 }, {  34,6712 }, {  35,6757 }, {  36,6823 },
 {  37,7080 }, {  38,6757 }, {  39,6730 }, {  40,6732 }, {  41,6732 },
 {  42,7080 }, {  43,7080 }, {  44,6732 }, {  45,7091 }, {  46,7110 },

 {  47,7181 }, {  48,7213 }, {  49,7213 }, {  50,7213 }, {  51,7213 },
 {  52,7213 }, {  53,7213 }, {  54,7213 }, {  55,7213 }, {  56,7213 },
 {  57,7213 }, {  58,6735 }, {  59,6732 }, {  60,7337 }, {  61,7348 },
 {  62,7415 }, {  63,6757 }, {  64,6757 }, {  65,7447 }, {  66,7704 },
 {  67,7447 }, {  68,7447 }, {  69,7961 }, {  70,7447 }, {  71,7447 },
 {  72,7447 }, {  73,7447 }, {  74,7447 }, {  75,7447 }, {  76,7447 },
 {  77,7447 }, {  78,8218 }, {  79,7447 }, {  80,7447 }, {  81,7447 },
 {  82,7447 }, {  83,7447 }, {  84,7447 }, {  85,8475 }, {  86,7447 },
 {  87,7447 }, {  88,8732 }, {  89,7447 }, {  90,7447 }, {  91,6732 },
 {  92,6708 }, {  93,6732 }, {  94,7080 }, {  95,7447 }, {  96,6757 },

 {  97,7447 }, {  98,7704 }, {  99,7447 }, { 100,7447 }, { 101,7961 },
 { 102,7447 }, { 103,7447 }, { 104,7447 }, { 105,7447 }, { 106,7447 },
 { 107,7447 }, { 108,7447 }, { 109,7447 }, { 110,8218 }, { 111,7447 },
 { 112,7447 }, { 113,7447 }, { 114,7447 }, { 115,7447 }, { 116,7447 },
 { 117,8475 }, { 118,7447 }, { 119,7447 }, { 120,8732 }, { 121,7447 },
 { 122,7447 }, { 123,6708 }, { 124,6757 }, { 125,6708 }, { 126,6757 },
 { 127,6708 }, { 128,7447 }, { 129,7447 }, { 130,7447 }, { 131,7447 },
 { 132,7447 }, { 133,7447 }, { 134,7447 }, { 135,7447 }, { 136,7447 },
 { 137,7447 }, { 138,7447 }, { 139,7447 }, { 140,7447 }, { 141,7447 },
 { 142,7447 }, { 143,7447 }, { 144,7447 }, { 145,7447 }, { 146,7447 },

 { 147,7447 }, { 148,7447 }, { 149,7447 }, { 150,7447 }, { 151,7447 },
 { 152,7447 }, { 153,7447 }, { 154,7447 }, { 155,7447 }, { 156,7447 },
 { 157,7447 }, { 158,7447 }, { 159,7447 }, { 160,7447 }, { 161,7447 },
 { 162,7447 }, { 163,7447 }, { 164,7447 }, { 165,7447 }, { 166,7447 },
 { 167,7447 }, { 168,7447 }, { 169,7447 }, { 170,7447 }, { 171,7447 },
 { 172,7447 }, { 173,7447 }, { 174,7447 }, { 175,7447 }, { 176,7447 },
 { 177,7447 }, { 178,7447 }, { 179,7447 }, { 180,7447 }, { 181,7447 },
 { 182,7447 }, { 183,7447 }, { 184,7447 }, { 185,7447 }, { 186,7447 },
 { 187,7447 }, { 188,7447 }, { 189,7447 }, { 190,7447 }, { 191,7447 },
 { 192,7447 }, { 193,7447 }, { 194,7447 }, { 195,7447 }, { 196,7447 },

 { 197,7447 }, { 198,7447 }, { 199,7447 }, { 200,7447 }, { 201,7447 },
 { 202,7447 }, { 203,7447 }, { 204,7447 }, { 205,7447 }, { 206,7447 },
 { 207,7447 }, { 208,7447 }, { 209,7447 }, { 210,7447 }, { 211,7447 },
 { 212,7447 }, { 213,7447 }, { 214,7447 }, { 215,7447 }, { 216,7447 },
 { 217,7447 }, { 218,7447 }, { 219,7447 }, { 220,7447 }, { 221,7447 },
 { 222,7447 }, { 223,7447 }, { 224,7447 }, { 225,7447 }, { 226,7447 },
 { 227,7447 }, { 228,7447 }, { 229,7447 }, { 230,7447 }, { 231,7447 },
 { 232,7447 }, { 233,7447 }, { 234,7447 }, { 235,7447 }, { 236,7447 },
 { 237,7447 }, { 238,7447 }, { 239,7447 }, { 240,7447 }, { 241,7447 },
 { 242,7447 }, { 243,7447 }, { 244,7447 }, { 245,7447 }, { 246,7447 },

 { 247,7447 }, { 248,7447 }, { 249,7447 }, { 250,7447 }, { 251,7447 },
 { 252,7447 }, { 253,7447 }, { 254,7447 }, { 255,7447 }, { 256,6708 },
 {   0,   0 }, {   0,36814 }, {   1,6450 }, {   2,6450 }, {   3,6450 },
 {   4,6450 }, {   5,6450 }, {   6,6450 }, {   7,6450 }, {   8,6450 },
 {   9,6452 }, {  10,6457 }, {  11,6450 }, {  12,6452 }, {  13,6452 },
 {  14,6450 }, {  15,6450 }, {  16,6450 }, {  17,6450 }, {  18,6450 },
 {  19,6450 }, {  20,6450 }, {  21,6450 }, {  22,6450 }, {  23,6450 },
 {  24,6450 }, {  25,6450 }, {  26,6450 }, {  27,6450 }, {  28,6450 },
 {  29,6450 }, {  30,6450 }, {  31,6450 }, {  32,6452 }, {  33,6459 },
 {  34,6454 }, {  35,6499 }, {  36,6565 }, {  37,6822 }, {  38,6499 },

 {  39,6472 }, {  40,6474 }, {  41,6474 }, {  42,6822 }, {  43,6822 },
 {  44,6474 }, {  45,6833 }, {  46,6852 }, {  47,6923 }, {  48,6955 },
 {  49,6955 }, {  50,6955 }, {  51,6955 }, {  52,6955 }, {  53,6955 },
 {  54,6955 }, {  55,6955 }, {  56,6955 }, {  57,6955 }, {  58,6477 },
 {  59,6474 }, {  60,7079 }, {  61,7090 }, {  62,7157 }, {  63,6499 },
 {  64,6499 }, {  65,7189 }, {  66,7446 }, {  67,7189 }, {  68,7189 },
 {  69,7703 }, {  70,7189 }, {  71,7189 }, {  72,7189 }, {  73,7189 },
 {  74,7189 }, {  75,7189 }, {  76,7189 }, {  77,7189 }, {  78,7960 },
 {  79,7189 }, {  80,7189 }, {  81,7189 }, {  82,7189 }, {  83,7189 },
 {  84,7189 }, {  85,8217 }, {  86,7189 }, {  87,7189 }, {  88,8474 },

 {  89,7189 }, {  90,7189 }, {  91,6474 }, {  92,6450 }, {  93,6474 },
 {  94,6822 }, {  95,7189 }, {  96,6499 }, {  97,7189 }, {  98,7446 },
 {  99,7189 }, { 100,7189 }, { 101,7703 }, { 102,7189 }, { 103,7189 },
 { 104,7189 }, { 105,7189 }, { 106,7189 }, { 107,7189 }, { 108,7189 },
 { 109,7189 }, { 110,7960 }, { 111,7189 }, { 112,7189 }, { 113,7189 },
 { 114,7189 }, { 115,7189 }, { 116,7189 }, { 117,8217 }, { 118,7189 },
 { 119,7189 }, { 120,8474 }, { 121,7189 }, { 122,7189 }, { 123,6450 },
 { 124,6499 }, { 125,6450 }, { 126,6499 }, { 127,6450 }, { 128,7189 },
 { 129,7189 }, { 130,7189 }, { 131,7189 }, { 132,7189 }, { 133,7189 },
 { 134,7189 }, { 135,7189 }, { 136,7189 }, { 137,7189 }, { 138,7189 },

 { 139,7189 }, { 140,7189 }, { 141,7189 }, { 142,7189 }, { 143,7189 },
 { 144,7189 }, { 145,7189 }, { 146,7189 }, { 147,7189 }, { 148,7189 },
 { 149,7189 }, { 150,7189 }, { 151,7189 }, { 152,7189 }, { 153,7189 },
 { 154,7189 }, { 155,7189 }, { 156,7189 }, { 157,7189 }, { 158,7189 },
 { 159,7189 }, { 160,7189 }, { 161,7189 }, { 162,7189 }, { 163,7189 },
 { 164,7189 }, { 165,7189 }, { 166,7189 }, { 167,7189 }, { 168,7189 },
 { 169,7189 }, { 170,7189 }, { 171,7189 }, { 172,7189 }, { 173,7189 },
 { 174,7189 }, { 175,7189 }, { 176,7189 }, { 177,7189 }, { 178,7189 },
 { 179,7189 }, { 180,7189 }, { 181,7189 }, { 182,7189 }, { 183,7189 },
 { 184,7189 }, { 185,7189 }, { 186,7189 }, { 187,7189 }, { 188,7189 },

 { 189,7189 }, { 190,7189 }, { 191,7189 }, { 192,7189 }, { 193,7189 },
 { 194,7189 }, { 195,7189 }, { 196,7189 }, { 197,7189 }, { 198,7189 },
 { 199,7189 }, { 200,7189 }, { 201,7189 }, { 202,7189 }, { 203,7189 },
 { 204,7189 }, { 205,7189 }, { 206,7189 }, { 207,7189 }, { 208,7189 },
 { 209,7189 }, { 210,7189 }, { 211,7189 }, { 212,7189 }, { 213,7189 },
 { 214,7189 }, { 215,7189 }, { 216,7189 }, { 217,7189 }, { 218,7189 },
 { 219,7189 }, { 220,7189 }, { 221,7189 }, { 222,7189 }, { 223,7189 },
 { 224,7189 }, { 225,7189 }, { 226,7189 }, { 227,7189 }, { 228,7189 },
 { 229,7189 }, { 230,7189 }, { 231,7189 }, { 232,7189 }, { 233,7189 },
 { 234,7189 }, { 235,7189 }, { 236,7189 }, { 237,7189 }, { 238,7189 },

 { 239,7189 }, { 240,7189 }, { 241,7189 }, { 242,7189 }, { 243,7189 },
 { 244,7189 }, { 245,7189 }, { 246,7189 }, { 247,7189 }, { 248,7189 },
 { 249,7189 }, { 250,7189 }, { 251,7189 }, { 252,7189 }, { 253,7189 },
 { 254,7189 }, { 255,7189 }, { 256,6450 }, {   0,  12 }, {   0,36556 },
 {   1,8473 }, {   2,8473 }, {   3,8473 }, {   4,8473 }, {   5,8473 },
 {   6,8473 }, {   7,8473 }, {   8,8473 }, {   9,8473 }, {  10,8473 },
 {  11,8473 }, {  12,8473 }, {  13,8473 }, {  14,8473 }, {  15,8473 },
 {  16,8473 }, {  17,8473 }, {  18,8473 }, {  19,8473 }, {  20,8473 },
 {  21,8473 }, {  22,8473 }, {  23,8473 }, {  24,8473 }, {  25,8473 },
 {  26,8473 }, {  27,8473 }, {  28,8473 }, {  29,8473 }, {  30,8473 },

 {  31,8473 }, {  32,8473 }, {  33,8473 }, {  34,8473 }, {  35,8473 },
 {  36,8473 }, {  37,8473 }, {  38,8473 }, {  39,8731 }, {  40,8473 },
 {  41,8473 }, {  42,8473 }, {  43,8473 }, {  44,8473 }, {  45,8473 },
 {  46,8473 }, {  47,8473 }, {  48,8473 }, {  49,8473 }, {  50,8473 },
 {  51,8473 }, {  52,8473 }, {  53,8473 }, {  54,8473 }, {  55,8473 },
 {  56,8473 }, {  57,8473 }, {  58,8473 }, {  59,8473 }, {  60,8473 },
 {  61,8473 }, {  62,8473 }, {  63,8473 }, {  64,8473 }, {  65,8473 },
 {  66,8473 }, {  67,8473 }, {  68,8473 }, {  69,8473 }, {  70,8473 },
 {  71,8473 }, {  72,8473 }, {  73,8473 }, {  74,8473 }, {  75,8473 },
 {  76,8473 }, {  77,8473 }, {  78,8473 }, {  79,8473 }, {  80,8473 },

 {  81,8473 }, {  82,8473 }, {  83,8473 }, {  84,8473 }, {  85,8473 },
 {  86,8473 }, {  87,8473 }, {  88,8473 }, {  89,8473 }, {  90,8473 },
 {  91,8473 }, {  92,8473 }, {  93,8473 }, {  94,8473 }, {  95,8473 },
 {  96,8473 }, {  97,8473 }, {  98,8473 }, {  99,8473 }, { 100,8473 },
 { 101,8473 }, { 102,8473 }, { 103,8473 }, { 104,8473 }, { 105,8473 },
 { 106,8473 }, { 107,8473 }, { 108,8473 }, { 109,8473 }, { 110,8473 },
 { 111,8473 }, { 112,8473 }, { 113,8473 }, { 114,8473 }, { 115,8473 },
 { 116,8473 }, { 117,8473 }, { 118,8473 }, { 119,8473 }, { 120,8473 },
 { 121,8473 }, { 122,8473 }, { 123,8473 }, { 124,8473 }, { 125,8473 },
 { 126,8473 }, { 127,8473 }, { 128,8473 }, { 129,8473 }, { 130,8473 },

 { 131,8473 }, { 132,8473 }, { 133,8473 }, { 134,8473 }, { 135,8473 },
 { 136,8473 }, { 137,8473 }, { 138,8473 }, { 139,8473 }, { 140,8473 },
 { 141,8473 }, { 142,8473 }, { 143,8473 }, { 144,8473 }, { 145,8473 },
 { 146,8473 }, { 147,8473 }, { 148,8473 }, { 149,8473 }, { 150,8473 },
 { 151,8473 }, { 152,8473 }, { 153,8473 }, { 154,8473 }, { 155,8473 },
 { 156,8473 }, { 157,8473 }, { 158,8473 }, { 159,8473 }, { 160,8473 },
 { 161,8473 }, { 162,8473 }, { 163,8473 }, { 164,8473 }, { 165,8473 },
 { 166,8473 }, { 167,8473 }, { 168,8473 }, { 169,8473 }, { 170,8473 },
 { 171,8473 }, { 172,8473 }, { 173,8473 }, { 174,8473 }, { 175,8473 },
 { 176,8473 }, { 177,8473 }, { 178,8473 }, { 179,8473 }, { 180,8473 },

 { 181,8473 }, { 182,8473 }, { 183,8473 }, { 184,8473 }, { 185,8473 },
 { 186,8473 }, { 187,8473 }, { 188,8473 }, { 189,8473 }, { 190,8473 },
 { 191,8473 }, { 192,8473 }, { 193,8473 }, { 194,8473 }, { 195,8473 },
 { 196,8473 }, { 197,8473 }, { 198,8473 }, { 199,8473 }, { 200,8473 },
 { 201,8473 }, { 202,8473 }, { 203,8473 }, { 204,8473 }, { 205,8473 },
 { 206,8473 }, { 207,8473 }, { 208,8473 }, { 209,8473 }, { 210,8473 },
 { 211,8473 }, { 212,8473 }, { 213,8473 }, { 214,8473 }, { 215,8473 },
 { 216,8473 }, { 217,8473 }, { 218,8473 }, { 219,8473 }, { 220,8473 },
 { 221,8473 }, { 222,8473 }, { 223,8473 }, { 224,8473 }, { 225,8473 },
 { 226,8473 }, { 227,8473 }, { 228,8473 }, { 229,8473 }, { 230,8473 },

 { 231,8473 }, { 232,8473 }, { 233,8473 }, { 234,8473 }, { 235,8473 },
 { 236,8473 }, { 237,8473 }, { 238,8473 }, { 239,8473 }, { 240,8473 },
 { 241,8473 }, { 242,8473 }, { 243,8473 }, { 244,8473 }, { 245,8473 },
 { 246,8473 }, { 247,8473 }, { 248,8473 }, { 249,8473 }, { 250,8473 },
 { 251,8473 }, { 252,8473 }, { 253,8473 }, { 254,8473 }, { 255,8473 },
 { 256,8473 }, {   0,  12 }, {   0,36298 }, {   1,8215 }, {   2,8215 },
 {   3,8215 }, {   4,8215 }, {   5,8215 }, {   6,8215 }, {   7,8215 },
 {   8,8215 }, {   9,8215 }, {  10,8215 }, {  11,8215 }, {  12,8215 },
 {  13,8215 }, {  14,8215 }, {  15,8215 }, {  16,8215 }, {  17,8215 },
 {  18,8215 }, {  19,8215 }, {  20,8215 }, {  21,8215 }, {  22,8215 },

 {  23,8215 }, {  24,8215 }, {  25,8215 }, {  26,8215 }, {  27,8215 },
 {  28,8215 }, {  29,8215 }, {  30,8215 }, {  31,8215 }, {  32,8215 },
 {  33,8215 }, {  34,8215 }, {  35,8215 }, {  36,8215 }, {  37,8215 },
 {  38,8215 }, {  39,8473 }, {  40,8215 }, {  41,8215 }, {  42,8215 },
 {  43,8215 }, {  44,8215 }, {  45,8215 }, {  46,8215 }, {  47,8215 },
 {  48,8215 }, {  49,8215 }, {  50,8215 }, {  51,8215 }, {  52,8215 },
 {  53,8215 }, {  54,8215 }, {  55,8215 }, {  56,8215 }, {  57,8215 },
 {  58,8215 }, {  59,8215 }, {  60,8215 }, {  61,8215 }, {  62,8215 },
 {  63,8215 }, {  64,8215 }, {  65,8215 }, {  66,8215 }, {  67,8215 },
 {  68,8215 }, {  69,8215 }, {  70,8215 }, {  71,8215 }, {  72,8215 },

 {  73,8215 }, {  74,8215 }, {  75,8215 }, {  76,8215 }, {  77,8215 },
 {  78,8215 }, {  79,8215 }, {  80,8215 }, {  81,8215 }, {  82,8215 },
 {  83,8215 }, {  84,8215 }, {  85,8215 }, {  86,8215 }, {  87,8215 },
 {  88,8215 }, {  89,8215 }, {  90,8215 }, {  91,8215 }, {  92,8215 },
 {  93,8215 }, {  94,8215 }, {  95,8215 }, {  96,8215 }, {  97,8215 },
 {  98,8215 }, {  99,8215 }, { 100,8215 }, { 101,8215 }, { 102,8215 },
 { 103,8215 }, { 104,8215 }, { 105,8215 }, { 106,8215 }, { 107,8215 },
 { 108,8215 }, { 109,8215 }, { 110,8215 }, { 111,8215 }, { 112,8215 },
 { 113,8215 }, { 114,8215 }, { 115,8215 }, { 116,8215 }, { 117,8215 },
 { 118,8215 }, { 119,8215 }, { 120,8215 }, { 121,8215 }, { 122,8215 },

 { 123,8215 }, { 124,8215 }, { 125,8215 }, { 126,8215 }, { 127,8215 },
 { 128,8215 }, { 129,8215 }, { 130,8215 }, { 131,8215 }, { 132,8215 },
 { 133,8215 }, { 134,8215 }, { 135,8215 }, { 136,8215 }, { 137,8215 },
 { 138,8215 }, { 139,8215 }, { 140,8215 }, { 141,8215 }, { 142,8215 },
 { 143,8215 }, { 144,8215 }, { 145,8215 }, { 146,8215 }, { 147,8215 },
 { 148,8215 }, { 149,8215 }, { 150,8215 }, { 151,8215 }, { 152,8215 },
 { 153,8215 }, { 154,8215 }, { 155,8215 }, { 156,8215 }, { 157,8215 },
 { 158,8215 }, { 159,8215 }, { 160,8215 }, { 161,8215 }, { 162,8215 },
 { 163,8215 }, { 164,8215 }, { 165,8215 }, { 166,8215 }, { 167,8215 },
 { 168,8215 }, { 169,8215 }, { 170,8215 }, { 171,8215 }, { 172,8215 },

 { 173,8215 }, { 174,8215 }, { 175,8215 }, { 176,8215 }, { 177,8215 },
 { 178,8215 }, { 179,8215 }, { 180,8215 }, { 181,8215 }, { 182,8215 },
 { 183,8215 }, { 184,8215 }, { 185,8215 }, { 186,8215 }, { 187,8215 },
 { 188,8215 }, { 189,8215 }, { 190,8215 }, { 191,8215 }, { 192,8215 },
 { 193,8215 }, { 194,8215 }, { 195,8215 }, { 196,8215 }, { 197,8215 },
 { 198,8215 }, { 199,8215 }, { 200,8215 }, { 201,8215 }, { 202,8215 },
 { 203,8215 }, { 204,8215 }, { 205,8215 }, { 206,8215 }, { 207,8215 },
 { 208,8215 }, { 209,8215 }, { 210,8215 }, { 211,8215 }, { 212,8215 },
 { 213,8215 }, { 214,8215 }, { 215,8215 }, { 216,8215 }, { 217,8215 },
 { 218,8215 }, { 219,8215 }, { 220,8215 }, { 221,8215 }, { 222,8215 },

 { 223,8215 }, { 224,8215 }, { 225,8215 }, { 226,8215 }, { 227,8215 },
 { 228,8215 }, { 229,8215 }, { 230,8215 }, { 231,8215 }, { 232,8215 },
 { 233,8215 }, { 234,8215 }, { 235,8215 }, { 236,8215 }, { 237,8215 },
 { 238,8215 }, { 239,8215 }, { 240,8215 }, { 241,8215 }, { 242,8215 },
 { 243,8215 }, { 244,8215 }, { 245,8215 }, { 246,8215 }, { 247,8215 },
 { 248,8215 }, { 249,8215 }, { 250,8215 }, { 251,8215 }, { 252,8215 },
 { 253,8215 }, { 254,8215 }, { 255,8215 }, { 256,8215 }, {   0,   0 },
 {   0,36040 }, {   1,8262 }, {   2,8262 }, {   3,8262 }, {   4,8262 },
 {   5,8262 }, {   6,8262 }, {   7,8262 }, {   8,8262 }, {   9,8262 },
 {  10,8262 }, {  11,8262 }, {  12,8262 }, {  13,8262 }, {  14,8262 },

 {  15,8262 }, {  16,8262 }, {  17,8262 }, {  18,8262 }, {  19,8262 },
 {  20,8262 }, {  21,8262 }, {  22,8262 }, {  23,8262 }, {  24,8262 },
 {  25,8262 }, {  26,8262 }, {  27,8262 }, {  28,8262 }, {  29,8262 },
 {  30,8262 }, {  31,8262 }, {  32,8262 }, {  33,8520 }, {  34,8262 },
 {  35,8520 }, {  36,8262 }, {  37,8520 }, {  38,8520 }, {  39,8262 },
 {  40,8262 }, {  41,8262 }, {  42,5708 }, {  43,8520 }, {  44,8262 },
 {  45,8520 }, {  46,8262 }, {  47,5712 }, {  48,8262 }, {  49,8262 },
 {  50,8262 }, {  51,8262 }, {  52,8262 }, {  53,8262 }, {  54,8262 },
 {  55,8262 }, {  56,8262 }, {  57,8262 }, {  58,8262 }, {  59,8262 },
 {  60,8520 }, {  61,8520 }, {  62,8520 }, {  63,8520 }, {  64,8520 },

 {  65,8262 }, {  66,8262 }, {  67,8262 }, {  68,8262 }, {  69,8262 },
 {  70,8262 }, {  71,8262 }, {  72,8262 }, {  73,8262 }, {  74,8262 },
 {  75,8262 }, {  76,8262 }, {  77,8262 }, {  78,8262 }, {  79,8262 },
 {  80,8262 }, {  81,8262 }, {  82,8262 }, {  83,8262 }, {  84,8262 },
 {  85,8262 }, {  86,8262 }, {  87,8262 }, {  88,8262 }, {  89,8262 },
 {  90,8262 }, {  91,8262 }, {  92,8262 }, {  93,8262 }, {  94,8520 },
 {  95,8262 }, {  96,8520 }, {  97,8262 }, {  98,8262 }, {  99,8262 },
 { 100,8262 }, { 101,8262 }, { 102,8262 }, { 103,8262 }, { 104,8262 },
 { 105,8262 }, { 106,8262 }, { 107,8262 }, { 108,8262 }, { 109,8262 },
 { 110,8262 }, { 111,8262 }, { 112,8262 }, { 113,8262 }, { 114,8262 },

 { 115,8262 }, { 116,8262 }, { 117,8262 }, { 118,8262 }, { 119,8262 },
 { 120,8262 }, { 121,8262 }, { 122,8262 }, { 123,8262 }, { 124,8520 },
 { 125,8262 }, { 126,8520 }, { 127,8262 }, { 128,8262 }, { 129,8262 },
 { 130,8262 }, { 131,8262 }, { 132,8262 }, { 133,8262 }, { 134,8262 },
 { 135,8262 }, { 136,8262 }, { 137,8262 }, { 138,8262 }, { 139,8262 },
 { 140,8262 }, { 141,8262 }, { 142,8262 }, { 143,8262 }, { 144,8262 },
 { 145,8262 }, { 146,8262 }, { 147,8262 }, { 148,8262 }, { 149,8262 },
 { 150,8262 }, { 151,8262 }, { 152,8262 }, { 153,8262 }, { 154,8262 },
 { 155,8262 }, { 156,8262 }, { 157,8262 }, { 158,8262 }, { 159,8262 },
 { 160,8262 }, { 161,8262 }, { 162,8262 }, { 163,8262 }, { 164,8262 },

 { 165,8262 }, { 166,8262 }, { 167,8262 }, { 168,8262 }, { 169,8262 },
 { 170,8262 }, { 171,8262 }, { 172,8262 }, { 173,8262 }, { 174,8262 },
 { 175,8262 }, { 176,8262 }, { 177,8262 }, { 178,8262 }, { 179,8262 },
 { 180,8262 }, { 181,8262 }, { 182,8262 }, { 183,8262 }, { 184,8262 },
 { 185,8262 }, { 186,8262 }, { 187,8262 }, { 188,8262 }, { 189,8262 },
 { 190,8262 }, { 191,8262 }, { 192,8262 }, { 193,8262 }, { 194,8262 },
 { 195,8262 }, { 196,8262 }, { 197,8262 }, { 198,8262 }, { 199,8262 },
 { 200,8262 }, { 201,8262 }, { 202,8262 }, { 203,8262 }, { 204,8262 },
 { 205,8262 }, { 206,8262 }, { 207,8262 }, { 208,8262 }, { 209,8262 },
 { 210,8262 }, { 211,8262 }, { 212,8262 }, { 213,8262 }, { 214,8262 },

 { 215,8262 }, { 216,8262 }, { 217,8262 }, { 218,8262 }, { 219,8262 },
 { 220,8262 }, { 221,8262 }, { 222,8262 }, { 223,8262 }, { 224,8262 },
 { 225,8262 }, { 226,8262 }, { 227,8262 }, { 228,8262 }, { 229,8262 },
 { 230,8262 }, { 231,8262 }, { 232,8262 }, { 233,8262 }, { 234,8262 },
 { 235,8262 }, { 236,8262 }, { 237,8262 }, { 238,8262 }, { 239,8262 },
 { 240,8262 }, { 241,8262 }, { 242,8262 }, { 243,8262 }, { 244,8262 },
 { 245,8262 }, { 246,8262 }, { 247,8262 }, { 248,8262 }, { 249,8262 },
 { 250,8262 }, { 251,8262 }, { 252,8262 }, { 253,8262 }, { 254,8262 },
 { 255,8262 }, { 256,8262 }, {   0,   0 }, {   0,35782 }, {   1,8004 },
 {   2,8004 }, {   3,8004 }, {   4,8004 }, {   5,8004 }, {   6,8004 },

 {   7,8004 }, {   8,8004 }, {   9,8004 }, {  10,8004 }, {  11,8004 },
 {  12,8004 }, {  13,8004 }, {  14,8004 }, {  15,8004 }, {  16,8004 },
 {  17,8004 }, {  18,8004 }, {  19,8004 }, {  20,8004 }, {  21,8004 },
 {  22,8004 }, {  23,8004 }, {  24,8004 }, {  25,8004 }, {  26,8004 },
 {  27,8004 }, {  28,8004 }, {  29,8004 }, {  30,8004 }, {  31,8004 },
 {  32,8004 }, {  33,8262 }, {  34,8004 }, {  35,8262 }, {  36,8004 },
 {  37,8262 }, {  38,8262 }, {  39,8004 }, {  40,8004 }, {  41,8004 },
 {  42,5450 }, {  43,8262 }, {  44,8004 }, {  45,8262 }, {  46,8004 },
 {  47,5454 }, {  48,8004 }, {  49,8004 }, {  50,8004 }, {  51,8004 },
 {  52,8004 }, {  53,8004 }, {  54,8004 }, {  55,8004 }, {  56,8004 },

 {  57,8004 }, {  58,8004 }, {  59,8004 }, {  60,8262 }, {  61,8262 },
 {  62,8262 }, {  63,8262 }, {  64,8262 }, {  65,8004 }, {  66,8004 },
 {  67,8004 }, {  68,8004 }, {  69,8004 }, {  70,8004 }, {  71,8004 },
 {  72,8004 }, {  73,8004 }, {  74,8004 }, {  75,8004 }, {  76,8004 },
 {  77,8004 }, {  78,8004 }, {  79,8004 }, {  80,8004 }, {  81,8004 },
 {  82,8004 }, {  83,8004 }, {  84,8004 }, {  85,8004 }, {  86,8004 },
 {  87,8004 }, {  88,8004 }, {  89,8004 }, {  90,8004 }, {  91,8004 },
 {  92,8004 }, {  93,8004 }, {  94,8262 }, {  95,8004 }, {  96,8262 },
 {  97,8004 }, {  98,8004 }, {  99,8004 }, { 100,8004 }, { 101,8004 },
 { 102,8004 }, { 103,8004 }, { 104,8004 }, { 105,8004 }, { 106,8004 },

 { 107,8004 }, { 108,8004 }, { 109,8004 }, { 110,8004 }, { 111,8004 },
 { 112,8004 }, { 113,8004 }, { 114,8004 }, { 115,8004 }, { 116,8004 },
 { 117,8004 }, { 118,8004 }, { 119,8004 }, { 120,8004 }, { 121,8004 },
 { 122,8004 }, { 123,8004 }, { 124,8262 }, { 125,8004 }, { 126,8262 },
 { 127,8004 }, { 128,8004 }, { 129,8004 }, { 130,8004 }, { 131,8004 },
 { 132,8004 }, { 133,8004 }, { 134,8004 }, { 135,8004 }, { 136,8004 },
 { 137,8004 }, { 138,8004 }, { 139,8004 }, { 140,8004 }, { 141,8004 },
 { 142,8004 }, { 143,8004 }, { 144,8004 }, { 145,8004 }, { 146,8004 },
 { 147,8004 }, { 148,8004 }, { 149,8004 }, { 150,8004 }, { 151,8004 },
 { 152,8004 }, { 153,8004 }, { 154,8004 }, { 155,8004 }, { 156,8004 },

 { 157,8004 }, { 158,8004 }, { 159,8004 }, { 160,8004 }, { 161,8004 },
 { 162,8004 }, { 163,8004 }, { 164,8004 }, { 165,8004 }, { 166,8004 },
 { 167,8004 }, { 168,8004 }, { 169,8004 }, { 170,8004 }, { 171,8004 },
 { 172,8004 }, { 173,8004 }, { 174,8004 }, { 175,8004 }, { 176,8004 },
 { 177,8004 }, { 178,8004 }, { 179,8004 }, { 180,8004 }, { 181,8004 },
 { 182,8004 }, { 183,8004 }, { 184,8004 }, { 185,8004 }, { 186,8004 },
 { 187,8004 }, { 188,8004 }, { 189,8004 }, { 190,8004 }, { 191,8004 },
 { 192,8004 }, { 193,8004 }, { 194,8004 }, { 195,8004 }, { 196,8004 },
 { 197,8004 }, { 198,8004 }, { 199,8004 }, { 200,8004 }, { 201,8004 },
 { 202,8004 }, { 203,8004 }, { 204,8004 }, { 205,8004 }, { 206,8004 },

 { 207,8004 }, { 208,8004 }, { 209,8004 }, { 210,8004 }, { 211,8004 },
 { 212,8004 }, { 213,8004 }, { 214,8004 }, { 215,8004 }, { 216,8004 },
 { 217,8004 }, { 218,8004 }, { 219,8004 }, { 220,8004 }, { 221,8004 },
 { 222,8004 }, { 223,8004 }, { 224,8004 }, { 225,8004 }, { 226,8004 },
 { 227,8004 }, { 228,8004 }, { 229,8004 }, { 230,8004 }, { 231,8004 },
 { 232,8004 }, { 233,8004 }, { 234,8004 }, { 235,8004 }, { 236,8004 },
 { 237,8004 }, { 238,8004 }, { 239,8004 }, { 240,8004 }, { 241,8004 },
 { 242,8004 }, { 243,8004 }, { 244,8004 }, { 245,8004 }, { 246,8004 },
 { 247,8004 }, { 248,8004 }, { 249,8004 }, { 250,8004 }, { 251,8004 },
 { 252,8004 }, { 253,8004 }, { 254,8004 }, { 255,8004 }, { 256,8004 },

 {   0,   0 }, {   0,35524 }, {   1,8262 }, {   2,8262 }, {   3,8262 },
 {   4,8262 }, {   5,8262 }, {   6,8262 }, {   7,8262 }, {   8,8262 },
 {   9,8262 }, {  10,8262 }, {  11,8262 }, {  12,8262 }, {  13,8262 },
 {  14,8262 }, {  15,8262 }, {  16,8262 }, {  17,8262 }, {  18,8262 },
 {  19,8262 }, {  20,8262 }, {  21,8262 }, {  22,8262 }, {  23,8262 },
 {  24,8262 }, {  25,8262 }, {  26,8262 }, {  27,8262 }, {  28,8262 },
 {  29,8262 }, {  30,8262 }, {  31,8262 }, {  32,8262 }, {  33,8262 },
 {  34,5201 }, {  35,8262 }, {  36,8262 }, {  37,8262 }, {  38,8262 },
 {  39,8262 }, {  40,8262 }, {  41,8262 }, {  42,8262 }, {  43,8262 },
 {  44,8262 }, {  45,8262 }, {  46,8262 }, {  47,8262 }, {  48,8262 },

 {  49,8262 }, {  50,8262 }, {  51,8262 }, {  52,8262 }, {  53,8262 },
 {  54,8262 }, {  55,8262 }, {  56,8262 }, {  57,8262 }, {  58,8262 },
 {  59,8262 }, {  60,8262 }, {  61,8262 }, {  62,8262 }, {  63,8262 },
 {  64,8262 }, {  65,8262 }, {  66,8262 }, {  67,8262 }, {  68,8262 },
 {  69,8262 }, {  70,8262 }, {  71,8262 }, {  72,8262 }, {  73,8262 },
 {  74,8262 }, {  75,8262 }, {  76,8262 }, {  77,8262 }, {  78,8262 },
 {  79,8262 }, {  80,8262 }, {  81,8262 }, {  82,8262 }, {  83,8262 },
 {  84,8262 }, {  85,8262 }, {  86,8262 }, {  87,8262 }, {  88,8262 },
 {  89,8262 }, {  90,8262 }, {  91,8262 }, {  92,8262 }, {  93,8262 },
 {  94,8262 }, {  95,8262 }, {  96,8262 }, {  97,8262 }, {  98,8262 },

 {  99,8262 }, { 100,8262 }, { 101,8262 }, { 102,8262 }, { 103,8262 },
 { 104,8262 }, { 105,8262 }, { 106,8262 }, { 107,8262 }, { 108,8262 },
 { 109,8262 }, { 110,8262 }, { 111,8262 }, { 112,8262 }, { 113,8262 },
 { 114,8262 }, { 115,8262 }, { 116,8262 }, { 117,8262 }, { 118,8262 },
 { 119,8262 }, { 120,8262 }, { 121,8262 }, { 122,8262 }, { 123,8262 },
 { 124,8262 }, { 125,8262 }, { 126,8262 }, { 127,8262 }, { 128,8262 },
 { 129,8262 }, { 130,8262 }, { 131,8262 }, { 132,8262 }, { 133,8262 },
 { 134,8262 }, { 135,8262 }, { 136,8262 }, { 137,8262 }, { 138,8262 },
 { 139,8262 }, { 140,8262 }, { 141,8262 }, { 142,8262 }, { 143,8262 },
 { 144,8262 }, { 145,8262 }, { 146,8262 }, { 147,8262 }, { 148,8262 },

 { 149,8262 }, { 150,8262 }, { 151,8262 }, { 152,8262 }, { 153,8262 },
 { 154,8262 }, { 155,8262 }, { 156,8262 }, { 157,8262 }, { 158,8262 },
 { 159,8262 }, { 160,8262 }, { 161,8262 }, { 162,8262 }, { 163,8262 },
 { 164,8262 }, { 165,8262 }, { 166,8262 }, { 167,8262 }, { 168,8262 },
 { 169,8262 }, { 170,8262 }, { 171,8262 }, { 172,8262 }, { 173,8262 },
 { 174,8262 }, { 175,8262 }, { 176,8262 }, { 177,8262 }, { 178,8262 },
 { 179,8262 }, { 180,8262 }, { 181,8262 }, { 182,8262 }, { 183,8262 },
 { 184,8262 }, { 185,8262 }, { 186,8262 }, { 187,8262 }, { 188,8262 },
 { 189,8262 }, { 190,8262 }, { 191,8262 }, { 192,8262 }, { 193,8262 },
 { 194,8262 }, { 195,8262 }, { 196,8262 }, { 197,8262 }, { 198,8262 },

 { 199,8262 }, { 200,8262 }, { 201,8262 }, { 202,8262 }, { 203,8262 },
 { 204,8262 }, { 205,8262 }, { 206,8262 }, { 207,8262 }, { 208,8262 },
 { 209,8262 }, { 210,8262 }, { 211,8262 }, { 212,8262 }, { 213,8262 },
 { 214,8262 }, { 215,8262 }, { 216,8262 }, { 217,8262 }, { 218,8262 },
 { 219,8262 }, { 220,8262 }, { 221,8262 }, { 222,8262 }, { 223,8262 },
 { 224,8262 }, { 225,8262 }, { 226,8262 }, { 227,8262 }, { 228,8262 },
 { 229,8262 }, { 230,8262 }, { 231,8262 }, { 232,8262 }, { 233,8262 },
 { 234,8262 }, { 235,8262 }, { 236,8262 }, { 237,8262 }, { 238,8262 },
 { 239,8262 }, { 240,8262 }, { 241,8262 }, { 242,8262 }, { 243,8262 },
 { 244,8262 }, { 245,8262 }, { 246,8262 }, { 247,8262 }, { 248,8262 },

 { 249,8262 }, { 250,8262 }, { 251,8262 }, { 252,8262 }, { 253,8262 },
 { 254,8262 }, { 255,8262 }, { 256,8262 }, {   0,   0 }, {   0,35266 },
 {   1,8004 }, {   2,8004 }, {   3,8004 }, {   4,8004 }, {   5,8004 },
 {   6,8004 }, {   7,8004 }, {   8,8004 }, {   9,8004 }, {  10,8004 },
 {  11,8004 }, {  12,8004 }, {  13,8004 }, {  14,8004 }, {  15,8004 },
 {  16,8004 }, {  17,8004 }, {  18,8004 }, {  19,8004 }, {  20,8004 },
 {  21,8004 }, {  22,8004 }, {  23,8004 }, {  24,8004 }, {  25,8004 },
 {  26,8004 }, {  27,8004 }, {  28,8004 }, {  29,8004 }, {  30,8004 },
 {  31,8004 }, {  32,8004 }, {  33,8004 }, {  34,4943 }, {  35,8004 },
 {  36,8004 }, {  37,8004 }, {  38,8004 }, {  39,8004 }, {  40,8004 },

 {  41,8004 }, {  42,8004 }, {  43,8004 }, {  44,8004 }, {  45,8004 },
 {  46,8004 }, {  47,8004 }, {  48,8004 }, {  49,8004 }, {  50,8004 },
 {  51,8004 }, {  52,8004 }, {  53,8004 }, {  54,8004 }, {  55,8004 },
 {  56,8004 }, {  57,8004 }, {  58,8004 }, {  59,8004 }, {  60,8004 },
 {  61,8004 }, {  62,8004 }, {  63,8004 }, {  64,8004 }, {  65,8004 },
 {  66,8004 }, {  67,8004 }, {  68,8004 }, {  69,8004 }, {  70,8004 },
 {  71,8004 }, {  72,8004 }, {  73,8004 }, {  74,8004 }, {  75,8004 },
 {  76,8004 }, {  77,8004 }, {  78,8004 }, {  79,8004 }, {  80,8004 },
 {  81,8004 }, {  82,8004 }, {  83,8004 }, {  84,8004 }, {  85,8004 },
 {  86,8004 }, {  87,8004 }, {  88,8004 }, {  89,8004 }, {  90,8004 },

 {  91,8004 }, {  92,8004 }, {  93,8004 }, {  94,8004 }, {  95,8004 },
 {  96,8004 }, {  97,8004 }, {  98,8004 }, {  99,8004 }, { 100,8004 },
 { 101,8004 }, { 102,8004 }, { 103,8004 }, { 104,8004 }, { 105,8004 },
 { 106,8004 }, { 107,8004 }, { 108,8004 }, { 109,8004 }, { 110,8004 },
 { 111,8004 }, { 112,8004 }, { 113,8004 }, { 114,8004 }, { 115,8004 },
 { 116,8004 }, { 117,8004 }, { 118,8004 }, { 119,8004 }, { 120,8004 },
 { 121,8004 }, { 122,8004 }, { 123,8004 }, { 124,8004 }, { 125,8004 },
 { 126,8004 }, { 127,8004 }, { 128,8004 }, { 129,8004 }, { 130,8004 },
 { 131,8004 }, { 132,8004 }, { 133,8004 }, { 134,8004 }, { 135,8004 },
 { 136,8004 }, { 137,8004 }, { 138,8004 }, { 139,8004 }, { 140,8004 },

 { 141,8004 }, { 142,8004 }, { 143,8004 }, { 144,8004 }, { 145,8004 },
 { 146,8004 }, { 147,8004 }, { 148,8004 }, { 149,8004 }, { 150,8004 },
 { 151,8004 }, { 152,8004 }, { 153,8004 }, { 154,8004 }, { 155,8004 },
 { 156,8004 }, { 157,8004 }, { 158,8004 }, { 159,8004 }, { 160,8004 },
 { 161,8004 }, { 162,8004 }, { 163,8004 }, { 164,8004 }, { 165,8004 },
 { 166,8004 }, { 167,8004 }, { 168,8004 }, { 169,8004 }, { 170,8004 },
 { 171,8004 }, { 172,8004 }, { 173,8004 }, { 174,8004 }, { 175,8004 },
 { 176,8004 }, { 177,8004 }, { 178,8004 }, { 179,8004 }, { 180,8004 },
 { 181,8004 }, { 182,8004 }, { 183,8004 }, { 184,8004 }, { 185,8004 },
 { 186,8004 }, { 187,8004 }, { 188,8004 }, { 189,8004 }, { 190,8004 },

 { 191,8004 }, { 192,8004 }, { 193,8004 }, { 194,8004 }, { 195,8004 },
 { 196,8004 }, { 197,8004 }, { 198,8004 }, { 199,8004 }, { 200,8004 },
 { 201,8004 }, { 202,8004 }, { 203,8004 }, { 204,8004 }, { 205,8004 },
 { 206,8004 }, { 207,8004 }, { 208,8004 }, { 209,8004 }, { 210,8004 },
 { 211,8004 }, { 212,8004 }, { 213,8004 }, { 214,8004 }, { 215,8004 },
 { 216,8004 }, { 217,8004 }, { 218,8004 }, { 219,8004 }, { 220,8004 },
 { 221,8004 }, { 222,8004 }, { 223,8004 }, { 224,8004 }, { 225,8004 },
 { 226,8004 }, { 227,8004 }, { 228,8004 }, { 229,8004 }, { 230,8004 },
 { 231,8004 }, { 232,8004 }, { 233,8004 }, { 234,8004 }, { 235,8004 },
 { 236,8004 }, { 237,8004 }, { 238,8004 }, { 239,8004 }, { 240,8004 },

 { 241,8004 }, { 242,8004 }, { 243,8004 }, { 244,8004 }, { 245,8004 },
 { 246,8004 }, { 247,8004 }, { 248,8004 }, { 249,8004 }, { 250,8004 },
 { 251,8004 }, { 252,8004 }, { 253,8004 }, { 254,8004 }, { 255,8004 },
 { 256,8004 }, {   0,  11 }, {   0,35008 }, {   1,8004 }, {   2,8004 },
 {   3,8004 }, {   4,8004 }, {   5,8004 }, {   6,8004 }, {   7,8004 },
 {   8,8004 }, {   9,8004 }, {  10,8004 }, {  11,8004 }, {  12,8004 },
 {  13,8004 }, {  14,8004 }, {  15,8004 }, {  16,8004 }, {  17,8004 },
 {  18,8004 }, {  19,8004 }, {  20,8004 }, {  21,8004 }, {  22,8004 },
 {  23,8004 }, {  24,8004 }, {  25,8004 }, {  26,8004 }, {  27,8004 },
 {  28,8004 }, {  29,8004 }, {  30,8004 }, {  31,8004 }, {  32,8004 },

 {  33,8004 }, {  34,8004 }, {  35,8004 }, {  36,8004 }, {  37,8004 },
 {  38,8004 }, {  39,8262 }, {  40,8004 }, {  41,8004 }, {  42,8004 },
 {  43,8004 }, {  44,8004 }, {  45,8004 }, {  46,8004 }, {  47,8004 },
 {  48,8004 }, {  49,8004 }, {  50,8004 }, {  51,8004 }, {  52,8004 },
 {  53,8004 }, {  54,8004 }, {  55,8004 }, {  56,8004 }, {  57,8004 },
 {  58,8004 }, {  59,8004 }, {  60,8004 }, {  61,8004 }, {  62,8004 },
 {  63,8004 }, {  64,8004 }, {  65,8004 }, {  66,8004 }, {  67,8004 },
 {  68,8004 }, {  69,8004 }, {  70,8004 }, {  71,8004 }, {  72,8004 },
 {  73,8004 }, {  74,8004 }, {  75,8004 }, {  76,8004 }, {  77,8004 },
 {  78,8004 }, {  79,8004 }, {  80,8004 }, {  81,8004 }, {  82,8004 },

 {  83,8004 }, {  84,8004 }, {  85,8004 }, {  86,8004 }, {  87,8004 },
 {  88,8004 }, {  89,8004 }, {  90,8004 }, {  91,8004 }, {  92,8004 },
 {  93,8004 }, {  94,8004 }, {  95,8004 }, {  96,8004 }, {  97,8004 },
 {  98,8004 }, {  99,8004 }, { 100,8004 }, { 101,8004 }, { 102,8004 },
 { 103,8004 }, { 104,8004 }, { 105,8004 }, { 106,8004 }, { 107,8004 },
 { 108,8004 }, { 109,8004 }, { 110,8004 }, { 111,8004 }, { 112,8004 },
 { 113,8004 }, { 114,8004 }, { 115,8004 }, { 116,8004 }, { 117,8004 },
 { 118,8004 }, { 119,8004 }, { 120,8004 }, { 121,8004 }, { 122,8004 },
 { 123,8004 }, { 124,8004 }, { 125,8004 }, { 126,8004 }, { 127,8004 },
 { 128,8004 }, { 129,8004 }, { 130,8004 }, { 131,8004 }, { 132,8004 },

 { 133,8004 }, { 134,8004 }, { 135,8004 }, { 136,8004 }, { 137,8004 },
 { 138,8004 }, { 139,8004 }, { 140,8004 }, { 141,8004 }, { 142,8004 },
 { 143,8004 }, { 144,8004 }, { 145,8004 }, { 146,8004 }, { 147,8004 },
 { 148,8004 }, { 149,8004 }, { 150,8004 }, { 151,8004 }, { 152,8004 },
 { 153,8004 }, { 154,8004 }, { 155,8004 }, { 156,8004 }, { 157,8004 },
 { 158,8004 }, { 159,8004 }, { 160,8004 }, { 161,8004 }, { 162,8004 },
 { 163,8004 }, { 164,8004 }, { 165,8004 }, { 166,8004 }, { 167,8004 },
 { 168,8004 }, { 169,8004 }, { 170,8004 }, { 171,8004 }, { 172,8004 },
 { 173,8004 }, { 174,8004 }, { 175,8004 }, { 176,8004 }, { 177,8004 },
 { 178,8004 }, { 179,8004 }, { 180,8004 }, { 181,8004 }, { 182,8004 },

 { 183,8004 }, { 184,8004 }, { 185,8004 }, { 186,8004 }, { 187,8004 },
 { 188,8004 }, { 189,8004 }, { 190,8004 }, { 191,8004 }, { 192,8004 },
 { 193,8004 }, { 194,8004 }, { 195,8004 }, { 196,8004 }, { 197,8004 },
 { 198,8004 }, { 199,8004 }, { 200,8004 }, { 201,8004 }, { 202,8004 },
 { 203,8004 }, { 204,8004 }, { 205,8004 }, { 206,8004 }, { 207,8004 },
 { 208,8004 }, { 209,8004 }, { 210,8004 }, { 211,8004 }, { 212,8004 },
 { 213,8004 }, { 214,8004 }, { 215,8004 }, { 216,8004 }, { 217,8004 },
 { 218,8004 }, { 219,8004 }, { 220,8004 }, { 221,8004 }, { 222,8004 },
 { 223,8004 }, { 224,8004 }, { 225,8004 }, { 226,8004 }, { 227,8004 },
 { 228,8004 }, { 229,8004 }, { 230,8004 }, { 231,8004 }, { 232,8004 },

 { 233,8004 }, { 234,8004 }, { 235,8004 }, { 236,8004 }, { 237,8004 },
 { 238,8004 }, { 239,8004 }, { 240,8004 }, { 241,8004 }, { 242,8004 },
 { 243,8004 }, { 244,8004 }, { 245,8004 }, { 246,8004 }, { 247,8004 },
 { 248,8004 }, { 249,8004 }, { 250,8004 }, { 251,8004 }, { 252,8004 },
 { 253,8004 }, { 254,8004 }, { 255,8004 }, { 256,8004 }, {   0,  11 },
 {   0,34750 }, {   1,7746 }, {   2,7746 }, {   3,7746 }, {   4,7746 },
 {   5,7746 }, {   6,7746 }, {   7,7746 }, {   8,7746 }, {   9,7746 },
 {  10,7746 }, {  11,7746 }, {  12,7746 }, {  13,7746 }, {  14,7746 },
 {  15,7746 }, {  16,7746 }, {  17,7746 }, {  18,7746 }, {  19,7746 },
 {  20,7746 }, {  21,7746 }, {  22,7746 }, {  23,7746 }, {  24,7746 },

 {  25,7746 }, {  26,7746 }, {  27,7746 }, {  28,7746 }, {  29,7746 },
 {  30,7746 }, {  31,7746 }, {  32,7746 }, {  33,7746 }, {  34,7746 },
 {  35,7746 }, {  36,7746 }, {  37,7746 }, {  38,7746 }, {  39,8004 },
 {  40,7746 }, {  41,7746 }, {  42,7746 }, {  43,7746 }, {  44,7746 },
 {  45,7746 }, {  46,7746 }, {  47,7746 }, {  48,7746 }, {  49,7746 },
 {  50,7746 }, {  51,7746 }, {  52,7746 }, {  53,7746 }, {  54,7746 },
 {  55,7746 }, {  56,7746 }, {  57,7746 }, {  58,7746 }, {  59,7746 },
 {  60,7746 }, {  61,7746 }, {  62,7746 }, {  63,7746 }, {  64,7746 },
 {  65,7746 }, {  66,7746 }, {  67,7746 }, {  68,7746 }, {  69,7746 },
 {  70,7746 }, {  71,7746 }, {  72,7746 }, {  73,7746 }, {  74,7746 },

 {  75,7746 }, {  76,7746 }, {  77,7746 }, {  78,7746 }, {  79,7746 },
 {  80,7746 }, {  81,7746 }, {  82,7746 }, {  83,7746 }, {  84,7746 },
 {  85,7746 }, {  86,7746 }, {  87,7746 }, {  88,7746 }, {  89,7746 },
 {  90,7746 }, {  91,7746 }, {  92,7746 }, {  93,7746 }, {  94,7746 },
 {  95,7746 }, {  96,7746 }, {  97,7746 }, {  98,7746 }, {  99,7746 },
 { 100,7746 }, { 101,7746 }, { 102,7746 }, { 103,7746 }, { 104,7746 },
 { 105,7746 }, { 106,7746 }, { 107,7746 }, { 108,7746 }, { 109,7746 },
 { 110,7746 }, { 111,7746 }, { 112,7746 }, { 113,7746 }, { 114,7746 },
 { 115,7746 }, { 116,7746 }, { 117,7746 }, { 118,7746 }, { 119,7746 },
 { 120,7746 }, { 121,7746 }, { 122,7746 }, { 123,7746 }, { 124,7746 },

 { 125,7746 }, { 126,7746 }, { 127,7746 }, { 128,7746 }, { 129,7746 },
 { 130,7746 }, { 131,7746 }, { 132,7746 }, { 133,7746 }, { 134,7746 },
 { 135,7746 }, { 136,7746 }, { 137,7746 }, { 138,7746 }, { 139,7746 },
 { 140,7746 }, { 141,7746 }, { 142,7746 }, { 143,7746 }, { 144,7746 },
 { 145,7746 }, { 146,7746 }, { 147,7746 }, { 148,7746 }, { 149,7746 },
 { 150,7746 }, { 151,7746 }, { 152,7746 }, { 153,7746 }, { 154,7746 },
 { 155,7746 }, { 156,7746 }, { 157,7746 }, { 158,7746 }, { 159,7746 },
 { 160,7746 }, { 161,7746 }, { 162,7746 }, { 163,7746 }, { 164,7746 },
 { 165,7746 }, { 166,7746 }, { 167,7746 }, { 168,7746 }, { 169,7746 },
 { 170,7746 }, { 171,7746 }, { 172,7746 }, { 173,7746 }, { 174,7746 },

 { 175,7746 }, { 176,7746 }, { 177,7746 }, { 178,7746 }, { 179,7746 },
 { 180,7746 }, { 181,7746 }, { 182,7746 }, { 183,7746 }, { 184,7746 },
 { 185,7746 }, { 186,7746 }, { 187,7746 }, { 188,7746 }, { 189,7746 },
 { 190,7746 }, { 191,7746 }, { 192,7746 }, { 193,7746 }, { 194,7746 },
 { 195,7746 }, { 196,7746 }, { 197,7746 }, { 198,7746 }, { 199,7746 },
 { 200,7746 }, { 201,7746 }, { 202,7746 }, { 203,7746 }, { 204,7746 },
 { 205,7746 }, { 206,7746 }, { 207,7746 }, { 208,7746 }, { 209,7746 },
 { 210,7746 }, { 211,7746 }, { 212,7746 }, { 213,7746 }, { 214,7746 },
 { 215,7746 }, { 216,7746 }, { 217,7746 }, { 218,7746 }, { 219,7746 },
 { 220,7746 }, { 221,7746 }, { 222,7746 }, { 223,7746 }, { 224,7746 },

 { 225,7746 }, { 226,7746 }, { 227,7746 }, { 228,7746 }, { 229,7746 },
 { 230,7746 }, { 231,7746 }, { 232,7746 }, { 233,7746 }, { 234,7746 },
 { 235,7746 }, { 236,7746 }, { 237,7746 }, { 238,7746 }, { 239,7746 },
 { 240,7746 }, { 241,7746 }, { 242,7746 }, { 243,7746 }, { 244,7746 },
 { 245,7746 }, { 246,7746 }, { 247,7746 }, { 248,7746 }, { 249,7746 },
 { 250,7746 }, { 251,7746 }, { 252,7746 }, { 253,7746 }, { 254,7746 },
 { 255,7746 }, { 256,7746 }, {   0,   0 }, {   0,34492 }, {   1,7793 },
 {   2,7793 }, {   3,7793 }, {   4,7793 }, {   5,7793 }, {   6,7793 },
 {   7,7793 }, {   8,7793 }, {   9,7793 }, {  10,8051 }, {  11,7793 },
 {  12,7793 }, {  13,7793 }, {  14,7793 }, {  15,7793 }, {  16,7793 },

 {  17,7793 }, {  18,7793 }, {  19,7793 }, {  20,7793 }, {  21,7793 },
 {  22,7793 }, {  23,7793 }, {  24,7793 }, {  25,7793 }, {  26,7793 },
 {  27,7793 }, {  28,7793 }, {  29,7793 }, {  30,7793 }, {  31,7793 },
 {  32,7793 }, {  33,7793 }, {  34,7793 }, {  35,7793 }, {  36,7793 },
 {  37,7793 }, {  38,7793 }, {  39,8309 }, {  40,7793 }, {  41,7793 },
 {  42,7793 }, {  43,7793 }, {  44,7793 }, {  45,7793 }, {  46,7793 },
 {  47,7793 }, {  48,7793 }, {  49,7793 }, {  50,7793 }, {  51,7793 },
 {  52,7793 }, {  53,7793 }, {  54,7793 }, {  55,7793 }, {  56,7793 },
 {  57,7793 }, {  58,7793 }, {  59,7793 }, {  60,7793 }, {  61,7793 },
 {  62,7793 }, {  63,7793 }, {  64,7793 }, {  65,7793 }, {  66,7793 },

 {  67,7793 }, {  68,7793 }, {  69,7793 }, {  70,7793 }, {  71,7793 },
 {  72,7793 }, {  73,7793 }, {  74,7793 }, {  75,7793 }, {  76,7793 },
 {  77,7793 }, {  78,7793 }, {  79,7793 }, {  80,7793 }, {  81,7793 },
 {  82,7793 }, {  83,7793 }, {  84,7793 }, {  85,7793 }, {  86,7793 },
 {  87,7793 }, {  88,7793 }, {  89,7793 }, {  90,7793 }, {  91,7793 },
 {  92,8356 }, {  93,7793 }, {  94,7793 }, {  95,7793 }, {  96,7793 },
 {  97,7793 }, {  98,7793 }, {  99,7793 }, { 100,7793 }, { 101,7793 },
 { 102,7793 }, { 103,7793 }, { 104,7793 }, { 105,7793 }, { 106,7793 },
 { 107,7793 }, { 108,7793 }, { 109,7793 }, { 110,7793 }, { 111,7793 },
 { 112,7793 }, { 113,7793 }, { 114,7793 }, { 115,7793 }, { 116,7793 },

 { 117,7793 }, { 118,7793 }, { 119,7793 }, { 120,7793 }, { 121,7793 },
 { 122,7793 }, { 123,7793 }, { 124,7793 }, { 125,7793 }, { 126,7793 },
 { 127,7793 }, { 128,7793 }, { 129,7793 }, { 130,7793 }, { 131,7793 },
 { 132,7793 }, { 133,7793 }, { 134,7793 }, { 135,7793 }, { 136,7793 },
 { 137,7793 }, { 138,7793 }, { 139,7793 }, { 140,7793 }, { 141,7793 },
 { 142,7793 }, { 143,7793 }, { 144,7793 }, { 145,7793 }, { 146,7793 },
 { 147,7793 }, { 148,7793 }, { 149,7793 }, { 150,7793 }, { 151,7793 },
 { 152,7793 }, { 153,7793 }, { 154,7793 }, { 155,7793 }, { 156,7793 },
 { 157,7793 }, { 158,7793 }, { 159,7793 }, { 160,7793 }, { 161,7793 },
 { 162,7793 }, { 163,7793 }, { 164,7793 }, { 165,7793 }, { 166,7793 },

 { 167,7793 }, { 168,7793 }, { 169,7793 }, { 170,7793 }, { 171,7793 },
 { 172,7793 }, { 173,7793 }, { 174,7793 }, { 175,7793 }, { 176,7793 },
 { 177,7793 }, { 178,7793 }, { 179,7793 }, { 180,7793 }, { 181,7793 },
 { 182,7793 }, { 183,7793 }, { 184,7793 }, { 185,7793 }, { 186,7793 },
 { 187,7793 }, { 188,7793 }, { 189,7793 }, { 190,7793 }, { 191,7793 },
 { 192,7793 }, { 193,7793 }, { 194,7793 }, { 195,7793 }, { 196,7793 },
 { 197,7793 }, { 198,7793 }, { 199,7793 }, { 200,7793 }, { 201,7793 },
 { 202,7793 }, { 203,7793 }, { 204,7793 }, { 205,7793 }, { 206,7793 },
 { 207,7793 }, { 208,7793 }, { 209,7793 }, { 210,7793 }, { 211,7793 },
 { 212,7793 }, { 213,7793 }, { 214,7793 }, { 215,7793 }, { 216,7793 },

 { 217,7793 }, { 218,7793 }, { 219,7793 }, { 220,7793 }, { 221,7793 },
 { 222,7793 }, { 223,7793 }, { 224,7793 }, { 225,7793 }, { 226,7793 },
 { 227,7793 }, { 228,7793 }, { 229,7793 }, { 230,7793 }, { 231,7793 },
 { 232,7793 }, { 233,7793 }, { 234,7793 }, { 235,7793 }, { 236,7793 },
 { 237,7793 }, { 238,7793 }, { 239,7793 }, { 240,7793 }, { 241,7793 },
 { 242,7793 }, { 243,7793 }, { 244,7793 }, { 245,7793 }, { 246,7793 },
 { 247,7793 }, { 248,7793 }, { 249,7793 }, { 250,7793 }, { 251,7793 },
 { 252,7793 }, { 253,7793 }, { 254,7793 }, { 255,7793 }, { 256,7793 },
 {   0,   0 }, {   0,34234 }, {   1,7535 }, {   2,7535 }, {   3,7535 },
 {   4,7535 }, {   5,7535 }, {   6,7535 }, {   7,7535 }, {   8,7535 },

 {   9,7535 }, {  10,7793 }, {  11,7535 }, {  12,7535 }, {  13,7535 },
 {  14,7535 }, {  15,7535 }, {  16,7535 }, {  17,7535 }, {  18,7535 },
 {  19,7535 }, {  20,7535 }, {  21,7535 }, {  22,7535 }, {  23,7535 },
 {  24,7535 }, {  25,7535 }, {  26,7535 }, {  27,7535 }, {  28,7535 },
 {  29,7535 }, {  30,7535 }, {  31,7535 }, {  32,7535 }, {  33,7535 },
 {  34,7535 }, {  35,7535 }, {  36,7535 }, {  37,7535 }, {  38,7535 },
 {  39,8051 }, {  40,7535 }, {  41,7535 }, {  42,7535 }, {  43,7535 },
 {  44,7535 }, {  45,7535 }, {  46,7535 }, {  47,7535 }, {  48,7535 },
 {  49,7535 }, {  50,7535 }, {  51,7535 }, {  52,7535 }, {  53,7535 },
 {  54,7535 }, {  55,7535 }, {  56,7535 }, {  57,7535 }, {  58,7535 },

 {  59,7535 }, {  60,7535 }, {  61,7535 }, {  62,7535 }, {  63,7535 },
 {  64,7535 }, {  65,7535 }, {  66,7535 }, {  67,7535 }, {  68,7535 },
 {  69,7535 }, {  70,7535 }, {  71,7535 }, {  72,7535 }, {  73,7535 },
 {  74,7535 }, {  75,7535 }, {  76,7535 }, {  77,7535 }, {  78,7535 },
 {  79,7535 }, {  80,7535 }, {  81,7535 }, {  82,7535 }, {  83,7535 },
 {  84,7535 }, {  85,7535 }, {  86,7535 }, {  87,7535 }, {  88,7535 },
 {  89,7535 }, {  90,7535 }, {  91,7535 }, {  92,8098 }, {  93,7535 },
 {  94,7535 }, {  95,7535 }, {  96,7535 }, {  97,7535 }, {  98,7535 },
 {  99,7535 }, { 100,7535 }, { 101,7535 }, { 102,7535 }, { 103,7535 },
 { 104,7535 }, { 105,7535 }, { 106,7535 }, { 107,7535 }, { 108,7535 },

 { 109,7535 }, { 110,7535 }, { 111,7535 }, { 112,7535 }, { 113,7535 },
 { 114,7535 }, { 115,7535 }, { 116,7535 }, { 117,7535 }, { 118,7535 },
 { 119,7535 }, { 120,7535 }, { 121,7535 }, { 122,7535 }, { 123,7535 },
 { 124,7535 }, { 125,7535 }, { 126,7535 }, { 127,7535 }, { 128,7535 },
 { 129,7535 }, { 130,7535 }, { 131,7535 }, { 132,7535 }, { 133,7535 },
 { 134,7535 }, { 135,7535 }, { 136,7535 }, { 137,7535 }, { 138,7535 },
 { 139,7535 }, { 140,7535 }, { 141,7535 }, { 142,7535 }, { 143,7535 },
 { 144,7535 }, { 145,7535 }, { 146,7535 }, { 147,7535 }, { 148,7535 },
 { 149,7535 }, { 150,7535 }, { 151,7535 }, { 152,7535 }, { 153,7535 },
 { 154,7535 }, { 155,7535 }, { 156,7535 }, { 157,7535 }, { 158,7535 },

 { 159,7535 }, { 160,7535 }, { 161,7535 }, { 162,7535 }, { 163,7535 },
 { 164,7535 }, { 165,7535 }, { 166,7535 }, { 167,7535 }, { 168,7535 },
 { 169,7535 }, { 170,7535 }, { 171,7535 }, { 172,7535 }, { 173,7535 },
 { 174,7535 }, { 175,7535 }, { 176,7535 }, { 177,7535 }, { 178,7535 },
 { 179,7535 }, { 180,7535 }, { 181,7535 }, { 182,7535 }, { 183,7535 },
 { 184,7535 }, { 185,7535 }, { 186,7535 }, { 187,7535 }, { 188,7535 },
 { 189,7535 }, { 190,7535 }, { 191,7535 }, { 192,7535 }, { 193,7535 },
 { 194,7535 }, { 195,7535 }, { 196,7535 }, { 197,7535 }, { 198,7535 },
 { 199,7535 }, { 200,7535 }, { 201,7535 }, { 202,7535 }, { 203,7535 },
 { 204,7535 }, { 205,7535 }, { 206,7535 }, { 207,7535 }, { 208,7535 },

 { 209,7535 }, { 210,7535 }, { 211,7535 }, { 212,7535 }, { 213,7535 },
 { 214,7535 }, { 215,7535 }, { 216,7535 }, { 217,7535 }, { 218,7535 },
 { 219,7535 }, { 220,7535 }, { 221,7535 }, { 222,7535 }, { 223,7535 },
 { 224,7535 }, { 225,7535 }, { 226,7535 }, { 227,7535 }, { 228,7535 },
 { 229,7535 }, { 230,7535 }, { 231,7535 }, { 232,7535 }, { 233,7535 },
 { 234,7535 }, { 235,7535 }, { 236,7535 }, { 237,7535 }, { 238,7535 },
 { 239,7535 }, { 240,7535 }, { 241,7535 }, { 242,7535 }, { 243,7535 },
 { 244,7535 }, { 245,7535 }, { 246,7535 }, { 247,7535 }, { 248,7535 },
 { 249,7535 }, { 250,7535 }, { 251,7535 }, { 252,7535 }, { 253,7535 },
 { 254,7535 }, { 255,7535 }, { 256,7535 }, {   0,   0 }, {   0,33976 },

 {   1,8098 }, {   2,8098 }, {   3,8098 }, {   4,8098 }, {   5,8098 },
 {   6,8098 }, {   7,8098 }, {   8,8098 }, {   9,8098 }, {  10,8098 },
 {  11,8098 }, {  12,8098 }, {  13,8098 }, {  14,8098 }, {  15,8098 },
 {  16,8098 }, {  17,8098 }, {  18,8098 }, {  19,8098 }, {  20,8098 },
 {  21,8098 }, {  22,8098 }, {  23,8098 }, {  24,8098 }, {  25,8098 },
 {  26,8098 }, {  27,8098 }, {  28,8098 }, {  29,8098 }, {  30,8098 },
 {  31,8098 }, {  32,8098 }, {  33,8098 }, {  34,8098 }, {  35,8098 },
 {  36,8098 }, {  37,8098 }, {  38,8098 }, {  39,8356 }, {  40,8098 },
 {  41,8098 }, {  42,8098 }, {  43,8098 }, {  44,8098 }, {  45,8098 },
 {  46,8098 }, {  47,8098 }, {  48,8098 }, {  49,8098 }, {  50,8098 },

 {  51,8098 }, {  52,8098 }, {  53,8098 }, {  54,8098 }, {  55,8098 },
 {  56,8098 }, {  57,8098 }, {  58,8098 }, {  59,8098 }, {  60,8098 },
 {  61,8098 }, {  62,8098 }, {  63,8098 }, {  64,8098 }, {  65,8098 },
 {  66,8098 }, {  67,8098 }, {  68,8098 }, {  69,8098 }, {  70,8098 },
 {  71,8098 }, {  72,8098 }, {  73,8098 }, {  74,8098 }, {  75,8098 },
 {  76,8098 }, {  77,8098 }, {  78,8098 }, {  79,8098 }, {  80,8098 },
 {  81,8098 }, {  82,8098 }, {  83,8098 }, {  84,8098 }, {  85,8098 },
 {  86,8098 }, {  87,8098 }, {  88,8098 }, {  89,8098 }, {  90,8098 },
 {  91,8098 }, {  92,8098 }, {  93,8098 }, {  94,8098 }, {  95,8098 },
 {  96,8098 }, {  97,8098 }, {  98,8098 }, {  99,8098 }, { 100,8098 },

 { 101,8098 }, { 102,8098 }, { 103,8098 }, { 104,8098 }, { 105,8098 },
 { 106,8098 }, { 107,8098 }, { 108,8098 }, { 109,8098 }, { 110,8098 },
 { 111,8098 }, { 112,8098 }, { 113,8098 }, { 114,8098 }, { 115,8098 },
 { 116,8098 }, { 117,8098 }, { 118,8098 }, { 119,8098 }, { 120,8098 },
 { 121,8098 }, { 122,8098 }, { 123,8098 }, { 124,8098 }, { 125,8098 },
 { 126,8098 }, { 127,8098 }, { 128,8098 }, { 129,8098 }, { 130,8098 },
 { 131,8098 }, { 132,8098 }, { 133,8098 }, { 134,8098 }, { 135,8098 },
 { 136,8098 }, { 137,8098 }, { 138,8098 }, { 139,8098 }, { 140,8098 },
 { 141,8098 }, { 142,8098 }, { 143,8098 }, { 144,8098 }, { 145,8098 },
 { 146,8098 }, { 147,8098 }, { 148,8098 }, { 149,8098 }, { 150,8098 },

 { 151,8098 }, { 152,8098 }, { 153,8098 }, { 154,8098 }, { 155,8098 },
 { 156,8098 }, { 157,8098 }, { 158,8098 }, { 159,8098 }, { 160,8098 },
 { 161,8098 }, { 162,8098 }, { 163,8098 }, { 164,8098 }, { 165,8098 },
 { 166,8098 }, { 167,8098 }, { 168,8098 }, { 169,8098 }, { 170,8098 },
 { 171,8098 }, { 172,8098 }, { 173,8098 }, { 174,8098 }, { 175,8098 },
 { 176,8098 }, { 177,8098 }, { 178,8098 }, { 179,8098 }, { 180,8098 },
 { 181,8098 }, { 182,8098 }, { 183,8098 }, { 184,8098 }, { 185,8098 },
 { 186,8098 }, { 187,8098 }, { 188,8098 }, { 189,8098 }, { 190,8098 },
 { 191,8098 }, { 192,8098 }, { 193,8098 }, { 194,8098 }, { 195,8098 },
 { 196,8098 }, { 197,8098 }, { 198,8098 }, { 199,8098 }, { 200,8098 },

 { 201,8098 }, { 202,8098 }, { 203,8098 }, { 204,8098 }, { 205,8098 },
 { 206,8098 }, { 207,8098 }, { 208,8098 }, { 209,8098 }, { 210,8098 },
 { 211,8098 }, { 212,8098 }, { 213,8098 }, { 214,8098 }, { 215,8098 },
 { 216,8098 }, { 217,8098 }, { 218,8098 }, { 219,8098 }, { 220,8098 },
 { 221,8098 }, { 222,8098 }, { 223,8098 }, { 224,8098 }, { 225,8098 },
 { 226,8098 }, { 227,8098 }, { 228,8098 }, { 229,8098 }, { 230,8098 },
 { 231,8098 }, { 232,8098 }, { 233,8098 }, { 234,8098 }, { 235,8098 },
 { 236,8098 }, { 237,8098 }, { 238,8098 }, { 239,8098 }, { 240,8098 },
 { 241,8098 }, { 242,8098 }, { 243,8098 }, { 244,8098 }, { 245,8098 },
 { 246,8098 }, { 247,8098 }, { 248,8098 }, { 249,8098 }, { 250,8098 },

 { 251,8098 }, { 252,8098 }, { 253,8098 }, { 254,8098 }, { 255,8098 },
 { 256,8098 }, {   0,   0 }, {   0,33718 }, {   1,7840 }, {   2,7840 },
 {   3,7840 }, {   4,7840 }, {   5,7840 }, {   6,7840 }, {   7,7840 },
 {   8,7840 }, {   9,7840 }, {  10,7840 }, {  11,7840 }, {  12,7840 },
 {  13,7840 }, {  14,7840 }, {  15,7840 }, {  16,7840 }, {  17,7840 },
 {  18,7840 }, {  19,7840 }, {  20,7840 }, {  21,7840 }, {  22,7840 },
 {  23,7840 }, {  24,7840 }, {  25,7840 }, {  26,7840 }, {  27,7840 },
 {  28,7840 }, {  29,7840 }, {  30,7840 }, {  31,7840 }, {  32,7840 },
 {  33,7840 }, {  34,7840 }, {  35,7840 }, {  36,7840 }, {  37,7840 },
 {  38,7840 }, {  39,8098 }, {  40,7840 }, {  41,7840 }, {  42,7840 },

 {  43,7840 }, {  44,7840 }, {  45,7840 }, {  46,7840 }, {  47,7840 },
 {  48,7840 }, {  49,7840 }, {  50,7840 }, {  51,7840 }, {  52,7840 },
 {  53,7840 }, {  54,7840 }, {  55,7840 }, {  56,7840 }, {  57,7840 },
 {  58,7840 }, {  59,7840 }, {  60,7840 }, {  61,7840 }, {  62,7840 },
 {  63,7840 }, {  64,7840 }, {  65,7840 }, {  66,7840 }, {  67,7840 },
 {  68,7840 }, {  69,7840 }, {  70,7840 }, {  71,7840 }, {  72,7840 },
 {  73,7840 }, {  74,7840 }, {  75,7840 }, {  76,7840 }, {  77,7840 },
 {  78,7840 }, {  79,7840 }, {  80,7840 }, {  81,7840 }, {  82,7840 },
 {  83,7840 }, {  84,7840 }, {  85,7840 }, {  86,7840 }, {  87,7840 },
 {  88,7840 }, {  89,7840 }, {  90,7840 }, {  91,7840 }, {  92,7840 },

 {  93,7840 }, {  94,7840 }, {  95,7840 }, {  96,7840 }, {  97,7840 },
 {  98,7840 }, {  99,7840 }, { 100,7840 }, { 101,7840 }, { 102,7840 },
 { 103,7840 }, { 104,7840 }, { 105,7840 }, { 106,7840 }, { 107,7840 },
 { 108,7840 }, { 109,7840 }, { 110,7840 }, { 111,7840 }, { 112,7840 },
 { 113,7840 }, { 114,7840 }, { 115,7840 }, { 116,7840 }, { 117,7840 },
 { 118,7840 }, { 119,7840 }, { 120,7840 }, { 121,7840 }, { 122,7840 },
 { 123,7840 }, { 124,7840 }, { 125,7840 }, { 126,7840 }, { 127,7840 },
 { 128,7840 }, { 129,7840 }, { 130,7840 }, { 131,7840 }, { 132,7840 },
 { 133,7840 }, { 134,7840 }, { 135,7840 }, { 136,7840 }, { 137,7840 },
 { 138,7840 }, { 139,7840 }, { 140,7840 }, { 141,7840 }, { 142,7840 },

 { 143,7840 }, { 144,7840 }, { 145,7840 }, { 146,7840 }, { 147,7840 },
 { 148,7840 }, { 149,7840 }, { 150,7840 }, { 151,7840 }, { 152,7840 },
 { 153,7840 }, { 154,7840 }, { 155,7840 }, { 156,7840 }, { 157,7840 },
 { 158,7840 }, { 159,7840 }, { 160,7840 }, { 161,7840 }, { 162,7840 },
 { 163,7840 }, { 164,7840 }, { 165,7840 }, { 166,7840 }, { 167,7840 },
 { 168,7840 }, { 169,7840 }, { 170,7840 }, { 171,7840 }, { 172,7840 },
 { 173,7840 }, { 174,7840 }, { 175,7840 }, { 176,7840 }, { 177,7840 },
 { 178,7840 }, { 179,7840 }, { 180,7840 }, { 181,7840 }, { 182,7840 },
 { 183,7840 }, { 184,7840 }, { 185,7840 }, { 186,7840 }, { 187,7840 },
 { 188,7840 }, { 189,7840 }, { 190,7840 }, { 191,7840 }, { 192,7840 },

 { 193,7840 }, { 194,7840 }, { 195,7840 }, { 196,7840 }, { 197,7840 },
 { 198,7840 }, { 199,7840 }, { 200,7840 }, { 201,7840 }, { 202,7840 },
 { 203,7840 }, { 204,7840 }, { 205,7840 }, { 206,7840 }, { 207,7840 },
 { 208,7840 }, { 209,7840 }, { 210,7840 }, { 211,7840 }, { 212,7840 },
 { 213,7840 }, { 214,7840 }, { 215,7840 }, { 216,7840 }, { 217,7840 },
 { 218,7840 }, { 219,7840 }, { 220,7840 }, { 221,7840 }, { 222,7840 },
 { 223,7840 }, { 224,7840 }, { 225,7840 }, { 226,7840 }, { 227,7840 },
 { 228,7840 }, { 229,7840 }, { 230,7840 }, { 231,7840 }, { 232,7840 },
 { 233,7840 }, { 234,7840 }, { 235,7840 }, { 236,7840 }, { 237,7840 },
 { 238,7840 }, { 239,7840 }, { 240,7840 }, { 241,7840 }, { 242,7840 },

 { 243,7840 }, { 244,7840 }, { 245,7840 }, { 246,7840 }, { 247,7840 },
 { 248,7840 }, { 249,7840 }, { 250,7840 }, { 251,7840 }, { 252,7840 },
 { 253,7840 }, { 254,7840 }, { 255,7840 }, { 256,7840 }, {   0,   0 },
 {   0,33460 }, {   1,7887 }, {   2,7887 }, {   3,7887 }, {   4,7887 },
 {   5,7887 }, {   6,7887 }, {   7,7887 }, {   8,7887 }, {   9,7887 },
 {  10,8145 }, {  11,7887 }, {  12,7887 }, {  13,7887 }, {  14,7887 },
 {  15,7887 }, {  16,7887 }, {  17,7887 }, {  18,7887 }, {  19,7887 },
 {  20,7887 }, {  21,7887 }, {  22,7887 }, {  23,7887 }, {  24,7887 },
 {  25,7887 }, {  26,7887 }, {  27,7887 }, {  28,7887 }, {  29,7887 },
 {  30,7887 }, {  31,7887 }, {  32,7887 }, {  33,7887 }, {  34,7887 },

 {  35,7887 }, {  36,8403 }, {  37,7887 }, {  38,7887 }, {  39,7887 },
 {  40,7887 }, {  41,7887 }, {  42,7887 }, {  43,7887 }, {  44,7887 },
 {  45,7887 }, {  46,7887 }, {  47,7887 }, {  48,7887 }, {  49,7887 },
 {  50,7887 }, {  51,7887 }, {  52,7887 }, {  53,7887 }, {  54,7887 },
 {  55,7887 }, {  56,7887 }, {  57,7887 }, {  58,7887 }, {  59,7887 },
 {  60,7887 }, {  61,7887 }, {  62,7887 }, {  63,7887 }, {  64,7887 },
 {  65,7887 }, {  66,7887 }, {  67,7887 }, {  68,7887 }, {  69,7887 },
 {  70,7887 }, {  71,7887 }, {  72,7887 }, {  73,7887 }, {  74,7887 },
 {  75,7887 }, {  76,7887 }, {  77,7887 }, {  78,7887 }, {  79,7887 },
 {  80,7887 }, {  81,7887 }, {  82,7887 }, {  83,7887 }, {  84,7887 },

 {  85,7887 }, {  86,7887 }, {  87,7887 }, {  88,7887 }, {  89,7887 },
 {  90,7887 }, {  91,7887 }, {  92,7887 }, {  93,7887 }, {  94,7887 },
 {  95,7887 }, {  96,7887 }, {  97,7887 }, {  98,7887 }, {  99,7887 },
 { 100,7887 }, { 101,7887 }, { 102,7887 }, { 103,7887 }, { 104,7887 },
 { 105,7887 }, { 106,7887 }, { 107,7887 }, { 108,7887 }, { 109,7887 },
 { 110,7887 }, { 111,7887 }, { 112,7887 }, { 113,7887 }, { 114,7887 },
 { 115,7887 }, { 116,7887 }, { 117,7887 }, { 118,7887 }, { 119,7887 },
 { 120,7887 }, { 121,7887 }, { 122,7887 }, { 123,7887 }, { 124,7887 },
 { 125,7887 }, { 126,7887 }, { 127,7887 }, { 128,7887 }, { 129,7887 },
 { 130,7887 }, { 131,7887 }, { 132,7887 }, { 133,7887 }, { 134,7887 },

 { 135,7887 }, { 136,7887 }, { 137,7887 }, { 138,7887 }, { 139,7887 },
 { 140,7887 }, { 141,7887 }, { 142,7887 }, { 143,7887 }, { 144,7887 },
 { 145,7887 }, { 146,7887 }, { 147,7887 }, { 148,7887 }, { 149,7887 },
 { 150,7887 }, { 151,7887 }, { 152,7887 }, { 153,7887 }, { 154,7887 },
 { 155,7887 }, { 156,7887 }, { 157,7887 }, { 158,7887 }, { 159,7887 },
 { 160,7887 }, { 161,7887 }, { 162,7887 }, { 163,7887 }, { 164,7887 },
 { 165,7887 }, { 166,7887 }, { 167,7887 }, { 168,7887 }, { 169,7887 },
 { 170,7887 }, { 171,7887 }, { 172,7887 }, { 173,7887 }, { 174,7887 },
 { 175,7887 }, { 176,7887 }, { 177,7887 }, { 178,7887 }, { 179,7887 },
 { 180,7887 }, { 181,7887 }, { 182,7887 }, { 183,7887 }, { 184,7887 },

 { 185,7887 }, { 186,7887 }, { 187,7887 }, { 188,7887 }, { 189,7887 },
 { 190,7887 }, { 191,7887 }, { 192,7887 }, { 193,7887 }, { 194,7887 },
 { 195,7887 }, { 196,7887 }, { 197,7887 }, { 198,7887 }, { 199,7887 },
 { 200,7887 }, { 201,7887 }, { 202,7887 }, { 203,7887 }, { 204,7887 },
 { 205,7887 }, { 206,7887 }, { 207,7887 }, { 208,7887 }, { 209,7887 },
 { 210,7887 }, { 211,7887 }, { 212,7887 }, { 213,7887 }, { 214,7887 },
 { 215,7887 }, { 216,7887 }, { 217,7887 }, { 218,7887 }, { 219,7887 },
 { 220,7887 }, { 221,7887 }, { 222,7887 }, { 223,7887 }, { 224,7887 },
 { 225,7887 }, { 226,7887 }, { 227,7887 }, { 228,7887 }, { 229,7887 },
 { 230,7887 }, { 231,7887 }, { 232,7887 }, { 233,7887 }, { 234,7887 },

 { 235,7887 }, { 236,7887 }, { 237,7887 }, { 238,7887 }, { 239,7887 },
 { 240,7887 }, { 241,7887 }, { 242,7887 }, { 243,7887 }, { 244,7887 },
 { 245,7887 }, { 246,7887 }, { 247,7887 }, { 248,7887 }, { 249,7887 },
 { 250,7887 }, { 251,7887 }, { 252,7887 }, { 253,7887 }, { 254,7887 },
 { 255,7887 }, { 256,7887 }, {   0,   0 }, {   0,33202 }, {   1,7629 },
 {   2,7629 }, {   3,7629 }, {   4,7629 }, {   5,7629 }, {   6,7629 },
 {   7,7629 }, {   8,7629 }, {   9,7629 }, {  10,7887 }, {  11,7629 },
 {  12,7629 }, {  13,7629 }, {  14,7629 }, {  15,7629 }, {  16,7629 },
 {  17,7629 }, {  18,7629 }, {  19,7629 }, {  20,7629 }, {  21,7629 },
 {  22,7629 }, {  23,7629 }, {  24,7629 }, {  25,7629 }, {  26,7629 },

 {  27,7629 }, {  28,7629 }, {  29,7629 }, {  30,7629 }, {  31,7629 },
 {  32,7629 }, {  33,7629 }, {  34,7629 }, {  35,7629 }, {  36,8145 },
 {  37,7629 }, {  38,7629 }, {  39,7629 }, {  40,7629 }, {  41,7629 },
 {  42,7629 }, {  43,7629 }, {  44,7629 }, {  45,7629 }, {  46,7629 },
 {  47,7629 }, {  48,7629 }, {  49,7629 }, {  50,7629 }, {  51,7629 },
 {  52,7629 }, {  53,7629 }, {  54,7629 }, {  55,7629 }, {  56,7629 },
 {  57,7629 }, {  58,7629 }, {  59,7629 }, {  60,7629 }, {  61,7629 },
 {  62,7629 }, {  63,7629 }, {  64,7629 }, {  65,7629 }, {  66,7629 },
 {  67,7629 }, {  68,7629 }, {  69,7629 }, {  70,7629 }, {  71,7629 },
 {  72,7629 }, {  73,7629 }, {  74,7629 }, {  75,7629 }, {  76,7629 },

 {  77,7629 }, {  78,7629 }, {  79,7629 }, {  80,7629 }, {  81,7629 },
 {  82,7629 }, {  83,7629 }, {  84,7629 }, {  85,7629 }, {  86,7629 },
 {  87,7629 }, {  88,7629 }, {  89,7629 }, {  90,7629 }, {  91,7629 },
 {  92,7629 }, {  93,7629 }, {  94,7629 }, {  95,7629 }, {  96,7629 },
 {  97,7629 }, {  98,7629 }, {  99,7629 }, { 100,7629 }, { 101,7629 },
 { 102,7629 }, { 103,7629 }, { 104,7629 }, { 105,7629 }, { 106,7629 },
 { 107,7629 }, { 108,7629 }, { 109,7629 }, { 110,7629 }, { 111,7629 },
 { 112,7629 }, { 113,7629 }, { 114,7629 }, { 115,7629 }, { 116,7629 },
 { 117,7629 }, { 118,7629 }, { 119,7629 }, { 120,7629 }, { 121,7629 },
 { 122,7629 }, { 123,7629 }, { 124,7629 }, { 125,7629 }, { 126,7629 },

 { 127,7629 }, { 128,7629 }, { 129,7629 }, { 130,7629 }, { 131,7629 },
 { 132,7629 }, { 133,7629 }, { 134,7629 }, { 135,7629 }, { 136,7629 },
 { 137,7629 }, { 138,7629 }, { 139,7629 }, { 140,7629 }, { 141,7629 },
 { 142,7629 }, { 143,7629 }, { 144,7629 }, { 145,7629 }, { 146,7629 },
 { 147,7629 }, { 148,7629 }, { 149,7629 }, { 150,7629 }, { 151,7629 },
 { 152,7629 }, { 153,7629 }, { 154,7629 }, { 155,7629 }, { 156,7629 },
 { 157,7629 }, { 158,7629 }, { 159,7629 }, { 160,7629 }, { 161,7629 },
 { 162,7629 }, { 163,7629 }, { 164,7629 }, { 165,7629 }, { 166,7629 },
 { 167,7629 }, { 168,7629 }, { 169,7629 }, { 170,7629 }, { 171,7629 },
 { 172,7629 }, { 173,7629 }, { 174,7629 }, { 175,7629 }, { 176,7629 },

 { 177,7629 }, { 178,7629 }, { 179,7629 }, { 180,7629 }, { 181,7629 },
 { 182,7629 }, { 183,7629 }, { 184,7629 }, { 185,7629 }, { 186,7629 },
 { 187,7629 }, { 188,7629 }, { 189,7629 }, { 190,7629 }, { 191,7629 },
 { 192,7629 }, { 193,7629 }, { 194,7629 }, { 195,7629 }, { 196,7629 },
 { 197,7629 }, { 198,7629 }, { 199,7629 }, { 200,7629 }, { 201,7629 },
 { 202,7629 }, { 203,7629 }, { 204,7629 }, { 205,7629 }, { 206,7629 },
 { 207,7629 }, { 208,7629 }, { 209,7629 }, { 210,7629 }, { 211,7629 },
 { 212,7629 }, { 213,7629 }, { 214,7629 }, { 215,7629 }, { 216,7629 },
 { 217,7629 }, { 218,7629 }, { 219,7629 }, { 220,7629 }, { 221,7629 },
 { 222,7629 }, { 223,7629 }, { 224,7629 }, { 225,7629 }, { 226,7629 },

 { 227,7629 }, { 228,7629 }, { 229,7629 }, { 230,7629 }, { 231,7629 },
 { 232,7629 }, { 233,7629 }, { 234,7629 }, { 235,7629 }, { 236,7629 },
 { 237,7629 }, { 238,7629 }, { 239,7629 }, { 240,7629 }, { 241,7629 },
 { 242,7629 }, { 243,7629 }, { 244,7629 }, { 245,7629 }, { 246,7629 },
 { 247,7629 }, { 248,7629 }, { 249,7629 }, { 250,7629 }, { 251,7629 },
 { 252,7629 }, { 253,7629 }, { 254,7629 }, { 255,7629 }, { 256,7629 },
 {   0,   0 }, {   0,32944 }, {   1,5682 }, {   2,5682 }, {   3,5682 },
 {   4,5682 }, {   5,5682 }, {   6,5682 }, {   7,5682 }, {   8,5682 },
 {   9,5682 }, {  10,5682 }, {  11,5682 }, {  12,5682 }, {  13,5682 },
 {  14,5682 }, {  15,5682 }, {  16,5682 }, {  17,5682 }, {  18,5682 },

 {  19,5682 }, {  20,5682 }, {  21,5682 }, {  22,5682 }, {  23,5682 },
 {  24,5682 }, {  25,5682 }, {  26,5682 }, {  27,5682 }, {  28,5682 },
 {  29,5682 }, {  30,5682 }, {  31,5682 }, {  32,5682 }, {  33,5682 },
 {  34,2639 }, {  35,5682 }, {  36,5682 }, {  37,5682 }, {  38,5682 },
 {  39,5682 }, {  40,5682 }, {  41,5682 }, {  42,5682 }, {  43,5682 },
 {  44,5682 }, {  45,5682 }, {  46,5682 }, {  47,5682 }, {  48,5682 },
 {  49,5682 }, {  50,5682 }, {  51,5682 }, {  52,5682 }, {  53,5682 },
 {  54,5682 }, {  55,5682 }, {  56,5682 }, {  57,5682 }, {  58,5682 },
 {  59,5682 }, {  60,5682 }, {  61,5682 }, {  62,5682 }, {  63,5682 },
 {  64,5682 }, {  65,5682 }, {  66,5682 }, {  67,5682 }, {  68,5682 },

 {  69,5682 }, {  70,5682 }, {  71,5682 }, {  72,5682 }, {  73,5682 },
 {  74,5682 }, {  75,5682 }, {  76,5682 }, {  77,5682 }, {  78,5682 },
 {  79,5682 }, {  80,5682 }, {  81,5682 }, {  82,5682 }, {  83,5682 },
 {  84,5682 }, {  85,5682 }, {  86,5682 }, {  87,5682 }, {  88,5682 },
 {  89,5682 }, {  90,5682 }, {  91,5682 }, {  92,5682 }, {  93,5682 },
 {  94,5682 }, {  95,5682 }, {  96,5682 }, {  97,5682 }, {  98,5682 },
 {  99,5682 }, { 100,5682 }, { 101,5682 }, { 102,5682 }, { 103,5682 },
 { 104,5682 }, { 105,5682 }, { 106,5682 }, { 107,5682 }, { 108,5682 },
 { 109,5682 }, { 110,5682 }, { 111,5682 }, { 112,5682 }, { 113,5682 },
 { 114,5682 }, { 115,5682 }, { 116,5682 }, { 117,5682 }, { 118,5682 },

 { 119,5682 }, { 120,5682 }, { 121,5682 }, { 122,5682 }, { 123,5682 },
 { 124,5682 }, { 125,5682 }, { 126,5682 }, { 127,5682 }, { 128,5682 },
 { 129,5682 }, { 130,5682 }, { 131,5682 }, { 132,5682 }, { 133,5682 },
 { 134,5682 }, { 135,5682 }, { 136,5682 }, { 137,5682 }, { 138,5682 },
 { 139,5682 }, { 140,5682 }, { 141,5682 }, { 142,5682 }, { 143,5682 },
 { 144,5682 }, { 145,5682 }, { 146,5682 }, { 147,5682 }, { 148,5682 },
 { 149,5682 }, { 150,5682 }, { 151,5682 }, { 152,5682 }, { 153,5682 },
 { 154,5682 }, { 155,5682 }, { 156,5682 }, { 157,5682 }, { 158,5682 },
 { 159,5682 }, { 160,5682 }, { 161,5682 }, { 162,5682 }, { 163,5682 },
 { 164,5682 }, { 165,5682 }, { 166,5682 }, { 167,5682 }, { 168,5682 },

 { 169,5682 }, { 170,5682 }, { 171,5682 }, { 172,5682 }, { 173,5682 },
 { 174,5682 }, { 175,5682 }, { 176,5682 }, { 177,5682 }, { 178,5682 },
 { 179,5682 }, { 180,5682 }, { 181,5682 }, { 182,5682 }, { 183,5682 },
 { 184,5682 }, { 185,5682 }, { 186,5682 }, { 187,5682 }, { 188,5682 },
 { 189,5682 }, { 190,5682 }, { 191,5682 }, { 192,5682 }, { 193,5682 },
 { 194,5682 }, { 195,5682 }, { 196,5682 }, { 197,5682 }, { 198,5682 },
 { 199,5682 }, { 200,5682 }, { 201,5682 }, { 202,5682 }, { 203,5682 },
 { 204,5682 }, { 205,5682 }, { 206,5682 }, { 207,5682 }, { 208,5682 },
 { 209,5682 }, { 210,5682 }, { 211,5682 }, { 212,5682 }, { 213,5682 },
 { 214,5682 }, { 215,5682 }, { 216,5682 }, { 217,5682 }, { 218,5682 },

 { 219,5682 }, { 220,5682 }, { 221,5682 }, { 222,5682 }, { 223,5682 },
 { 224,5682 }, { 225,5682 }, { 226,5682 }, { 227,5682 }, { 228,5682 },
 { 229,5682 }, { 230,5682 }, { 231,5682 }, { 232,5682 }, { 233,5682 },
 { 234,5682 }, { 235,5682 }, { 236,5682 }, { 237,5682 }, { 238,5682 },
 { 239,5682 }, { 240,5682 }, { 241,5682 }, { 242,5682 }, { 243,5682 },
 { 244,5682 }, { 245,5682 }, { 246,5682 }, { 247,5682 }, { 248,5682 },
 { 249,5682 }, { 250,5682 }, { 251,5682 }, { 252,5682 }, { 253,5682 },
 { 254,5682 }, { 255,5682 }, { 256,5682 }, {   0,   0 }, {   0,32686 },
 {   1,5424 }, {   2,5424 }, {   3,5424 }, {   4,5424 }, {   5,5424 },
 {   6,5424 }, {   7,5424 }, {   8,5424 }, {   9,5424 }, {  10,5424 },

 {  11,5424 }, {  12,5424 }, {  13,5424 }, {  14,5424 }, {  15,5424 },
 {  16,5424 }, {  17,5424 }, {  18,5424 }, {  19,5424 }, {  20,5424 },
 {  21,5424 }, {  22,5424 }, {  23,5424 }, {  24,5424 }, {  25,5424 },
 {  26,5424 }, {  27,5424 }, {  28,5424 }, {  29,5424 }, {  30,5424 },
 {  31,5424 }, {  32,5424 }, {  33,5424 }, {  34,2381 }, {  35,5424 },
 {  36,5424 }, {  37,5424 }, {  38,5424 }, {  39,5424 }, {  40,5424 },
 {  41,5424 }, {  42,5424 }, {  43,5424 }, {  44,5424 }, {  45,5424 },
 {  46,5424 }, {  47,5424 }, {  48,5424 }, {  49,5424 }, {  50,5424 },
 {  51,5424 }, {  52,5424 }, {  53,5424 }, {  54,5424 }, {  55,5424 },
 {  56,5424 }, {  57,5424 }, {  58,5424 }, {  59,5424 }, {  60,5424 },

 {  61,5424 }, {  62,5424 }, {  63,5424 }, {  64,5424 }, {  65,5424 },
 {  66,5424 }, {  67,5424 }, {  68,5424 }, {  69,5424 }, {  70,5424 },
 {  71,5424 }, {  72,5424 }, {  73,5424 }, {  74,5424 }, {  75,5424 },
 {  76,5424 }, {  77,5424 }, {  78,5424 }, {  79,5424 }, {  80,5424 },
 {  81,5424 }, {  82,5424 }, {  83,5424 }, {  84,5424 }, {  85,5424 },
 {  86,5424 }, {  87,5424 }, {  88,5424 }, {  89,5424 }, {  90,5424 },
 {  91,5424 }, {  92,5424 }, {  93,5424 }, {  94,5424 }, {  95,5424 },
 {  96,5424 }, {  97,5424 }, {  98,5424 }, {  99,5424 }, { 100,5424 },
 { 101,5424 }, { 102,5424 }, { 103,5424 }, { 104,5424 }, { 105,5424 },
 { 106,5424 }, { 107,5424 }, { 108,5424 }, { 109,5424 }, { 110,5424 },

 { 111,5424 }, { 112,5424 }, { 113,5424 }, { 114,5424 }, { 115,5424 },
 { 116,5424 }, { 117,5424 }, { 118,5424 }, { 119,5424 }, { 120,5424 },
 { 121,5424 }, { 122,5424 }, { 123,5424 }, { 124,5424 }, { 125,5424 },
 { 126,5424 }, { 127,5424 }, { 128,5424 }, { 129,5424 }, { 130,5424 },
 { 131,5424 }, { 132,5424 }, { 133,5424 }, { 134,5424 }, { 135,5424 },
 { 136,5424 }, { 137,5424 }, { 138,5424 }, { 139,5424 }, { 140,5424 },
 { 141,5424 }, { 142,5424 }, { 143,5424 }, { 144,5424 }, { 145,5424 },
 { 146,5424 }, { 147,5424 }, { 148,5424 }, { 149,5424 }, { 150,5424 },
 { 151,5424 }, { 152,5424 }, { 153,5424 }, { 154,5424 }, { 155,5424 },
 { 156,5424 }, { 157,5424 }, { 158,5424 }, { 159,5424 }, { 160,5424 },

 { 161,5424 }, { 162,5424 }, { 163,5424 }, { 164,5424 }, { 165,5424 },
 { 166,5424 }, { 167,5424 }, { 168,5424 }, { 169,5424 }, { 170,5424 },
 { 171,5424 }, { 172,5424 }, { 173,5424 }, { 174,5424 }, { 175,5424 },
 { 176,5424 }, { 177,5424 }, { 178,5424 }, { 179,5424 }, { 180,5424 },
 { 181,5424 }, { 182,5424 }, { 183,5424 }, { 184,5424 }, { 185,5424 },
 { 186,5424 }, { 187,5424 }, { 188,5424 }, { 189,5424 }, { 190,5424 },
 { 191,5424 }, { 192,5424 }, { 193,5424 }, { 194,5424 }, { 195,5424 },
 { 196,5424 }, { 197,5424 }, { 198,5424 }, { 199,5424 }, { 200,5424 },
 { 201,5424 }, { 202,5424 }, { 203,5424 }, { 204,5424 }, { 205,5424 },
 { 206,5424 }, { 207,5424 }, { 208,5424 }, { 209,5424 }, { 210,5424 },

 { 211,5424 }, { 212,5424 }, { 213,5424 }, { 214,5424 }, { 215,5424 },
 { 216,5424 }, { 217,5424 }, { 218,5424 }, { 219,5424 }, { 220,5424 },
 { 221,5424 }, { 222,5424 }, { 223,5424 }, { 224,5424 }, { 225,5424 },
 { 226,5424 }, { 227,5424 }, { 228,5424 }, { 229,5424 }, { 230,5424 },
 { 231,5424 }, { 232,5424 }, { 233,5424 }, { 234,5424 }, { 235,5424 },
 { 236,5424 }, { 237,5424 }, { 238,5424 }, { 239,5424 }, { 240,5424 },
 { 241,5424 }, { 242,5424 }, { 243,5424 }, { 244,5424 }, { 245,5424 },
 { 246,5424 }, { 247,5424 }, { 248,5424 }, { 249,5424 }, { 250,5424 },
 { 251,5424 }, { 252,5424 }, { 253,5424 }, { 254,5424 }, { 255,5424 },
 { 256,5424 }, {   0,  55 }, {   0,32428 }, {   1,2125 }, {   2,2125 },

 {   3,2125 }, {   4,2125 }, {   5,2125 }, {   6,2125 }, {   7,2125 },
 {   8,2125 }, {   9,7373 }, {  10,7378 }, {  11,2125 }, {  12,7373 },
 {  13,7373 }, {  14,2125 }, {  15,2125 }, {  16,2125 }, {  17,2125 },
 {  18,2125 }, {  19,2125 }, {  20,2125 }, {  21,2125 }, {  22,2125 },
 {  23,2125 }, {  24,2125 }, {  25,2125 }, {  26,2125 }, {  27,2125 },
 {  28,2125 }, {  29,2125 }, {  30,2125 }, {  31,2125 }, {  32,7373 },
 {  33,2125 }, {  34,2125 }, {  35,2125 }, {  36,2125 }, {  37,2125 },
 {  38,2125 }, {  39,2125 }, {  40,2125 }, {  41,2125 }, {  42,2125 },
 {  43,2125 }, {  44,2125 }, {  45,2127 }, {  46,2125 }, {  47,2125 },
 {  48,2125 }, {  49,2125 }, {  50,2125 }, {  51,2125 }, {  52,2125 },

 {  53,2125 }, {  54,2125 }, {  55,2125 }, {  56,2125 }, {  57,2125 },
 {  58,2125 }, {  59,2125 }, {  60,2125 }, {  61,2125 }, {  62,2125 },
 {  63,2125 }, {  64,2125 }, {  65,2125 }, {  66,2125 }, {  67,2125 },
 {  68,2125 }, {  69,2125 }, {  70,2125 }, {  71,2125 }, {  72,2125 },
 {  73,2125 }, {  74,2125 }, {  75,2125 }, {  76,2125 }, {  77,2125 },
 {  78,2125 }, {  79,2125 }, {  80,2125 }, {  81,2125 }, {  82,2125 },
 {  83,2125 }, {  84,2125 }, {  85,2141 }, {  86,2125 }, {  87,2125 },
 {  88,2125 }, {  89,2125 }, {  90,2125 }, {  91,2125 }, {  92,2125 },
 {  93,2125 }, {  94,2125 }, {  95,2125 }, {  96,2125 }, {  97,2125 },
 {  98,2125 }, {  99,2125 }, { 100,2125 }, { 101,2125 }, { 102,2125 },

 { 103,2125 }, { 104,2125 }, { 105,2125 }, { 106,2125 }, { 107,2125 },
 { 108,2125 }, { 109,2125 }, { 110,2125 }, { 111,2125 }, { 112,2125 },
 { 113,2125 }, { 114,2125 }, { 115,2125 }, { 116,2125 }, { 117,2141 },
 { 118,2125 }, { 119,2125 }, { 120,2125 }, { 121,2125 }, { 122,2125 },
 { 123,2125 }, { 124,2125 }, { 125,2125 }, { 126,2125 }, { 127,2125 },
 { 128,2125 }, { 129,2125 }, { 130,2125 }, { 131,2125 }, { 132,2125 },
 { 133,2125 }, { 134,2125 }, { 135,2125 }, { 136,2125 }, { 137,2125 },
 { 138,2125 }, { 139,2125 }, { 140,2125 }, { 141,2125 }, { 142,2125 },
 { 143,2125 }, { 144,2125 }, { 145,2125 }, { 146,2125 }, { 147,2125 },
 { 148,2125 }, { 149,2125 }, { 150,2125 }, { 151,2125 }, { 152,2125 },

 { 153,2125 }, { 154,2125 }, { 155,2125 }, { 156,2125 }, { 157,2125 },
 { 158,2125 }, { 159,2125 }, { 160,2125 }, { 161,2125 }, { 162,2125 },
 { 163,2125 }, { 164,2125 }, { 165,2125 }, { 166,2125 }, { 167,2125 },
 { 168,2125 }, { 169,2125 }, { 170,2125 }, { 171,2125 }, { 172,2125 },
 { 173,2125 }, { 174,2125 }, { 175,2125 }, { 176,2125 }, { 177,2125 },
 { 178,2125 }, { 179,2125 }, { 180,2125 }, { 181,2125 }, { 182,2125 },
 { 183,2125 }, { 184,2125 }, { 185,2125 }, { 186,2125 }, { 187,2125 },
 { 188,2125 }, { 189,2125 }, { 190,2125 }, { 191,2125 }, { 192,2125 },
 { 193,2125 }, { 194,2125 }, { 195,2125 }, { 196,2125 }, { 197,2125 },
 { 198,2125 }, { 199,2125 }, { 200,2125 }, { 201,2125 }, { 202,2125 },

 { 203,2125 }, { 204,2125 }, { 205,2125 }, { 206,2125 }, { 207,2125 },
 { 208,2125 }, { 209,2125 }, { 210,2125 }, { 211,2125 }, { 212,2125 },
 { 213,2125 }, { 214,2125 }, { 215,2125 }, { 216,2125 }, { 217,2125 },
 { 218,2125 }, { 219,2125 }, { 220,2125 }, { 221,2125 }, { 222,2125 },
 { 223,2125 }, { 224,2125 }, { 225,2125 }, { 226,2125 }, { 227,2125 },
 { 228,2125 }, { 229,2125 }, { 230,2125 }, { 231,2125 }, { 232,2125 },
 { 233,2125 }, { 234,2125 }, { 235,2125 }, { 236,2125 }, { 237,2125 },
 { 238,2125 }, { 239,2125 }, { 240,2125 }, { 241,2125 }, { 242,2125 },
 { 243,2125 }, { 244,2125 }, { 245,2125 }, { 246,2125 }, { 247,2125 },
 { 248,2125 }, { 249,2125 }, { 250,2125 }, { 251,2125 }, { 252,2125 },

 { 253,2125 }, { 254,2125 }, { 255,2125 }, { 256,2125 }, {   0,  55 },
 {   0,32170 }, {   1,1867 }, {   2,1867 }, {   3,1867 }, {   4,1867 },
 {   5,1867 }, {   6,1867 }, {   7,1867 }, {   8,1867 }, {   9,7115 },
 {  10,7120 }, {  11,1867 }, {  12,7115 }, {  13,7115 }, {  14,1867 },
 {  15,1867 }, {  16,1867 }, {  17,1867 }, {  18,1867 }, {  19,1867 },
 {  20,1867 }, {  21,1867 }, {  22,1867 }, {  23,1867 }, {  24,1867 },
 {  25,1867 }, {  26,1867 }, {  27,1867 }, {  28,1867 }, {  29,1867 },
 {  30,1867 }, {  31,1867 }, {  32,7115 }, {  33,1867 }, {  34,1867 },
 {  35,1867 }, {  36,1867 }, {  37,1867 }, {  38,1867 }, {  39,1867 },
 {  40,1867 }, {  41,1867 }, {  42,1867 }, {  43,1867 }, {  44,1867 },

 {  45,1869 }, {  46,1867 }, {  47,1867 }, {  48,1867 }, {  49,1867 },
 {  50,1867 }, {  51,1867 }, {  52,1867 }, {  53,1867 }, {  54,1867 },
 {  55,1867 }, {  56,1867 }, {  57,1867 }, {  58,1867 }, {  59,1867 },
 {  60,1867 }, {  61,1867 }, {  62,1867 }, {  63,1867 }, {  64,1867 },
 {  65,1867 }, {  66,1867 }, {  67,1867 }, {  68,1867 }, {  69,1867 },
 {  70,1867 }, {  71,1867 }, {  72,1867 }, {  73,1867 }, {  74,1867 },
 {  75,1867 }, {  76,1867 }, {  77,1867 }, {  78,1867 }, {  79,1867 },
 {  80,1867 }, {  81,1867 }, {  82,1867 }, {  83,1867 }, {  84,1867 },
 {  85,1883 }, {  86,1867 }, {  87,1867 }, {  88,1867 }, {  89,1867 },
 {  90,1867 }, {  91,1867 }, {  92,1867 }, {  93,1867 }, {  94,1867 },

 {  95,1867 }, {  96,1867 }, {  97,1867 }, {  98,1867 }, {  99,1867 },
 { 100,1867 }, { 101,1867 }, { 102,1867 }, { 103,1867 }, { 104,1867 },
 { 105,1867 }, { 106,1867 }, { 107,1867 }, { 108,1867 }, { 109,1867 },
 { 110,1867 }, { 111,1867 }, { 112,1867 }, { 113,1867 }, { 114,1867 },
 { 115,1867 }, { 116,1867 }, { 117,1883 }, { 118,1867 }, { 119,1867 },
 { 120,1867 }, { 121,1867 }, { 122,1867 }, { 123,1867 }, { 124,1867 },
 { 125,1867 }, { 126,1867 }, { 127,1867 }, { 128,1867 }, { 129,1867 },
 { 130,1867 }, { 131,1867 }, { 132,1867 }, { 133,1867 }, { 134,1867 },
 { 135,1867 }, { 136,1867 }, { 137,1867 }, { 138,1867 }, { 139,1867 },
 { 140,1867 }, { 141,1867 }, { 142,1867 }, { 143,1867 }, { 144,1867 },

 { 145,1867 }, { 146,1867 }, { 147,1867 }, { 148,1867 }, { 149,1867 },
 { 150,1867 }, { 151,1867 }, { 152,1867 }, { 153,1867 }, { 154,1867 },
 { 155,1867 }, { 156,1867 }, { 157,1867 }, { 158,1867 }, { 159,1867 },
 { 160,1867 }, { 161,1867 }, { 162,1867 }, { 163,1867 }, { 164,1867 },
 { 165,1867 }, { 166,1867 }, { 167,1867 }, { 168,1867 }, { 169,1867 },
 { 170,1867 }, { 171,1867 }, { 172,1867 }, { 173,1867 }, { 174,1867 },
 { 175,1867 }, { 176,1867 }, { 177,1867 }, { 178,1867 }, { 179,1867 },
 { 180,1867 }, { 181,1867 }, { 182,1867 }, { 183,1867 }, { 184,1867 },
 { 185,1867 }, { 186,1867 }, { 187,1867 }, { 188,1867 }, { 189,1867 },
 { 190,1867 }, { 191,1867 }, { 192,1867 }, { 193,1867 }, { 194,1867 },

 { 195,1867 }, { 196,1867 }, { 197,1867 }, { 198,1867 }, { 199,1867 },
 { 200,1867 }, { 201,1867 }, { 202,1867 }, { 203,1867 }, { 204,1867 },
 { 205,1867 }, { 206,1867 }, { 207,1867 }, { 208,1867 }, { 209,1867 },
 { 210,1867 }, { 211,1867 }, { 212,1867 }, { 213,1867 }, { 214,1867 },
 { 215,1867 }, { 216,1867 }, { 217,1867 }, { 218,1867 }, { 219,1867 },
 { 220,1867 }, { 221,1867 }, { 222,1867 }, { 223,1867 }, { 224,1867 },
 { 225,1867 }, { 226,1867 }, { 227,1867 }, { 228,1867 }, { 229,1867 },
 { 230,1867 }, { 231,1867 }, { 232,1867 }, { 233,1867 }, { 234,1867 },
 { 235,1867 }, { 236,1867 }, { 237,1867 }, { 238,1867 }, { 239,1867 },
 { 240,1867 }, { 241,1867 }, { 242,1867 }, { 243,1867 }, { 244,1867 },

 { 245,1867 }, { 246,1867 }, { 247,1867 }, { 248,1867 }, { 249,1867 },
 { 250,1867 }, { 251,1867 }, { 252,1867 }, { 253,1867 }, { 254,1867 },
 { 255,1867 }, { 256,1867 }, {   0,   0 }, {   0,31912 }, {   1,6034 },
 {   2,6034 }, {   3,6034 }, {   4,6034 }, {   5,6034 }, {   6,6034 },
 {   7,6034 }, {   8,6034 }, {   9,6034 }, {  10,6034 }, {  11,6034 },
 {  12,6034 }, {  13,6034 }, {  14,6034 }, {  15,6034 }, {  16,6034 },
 {  17,6034 }, {  18,6034 }, {  19,6034 }, {  20,6034 }, {  21,6034 },
 {  22,6034 }, {  23,6034 }, {  24,6034 }, {  25,6034 }, {  26,6034 },
 {  27,6034 }, {  28,6034 }, {  29,6034 }, {  30,6034 }, {  31,6034 },
 {  32,6034 }, {  33,6034 }, {  34,6034 }, {  35,6034 }, {  36,6034 },

 {  37,6034 }, {  38,6034 }, {  39,7112 }, {  40,6034 }, {  41,6034 },
 {  42,6034 }, {  43,6034 }, {  44,6034 }, {  45,6034 }, {  46,6034 },
 {  47,6034 }, {  48,6034 }, {  49,6034 }, {  50,6034 }, {  51,6034 },
 {  52,6034 }, {  53,6034 }, {  54,6034 }, {  55,6034 }, {  56,6034 },
 {  57,6034 }, {  58,6034 }, {  59,6034 }, {  60,6034 }, {  61,6034 },
 {  62,6034 }, {  63,6034 }, {  64,6034 }, {  65,6034 }, {  66,6034 },
 {  67,6034 }, {  68,6034 }, {  69,6034 }, {  70,6034 }, {  71,6034 },
 {  72,6034 }, {  73,6034 }, {  74,6034 }, {  75,6034 }, {  76,6034 },
 {  77,6034 }, {  78,6034 }, {  79,6034 }, {  80,6034 }, {  81,6034 },
 {  82,6034 }, {  83,6034 }, {  84,6034 }, {  85,6034 }, {  86,6034 },

 {  87,6034 }, {  88,6034 }, {  89,6034 }, {  90,6034 }, {  91,6034 },
 {  92,6034 }, {  93,6034 }, {  94,6034 }, {  95,6034 }, {  96,6034 },
 {  97,6034 }, {  98,6034 }, {  99,6034 }, { 100,6034 }, { 101,6034 },
 { 102,6034 }, { 103,6034 }, { 104,6034 }, { 105,6034 }, { 106,6034 },
 { 107,6034 }, { 108,6034 }, { 109,6034 }, { 110,6034 }, { 111,6034 },
 { 112,6034 }, { 113,6034 }, { 114,6034 }, { 115,6034 }, { 116,6034 },
 { 117,6034 }, { 118,6034 }, { 119,6034 }, { 120,6034 }, { 121,6034 },
 { 122,6034 }, { 123,6034 }, { 124,6034 }, { 125,6034 }, { 126,6034 },
 { 127,6034 }, { 128,6034 }, { 129,6034 }, { 130,6034 }, { 131,6034 },
 { 132,6034 }, { 133,6034 }, { 134,6034 }, { 135,6034 }, { 136,6034 },

 { 137,6034 }, { 138,6034 }, { 139,6034 }, { 140,6034 }, { 141,6034 },
 { 142,6034 }, { 143,6034 }, { 144,6034 }, { 145,6034 }, { 146,6034 },
 { 147,6034 }, { 148,6034 }, { 149,6034 }, { 150,6034 }, { 151,6034 },
 { 152,6034 }, { 153,6034 }, { 154,6034 }, { 155,6034 }, { 156,6034 },
 { 157,6034 }, { 158,6034 }, { 159,6034 }, { 160,6034 }, { 161,6034 },
 { 162,6034 }, { 163,6034 }, { 164,6034 }, { 165,6034 }, { 166,6034 },
 { 167,6034 }, { 168,6034 }, { 169,6034 }, { 170,6034 }, { 171,6034 },
 { 172,6034 }, { 173,6034 }, { 174,6034 }, { 175,6034 }, { 176,6034 },
 { 177,6034 }, { 178,6034 }, { 179,6034 }, { 180,6034 }, { 181,6034 },
 { 182,6034 }, { 183,6034 }, { 184,6034 }, { 185,6034 }, { 186,6034 },

 { 187,6034 }, { 188,6034 }, { 189,6034 }, { 190,6034 }, { 191,6034 },
 { 192,6034 }, { 193,6034 }, { 194,6034 }, { 195,6034 }, { 196,6034 },
 { 197,6034 }, { 198,6034 }, { 199,6034 }, { 200,6034 }, { 201,6034 },
 { 202,6034 }, { 203,6034 }, { 204,6034 }, { 205,6034 }, { 206,6034 },
 { 207,6034 }, { 208,6034 }, { 209,6034 }, { 210,6034 }, { 211,6034 },
 { 212,6034 }, { 213,6034 }, { 214,6034 }, { 215,6034 }, { 216,6034 },
 { 217,6034 }, { 218,6034 }, { 219,6034 }, { 220,6034 }, { 221,6034 },
 { 222,6034 }, { 223,6034 }, { 224,6034 }, { 225,6034 }, { 226,6034 },
 { 227,6034 }, { 228,6034 }, { 229,6034 }, { 230,6034 }, { 231,6034 },
 { 232,6034 }, { 233,6034 }, { 234,6034 }, { 235,6034 }, { 236,6034 },

 { 237,6034 }, { 238,6034 }, { 239,6034 }, { 240,6034 }, { 241,6034 },
 { 242,6034 }, { 243,6034 }, { 244,6034 }, { 245,6034 }, { 246,6034 },
 { 247,6034 }, { 248,6034 }, { 249,6034 }, { 250,6034 }, { 251,6034 },
 { 252,6034 }, { 253,6034 }, { 254,6034 }, { 255,6034 }, { 256,6034 },
 {   0,   0 }, {   0,31654 }, {   1,5776 }, {   2,5776 }, {   3,5776 },
 {   4,5776 }, {   5,5776 }, {   6,5776 }, {   7,5776 }, {   8,5776 },
 {   9,5776 }, {  10,5776 }, {  11,5776 }, {  12,5776 }, {  13,5776 },
 {  14,5776 }, {  15,5776 }, {  16,5776 }, {  17,5776 }, {  18,5776 },
 {  19,5776 }, {  20,5776 }, {  21,5776 }, {  22,5776 }, {  23,5776 },
 {  24,5776 }, {  25,5776 }, {  26,5776 }, {  27,5776 }, {  28,5776 },

 {  29,5776 }, {  30,5776 }, {  31,5776 }, {  32,5776 }, {  33,5776 },
 {  34,5776 }, {  35,5776 }, {  36,5776 }, {  37,5776 }, {  38,5776 },
 {  39,6854 }, {  40,5776 }, {  41,5776 }, {  42,5776 }, {  43,5776 },
 {  44,5776 }, {  45,5776 }, {  46,5776 }, {  47,5776 }, {  48,5776 },
 {  49,5776 }, {  50,5776 }, {  51,5776 }, {  52,5776 }, {  53,5776 },
 {  54,5776 }, {  55,5776 }, {  56,5776 }, {  57,5776 }, {  58,5776 },
 {  59,5776 }, {  60,5776 }, {  61,5776 }, {  62,5776 }, {  63,5776 },
 {  64,5776 }, {  65,5776 }, {  66,5776 }, {  67,5776 }, {  68,5776 },
 {  69,5776 }, {  70,5776 }, {  71,5776 }, {  72,5776 }, {  73,5776 },
 {  74,5776 }, {  75,5776 }, {  76,5776 }, {  77,5776 }, {  78,5776 },

 {  79,5776 }, {  80,5776 }, {  81,5776 }, {  82,5776 }, {  83,5776 },
 {  84,5776 }, {  85,5776 }, {  86,5776 }, {  87,5776 }, {  88,5776 },
 {  89,5776 }, {  90,5776 }, {  91,5776 }, {  92,5776 }, {  93,5776 },
 {  94,5776 }, {  95,5776 }, {  96,5776 }, {  97,5776 }, {  98,5776 },
 {  99,5776 }, { 100,5776 }, { 101,5776 }, { 102,5776 }, { 103,5776 },
 { 104,5776 }, { 105,5776 }, { 106,5776 }, { 107,5776 }, { 108,5776 },
 { 109,5776 }, { 110,5776 }, { 111,5776 }, { 112,5776 }, { 113,5776 },
 { 114,5776 }, { 115,5776 }, { 116,5776 }, { 117,5776 }, { 118,5776 },
 { 119,5776 }, { 120,5776 }, { 121,5776 }, { 122,5776 }, { 123,5776 },
 { 124,5776 }, { 125,5776 }, { 126,5776 }, { 127,5776 }, { 128,5776 },

 { 129,5776 }, { 130,5776 }, { 131,5776 }, { 132,5776 }, { 133,5776 },
 { 134,5776 }, { 135,5776 }, { 136,5776 }, { 137,5776 }, { 138,5776 },
 { 139,5776 }, { 140,5776 }, { 141,5776 }, { 142,5776 }, { 143,5776 },
 { 144,5776 }, { 145,5776 }, { 146,5776 }, { 147,5776 }, { 148,5776 },
 { 149,5776 }, { 150,5776 }, { 151,5776 }, { 152,5776 }, { 153,5776 },
 { 154,5776 }, { 155,5776 }, { 156,5776 }, { 157,5776 }, { 158,5776 },
 { 159,5776 }, { 160,5776 }, { 161,5776 }, { 162,5776 }, { 163,5776 },
 { 164,5776 }, { 165,5776 }, { 166,5776 }, { 167,5776 }, { 168,5776 },
 { 169,5776 }, { 170,5776 }, { 171,5776 }, { 172,5776 }, { 173,5776 },
 { 174,5776 }, { 175,5776 }, { 176,5776 }, { 177,5776 }, { 178,5776 },

 { 179,5776 }, { 180,5776 }, { 181,5776 }, { 182,5776 }, { 183,5776 },
 { 184,5776 }, { 185,5776 }, { 186,5776 }, { 187,5776 }, { 188,5776 },
 { 189,5776 }, { 190,5776 }, { 191,5776 }, { 192,5776 }, { 193,5776 },
 { 194,5776 }, { 195,5776 }, { 196,5776 }, { 197,5776 }, { 198,5776 },
 { 199,5776 }, { 200,5776 }, { 201,5776 }, { 202,5776 }, { 203,5776 },
 { 204,5776 }, { 205,5776 }, { 206,5776 }, { 207,5776 }, { 208,5776 },
 { 209,5776 }, { 210,5776 }, { 211,5776 }, { 212,5776 }, { 213,5776 },
 { 214,5776 }, { 215,5776 }, { 216,5776 }, { 217,5776 }, { 218,5776 },
 { 219,5776 }, { 220,5776 }, { 221,5776 }, { 222,5776 }, { 223,5776 },
 { 224,5776 }, { 225,5776 }, { 226,5776 }, { 227,5776 }, { 228,5776 },

 { 229,5776 }, { 230,5776 }, { 231,5776 }, { 232,5776 }, { 233,5776 },
 { 234,5776 }, { 235,5776 }, { 236,5776 }, { 237,5776 }, { 238,5776 },
 { 239,5776 }, { 240,5776 }, { 241,5776 }, { 242,5776 }, { 243,5776 },
 { 244,5776 }, { 245,5776 }, { 246,5776 }, { 247,5776 }, { 248,5776 },
 { 249,5776 }, { 250,5776 }, { 251,5776 }, { 252,5776 }, { 253,5776 },
 { 254,5776 }, { 255,5776 }, { 256,5776 }, {   0,  28 }, {   0,31396 },
 {   1,1113 }, {   2,1113 }, {   3,1113 }, {   4,1113 }, {   5,1113 },
 {   6,1113 }, {   7,1113 }, {   8,1113 }, {   9,6601 }, {  10,6617 },
 {  11,1113 }, {  12,6601 }, {  13,6601 }, {  14,1113 }, {  15,1113 },
 {  16,1113 }, {  17,1113 }, {  18,1113 }, {  19,1113 }, {  20,1113 },

 {  21,1113 }, {  22,1113 }, {  23,1113 }, {  24,1113 }, {  25,1113 },
 {  26,1113 }, {  27,1113 }, {  28,1113 }, {  29,1113 }, {  30,1113 },
 {  31,1113 }, {  32,6601 }, {  33,1113 }, {  34,1113 }, {  35,1113 },
 {  36,1113 }, {  37,1113 }, {  38,1113 }, {  39,1113 }, {  40,1113 },
 {  41,1113 }, {  42,1113 }, {  43,1113 }, {  44,1113 }, {  45,1131 },
 {  46,1113 }, {  47,1113 }, {  48,1113 }, {  49,1113 }, {  50,1113 },
 {  51,1113 }, {  52,1113 }, {  53,1113 }, {  54,1113 }, {  55,1113 },
 {  56,1113 }, {  57,1113 }, {  58,1113 }, {  59,1113 }, {  60,1113 },
 {  61,1113 }, {  62,1113 }, {  63,1113 }, {  64,1113 }, {  65,1113 },
 {  66,1113 }, {  67,1113 }, {  68,1113 }, {  69,1113 }, {  70,1113 },

 {  71,1113 }, {  72,1113 }, {  73,1113 }, {  74,1113 }, {  75,1113 },
 {  76,1113 }, {  77,1113 }, {  78,1113 }, {  79,1113 }, {  80,1113 },
 {  81,1113 }, {  82,1113 }, {  83,1113 }, {  84,1113 }, {  85,1139 },
 {  86,1113 }, {  87,1113 }, {  88,1113 }, {  89,1113 }, {  90,1113 },
 {  91,1113 }, {  92,1113 }, {  93,1113 }, {  94,1113 }, {  95,1113 },
 {  96,1113 }, {  97,1113 }, {  98,1113 }, {  99,1113 }, { 100,1113 },
 { 101,1113 }, { 102,1113 }, { 103,1113 }, { 104,1113 }, { 105,1113 },
 { 106,1113 }, { 107,1113 }, { 108,1113 }, { 109,1113 }, { 110,1113 },
 { 111,1113 }, { 112,1113 }, { 113,1113 }, { 114,1113 }, { 115,1113 },
 { 116,1113 }, { 117,1139 }, { 118,1113 }, { 119,1113 }, { 120,1113 },

 { 121,1113 }, { 122,1113 }, { 123,1113 }, { 124,1113 }, { 125,1113 },
 { 126,1113 }, { 127,1113 }, { 128,1113 }, { 129,1113 }, { 130,1113 },
 { 131,1113 }, { 132,1113 }, { 133,1113 }, { 134,1113 }, { 135,1113 },
 { 136,1113 }, { 137,1113 }, { 138,1113 }, { 139,1113 }, { 140,1113 },
 { 141,1113 }, { 142,1113 }, { 143,1113 }, { 144,1113 }, { 145,1113 },
 { 146,1113 }, { 147,1113 }, { 148,1113 }, { 149,1113 }, { 150,1113 },
 { 151,1113 }, { 152,1113 }, { 153,1113 }, { 154,1113 }, { 155,1113 },
 { 156,1113 }, { 157,1113 }, { 158,1113 }, { 159,1113 }, { 160,1113 },
 { 161,1113 }, { 162,1113 }, { 163,1113 }, { 164,1113 }, { 165,1113 },
 { 166,1113 }, { 167,1113 }, { 168,1113 }, { 169,1113 }, { 170,1113 },

 { 171,1113 }, { 172,1113 }, { 173,1113 }, { 174,1113 }, { 175,1113 },
 { 176,1113 }, { 177,1113 }, { 178,1113 }, { 179,1113 }, { 180,1113 },
 { 181,1113 }, { 182,1113 }, { 183,1113 }, { 184,1113 }, { 185,1113 },
 { 186,1113 }, { 187,1113 }, { 188,1113 }, { 189,1113 }, { 190,1113 },
 { 191,1113 }, { 192,1113 }, { 193,1113 }, { 194,1113 }, { 195,1113 },
 { 196,1113 }, { 197,1113 }, { 198,1113 }, { 199,1113 }, { 200,1113 },
 { 201,1113 }, { 202,1113 }, { 203,1113 }, { 204,1113 }, { 205,1113 },
 { 206,1113 }, { 207,1113 }, { 208,1113 }, { 209,1113 }, { 210,1113 },
 { 211,1113 }, { 212,1113 }, { 213,1113 }, { 214,1113 }, { 215,1113 },
 { 216,1113 }, { 217,1113 }, { 218,1113 }, { 219,1113 }, { 220,1113 },

 { 221,1113 }, { 222,1113 }, { 223,1113 }, { 224,1113 }, { 225,1113 },
 { 226,1113 }, { 227,1113 }, { 228,1113 }, { 229,1113 }, { 230,1113 },
 { 231,1113 }, { 232,1113 }, { 233,1113 }, { 234,1113 }, { 235,1113 },
 { 236,1113 }, { 237,1113 }, { 238,1113 }, { 239,1113 }, { 240,1113 },
 { 241,1113 }, { 242,1113 }, { 243,1113 }, { 244,1113 }, { 245,1113 },
 { 246,1113 }, { 247,1113 }, { 248,1113 }, { 249,1113 }, { 250,1113 },
 { 251,1113 }, { 252,1113 }, { 253,1113 }, { 254,1113 }, { 255,1113 },
 { 256,1113 }, {   0,  28 }, {   0,31138 }, {   1, 855 }, {   2, 855 },
 {   3, 855 }, {   4, 855 }, {   5, 855 }, {   6, 855 }, {   7, 855 },
 {   8, 855 }, {   9,6343 }, {  10,6359 }, {  11, 855 }, {  12,6343 },

 {  13,6343 }, {  14, 855 }, {  15, 855 }, {  16, 855 }, {  17, 855 },
 {  18, 855 }, {  19, 855 }, {  20, 855 }, {  21, 855 }, {  22, 855 },
 {  23, 855 }, {  24, 855 }, {  25, 855 }, {  26, 855 }, {  27, 855 },
 {  28, 855 }, {  29, 855 }, {  30, 855 }, {  31, 855 }, {  32,6343 },
 {  33, 855 }, {  34, 855 }, {  35, 855 }, {  36, 855 }, {  37, 855 },
 {  38, 855 }, {  39, 855 }, {  40, 855 }, {  41, 855 }, {  42, 855 },
 {  43, 855 }, {  44, 855 }, {  45, 873 }, {  46, 855 }, {  47, 855 },
 {  48, 855 }, {  49, 855 }, {  50, 855 }, {  51, 855 }, {  52, 855 },
 {  53, 855 }, {  54, 855 }, {  55, 855 }, {  56, 855 }, {  57, 855 },
 {  58, 855 }, {  59, 855 }, {  60, 855 }, {  61, 855 }, {  62, 855 },

 {  63, 855 }, {  64, 855 }, {  65, 855 }, {  66, 855 }, {  67, 855 },
 {  68, 855 }, {  69, 855 }, {  70, 855 }, {  71, 855 }, {  72, 855 },
 {  73, 855 }, {  74, 855 }, {  75, 855 }, {  76, 855 }, {  77, 855 },
 {  78, 855 }, {  79, 855 }, {  80, 855 }, {  81, 855 }, {  82, 855 },
 {  83, 855 }, {  84, 855 }, {  85, 881 }, {  86, 855 }, {  87, 855 },
 {  88, 855 }, {  89, 855 }, {  90, 855 }, {  91, 855 }, {  92, 855 },
 {  93, 855 }, {  94, 855 }, {  95, 855 }, {  96, 855 }, {  97, 855 },
 {  98, 855 }, {  99, 855 }, { 100, 855 }, { 101, 855 }, { 102, 855 },
 { 103, 855 }, { 104, 855 }, { 105, 855 }, { 106, 855 }, { 107, 855 },
 { 108, 855 }, { 109, 855 }, { 110, 855 }, { 111, 855 }, { 112, 855 },

 { 113, 855 }, { 114, 855 }, { 115, 855 }, { 116, 855 }, { 117, 881 },
 { 118, 855 }, { 119, 855 }, { 120, 855 }, { 121, 855 }, { 122, 855 },
 { 123, 855 }, { 124, 855 }, { 125, 855 }, { 126, 855 }, { 127, 855 },
 { 128, 855 }, { 129, 855 }, { 130, 855 }, { 131, 855 }, { 132, 855 },
 { 133, 855 }, { 134, 855 }, { 135, 855 }, { 136, 855 }, { 137, 855 },
 { 138, 855 }, { 139, 855 }, { 140, 855 }, { 141, 855 }, { 142, 855 },
 { 143, 855 }, { 144, 855 }, { 145, 855 }, { 146, 855 }, { 147, 855 },
 { 148, 855 }, { 149, 855 }, { 150, 855 }, { 151, 855 }, { 152, 855 },
 { 153, 855 }, { 154, 855 }, { 155, 855 }, { 156, 855 }, { 157, 855 },
 { 158, 855 }, { 159, 855 }, { 160, 855 }, { 161, 855 }, { 162, 855 },

 { 163, 855 }, { 164, 855 }, { 165, 855 }, { 166, 855 }, { 167, 855 },
 { 168, 855 }, { 169, 855 }, { 170, 855 }, { 171, 855 }, { 172, 855 },
 { 173, 855 }, { 174, 855 }, { 175, 855 }, { 176, 855 }, { 177, 855 },
 { 178, 855 }, { 179, 855 }, { 180, 855 }, { 181, 855 }, { 182, 855 },
 { 183, 855 }, { 184, 855 }, { 185, 855 }, { 186, 855 }, { 187, 855 },
 { 188, 855 }, { 189, 855 }, { 190, 855 }, { 191, 855 }, { 192, 855 },
 { 193, 855 }, { 194, 855 }, { 195, 855 }, { 196, 855 }, { 197, 855 },
 { 198, 855 }, { 199, 855 }, { 200, 855 }, { 201, 855 }, { 202, 855 },
 { 203, 855 }, { 204, 855 }, { 205, 855 }, { 206, 855 }, { 207, 855 },
 { 208, 855 }, { 209, 855 }, { 210, 855 }, { 211, 855 }, { 212, 855 },

 { 213, 855 }, { 214, 855 }, { 215, 855 }, { 216, 855 }, { 217, 855 },
 { 218, 855 }, { 219, 855 }, { 220, 855 }, { 221, 855 }, { 222, 855 },
 { 223, 855 }, { 224, 855 }, { 225, 855 }, { 226, 855 }, { 227, 855 },
 { 228, 855 }, { 229, 855 }, { 230, 855 }, { 231, 855 }, { 232, 855 },
 { 233, 855 }, { 234, 855 }, { 235, 855 }, { 236, 855 }, { 237, 855 },
 { 238, 855 }, { 239, 855 }, { 240, 855 }, { 241, 855 }, { 242, 855 },
 { 243, 855 }, { 244, 855 }, { 245, 855 }, { 246, 855 }, { 247, 855 },
 { 248, 855 }, { 249, 855 }, { 250, 855 }, { 251, 855 }, { 252, 855 },
 { 253, 855 }, { 254, 855 }, { 255, 855 }, { 256, 855 }, {   0,   0 },
 {   0,30880 }, {   1, 633 }, {   2, 633 }, {   3, 633 }, {   4, 633 },

 {   5, 633 }, {   6, 633 }, {   7, 633 }, {   8, 633 }, {   9, 633 },
 {  10, 635 }, {  11, 633 }, {  12, 633 }, {  13, 633 }, {  14, 633 },
 {  15, 633 }, {  16, 633 }, {  17, 633 }, {  18, 633 }, {  19, 633 },
 {  20, 633 }, {  21, 633 }, {  22, 633 }, {  23, 633 }, {  24, 633 },
 {  25, 633 }, {  26, 633 }, {  27, 633 }, {  28, 633 }, {  29, 633 },
 {  30, 633 }, {  31, 633 }, {  32, 633 }, {  33, 633 }, {  34, 633 },
 {  35, 633 }, {  36, 633 }, {  37, 633 }, {  38, 633 }, {  39, 633 },
 {  40, 633 }, {  41, 633 }, {  42, 633 }, {  43, 633 }, {  44, 633 },
 {  45, 633 }, {  46, 633 }, {  47, 633 }, {  48, 633 }, {  49, 633 },
 {  50, 633 }, {  51, 633 }, {  52, 633 }, {  53, 633 }, {  54, 633 },

 {  55, 633 }, {  56, 633 }, {  57, 633 }, {  58, 633 }, {  59, 633 },
 {  60, 633 }, {  61, 633 }, {  62, 633 }, {  63, 633 }, {  64, 633 },
 {  65, 633 }, {  66, 633 }, {  67, 633 }, {  68, 633 }, {  69, 633 },
 {  70, 633 }, {  71, 633 }, {  72, 633 }, {  73, 633 }, {  74, 633 },
 {  75, 633 }, {  76, 633 }, {  77, 633 }, {  78, 633 }, {  79, 633 },
 {  80, 633 }, {  81, 633 }, {  82, 633 }, {  83, 633 }, {  84, 633 },
 {  85, 633 }, {  86, 633 }, {  87, 633 }, {  88, 633 }, {  89, 633 },
 {  90, 633 }, {  91, 633 }, {  92, 637 }, {  93, 633 }, {  94, 633 },
 {  95, 633 }, {  96, 633 }, {  97, 633 }, {  98, 633 }, {  99, 633 },
 { 100, 633 }, { 101, 633 }, { 102, 633 }, { 103, 633 }, { 104, 633 },

 { 105, 633 }, { 106, 633 }, { 107, 633 }, { 108, 633 }, { 109, 633 },
 { 110, 633 }, { 111, 633 }, { 112, 633 }, { 113, 633 }, { 114, 633 },
 { 115, 633 }, { 116, 633 }, { 117, 633 }, { 118, 633 }, { 119, 633 },
 { 120, 633 }, { 121, 633 }, { 122, 633 }, { 123, 633 }, { 124, 633 },
 { 125, 633 }, { 126, 633 }, { 127, 633 }, { 128, 633 }, { 129, 633 },
 { 130, 633 }, { 131, 633 }, { 132, 633 }, { 133, 633 }, { 134, 633 },
 { 135, 633 }, { 136, 633 }, { 137, 633 }, { 138, 633 }, { 139, 633 },
 { 140, 633 }, { 141, 633 }, { 142, 633 }, { 143, 633 }, { 144, 633 },
 { 145, 633 }, { 146, 633 }, { 147, 633 }, { 148, 633 }, { 149, 633 },
 { 150, 633 }, { 151, 633 }, { 152, 633 }, { 153, 633 }, { 154, 633 },

 { 155, 633 }, { 156, 633 }, { 157, 633 }, { 158, 633 }, { 159, 633 },
 { 160, 633 }, { 161, 633 }, { 162, 633 }, { 163, 633 }, { 164, 633 },
 { 165, 633 }, { 166, 633 }, { 167, 633 }, { 168, 633 }, { 169, 633 },
 { 170, 633 }, { 171, 633 }, { 172, 633 }, { 173, 633 }, { 174, 633 },
 { 175, 633 }, { 176, 633 }, { 177, 633 }, { 178, 633 }, { 179, 633 },
 { 180, 633 }, { 181, 633 }, { 182, 633 }, { 183, 633 }, { 184, 633 },
 { 185, 633 }, { 186, 633 }, { 187, 633 }, { 188, 633 }, { 189, 633 },
 { 190, 633 }, { 191, 633 }, { 192, 633 }, { 193, 633 }, { 194, 633 },
 { 195, 633 }, { 196, 633 }, { 197, 633 }, { 198, 633 }, { 199, 633 },
 { 200, 633 }, { 201, 633 }, { 202, 633 }, { 203, 633 }, { 204, 633 },

 { 205, 633 }, { 206, 633 }, { 207, 633 }, { 208, 633 }, { 209, 633 },
 { 210, 633 }, { 211, 633 }, { 212, 633 }, { 213, 633 }, { 214, 633 },
 { 215, 633 }, { 216, 633 }, { 217, 633 }, { 218, 633 }, { 219, 633 },
 { 220, 633 }, { 221, 633 }, { 222, 633 }, { 223, 633 }, { 224, 633 },
 { 225, 633 }, { 226, 633 }, { 227, 633 }, { 228, 633 }, { 229, 633 },
 { 230, 633 }, { 231, 633 }, { 232, 633 }, { 233, 633 }, { 234, 633 },
 { 235, 633 }, { 236, 633 }, { 237, 633 }, { 238, 633 }, { 239, 633 },
 { 240, 633 }, { 241, 633 }, { 242, 633 }, { 243, 633 }, { 244, 633 },
 { 245, 633 }, { 246, 633 }, { 247, 633 }, { 248, 633 }, { 249, 633 },
 { 250, 633 }, { 251, 633 }, { 252, 633 }, { 253, 633 }, { 254, 633 },

 { 255, 633 }, { 256, 633 }, {   0,   0 }, {   0,30622 }, {   1, 375 },
 {   2, 375 }, {   3, 375 }, {   4, 375 }, {   5, 375 }, {   6, 375 },
 {   7, 375 }, {   8, 375 }, {   9, 375 }, {  10, 377 }, {  11, 375 },
 {  12, 375 }, {  13, 375 }, {  14, 375 }, {  15, 375 }, {  16, 375 },
 {  17, 375 }, {  18, 375 }, {  19, 375 }, {  20, 375 }, {  21, 375 },
 {  22, 375 }, {  23, 375 }, {  24, 375 }, {  25, 375 }, {  26, 375 },
 {  27, 375 }, {  28, 375 }, {  29, 375 }, {  30, 375 }, {  31, 375 },
 {  32, 375 }, {  33, 375 }, {  34, 375 }, {  35, 375 }, {  36, 375 },
 {  37, 375 }, {  38, 375 }, {  39, 375 }, {  40, 375 }, {  41, 375 },
 {  42, 375 }, {  43, 375 }, {  44, 375 }, {  45, 375 }, {  46, 375 },

 {  47, 375 }, {  48, 375 }, {  49, 375 }, {  50, 375 }, {  51, 375 },
 {  52, 375 }, {  53, 375 }, {  54, 375 }, {  55, 375 }, {  56, 375 },
 {  57, 375 }, {  58, 375 }, {  59, 375 }, {  60, 375 }, {  61, 375 },
 {  62, 375 }, {  63, 375 }, {  64, 375 }, {  65, 375 }, {  66, 375 },
 {  67, 375 }, {  68, 375 }, {  69, 375 }, {  70, 375 }, {  71, 375 },
 {  72, 375 }, {  73, 375 }, {  74, 375 }, {  75, 375 }, {  76, 375 },
 {  77, 375 }, {  78, 375 }, {  79, 375 }, {  80, 375 }, {  81, 375 },
 {  82, 375 }, {  83, 375 }, {  84, 375 }, {  85, 375 }, {  86, 375 },
 {  87, 375 }, {  88, 375 }, {  89, 375 }, {  90, 375 }, {  91, 375 },
 {  92, 379 }, {  93, 375 }, {  94, 375 }, {  95, 375 }, {  96, 375 },

 {  97, 375 }, {  98, 375 }, {  99, 375 }, { 100, 375 }, { 101, 375 },
 { 102, 375 }, { 103, 375 }, { 104, 375 }, { 105, 375 }, { 106, 375 },
 { 107, 375 }, { 108, 375 }, { 109, 375 }, { 110, 375 }, { 111, 375 },
 { 112, 375 }, { 113, 375 }, { 114, 375 }, { 115, 375 }, { 116, 375 },
 { 117, 375 }, { 118, 375 }, { 119, 375 }, { 120, 375 }, { 121, 375 },
 { 122, 375 }, { 123, 375 }, { 124, 375 }, { 125, 375 }, { 126, 375 },
 { 127, 375 }, { 128, 375 }, { 129, 375 }, { 130, 375 }, { 131, 375 },
 { 132, 375 }, { 133, 375 }, { 134, 375 }, { 135, 375 }, { 136, 375 },
 { 137, 375 }, { 138, 375 }, { 139, 375 }, { 140, 375 }, { 141, 375 },
 { 142, 375 }, { 143, 375 }, { 144, 375 }, { 145, 375 }, { 146, 375 },

 { 147, 375 }, { 148, 375 }, { 149, 375 }, { 150, 375 }, { 151, 375 },
 { 152, 375 }, { 153, 375 }, { 154, 375 }, { 155, 375 }, { 156, 375 },
 { 157, 375 }, { 158, 375 }, { 159, 375 }, { 160, 375 }, { 161, 375 },
 { 162, 375 }, { 163, 375 }, { 164, 375 }, { 165, 375 }, { 166, 375 },
 { 167, 375 }, { 168, 375 }, { 169, 375 }, { 170, 375 }, { 171, 375 },
 { 172, 375 }, { 173, 375 }, { 174, 375 }, { 175, 375 }, { 176, 375 },
 { 177, 375 }, { 178, 375 }, { 179, 375 }, { 180, 375 }, { 181, 375 },
 { 182, 375 }, { 183, 375 }, { 184, 375 }, { 185, 375 }, { 186, 375 },
 { 187, 375 }, { 188, 375 }, { 189, 375 }, { 190, 375 }, { 191, 375 },
 { 192, 375 }, { 193, 375 }, { 194, 375 }, { 195, 375 }, { 196, 375 },

 { 197, 375 }, { 198, 375 }, { 199, 375 }, { 200, 375 }, { 201, 375 },
 { 202, 375 }, { 203, 375 }, { 204, 375 }, { 205, 375 }, { 206, 375 },
 { 207, 375 }, { 208, 375 }, { 209, 375 }, { 210, 375 }, { 211, 375 },
 { 212, 375 }, { 213, 375 }, { 214, 375 }, { 215, 375 }, { 216, 375 },
 { 217, 375 }, { 218, 375 }, { 219, 375 }, { 220, 375 }, { 221, 375 },
 { 222, 375 }, { 223, 375 }, { 224, 375 }, { 225, 375 }, { 226, 375 },
 { 227, 375 }, { 228, 375 }, { 229, 375 }, { 230, 375 }, { 231, 375 },
 { 232, 375 }, { 233, 375 }, { 234, 375 }, { 235, 375 }, { 236, 375 },
 { 237, 375 }, { 238, 375 }, { 239, 375 }, { 240, 375 }, { 241, 375 },
 { 242, 375 }, { 243, 375 }, { 244, 375 }, { 245, 375 }, { 246, 375 },

 { 247, 375 }, { 248, 375 }, { 249, 375 }, { 250, 375 }, { 251, 375 },
 { 252, 375 }, { 253, 375 }, { 254, 375 }, { 255, 375 }, { 256, 375 },
 {   0,  79 }, {   0,30364 }, {   0,   1 }, {   0,30362 }, {   0,  49 },
 {   0,30360 }, {   0,   0 }, {   0,   1 }, {   0,30357 }, {   0,  69 },
 {   0,30355 }, {   0,   0 }, {   9,5604 }, {  10,5604 }, {   0,   0 },
 {  12,5604 }, {  13,5604 }, {   9,5599 }, {  10,5599 }, {   0,   0 },
 {  12,5599 }, {  13,5599 }, {   0,  19 }, {   0,30342 }, {   0,  68 },
 {   0,30340 }, {   0,   0 }, {   0,  68 }, {   0,30337 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   6 }, {   0,30332 }, {   0,   0 },
 {  32,5604 }, {   0,   6 }, {   0,30328 }, {   0,   0 }, {   0,   0 },

 {  32,5599 }, {   0,  51 }, {   0,30323 }, {  33,5599 }, {   0,   0 },
 {  35,5599 }, {   0,   0 }, {  37,5599 }, {  38,5599 }, {   0,  69 },
 {   0,30315 }, {   0,   0 }, {  42,5599 }, {  43,5599 }, {   0,   0 },
 {  45,5599 }, {   0,   0 }, {  47,5599 }, {   0,   0 }, {   0,  52 },
 {   0,30305 }, {   0,  54 }, {   0,30303 }, {   0,  54 }, {   0,30301 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  60,5599 }, {  61,5639 }, {  62,5599 }, {  63,5599 }, {  64,5599 },
 {  42, 348 }, {  34, 346 }, {   0,  54 }, {   0,30287 }, {  42,7554 },
 {  47, 353 }, {   0,  27 }, {   0,30283 }, {  33,5559 }, {   0,   0 },
 {  35,5559 }, {  58, 100 }, {  37,5559 }, {  38,5559 }, {  61, 102 },

 {   0,   0 }, {   0,   0 }, {  42,5559 }, {  43,5559 }, {  34, 328 },
 {  45,5559 }, {   0,   0 }, {  47,5559 }, {   0,   0 }, {   0,  27 },
 {   0,30265 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  94,5599 },
 {   0,   0 }, {  96,5599 }, {   0,  27 }, {   0,30257 }, {  45,9439 },
 {  60,5559 }, {  61,5559 }, {  62,5559 }, {  63,5559 }, {  64,5559 },
 {   0,  79 }, {   0,30249 }, {   0,  35 }, {   0,30247 }, {   0,  36 },
 {   0,30245 }, {   0,  35 }, {   0,30243 }, {   0,  43 }, {   0,30241 },
 {   0,  61 }, {   0,30239 }, {   0,  60 }, {   0,30237 }, {   0,  62 },
 {   0,30235 }, {   0,   8 }, {   0,30233 }, {   0,   0 }, { 124,5599 },
 {   0,   0 }, { 126,5599 }, {   0,  20 }, {   0,30227 }, {   0,  18 },

 {   0,30225 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  94,5559 },
 {  45,9716 }, {  96,5559 }, {  69, 327 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  36,   8 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  48,5550 },
 {  49,5550 }, {  50,5550 }, {  51,5550 }, {  52,5550 }, {  53,5550 },
 {  54,5550 }, {  55,5550 }, {  56,5550 }, {  57,5550 }, { 124,5559 },
 {   0,   0 }, { 126,5559 }, {  69, 374 }, {   0,   0 }, { 101, 327 },
 {   0,   0 }, {  65,5612 }, {  66,5612 }, {  67,5612 }, {  68,5612 },
 {  69,5612 }, {  70,5612 }, {  71,5612 }, {  72,5612 }, {  73,5612 },

 {  74,5612 }, {  75,5612 }, {  76,5612 }, {  77,5612 }, {  78,5612 },
 {  79,5612 }, {  80,5612 }, {  81,5612 }, {  82,5612 }, {  83,5612 },
 {  84,5612 }, {  85,5612 }, {  86,5612 }, {  87,5612 }, {  88,5612 },
 {  89,5612 }, {  90,5612 }, {  85,9952 }, {   0,   0 }, { 101, 374 },
 {   0,   0 }, {  95,5612 }, {   0,   0 }, {  97,5612 }, {  98,5612 },
 {  99,5612 }, { 100,5612 }, { 101,5612 }, { 102,5612 }, { 103,5612 },
 { 104,5612 }, { 105,5612 }, { 106,5612 }, { 107,5612 }, { 108,5612 },
 { 109,5612 }, { 110,5612 }, { 111,5612 }, { 112,5612 }, { 113,5612 },
 { 114,5612 }, { 115,5612 }, { 116,5612 }, { 117,5612 }, { 118,5612 },
 { 119,5612 }, { 120,5612 }, { 121,5612 }, { 122,5612 }, { 117,9975 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, { 128,5612 },
 { 129,5612 }, { 130,5612 }, { 131,5612 }, { 132,5612 }, { 133,5612 },
 { 134,5612 }, { 135,5612 }, { 136,5612 }, { 137,5612 }, { 138,5612 },
 { 139,5612 }, { 140,5612 }, { 141,5612 }, { 142,5612 }, { 143,5612 },
 { 144,5612 }, { 145,5612 }, { 146,5612 }, { 147,5612 }, { 148,5612 },
 { 149,5612 }, { 150,5612 }, { 151,5612 }, { 152,5612 }, { 153,5612 },
 { 154,5612 }, { 155,5612 }, { 156,5612 }, { 157,5612 }, { 158,5612 },
 { 159,5612 }, { 160,5612 }, { 161,5612 }, { 162,5612 }, { 163,5612 },
 { 164,5612 }, { 165,5612 }, { 166,5612 }, { 167,5612 }, { 168,5612 },
 { 169,5612 }, { 170,5612 }, { 171,5612 }, { 172,5612 }, { 173,5612 },

 { 174,5612 }, { 175,5612 }, { 176,5612 }, { 177,5612 }, { 178,5612 },
 { 179,5612 }, { 180,5612 }, { 181,5612 }, { 182,5612 }, { 183,5612 },
 { 184,5612 }, { 185,5612 }, { 186,5612 }, { 187,5612 }, { 188,5612 },
 { 189,5612 }, { 190,5612 }, { 191,5612 }, { 192,5612 }, { 193,5612 },
 { 194,5612 }, { 195,5612 }, { 196,5612 }, { 197,5612 }, { 198,5612 },
 { 199,5612 }, { 200,5612 }, { 201,5612 }, { 202,5612 }, { 203,5612 },
 { 204,5612 }, { 205,5612 }, { 206,5612 }, { 207,5612 }, { 208,5612 },
 { 209,5612 }, { 210,5612 }, { 211,5612 }, { 212,5612 }, { 213,5612 },
 { 214,5612 }, { 215,5612 }, { 216,5612 }, { 217,5612 }, { 218,5612 },
 { 219,5612 }, { 220,5612 }, { 221,5612 }, { 222,5612 }, { 223,5612 },

 { 224,5612 }, { 225,5612 }, { 226,5612 }, { 227,5612 }, { 228,5612 },
 { 229,5612 }, { 230,5612 }, { 231,5612 }, { 232,5612 }, { 233,5612 },
 { 234,5612 }, { 235,5612 }, { 236,5612 }, { 237,5612 }, { 238,5612 },
 { 239,5612 }, { 240,5612 }, { 241,5612 }, { 242,5612 }, { 243,5612 },
 { 244,5612 }, { 245,5612 }, { 246,5612 }, { 247,5612 }, { 248,5612 },
 { 249,5612 }, { 250,5612 }, { 251,5612 }, { 252,5612 }, { 253,5612 },
 { 254,5612 }, { 255,5612 }, {   0,  68 }, {   0,29992 }, {   0,  59 },
 {   0,29990 }, {   0,  15 }, {   0,29988 }, {   0,  10 }, {   0,29986 },
 {   0,   7 }, {   0,29984 }, {   0,   0 }, {   0,  68 }, {   0,29981 },
 {   0,   4 }, {   0,29979 }, {   0,  57 }, {   0,29977 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,  17 }, {   0,29972 }, {   0,  30 },
 {   0,29970 }, {   0,  23 }, {   0,29968 }, {   0,  38 }, {   0,29966 },
 {   0,  45 }, {   0,29964 }, {   0,  68 }, {   0,29962 }, {   0,  55 },
 {   0,29960 }, {  33,5236 }, {   0,   0 }, {  35,5236 }, {  34, 112 },
 {  37,5236 }, {  38,5236 }, {   0,   0 }, {   0,   0 }, {  39, 115 },
 {  42,5236 }, {  43,5236 }, {  33,5225 }, {  45,5236 }, {  35,5225 },
 {  47,5236 }, {  37,5225 }, {  38,5225 }, {  42,   0 }, {  45,10760 },
 {   0,   0 }, {  42,5225 }, {  43,5225 }, {  47,   5 }, {  45,5601 },
 {   0,   0 }, {  47,5225 }, {   0,   0 }, {  60,5236 }, {  61,5236 },
 {  62,5236 }, {  63,5236 }, {  64,5236 }, {  45,11132 }, {   0,  25 },

 {   0,29925 }, {   0,   0 }, {  45,11433 }, {   0,   0 }, {  60,5225 },
 {  61,5225 }, {  62,5225 }, {  63,5225 }, {  64,5225 }, {  46,-277 },
 {   0,   0 }, {  48,5840 }, {  49,5840 }, {  50,5840 }, {  51,5840 },
 {  52,5840 }, {  53,5840 }, {  54,5840 }, {  55,5840 }, {  56,5840 },
 {  57,5840 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  94,5236 }, {   0,   0 }, {  96,5236 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,  68 }, {   0,29891 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  94,5225 }, {   0,   0 },
 {  96,5225 }, {   0,  28 }, {   0,29883 }, {   0,  74 }, {   0,29881 },
 {  45,12351 }, {   0,  50 }, {   0,29878 }, {  83, 127 }, {   0,  21 },

 {   0,29875 }, {   0,  14 }, {   0,29873 }, {   0,  10 }, {   0,29871 },
 {   0,  13 }, {   0,29869 }, { 124,5236 }, {   0,   0 }, { 126,5236 },
 {   0,  17 }, {   0,29864 }, {   0,  41 }, {   0,29862 }, {   0,   0 },
 {   0,  72 }, {   0,29859 }, {  33,5135 }, { 124,5225 }, {  35,5135 },
 { 126,5225 }, {  37,5135 }, {  38,5135 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  42,5794 }, {  43,5135 }, {   0,   0 }, {  45,5135 },
 { 115, 127 }, {  47,5135 }, {   0,   0 }, {   0,  23 }, {   0,29841 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,  55 }, {   0,29833 }, {   0,   0 }, {  60,5135 },
 {  61,5135 }, {  62,5135 }, {  63,5135 }, {  64,5135 }, {  45,12940 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  45,13965 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  46,5764 }, {   0,   0 }, {  48,5828 },
 {  49,5828 }, {  50,5828 }, {  51,5828 }, {  52,5828 }, {  53,5828 },
 {  54,5828 }, {  55,5828 }, {  56,5828 }, {  57,5828 }, {   0,   0 },
 {  83, 157 }, {   0,   0 }, {   0,   0 }, {  94,5135 }, {  45,14974 },
 {  96,5135 }, {  65,5890 }, {  66,5890 }, {  67,5890 }, {  68,5890 },
 {  69,5982 }, {  70,5890 }, {  71,5890 }, {  72,5890 }, {  73,5890 },
 {  74,5890 }, {  75,5890 }, {  76,5890 }, {  77,5890 }, {  78,5890 },
 {  79,5890 }, {  80,5890 }, {  81,5890 }, {  82,5890 }, {  83,5890 },

 {  84,5890 }, {  85,5890 }, {  86,5890 }, {  87,5890 }, {  88,5890 },
 {  89,5890 }, {  90,5890 }, { 115, 157 }, { 124,5135 }, {  67, 115 },
 { 126,5135 }, {  95,5890 }, {   0,   0 }, {  97,5890 }, {  98,5890 },
 {  99,5890 }, { 100,5890 }, { 101,5982 }, { 102,5890 }, { 103,5890 },
 { 104,5890 }, { 105,5890 }, { 106,5890 }, { 107,5890 }, { 108,5890 },
 { 109,5890 }, { 110,5890 }, { 111,5890 }, { 112,5890 }, { 113,5890 },
 { 114,5890 }, { 115,5890 }, { 116,5890 }, { 117,5890 }, { 118,5890 },
 { 119,5890 }, { 120,5890 }, { 121,5890 }, { 122,5890 }, {   0,  68 },
 {   0,29735 }, {  99, 115 }, {   0,   0 }, {   0,   0 }, {   0,  25 },
 {   0,29730 }, {   0,   0 }, {   0,   0 }, {   0,  28 }, {   0,29726 },

 {   0,  68 }, {   0,29724 }, {   0,  39 }, {   0,29722 }, {   0,  40 },
 {   0,29720 }, {   0,  55 }, {   0,29718 }, {   0,   0 }, {   0,  28 },
 {   0,29715 }, {   0,  55 }, {   0,29713 }, {   0,  28 }, {   0,29711 },
 {   0,  33 }, {   0,29709 }, {   0,  55 }, {   0,29707 }, {   0,  28 },
 {   0,29705 }, {   0,  34 }, {   0,29703 }, {  33,4979 }, {   0,   0 },
 {  35,4979 }, {   0,   0 }, {  37,4979 }, {  38,4979 }, {   0,   0 },
 {   0,  55 }, {   0,29694 }, {  42,4979 }, {  43,4979 }, {  33,4968 },
 {  45,4979 }, {  35,4968 }, {  47,4979 }, {  37,4968 }, {  38,4968 },
 {  45,16022 }, {   0,   0 }, {   0,   0 }, {  42,4968 }, {  43,4968 },
 {   0,   0 }, {  45,4968 }, {   0,   0 }, {  47,4968 }, {   0,   0 },

 {  60,4979 }, {  61,5950 }, {  62,5990 }, {  63,4979 }, {  64,4979 },
 {   0,   0 }, {   0,  28 }, {   0,29668 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  60,4968 }, {  61,4968 }, {  62,6019 }, {  63,4968 },
 {  64,4968 }, {  67,  11 }, {   0,  68 }, {   0,29657 }, {   0,  55 },
 {   0,29655 }, {   0,   0 }, {  65,   5 }, {   0,   0 }, {   0,   0 },
 {  65,   4 }, {  45,23820 }, {   0,  28 }, {   0,29647 }, {   0,  56 },
 {   0,29645 }, {   0,  29 }, {   0,29643 }, {   0,   0 }, {  94,4979 },
 {   0,   0 }, {  96,4979 }, {  69,22959 }, {   0,   0 }, {  69,22981 },
 {   0,   0 }, {   0,   0 }, {  80,   6 }, {   0,   0 }, {  80,   6 },
 {  94,4968 }, {   0,   0 }, {  96,4968 }, {  99,  11 }, {   0,  78 },

 {   0,29625 }, {  33,4901 }, {  45,24052 }, {  35,4901 }, {  97,   5 },
 {  37,4901 }, {  38,4901 }, {  97,   4 }, {   0,   0 }, {  39,  10 },
 {  42,4901 }, {  43,4901 }, {   0,   0 }, {  45,4901 }, { 124,4979 },
 {  47,4901 }, { 126,4979 }, {  39,   4 }, {   0,   0 }, { 101,22959 },
 {   0,   0 }, { 101,22981 }, {   0,   0 }, {   0,   0 }, { 112,   6 },
 { 124,4968 }, { 112,   6 }, { 126,4968 }, {  60,4901 }, {  61,5992 },
 {  62,4901 }, {  63,4901 }, {  64,4901 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  36,6026 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  48,6026 }, {  49,6026 },

 {  50,6026 }, {  51,6026 }, {  52,6026 }, {  53,6026 }, {  54,6026 },
 {  55,6026 }, {  56,6026 }, {  57,6026 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  94,4901 }, {   0,   0 }, {  96,4901 },
 {  65,6026 }, {  66,6026 }, {  67,6026 }, {  68,6026 }, {  69,6026 },
 {  70,6026 }, {  71,6026 }, {  72,6026 }, {  73,6026 }, {  74,6026 },
 {  75,6026 }, {  76,6026 }, {  77,6026 }, {  78,6026 }, {  79,6026 },
 {  80,6026 }, {  81,6026 }, {  82,6026 }, {  83,6026 }, {  84,6026 },
 {  85,6026 }, {  86,6026 }, {  87,6026 }, {  88,6026 }, {  89,6026 },
 {  90,6026 }, {   0,   0 }, { 124,4901 }, {   0,   0 }, { 126,4901 },
 {  95,6026 }, {   0,   0 }, {  97,6026 }, {  98,6026 }, {  99,6026 },

 { 100,6026 }, { 101,6026 }, { 102,6026 }, { 103,6026 }, { 104,6026 },
 { 105,6026 }, { 106,6026 }, { 107,6026 }, { 108,6026 }, { 109,6026 },
 { 110,6026 }, { 111,6026 }, { 112,6026 }, { 113,6026 }, { 114,6026 },
 { 115,6026 }, { 116,6026 }, { 117,6026 }, { 118,6026 }, { 119,6026 },
 { 120,6026 }, { 121,6026 }, { 122,6026 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, { 128,6026 }, { 129,6026 },
 { 130,6026 }, { 131,6026 }, { 132,6026 }, { 133,6026 }, { 134,6026 },
 { 135,6026 }, { 136,6026 }, { 137,6026 }, { 138,6026 }, { 139,6026 },
 { 140,6026 }, { 141,6026 }, { 142,6026 }, { 143,6026 }, { 144,6026 },
 { 145,6026 }, { 146,6026 }, { 147,6026 }, { 148,6026 }, { 149,6026 },

 { 150,6026 }, { 151,6026 }, { 152,6026 }, { 153,6026 }, { 154,6026 },
 { 155,6026 }, { 156,6026 }, { 157,6026 }, { 158,6026 }, { 159,6026 },
 { 160,6026 }, { 161,6026 }, { 162,6026 }, { 163,6026 }, { 164,6026 },
 { 165,6026 }, { 166,6026 }, { 167,6026 }, { 168,6026 }, { 169,6026 },
 { 170,6026 }, { 171,6026 }, { 172,6026 }, { 173,6026 }, { 174,6026 },
 { 175,6026 }, { 176,6026 }, { 177,6026 }, { 178,6026 }, { 179,6026 },
 { 180,6026 }, { 181,6026 }, { 182,6026 }, { 183,6026 }, { 184,6026 },
 { 185,6026 }, { 186,6026 }, { 187,6026 }, { 188,6026 }, { 189,6026 },
 { 190,6026 }, { 191,6026 }, { 192,6026 }, { 193,6026 }, { 194,6026 },
 { 195,6026 }, { 196,6026 }, { 197,6026 }, { 198,6026 }, { 199,6026 },

 { 200,6026 }, { 201,6026 }, { 202,6026 }, { 203,6026 }, { 204,6026 },
 { 205,6026 }, { 206,6026 }, { 207,6026 }, { 208,6026 }, { 209,6026 },
 { 210,6026 }, { 211,6026 }, { 212,6026 }, { 213,6026 }, { 214,6026 },
 { 215,6026 }, { 216,6026 }, { 217,6026 }, { 218,6026 }, { 219,6026 },
 { 220,6026 }, { 221,6026 }, { 222,6026 }, { 223,6026 }, { 224,6026 },
 { 225,6026 }, { 226,6026 }, { 227,6026 }, { 228,6026 }, { 229,6026 },
 { 230,6026 }, { 231,6026 }, { 232,6026 }, { 233,6026 }, { 234,6026 },
 { 235,6026 }, { 236,6026 }, { 237,6026 }, { 238,6026 }, { 239,6026 },
 { 240,6026 }, { 241,6026 }, { 242,6026 }, { 243,6026 }, { 244,6026 },
 { 245,6026 }, { 246,6026 }, { 247,6026 }, { 248,6026 }, { 249,6026 },

 { 250,6026 }, { 251,6026 }, { 252,6026 }, { 253,6026 }, { 254,6026 },
 { 255,6026 }, {   0,  78 }, {   0,29368 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  36,5769 }, {   0,   0 },
 {   0,   0 }, {  39,-865 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  48,5769 }, {  49,5769 }, {  50,5769 }, {  51,5769 }, {  52,5769 },
 {  53,5769 }, {  54,5769 }, {  55,5769 }, {  56,5769 }, {  57,5769 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  65,5769 }, {  66,5769 }, {  67,5769 },
 {  68,5769 }, {  69,5769 }, {  70,5769 }, {  71,5769 }, {  72,5769 },
 {  73,5769 }, {  74,5769 }, {  75,5769 }, {  76,5769 }, {  77,5769 },
 {  78,5769 }, {  79,5769 }, {  80,5769 }, {  81,5769 }, {  82,5769 },
 {  83,5769 }, {  84,5769 }, {  85,5769 }, {  86,5769 }, {  87,5769 },
 {  88,5769 }, {  89,5769 }, {  90,5769 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {  95,5769 }, {   0,   0 }, {  97,5769 },
 {  98,5769 }, {  99,5769 }, { 100,5769 }, { 101,5769 }, { 102,5769 },
 { 103,5769 }, { 104,5769 }, { 105,5769 }, { 106,5769 }, { 107,5769 },
 { 108,5769 }, { 109,5769 }, { 110,5769 }, { 111,5769 }, { 112,5769 },
 { 113,5769 }, { 114,5769 }, { 115,5769 }, { 116,5769 }, { 117,5769 },
 { 118,5769 }, { 119,5769 }, { 120,5769 }, { 121,5769 }, { 122,5769 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 { 128,5769 }, { 129,5769 }, { 130,5769 }, { 131,5769 }, { 132,5769 },
 { 133,5769 }, { 134,5769 }, { 135,5769 }, { 136,5769 }, { 137,5769 },
 { 138,5769 }, { 139,5769 }, { 140,5769 }, { 141,5769 }, { 142,5769 },

 { 143,5769 }, { 144,5769 }, { 145,5769 }, { 146,5769 }, { 147,5769 },
 { 148,5769 }, { 149,5769 }, { 150,5769 }, { 151,5769 }, { 152,5769 },
 { 153,5769 }, { 154,5769 }, { 155,5769 }, { 156,5769 }, { 157,5769 },
 { 158,5769 }, { 159,5769 }, { 160,5769 }, { 161,5769 }, { 162,5769 },
 { 163,5769 }, { 164,5769 }, { 165,5769 }, { 166,5769 }, { 167,5769 },
 { 168,5769 }, { 169,5769 }, { 170,5769 }, { 171,5769 }, { 172,5769 },
 { 173,5769 }, { 174,5769 }, { 175,5769 }, { 176,5769 }, { 177,5769 },
 { 178,5769 }, { 179,5769 }, { 180,5769 }, { 181,5769 }, { 182,5769 },
 { 183,5769 }, { 184,5769 }, { 185,5769 }, { 186,5769 }, { 187,5769 },
 { 188,5769 }, { 189,5769 }, { 190,5769 }, { 191,5769 }, { 192,5769 },

 { 193,5769 }, { 194,5769 }, { 195,5769 }, { 196,5769 }, { 197,5769 },
 { 198,5769 }, { 199,5769 }, { 200,5769 }, { 201,5769 }, { 202,5769 },
 { 203,5769 }, { 204,5769 }, { 205,5769 }, { 206,5769 }, { 207,5769 },
 { 208,5769 }, { 209,5769 }, { 210,5769 }, { 211,5769 }, { 212,5769 },
 { 213,5769 }, { 214,5769 }, { 215,5769 }, { 216,5769 }, { 217,5769 },
 { 218,5769 }, { 219,5769 }, { 220,5769 }, { 221,5769 }, { 222,5769 },
 { 223,5769 }, { 224,5769 }, { 225,5769 }, { 226,5769 }, { 227,5769 },
 { 228,5769 }, { 229,5769 }, { 230,5769 }, { 231,5769 }, { 232,5769 },
 { 233,5769 }, { 234,5769 }, { 235,5769 }, { 236,5769 }, { 237,5769 },
 { 238,5769 }, { 239,5769 }, { 240,5769 }, { 241,5769 }, { 242,5769 },

 { 243,5769 }, { 244,5769 }, { 245,5769 }, { 246,5769 }, { 247,5769 },
 { 248,5769 }, { 249,5769 }, { 250,5769 }, { 251,5769 }, { 252,5769 },
 { 253,5769 }, { 254,5769 }, { 255,5769 }, {   0,  78 }, {   0,29111 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {  36,5512 }, {   0,   0 }, {   0,   0 }, {  39,-1116 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  48,5512 }, {  49,5512 }, {  50,5512 },
 {  51,5512 }, {  52,5512 }, {  53,5512 }, {  54,5512 }, {  55,5512 },
 {  56,5512 }, {  57,5512 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65,5512 },
 {  66,5512 }, {  67,5512 }, {  68,5512 }, {  69,5512 }, {  70,5512 },
 {  71,5512 }, {  72,5512 }, {  73,5512 }, {  74,5512 }, {  75,5512 },
 {  76,5512 }, {  77,5512 }, {  78,5512 }, {  79,5512 }, {  80,5512 },
 {  81,5512 }, {  82,5512 }, {  83,5512 }, {  84,5512 }, {  85,5512 },

 {  86,5512 }, {  87,5512 }, {  88,5512 }, {  89,5512 }, {  90,5512 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  95,5512 },
 {   0,   0 }, {  97,5512 }, {  98,5512 }, {  99,5512 }, { 100,5512 },
 { 101,5512 }, { 102,5512 }, { 103,5512 }, { 104,5512 }, { 105,5512 },
 { 106,5512 }, { 107,5512 }, { 108,5512 }, { 109,5512 }, { 110,5512 },
 { 111,5512 }, { 112,5512 }, { 113,5512 }, { 114,5512 }, { 115,5512 },
 { 116,5512 }, { 117,5512 }, { 118,5512 }, { 119,5512 }, { 120,5512 },
 { 121,5512 }, { 122,5512 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, { 128,5512 }, { 129,5512 }, { 130,5512 },
 { 131,5512 }, { 132,5512 }, { 133,5512 }, { 134,5512 }, { 135,5512 },

 { 136,5512 }, { 137,5512 }, { 138,5512 }, { 139,5512 }, { 140,5512 },
 { 141,5512 }, { 142,5512 }, { 143,5512 }, { 144,5512 }, { 145,5512 },
 { 146,5512 }, { 147,5512 }, { 148,5512 }, { 149,5512 }, { 150,5512 },
 { 151,5512 }, { 152,5512 }, { 153,5512 }, { 154,5512 }, { 155,5512 },
 { 156,5512 }, { 157,5512 }, { 158,5512 }, { 159,5512 }, { 160,5512 },
 { 161,5512 }, { 162,5512 }, { 163,5512 }, { 164,5512 }, { 165,5512 },
 { 166,5512 }, { 167,5512 }, { 168,5512 }, { 169,5512 }, { 170,5512 },
 { 171,5512 }, { 172,5512 }, { 173,5512 }, { 174,5512 }, { 175,5512 },
 { 176,5512 }, { 177,5512 }, { 178,5512 }, { 179,5512 }, { 180,5512 },
 { 181,5512 }, { 182,5512 }, { 183,5512 }, { 184,5512 }, { 185,5512 },

 { 186,5512 }, { 187,5512 }, { 188,5512 }, { 189,5512 }, { 190,5512 },
 { 191,5512 }, { 192,5512 }, { 193,5512 }, { 194,5512 }, { 195,5512 },
 { 196,5512 }, { 197,5512 }, { 198,5512 }, { 199,5512 }, { 200,5512 },
 { 201,5512 }, { 202,5512 }, { 203,5512 }, { 204,5512 }, { 205,5512 },
 { 206,5512 }, { 207,5512 }, { 208,5512 }, { 209,5512 }, { 210,5512 },
 { 211,5512 }, { 212,5512 }, { 213,5512 }, { 214,5512 }, { 215,5512 },
 { 216,5512 }, { 217,5512 }, { 218,5512 }, { 219,5512 }, { 220,5512 },
 { 221,5512 }, { 222,5512 }, { 223,5512 }, { 224,5512 }, { 225,5512 },
 { 226,5512 }, { 227,5512 }, { 228,5512 }, { 229,5512 }, { 230,5512 },
 { 231,5512 }, { 232,5512 }, { 233,5512 }, { 234,5512 }, { 235,5512 },

 { 236,5512 }, { 237,5512 }, { 238,5512 }, { 239,5512 }, { 240,5512 },
 { 241,5512 }, { 242,5512 }, { 243,5512 }, { 244,5512 }, { 245,5512 },
 { 246,5512 }, { 247,5512 }, { 248,5512 }, { 249,5512 }, { 250,5512 },
 { 251,5512 }, { 252,5512 }, { 253,5512 }, { 254,5512 }, { 255,5512 },
 {   0,  78 }, {   0,28854 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  36,5255 }, {   0,   0 }, {   0,   0 },
 {  39,-1371 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  48,5255 },
 {  49,5255 }, {  50,5255 }, {  51,5255 }, {  52,5255 }, {  53,5255 },
 {  54,5255 }, {  55,5255 }, {  56,5255 }, {  57,5255 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  65,5255 }, {  66,5255 }, {  67,5255 }, {  68,5255 },
 {  69,5255 }, {  70,5255 }, {  71,5255 }, {  72,5255 }, {  73,5255 },
 {  74,5255 }, {  75,5255 }, {  76,5255 }, {  77,5255 }, {  78,5255 },

 {  79,5255 }, {  80,5255 }, {  81,5255 }, {  82,5255 }, {  83,5255 },
 {  84,5255 }, {  85,5255 }, {  86,5255 }, {  87,5255 }, {  88,5255 },
 {  89,5255 }, {  90,5255 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  95,5255 }, {   0,   0 }, {  97,5255 }, {  98,5255 },
 {  99,5255 }, { 100,5255 }, { 101,5255 }, { 102,5255 }, { 103,5255 },
 { 104,5255 }, { 105,5255 }, { 106,5255 }, { 107,5255 }, { 108,5255 },
 { 109,5255 }, { 110,5255 }, { 111,5255 }, { 112,5255 }, { 113,5255 },
 { 114,5255 }, { 115,5255 }, { 116,5255 }, { 117,5255 }, { 118,5255 },
 { 119,5255 }, { 120,5255 }, { 121,5255 }, { 122,5255 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, { 128,5255 },

 { 129,5255 }, { 130,5255 }, { 131,5255 }, { 132,5255 }, { 133,5255 },
 { 134,5255 }, { 135,5255 }, { 136,5255 }, { 137,5255 }, { 138,5255 },
 { 139,5255 }, { 140,5255 }, { 141,5255 }, { 142,5255 }, { 143,5255 },
 { 144,5255 }, { 145,5255 }, { 146,5255 }, { 147,5255 }, { 148,5255 },
 { 149,5255 }, { 150,5255 }, { 151,5255 }, { 152,5255 }, { 153,5255 },
 { 154,5255 }, { 155,5255 }, { 156,5255 }, { 157,5255 }, { 158,5255 },
 { 159,5255 }, { 160,5255 }, { 161,5255 }, { 162,5255 }, { 163,5255 },
 { 164,5255 }, { 165,5255 }, { 166,5255 }, { 167,5255 }, { 168,5255 },
 { 169,5255 }, { 170,5255 }, { 171,5255 }, { 172,5255 }, { 173,5255 },
 { 174,5255 }, { 175,5255 }, { 176,5255 }, { 177,5255 }, { 178,5255 },

 { 179,5255 }, { 180,5255 }, { 181,5255 }, { 182,5255 }, { 183,5255 },
 { 184,5255 }, { 185,5255 }, { 186,5255 }, { 187,5255 }, { 188,5255 },
 { 189,5255 }, { 190,5255 }, { 191,5255 }, { 192,5255 }, { 193,5255 },
 { 194,5255 }, { 195,5255 }, { 196,5255 }, { 197,5255 }, { 198,5255 },
 { 199,5255 }, { 200,5255 }, { 201,5255 }, { 202,5255 }, { 203,5255 },
 { 204,5255 }, { 205,5255 }, { 206,5255 }, { 207,5255 }, { 208,5255 },
 { 209,5255 }, { 210,5255 }, { 211,5255 }, { 212,5255 }, { 213,5255 },
 { 214,5255 }, { 215,5255 }, { 216,5255 }, { 217,5255 }, { 218,5255 },
 { 219,5255 }, { 220,5255 }, { 221,5255 }, { 222,5255 }, { 223,5255 },
 { 224,5255 }, { 225,5255 }, { 226,5255 }, { 227,5255 }, { 228,5255 },

 { 229,5255 }, { 230,5255 }, { 231,5255 }, { 232,5255 }, { 233,5255 },
 { 234,5255 }, { 235,5255 }, { 236,5255 }, { 237,5255 }, { 238,5255 },
 { 239,5255 }, { 240,5255 }, { 241,5255 }, { 242,5255 }, { 243,5255 },
 { 244,5255 }, { 245,5255 }, { 246,5255 }, { 247,5255 }, { 248,5255 },
 { 249,5255 }, { 250,5255 }, { 251,5255 }, { 252,5255 }, { 253,5255 },
 { 254,5255 }, { 255,5255 }, {   0,  78 }, {   0,28597 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  36,4998 },
 {   0,   0 }, {  38,-1393 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  48,4998 }, {  49,4998 }, {  50,4998 }, {  51,4998 },
 {  52,4998 }, {  53,4998 }, {  54,4998 }, {  55,4998 }, {  56,4998 },
 {  57,4998 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65,4998 }, {  66,4998 },
 {  67,4998 }, {  68,4998 }, {  69,4998 }, {  70,4998 }, {  71,4998 },

 {  72,4998 }, {  73,4998 }, {  74,4998 }, {  75,4998 }, {  76,4998 },
 {  77,4998 }, {  78,4998 }, {  79,4998 }, {  80,4998 }, {  81,4998 },
 {  82,4998 }, {  83,4998 }, {  84,4998 }, {  85,4998 }, {  86,4998 },
 {  87,4998 }, {  88,4998 }, {  89,4998 }, {  90,4998 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  95,4998 }, {   0,   0 },
 {  97,4998 }, {  98,4998 }, {  99,4998 }, { 100,4998 }, { 101,4998 },
 { 102,4998 }, { 103,4998 }, { 104,4998 }, { 105,4998 }, { 106,4998 },
 { 107,4998 }, { 108,4998 }, { 109,4998 }, { 110,4998 }, { 111,4998 },
 { 112,4998 }, { 113,4998 }, { 114,4998 }, { 115,4998 }, { 116,4998 },
 { 117,4998 }, { 118,4998 }, { 119,4998 }, { 120,4998 }, { 121,4998 },

 { 122,4998 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, { 128,4998 }, { 129,4998 }, { 130,4998 }, { 131,4998 },
 { 132,4998 }, { 133,4998 }, { 134,4998 }, { 135,4998 }, { 136,4998 },
 { 137,4998 }, { 138,4998 }, { 139,4998 }, { 140,4998 }, { 141,4998 },
 { 142,4998 }, { 143,4998 }, { 144,4998 }, { 145,4998 }, { 146,4998 },
 { 147,4998 }, { 148,4998 }, { 149,4998 }, { 150,4998 }, { 151,4998 },
 { 152,4998 }, { 153,4998 }, { 154,4998 }, { 155,4998 }, { 156,4998 },
 { 157,4998 }, { 158,4998 }, { 159,4998 }, { 160,4998 }, { 161,4998 },
 { 162,4998 }, { 163,4998 }, { 164,4998 }, { 165,4998 }, { 166,4998 },
 { 167,4998 }, { 168,4998 }, { 169,4998 }, { 170,4998 }, { 171,4998 },

 { 172,4998 }, { 173,4998 }, { 174,4998 }, { 175,4998 }, { 176,4998 },
 { 177,4998 }, { 178,4998 }, { 179,4998 }, { 180,4998 }, { 181,4998 },
 { 182,4998 }, { 183,4998 }, { 184,4998 }, { 185,4998 }, { 186,4998 },
 { 187,4998 }, { 188,4998 }, { 189,4998 }, { 190,4998 }, { 191,4998 },
 { 192,4998 }, { 193,4998 }, { 194,4998 }, { 195,4998 }, { 196,4998 },
 { 197,4998 }, { 198,4998 }, { 199,4998 }, { 200,4998 }, { 201,4998 },
 { 202,4998 }, { 203,4998 }, { 204,4998 }, { 205,4998 }, { 206,4998 },
 { 207,4998 }, { 208,4998 }, { 209,4998 }, { 210,4998 }, { 211,4998 },
 { 212,4998 }, { 213,4998 }, { 214,4998 }, { 215,4998 }, { 216,4998 },
 { 217,4998 }, { 218,4998 }, { 219,4998 }, { 220,4998 }, { 221,4998 },

 { 222,4998 }, { 223,4998 }, { 224,4998 }, { 225,4998 }, { 226,4998 },
 { 227,4998 }, { 228,4998 }, { 229,4998 }, { 230,4998 }, { 231,4998 },
 { 232,4998 }, { 233,4998 }, { 234,4998 }, { 235,4998 }, { 236,4998 },
 { 237,4998 }, { 238,4998 }, { 239,4998 }, { 240,4998 }, { 241,4998 },
 { 242,4998 }, { 243,4998 }, { 244,4998 }, { 245,4998 }, { 246,4998 },
 { 247,4998 }, { 248,4998 }, { 249,4998 }, { 250,4998 }, { 251,4998 },
 { 252,4998 }, { 253,4998 }, { 254,4998 }, { 255,4998 }, {   0,  78 },
 {   0,28340 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  36,4741 }, {   0,   0 }, {   0,   0 }, {  39,-1648 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  48,4741 }, {  49,4741 },
 {  50,4741 }, {  51,4741 }, {  52,4741 }, {  53,4741 }, {  54,4741 },
 {  55,4741 }, {  56,4741 }, {  57,4741 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {  65,4741 }, {  66,4741 }, {  67,4741 }, {  68,4741 }, {  69,4741 },
 {  70,4741 }, {  71,4741 }, {  72,4741 }, {  73,4741 }, {  74,4741 },
 {  75,4741 }, {  76,4741 }, {  77,4741 }, {  78,4741 }, {  79,4741 },
 {  80,4741 }, {  81,4741 }, {  82,4741 }, {  83,4741 }, {  84,4741 },
 {  85,4741 }, {  86,4741 }, {  87,4741 }, {  88,4741 }, {  89,4741 },
 {  90,4741 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  95,4741 }, {   0,   0 }, {  97,4741 }, {  98,4741 }, {  99,4741 },
 { 100,4741 }, { 101,4741 }, { 102,4741 }, { 103,4741 }, { 104,4741 },
 { 105,4741 }, { 106,4741 }, { 107,4741 }, { 108,4741 }, { 109,4741 },
 { 110,4741 }, { 111,4741 }, { 112,4741 }, { 113,4741 }, { 114,4741 },

 { 115,4741 }, { 116,4741 }, { 117,4741 }, { 118,4741 }, { 119,4741 },
 { 120,4741 }, { 121,4741 }, { 122,4741 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, { 128,4741 }, { 129,4741 },
 { 130,4741 }, { 131,4741 }, { 132,4741 }, { 133,4741 }, { 134,4741 },
 { 135,4741 }, { 136,4741 }, { 137,4741 }, { 138,4741 }, { 139,4741 },
 { 140,4741 }, { 141,4741 }, { 142,4741 }, { 143,4741 }, { 144,4741 },
 { 145,4741 }, { 146,4741 }, { 147,4741 }, { 148,4741 }, { 149,4741 },
 { 150,4741 }, { 151,4741 }, { 152,4741 }, { 153,4741 }, { 154,4741 },
 { 155,4741 }, { 156,4741 }, { 157,4741 }, { 158,4741 }, { 159,4741 },
 { 160,4741 }, { 161,4741 }, { 162,4741 }, { 163,4741 }, { 164,4741 },

 { 165,4741 }, { 166,4741 }, { 167,4741 }, { 168,4741 }, { 169,4741 },
 { 170,4741 }, { 171,4741 }, { 172,4741 }, { 173,4741 }, { 174,4741 },
 { 175,4741 }, { 176,4741 }, { 177,4741 }, { 178,4741 }, { 179,4741 },
 { 180,4741 }, { 181,4741 }, { 182,4741 }, { 183,4741 }, { 184,4741 },
 { 185,4741 }, { 186,4741 }, { 187,4741 }, { 188,4741 }, { 189,4741 },
 { 190,4741 }, { 191,4741 }, { 192,4741 }, { 193,4741 }, { 194,4741 },
 { 195,4741 }, { 196,4741 }, { 197,4741 }, { 198,4741 }, { 199,4741 },
 { 200,4741 }, { 201,4741 }, { 202,4741 }, { 203,4741 }, { 204,4741 },
 { 205,4741 }, { 206,4741 }, { 207,4741 }, { 208,4741 }, { 209,4741 },
 { 210,4741 }, { 211,4741 }, { 212,4741 }, { 213,4741 }, { 214,4741 },

 { 215,4741 }, { 216,4741 }, { 217,4741 }, { 218,4741 }, { 219,4741 },
 { 220,4741 }, { 221,4741 }, { 222,4741 }, { 223,4741 }, { 224,4741 },
 { 225,4741 }, { 226,4741 }, { 227,4741 }, { 228,4741 }, { 229,4741 },
 { 230,4741 }, { 231,4741 }, { 232,4741 }, { 233,4741 }, { 234,4741 },
 { 235,4741 }, { 236,4741 }, { 237,4741 }, { 238,4741 }, { 239,4741 },
 { 240,4741 }, { 241,4741 }, { 242,4741 }, { 243,4741 }, { 244,4741 },
 { 245,4741 }, { 246,4741 }, { 247,4741 }, { 248,4741 }, { 249,4741 },
 { 250,4741 }, { 251,4741 }, { 252,4741 }, { 253,4741 }, { 254,4741 },
 { 255,4741 }, {   0,  12 }, {   0,28083 }, {   1,4741 }, {   2,4741 },
 {   3,4741 }, {   4,4741 }, {   5,4741 }, {   6,4741 }, {   7,4741 },

 {   8,4741 }, {   9,4741 }, {  10,4741 }, {  11,4741 }, {  12,4741 },
 {  13,4741 }, {  14,4741 }, {  15,4741 }, {  16,4741 }, {  17,4741 },
 {  18,4741 }, {  19,4741 }, {  20,4741 }, {  21,4741 }, {  22,4741 },
 {  23,4741 }, {  24,4741 }, {  25,4741 }, {  26,4741 }, {  27,4741 },
 {  28,4741 }, {  29,4741 }, {  30,4741 }, {  31,4741 }, {  32,4741 },
 {  33,4741 }, {  34,4741 }, {  35,4741 }, {  36,4741 }, {  37,4741 },
 {  38,4741 }, {   0,   0 }, {  40,4741 }, {  41,4741 }, {  42,4741 },
 {  43,4741 }, {  44,4741 }, {  45,4741 }, {  46,4741 }, {  47,4741 },
 {  48,4741 }, {  49,4741 }, {  50,4741 }, {  51,4741 }, {  52,4741 },
 {  53,4741 }, {  54,4741 }, {  55,4741 }, {  56,4741 }, {  57,4741 },

 {  58,4741 }, {  59,4741 }, {  60,4741 }, {  61,4741 }, {  62,4741 },
 {  63,4741 }, {  64,4741 }, {  65,4741 }, {  66,4741 }, {  67,4741 },
 {  68,4741 }, {  69,4741 }, {  70,4741 }, {  71,4741 }, {  72,4741 },
 {  73,4741 }, {  74,4741 }, {  75,4741 }, {  76,4741 }, {  77,4741 },
 {  78,4741 }, {  79,4741 }, {  80,4741 }, {  81,4741 }, {  82,4741 },
 {  83,4741 }, {  84,4741 }, {  85,4741 }, {  86,4741 }, {  87,4741 },
 {  88,4741 }, {  89,4741 }, {  90,4741 }, {  91,4741 }, {  92,4741 },
 {  93,4741 }, {  94,4741 }, {  95,4741 }, {  96,4741 }, {  97,4741 },
 {  98,4741 }, {  99,4741 }, { 100,4741 }, { 101,4741 }, { 102,4741 },
 { 103,4741 }, { 104,4741 }, { 105,4741 }, { 106,4741 }, { 107,4741 },

 { 108,4741 }, { 109,4741 }, { 110,4741 }, { 111,4741 }, { 112,4741 },
 { 113,4741 }, { 114,4741 }, { 115,4741 }, { 116,4741 }, { 117,4741 },
 { 118,4741 }, { 119,4741 }, { 120,4741 }, { 121,4741 }, { 122,4741 },
 { 123,4741 }, { 124,4741 }, { 125,4741 }, { 126,4741 }, { 127,4741 },
 { 128,4741 }, { 129,4741 }, { 130,4741 }, { 131,4741 }, { 132,4741 },
 { 133,4741 }, { 134,4741 }, { 135,4741 }, { 136,4741 }, { 137,4741 },
 { 138,4741 }, { 139,4741 }, { 140,4741 }, { 141,4741 }, { 142,4741 },
 { 143,4741 }, { 144,4741 }, { 145,4741 }, { 146,4741 }, { 147,4741 },
 { 148,4741 }, { 149,4741 }, { 150,4741 }, { 151,4741 }, { 152,4741 },
 { 153,4741 }, { 154,4741 }, { 155,4741 }, { 156,4741 }, { 157,4741 },

 { 158,4741 }, { 159,4741 }, { 160,4741 }, { 161,4741 }, { 162,4741 },
 { 163,4741 }, { 164,4741 }, { 165,4741 }, { 166,4741 }, { 167,4741 },
 { 168,4741 }, { 169,4741 }, { 170,4741 }, { 171,4741 }, { 172,4741 },
 { 173,4741 }, { 174,4741 }, { 175,4741 }, { 176,4741 }, { 177,4741 },
 { 178,4741 }, { 179,4741 }, { 180,4741 }, { 181,4741 }, { 182,4741 },
 { 183,4741 }, { 184,4741 }, { 185,4741 }, { 186,4741 }, { 187,4741 },
 { 188,4741 }, { 189,4741 }, { 190,4741 }, { 191,4741 }, { 192,4741 },
 { 193,4741 }, { 194,4741 }, { 195,4741 }, { 196,4741 }, { 197,4741 },
 { 198,4741 }, { 199,4741 }, { 200,4741 }, { 201,4741 }, { 202,4741 },
 { 203,4741 }, { 204,4741 }, { 205,4741 }, { 206,4741 }, { 207,4741 },

 { 208,4741 }, { 209,4741 }, { 210,4741 }, { 211,4741 }, { 212,4741 },
 { 213,4741 }, { 214,4741 }, { 215,4741 }, { 216,4741 }, { 217,4741 },
 { 218,4741 }, { 219,4741 }, { 220,4741 }, { 221,4741 }, { 222,4741 },
 { 223,4741 }, { 224,4741 }, { 225,4741 }, { 226,4741 }, { 227,4741 },
 { 228,4741 }, { 229,4741 }, { 230,4741 }, { 231,4741 }, { 232,4741 },
 { 233,4741 }, { 234,4741 }, { 235,4741 }, { 236,4741 }, { 237,4741 },
 { 238,4741 }, { 239,4741 }, { 240,4741 }, { 241,4741 }, { 242,4741 },
 { 243,4741 }, { 244,4741 }, { 245,4741 }, { 246,4741 }, { 247,4741 },
 { 248,4741 }, { 249,4741 }, { 250,4741 }, { 251,4741 }, { 252,4741 },
 { 253,4741 }, { 254,4741 }, { 255,4741 }, { 256,4741 }, {   0,   9 },

 {   0,27825 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   9,4741 },
 {  10,4746 }, {   0,   0 }, {  12,4741 }, {  13,4746 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  32,4741 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  45,-2161 }, {   0,   5 }, {   0,27778 }, {   1,4746 }, {   2,4746 },

 {   3,4746 }, {   4,4746 }, {   5,4746 }, {   6,4746 }, {   7,4746 },
 {   8,4746 }, {   9,4746 }, {  10,4746 }, {  11,4746 }, {  12,4746 },
 {  13,4746 }, {  14,4746 }, {  15,4746 }, {  16,4746 }, {  17,4746 },
 {  18,4746 }, {  19,4746 }, {  20,4746 }, {  21,4746 }, {  22,4746 },
 {  23,4746 }, {  24,4746 }, {  25,4746 }, {  26,4746 }, {  27,4746 },
 {  28,4746 }, {  29,4746 }, {  30,4746 }, {  31,4746 }, {  32,4746 },
 {  33,4746 }, {  34,4746 }, {  35,4746 }, {  36,4746 }, {  37,4746 },
 {  38,4746 }, {  39,4746 }, {  40,4746 }, {  41,4746 }, {   0,   0 },
 {  43,4746 }, {  44,4746 }, {  45,4746 }, {  46,4746 }, {   0,   0 },
 {  48,4746 }, {  49,4746 }, {  50,4746 }, {  51,4746 }, {  52,4746 },

 {  53,4746 }, {  54,4746 }, {  55,4746 }, {  56,4746 }, {  57,4746 },
 {  58,4746 }, {  59,4746 }, {  60,4746 }, {  61,4746 }, {  62,4746 },
 {  63,4746 }, {  64,4746 }, {  65,4746 }, {  66,4746 }, {  67,4746 },
 {  68,4746 }, {  69,4746 }, {  70,4746 }, {  71,4746 }, {  72,4746 },
 {  73,4746 }, {  74,4746 }, {  75,4746 }, {  76,4746 }, {  77,4746 },
 {  78,4746 }, {  79,4746 }, {  80,4746 }, {  81,4746 }, {  82,4746 },
 {  83,4746 }, {  84,4746 }, {  85,4746 }, {  86,4746 }, {  87,4746 },
 {  88,4746 }, {  89,4746 }, {  90,4746 }, {  91,4746 }, {  92,4746 },
 {  93,4746 }, {  94,4746 }, {  95,4746 }, {  96,4746 }, {  97,4746 },
 {  98,4746 }, {  99,4746 }, { 100,4746 }, { 101,4746 }, { 102,4746 },

 { 103,4746 }, { 104,4746 }, { 105,4746 }, { 106,4746 }, { 107,4746 },
 { 108,4746 }, { 109,4746 }, { 110,4746 }, { 111,4746 }, { 112,4746 },
 { 113,4746 }, { 114,4746 }, { 115,4746 }, { 116,4746 }, { 117,4746 },
 { 118,4746 }, { 119,4746 }, { 120,4746 }, { 121,4746 }, { 122,4746 },
 { 123,4746 }, { 124,4746 }, { 125,4746 }, { 126,4746 }, { 127,4746 },
 { 128,4746 }, { 129,4746 }, { 130,4746 }, { 131,4746 }, { 132,4746 },
 { 133,4746 }, { 134,4746 }, { 135,4746 }, { 136,4746 }, { 137,4746 },
 { 138,4746 }, { 139,4746 }, { 140,4746 }, { 141,4746 }, { 142,4746 },
 { 143,4746 }, { 144,4746 }, { 145,4746 }, { 146,4746 }, { 147,4746 },
 { 148,4746 }, { 149,4746 }, { 150,4746 }, { 151,4746 }, { 152,4746 },

 { 153,4746 }, { 154,4746 }, { 155,4746 }, { 156,4746 }, { 157,4746 },
 { 158,4746 }, { 159,4746 }, { 160,4746 }, { 161,4746 }, { 162,4746 },
 { 163,4746 }, { 164,4746 }, { 165,4746 }, { 166,4746 }, { 167,4746 },
 { 168,4746 }, { 169,4746 }, { 170,4746 }, { 171,4746 }, { 172,4746 },
 { 173,4746 }, { 174,4746 }, { 175,4746 }, { 176,4746 }, { 177,4746 },
 { 178,4746 }, { 179,4746 }, { 180,4746 }, { 181,4746 }, { 182,4746 },
 { 183,4746 }, { 184,4746 }, { 185,4746 }, { 186,4746 }, { 187,4746 },
 { 188,4746 }, { 189,4746 }, { 190,4746 }, { 191,4746 }, { 192,4746 },
 { 193,4746 }, { 194,4746 }, { 195,4746 }, { 196,4746 }, { 197,4746 },
 { 198,4746 }, { 199,4746 }, { 200,4746 }, { 201,4746 }, { 202,4746 },

 { 203,4746 }, { 204,4746 }, { 205,4746 }, { 206,4746 }, { 207,4746 },
 { 208,4746 }, { 209,4746 }, { 210,4746 }, { 211,4746 }, { 212,4746 },
 { 213,4746 }, { 214,4746 }, { 215,4746 }, { 216,4746 }, { 217,4746 },
 { 218,4746 }, { 219,4746 }, { 220,4746 }, { 221,4746 }, { 222,4746 },
 { 223,4746 }, { 224,4746 }, { 225,4746 }, { 226,4746 }, { 227,4746 },
 { 228,4746 }, { 229,4746 }, { 230,4746 }, { 231,4746 }, { 232,4746 },
 { 233,4746 }, { 234,4746 }, { 235,4746 }, { 236,4746 }, { 237,4746 },
 { 238,4746 }, { 239,4746 }, { 240,4746 }, { 241,4746 }, { 242,4746 },
 { 243,4746 }, { 244,4746 }, { 245,4746 }, { 246,4746 }, { 247,4746 },
 { 248,4746 }, { 249,4746 }, { 250,4746 }, { 251,4746 }, { 252,4746 },

 { 253,4746 }, { 254,4746 }, { 255,4746 }, { 256,4746 }, {   0,   5 },
 {   0,27520 }, {   1,4488 }, {   2,4488 }, {   3,4488 }, {   4,4488 },
 {   5,4488 }, {   6,4488 }, {   7,4488 }, {   8,4488 }, {   9,4488 },
 {  10,4488 }, {  11,4488 }, {  12,4488 }, {  13,4488 }, {  14,4488 },
 {  15,4488 }, {  16,4488 }, {  17,4488 }, {  18,4488 }, {  19,4488 },
 {  20,4488 }, {  21,4488 }, {  22,4488 }, {  23,4488 }, {  24,4488 },
 {  25,4488 }, {  26,4488 }, {  27,4488 }, {  28,4488 }, {  29,4488 },
 {  30,4488 }, {  31,4488 }, {  32,4488 }, {  33,4488 }, {  34,4488 },
 {  35,4488 }, {  36,4488 }, {  37,4488 }, {  38,4488 }, {  39,4488 },
 {  40,4488 }, {  41,4488 }, {   0,   0 }, {  43,4488 }, {  44,4488 },

 {  45,4488 }, {  46,4488 }, {   0,   0 }, {  48,4488 }, {  49,4488 },
 {  50,4488 }, {  51,4488 }, {  52,4488 }, {  53,4488 }, {  54,4488 },
 {  55,4488 }, {  56,4488 }, {  57,4488 }, {  58,4488 }, {  59,4488 },
 {  60,4488 }, {  61,4488 }, {  62,4488 }, {  63,4488 }, {  64,4488 },
 {  65,4488 }, {  66,4488 }, {  67,4488 }, {  68,4488 }, {  69,4488 },
 {  70,4488 }, {  71,4488 }, {  72,4488 }, {  73,4488 }, {  74,4488 },
 {  75,4488 }, {  76,4488 }, {  77,4488 }, {  78,4488 }, {  79,4488 },
 {  80,4488 }, {  81,4488 }, {  82,4488 }, {  83,4488 }, {  84,4488 },
 {  85,4488 }, {  86,4488 }, {  87,4488 }, {  88,4488 }, {  89,4488 },
 {  90,4488 }, {  91,4488 }, {  92,4488 }, {  93,4488 }, {  94,4488 },

 {  95,4488 }, {  96,4488 }, {  97,4488 }, {  98,4488 }, {  99,4488 },
 { 100,4488 }, { 101,4488 }, { 102,4488 }, { 103,4488 }, { 104,4488 },
 { 105,4488 }, { 106,4488 }, { 107,4488 }, { 108,4488 }, { 109,4488 },
 { 110,4488 }, { 111,4488 }, { 112,4488 }, { 113,4488 }, { 114,4488 },
 { 115,4488 }, { 116,4488 }, { 117,4488 }, { 118,4488 }, { 119,4488 },
 { 120,4488 }, { 121,4488 }, { 122,4488 }, { 123,4488 }, { 124,4488 },
 { 125,4488 }, { 126,4488 }, { 127,4488 }, { 128,4488 }, { 129,4488 },
 { 130,4488 }, { 131,4488 }, { 132,4488 }, { 133,4488 }, { 134,4488 },
 { 135,4488 }, { 136,4488 }, { 137,4488 }, { 138,4488 }, { 139,4488 },
 { 140,4488 }, { 141,4488 }, { 142,4488 }, { 143,4488 }, { 144,4488 },

 { 145,4488 }, { 146,4488 }, { 147,4488 }, { 148,4488 }, { 149,4488 },
 { 150,4488 }, { 151,4488 }, { 152,4488 }, { 153,4488 }, { 154,4488 },
 { 155,4488 }, { 156,4488 }, { 157,4488 }, { 158,4488 }, { 159,4488 },
 { 160,4488 }, { 161,4488 }, { 162,4488 }, { 163,4488 }, { 164,4488 },
 { 165,4488 }, { 166,4488 }, { 167,4488 }, { 168,4488 }, { 169,4488 },
 { 170,4488 }, { 171,4488 }, { 172,4488 }, { 173,4488 }, { 174,4488 },
 { 175,4488 }, { 176,4488 }, { 177,4488 }, { 178,4488 }, { 179,4488 },
 { 180,4488 }, { 181,4488 }, { 182,4488 }, { 183,4488 }, { 184,4488 },
 { 185,4488 }, { 186,4488 }, { 187,4488 }, { 188,4488 }, { 189,4488 },
 { 190,4488 }, { 191,4488 }, { 192,4488 }, { 193,4488 }, { 194,4488 },

 { 195,4488 }, { 196,4488 }, { 197,4488 }, { 198,4488 }, { 199,4488 },
 { 200,4488 }, { 201,4488 }, { 202,4488 }, { 203,4488 }, { 204,4488 },
 { 205,4488 }, { 206,4488 }, { 207,4488 }, { 208,4488 }, { 209,4488 },
 { 210,4488 }, { 211,4488 }, { 212,4488 }, { 213,4488 }, { 214,4488 },
 { 215,4488 }, { 216,4488 }, { 217,4488 }, { 218,4488 }, { 219,4488 },
 { 220,4488 }, { 221,4488 }, { 222,4488 }, { 223,4488 }, { 224,4488 },
 { 225,4488 }, { 226,4488 }, { 227,4488 }, { 228,4488 }, { 229,4488 },
 { 230,4488 }, { 231,4488 }, { 232,4488 }, { 233,4488 }, { 234,4488 },
 { 235,4488 }, { 236,4488 }, { 237,4488 }, { 238,4488 }, { 239,4488 },
 { 240,4488 }, { 241,4488 }, { 242,4488 }, { 243,4488 }, { 244,4488 },

 { 245,4488 }, { 246,4488 }, { 247,4488 }, { 248,4488 }, { 249,4488 },
 { 250,4488 }, { 251,4488 }, { 252,4488 }, { 253,4488 }, { 254,4488 },
 { 255,4488 }, { 256,4488 }, {   0,  58 }, {   0,27262 }, {   1,4616 },
 {   2,4616 }, {   3,4616 }, {   4,4616 }, {   5,4616 }, {   6,4616 },
 {   7,4616 }, {   8,4616 }, {   9,4616 }, {  10,4616 }, {  11,4616 },
 {  12,4616 }, {  13,4616 }, {  14,4616 }, {  15,4616 }, {  16,4616 },
 {  17,4616 }, {  18,4616 }, {  19,4616 }, {  20,4616 }, {  21,4616 },
 {  22,4616 }, {  23,4616 }, {  24,4616 }, {  25,4616 }, {  26,4616 },
 {  27,4616 }, {  28,4616 }, {  29,4616 }, {  30,4616 }, {  31,4616 },
 {  32,4616 }, {  33,4616 }, {   0,   0 }, {  35,4616 }, {  36,4616 },

 {  37,4616 }, {  38,4616 }, {  39,4616 }, {  40,4616 }, {  41,4616 },
 {  42,4616 }, {  43,4616 }, {  44,4616 }, {  45,4616 }, {  46,4616 },
 {  47,4616 }, {  48,4616 }, {  49,4616 }, {  50,4616 }, {  51,4616 },
 {  52,4616 }, {  53,4616 }, {  54,4616 }, {  55,4616 }, {  56,4616 },
 {  57,4616 }, {  58,4616 }, {  59,4616 }, {  60,4616 }, {  61,4616 },
 {  62,4616 }, {  63,4616 }, {  64,4616 }, {  65,4616 }, {  66,4616 },
 {  67,4616 }, {  68,4616 }, {  69,4616 }, {  70,4616 }, {  71,4616 },
 {  72,4616 }, {  73,4616 }, {  74,4616 }, {  75,4616 }, {  76,4616 },
 {  77,4616 }, {  78,4616 }, {  79,4616 }, {  80,4616 }, {  81,4616 },
 {  82,4616 }, {  83,4616 }, {  84,4616 }, {  85,4616 }, {  86,4616 },

 {  87,4616 }, {  88,4616 }, {  89,4616 }, {  90,4616 }, {  91,4616 },
 {  92,4616 }, {  93,4616 }, {  94,4616 }, {  95,4616 }, {  96,4616 },
 {  97,4616 }, {  98,4616 }, {  99,4616 }, { 100,4616 }, { 101,4616 },
 { 102,4616 }, { 103,4616 }, { 104,4616 }, { 105,4616 }, { 106,4616 },
 { 107,4616 }, { 108,4616 }, { 109,4616 }, { 110,4616 }, { 111,4616 },
 { 112,4616 }, { 113,4616 }, { 114,4616 }, { 115,4616 }, { 116,4616 },
 { 117,4616 }, { 118,4616 }, { 119,4616 }, { 120,4616 }, { 121,4616 },
 { 122,4616 }, { 123,4616 }, { 124,4616 }, { 125,4616 }, { 126,4616 },
 { 127,4616 }, { 128,4616 }, { 129,4616 }, { 130,4616 }, { 131,4616 },
 { 132,4616 }, { 133,4616 }, { 134,4616 }, { 135,4616 }, { 136,4616 },

 { 137,4616 }, { 138,4616 }, { 139,4616 }, { 140,4616 }, { 141,4616 },
 { 142,4616 }, { 143,4616 }, { 144,4616 }, { 145,4616 }, { 146,4616 },
 { 147,4616 }, { 148,4616 }, { 149,4616 }, { 150,4616 }, { 151,4616 },
 { 152,4616 }, { 153,4616 }, { 154,4616 }, { 155,4616 }, { 156,4616 },
 { 157,4616 }, { 158,4616 }, { 159,4616 }, { 160,4616 }, { 161,4616 },
 { 162,4616 }, { 163,4616 }, { 164,4616 }, { 165,4616 }, { 166,4616 },
 { 167,4616 }, { 168,4616 }, { 169,4616 }, { 170,4616 }, { 171,4616 },
 { 172,4616 }, { 173,4616 }, { 174,4616 }, { 175,4616 }, { 176,4616 },
 { 177,4616 }, { 178,4616 }, { 179,4616 }, { 180,4616 }, { 181,4616 },
 { 182,4616 }, { 183,4616 }, { 184,4616 }, { 185,4616 }, { 186,4616 },

 { 187,4616 }, { 188,4616 }, { 189,4616 }, { 190,4616 }, { 191,4616 },
 { 192,4616 }, { 193,4616 }, { 194,4616 }, { 195,4616 }, { 196,4616 },
 { 197,4616 }, { 198,4616 }, { 199,4616 }, { 200,4616 }, { 201,4616 },
 { 202,4616 }, { 203,4616 }, { 204,4616 }, { 205,4616 }, { 206,4616 },
 { 207,4616 }, { 208,4616 }, { 209,4616 }, { 210,4616 }, { 211,4616 },
 { 212,4616 }, { 213,4616 }, { 214,4616 }, { 215,4616 }, { 216,4616 },
 { 217,4616 }, { 218,4616 }, { 219,4616 }, { 220,4616 }, { 221,4616 },
 { 222,4616 }, { 223,4616 }, { 224,4616 }, { 225,4616 }, { 226,4616 },
 { 227,4616 }, { 228,4616 }, { 229,4616 }, { 230,4616 }, { 231,4616 },
 { 232,4616 }, { 233,4616 }, { 234,4616 }, { 235,4616 }, { 236,4616 },

 { 237,4616 }, { 238,4616 }, { 239,4616 }, { 240,4616 }, { 241,4616 },
 { 242,4616 }, { 243,4616 }, { 244,4616 }, { 245,4616 }, { 246,4616 },
 { 247,4616 }, { 248,4616 }, { 249,4616 }, { 250,4616 }, { 251,4616 },
 { 252,4616 }, { 253,4616 }, { 254,4616 }, { 255,4616 }, { 256,4616 },
 {   0,  11 }, {   0,27004 }, {   1,4616 }, {   2,4616 }, {   3,4616 },
 {   4,4616 }, {   5,4616 }, {   6,4616 }, {   7,4616 }, {   8,4616 },
 {   9,4616 }, {  10,4616 }, {  11,4616 }, {  12,4616 }, {  13,4616 },
 {  14,4616 }, {  15,4616 }, {  16,4616 }, {  17,4616 }, {  18,4616 },
 {  19,4616 }, {  20,4616 }, {  21,4616 }, {  22,4616 }, {  23,4616 },
 {  24,4616 }, {  25,4616 }, {  26,4616 }, {  27,4616 }, {  28,4616 },

 {  29,4616 }, {  30,4616 }, {  31,4616 }, {  32,4616 }, {  33,4616 },
 {  34,4616 }, {  35,4616 }, {  36,4616 }, {  37,4616 }, {  38,4616 },
 {   0,   0 }, {  40,4616 }, {  41,4616 }, {  42,4616 }, {  43,4616 },
 {  44,4616 }, {  45,4616 }, {  46,4616 }, {  47,4616 }, {  48,4616 },
 {  49,4616 }, {  50,4616 }, {  51,4616 }, {  52,4616 }, {  53,4616 },
 {  54,4616 }, {  55,4616 }, {  56,4616 }, {  57,4616 }, {  58,4616 },
 {  59,4616 }, {  60,4616 }, {  61,4616 }, {  62,4616 }, {  63,4616 },
 {  64,4616 }, {  65,4616 }, {  66,4616 }, {  67,4616 }, {  68,4616 },
 {  69,4616 }, {  70,4616 }, {  71,4616 }, {  72,4616 }, {  73,4616 },
 {  74,4616 }, {  75,4616 }, {  76,4616 }, {  77,4616 }, {  78,4616 },

 {  79,4616 }, {  80,4616 }, {  81,4616 }, {  82,4616 }, {  83,4616 },
 {  84,4616 }, {  85,4616 }, {  86,4616 }, {  87,4616 }, {  88,4616 },
 {  89,4616 }, {  90,4616 }, {  91,4616 }, {  92,4616 }, {  93,4616 },
 {  94,4616 }, {  95,4616 }, {  96,4616 }, {  97,4616 }, {  98,4616 },
 {  99,4616 }, { 100,4616 }, { 101,4616 }, { 102,4616 }, { 103,4616 },
 { 104,4616 }, { 105,4616 }, { 106,4616 }, { 107,4616 }, { 108,4616 },
 { 109,4616 }, { 110,4616 }, { 111,4616 }, { 112,4616 }, { 113,4616 },
 { 114,4616 }, { 115,4616 }, { 116,4616 }, { 117,4616 }, { 118,4616 },
 { 119,4616 }, { 120,4616 }, { 121,4616 }, { 122,4616 }, { 123,4616 },
 { 124,4616 }, { 125,4616 }, { 126,4616 }, { 127,4616 }, { 128,4616 },

 { 129,4616 }, { 130,4616 }, { 131,4616 }, { 132,4616 }, { 133,4616 },
 { 134,4616 }, { 135,4616 }, { 136,4616 }, { 137,4616 }, { 138,4616 },
 { 139,4616 }, { 140,4616 }, { 141,4616 }, { 142,4616 }, { 143,4616 },
 { 144,4616 }, { 145,4616 }, { 146,4616 }, { 147,4616 }, { 148,4616 },
 { 149,4616 }, { 150,4616 }, { 151,4616 }, { 152,4616 }, { 153,4616 },
 { 154,4616 }, { 155,4616 }, { 156,4616 }, { 157,4616 }, { 158,4616 },
 { 159,4616 }, { 160,4616 }, { 161,4616 }, { 162,4616 }, { 163,4616 },
 { 164,4616 }, { 165,4616 }, { 166,4616 }, { 167,4616 }, { 168,4616 },
 { 169,4616 }, { 170,4616 }, { 171,4616 }, { 172,4616 }, { 173,4616 },
 { 174,4616 }, { 175,4616 }, { 176,4616 }, { 177,4616 }, { 178,4616 },

 { 179,4616 }, { 180,4616 }, { 181,4616 }, { 182,4616 }, { 183,4616 },
 { 184,4616 }, { 185,4616 }, { 186,4616 }, { 187,4616 }, { 188,4616 },
 { 189,4616 }, { 190,4616 }, { 191,4616 }, { 192,4616 }, { 193,4616 },
 { 194,4616 }, { 195,4616 }, { 196,4616 }, { 197,4616 }, { 198,4616 },
 { 199,4616 }, { 200,4616 }, { 201,4616 }, { 202,4616 }, { 203,4616 },
 { 204,4616 }, { 205,4616 }, { 206,4616 }, { 207,4616 }, { 208,4616 },
 { 209,4616 }, { 210,4616 }, { 211,4616 }, { 212,4616 }, { 213,4616 },
 { 214,4616 }, { 215,4616 }, { 216,4616 }, { 217,4616 }, { 218,4616 },
 { 219,4616 }, { 220,4616 }, { 221,4616 }, { 222,4616 }, { 223,4616 },
 { 224,4616 }, { 225,4616 }, { 226,4616 }, { 227,4616 }, { 228,4616 },

 { 229,4616 }, { 230,4616 }, { 231,4616 }, { 232,4616 }, { 233,4616 },
 { 234,4616 }, { 235,4616 }, { 236,4616 }, { 237,4616 }, { 238,4616 },
 { 239,4616 }, { 240,4616 }, { 241,4616 }, { 242,4616 }, { 243,4616 },
 { 244,4616 }, { 245,4616 }, { 246,4616 }, { 247,4616 }, { 248,4616 },
 { 249,4616 }, { 250,4616 }, { 251,4616 }, { 252,4616 }, { 253,4616 },
 { 254,4616 }, { 255,4616 }, { 256,4616 }, {   0,  16 }, {   0,26746 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   9,4616 }, {  10,4621 },
 {   0,   0 }, {  12,4616 }, {  13,4621 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  32,4616 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  45,-3226 },
 {   0,  32 }, {   0,26699 }, {   1,4621 }, {   2,4621 }, {   3,4621 },
 {   4,4621 }, {   5,4621 }, {   6,4621 }, {   7,4621 }, {   8,4621 },
 {   9,4621 }, {  10,4621 }, {  11,4621 }, {  12,4621 }, {  13,4621 },
 {  14,4621 }, {  15,4621 }, {  16,4621 }, {  17,4621 }, {  18,4621 },
 {  19,4621 }, {  20,4621 }, {  21,4621 }, {  22,4621 }, {  23,4621 },

 {  24,4621 }, {  25,4621 }, {  26,4621 }, {  27,4621 }, {  28,4621 },
 {  29,4621 }, {  30,4621 }, {  31,4621 }, {  32,4621 }, {  33,4621 },
 {  34,4621 }, {  35,4621 }, {  36,4621 }, {  37,4621 }, {  38,4621 },
 {   0,   0 }, {  40,4621 }, {  41,4621 }, {  42,4621 }, {  43,4621 },
 {  44,4621 }, {  45,4621 }, {  46,4621 }, {  47,4621 }, {  48,4621 },
 {  49,4621 }, {  50,4621 }, {  51,4621 }, {  52,4621 }, {  53,4621 },
 {  54,4621 }, {  55,4621 }, {  56,4621 }, {  57,4621 }, {  58,4621 },
 {  59,4621 }, {  60,4621 }, {  61,4621 }, {  62,4621 }, {  63,4621 },
 {  64,4621 }, {  65,4621 }, {  66,4621 }, {  67,4621 }, {  68,4621 },
 {  69,4621 }, {  70,4621 }, {  71,4621 }, {  72,4621 }, {  73,4621 },

 {  74,4621 }, {  75,4621 }, {  76,4621 }, {  77,4621 }, {  78,4621 },
 {  79,4621 }, {  80,4621 }, {  81,4621 }, {  82,4621 }, {  83,4621 },
 {  84,4621 }, {  85,4621 }, {  86,4621 }, {  87,4621 }, {  88,4621 },
 {  89,4621 }, {  90,4621 }, {  91,4621 }, {   0,   0 }, {  93,4621 },
 {  94,4621 }, {  95,4621 }, {  96,4621 }, {  97,4621 }, {  98,4621 },
 {  99,4621 }, { 100,4621 }, { 101,4621 }, { 102,4621 }, { 103,4621 },
 { 104,4621 }, { 105,4621 }, { 106,4621 }, { 107,4621 }, { 108,4621 },
 { 109,4621 }, { 110,4621 }, { 111,4621 }, { 112,4621 }, { 113,4621 },
 { 114,4621 }, { 115,4621 }, { 116,4621 }, { 117,4621 }, { 118,4621 },
 { 119,4621 }, { 120,4621 }, { 121,4621 }, { 122,4621 }, { 123,4621 },

 { 124,4621 }, { 125,4621 }, { 126,4621 }, { 127,4621 }, { 128,4621 },
 { 129,4621 }, { 130,4621 }, { 131,4621 }, { 132,4621 }, { 133,4621 },
 { 134,4621 }, { 135,4621 }, { 136,4621 }, { 137,4621 }, { 138,4621 },
 { 139,4621 }, { 140,4621 }, { 141,4621 }, { 142,4621 }, { 143,4621 },
 { 144,4621 }, { 145,4621 }, { 146,4621 }, { 147,4621 }, { 148,4621 },
 { 149,4621 }, { 150,4621 }, { 151,4621 }, { 152,4621 }, { 153,4621 },
 { 154,4621 }, { 155,4621 }, { 156,4621 }, { 157,4621 }, { 158,4621 },
 { 159,4621 }, { 160,4621 }, { 161,4621 }, { 162,4621 }, { 163,4621 },
 { 164,4621 }, { 165,4621 }, { 166,4621 }, { 167,4621 }, { 168,4621 },
 { 169,4621 }, { 170,4621 }, { 171,4621 }, { 172,4621 }, { 173,4621 },

 { 174,4621 }, { 175,4621 }, { 176,4621 }, { 177,4621 }, { 178,4621 },
 { 179,4621 }, { 180,4621 }, { 181,4621 }, { 182,4621 }, { 183,4621 },
 { 184,4621 }, { 185,4621 }, { 186,4621 }, { 187,4621 }, { 188,4621 },
 { 189,4621 }, { 190,4621 }, { 191,4621 }, { 192,4621 }, { 193,4621 },
 { 194,4621 }, { 195,4621 }, { 196,4621 }, { 197,4621 }, { 198,4621 },
 { 199,4621 }, { 200,4621 }, { 201,4621 }, { 202,4621 }, { 203,4621 },
 { 204,4621 }, { 205,4621 }, { 206,4621 }, { 207,4621 }, { 208,4621 },
 { 209,4621 }, { 210,4621 }, { 211,4621 }, { 212,4621 }, { 213,4621 },
 { 214,4621 }, { 215,4621 }, { 216,4621 }, { 217,4621 }, { 218,4621 },
 { 219,4621 }, { 220,4621 }, { 221,4621 }, { 222,4621 }, { 223,4621 },

 { 224,4621 }, { 225,4621 }, { 226,4621 }, { 227,4621 }, { 228,4621 },
 { 229,4621 }, { 230,4621 }, { 231,4621 }, { 232,4621 }, { 233,4621 },
 { 234,4621 }, { 235,4621 }, { 236,4621 }, { 237,4621 }, { 238,4621 },
 { 239,4621 }, { 240,4621 }, { 241,4621 }, { 242,4621 }, { 243,4621 },
 { 244,4621 }, { 245,4621 }, { 246,4621 }, { 247,4621 }, { 248,4621 },
 { 249,4621 }, { 250,4621 }, { 251,4621 }, { 252,4621 }, { 253,4621 },
 { 254,4621 }, { 255,4621 }, { 256,4621 }, {   0,  32 }, {   0,26441 },
 {   1,4363 }, {   2,4363 }, {   3,4363 }, {   4,4363 }, {   5,4363 },
 {   6,4363 }, {   7,4363 }, {   8,4363 }, {   9,4363 }, {  10,4363 },
 {  11,4363 }, {  12,4363 }, {  13,4363 }, {  14,4363 }, {  15,4363 },

 {  16,4363 }, {  17,4363 }, {  18,4363 }, {  19,4363 }, {  20,4363 },
 {  21,4363 }, {  22,4363 }, {  23,4363 }, {  24,4363 }, {  25,4363 },
 {  26,4363 }, {  27,4363 }, {  28,4363 }, {  29,4363 }, {  30,4363 },
 {  31,4363 }, {  32,4363 }, {  33,4363 }, {  34,4363 }, {  35,4363 },
 {  36,4363 }, {  37,4363 }, {  38,4363 }, {   0,   0 }, {  40,4363 },
 {  41,4363 }, {  42,4363 }, {  43,4363 }, {  44,4363 }, {  45,4363 },
 {  46,4363 }, {  47,4363 }, {  48,4363 }, {  49,4363 }, {  50,4363 },
 {  51,4363 }, {  52,4363 }, {  53,4363 }, {  54,4363 }, {  55,4363 },
 {  56,4363 }, {  57,4363 }, {  58,4363 }, {  59,4363 }, {  60,4363 },
 {  61,4363 }, {  62,4363 }, {  63,4363 }, {  64,4363 }, {  65,4363 },

 {  66,4363 }, {  67,4363 }, {  68,4363 }, {  69,4363 }, {  70,4363 },
 {  71,4363 }, {  72,4363 }, {  73,4363 }, {  74,4363 }, {  75,4363 },
 {  76,4363 }, {  77,4363 }, {  78,4363 }, {  79,4363 }, {  80,4363 },
 {  81,4363 }, {  82,4363 }, {  83,4363 }, {  84,4363 }, {  85,4363 },
 {  86,4363 }, {  87,4363 }, {  88,4363 }, {  89,4363 }, {  90,4363 },
 {  91,4363 }, {   0,   0 }, {  93,4363 }, {  94,4363 }, {  95,4363 },
 {  96,4363 }, {  97,4363 }, {  98,4363 }, {  99,4363 }, { 100,4363 },
 { 101,4363 }, { 102,4363 }, { 103,4363 }, { 104,4363 }, { 105,4363 },
 { 106,4363 }, { 107,4363 }, { 108,4363 }, { 109,4363 }, { 110,4363 },
 { 111,4363 }, { 112,4363 }, { 113,4363 }, { 114,4363 }, { 115,4363 },

 { 116,4363 }, { 117,4363 }, { 118,4363 }, { 119,4363 }, { 120,4363 },
 { 121,4363 }, { 122,4363 }, { 123,4363 }, { 124,4363 }, { 125,4363 },
 { 126,4363 }, { 127,4363 }, { 128,4363 }, { 129,4363 }, { 130,4363 },
 { 131,4363 }, { 132,4363 }, { 133,4363 }, { 134,4363 }, { 135,4363 },
 { 136,4363 }, { 137,4363 }, { 138,4363 }, { 139,4363 }, { 140,4363 },
 { 141,4363 }, { 142,4363 }, { 143,4363 }, { 144,4363 }, { 145,4363 },
 { 146,4363 }, { 147,4363 }, { 148,4363 }, { 149,4363 }, { 150,4363 },
 { 151,4363 }, { 152,4363 }, { 153,4363 }, { 154,4363 }, { 155,4363 },
 { 156,4363 }, { 157,4363 }, { 158,4363 }, { 159,4363 }, { 160,4363 },
 { 161,4363 }, { 162,4363 }, { 163,4363 }, { 164,4363 }, { 165,4363 },

 { 166,4363 }, { 167,4363 }, { 168,4363 }, { 169,4363 }, { 170,4363 },
 { 171,4363 }, { 172,4363 }, { 173,4363 }, { 174,4363 }, { 175,4363 },
 { 176,4363 }, { 177,4363 }, { 178,4363 }, { 179,4363 }, { 180,4363 },
 { 181,4363 }, { 182,4363 }, { 183,4363 }, { 184,4363 }, { 185,4363 },
 { 186,4363 }, { 187,4363 }, { 188,4363 }, { 189,4363 }, { 190,4363 },
 { 191,4363 }, { 192,4363 }, { 193,4363 }, { 194,4363 }, { 195,4363 },
 { 196,4363 }, { 197,4363 }, { 198,4363 }, { 199,4363 }, { 200,4363 },
 { 201,4363 }, { 202,4363 }, { 203,4363 }, { 204,4363 }, { 205,4363 },
 { 206,4363 }, { 207,4363 }, { 208,4363 }, { 209,4363 }, { 210,4363 },
 { 211,4363 }, { 212,4363 }, { 213,4363 }, { 214,4363 }, { 215,4363 },

 { 216,4363 }, { 217,4363 }, { 218,4363 }, { 219,4363 }, { 220,4363 },
 { 221,4363 }, { 222,4363 }, { 223,4363 }, { 224,4363 }, { 225,4363 },
 { 226,4363 }, { 227,4363 }, { 228,4363 }, { 229,4363 }, { 230,4363 },
 { 231,4363 }, { 232,4363 }, { 233,4363 }, { 234,4363 }, { 235,4363 },
 { 236,4363 }, { 237,4363 }, { 238,4363 }, { 239,4363 }, { 240,4363 },
 { 241,4363 }, { 242,4363 }, { 243,4363 }, { 244,4363 }, { 245,4363 },
 { 246,4363 }, { 247,4363 }, { 248,4363 }, { 249,4363 }, { 250,4363 },
 { 251,4363 }, { 252,4363 }, { 253,4363 }, { 254,4363 }, { 255,4363 },
 { 256,4363 }, {   0,  22 }, {   0,26183 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   9,4363 }, {  10,4368 }, {   0,   0 }, {  12,4363 },
 {  13,4368 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  32,4363 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  39,-3787 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  45,-3785 }, {   0,  42 }, {   0,26136 },
 {   1,-3830 }, {   2,-3830 }, {   3,-3830 }, {   4,-3830 }, {   5,-3830 },
 {   6,-3830 }, {   7,-3830 }, {   8,-3830 }, {   9,-3830 }, {  10,-3830 },

 {  11,-3830 }, {  12,-3830 }, {  13,-3830 }, {  14,-3830 }, {  15,-3830 },
 {  16,-3830 }, {  17,-3830 }, {  18,-3830 }, {  19,-3830 }, {  20,-3830 },
 {  21,-3830 }, {  22,-3830 }, {  23,-3830 }, {  24,-3830 }, {  25,-3830 },
 {  26,-3830 }, {  27,-3830 }, {  28,-3830 }, {  29,-3830 }, {  30,-3830 },
 {  31,-3830 }, {  32,-3830 }, {  33,-3830 }, {  34,-3830 }, {  35,-3830 },
 {  36,-3830 }, {  37,-3830 }, {  38,-3830 }, {  39,-3830 }, {  40,-3830 },
 {  41,-3830 }, {  42,-3830 }, {  43,-3830 }, {  44,-3830 }, {  45,-3830 },
 {  46,-3830 }, {  47,-3830 }, {  48,4323 }, {  49,4323 }, {  50,4323 },
 {  51,4323 }, {  52,4323 }, {  53,4323 }, {  54,4323 }, {  55,4323 },
 {  56,-3830 }, {  57,-3830 }, {  58,-3830 }, {  59,-3830 }, {  60,-3830 },

 {  61,-3830 }, {  62,-3830 }, {  63,-3830 }, {  64,-3830 }, {  65,-3830 },
 {  66,-3830 }, {  67,-3830 }, {  68,-3830 }, {  69,-3830 }, {  70,-3830 },
 {  71,-3830 }, {  72,-3830 }, {  73,-3830 }, {  74,-3830 }, {  75,-3830 },
 {  76,-3830 }, {  77,-3830 }, {  78,-3830 }, {  79,-3830 }, {  80,-3830 },
 {  81,-3830 }, {  82,-3830 }, {  83,-3830 }, {  84,-3830 }, {  85,4336 },
 {  86,-3830 }, {  87,-3830 }, {  88,-3830 }, {  89,-3830 }, {  90,-3830 },
 {  91,-3830 }, {  92,-3830 }, {  93,-3830 }, {  94,-3830 }, {  95,-3830 },
 {  96,-3830 }, {  97,-3830 }, {  98,-3830 }, {  99,-3830 }, { 100,-3830 },
 { 101,-3830 }, { 102,-3830 }, { 103,-3830 }, { 104,-3830 }, { 105,-3830 },
 { 106,-3830 }, { 107,-3830 }, { 108,-3830 }, { 109,-3830 }, { 110,-3830 },

 { 111,-3830 }, { 112,-3830 }, { 113,-3830 }, { 114,-3830 }, { 115,-3830 },
 { 116,-3830 }, { 117,4359 }, { 118,-3830 }, { 119,-3830 }, { 120,4397 },
 { 121,-3830 }, { 122,-3830 }, { 123,-3830 }, { 124,-3830 }, { 125,-3830 },
 { 126,-3830 }, { 127,-3830 }, { 128,-3830 }, { 129,-3830 }, { 130,-3830 },
 { 131,-3830 }, { 132,-3830 }, { 133,-3830 }, { 134,-3830 }, { 135,-3830 },
 { 136,-3830 }, { 137,-3830 }, { 138,-3830 }, { 139,-3830 }, { 140,-3830 },
 { 141,-3830 }, { 142,-3830 }, { 143,-3830 }, { 144,-3830 }, { 145,-3830 },
 { 146,-3830 }, { 147,-3830 }, { 148,-3830 }, { 149,-3830 }, { 150,-3830 },
 { 151,-3830 }, { 152,-3830 }, { 153,-3830 }, { 154,-3830 }, { 155,-3830 },
 { 156,-3830 }, { 157,-3830 }, { 158,-3830 }, { 159,-3830 }, { 160,-3830 },

 { 161,-3830 }, { 162,-3830 }, { 163,-3830 }, { 164,-3830 }, { 165,-3830 },
 { 166,-3830 }, { 167,-3830 }, { 168,-3830 }, { 169,-3830 }, { 170,-3830 },
 { 171,-3830 }, { 172,-3830 }, { 173,-3830 }, { 174,-3830 }, { 175,-3830 },
 { 176,-3830 }, { 177,-3830 }, { 178,-3830 }, { 179,-3830 }, { 180,-3830 },
 { 181,-3830 }, { 182,-3830 }, { 183,-3830 }, { 184,-3830 }, { 185,-3830 },
 { 186,-3830 }, { 187,-3830 }, { 188,-3830 }, { 189,-3830 }, { 190,-3830 },
 { 191,-3830 }, { 192,-3830 }, { 193,-3830 }, { 194,-3830 }, { 195,-3830 },
 { 196,-3830 }, { 197,-3830 }, { 198,-3830 }, { 199,-3830 }, { 200,-3830 },
 { 201,-3830 }, { 202,-3830 }, { 203,-3830 }, { 204,-3830 }, { 205,-3830 },
 { 206,-3830 }, { 207,-3830 }, { 208,-3830 }, { 209,-3830 }, { 210,-3830 },

 { 211,-3830 }, { 212,-3830 }, { 213,-3830 }, { 214,-3830 }, { 215,-3830 },
 { 216,-3830 }, { 217,-3830 }, { 218,-3830 }, { 219,-3830 }, { 220,-3830 },
 { 221,-3830 }, { 222,-3830 }, { 223,-3830 }, { 224,-3830 }, { 225,-3830 },
 { 226,-3830 }, { 227,-3830 }, { 228,-3830 }, { 229,-3830 }, { 230,-3830 },
 { 231,-3830 }, { 232,-3830 }, { 233,-3830 }, { 234,-3830 }, { 235,-3830 },
 { 236,-3830 }, { 237,-3830 }, { 238,-3830 }, { 239,-3830 }, { 240,-3830 },
 { 241,-3830 }, { 242,-3830 }, { 243,-3830 }, { 244,-3830 }, { 245,-3830 },
 { 246,-3830 }, { 247,-3830 }, { 248,-3830 }, { 249,-3830 }, { 250,-3830 },
 { 251,-3830 }, { 252,-3830 }, { 253,-3830 }, { 254,-3830 }, { 255,-3830 },
 { 256,-3830 }, {   0,  31 }, {   0,25878 }, {   1,4243 }, {   2,4243 },

 {   3,4243 }, {   4,4243 }, {   5,4243 }, {   6,4243 }, {   7,4243 },
 {   8,4243 }, {   9,4243 }, {  10,4243 }, {  11,4243 }, {  12,4243 },
 {  13,4243 }, {  14,4243 }, {  15,4243 }, {  16,4243 }, {  17,4243 },
 {  18,4243 }, {  19,4243 }, {  20,4243 }, {  21,4243 }, {  22,4243 },
 {  23,4243 }, {  24,4243 }, {  25,4243 }, {  26,4243 }, {  27,4243 },
 {  28,4243 }, {  29,4243 }, {  30,4243 }, {  31,4243 }, {  32,4243 },
 {  33,4243 }, {  34,4243 }, {  35,4243 }, {  36,4243 }, {  37,4243 },
 {  38,4243 }, {   0,   0 }, {  40,4243 }, {  41,4243 }, {  42,4243 },
 {  43,4243 }, {  44,4243 }, {  45,4243 }, {  46,4243 }, {  47,4243 },
 {  48,4243 }, {  49,4243 }, {  50,4243 }, {  51,4243 }, {  52,4243 },

 {  53,4243 }, {  54,4243 }, {  55,4243 }, {  56,4243 }, {  57,4243 },
 {  58,4243 }, {  59,4243 }, {  60,4243 }, {  61,4243 }, {  62,4243 },
 {  63,4243 }, {  64,4243 }, {  65,4243 }, {  66,4243 }, {  67,4243 },
 {  68,4243 }, {  69,4243 }, {  70,4243 }, {  71,4243 }, {  72,4243 },
 {  73,4243 }, {  74,4243 }, {  75,4243 }, {  76,4243 }, {  77,4243 },
 {  78,4243 }, {  79,4243 }, {  80,4243 }, {  81,4243 }, {  82,4243 },
 {  83,4243 }, {  84,4243 }, {  85,4243 }, {  86,4243 }, {  87,4243 },
 {  88,4243 }, {  89,4243 }, {  90,4243 }, {  91,4243 }, {  92,4243 },
 {  93,4243 }, {  94,4243 }, {  95,4243 }, {  96,4243 }, {  97,4243 },
 {  98,4243 }, {  99,4243 }, { 100,4243 }, { 101,4243 }, { 102,4243 },

 { 103,4243 }, { 104,4243 }, { 105,4243 }, { 106,4243 }, { 107,4243 },
 { 108,4243 }, { 109,4243 }, { 110,4243 }, { 111,4243 }, { 112,4243 },
 { 113,4243 }, { 114,4243 }, { 115,4243 }, { 116,4243 }, { 117,4243 },
 { 118,4243 }, { 119,4243 }, { 120,4243 }, { 121,4243 }, { 122,4243 },
 { 123,4243 }, { 124,4243 }, { 125,4243 }, { 126,4243 }, { 127,4243 },
 { 128,4243 }, { 129,4243 }, { 130,4243 }, { 131,4243 }, { 132,4243 },
 { 133,4243 }, { 134,4243 }, { 135,4243 }, { 136,4243 }, { 137,4243 },
 { 138,4243 }, { 139,4243 }, { 140,4243 }, { 141,4243 }, { 142,4243 },
 { 143,4243 }, { 144,4243 }, { 145,4243 }, { 146,4243 }, { 147,4243 },
 { 148,4243 }, { 149,4243 }, { 150,4243 }, { 151,4243 }, { 152,4243 },

 { 153,4243 }, { 154,4243 }, { 155,4243 }, { 156,4243 }, { 157,4243 },
 { 158,4243 }, { 159,4243 }, { 160,4243 }, { 161,4243 }, { 162,4243 },
 { 163,4243 }, { 164,4243 }, { 165,4243 }, { 166,4243 }, { 167,4243 },
 { 168,4243 }, { 169,4243 }, { 170,4243 }, { 171,4243 }, { 172,4243 },
 { 173,4243 }, { 174,4243 }, { 175,4243 }, { 176,4243 }, { 177,4243 },
 { 178,4243 }, { 179,4243 }, { 180,4243 }, { 181,4243 }, { 182,4243 },
 { 183,4243 }, { 184,4243 }, { 185,4243 }, { 186,4243 }, { 187,4243 },
 { 188,4243 }, { 189,4243 }, { 190,4243 }, { 191,4243 }, { 192,4243 },
 { 193,4243 }, { 194,4243 }, { 195,4243 }, { 196,4243 }, { 197,4243 },
 { 198,4243 }, { 199,4243 }, { 200,4243 }, { 201,4243 }, { 202,4243 },

 { 203,4243 }, { 204,4243 }, { 205,4243 }, { 206,4243 }, { 207,4243 },
 { 208,4243 }, { 209,4243 }, { 210,4243 }, { 211,4243 }, { 212,4243 },
 { 213,4243 }, { 214,4243 }, { 215,4243 }, { 216,4243 }, { 217,4243 },
 { 218,4243 }, { 219,4243 }, { 220,4243 }, { 221,4243 }, { 222,4243 },
 { 223,4243 }, { 224,4243 }, { 225,4243 }, { 226,4243 }, { 227,4243 },
 { 228,4243 }, { 229,4243 }, { 230,4243 }, { 231,4243 }, { 232,4243 },
 { 233,4243 }, { 234,4243 }, { 235,4243 }, { 236,4243 }, { 237,4243 },
 { 238,4243 }, { 239,4243 }, { 240,4243 }, { 241,4243 }, { 242,4243 },
 { 243,4243 }, { 244,4243 }, { 245,4243 }, { 246,4243 }, { 247,4243 },
 { 248,4243 }, { 249,4243 }, { 250,4243 }, { 251,4243 }, { 252,4243 },

 { 253,4243 }, { 254,4243 }, { 255,4243 }, { 256,4243 }, {   0,  22 },
 {   0,25620 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   9,3800 },
 {  10,3805 }, {   0,   0 }, {  12,3800 }, {  13,3805 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  32,3800 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  39,-4350 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {  45,-4348 }, {   0,  46 }, {   0,25573 }, {   1,4196 }, {   2,4196 },
 {   3,4196 }, {   4,4196 }, {   5,4196 }, {   6,4196 }, {   7,4196 },
 {   8,4196 }, {   9,4196 }, {  10,4196 }, {  11,4196 }, {  12,4196 },
 {  13,4196 }, {  14,4196 }, {  15,4196 }, {  16,4196 }, {  17,4196 },
 {  18,4196 }, {  19,4196 }, {  20,4196 }, {  21,4196 }, {  22,4196 },
 {  23,4196 }, {  24,4196 }, {  25,4196 }, {  26,4196 }, {  27,4196 },
 {  28,4196 }, {  29,4196 }, {  30,4196 }, {  31,4196 }, {  32,4196 },
 {  33,4196 }, {  34,4196 }, {  35,4196 }, {   0,   0 }, {  37,4196 },
 {  38,4196 }, {  39,4196 }, {  40,4196 }, {  41,4196 }, {  42,4196 },
 {  43,4196 }, {  44,4196 }, {  45,4196 }, {  46,4196 }, {  47,4196 },

 {  48,4196 }, {  49,4196 }, {  50,4196 }, {  51,4196 }, {  52,4196 },
 {  53,4196 }, {  54,4196 }, {  55,4196 }, {  56,4196 }, {  57,4196 },
 {  58,4196 }, {  59,4196 }, {  60,4196 }, {  61,4196 }, {  62,4196 },
 {  63,4196 }, {  64,4196 }, {  65,4196 }, {  66,4196 }, {  67,4196 },
 {  68,4196 }, {  69,4196 }, {  70,4196 }, {  71,4196 }, {  72,4196 },
 {  73,4196 }, {  74,4196 }, {  75,4196 }, {  76,4196 }, {  77,4196 },
 {  78,4196 }, {  79,4196 }, {  80,4196 }, {  81,4196 }, {  82,4196 },
 {  83,4196 }, {  84,4196 }, {  85,4196 }, {  86,4196 }, {  87,4196 },
 {  88,4196 }, {  89,4196 }, {  90,4196 }, {  91,4196 }, {  92,4196 },
 {  93,4196 }, {  94,4196 }, {  95,4196 }, {  96,4196 }, {  97,4196 },

 {  98,4196 }, {  99,4196 }, { 100,4196 }, { 101,4196 }, { 102,4196 },
 { 103,4196 }, { 104,4196 }, { 105,4196 }, { 106,4196 }, { 107,4196 },
 { 108,4196 }, { 109,4196 }, { 110,4196 }, { 111,4196 }, { 112,4196 },
 { 113,4196 }, { 114,4196 }, { 115,4196 }, { 116,4196 }, { 117,4196 },
 { 118,4196 }, { 119,4196 }, { 120,4196 }, { 121,4196 }, { 122,4196 },
 { 123,4196 }, { 124,4196 }, { 125,4196 }, { 126,4196 }, { 127,4196 },
 { 128,4196 }, { 129,4196 }, { 130,4196 }, { 131,4196 }, { 132,4196 },
 { 133,4196 }, { 134,4196 }, { 135,4196 }, { 136,4196 }, { 137,4196 },
 { 138,4196 }, { 139,4196 }, { 140,4196 }, { 141,4196 }, { 142,4196 },
 { 143,4196 }, { 144,4196 }, { 145,4196 }, { 146,4196 }, { 147,4196 },

 { 148,4196 }, { 149,4196 }, { 150,4196 }, { 151,4196 }, { 152,4196 },
 { 153,4196 }, { 154,4196 }, { 155,4196 }, { 156,4196 }, { 157,4196 },
 { 158,4196 }, { 159,4196 }, { 160,4196 }, { 161,4196 }, { 162,4196 },
 { 163,4196 }, { 164,4196 }, { 165,4196 }, { 166,4196 }, { 167,4196 },
 { 168,4196 }, { 169,4196 }, { 170,4196 }, { 171,4196 }, { 172,4196 },
 { 173,4196 }, { 174,4196 }, { 175,4196 }, { 176,4196 }, { 177,4196 },
 { 178,4196 }, { 179,4196 }, { 180,4196 }, { 181,4196 }, { 182,4196 },
 { 183,4196 }, { 184,4196 }, { 185,4196 }, { 186,4196 }, { 187,4196 },
 { 188,4196 }, { 189,4196 }, { 190,4196 }, { 191,4196 }, { 192,4196 },
 { 193,4196 }, { 194,4196 }, { 195,4196 }, { 196,4196 }, { 197,4196 },

 { 198,4196 }, { 199,4196 }, { 200,4196 }, { 201,4196 }, { 202,4196 },
 { 203,4196 }, { 204,4196 }, { 205,4196 }, { 206,4196 }, { 207,4196 },
 { 208,4196 }, { 209,4196 }, { 210,4196 }, { 211,4196 }, { 212,4196 },
 { 213,4196 }, { 214,4196 }, { 215,4196 }, { 216,4196 }, { 217,4196 },
 { 218,4196 }, { 219,4196 }, { 220,4196 }, { 221,4196 }, { 222,4196 },
 { 223,4196 }, { 224,4196 }, { 225,4196 }, { 226,4196 }, { 227,4196 },
 { 228,4196 }, { 229,4196 }, { 230,4196 }, { 231,4196 }, { 232,4196 },
 { 233,4196 }, { 234,4196 }, { 235,4196 }, { 236,4196 }, { 237,4196 },
 { 238,4196 }, { 239,4196 }, { 240,4196 }, { 241,4196 }, { 242,4196 },
 { 243,4196 }, { 244,4196 }, { 245,4196 }, { 246,4196 }, { 247,4196 },

 { 248,4196 }, { 249,4196 }, { 250,4196 }, { 251,4196 }, { 252,4196 },
 { 253,4196 }, { 254,4196 }, { 255,4196 }, { 256,4196 }, {   0,  46 },
 {   0,25315 }, {   1,3938 }, {   2,3938 }, {   3,3938 }, {   4,3938 },
 {   5,3938 }, {   6,3938 }, {   7,3938 }, {   8,3938 }, {   9,3938 },
 {  10,3938 }, {  11,3938 }, {  12,3938 }, {  13,3938 }, {  14,3938 },
 {  15,3938 }, {  16,3938 }, {  17,3938 }, {  18,3938 }, {  19,3938 },
 {  20,3938 }, {  21,3938 }, {  22,3938 }, {  23,3938 }, {  24,3938 },
 {  25,3938 }, {  26,3938 }, {  27,3938 }, {  28,3938 }, {  29,3938 },
 {  30,3938 }, {  31,3938 }, {  32,3938 }, {  33,3938 }, {  34,3938 },
 {  35,3938 }, {   0,   0 }, {  37,3938 }, {  38,3938 }, {  39,3938 },

 {  40,3938 }, {  41,3938 }, {  42,3938 }, {  43,3938 }, {  44,3938 },
 {  45,3938 }, {  46,3938 }, {  47,3938 }, {  48,3938 }, {  49,3938 },
 {  50,3938 }, {  51,3938 }, {  52,3938 }, {  53,3938 }, {  54,3938 },
 {  55,3938 }, {  56,3938 }, {  57,3938 }, {  58,3938 }, {  59,3938 },
 {  60,3938 }, {  61,3938 }, {  62,3938 }, {  63,3938 }, {  64,3938 },
 {  65,3938 }, {  66,3938 }, {  67,3938 }, {  68,3938 }, {  69,3938 },
 {  70,3938 }, {  71,3938 }, {  72,3938 }, {  73,3938 }, {  74,3938 },
 {  75,3938 }, {  76,3938 }, {  77,3938 }, {  78,3938 }, {  79,3938 },
 {  80,3938 }, {  81,3938 }, {  82,3938 }, {  83,3938 }, {  84,3938 },
 {  85,3938 }, {  86,3938 }, {  87,3938 }, {  88,3938 }, {  89,3938 },

 {  90,3938 }, {  91,3938 }, {  92,3938 }, {  93,3938 }, {  94,3938 },
 {  95,3938 }, {  96,3938 }, {  97,3938 }, {  98,3938 }, {  99,3938 },
 { 100,3938 }, { 101,3938 }, { 102,3938 }, { 103,3938 }, { 104,3938 },
 { 105,3938 }, { 106,3938 }, { 107,3938 }, { 108,3938 }, { 109,3938 },
 { 110,3938 }, { 111,3938 }, { 112,3938 }, { 113,3938 }, { 114,3938 },
 { 115,3938 }, { 116,3938 }, { 117,3938 }, { 118,3938 }, { 119,3938 },
 { 120,3938 }, { 121,3938 }, { 122,3938 }, { 123,3938 }, { 124,3938 },
 { 125,3938 }, { 126,3938 }, { 127,3938 }, { 128,3938 }, { 129,3938 },
 { 130,3938 }, { 131,3938 }, { 132,3938 }, { 133,3938 }, { 134,3938 },
 { 135,3938 }, { 136,3938 }, { 137,3938 }, { 138,3938 }, { 139,3938 },

 { 140,3938 }, { 141,3938 }, { 142,3938 }, { 143,3938 }, { 144,3938 },
 { 145,3938 }, { 146,3938 }, { 147,3938 }, { 148,3938 }, { 149,3938 },
 { 150,3938 }, { 151,3938 }, { 152,3938 }, { 153,3938 }, { 154,3938 },
 { 155,3938 }, { 156,3938 }, { 157,3938 }, { 158,3938 }, { 159,3938 },
 { 160,3938 }, { 161,3938 }, { 162,3938 }, { 163,3938 }, { 164,3938 },
 { 165,3938 }, { 166,3938 }, { 167,3938 }, { 168,3938 }, { 169,3938 },
 { 170,3938 }, { 171,3938 }, { 172,3938 }, { 173,3938 }, { 174,3938 },
 { 175,3938 }, { 176,3938 }, { 177,3938 }, { 178,3938 }, { 179,3938 },
 { 180,3938 }, { 181,3938 }, { 182,3938 }, { 183,3938 }, { 184,3938 },
 { 185,3938 }, { 186,3938 }, { 187,3938 }, { 188,3938 }, { 189,3938 },

 { 190,3938 }, { 191,3938 }, { 192,3938 }, { 193,3938 }, { 194,3938 },
 { 195,3938 }, { 196,3938 }, { 197,3938 }, { 198,3938 }, { 199,3938 },
 { 200,3938 }, { 201,3938 }, { 202,3938 }, { 203,3938 }, { 204,3938 },
 { 205,3938 }, { 206,3938 }, { 207,3938 }, { 208,3938 }, { 209,3938 },
 { 210,3938 }, { 211,3938 }, { 212,3938 }, { 213,3938 }, { 214,3938 },
 { 215,3938 }, { 216,3938 }, { 217,3938 }, { 218,3938 }, { 219,3938 },
 { 220,3938 }, { 221,3938 }, { 222,3938 }, { 223,3938 }, { 224,3938 },
 { 225,3938 }, { 226,3938 }, { 227,3938 }, { 228,3938 }, { 229,3938 },
 { 230,3938 }, { 231,3938 }, { 232,3938 }, { 233,3938 }, { 234,3938 },
 { 235,3938 }, { 236,3938 }, { 237,3938 }, { 238,3938 }, { 239,3938 },

 { 240,3938 }, { 241,3938 }, { 242,3938 }, { 243,3938 }, { 244,3938 },
 { 245,3938 }, { 246,3938 }, { 247,3938 }, { 248,3938 }, { 249,3938 },
 { 250,3938 }, { 251,3938 }, { 252,3938 }, { 253,3938 }, { 254,3938 },
 { 255,3938 }, { 256,3938 }, {   0,  48 }, {   0,25057 }, {   0,  53 },
 {   0,25055 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,  53 },
 {   0,25050 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   9,3938 },
 {  10,3938 }, {   0,   0 }, {  12,3938 }, {  13,3938 }, {   9,3933 },
 {  10,3933 }, {   0,   0 }, {  12,3933 }, {  13,3933 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {  32,3938 }, {   0,   0 }, {  36,-4907 },
 {   0,   0 }, {   0,   0 }, {  32,3933 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65,3938 }, {  66,3938 },
 {  67,3938 }, {  68,3938 }, {  69,3938 }, {  70,3938 }, {  71,3938 },
 {  72,3938 }, {  73,3938 }, {  74,3938 }, {  75,3938 }, {  76,3938 },
 {  77,3938 }, {  78,3938 }, {  79,3938 }, {  80,3938 }, {  81,3938 },

 {  82,3938 }, {  83,3938 }, {  84,3938 }, {  85,3938 }, {  86,3938 },
 {  87,3938 }, {  88,3938 }, {  89,3938 }, {  90,3938 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  95,3938 }, {   0,   0 },
 {  97,3938 }, {  98,3938 }, {  99,3938 }, { 100,3938 }, { 101,3938 },
 { 102,3938 }, { 103,3938 }, { 104,3938 }, { 105,3938 }, { 106,3938 },
 { 107,3938 }, { 108,3938 }, { 109,3938 }, { 110,3938 }, { 111,3938 },
 { 112,3938 }, { 113,3938 }, { 114,3938 }, { 115,3938 }, { 116,3938 },
 { 117,3938 }, { 118,3938 }, { 119,3938 }, { 120,3938 }, { 121,3938 },
 { 122,3938 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, { 128,3938 }, { 129,3938 }, { 130,3938 }, { 131,3938 },

 { 132,3938 }, { 133,3938 }, { 134,3938 }, { 135,3938 }, { 136,3938 },
 { 137,3938 }, { 138,3938 }, { 139,3938 }, { 140,3938 }, { 141,3938 },
 { 142,3938 }, { 143,3938 }, { 144,3938 }, { 145,3938 }, { 146,3938 },
 { 147,3938 }, { 148,3938 }, { 149,3938 }, { 150,3938 }, { 151,3938 },
 { 152,3938 }, { 153,3938 }, { 154,3938 }, { 155,3938 }, { 156,3938 },
 { 157,3938 }, { 158,3938 }, { 159,3938 }, { 160,3938 }, { 161,3938 },
 { 162,3938 }, { 163,3938 }, { 164,3938 }, { 165,3938 }, { 166,3938 },
 { 167,3938 }, { 168,3938 }, { 169,3938 }, { 170,3938 }, { 171,3938 },
 { 172,3938 }, { 173,3938 }, { 174,3938 }, { 175,3938 }, { 176,3938 },
 { 177,3938 }, { 178,3938 }, { 179,3938 }, { 180,3938 }, { 181,3938 },

 { 182,3938 }, { 183,3938 }, { 184,3938 }, { 185,3938 }, { 186,3938 },
 { 187,3938 }, { 188,3938 }, { 189,3938 }, { 190,3938 }, { 191,3938 },
 { 192,3938 }, { 193,3938 }, { 194,3938 }, { 195,3938 }, { 196,3938 },
 { 197,3938 }, { 198,3938 }, { 199,3938 }, { 200,3938 }, { 201,3938 },
 { 202,3938 }, { 203,3938 }, { 204,3938 }, { 205,3938 }, { 206,3938 },
 { 207,3938 }, { 208,3938 }, { 209,3938 }, { 210,3938 }, { 211,3938 },
 { 212,3938 }, { 213,3938 }, { 214,3938 }, { 215,3938 }, { 216,3938 },
 { 217,3938 }, { 218,3938 }, { 219,3938 }, { 220,3938 }, { 221,3938 },
 { 222,3938 }, { 223,3938 }, { 224,3938 }, { 225,3938 }, { 226,3938 },
 { 227,3938 }, { 228,3938 }, { 229,3938 }, { 230,3938 }, { 231,3938 },

 { 232,3938 }, { 233,3938 }, { 234,3938 }, { 235,3938 }, { 236,3938 },
 { 237,3938 }, { 238,3938 }, { 239,3938 }, { 240,3938 }, { 241,3938 },
 { 242,3938 }, { 243,3938 }, { 244,3938 }, { 245,3938 }, { 246,3938 },
 { 247,3938 }, { 248,3938 }, { 249,3938 }, { 250,3938 }, { 251,3938 },
 { 252,3938 }, { 253,3938 }, { 254,3938 }, { 255,3938 }, {   0,  24 },
 {   0,24800 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,  26 },
 {   0,24795 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   9,4196 },
 {  10,4201 }, {   0,   0 }, {  12,4196 }, {  13,4201 }, {   9,4212 },
 {  10,4212 }, {   0,   0 }, {  12,4212 }, {  13,4212 }, {   0,   0 },
 {   0,  26 }, {   0,24779 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   9,4196 }, {  10,4196 }, {  32,4196 }, {  12,4196 }, {  13,4196 },
 {   0,   0 }, {   0,   0 }, {  32,4212 }, {   0,   0 }, {  39,-5170 },
 {   0,   0 }, {   0,   1 }, {   0,24758 }, {   0,  69 }, {   0,24756 },
 {  45,-5125 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   9,   0 }, {  10,   0 }, {  32,4196 }, {  12,   0 },
 {  13,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  32,   0 },

 {   0,   0 }, {   0,   0 }, {  33,   0 }, {   0,   0 }, {  35,   0 },
 {   0,   0 }, {  37,   0 }, {  38,   0 }, {   0,  67 }, {   0,24716 },
 {   0,   0 }, {  42,   0 }, {  43,   0 }, {   0,   0 }, {  45,   0 },
 {   0,   0 }, {  47,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,  70 }, {   0,24699 }, {   0,   0 }, {   0,   0 }, {  60,   0 },
 {  61,   0 }, {  62,   0 }, {  63,   0 }, {  64,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  33, -40 }, {   0,   0 }, {  35, -40 },
 {   0,   0 }, {  37, -40 }, {  38, -40 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {  42, -40 }, {  43, -40 }, {   0,   0 }, {  45, -40 },
 {   0,   0 }, {  47, -40 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  94,   0 }, {   0,   0 },
 {  96,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  60, -40 },
 {  61, -40 }, {  62, -40 }, {  63, -40 }, {  64, -40 }, {  48,   0 },
 {  49,   0 }, {  50,   0 }, {  51,   0 }, {  52,   0 }, {  53,   0 },
 {  54,   0 }, {  55,   0 }, {  56,   0 }, {  57,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,  44 }, {   0,24637 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, { 124,   0 }, {   0,   0 },
 { 126,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  94, -40 }, {   0,   0 },
 {  96, -40 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  36,-5604 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, { 124, -40 }, {   0,   0 },
 { 126, -40 }, {  48,4428 }, {  49,4428 }, {  50,4428 }, {  51,4428 },
 {  52,4428 }, {  53,4428 }, {  54,4428 }, {  55,4428 }, {  56,4428 },
 {  57,4428 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65,4428 }, {  66,4428 },
 {  67,4428 }, {  68,4428 }, {  69,4428 }, {  70,4428 }, {  71,4428 },
 {  72,4428 }, {  73,4428 }, {  74,4428 }, {  75,4428 }, {  76,4428 },
 {  77,4428 }, {  78,4428 }, {  79,4428 }, {  80,4428 }, {  81,4428 },
 {  82,4428 }, {  83,4428 }, {  84,4428 }, {  85,4428 }, {  86,4428 },
 {  87,4428 }, {  88,4428 }, {  89,4428 }, {  90,4428 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  95,4428 }, {   0,   0 },
 {  97,4428 }, {  98,4428 }, {  99,4428 }, { 100,4428 }, { 101,4428 },
 { 102,4428 }, { 103,4428 }, { 104,4428 }, { 105,4428 }, { 106,4428 },
 { 107,4428 }, { 108,4428 }, { 109,4428 }, { 110,4428 }, { 111,4428 },

 { 112,4428 }, { 113,4428 }, { 114,4428 }, { 115,4428 }, { 116,4428 },
 { 117,4428 }, { 118,4428 }, { 119,4428 }, { 120,4428 }, { 121,4428 },
 { 122,4428 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, { 128,4428 }, { 129,4428 }, { 130,4428 }, { 131,4428 },
 { 132,4428 }, { 133,4428 }, { 134,4428 }, { 135,4428 }, { 136,4428 },
 { 137,4428 }, { 138,4428 }, { 139,4428 }, { 140,4428 }, { 141,4428 },
 { 142,4428 }, { 143,4428 }, { 144,4428 }, { 145,4428 }, { 146,4428 },
 { 147,4428 }, { 148,4428 }, { 149,4428 }, { 150,4428 }, { 151,4428 },
 { 152,4428 }, { 153,4428 }, { 154,4428 }, { 155,4428 }, { 156,4428 },
 { 157,4428 }, { 158,4428 }, { 159,4428 }, { 160,4428 }, { 161,4428 },

 { 162,4428 }, { 163,4428 }, { 164,4428 }, { 165,4428 }, { 166,4428 },
 { 167,4428 }, { 168,4428 }, { 169,4428 }, { 170,4428 }, { 171,4428 },
 { 172,4428 }, { 173,4428 }, { 174,4428 }, { 175,4428 }, { 176,4428 },
 { 177,4428 }, { 178,4428 }, { 179,4428 }, { 180,4428 }, { 181,4428 },
 { 182,4428 }, { 183,4428 }, { 184,4428 }, { 185,4428 }, { 186,4428 },
 { 187,4428 }, { 188,4428 }, { 189,4428 }, { 190,4428 }, { 191,4428 },
 { 192,4428 }, { 193,4428 }, { 194,4428 }, { 195,4428 }, { 196,4428 },
 { 197,4428 }, { 198,4428 }, { 199,4428 }, { 200,4428 }, { 201,4428 },
 { 202,4428 }, { 203,4428 }, { 204,4428 }, { 205,4428 }, { 206,4428 },
 { 207,4428 }, { 208,4428 }, { 209,4428 }, { 210,4428 }, { 211,4428 },

 { 212,4428 }, { 213,4428 }, { 214,4428 }, { 215,4428 }, { 216,4428 },
 { 217,4428 }, { 218,4428 }, { 219,4428 }, { 220,4428 }, { 221,4428 },
 { 222,4428 }, { 223,4428 }, { 224,4428 }, { 225,4428 }, { 226,4428 },
 { 227,4428 }, { 228,4428 }, { 229,4428 }, { 230,4428 }, { 231,4428 },
 { 232,4428 }, { 233,4428 }, { 234,4428 }, { 235,4428 }, { 236,4428 },
 { 237,4428 }, { 238,4428 }, { 239,4428 }, { 240,4428 }, { 241,4428 },
 { 242,4428 }, { 243,4428 }, { 244,4428 }, { 245,4428 }, { 246,4428 },
 { 247,4428 }, { 248,4428 }, { 249,4428 }, { 250,4428 }, { 251,4428 },
 { 252,4428 }, { 253,4428 }, { 254,4428 }, { 255,4428 }, {   0,   1 },
 {   0,24380 }, {   1,4428 }, {   2,4428 }, {   3,4428 }, {   4,4428 },

 {   5,4428 }, {   6,4428 }, {   7,4428 }, {   8,4428 }, {   9,4428 },
 {   0,   0 }, {  11,4428 }, {  12,4428 }, {   0,   0 }, {  14,4428 },
 {  15,4428 }, {  16,4428 }, {  17,4428 }, {  18,4428 }, {  19,4428 },
 {  20,4428 }, {  21,4428 }, {  22,4428 }, {  23,4428 }, {  24,4428 },
 {  25,4428 }, {  26,4428 }, {  27,4428 }, {  28,4428 }, {  29,4428 },
 {  30,4428 }, {  31,4428 }, {  32,4428 }, {  33,4686 }, {  34,4428 },
 {  35,4686 }, {  36,4428 }, {  37,4686 }, {  38,4686 }, {  39,4428 },
 {  40,4428 }, {  41,4428 }, {  42,4686 }, {  43,4686 }, {  44,4428 },
 {  45,4686 }, {  46,4428 }, {  47,4686 }, {  48,4428 }, {  49,4428 },
 {  50,4428 }, {  51,4428 }, {  52,4428 }, {  53,4428 }, {  54,4428 },

 {  55,4428 }, {  56,4428 }, {  57,4428 }, {  58,4428 }, {  59,4428 },
 {  60,4686 }, {  61,4686 }, {  62,4686 }, {  63,4686 }, {  64,4686 },
 {  65,4428 }, {  66,4428 }, {  67,4428 }, {  68,4428 }, {  69,4428 },
 {  70,4428 }, {  71,4428 }, {  72,4428 }, {  73,4428 }, {  74,4428 },
 {  75,4428 }, {  76,4428 }, {  77,4428 }, {  78,4428 }, {  79,4428 },
 {  80,4428 }, {  81,4428 }, {  82,4428 }, {  83,4428 }, {  84,4428 },
 {  85,4428 }, {  86,4428 }, {  87,4428 }, {  88,4428 }, {  89,4428 },
 {  90,4428 }, {  91,4428 }, {  92,4428 }, {  93,4428 }, {  94,4686 },
 {  95,4428 }, {  96,4686 }, {  97,4428 }, {  98,4428 }, {  99,4428 },
 { 100,4428 }, { 101,4428 }, { 102,4428 }, { 103,4428 }, { 104,4428 },

 { 105,4428 }, { 106,4428 }, { 107,4428 }, { 108,4428 }, { 109,4428 },
 { 110,4428 }, { 111,4428 }, { 112,4428 }, { 113,4428 }, { 114,4428 },
 { 115,4428 }, { 116,4428 }, { 117,4428 }, { 118,4428 }, { 119,4428 },
 { 120,4428 }, { 121,4428 }, { 122,4428 }, { 123,4428 }, { 124,4686 },
 { 125,4428 }, { 126,4686 }, { 127,4428 }, { 128,4428 }, { 129,4428 },
 { 130,4428 }, { 131,4428 }, { 132,4428 }, { 133,4428 }, { 134,4428 },
 { 135,4428 }, { 136,4428 }, { 137,4428 }, { 138,4428 }, { 139,4428 },
 { 140,4428 }, { 141,4428 }, { 142,4428 }, { 143,4428 }, { 144,4428 },
 { 145,4428 }, { 146,4428 }, { 147,4428 }, { 148,4428 }, { 149,4428 },
 { 150,4428 }, { 151,4428 }, { 152,4428 }, { 153,4428 }, { 154,4428 },

 { 155,4428 }, { 156,4428 }, { 157,4428 }, { 158,4428 }, { 159,4428 },
 { 160,4428 }, { 161,4428 }, { 162,4428 }, { 163,4428 }, { 164,4428 },
 { 165,4428 }, { 166,4428 }, { 167,4428 }, { 168,4428 }, { 169,4428 },
 { 170,4428 }, { 171,4428 }, { 172,4428 }, { 173,4428 }, { 174,4428 },
 { 175,4428 }, { 176,4428 }, { 177,4428 }, { 178,4428 }, { 179,4428 },
 { 180,4428 }, { 181,4428 }, { 182,4428 }, { 183,4428 }, { 184,4428 },
 { 185,4428 }, { 186,4428 }, { 187,4428 }, { 188,4428 }, { 189,4428 },
 { 190,4428 }, { 191,4428 }, { 192,4428 }, { 193,4428 }, { 194,4428 },
 { 195,4428 }, { 196,4428 }, { 197,4428 }, { 198,4428 }, { 199,4428 },
 { 200,4428 }, { 201,4428 }, { 202,4428 }, { 203,4428 }, { 204,4428 },

 { 205,4428 }, { 206,4428 }, { 207,4428 }, { 208,4428 }, { 209,4428 },
 { 210,4428 }, { 211,4428 }, { 212,4428 }, { 213,4428 }, { 214,4428 },
 { 215,4428 }, { 216,4428 }, { 217,4428 }, { 218,4428 }, { 219,4428 },
 { 220,4428 }, { 221,4428 }, { 222,4428 }, { 223,4428 }, { 224,4428 },
 { 225,4428 }, { 226,4428 }, { 227,4428 }, { 228,4428 }, { 229,4428 },
 { 230,4428 }, { 231,4428 }, { 232,4428 }, { 233,4428 }, { 234,4428 },
 { 235,4428 }, { 236,4428 }, { 237,4428 }, { 238,4428 }, { 239,4428 },
 { 240,4428 }, { 241,4428 }, { 242,4428 }, { 243,4428 }, { 244,4428 },
 { 245,4428 }, { 246,4428 }, { 247,4428 }, { 248,4428 }, { 249,4428 },
 { 250,4428 }, { 251,4428 }, { 252,4428 }, { 253,4428 }, { 254,4428 },

 { 255,4428 }, { 256,4428 }, {   0,  73 }, {   0,24122 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   2 }, {   0,24097 }, {   0,  73 },
 {   0,24095 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {  48,   0 }, {  49,   0 }, {  50,   0 }, {  51,   0 },
 {  52,   0 }, {  53,   0 }, {  54,   0 }, {  55,   0 }, {  56,   0 },
 {  57,   0 }, {  33,4686 }, {   0,   0 }, {  35,4686 }, {   0,   0 },
 {  37,4686 }, {  38,4686 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  42,4686 }, {  43,4686 }, {  69,4686 }, {  45,4686 }, {   0,   0 },
 {  47,4686 }, {  46,-5786 }, {   0,   0 }, {  48,4686 }, {  49,4686 },
 {  50,4686 }, {  51,4686 }, {  52,4686 }, {  53,4686 }, {  54,4686 },
 {  55,4686 }, {  56,4686 }, {  57,4686 }, {  60,4686 }, {  61,4686 },
 {  62,4686 }, {  63,4686 }, {  64,4686 }, {   0,  72 }, {   0,24031 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  69,4659 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, { 101,4686 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  94,4686 }, {   0,   0 }, {  96,4686 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, { 101,4659 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  46, -64 }, {   0,   0 }, {  48,   0 }, {  49,   0 }, {  50,   0 },
 {  51,   0 }, {  52,   0 }, {  53,   0 }, {  54,   0 }, {  55,   0 },

 {  56,   0 }, {  57,   0 }, { 124,4686 }, {   0,   0 }, { 126,4686 },
 {   0,  71 }, {   0,23969 }, {   0,   0 }, {   0,   0 }, {  65,  62 },
 {  66,  62 }, {  67,  62 }, {  68,  62 }, {  69, 154 }, {  70,  62 },
 {  71,  62 }, {  72,  62 }, {  73,  62 }, {  74,  62 }, {  75,  62 },
 {  76,  62 }, {  77,  62 }, {  78,  62 }, {  79,  62 }, {  80,  62 },
 {  81,  62 }, {  82,  62 }, {  83,  62 }, {  84,  62 }, {  85,  62 },
 {  86,  62 }, {  87,  62 }, {  88,  62 }, {  89,  62 }, {  90,  62 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  95,  62 },
 {   0,   0 }, {  97,  62 }, {  98,  62 }, {  99,  62 }, { 100,  62 },
 { 101, 154 }, { 102,  62 }, { 103,  62 }, { 104,  62 }, { 105,  62 },

 { 106,  62 }, { 107,  62 }, { 108,  62 }, { 109,  62 }, { 110,  62 },
 { 111,  62 }, { 112,  62 }, { 113,  62 }, { 114,  62 }, { 115,  62 },
 { 116,  62 }, { 117,  62 }, { 118,  62 }, { 119,  62 }, { 120,  62 },
 { 121,  62 }, { 122,  62 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  65,   0 }, {  66,   0 }, {  67,   0 }, {  68,   0 },
 {  69,   0 }, {  70,   0 }, {  71,   0 }, {  72,   0 }, {  73,   0 },
 {  74,   0 }, {  75,   0 }, {  76,   0 }, {  77,   0 }, {  78,   0 },
 {  79,   0 }, {  80,   0 }, {  81,   0 }, {  82,   0 }, {  83,   0 },
 {  84,   0 }, {  85,   0 }, {  86,   0 }, {  87,   0 }, {  88,   0 },
 {  89,   0 }, {  90,   0 }, {   0,  71 }, {   0,23877 }, {   0,   0 },

 {   0,   0 }, {  95,   0 }, {   0,   0 }, {  97,   0 }, {  98,   0 },
 {  99,   0 }, { 100,   0 }, { 101,   0 }, { 102,   0 }, { 103,   0 },
 { 104,   0 }, { 105,   0 }, { 106,   0 }, { 107,   0 }, { 108,   0 },
 { 109,   0 }, { 110,   0 }, { 111,   0 }, { 112,   0 }, { 113,   0 },
 { 114,   0 }, { 115,   0 }, { 116,   0 }, { 117,   0 }, { 118,   0 },
 { 119,   0 }, { 120,   0 }, { 121,   0 }, { 122,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  43,4532 }, {   0,   0 }, {  45,4532 }, {   0,   0 },
 {   0,   0 }, {  48,4545 }, {  49,4545 }, {  50,4545 }, {  51,4545 },

 {  52,4545 }, {  53,4545 }, {  54,4545 }, {  55,4545 }, {  56,4545 },
 {  57,4545 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65, -92 }, {  66, -92 },
 {  67, -92 }, {  68, -92 }, {  69, -92 }, {  70, -92 }, {  71, -92 },
 {  72, -92 }, {  73, -92 }, {  74, -92 }, {  75, -92 }, {  76, -92 },
 {  77, -92 }, {  78, -92 }, {  79, -92 }, {  80, -92 }, {  81, -92 },
 {  82, -92 }, {  83, -92 }, {  84, -92 }, {  85, -92 }, {  86, -92 },
 {  87, -92 }, {  88, -92 }, {  89, -92 }, {  90, -92 }, {   0,  64 },
 {   0,23785 }, {   0,   0 }, {   0,   0 }, {  95, -92 }, {   0,   0 },
 {  97, -92 }, {  98, -92 }, {  99, -92 }, { 100, -92 }, { 101, -92 },

 { 102, -92 }, { 103, -92 }, { 104, -92 }, { 105, -92 }, { 106, -92 },
 { 107, -92 }, { 108, -92 }, { 109, -92 }, { 110, -92 }, { 111, -92 },
 { 112, -92 }, { 113, -92 }, { 114, -92 }, { 115, -92 }, { 116, -92 },
 { 117, -92 }, { 118, -92 }, { 119, -92 }, { 120, -92 }, { 121, -92 },
 { 122, -92 }, {   0,   0 }, {   0,   0 }, {  33,-971 }, {   0,   0 },
 {  35,-971 }, {   0,   0 }, {  37,-971 }, {  38,-971 }, {   0,  66 },
 {   0,23745 }, {   0,   0 }, {  42,-971 }, {  43,-971 }, {   0,   0 },
 {  45,-971 }, {   0,   0 }, {  47,-971 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {  60,-971 }, {  61,-971 }, {  62,-971 }, {  63,-971 }, {  64,-971 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  33,-1011 }, {   0,   0 },
 {  35,-1011 }, {   0,   0 }, {  37,-1011 }, {  38,-1011 }, {   0,  63 },
 {   0,23705 }, {   0,   0 }, {  42,-1011 }, {  43,-1011 }, {   0,   0 },
 {  45,-1011 }, {   0,   0 }, {  47,-1011 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  94,-971 },
 {   0,   0 }, {  96,-971 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  60,-1011 }, {  61,-1011 }, {  62,-1011 }, {  63,-1011 }, {  64,-1011 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  33,-1051 }, {   0,   0 },
 {  35,-1051 }, {   0,   0 }, {  37,-1051 }, {  38,-1051 }, {   0,  65 },
 {   0,23665 }, {   0,   0 }, {  42,-1051 }, {  43,-1051 }, { 124,-971 },
 {  45,-1051 }, { 126,-971 }, {  47,-1051 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  94,-1011 },
 {   0,   0 }, {  96,-1011 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  60,-1051 }, {  61,-1051 }, {  62,-1051 }, {  63,-1051 }, {  64,-1051 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  33,-1091 }, {   0,   0 },
 {  35,-1091 }, {   0,   0 }, {  37,-1091 }, {  38,-1091 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {  42,-1091 }, {  43,-1091 }, { 124,-1011 },
 {  45,-1091 }, { 126,-1011 }, {  47,-1091 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  94,-1051 },
 {   0,   0 }, {  96,-1051 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  60,-1091 }, {  61,-1091 }, {  62,-1091 }, {  63,-1091 }, {  64,-1091 },
 {   0,  78 }, {   0,23599 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, { 124,-1051 },
 {   0,   0 }, { 126,-1051 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  94,-1091 },
 {   0,   0 }, {  96,-1091 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  36,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  48,   0 },
 {  49,   0 }, {  50,   0 }, {  51,   0 }, {  52,   0 }, {  53,   0 },
 {  54,   0 }, {  55,   0 }, {  56,   0 }, {  57,   0 }, { 124,-1091 },
 {   0,   0 }, { 126,-1091 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  65,   0 }, {  66,   0 }, {  67,   0 }, {  68,   0 },
 {  69,   0 }, {  70,   0 }, {  71,   0 }, {  72,   0 }, {  73,   0 },

 {  74,   0 }, {  75,   0 }, {  76,   0 }, {  77,   0 }, {  78,   0 },
 {  79,   0 }, {  80,   0 }, {  81,   0 }, {  82,   0 }, {  83,   0 },
 {  84,   0 }, {  85,   0 }, {  86,   0 }, {  87,   0 }, {  88,   0 },
 {  89,   0 }, {  90,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  95,   0 }, {   0,   0 }, {  97,   0 }, {  98,   0 },
 {  99,   0 }, { 100,   0 }, { 101,   0 }, { 102,   0 }, { 103,   0 },
 { 104,   0 }, { 105,   0 }, { 106,   0 }, { 107,   0 }, { 108,   0 },
 { 109,   0 }, { 110,   0 }, { 111,   0 }, { 112,   0 }, { 113,   0 },
 { 114,   0 }, { 115,   0 }, { 116,   0 }, { 117,   0 }, { 118,   0 },
 { 119,   0 }, { 120,   0 }, { 121,   0 }, { 122,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, { 128,   0 },
 { 129,   0 }, { 130,   0 }, { 131,   0 }, { 132,   0 }, { 133,   0 },
 { 134,   0 }, { 135,   0 }, { 136,   0 }, { 137,   0 }, { 138,   0 },
 { 139,   0 }, { 140,   0 }, { 141,   0 }, { 142,   0 }, { 143,   0 },
 { 144,   0 }, { 145,   0 }, { 146,   0 }, { 147,   0 }, { 148,   0 },
 { 149,   0 }, { 150,   0 }, { 151,   0 }, { 152,   0 }, { 153,   0 },
 { 154,   0 }, { 155,   0 }, { 156,   0 }, { 157,   0 }, { 158,   0 },
 { 159,   0 }, { 160,   0 }, { 161,   0 }, { 162,   0 }, { 163,   0 },
 { 164,   0 }, { 165,   0 }, { 166,   0 }, { 167,   0 }, { 168,   0 },
 { 169,   0 }, { 170,   0 }, { 171,   0 }, { 172,   0 }, { 173,   0 },

 { 174,   0 }, { 175,   0 }, { 176,   0 }, { 177,   0 }, { 178,   0 },
 { 179,   0 }, { 180,   0 }, { 181,   0 }, { 182,   0 }, { 183,   0 },
 { 184,   0 }, { 185,   0 }, { 186,   0 }, { 187,   0 }, { 188,   0 },
 { 189,   0 }, { 190,   0 }, { 191,   0 }, { 192,   0 }, { 193,   0 },
 { 194,   0 }, { 195,   0 }, { 196,   0 }, { 197,   0 }, { 198,   0 },
 { 199,   0 }, { 200,   0 }, { 201,   0 }, { 202,   0 }, { 203,   0 },
 { 204,   0 }, { 205,   0 }, { 206,   0 }, { 207,   0 }, { 208,   0 },
 { 209,   0 }, { 210,   0 }, { 211,   0 }, { 212,   0 }, { 213,   0 },
 { 214,   0 }, { 215,   0 }, { 216,   0 }, { 217,   0 }, { 218,   0 },
 { 219,   0 }, { 220,   0 }, { 221,   0 }, { 222,   0 }, { 223,   0 },

 { 224,   0 }, { 225,   0 }, { 226,   0 }, { 227,   0 }, { 228,   0 },
 { 229,   0 }, { 230,   0 }, { 231,   0 }, { 232,   0 }, { 233,   0 },
 { 234,   0 }, { 235,   0 }, { 236,   0 }, { 237,   0 }, { 238,   0 },
 { 239,   0 }, { 240,   0 }, { 241,   0 }, { 242,   0 }, { 243,   0 },
 { 244,   0 }, { 245,   0 }, { 246,   0 }, { 247,   0 }, { 248,   0 },
 { 249,   0 }, { 250,   0 }, { 251,   0 }, { 252,   0 }, { 253,   0 },
 { 254,   0 }, { 255,   0 }, {   0,  12 }, {   0,23342 }, {   1,   0 },
 {   2,   0 }, {   3,   0 }, {   4,   0 }, {   5,   0 }, {   6,   0 },
 {   7,   0 }, {   8,   0 }, {   9,   0 }, {  10,   0 }, {  11,   0 },
 {  12,   0 }, {  13,   0 }, {  14,   0 }, {  15,   0 }, {  16,   0 },

 {  17,   0 }, {  18,   0 }, {  19,   0 }, {  20,   0 }, {  21,   0 },
 {  22,   0 }, {  23,   0 }, {  24,   0 }, {  25,   0 }, {  26,   0 },
 {  27,   0 }, {  28,   0 }, {  29,   0 }, {  30,   0 }, {  31,   0 },
 {  32,   0 }, {  33,   0 }, {  34,   0 }, {  35,   0 }, {  36,   0 },
 {  37,   0 }, {  38,   0 }, {   0,   0 }, {  40,   0 }, {  41,   0 },
 {  42,   0 }, {  43,   0 }, {  44,   0 }, {  45,   0 }, {  46,   0 },
 {  47,   0 }, {  48,   0 }, {  49,   0 }, {  50,   0 }, {  51,   0 },
 {  52,   0 }, {  53,   0 }, {  54,   0 }, {  55,   0 }, {  56,   0 },
 {  57,   0 }, {  58,   0 }, {  59,   0 }, {  60,   0 }, {  61,   0 },
 {  62,   0 }, {  63,   0 }, {  64,   0 }, {  65,   0 }, {  66,   0 },

 {  67,   0 }, {  68,   0 }, {  69,   0 }, {  70,   0 }, {  71,   0 },
 {  72,   0 }, {  73,   0 }, {  74,   0 }, {  75,   0 }, {  76,   0 },
 {  77,   0 }, {  78,   0 }, {  79,   0 }, {  80,   0 }, {  81,   0 },
 {  82,   0 }, {  83,   0 }, {  84,   0 }, {  85,   0 }, {  86,   0 },
 {  87,   0 }, {  88,   0 }, {  89,   0 }, {  90,   0 }, {  91,   0 },
 {  92,   0 }, {  93,   0 }, {  94,   0 }, {  95,   0 }, {  96,   0 },
 {  97,   0 }, {  98,   0 }, {  99,   0 }, { 100,   0 }, { 101,   0 },
 { 102,   0 }, { 103,   0 }, { 104,   0 }, { 105,   0 }, { 106,   0 },
 { 107,   0 }, { 108,   0 }, { 109,   0 }, { 110,   0 }, { 111,   0 },
 { 112,   0 }, { 113,   0 }, { 114,   0 }, { 115,   0 }, { 116,   0 },

 { 117,   0 }, { 118,   0 }, { 119,   0 }, { 120,   0 }, { 121,   0 },
 { 122,   0 }, { 123,   0 }, { 124,   0 }, { 125,   0 }, { 126,   0 },
 { 127,   0 }, { 128,   0 }, { 129,   0 }, { 130,   0 }, { 131,   0 },
 { 132,   0 }, { 133,   0 }, { 134,   0 }, { 135,   0 }, { 136,   0 },
 { 137,   0 }, { 138,   0 }, { 139,   0 }, { 140,   0 }, { 141,   0 },
 { 142,   0 }, { 143,   0 }, { 144,   0 }, { 145,   0 }, { 146,   0 },
 { 147,   0 }, { 148,   0 }, { 149,   0 }, { 150,   0 }, { 151,   0 },
 { 152,   0 }, { 153,   0 }, { 154,   0 }, { 155,   0 }, { 156,   0 },
 { 157,   0 }, { 158,   0 }, { 159,   0 }, { 160,   0 }, { 161,   0 },
 { 162,   0 }, { 163,   0 }, { 164,   0 }, { 165,   0 }, { 166,   0 },

 { 167,   0 }, { 168,   0 }, { 169,   0 }, { 170,   0 }, { 171,   0 },
 { 172,   0 }, { 173,   0 }, { 174,   0 }, { 175,   0 }, { 176,   0 },
 { 177,   0 }, { 178,   0 }, { 179,   0 }, { 180,   0 }, { 181,   0 },
 { 182,   0 }, { 183,   0 }, { 184,   0 }, { 185,   0 }, { 186,   0 },
 { 187,   0 }, { 188,   0 }, { 189,   0 }, { 190,   0 }, { 191,   0 },
 { 192,   0 }, { 193,   0 }, { 194,   0 }, { 195,   0 }, { 196,   0 },
 { 197,   0 }, { 198,   0 }, { 199,   0 }, { 200,   0 }, { 201,   0 },
 { 202,   0 }, { 203,   0 }, { 204,   0 }, { 205,   0 }, { 206,   0 },
 { 207,   0 }, { 208,   0 }, { 209,   0 }, { 210,   0 }, { 211,   0 },
 { 212,   0 }, { 213,   0 }, { 214,   0 }, { 215,   0 }, { 216,   0 },

 { 217,   0 }, { 218,   0 }, { 219,   0 }, { 220,   0 }, { 221,   0 },
 { 222,   0 }, { 223,   0 }, { 224,   0 }, { 225,   0 }, { 226,   0 },
 { 227,   0 }, { 228,   0 }, { 229,   0 }, { 230,   0 }, { 231,   0 },
 { 232,   0 }, { 233,   0 }, { 234,   0 }, { 235,   0 }, { 236,   0 },
 { 237,   0 }, { 238,   0 }, { 239,   0 }, { 240,   0 }, { 241,   0 },
 { 242,   0 }, { 243,   0 }, { 244,   0 }, { 245,   0 }, { 246,   0 },
 { 247,   0 }, { 248,   0 }, { 249,   0 }, { 250,   0 }, { 251,   0 },
 { 252,   0 }, { 253,   0 }, { 254,   0 }, { 255,   0 }, { 256,   0 },
 {   0,   9 }, {   0,23084 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   9 }, {   0,23079 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   9,   0 }, {  10,   5 }, {   0,   0 }, {  12,   0 }, {  13,   5 },
 {   9,3806 }, {  10,3806 }, {   0,   0 }, {  12,3806 }, {  13,3806 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  32,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  32,3806 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  39,-6794 }, {  45,-6902 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  45,-6792 }, {   0,   5 }, {   0,23032 }, {   1,   0 },
 {   2,   0 }, {   3,   0 }, {   4,   0 }, {   5,   0 }, {   6,   0 },

 {   7,   0 }, {   8,   0 }, {   9,   0 }, {  10,   0 }, {  11,   0 },
 {  12,   0 }, {  13,   0 }, {  14,   0 }, {  15,   0 }, {  16,   0 },
 {  17,   0 }, {  18,   0 }, {  19,   0 }, {  20,   0 }, {  21,   0 },
 {  22,   0 }, {  23,   0 }, {  24,   0 }, {  25,   0 }, {  26,   0 },
 {  27,   0 }, {  28,   0 }, {  29,   0 }, {  30,   0 }, {  31,   0 },
 {  32,   0 }, {  33,   0 }, {  34,   0 }, {  35,   0 }, {  36,   0 },
 {  37,   0 }, {  38,   0 }, {  39,   0 }, {  40,   0 }, {  41,   0 },
 {   0,   0 }, {  43,   0 }, {  44,   0 }, {  45,   0 }, {  46,   0 },
 {   0,   0 }, {  48,   0 }, {  49,   0 }, {  50,   0 }, {  51,   0 },
 {  52,   0 }, {  53,   0 }, {  54,   0 }, {  55,   0 }, {  56,   0 },

 {  57,   0 }, {  58,   0 }, {  59,   0 }, {  60,   0 }, {  61,   0 },
 {  62,   0 }, {  63,   0 }, {  64,   0 }, {  65,   0 }, {  66,   0 },
 {  67,   0 }, {  68,   0 }, {  69,   0 }, {  70,   0 }, {  71,   0 },
 {  72,   0 }, {  73,   0 }, {  74,   0 }, {  75,   0 }, {  76,   0 },
 {  77,   0 }, {  78,   0 }, {  79,   0 }, {  80,   0 }, {  81,   0 },
 {  82,   0 }, {  83,   0 }, {  84,   0 }, {  85,   0 }, {  86,   0 },
 {  87,   0 }, {  88,   0 }, {  89,   0 }, {  90,   0 }, {  91,   0 },
 {  92,   0 }, {  93,   0 }, {  94,   0 }, {  95,   0 }, {  96,   0 },
 {  97,   0 }, {  98,   0 }, {  99,   0 }, { 100,   0 }, { 101,   0 },
 { 102,   0 }, { 103,   0 }, { 104,   0 }, { 105,   0 }, { 106,   0 },

 { 107,   0 }, { 108,   0 }, { 109,   0 }, { 110,   0 }, { 111,   0 },
 { 112,   0 }, { 113,   0 }, { 114,   0 }, { 115,   0 }, { 116,   0 },
 { 117,   0 }, { 118,   0 }, { 119,   0 }, { 120,   0 }, { 121,   0 },
 { 122,   0 }, { 123,   0 }, { 124,   0 }, { 125,   0 }, { 126,   0 },
 { 127,   0 }, { 128,   0 }, { 129,   0 }, { 130,   0 }, { 131,   0 },
 { 132,   0 }, { 133,   0 }, { 134,   0 }, { 135,   0 }, { 136,   0 },
 { 137,   0 }, { 138,   0 }, { 139,   0 }, { 140,   0 }, { 141,   0 },
 { 142,   0 }, { 143,   0 }, { 144,   0 }, { 145,   0 }, { 146,   0 },
 { 147,   0 }, { 148,   0 }, { 149,   0 }, { 150,   0 }, { 151,   0 },
 { 152,   0 }, { 153,   0 }, { 154,   0 }, { 155,   0 }, { 156,   0 },

 { 157,   0 }, { 158,   0 }, { 159,   0 }, { 160,   0 }, { 161,   0 },
 { 162,   0 }, { 163,   0 }, { 164,   0 }, { 165,   0 }, { 166,   0 },
 { 167,   0 }, { 168,   0 }, { 169,   0 }, { 170,   0 }, { 171,   0 },
 { 172,   0 }, { 173,   0 }, { 174,   0 }, { 175,   0 }, { 176,   0 },
 { 177,   0 }, { 178,   0 }, { 179,   0 }, { 180,   0 }, { 181,   0 },
 { 182,   0 }, { 183,   0 }, { 184,   0 }, { 185,   0 }, { 186,   0 },
 { 187,   0 }, { 188,   0 }, { 189,   0 }, { 190,   0 }, { 191,   0 },
 { 192,   0 }, { 193,   0 }, { 194,   0 }, { 195,   0 }, { 196,   0 },
 { 197,   0 }, { 198,   0 }, { 199,   0 }, { 200,   0 }, { 201,   0 },
 { 202,   0 }, { 203,   0 }, { 204,   0 }, { 205,   0 }, { 206,   0 },

 { 207,   0 }, { 208,   0 }, { 209,   0 }, { 210,   0 }, { 211,   0 },
 { 212,   0 }, { 213,   0 }, { 214,   0 }, { 215,   0 }, { 216,   0 },
 { 217,   0 }, { 218,   0 }, { 219,   0 }, { 220,   0 }, { 221,   0 },
 { 222,   0 }, { 223,   0 }, { 224,   0 }, { 225,   0 }, { 226,   0 },
 { 227,   0 }, { 228,   0 }, { 229,   0 }, { 230,   0 }, { 231,   0 },
 { 232,   0 }, { 233,   0 }, { 234,   0 }, { 235,   0 }, { 236,   0 },
 { 237,   0 }, { 238,   0 }, { 239,   0 }, { 240,   0 }, { 241,   0 },
 { 242,   0 }, { 243,   0 }, { 244,   0 }, { 245,   0 }, { 246,   0 },
 { 247,   0 }, { 248,   0 }, { 249,   0 }, { 250,   0 }, { 251,   0 },
 { 252,   0 }, { 253,   0 }, { 254,   0 }, { 255,   0 }, { 256,   0 },

 {   0,   3 }, {   0,22774 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  33,3806 },
 {   0,   0 }, {  35,3806 }, {   0,   0 }, {  37,3806 }, {  38,3806 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  42,3806 }, {  43,3806 },
 {   0,   0 }, {  45,3806 }, {   0,   0 }, {  47,3806 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  60,3806 }, {  61,3806 }, {  62,3806 }, {  63,3806 },
 {  64,3806 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  94,3806 }, {   0,   0 }, {  96,3806 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 { 124,3806 }, {   0,   0 }, { 126,3806 }, {   0,  58 }, {   0,22646 },
 {   1,   0 }, {   2,   0 }, {   3,   0 }, {   4,   0 }, {   5,   0 },
 {   6,   0 }, {   7,   0 }, {   8,   0 }, {   9,   0 }, {  10,   0 },
 {  11,   0 }, {  12,   0 }, {  13,   0 }, {  14,   0 }, {  15,   0 },
 {  16,   0 }, {  17,   0 }, {  18,   0 }, {  19,   0 }, {  20,   0 },

 {  21,   0 }, {  22,   0 }, {  23,   0 }, {  24,   0 }, {  25,   0 },
 {  26,   0 }, {  27,   0 }, {  28,   0 }, {  29,   0 }, {  30,   0 },
 {  31,   0 }, {  32,   0 }, {  33,   0 }, {   0,   0 }, {  35,   0 },
 {  36,   0 }, {  37,   0 }, {  38,   0 }, {  39,   0 }, {  40,   0 },
 {  41,   0 }, {  42,   0 }, {  43,   0 }, {  44,   0 }, {  45,   0 },
 {  46,   0 }, {  47,   0 }, {  48,   0 }, {  49,   0 }, {  50,   0 },
 {  51,   0 }, {  52,   0 }, {  53,   0 }, {  54,   0 }, {  55,   0 },
 {  56,   0 }, {  57,   0 }, {  58,   0 }, {  59,   0 }, {  60,   0 },
 {  61,   0 }, {  62,   0 }, {  63,   0 }, {  64,   0 }, {  65,   0 },
 {  66,   0 }, {  67,   0 }, {  68,   0 }, {  69,   0 }, {  70,   0 },

 {  71,   0 }, {  72,   0 }, {  73,   0 }, {  74,   0 }, {  75,   0 },
 {  76,   0 }, {  77,   0 }, {  78,   0 }, {  79,   0 }, {  80,   0 },
 {  81,   0 }, {  82,   0 }, {  83,   0 }, {  84,   0 }, {  85,   0 },
 {  86,   0 }, {  87,   0 }, {  88,   0 }, {  89,   0 }, {  90,   0 },
 {  91,   0 }, {  92,   0 }, {  93,   0 }, {  94,   0 }, {  95,   0 },
 {  96,   0 }, {  97,   0 }, {  98,   0 }, {  99,   0 }, { 100,   0 },
 { 101,   0 }, { 102,   0 }, { 103,   0 }, { 104,   0 }, { 105,   0 },
 { 106,   0 }, { 107,   0 }, { 108,   0 }, { 109,   0 }, { 110,   0 },
 { 111,   0 }, { 112,   0 }, { 113,   0 }, { 114,   0 }, { 115,   0 },
 { 116,   0 }, { 117,   0 }, { 118,   0 }, { 119,   0 }, { 120,   0 },

 { 121,   0 }, { 122,   0 }, { 123,   0 }, { 124,   0 }, { 125,   0 },
 { 126,   0 }, { 127,   0 }, { 128,   0 }, { 129,   0 }, { 130,   0 },
 { 131,   0 }, { 132,   0 }, { 133,   0 }, { 134,   0 }, { 135,   0 },
 { 136,   0 }, { 137,   0 }, { 138,   0 }, { 139,   0 }, { 140,   0 },
 { 141,   0 }, { 142,   0 }, { 143,   0 }, { 144,   0 }, { 145,   0 },
 { 146,   0 }, { 147,   0 }, { 148,   0 }, { 149,   0 }, { 150,   0 },
 { 151,   0 }, { 152,   0 }, { 153,   0 }, { 154,   0 }, { 155,   0 },
 { 156,   0 }, { 157,   0 }, { 158,   0 }, { 159,   0 }, { 160,   0 },
 { 161,   0 }, { 162,   0 }, { 163,   0 }, { 164,   0 }, { 165,   0 },
 { 166,   0 }, { 167,   0 }, { 168,   0 }, { 169,   0 }, { 170,   0 },

 { 171,   0 }, { 172,   0 }, { 173,   0 }, { 174,   0 }, { 175,   0 },
 { 176,   0 }, { 177,   0 }, { 178,   0 }, { 179,   0 }, { 180,   0 },
 { 181,   0 }, { 182,   0 }, { 183,   0 }, { 184,   0 }, { 185,   0 },
 { 186,   0 }, { 187,   0 }, { 188,   0 }, { 189,   0 }, { 190,   0 },
 { 191,   0 }, { 192,   0 }, { 193,   0 }, { 194,   0 }, { 195,   0 },
 { 196,   0 }, { 197,   0 }, { 198,   0 }, { 199,   0 }, { 200,   0 },
 { 201,   0 }, { 202,   0 }, { 203,   0 }, { 204,   0 }, { 205,   0 },
 { 206,   0 }, { 207,   0 }, { 208,   0 }, { 209,   0 }, { 210,   0 },
 { 211,   0 }, { 212,   0 }, { 213,   0 }, { 214,   0 }, { 215,   0 },
 { 216,   0 }, { 217,   0 }, { 218,   0 }, { 219,   0 }, { 220,   0 },

 { 221,   0 }, { 222,   0 }, { 223,   0 }, { 224,   0 }, { 225,   0 },
 { 226,   0 }, { 227,   0 }, { 228,   0 }, { 229,   0 }, { 230,   0 },
 { 231,   0 }, { 232,   0 }, { 233,   0 }, { 234,   0 }, { 235,   0 },
 { 236,   0 }, { 237,   0 }, { 238,   0 }, { 239,   0 }, { 240,   0 },
 { 241,   0 }, { 242,   0 }, { 243,   0 }, { 244,   0 }, { 245,   0 },
 { 246,   0 }, { 247,   0 }, { 248,   0 }, { 249,   0 }, { 250,   0 },
 { 251,   0 }, { 252,   0 }, { 253,   0 }, { 254,   0 }, { 255,   0 },
 { 256,   0 }, {   0,  11 }, {   0,22388 }, {   1,   0 }, {   2,   0 },
 {   3,   0 }, {   4,   0 }, {   5,   0 }, {   6,   0 }, {   7,   0 },
 {   8,   0 }, {   9,   0 }, {  10,   0 }, {  11,   0 }, {  12,   0 },

 {  13,   0 }, {  14,   0 }, {  15,   0 }, {  16,   0 }, {  17,   0 },
 {  18,   0 }, {  19,   0 }, {  20,   0 }, {  21,   0 }, {  22,   0 },
 {  23,   0 }, {  24,   0 }, {  25,   0 }, {  26,   0 }, {  27,   0 },
 {  28,   0 }, {  29,   0 }, {  30,   0 }, {  31,   0 }, {  32,   0 },
 {  33,   0 }, {  34,   0 }, {  35,   0 }, {  36,   0 }, {  37,   0 },
 {  38,   0 }, {   0,   0 }, {  40,   0 }, {  41,   0 }, {  42,   0 },
 {  43,   0 }, {  44,   0 }, {  45,   0 }, {  46,   0 }, {  47,   0 },
 {  48,   0 }, {  49,   0 }, {  50,   0 }, {  51,   0 }, {  52,   0 },
 {  53,   0 }, {  54,   0 }, {  55,   0 }, {  56,   0 }, {  57,   0 },
 {  58,   0 }, {  59,   0 }, {  60,   0 }, {  61,   0 }, {  62,   0 },

 {  63,   0 }, {  64,   0 }, {  65,   0 }, {  66,   0 }, {  67,   0 },
 {  68,   0 }, {  69,   0 }, {  70,   0 }, {  71,   0 }, {  72,   0 },
 {  73,   0 }, {  74,   0 }, {  75,   0 }, {  76,   0 }, {  77,   0 },
 {  78,   0 }, {  79,   0 }, {  80,   0 }, {  81,   0 }, {  82,   0 },
 {  83,   0 }, {  84,   0 }, {  85,   0 }, {  86,   0 }, {  87,   0 },
 {  88,   0 }, {  89,   0 }, {  90,   0 }, {  91,   0 }, {  92,   0 },
 {  93,   0 }, {  94,   0 }, {  95,   0 }, {  96,   0 }, {  97,   0 },
 {  98,   0 }, {  99,   0 }, { 100,   0 }, { 101,   0 }, { 102,   0 },
 { 103,   0 }, { 104,   0 }, { 105,   0 }, { 106,   0 }, { 107,   0 },
 { 108,   0 }, { 109,   0 }, { 110,   0 }, { 111,   0 }, { 112,   0 },

 { 113,   0 }, { 114,   0 }, { 115,   0 }, { 116,   0 }, { 117,   0 },
 { 118,   0 }, { 119,   0 }, { 120,   0 }, { 121,   0 }, { 122,   0 },
 { 123,   0 }, { 124,   0 }, { 125,   0 }, { 126,   0 }, { 127,   0 },
 { 128,   0 }, { 129,   0 }, { 130,   0 }, { 131,   0 }, { 132,   0 },
 { 133,   0 }, { 134,   0 }, { 135,   0 }, { 136,   0 }, { 137,   0 },
 { 138,   0 }, { 139,   0 }, { 140,   0 }, { 141,   0 }, { 142,   0 },
 { 143,   0 }, { 144,   0 }, { 145,   0 }, { 146,   0 }, { 147,   0 },
 { 148,   0 }, { 149,   0 }, { 150,   0 }, { 151,   0 }, { 152,   0 },
 { 153,   0 }, { 154,   0 }, { 155,   0 }, { 156,   0 }, { 157,   0 },
 { 158,   0 }, { 159,   0 }, { 160,   0 }, { 161,   0 }, { 162,   0 },

 { 163,   0 }, { 164,   0 }, { 165,   0 }, { 166,   0 }, { 167,   0 },
 { 168,   0 }, { 169,   0 }, { 170,   0 }, { 171,   0 }, { 172,   0 },
 { 173,   0 }, { 174,   0 }, { 175,   0 }, { 176,   0 }, { 177,   0 },
 { 178,   0 }, { 179,   0 }, { 180,   0 }, { 181,   0 }, { 182,   0 },
 { 183,   0 }, { 184,   0 }, { 185,   0 }, { 186,   0 }, { 187,   0 },
 { 188,   0 }, { 189,   0 }, { 190,   0 }, { 191,   0 }, { 192,   0 },
 { 193,   0 }, { 194,   0 }, { 195,   0 }, { 196,   0 }, { 197,   0 },
 { 198,   0 }, { 199,   0 }, { 200,   0 }, { 201,   0 }, { 202,   0 },
 { 203,   0 }, { 204,   0 }, { 205,   0 }, { 206,   0 }, { 207,   0 },
 { 208,   0 }, { 209,   0 }, { 210,   0 }, { 211,   0 }, { 212,   0 },

 { 213,   0 }, { 214,   0 }, { 215,   0 }, { 216,   0 }, { 217,   0 },
 { 218,   0 }, { 219,   0 }, { 220,   0 }, { 221,   0 }, { 222,   0 },
 { 223,   0 }, { 224,   0 }, { 225,   0 }, { 226,   0 }, { 227,   0 },
 { 228,   0 }, { 229,   0 }, { 230,   0 }, { 231,   0 }, { 232,   0 },
 { 233,   0 }, { 234,   0 }, { 235,   0 }, { 236,   0 }, { 237,   0 },
 { 238,   0 }, { 239,   0 }, { 240,   0 }, { 241,   0 }, { 242,   0 },
 { 243,   0 }, { 244,   0 }, { 245,   0 }, { 246,   0 }, { 247,   0 },
 { 248,   0 }, { 249,   0 }, { 250,   0 }, { 251,   0 }, { 252,   0 },
 { 253,   0 }, { 254,   0 }, { 255,   0 }, { 256,   0 }, {   0,  16 },
 {   0,22130 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,  16 },

 {   0,22125 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   9,   0 },
 {  10,   5 }, {   0,   0 }, {  12,   0 }, {  13,   5 }, {   9,3164 },
 {  10,3164 }, {   0,   0 }, {  12,3164 }, {  13,3164 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  32,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  32,3164 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  39,-7744 },
 {  45,-7842 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  45,-7739 }, {   0,  32 }, {   0,22078 }, {   1,   0 }, {   2,   0 },

 {   3,   0 }, {   4,   0 }, {   5,   0 }, {   6,   0 }, {   7,   0 },
 {   8,   0 }, {   9,   0 }, {  10,   0 }, {  11,   0 }, {  12,   0 },
 {  13,   0 }, {  14,   0 }, {  15,   0 }, {  16,   0 }, {  17,   0 },
 {  18,   0 }, {  19,   0 }, {  20,   0 }, {  21,   0 }, {  22,   0 },
 {  23,   0 }, {  24,   0 }, {  25,   0 }, {  26,   0 }, {  27,   0 },
 {  28,   0 }, {  29,   0 }, {  30,   0 }, {  31,   0 }, {  32,   0 },
 {  33,   0 }, {  34,   0 }, {  35,   0 }, {  36,   0 }, {  37,   0 },
 {  38,   0 }, {   0,   0 }, {  40,   0 }, {  41,   0 }, {  42,   0 },
 {  43,   0 }, {  44,   0 }, {  45,   0 }, {  46,   0 }, {  47,   0 },
 {  48,   0 }, {  49,   0 }, {  50,   0 }, {  51,   0 }, {  52,   0 },

 {  53,   0 }, {  54,   0 }, {  55,   0 }, {  56,   0 }, {  57,   0 },
 {  58,   0 }, {  59,   0 }, {  60,   0 }, {  61,   0 }, {  62,   0 },
 {  63,   0 }, {  64,   0 }, {  65,   0 }, {  66,   0 }, {  67,   0 },
 {  68,   0 }, {  69,   0 }, {  70,   0 }, {  71,   0 }, {  72,   0 },
 {  73,   0 }, {  74,   0 }, {  75,   0 }, {  76,   0 }, {  77,   0 },
 {  78,   0 }, {  79,   0 }, {  80,   0 }, {  81,   0 }, {  82,   0 },
 {  83,   0 }, {  84,   0 }, {  85,   0 }, {  86,   0 }, {  87,   0 },
 {  88,   0 }, {  89,   0 }, {  90,   0 }, {  91,   0 }, {   0,   0 },
 {  93,   0 }, {  94,   0 }, {  95,   0 }, {  96,   0 }, {  97,   0 },
 {  98,   0 }, {  99,   0 }, { 100,   0 }, { 101,   0 }, { 102,   0 },

 { 103,   0 }, { 104,   0 }, { 105,   0 }, { 106,   0 }, { 107,   0 },
 { 108,   0 }, { 109,   0 }, { 110,   0 }, { 111,   0 }, { 112,   0 },
 { 113,   0 }, { 114,   0 }, { 115,   0 }, { 116,   0 }, { 117,   0 },
 { 118,   0 }, { 119,   0 }, { 120,   0 }, { 121,   0 }, { 122,   0 },
 { 123,   0 }, { 124,   0 }, { 125,   0 }, { 126,   0 }, { 127,   0 },
 { 128,   0 }, { 129,   0 }, { 130,   0 }, { 131,   0 }, { 132,   0 },
 { 133,   0 }, { 134,   0 }, { 135,   0 }, { 136,   0 }, { 137,   0 },
 { 138,   0 }, { 139,   0 }, { 140,   0 }, { 141,   0 }, { 142,   0 },
 { 143,   0 }, { 144,   0 }, { 145,   0 }, { 146,   0 }, { 147,   0 },
 { 148,   0 }, { 149,   0 }, { 150,   0 }, { 151,   0 }, { 152,   0 },

 { 153,   0 }, { 154,   0 }, { 155,   0 }, { 156,   0 }, { 157,   0 },
 { 158,   0 }, { 159,   0 }, { 160,   0 }, { 161,   0 }, { 162,   0 },
 { 163,   0 }, { 164,   0 }, { 165,   0 }, { 166,   0 }, { 167,   0 },
 { 168,   0 }, { 169,   0 }, { 170,   0 }, { 171,   0 }, { 172,   0 },
 { 173,   0 }, { 174,   0 }, { 175,   0 }, { 176,   0 }, { 177,   0 },
 { 178,   0 }, { 179,   0 }, { 180,   0 }, { 181,   0 }, { 182,   0 },
 { 183,   0 }, { 184,   0 }, { 185,   0 }, { 186,   0 }, { 187,   0 },
 { 188,   0 }, { 189,   0 }, { 190,   0 }, { 191,   0 }, { 192,   0 },
 { 193,   0 }, { 194,   0 }, { 195,   0 }, { 196,   0 }, { 197,   0 },
 { 198,   0 }, { 199,   0 }, { 200,   0 }, { 201,   0 }, { 202,   0 },

 { 203,   0 }, { 204,   0 }, { 205,   0 }, { 206,   0 }, { 207,   0 },
 { 208,   0 }, { 209,   0 }, { 210,   0 }, { 211,   0 }, { 212,   0 },
 { 213,   0 }, { 214,   0 }, { 215,   0 }, { 216,   0 }, { 217,   0 },
 { 218,   0 }, { 219,   0 }, { 220,   0 }, { 221,   0 }, { 222,   0 },
 { 223,   0 }, { 224,   0 }, { 225,   0 }, { 226,   0 }, { 227,   0 },
 { 228,   0 }, { 229,   0 }, { 230,   0 }, { 231,   0 }, { 232,   0 },
 { 233,   0 }, { 234,   0 }, { 235,   0 }, { 236,   0 }, { 237,   0 },
 { 238,   0 }, { 239,   0 }, { 240,   0 }, { 241,   0 }, { 242,   0 },
 { 243,   0 }, { 244,   0 }, { 245,   0 }, { 246,   0 }, { 247,   0 },
 { 248,   0 }, { 249,   0 }, { 250,   0 }, { 251,   0 }, { 252,   0 },

 { 253,   0 }, { 254,   0 }, { 255,   0 }, { 256,   0 }, {   0,  22 },
 {   0,21820 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,  22 },
 {   0,21815 }, {   0,  39 }, {   0,21813 }, {   0,   0 }, {   9,   0 },
 {  10,   5 }, {   0,   0 }, {  12,   0 }, {  13,   5 }, {   9,3233 },
 {  10,3233 }, {   0,   0 }, {  12,3233 }, {  13,3233 }, {   0,  37 },
 {   0,21800 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  32,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  32,3233 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,  37 }, {   0,21777 }, {  39,-8047 },

 {  45,-8148 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  45,-8026 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  48,3536 }, {  49,3536 }, {  50,3536 }, {  51,3536 }, {  52,3536 },
 {  53,3536 }, {  54,3536 }, {  55,3536 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  48,3531 }, {  49,3531 },
 {  50,3531 }, {  51,3531 }, {  52,3531 }, {  53,3531 }, {  54,3531 },
 {  55,3531 }, {  56,3531 }, {  57,3531 }, {   0,   0 }, {   0,   0 },
 {   0,  38 }, {   0,21739 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  65,3531 }, {  66,3531 }, {  67,3531 }, {  68,3531 }, {  69,3531 },
 {  70,3531 }, {  48,3531 }, {  49,3531 }, {  50,3531 }, {  51,3531 },

 {  52,3531 }, {  53,3531 }, {  54,3531 }, {  55,3531 }, {  56,3531 },
 {  57,3531 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65,3531 }, {  66,3531 },
 {  67,3531 }, {  68,3531 }, {  69,3531 }, {  70,3531 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  97,3531 }, {  98,3531 }, {  99,3531 },
 { 100,3531 }, { 101,3531 }, { 102,3531 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  48,3531 },
 {  49,3531 }, {  50,3531 }, {  51,3531 }, {  52,3531 }, {  53,3531 },
 {  54,3531 }, {  55,3531 }, {  56,3531 }, {  57,3531 }, {   0,   0 },
 {  97,3531 }, {  98,3531 }, {  99,3531 }, { 100,3531 }, { 101,3531 },

 { 102,3531 }, {  65,3531 }, {  66,3531 }, {  67,3531 }, {  68,3531 },
 {  69,3531 }, {  70,3531 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  97,3531 }, {  98,3531 },
 {  99,3531 }, { 100,3531 }, { 101,3531 }, { 102,3531 }, {   0,  31 },
 {   0,21635 }, {   1,   0 }, {   2,   0 }, {   3,   0 }, {   4,   0 },
 {   5,   0 }, {   6,   0 }, {   7,   0 }, {   8,   0 }, {   9,   0 },

 {  10,   0 }, {  11,   0 }, {  12,   0 }, {  13,   0 }, {  14,   0 },
 {  15,   0 }, {  16,   0 }, {  17,   0 }, {  18,   0 }, {  19,   0 },
 {  20,   0 }, {  21,   0 }, {  22,   0 }, {  23,   0 }, {  24,   0 },
 {  25,   0 }, {  26,   0 }, {  27,   0 }, {  28,   0 }, {  29,   0 },
 {  30,   0 }, {  31,   0 }, {  32,   0 }, {  33,   0 }, {  34,   0 },
 {  35,   0 }, {  36,   0 }, {  37,   0 }, {  38,   0 }, {   0,   0 },
 {  40,   0 }, {  41,   0 }, {  42,   0 }, {  43,   0 }, {  44,   0 },
 {  45,   0 }, {  46,   0 }, {  47,   0 }, {  48,   0 }, {  49,   0 },
 {  50,   0 }, {  51,   0 }, {  52,   0 }, {  53,   0 }, {  54,   0 },
 {  55,   0 }, {  56,   0 }, {  57,   0 }, {  58,   0 }, {  59,   0 },

 {  60,   0 }, {  61,   0 }, {  62,   0 }, {  63,   0 }, {  64,   0 },
 {  65,   0 }, {  66,   0 }, {  67,   0 }, {  68,   0 }, {  69,   0 },
 {  70,   0 }, {  71,   0 }, {  72,   0 }, {  73,   0 }, {  74,   0 },
 {  75,   0 }, {  76,   0 }, {  77,   0 }, {  78,   0 }, {  79,   0 },
 {  80,   0 }, {  81,   0 }, {  82,   0 }, {  83,   0 }, {  84,   0 },
 {  85,   0 }, {  86,   0 }, {  87,   0 }, {  88,   0 }, {  89,   0 },
 {  90,   0 }, {  91,   0 }, {  92,   0 }, {  93,   0 }, {  94,   0 },
 {  95,   0 }, {  96,   0 }, {  97,   0 }, {  98,   0 }, {  99,   0 },
 { 100,   0 }, { 101,   0 }, { 102,   0 }, { 103,   0 }, { 104,   0 },
 { 105,   0 }, { 106,   0 }, { 107,   0 }, { 108,   0 }, { 109,   0 },

 { 110,   0 }, { 111,   0 }, { 112,   0 }, { 113,   0 }, { 114,   0 },
 { 115,   0 }, { 116,   0 }, { 117,   0 }, { 118,   0 }, { 119,   0 },
 { 120,   0 }, { 121,   0 }, { 122,   0 }, { 123,   0 }, { 124,   0 },
 { 125,   0 }, { 126,   0 }, { 127,   0 }, { 128,   0 }, { 129,   0 },
 { 130,   0 }, { 131,   0 }, { 132,   0 }, { 133,   0 }, { 134,   0 },
 { 135,   0 }, { 136,   0 }, { 137,   0 }, { 138,   0 }, { 139,   0 },
 { 140,   0 }, { 141,   0 }, { 142,   0 }, { 143,   0 }, { 144,   0 },
 { 145,   0 }, { 146,   0 }, { 147,   0 }, { 148,   0 }, { 149,   0 },
 { 150,   0 }, { 151,   0 }, { 152,   0 }, { 153,   0 }, { 154,   0 },
 { 155,   0 }, { 156,   0 }, { 157,   0 }, { 158,   0 }, { 159,   0 },

 { 160,   0 }, { 161,   0 }, { 162,   0 }, { 163,   0 }, { 164,   0 },
 { 165,   0 }, { 166,   0 }, { 167,   0 }, { 168,   0 }, { 169,   0 },
 { 170,   0 }, { 171,   0 }, { 172,   0 }, { 173,   0 }, { 174,   0 },
 { 175,   0 }, { 176,   0 }, { 177,   0 }, { 178,   0 }, { 179,   0 },
 { 180,   0 }, { 181,   0 }, { 182,   0 }, { 183,   0 }, { 184,   0 },
 { 185,   0 }, { 186,   0 }, { 187,   0 }, { 188,   0 }, { 189,   0 },
 { 190,   0 }, { 191,   0 }, { 192,   0 }, { 193,   0 }, { 194,   0 },
 { 195,   0 }, { 196,   0 }, { 197,   0 }, { 198,   0 }, { 199,   0 },
 { 200,   0 }, { 201,   0 }, { 202,   0 }, { 203,   0 }, { 204,   0 },
 { 205,   0 }, { 206,   0 }, { 207,   0 }, { 208,   0 }, { 209,   0 },

 { 210,   0 }, { 211,   0 }, { 212,   0 }, { 213,   0 }, { 214,   0 },
 { 215,   0 }, { 216,   0 }, { 217,   0 }, { 218,   0 }, { 219,   0 },
 { 220,   0 }, { 221,   0 }, { 222,   0 }, { 223,   0 }, { 224,   0 },
 { 225,   0 }, { 226,   0 }, { 227,   0 }, { 228,   0 }, { 229,   0 },
 { 230,   0 }, { 231,   0 }, { 232,   0 }, { 233,   0 }, { 234,   0 },
 { 235,   0 }, { 236,   0 }, { 237,   0 }, { 238,   0 }, { 239,   0 },
 { 240,   0 }, { 241,   0 }, { 242,   0 }, { 243,   0 }, { 244,   0 },
 { 245,   0 }, { 246,   0 }, { 247,   0 }, { 248,   0 }, { 249,   0 },
 { 250,   0 }, { 251,   0 }, { 252,   0 }, { 253,   0 }, { 254,   0 },
 { 255,   0 }, { 256,   0 }, {   0,  46 }, {   0,21377 }, {   1,   0 },

 {   2,   0 }, {   3,   0 }, {   4,   0 }, {   5,   0 }, {   6,   0 },
 {   7,   0 }, {   8,   0 }, {   9,   0 }, {  10,   0 }, {  11,   0 },
 {  12,   0 }, {  13,   0 }, {  14,   0 }, {  15,   0 }, {  16,   0 },
 {  17,   0 }, {  18,   0 }, {  19,   0 }, {  20,   0 }, {  21,   0 },
 {  22,   0 }, {  23,   0 }, {  24,   0 }, {  25,   0 }, {  26,   0 },
 {  27,   0 }, {  28,   0 }, {  29,   0 }, {  30,   0 }, {  31,   0 },
 {  32,   0 }, {  33,   0 }, {  34,   0 }, {  35,   0 }, {   0,   0 },
 {  37,   0 }, {  38,   0 }, {  39,   0 }, {  40,   0 }, {  41,   0 },
 {  42,   0 }, {  43,   0 }, {  44,   0 }, {  45,   0 }, {  46,   0 },
 {  47,   0 }, {  48,   0 }, {  49,   0 }, {  50,   0 }, {  51,   0 },

 {  52,   0 }, {  53,   0 }, {  54,   0 }, {  55,   0 }, {  56,   0 },
 {  57,   0 }, {  58,   0 }, {  59,   0 }, {  60,   0 }, {  61,   0 },
 {  62,   0 }, {  63,   0 }, {  64,   0 }, {  65,   0 }, {  66,   0 },
 {  67,   0 }, {  68,   0 }, {  69,   0 }, {  70,   0 }, {  71,   0 },
 {  72,   0 }, {  73,   0 }, {  74,   0 }, {  75,   0 }, {  76,   0 },
 {  77,   0 }, {  78,   0 }, {  79,   0 }, {  80,   0 }, {  81,   0 },
 {  82,   0 }, {  83,   0 }, {  84,   0 }, {  85,   0 }, {  86,   0 },
 {  87,   0 }, {  88,   0 }, {  89,   0 }, {  90,   0 }, {  91,   0 },
 {  92,   0 }, {  93,   0 }, {  94,   0 }, {  95,   0 }, {  96,   0 },
 {  97,   0 }, {  98,   0 }, {  99,   0 }, { 100,   0 }, { 101,   0 },

 { 102,   0 }, { 103,   0 }, { 104,   0 }, { 105,   0 }, { 106,   0 },
 { 107,   0 }, { 108,   0 }, { 109,   0 }, { 110,   0 }, { 111,   0 },
 { 112,   0 }, { 113,   0 }, { 114,   0 }, { 115,   0 }, { 116,   0 },
 { 117,   0 }, { 118,   0 }, { 119,   0 }, { 120,   0 }, { 121,   0 },
 { 122,   0 }, { 123,   0 }, { 124,   0 }, { 125,   0 }, { 126,   0 },
 { 127,   0 }, { 128,   0 }, { 129,   0 }, { 130,   0 }, { 131,   0 },
 { 132,   0 }, { 133,   0 }, { 134,   0 }, { 135,   0 }, { 136,   0 },
 { 137,   0 }, { 138,   0 }, { 139,   0 }, { 140,   0 }, { 141,   0 },
 { 142,   0 }, { 143,   0 }, { 144,   0 }, { 145,   0 }, { 146,   0 },
 { 147,   0 }, { 148,   0 }, { 149,   0 }, { 150,   0 }, { 151,   0 },

 { 152,   0 }, { 153,   0 }, { 154,   0 }, { 155,   0 }, { 156,   0 },
 { 157,   0 }, { 158,   0 }, { 159,   0 }, { 160,   0 }, { 161,   0 },
 { 162,   0 }, { 163,   0 }, { 164,   0 }, { 165,   0 }, { 166,   0 },
 { 167,   0 }, { 168,   0 }, { 169,   0 }, { 170,   0 }, { 171,   0 },
 { 172,   0 }, { 173,   0 }, { 174,   0 }, { 175,   0 }, { 176,   0 },
 { 177,   0 }, { 178,   0 }, { 179,   0 }, { 180,   0 }, { 181,   0 },
 { 182,   0 }, { 183,   0 }, { 184,   0 }, { 185,   0 }, { 186,   0 },
 { 187,   0 }, { 188,   0 }, { 189,   0 }, { 190,   0 }, { 191,   0 },
 { 192,   0 }, { 193,   0 }, { 194,   0 }, { 195,   0 }, { 196,   0 },
 { 197,   0 }, { 198,   0 }, { 199,   0 }, { 200,   0 }, { 201,   0 },

 { 202,   0 }, { 203,   0 }, { 204,   0 }, { 205,   0 }, { 206,   0 },
 { 207,   0 }, { 208,   0 }, { 209,   0 }, { 210,   0 }, { 211,   0 },
 { 212,   0 }, { 213,   0 }, { 214,   0 }, { 215,   0 }, { 216,   0 },
 { 217,   0 }, { 218,   0 }, { 219,   0 }, { 220,   0 }, { 221,   0 },
 { 222,   0 }, { 223,   0 }, { 224,   0 }, { 225,   0 }, { 226,   0 },
 { 227,   0 }, { 228,   0 }, { 229,   0 }, { 230,   0 }, { 231,   0 },
 { 232,   0 }, { 233,   0 }, { 234,   0 }, { 235,   0 }, { 236,   0 },
 { 237,   0 }, { 238,   0 }, { 239,   0 }, { 240,   0 }, { 241,   0 },
 { 242,   0 }, { 243,   0 }, { 244,   0 }, { 245,   0 }, { 246,   0 },
 { 247,   0 }, { 248,   0 }, { 249,   0 }, { 250,   0 }, { 251,   0 },

 { 252,   0 }, { 253,   0 }, { 254,   0 }, { 255,   0 }, { 256,   0 },
 {   0,  47 }, {   0,21119 }, {   0,  53 }, {   0,21117 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   9,   0 }, {  10,   0 }, {   0,   0 },
 {  12,   0 }, {  13,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  32,   0 }, {   0,   0 }, {  36,-8845 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  48,2983 },
 {  49,2983 }, {  50,2983 }, {  51,2983 }, {  52,2983 }, {  53,2983 },
 {  54,2983 }, {  55,2983 }, {  56,2983 }, {  57,2983 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  65,2983 }, {  66,2983 }, {  67,2983 }, {  68,2983 },
 {  69,2983 }, {  70,2983 }, {  71,2983 }, {  72,2983 }, {  73,2983 },
 {  74,2983 }, {  75,2983 }, {  76,2983 }, {  77,2983 }, {  78,2983 },
 {  79,2983 }, {  80,2983 }, {  81,2983 }, {  82,2983 }, {  83,2983 },
 {  84,2983 }, {  85,2983 }, {  86,2983 }, {  87,2983 }, {  88,2983 },
 {  89,2983 }, {  90,2983 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {  95,2983 }, {   0,   0 }, {  97,2983 }, {  98,2983 },
 {  99,2983 }, { 100,2983 }, { 101,2983 }, { 102,2983 }, { 103,2983 },
 { 104,2983 }, { 105,2983 }, { 106,2983 }, { 107,2983 }, { 108,2983 },
 { 109,2983 }, { 110,2983 }, { 111,2983 }, { 112,2983 }, { 113,2983 },
 { 114,2983 }, { 115,2983 }, { 116,2983 }, { 117,2983 }, { 118,2983 },
 { 119,2983 }, { 120,2983 }, { 121,2983 }, { 122,2983 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, { 128,2983 },
 { 129,2983 }, { 130,2983 }, { 131,2983 }, { 132,2983 }, { 133,2983 },
 { 134,2983 }, { 135,2983 }, { 136,2983 }, { 137,2983 }, { 138,2983 },
 { 139,2983 }, { 140,2983 }, { 141,2983 }, { 142,2983 }, { 143,2983 },

 { 144,2983 }, { 145,2983 }, { 146,2983 }, { 147,2983 }, { 148,2983 },
 { 149,2983 }, { 150,2983 }, { 151,2983 }, { 152,2983 }, { 153,2983 },
 { 154,2983 }, { 155,2983 }, { 156,2983 }, { 157,2983 }, { 158,2983 },
 { 159,2983 }, { 160,2983 }, { 161,2983 }, { 162,2983 }, { 163,2983 },
 { 164,2983 }, { 165,2983 }, { 166,2983 }, { 167,2983 }, { 168,2983 },
 { 169,2983 }, { 170,2983 }, { 171,2983 }, { 172,2983 }, { 173,2983 },
 { 174,2983 }, { 175,2983 }, { 176,2983 }, { 177,2983 }, { 178,2983 },
 { 179,2983 }, { 180,2983 }, { 181,2983 }, { 182,2983 }, { 183,2983 },
 { 184,2983 }, { 185,2983 }, { 186,2983 }, { 187,2983 }, { 188,2983 },
 { 189,2983 }, { 190,2983 }, { 191,2983 }, { 192,2983 }, { 193,2983 },

 { 194,2983 }, { 195,2983 }, { 196,2983 }, { 197,2983 }, { 198,2983 },
 { 199,2983 }, { 200,2983 }, { 201,2983 }, { 202,2983 }, { 203,2983 },
 { 204,2983 }, { 205,2983 }, { 206,2983 }, { 207,2983 }, { 208,2983 },
 { 209,2983 }, { 210,2983 }, { 211,2983 }, { 212,2983 }, { 213,2983 },
 { 214,2983 }, { 215,2983 }, { 216,2983 }, { 217,2983 }, { 218,2983 },
 { 219,2983 }, { 220,2983 }, { 221,2983 }, { 222,2983 }, { 223,2983 },
 { 224,2983 }, { 225,2983 }, { 226,2983 }, { 227,2983 }, { 228,2983 },
 { 229,2983 }, { 230,2983 }, { 231,2983 }, { 232,2983 }, { 233,2983 },
 { 234,2983 }, { 235,2983 }, { 236,2983 }, { 237,2983 }, { 238,2983 },
 { 239,2983 }, { 240,2983 }, { 241,2983 }, { 242,2983 }, { 243,2983 },

 { 244,2983 }, { 245,2983 }, { 246,2983 }, { 247,2983 }, { 248,2983 },
 { 249,2983 }, { 250,2983 }, { 251,2983 }, { 252,2983 }, { 253,2983 },
 { 254,2983 }, { 255,2983 }, {   0,  53 }, {   0,20862 }, {   1,2983 },
 {   2,2983 }, {   3,2983 }, {   4,2983 }, {   5,2983 }, {   6,2983 },
 {   7,2983 }, {   8,2983 }, {   9,2983 }, {   0,   0 }, {  11,2983 },
 {  12,2983 }, {   0,   0 }, {  14,2983 }, {  15,2983 }, {  16,2983 },
 {  17,2983 }, {  18,2983 }, {  19,2983 }, {  20,2983 }, {  21,2983 },
 {  22,2983 }, {  23,2983 }, {  24,2983 }, {  25,2983 }, {  26,2983 },
 {  27,2983 }, {  28,2983 }, {  29,2983 }, {  30,2983 }, {  31,2983 },
 {  32,2983 }, {  33,2983 }, {  34,2983 }, {  35,2983 }, {  36,2983 },

 {  37,2983 }, {  38,2983 }, {  39,2983 }, {  40,2983 }, {  41,2983 },
 {  42,2983 }, {  43,2983 }, {  44,2983 }, {  45,2983 }, {  46,2983 },
 {  47,2983 }, {  48,2983 }, {  49,2983 }, {  50,2983 }, {  51,2983 },
 {  52,2983 }, {  53,2983 }, {  54,2983 }, {  55,2983 }, {  56,2983 },
 {  57,2983 }, {  58,2983 }, {  59,2983 }, {  60,2983 }, {  61,2983 },
 {  62,2983 }, {  63,2983 }, {  64,2983 }, {  65,2983 }, {  66,2983 },
 {  67,2983 }, {  68,2983 }, {  69,2983 }, {  70,2983 }, {  71,2983 },
 {  72,2983 }, {  73,2983 }, {  74,2983 }, {  75,2983 }, {  76,2983 },
 {  77,2983 }, {  78,2983 }, {  79,2983 }, {  80,2983 }, {  81,2983 },
 {  82,2983 }, {  83,2983 }, {  84,2983 }, {  85,2983 }, {  86,2983 },

 {  87,2983 }, {  88,2983 }, {  89,2983 }, {  90,2983 }, {  91,2983 },
 {  92,2983 }, {  93,2983 }, {  94,2983 }, {  95,2983 }, {  96,2983 },
 {  97,2983 }, {  98,2983 }, {  99,2983 }, { 100,2983 }, { 101,2983 },
 { 102,2983 }, { 103,2983 }, { 104,2983 }, { 105,2983 }, { 106,2983 },
 { 107,2983 }, { 108,2983 }, { 109,2983 }, { 110,2983 }, { 111,2983 },
 { 112,2983 }, { 113,2983 }, { 114,2983 }, { 115,2983 }, { 116,2983 },
 { 117,2983 }, { 118,2983 }, { 119,2983 }, { 120,2983 }, { 121,2983 },
 { 122,2983 }, { 123,2983 }, { 124,2983 }, { 125,2983 }, { 126,2983 },
 { 127,2983 }, { 128,2983 }, { 129,2983 }, { 130,2983 }, { 131,2983 },
 { 132,2983 }, { 133,2983 }, { 134,2983 }, { 135,2983 }, { 136,2983 },

 { 137,2983 }, { 138,2983 }, { 139,2983 }, { 140,2983 }, { 141,2983 },
 { 142,2983 }, { 143,2983 }, { 144,2983 }, { 145,2983 }, { 146,2983 },
 { 147,2983 }, { 148,2983 }, { 149,2983 }, { 150,2983 }, { 151,2983 },
 { 152,2983 }, { 153,2983 }, { 154,2983 }, { 155,2983 }, { 156,2983 },
 { 157,2983 }, { 158,2983 }, { 159,2983 }, { 160,2983 }, { 161,2983 },
 { 162,2983 }, { 163,2983 }, { 164,2983 }, { 165,2983 }, { 166,2983 },
 { 167,2983 }, { 168,2983 }, { 169,2983 }, { 170,2983 }, { 171,2983 },
 { 172,2983 }, { 173,2983 }, { 174,2983 }, { 175,2983 }, { 176,2983 },
 { 177,2983 }, { 178,2983 }, { 179,2983 }, { 180,2983 }, { 181,2983 },
 { 182,2983 }, { 183,2983 }, { 184,2983 }, { 185,2983 }, { 186,2983 },

 { 187,2983 }, { 188,2983 }, { 189,2983 }, { 190,2983 }, { 191,2983 },
 { 192,2983 }, { 193,2983 }, { 194,2983 }, { 195,2983 }, { 196,2983 },
 { 197,2983 }, { 198,2983 }, { 199,2983 }, { 200,2983 }, { 201,2983 },
 { 202,2983 }, { 203,2983 }, { 204,2983 }, { 205,2983 }, { 206,2983 },
 { 207,2983 }, { 208,2983 }, { 209,2983 }, { 210,2983 }, { 211,2983 },
 { 212,2983 }, { 213,2983 }, { 214,2983 }, { 215,2983 }, { 216,2983 },
 { 217,2983 }, { 218,2983 }, { 219,2983 }, { 220,2983 }, { 221,2983 },
 { 222,2983 }, { 223,2983 }, { 224,2983 }, { 225,2983 }, { 226,2983 },
 { 227,2983 }, { 228,2983 }, { 229,2983 }, { 230,2983 }, { 231,2983 },
 { 232,2983 }, { 233,2983 }, { 234,2983 }, { 235,2983 }, { 236,2983 },

 { 237,2983 }, { 238,2983 }, { 239,2983 }, { 240,2983 }, { 241,2983 },
 { 242,2983 }, { 243,2983 }, { 244,2983 }, { 245,2983 }, { 246,2983 },
 { 247,2983 }, { 248,2983 }, { 249,2983 }, { 250,2983 }, { 251,2983 },
 { 252,2983 }, { 253,2983 }, { 254,2983 }, { 255,2983 }, { 256,2983 },
 {   0,  24 }, {   0,20604 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,  24 }, {   0,20599 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   9,   0 }, {  10,   5 }, {   0,   0 }, {  12,   0 }, {  13,   5 },
 {   9,2978 }, {  10,2978 }, {   0,   0 }, {  12,2978 }, {  13,2978 },
 {   0,   0 }, {   0,  26 }, {   0,20583 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   9,   0 }, {  10,   0 }, {  32,   0 }, {  12,   0 },
 {  13,   0 }, {   0,   0 }, {   0,   0 }, {  32,2978 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  39,-9263 }, {  45,-9321 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  45,-9131 }, {   0,   0 }, {   0,   0 }, {  32,   0 },
 {   0,  26 }, {   0,20549 }, {   1,3233 }, {   2,3233 }, {   3,3233 },
 {   4,3233 }, {   5,3233 }, {   6,3233 }, {   7,3233 }, {   8,3233 },
 {   9,3233 }, {   0,   0 }, {  11,3233 }, {  12,3233 }, {   0,   0 },
 {  14,3233 }, {  15,3233 }, {  16,3233 }, {  17,3233 }, {  18,3233 },
 {  19,3233 }, {  20,3233 }, {  21,3233 }, {  22,3233 }, {  23,3233 },

 {  24,3233 }, {  25,3233 }, {  26,3233 }, {  27,3233 }, {  28,3233 },
 {  29,3233 }, {  30,3233 }, {  31,3233 }, {  32,3233 }, {  33,3233 },
 {  34,3233 }, {  35,3233 }, {  36,3233 }, {  37,3233 }, {  38,3233 },
 {  39,3233 }, {  40,3233 }, {  41,3233 }, {  42,3233 }, {  43,3233 },
 {  44,3233 }, {  45,3233 }, {  46,3233 }, {  47,3233 }, {  48,3233 },
 {  49,3233 }, {  50,3233 }, {  51,3233 }, {  52,3233 }, {  53,3233 },
 {  54,3233 }, {  55,3233 }, {  56,3233 }, {  57,3233 }, {  58,3233 },
 {  59,3233 }, {  60,3233 }, {  61,3233 }, {  62,3233 }, {  63,3233 },
 {  64,3233 }, {  65,3233 }, {  66,3233 }, {  67,3233 }, {  68,3233 },
 {  69,3233 }, {  70,3233 }, {  71,3233 }, {  72,3233 }, {  73,3233 },

 {  74,3233 }, {  75,3233 }, {  76,3233 }, {  77,3233 }, {  78,3233 },
 {  79,3233 }, {  80,3233 }, {  81,3233 }, {  82,3233 }, {  83,3233 },
 {  84,3233 }, {  85,3233 }, {  86,3233 }, {  87,3233 }, {  88,3233 },
 {  89,3233 }, {  90,3233 }, {  91,3233 }, {  92,3233 }, {  93,3233 },
 {  94,3233 }, {  95,3233 }, {  96,3233 }, {  97,3233 }, {  98,3233 },
 {  99,3233 }, { 100,3233 }, { 101,3233 }, { 102,3233 }, { 103,3233 },
 { 104,3233 }, { 105,3233 }, { 106,3233 }, { 107,3233 }, { 108,3233 },
 { 109,3233 }, { 110,3233 }, { 111,3233 }, { 112,3233 }, { 113,3233 },
 { 114,3233 }, { 115,3233 }, { 116,3233 }, { 117,3233 }, { 118,3233 },
 { 119,3233 }, { 120,3233 }, { 121,3233 }, { 122,3233 }, { 123,3233 },

 { 124,3233 }, { 125,3233 }, { 126,3233 }, { 127,3233 }, { 128,3233 },
 { 129,3233 }, { 130,3233 }, { 131,3233 }, { 132,3233 }, { 133,3233 },
 { 134,3233 }, { 135,3233 }, { 136,3233 }, { 137,3233 }, { 138,3233 },
 { 139,3233 }, { 140,3233 }, { 141,3233 }, { 142,3233 }, { 143,3233 },
 { 144,3233 }, { 145,3233 }, { 146,3233 }, { 147,3233 }, { 148,3233 },
 { 149,3233 }, { 150,3233 }, { 151,3233 }, { 152,3233 }, { 153,3233 },
 { 154,3233 }, { 155,3233 }, { 156,3233 }, { 157,3233 }, { 158,3233 },
 { 159,3233 }, { 160,3233 }, { 161,3233 }, { 162,3233 }, { 163,3233 },
 { 164,3233 }, { 165,3233 }, { 166,3233 }, { 167,3233 }, { 168,3233 },
 { 169,3233 }, { 170,3233 }, { 171,3233 }, { 172,3233 }, { 173,3233 },

 { 174,3233 }, { 175,3233 }, { 176,3233 }, { 177,3233 }, { 178,3233 },
 { 179,3233 }, { 180,3233 }, { 181,3233 }, { 182,3233 }, { 183,3233 },
 { 184,3233 }, { 185,3233 }, { 186,3233 }, { 187,3233 }, { 188,3233 },
 { 189,3233 }, { 190,3233 }, { 191,3233 }, { 192,3233 }, { 193,3233 },
 { 194,3233 }, { 195,3233 }, { 196,3233 }, { 197,3233 }, { 198,3233 },
 { 199,3233 }, { 200,3233 }, { 201,3233 }, { 202,3233 }, { 203,3233 },
 { 204,3233 }, { 205,3233 }, { 206,3233 }, { 207,3233 }, { 208,3233 },
 { 209,3233 }, { 210,3233 }, { 211,3233 }, { 212,3233 }, { 213,3233 },
 { 214,3233 }, { 215,3233 }, { 216,3233 }, { 217,3233 }, { 218,3233 },
 { 219,3233 }, { 220,3233 }, { 221,3233 }, { 222,3233 }, { 223,3233 },

 { 224,3233 }, { 225,3233 }, { 226,3233 }, { 227,3233 }, { 228,3233 },
 { 229,3233 }, { 230,3233 }, { 231,3233 }, { 232,3233 }, { 233,3233 },
 { 234,3233 }, { 235,3233 }, { 236,3233 }, { 237,3233 }, { 238,3233 },
 { 239,3233 }, { 240,3233 }, { 241,3233 }, { 242,3233 }, { 243,3233 },
 { 244,3233 }, { 245,3233 }, { 246,3233 }, { 247,3233 }, { 248,3233 },
 { 249,3233 }, { 250,3233 }, { 251,3233 }, { 252,3233 }, { 253,3233 },
 { 254,3233 }, { 255,3233 }, { 256,3233 }, {   0,  37 }, {   0,20291 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,  37 }, {   0,20268 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  48,3233 }, {  49,3233 }, {  50,3233 },
 {  51,3233 }, {  52,3233 }, {  53,3233 }, {  54,3233 }, {  55,3233 },
 {  56,3233 }, {  57,3233 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65,3233 },

 {  66,3233 }, {  67,3233 }, {  68,3233 }, {  69,3233 }, {  70,3233 },
 {  48,3233 }, {  49,3233 }, {  50,3233 }, {  51,3233 }, {  52,3233 },
 {  53,3233 }, {  54,3233 }, {  55,3233 }, {  56,3233 }, {  57,3233 },
 {   0,  44 }, {   0,20209 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  65,3233 }, {  66,3233 }, {  67,3233 },
 {  68,3233 }, {  69,3233 }, {  70,3233 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  97,3233 }, {  98,3233 }, {  99,3233 }, { 100,3233 },
 { 101,3233 }, { 102,3233 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {  36,-10032 }, {   0,   0 }, {  97,3233 },
 {  98,3233 }, {  99,3233 }, { 100,3233 }, { 101,3233 }, { 102,3233 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  48,   0 },
 {  49,   0 }, {  50,   0 }, {  51,   0 }, {  52,   0 }, {  53,   0 },
 {  54,   0 }, {  55,   0 }, {  56,   0 }, {  57,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  65,   0 }, {  66,   0 }, {  67,   0 }, {  68,   0 },
 {  69,   0 }, {  70,   0 }, {  71,   0 }, {  72,   0 }, {  73,   0 },
 {  74,   0 }, {  75,   0 }, {  76,   0 }, {  77,   0 }, {  78,   0 },
 {  79,   0 }, {  80,   0 }, {  81,   0 }, {  82,   0 }, {  83,   0 },

 {  84,   0 }, {  85,   0 }, {  86,   0 }, {  87,   0 }, {  88,   0 },
 {  89,   0 }, {  90,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  95,   0 }, {   0,   0 }, {  97,   0 }, {  98,   0 },
 {  99,   0 }, { 100,   0 }, { 101,   0 }, { 102,   0 }, { 103,   0 },
 { 104,   0 }, { 105,   0 }, { 106,   0 }, { 107,   0 }, { 108,   0 },
 { 109,   0 }, { 110,   0 }, { 111,   0 }, { 112,   0 }, { 113,   0 },
 { 114,   0 }, { 115,   0 }, { 116,   0 }, { 117,   0 }, { 118,   0 },
 { 119,   0 }, { 120,   0 }, { 121,   0 }, { 122,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, { 128,   0 },
 { 129,   0 }, { 130,   0 }, { 131,   0 }, { 132,   0 }, { 133,   0 },

 { 134,   0 }, { 135,   0 }, { 136,   0 }, { 137,   0 }, { 138,   0 },
 { 139,   0 }, { 140,   0 }, { 141,   0 }, { 142,   0 }, { 143,   0 },
 { 144,   0 }, { 145,   0 }, { 146,   0 }, { 147,   0 }, { 148,   0 },
 { 149,   0 }, { 150,   0 }, { 151,   0 }, { 152,   0 }, { 153,   0 },
 { 154,   0 }, { 155,   0 }, { 156,   0 }, { 157,   0 }, { 158,   0 },
 { 159,   0 }, { 160,   0 }, { 161,   0 }, { 162,   0 }, { 163,   0 },
 { 164,   0 }, { 165,   0 }, { 166,   0 }, { 167,   0 }, { 168,   0 },
 { 169,   0 }, { 170,   0 }, { 171,   0 }, { 172,   0 }, { 173,   0 },
 { 174,   0 }, { 175,   0 }, { 176,   0 }, { 177,   0 }, { 178,   0 },
 { 179,   0 }, { 180,   0 }, { 181,   0 }, { 182,   0 }, { 183,   0 },

 { 184,   0 }, { 185,   0 }, { 186,   0 }, { 187,   0 }, { 188,   0 },
 { 189,   0 }, { 190,   0 }, { 191,   0 }, { 192,   0 }, { 193,   0 },
 { 194,   0 }, { 195,   0 }, { 196,   0 }, { 197,   0 }, { 198,   0 },
 { 199,   0 }, { 200,   0 }, { 201,   0 }, { 202,   0 }, { 203,   0 },
 { 204,   0 }, { 205,   0 }, { 206,   0 }, { 207,   0 }, { 208,   0 },
 { 209,   0 }, { 210,   0 }, { 211,   0 }, { 212,   0 }, { 213,   0 },
 { 214,   0 }, { 215,   0 }, { 216,   0 }, { 217,   0 }, { 218,   0 },
 { 219,   0 }, { 220,   0 }, { 221,   0 }, { 222,   0 }, { 223,   0 },
 { 224,   0 }, { 225,   0 }, { 226,   0 }, { 227,   0 }, { 228,   0 },
 { 229,   0 }, { 230,   0 }, { 231,   0 }, { 232,   0 }, { 233,   0 },

 { 234,   0 }, { 235,   0 }, { 236,   0 }, { 237,   0 }, { 238,   0 },
 { 239,   0 }, { 240,   0 }, { 241,   0 }, { 242,   0 }, { 243,   0 },
 { 244,   0 }, { 245,   0 }, { 246,   0 }, { 247,   0 }, { 248,   0 },
 { 249,   0 }, { 250,   0 }, { 251,   0 }, { 252,   0 }, { 253,   0 },
 { 254,   0 }, { 255,   0 }, {   0,   1 }, {   0,19952 }, {   1,   0 },
 {   2,   0 }, {   3,   0 }, {   4,   0 }, {   5,   0 }, {   6,   0 },
 {   7,   0 }, {   8,   0 }, {   9,   0 }, {   0,   0 }, {  11,   0 },
 {  12,   0 }, {   0,   0 }, {  14,   0 }, {  15,   0 }, {  16,   0 },
 {  17,   0 }, {  18,   0 }, {  19,   0 }, {  20,   0 }, {  21,   0 },
 {  22,   0 }, {  23,   0 }, {  24,   0 }, {  25,   0 }, {  26,   0 },

 {  27,   0 }, {  28,   0 }, {  29,   0 }, {  30,   0 }, {  31,   0 },
 {  32,   0 }, {  33,   0 }, {  34,   0 }, {  35,   0 }, {  36,   0 },
 {  37,   0 }, {  38,   0 }, {  39,   0 }, {  40,   0 }, {  41,   0 },
 {  42,   0 }, {  43,   0 }, {  44,   0 }, {  45,   0 }, {  46,   0 },
 {  47,   0 }, {  48,   0 }, {  49,   0 }, {  50,   0 }, {  51,   0 },
 {  52,   0 }, {  53,   0 }, {  54,   0 }, {  55,   0 }, {  56,   0 },
 {  57,   0 }, {  58,   0 }, {  59,   0 }, {  60,   0 }, {  61,   0 },
 {  62,   0 }, {  63,   0 }, {  64,   0 }, {  65,   0 }, {  66,   0 },
 {  67,   0 }, {  68,   0 }, {  69,   0 }, {  70,   0 }, {  71,   0 },
 {  72,   0 }, {  73,   0 }, {  74,   0 }, {  75,   0 }, {  76,   0 },

 {  77,   0 }, {  78,   0 }, {  79,   0 }, {  80,   0 }, {  81,   0 },
 {  82,   0 }, {  83,   0 }, {  84,   0 }, {  85,   0 }, {  86,   0 },
 {  87,   0 }, {  88,   0 }, {  89,   0 }, {  90,   0 }, {  91,   0 },
 {  92,   0 }, {  93,   0 }, {  94,   0 }, {  95,   0 }, {  96,   0 },
 {  97,   0 }, {  98,   0 }, {  99,   0 }, { 100,   0 }, { 101,   0 },
 { 102,   0 }, { 103,   0 }, { 104,   0 }, { 105,   0 }, { 106,   0 },
 { 107,   0 }, { 108,   0 }, { 109,   0 }, { 110,   0 }, { 111,   0 },
 { 112,   0 }, { 113,   0 }, { 114,   0 }, { 115,   0 }, { 116,   0 },
 { 117,   0 }, { 118,   0 }, { 119,   0 }, { 120,   0 }, { 121,   0 },
 { 122,   0 }, { 123,   0 }, { 124,   0 }, { 125,   0 }, { 126,   0 },

 { 127,   0 }, { 128,   0 }, { 129,   0 }, { 130,   0 }, { 131,   0 },
 { 132,   0 }, { 133,   0 }, { 134,   0 }, { 135,   0 }, { 136,   0 },
 { 137,   0 }, { 138,   0 }, { 139,   0 }, { 140,   0 }, { 141,   0 },
 { 142,   0 }, { 143,   0 }, { 144,   0 }, { 145,   0 }, { 146,   0 },
 { 147,   0 }, { 148,   0 }, { 149,   0 }, { 150,   0 }, { 151,   0 },
 { 152,   0 }, { 153,   0 }, { 154,   0 }, { 155,   0 }, { 156,   0 },
 { 157,   0 }, { 158,   0 }, { 159,   0 }, { 160,   0 }, { 161,   0 },
 { 162,   0 }, { 163,   0 }, { 164,   0 }, { 165,   0 }, { 166,   0 },
 { 167,   0 }, { 168,   0 }, { 169,   0 }, { 170,   0 }, { 171,   0 },
 { 172,   0 }, { 173,   0 }, { 174,   0 }, { 175,   0 }, { 176,   0 },

 { 177,   0 }, { 178,   0 }, { 179,   0 }, { 180,   0 }, { 181,   0 },
 { 182,   0 }, { 183,   0 }, { 184,   0 }, { 185,   0 }, { 186,   0 },
 { 187,   0 }, { 188,   0 }, { 189,   0 }, { 190,   0 }, { 191,   0 },
 { 192,   0 }, { 193,   0 }, { 194,   0 }, { 195,   0 }, { 196,   0 },
 { 197,   0 }, { 198,   0 }, { 199,   0 }, { 200,   0 }, { 201,   0 },
 { 202,   0 }, { 203,   0 }, { 204,   0 }, { 205,   0 }, { 206,   0 },
 { 207,   0 }, { 208,   0 }, { 209,   0 }, { 210,   0 }, { 211,   0 },
 { 212,   0 }, { 213,   0 }, { 214,   0 }, { 215,   0 }, { 216,   0 },
 { 217,   0 }, { 218,   0 }, { 219,   0 }, { 220,   0 }, { 221,   0 },
 { 222,   0 }, { 223,   0 }, { 224,   0 }, { 225,   0 }, { 226,   0 },

 { 227,   0 }, { 228,   0 }, { 229,   0 }, { 230,   0 }, { 231,   0 },
 { 232,   0 }, { 233,   0 }, { 234,   0 }, { 235,   0 }, { 236,   0 },
 { 237,   0 }, { 238,   0 }, { 239,   0 }, { 240,   0 }, { 241,   0 },
 { 242,   0 }, { 243,   0 }, { 244,   0 }, { 245,   0 }, { 246,   0 },
 { 247,   0 }, { 248,   0 }, { 249,   0 }, { 250,   0 }, { 251,   0 },
 { 252,   0 }, { 253,   0 }, { 254,   0 }, { 255,   0 }, { 256,   0 },
 {   0,   1 }, {   0,19694 }, {   1,-258 }, {   2,-258 }, {   3,-258 },
 {   4,-258 }, {   5,-258 }, {   6,-258 }, {   7,-258 }, {   8,-258 },
 {   9,-258 }, {   0,   0 }, {  11,-258 }, {  12,-258 }, {   0,   0 },
 {  14,-258 }, {  15,-258 }, {  16,-258 }, {  17,-258 }, {  18,-258 },

 {  19,-258 }, {  20,-258 }, {  21,-258 }, {  22,-258 }, {  23,-258 },
 {  24,-258 }, {  25,-258 }, {  26,-258 }, {  27,-258 }, {  28,-258 },
 {  29,-258 }, {  30,-258 }, {  31,-258 }, {  32,-258 }, {  33,   0 },
 {  34,-258 }, {  35,   0 }, {  36,-258 }, {  37,   0 }, {  38,   0 },
 {  39,-258 }, {  40,-258 }, {  41,-258 }, {  42,   0 }, {  43,   0 },
 {  44,-258 }, {  45,   0 }, {  46,-258 }, {  47,   0 }, {  48,-258 },
 {  49,-258 }, {  50,-258 }, {  51,-258 }, {  52,-258 }, {  53,-258 },
 {  54,-258 }, {  55,-258 }, {  56,-258 }, {  57,-258 }, {  58,-258 },
 {  59,-258 }, {  60,   0 }, {  61,   0 }, {  62,   0 }, {  63,   0 },
 {  64,   0 }, {  65,-258 }, {  66,-258 }, {  67,-258 }, {  68,-258 },

 {  69,-258 }, {  70,-258 }, {  71,-258 }, {  72,-258 }, {  73,-258 },
 {  74,-258 }, {  75,-258 }, {  76,-258 }, {  77,-258 }, {  78,-258 },
 {  79,-258 }, {  80,-258 }, {  81,-258 }, {  82,-258 }, {  83,-258 },
 {  84,-258 }, {  85,-258 }, {  86,-258 }, {  87,-258 }, {  88,-258 },
 {  89,-258 }, {  90,-258 }, {  91,-258 }, {  92,-258 }, {  93,-258 },
 {  94,   0 }, {  95,-258 }, {  96,   0 }, {  97,-258 }, {  98,-258 },
 {  99,-258 }, { 100,-258 }, { 101,-258 }, { 102,-258 }, { 103,-258 },
 { 104,-258 }, { 105,-258 }, { 106,-258 }, { 107,-258 }, { 108,-258 },
 { 109,-258 }, { 110,-258 }, { 111,-258 }, { 112,-258 }, { 113,-258 },
 { 114,-258 }, { 115,-258 }, { 116,-258 }, { 117,-258 }, { 118,-258 },

 { 119,-258 }, { 120,-258 }, { 121,-258 }, { 122,-258 }, { 123,-258 },
 { 124,   0 }, { 125,-258 }, { 126,   0 }, { 127,-258 }, { 128,-258 },
 { 129,-258 }, { 130,-258 }, { 131,-258 }, { 132,-258 }, { 133,-258 },
 { 134,-258 }, { 135,-258 }, { 136,-258 }, { 137,-258 }, { 138,-258 },
 { 139,-258 }, { 140,-258 }, { 141,-258 }, { 142,-258 }, { 143,-258 },
 { 144,-258 }, { 145,-258 }, { 146,-258 }, { 147,-258 }, { 148,-258 },
 { 149,-258 }, { 150,-258 }, { 151,-258 }, { 152,-258 }, { 153,-258 },
 { 154,-258 }, { 155,-258 }, { 156,-258 }, { 157,-258 }, { 158,-258 },
 { 159,-258 }, { 160,-258 }, { 161,-258 }, { 162,-258 }, { 163,-258 },
 { 164,-258 }, { 165,-258 }, { 166,-258 }, { 167,-258 }, { 168,-258 },

 { 169,-258 }, { 170,-258 }, { 171,-258 }, { 172,-258 }, { 173,-258 },
 { 174,-258 }, { 175,-258 }, { 176,-258 }, { 177,-258 }, { 178,-258 },
 { 179,-258 }, { 180,-258 }, { 181,-258 }, { 182,-258 }, { 183,-258 },
 { 184,-258 }, { 185,-258 }, { 186,-258 }, { 187,-258 }, { 188,-258 },
 { 189,-258 }, { 190,-258 }, { 191,-258 }, { 192,-258 }, { 193,-258 },
 { 194,-258 }, { 195,-258 }, { 196,-258 }, { 197,-258 }, { 198,-258 },
 { 199,-258 }, { 200,-258 }, { 201,-258 }, { 202,-258 }, { 203,-258 },
 { 204,-258 }, { 205,-258 }, { 206,-258 }, { 207,-258 }, { 208,-258 },
 { 209,-258 }, { 210,-258 }, { 211,-258 }, { 212,-258 }, { 213,-258 },
 { 214,-258 }, { 215,-258 }, { 216,-258 }, { 217,-258 }, { 218,-258 },

 { 219,-258 }, { 220,-258 }, { 221,-258 }, { 222,-258 }, { 223,-258 },
 { 224,-258 }, { 225,-258 }, { 226,-258 }, { 227,-258 }, { 228,-258 },
 { 229,-258 }, { 230,-258 }, { 231,-258 }, { 232,-258 }, { 233,-258 },
 { 234,-258 }, { 235,-258 }, { 236,-258 }, { 237,-258 }, { 238,-258 },
 { 239,-258 }, { 240,-258 }, { 241,-258 }, { 242,-258 }, { 243,-258 },
 { 244,-258 }, { 245,-258 }, { 246,-258 }, { 247,-258 }, { 248,-258 },
 { 249,-258 }, { 250,-258 }, { 251,-258 }, { 252,-258 }, { 253,-258 },
 { 254,-258 }, { 255,-258 }, { 256,-258 }, {   0,  76 }, {   0,19436 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   2 }, {   0,19411 },
 {   0,  73 }, {   0,19409 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  43,  91 }, {   0,   0 }, {  45,  91 },
 {   0,   0 }, {   0,   0 }, {  48, 104 }, {  49, 104 }, {  50, 104 },
 {  51, 104 }, {  52, 104 }, {  53, 104 }, {  54, 104 }, {  55, 104 },
 {  56, 104 }, {  57, 104 }, {  33,   0 }, {   0,   0 }, {  35,   0 },

 {   0,   0 }, {  37,   0 }, {  38,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  42,   0 }, {  43,   0 }, {   0,   0 }, {  45,   0 },
 {   0,   0 }, {  47,   0 }, {   0,   0 }, {   0,   0 }, {  48,   0 },
 {  49,   0 }, {  50,   0 }, {  51,   0 }, {  52,   0 }, {  53,   0 },
 {  54,   0 }, {  55,   0 }, {  56,   0 }, {  57,   0 }, {  60,   0 },
 {  61,   0 }, {  62,   0 }, {  63,   0 }, {  64,   0 }, {   0,  77 },
 {   0,19345 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  69, -27 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,  75 }, {   0,19332 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  94,   0 }, {   0,   0 },
 {  96,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, { 101, -27 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  48,  13 }, {  49,  13 },
 {  50,  13 }, {  51,  13 }, {  52,  13 }, {  53,  13 }, {  54,  13 },
 {  55,  13 }, {  56,  13 }, {  57,  13 }, { 124,   0 }, {   0,   0 },
 { 126,   0 }, {  48,   0 }, {  49,   0 }, {  50,   0 }, {  51,   0 },
 {  52,   0 }, {  53,   0 }, {  54,   0 }, {  55,   0 }, {  56,   0 },

 {  57,   0 }, {   0,   9 }, {   0,19273 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   9,   0 }, {  10,   0 }, {   0,   0 }, {  12,   0 },
 {  13,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  32,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  39,-10600 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  45,-10598 }, {   0,   9 }, {   0,19226 },

 {   1,2553 }, {   2,2553 }, {   3,2553 }, {   4,2553 }, {   5,2553 },
 {   6,2553 }, {   7,2553 }, {   8,2553 }, {   9,2811 }, {  10,-3853 },
 {  11,2553 }, {  12,2811 }, {  13,-3853 }, {  14,2553 }, {  15,2553 },
 {  16,2553 }, {  17,2553 }, {  18,2553 }, {  19,2553 }, {  20,2553 },
 {  21,2553 }, {  22,2553 }, {  23,2553 }, {  24,2553 }, {  25,2553 },
 {  26,2553 }, {  27,2553 }, {  28,2553 }, {  29,2553 }, {  30,2553 },
 {  31,2553 }, {  32,2811 }, {  33,2553 }, {  34,2553 }, {  35,2553 },
 {  36,2553 }, {  37,2553 }, {  38,2553 }, {  39,2553 }, {  40,2553 },
 {  41,2553 }, {  42,2553 }, {  43,2553 }, {  44,2553 }, {  45,3069 },
 {  46,2553 }, {  47,2553 }, {  48,2553 }, {  49,2553 }, {  50,2553 },

 {  51,2553 }, {  52,2553 }, {  53,2553 }, {  54,2553 }, {  55,2553 },
 {  56,2553 }, {  57,2553 }, {  58,2553 }, {  59,2553 }, {  60,2553 },
 {  61,2553 }, {  62,2553 }, {  63,2553 }, {  64,2553 }, {  65,2553 },
 {  66,2553 }, {  67,2553 }, {  68,2553 }, {  69,2553 }, {  70,2553 },
 {  71,2553 }, {  72,2553 }, {  73,2553 }, {  74,2553 }, {  75,2553 },
 {  76,2553 }, {  77,2553 }, {  78,2553 }, {  79,2553 }, {  80,2553 },
 {  81,2553 }, {  82,2553 }, {  83,2553 }, {  84,2553 }, {  85,2553 },
 {  86,2553 }, {  87,2553 }, {  88,2553 }, {  89,2553 }, {  90,2553 },
 {  91,2553 }, {  92,2553 }, {  93,2553 }, {  94,2553 }, {  95,2553 },
 {  96,2553 }, {  97,2553 }, {  98,2553 }, {  99,2553 }, { 100,2553 },

 { 101,2553 }, { 102,2553 }, { 103,2553 }, { 104,2553 }, { 105,2553 },
 { 106,2553 }, { 107,2553 }, { 108,2553 }, { 109,2553 }, { 110,2553 },
 { 111,2553 }, { 112,2553 }, { 113,2553 }, { 114,2553 }, { 115,2553 },
 { 116,2553 }, { 117,2553 }, { 118,2553 }, { 119,2553 }, { 120,2553 },
 { 121,2553 }, { 122,2553 }, { 123,2553 }, { 124,2553 }, { 125,2553 },
 { 126,2553 }, { 127,2553 }, { 128,2553 }, { 129,2553 }, { 130,2553 },
 { 131,2553 }, { 132,2553 }, { 133,2553 }, { 134,2553 }, { 135,2553 },
 { 136,2553 }, { 137,2553 }, { 138,2553 }, { 139,2553 }, { 140,2553 },
 { 141,2553 }, { 142,2553 }, { 143,2553 }, { 144,2553 }, { 145,2553 },
 { 146,2553 }, { 147,2553 }, { 148,2553 }, { 149,2553 }, { 150,2553 },

 { 151,2553 }, { 152,2553 }, { 153,2553 }, { 154,2553 }, { 155,2553 },
 { 156,2553 }, { 157,2553 }, { 158,2553 }, { 159,2553 }, { 160,2553 },
 { 161,2553 }, { 162,2553 }, { 163,2553 }, { 164,2553 }, { 165,2553 },
 { 166,2553 }, { 167,2553 }, { 168,2553 }, { 169,2553 }, { 170,2553 },
 { 171,2553 }, { 172,2553 }, { 173,2553 }, { 174,2553 }, { 175,2553 },
 { 176,2553 }, { 177,2553 }, { 178,2553 }, { 179,2553 }, { 180,2553 },
 { 181,2553 }, { 182,2553 }, { 183,2553 }, { 184,2553 }, { 185,2553 },
 { 186,2553 }, { 187,2553 }, { 188,2553 }, { 189,2553 }, { 190,2553 },
 { 191,2553 }, { 192,2553 }, { 193,2553 }, { 194,2553 }, { 195,2553 },
 { 196,2553 }, { 197,2553 }, { 198,2553 }, { 199,2553 }, { 200,2553 },

 { 201,2553 }, { 202,2553 }, { 203,2553 }, { 204,2553 }, { 205,2553 },
 { 206,2553 }, { 207,2553 }, { 208,2553 }, { 209,2553 }, { 210,2553 },
 { 211,2553 }, { 212,2553 }, { 213,2553 }, { 214,2553 }, { 215,2553 },
 { 216,2553 }, { 217,2553 }, { 218,2553 }, { 219,2553 }, { 220,2553 },
 { 221,2553 }, { 222,2553 }, { 223,2553 }, { 224,2553 }, { 225,2553 },
 { 226,2553 }, { 227,2553 }, { 228,2553 }, { 229,2553 }, { 230,2553 },
 { 231,2553 }, { 232,2553 }, { 233,2553 }, { 234,2553 }, { 235,2553 },
 { 236,2553 }, { 237,2553 }, { 238,2553 }, { 239,2553 }, { 240,2553 },
 { 241,2553 }, { 242,2553 }, { 243,2553 }, { 244,2553 }, { 245,2553 },
 { 246,2553 }, { 247,2553 }, { 248,2553 }, { 249,2553 }, { 250,2553 },

 { 251,2553 }, { 252,2553 }, { 253,2553 }, { 254,2553 }, { 255,2553 },
 { 256,2553 }, {   0,   3 }, {   0,18968 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,  16 }, {   0,18961 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   9,   0 }, {  10,   0 },
 {   0,   0 }, {  12,   0 }, {  13,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  33,   0 }, {   0,   0 }, {  35,   0 }, {   0,   0 }, {  37,   0 },
 {  38,   0 }, {  32,   0 }, {   0,   0 }, {   0,   0 }, {  42,   0 },

 {  43,   0 }, {   0,   0 }, {  45,   0 }, {  39,-10908 }, {  47,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  45,-10903 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  60,   0 }, {  61,   0 }, {  62,   0 },
 {  63,   0 }, {  64,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {  94,   0 }, {   0,   0 }, {  96,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, { 124,   0 }, {   0,   0 }, { 126,   0 }, {   0,  16 },
 {   0,18840 }, {   1,3199 }, {   2,3199 }, {   3,3199 }, {   4,3199 },
 {   5,3199 }, {   6,3199 }, {   7,3199 }, {   8,3199 }, {   9,3457 },
 {  10,-3285 }, {  11,3199 }, {  12,3457 }, {  13,-3285 }, {  14,3199 },

 {  15,3199 }, {  16,3199 }, {  17,3199 }, {  18,3199 }, {  19,3199 },
 {  20,3199 }, {  21,3199 }, {  22,3199 }, {  23,3199 }, {  24,3199 },
 {  25,3199 }, {  26,3199 }, {  27,3199 }, {  28,3199 }, {  29,3199 },
 {  30,3199 }, {  31,3199 }, {  32,3457 }, {  33,3199 }, {  34,3199 },
 {  35,3199 }, {  36,3199 }, {  37,3199 }, {  38,3199 }, {  39,3199 },
 {  40,3199 }, {  41,3199 }, {  42,3199 }, {  43,3199 }, {  44,3199 },
 {  45,3715 }, {  46,3199 }, {  47,3199 }, {  48,3199 }, {  49,3199 },
 {  50,3199 }, {  51,3199 }, {  52,3199 }, {  53,3199 }, {  54,3199 },
 {  55,3199 }, {  56,3199 }, {  57,3199 }, {  58,3199 }, {  59,3199 },
 {  60,3199 }, {  61,3199 }, {  62,3199 }, {  63,3199 }, {  64,3199 },

 {  65,3199 }, {  66,3199 }, {  67,3199 }, {  68,3199 }, {  69,3199 },
 {  70,3199 }, {  71,3199 }, {  72,3199 }, {  73,3199 }, {  74,3199 },
 {  75,3199 }, {  76,3199 }, {  77,3199 }, {  78,3199 }, {  79,3199 },
 {  80,3199 }, {  81,3199 }, {  82,3199 }, {  83,3199 }, {  84,3199 },
 {  85,3199 }, {  86,3199 }, {  87,3199 }, {  88,3199 }, {  89,3199 },
 {  90,3199 }, {  91,3199 }, {  92,3199 }, {  93,3199 }, {  94,3199 },
 {  95,3199 }, {  96,3199 }, {  97,3199 }, {  98,3199 }, {  99,3199 },
 { 100,3199 }, { 101,3199 }, { 102,3199 }, { 103,3199 }, { 104,3199 },
 { 105,3199 }, { 106,3199 }, { 107,3199 }, { 108,3199 }, { 109,3199 },
 { 110,3199 }, { 111,3199 }, { 112,3199 }, { 113,3199 }, { 114,3199 },

 { 115,3199 }, { 116,3199 }, { 117,3199 }, { 118,3199 }, { 119,3199 },
 { 120,3199 }, { 121,3199 }, { 122,3199 }, { 123,3199 }, { 124,3199 },
 { 125,3199 }, { 126,3199 }, { 127,3199 }, { 128,3199 }, { 129,3199 },
 { 130,3199 }, { 131,3199 }, { 132,3199 }, { 133,3199 }, { 134,3199 },
 { 135,3199 }, { 136,3199 }, { 137,3199 }, { 138,3199 }, { 139,3199 },
 { 140,3199 }, { 141,3199 }, { 142,3199 }, { 143,3199 }, { 144,3199 },
 { 145,3199 }, { 146,3199 }, { 147,3199 }, { 148,3199 }, { 149,3199 },
 { 150,3199 }, { 151,3199 }, { 152,3199 }, { 153,3199 }, { 154,3199 },
 { 155,3199 }, { 156,3199 }, { 157,3199 }, { 158,3199 }, { 159,3199 },
 { 160,3199 }, { 161,3199 }, { 162,3199 }, { 163,3199 }, { 164,3199 },

 { 165,3199 }, { 166,3199 }, { 167,3199 }, { 168,3199 }, { 169,3199 },
 { 170,3199 }, { 171,3199 }, { 172,3199 }, { 173,3199 }, { 174,3199 },
 { 175,3199 }, { 176,3199 }, { 177,3199 }, { 178,3199 }, { 179,3199 },
 { 180,3199 }, { 181,3199 }, { 182,3199 }, { 183,3199 }, { 184,3199 },
 { 185,3199 }, { 186,3199 }, { 187,3199 }, { 188,3199 }, { 189,3199 },
 { 190,3199 }, { 191,3199 }, { 192,3199 }, { 193,3199 }, { 194,3199 },
 { 195,3199 }, { 196,3199 }, { 197,3199 }, { 198,3199 }, { 199,3199 },
 { 200,3199 }, { 201,3199 }, { 202,3199 }, { 203,3199 }, { 204,3199 },
 { 205,3199 }, { 206,3199 }, { 207,3199 }, { 208,3199 }, { 209,3199 },
 { 210,3199 }, { 211,3199 }, { 212,3199 }, { 213,3199 }, { 214,3199 },

 { 215,3199 }, { 216,3199 }, { 217,3199 }, { 218,3199 }, { 219,3199 },
 { 220,3199 }, { 221,3199 }, { 222,3199 }, { 223,3199 }, { 224,3199 },
 { 225,3199 }, { 226,3199 }, { 227,3199 }, { 228,3199 }, { 229,3199 },
 { 230,3199 }, { 231,3199 }, { 232,3199 }, { 233,3199 }, { 234,3199 },
 { 235,3199 }, { 236,3199 }, { 237,3199 }, { 238,3199 }, { 239,3199 },
 { 240,3199 }, { 241,3199 }, { 242,3199 }, { 243,3199 }, { 244,3199 },
 { 245,3199 }, { 246,3199 }, { 247,3199 }, { 248,3199 }, { 249,3199 },
 { 250,3199 }, { 251,3199 }, { 252,3199 }, { 253,3199 }, { 254,3199 },
 { 255,3199 }, { 256,3199 }, {   0,  22 }, {   0,18582 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   9,   0 }, {  10,   0 }, {   0,   0 },
 {  12,   0 }, {  13,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  32,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  39,-11280 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  45,-11259 }, {   0,  22 },
 {   0,18535 }, {   1,3926 }, {   2,3926 }, {   3,3926 }, {   4,3926 },
 {   5,3926 }, {   6,3926 }, {   7,3926 }, {   8,3926 }, {   9,4184 },

 {  10,-3280 }, {  11,3926 }, {  12,4184 }, {  13,-3280 }, {  14,3926 },
 {  15,3926 }, {  16,3926 }, {  17,3926 }, {  18,3926 }, {  19,3926 },
 {  20,3926 }, {  21,3926 }, {  22,3926 }, {  23,3926 }, {  24,3926 },
 {  25,3926 }, {  26,3926 }, {  27,3926 }, {  28,3926 }, {  29,3926 },
 {  30,3926 }, {  31,3926 }, {  32,4184 }, {  33,3926 }, {  34,3926 },
 {  35,3926 }, {  36,3926 }, {  37,3926 }, {  38,3926 }, {  39,3926 },
 {  40,3926 }, {  41,3926 }, {  42,3926 }, {  43,3926 }, {  44,3926 },
 {  45,4442 }, {  46,3926 }, {  47,3926 }, {  48,3926 }, {  49,3926 },
 {  50,3926 }, {  51,3926 }, {  52,3926 }, {  53,3926 }, {  54,3926 },
 {  55,3926 }, {  56,3926 }, {  57,3926 }, {  58,3926 }, {  59,3926 },

 {  60,3926 }, {  61,3926 }, {  62,3926 }, {  63,3926 }, {  64,3926 },
 {  65,3926 }, {  66,3926 }, {  67,3926 }, {  68,3926 }, {  69,3926 },
 {  70,3926 }, {  71,3926 }, {  72,3926 }, {  73,3926 }, {  74,3926 },
 {  75,3926 }, {  76,3926 }, {  77,3926 }, {  78,3926 }, {  79,3926 },
 {  80,3926 }, {  81,3926 }, {  82,3926 }, {  83,3926 }, {  84,3926 },
 {  85,3926 }, {  86,3926 }, {  87,3926 }, {  88,3926 }, {  89,3926 },
 {  90,3926 }, {  91,3926 }, {  92,3926 }, {  93,3926 }, {  94,3926 },
 {  95,3926 }, {  96,3926 }, {  97,3926 }, {  98,3926 }, {  99,3926 },
 { 100,3926 }, { 101,3926 }, { 102,3926 }, { 103,3926 }, { 104,3926 },
 { 105,3926 }, { 106,3926 }, { 107,3926 }, { 108,3926 }, { 109,3926 },

 { 110,3926 }, { 111,3926 }, { 112,3926 }, { 113,3926 }, { 114,3926 },
 { 115,3926 }, { 116,3926 }, { 117,3926 }, { 118,3926 }, { 119,3926 },
 { 120,3926 }, { 121,3926 }, { 122,3926 }, { 123,3926 }, { 124,3926 },
 { 125,3926 }, { 126,3926 }, { 127,3926 }, { 128,3926 }, { 129,3926 },
 { 130,3926 }, { 131,3926 }, { 132,3926 }, { 133,3926 }, { 134,3926 },
 { 135,3926 }, { 136,3926 }, { 137,3926 }, { 138,3926 }, { 139,3926 },
 { 140,3926 }, { 141,3926 }, { 142,3926 }, { 143,3926 }, { 144,3926 },
 { 145,3926 }, { 146,3926 }, { 147,3926 }, { 148,3926 }, { 149,3926 },
 { 150,3926 }, { 151,3926 }, { 152,3926 }, { 153,3926 }, { 154,3926 },
 { 155,3926 }, { 156,3926 }, { 157,3926 }, { 158,3926 }, { 159,3926 },

 { 160,3926 }, { 161,3926 }, { 162,3926 }, { 163,3926 }, { 164,3926 },
 { 165,3926 }, { 166,3926 }, { 167,3926 }, { 168,3926 }, { 169,3926 },
 { 170,3926 }, { 171,3926 }, { 172,3926 }, { 173,3926 }, { 174,3926 },
 { 175,3926 }, { 176,3926 }, { 177,3926 }, { 178,3926 }, { 179,3926 },
 { 180,3926 }, { 181,3926 }, { 182,3926 }, { 183,3926 }, { 184,3926 },
 { 185,3926 }, { 186,3926 }, { 187,3926 }, { 188,3926 }, { 189,3926 },
 { 190,3926 }, { 191,3926 }, { 192,3926 }, { 193,3926 }, { 194,3926 },
 { 195,3926 }, { 196,3926 }, { 197,3926 }, { 198,3926 }, { 199,3926 },
 { 200,3926 }, { 201,3926 }, { 202,3926 }, { 203,3926 }, { 204,3926 },
 { 205,3926 }, { 206,3926 }, { 207,3926 }, { 208,3926 }, { 209,3926 },

 { 210,3926 }, { 211,3926 }, { 212,3926 }, { 213,3926 }, { 214,3926 },
 { 215,3926 }, { 216,3926 }, { 217,3926 }, { 218,3926 }, { 219,3926 },
 { 220,3926 }, { 221,3926 }, { 222,3926 }, { 223,3926 }, { 224,3926 },
 { 225,3926 }, { 226,3926 }, { 227,3926 }, { 228,3926 }, { 229,3926 },
 { 230,3926 }, { 231,3926 }, { 232,3926 }, { 233,3926 }, { 234,3926 },
 { 235,3926 }, { 236,3926 }, { 237,3926 }, { 238,3926 }, { 239,3926 },
 { 240,3926 }, { 241,3926 }, { 242,3926 }, { 243,3926 }, { 244,3926 },
 { 245,3926 }, { 246,3926 }, { 247,3926 }, { 248,3926 }, { 249,3926 },
 { 250,3926 }, { 251,3926 }, { 252,3926 }, { 253,3926 }, { 254,3926 },
 { 255,3926 }, { 256,3926 }, {   0,  39 }, {   0,18277 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,  37 }, {   0,18269 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,  37 }, {   0,18246 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  48,-11445 }, {  49,-11445 }, {  50,-11445 }, {  51,-11445 },

 {  52,-11445 }, {  53,-11445 }, {  54,-11445 }, {  55,-11445 }, {  48,4434 },
 {  49,4434 }, {  50,4434 }, {  51,4434 }, {  52,4434 }, {  53,4434 },
 {  54,4434 }, {  55,4434 }, {  56,4434 }, {  57,4434 }, {   0,   0 },
 {   0,   0 }, {   0,  40 }, {   0,18208 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  65,4434 }, {  66,4434 }, {  67,4434 }, {  68,4434 },
 {  69,4434 }, {  70,4434 }, {  48,4434 }, {  49,4434 }, {  50,4434 },
 {  51,4434 }, {  52,4434 }, {  53,4434 }, {  54,4434 }, {  55,4434 },
 {  56,4434 }, {  57,4434 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65,4434 },
 {  66,4434 }, {  67,4434 }, {  68,4434 }, {  69,4434 }, {  70,4434 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  97,4434 }, {  98,4434 },
 {  99,4434 }, { 100,4434 }, { 101,4434 }, { 102,4434 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  48,-11512 }, {  49,-11512 }, {  50,-11512 }, {  51,-11512 }, {  52,-11512 },
 {  53,-11512 }, {  54,-11512 }, {  55,-11512 }, {  56,-11512 }, {  57,-11512 },
 {   0,   0 }, {  97,4434 }, {  98,4434 }, {  99,4434 }, { 100,4434 },
 { 101,4434 }, { 102,4434 }, {  65,-11512 }, {  66,-11512 }, {  67,-11512 },
 {  68,-11512 }, {  69,-11512 }, {  70,-11512 }, {   0,  47 }, {   0,18136 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  97,-11512 },
 {  98,-11512 }, {  99,-11512 }, { 100,-11512 }, { 101,-11512 }, { 102,-11512 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  36,-11828 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  48,   0 }, {  49,   0 }, {  50,   0 },
 {  51,   0 }, {  52,   0 }, {  53,   0 }, {  54,   0 }, {  55,   0 },
 {  56,   0 }, {  57,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65,   0 },
 {  66,   0 }, {  67,   0 }, {  68,   0 }, {  69,   0 }, {  70,   0 },
 {  71,   0 }, {  72,   0 }, {  73,   0 }, {  74,   0 }, {  75,   0 },
 {  76,   0 }, {  77,   0 }, {  78,   0 }, {  79,   0 }, {  80,   0 },
 {  81,   0 }, {  82,   0 }, {  83,   0 }, {  84,   0 }, {  85,   0 },
 {  86,   0 }, {  87,   0 }, {  88,   0 }, {  89,   0 }, {  90,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  95,   0 },
 {   0,   0 }, {  97,   0 }, {  98,   0 }, {  99,   0 }, { 100,   0 },
 { 101,   0 }, { 102,   0 }, { 103,   0 }, { 104,   0 }, { 105,   0 },
 { 106,   0 }, { 107,   0 }, { 108,   0 }, { 109,   0 }, { 110,   0 },

 { 111,   0 }, { 112,   0 }, { 113,   0 }, { 114,   0 }, { 115,   0 },
 { 116,   0 }, { 117,   0 }, { 118,   0 }, { 119,   0 }, { 120,   0 },
 { 121,   0 }, { 122,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, { 128,   0 }, { 129,   0 }, { 130,   0 },
 { 131,   0 }, { 132,   0 }, { 133,   0 }, { 134,   0 }, { 135,   0 },
 { 136,   0 }, { 137,   0 }, { 138,   0 }, { 139,   0 }, { 140,   0 },
 { 141,   0 }, { 142,   0 }, { 143,   0 }, { 144,   0 }, { 145,   0 },
 { 146,   0 }, { 147,   0 }, { 148,   0 }, { 149,   0 }, { 150,   0 },
 { 151,   0 }, { 152,   0 }, { 153,   0 }, { 154,   0 }, { 155,   0 },
 { 156,   0 }, { 157,   0 }, { 158,   0 }, { 159,   0 }, { 160,   0 },

 { 161,   0 }, { 162,   0 }, { 163,   0 }, { 164,   0 }, { 165,   0 },
 { 166,   0 }, { 167,   0 }, { 168,   0 }, { 169,   0 }, { 170,   0 },
 { 171,   0 }, { 172,   0 }, { 173,   0 }, { 174,   0 }, { 175,   0 },
 { 176,   0 }, { 177,   0 }, { 178,   0 }, { 179,   0 }, { 180,   0 },
 { 181,   0 }, { 182,   0 }, { 183,   0 }, { 184,   0 }, { 185,   0 },
 { 186,   0 }, { 187,   0 }, { 188,   0 }, { 189,   0 }, { 190,   0 },
 { 191,   0 }, { 192,   0 }, { 193,   0 }, { 194,   0 }, { 195,   0 },
 { 196,   0 }, { 197,   0 }, { 198,   0 }, { 199,   0 }, { 200,   0 },
 { 201,   0 }, { 202,   0 }, { 203,   0 }, { 204,   0 }, { 205,   0 },
 { 206,   0 }, { 207,   0 }, { 208,   0 }, { 209,   0 }, { 210,   0 },

 { 211,   0 }, { 212,   0 }, { 213,   0 }, { 214,   0 }, { 215,   0 },
 { 216,   0 }, { 217,   0 }, { 218,   0 }, { 219,   0 }, { 220,   0 },
 { 221,   0 }, { 222,   0 }, { 223,   0 }, { 224,   0 }, { 225,   0 },
 { 226,   0 }, { 227,   0 }, { 228,   0 }, { 229,   0 }, { 230,   0 },
 { 231,   0 }, { 232,   0 }, { 233,   0 }, { 234,   0 }, { 235,   0 },
 { 236,   0 }, { 237,   0 }, { 238,   0 }, { 239,   0 }, { 240,   0 },
 { 241,   0 }, { 242,   0 }, { 243,   0 }, { 244,   0 }, { 245,   0 },
 { 246,   0 }, { 247,   0 }, { 248,   0 }, { 249,   0 }, { 250,   0 },
 { 251,   0 }, { 252,   0 }, { 253,   0 }, { 254,   0 }, { 255,   0 },
 {   0,  53 }, {   0,17879 }, {   1,   0 }, {   2,   0 }, {   3,   0 },

 {   4,   0 }, {   5,   0 }, {   6,   0 }, {   7,   0 }, {   8,   0 },
 {   9,   0 }, {   0,   0 }, {  11,   0 }, {  12,   0 }, {   0,   0 },
 {  14,   0 }, {  15,   0 }, {  16,   0 }, {  17,   0 }, {  18,   0 },
 {  19,   0 }, {  20,   0 }, {  21,   0 }, {  22,   0 }, {  23,   0 },
 {  24,   0 }, {  25,   0 }, {  26,   0 }, {  27,   0 }, {  28,   0 },
 {  29,   0 }, {  30,   0 }, {  31,   0 }, {  32,   0 }, {  33,   0 },
 {  34,   0 }, {  35,   0 }, {  36,   0 }, {  37,   0 }, {  38,   0 },
 {  39,   0 }, {  40,   0 }, {  41,   0 }, {  42,   0 }, {  43,   0 },
 {  44,   0 }, {  45,   0 }, {  46,   0 }, {  47,   0 }, {  48,   0 },
 {  49,   0 }, {  50,   0 }, {  51,   0 }, {  52,   0 }, {  53,   0 },

 {  54,   0 }, {  55,   0 }, {  56,   0 }, {  57,   0 }, {  58,   0 },
 {  59,   0 }, {  60,   0 }, {  61,   0 }, {  62,   0 }, {  63,   0 },
 {  64,   0 }, {  65,   0 }, {  66,   0 }, {  67,   0 }, {  68,   0 },
 {  69,   0 }, {  70,   0 }, {  71,   0 }, {  72,   0 }, {  73,   0 },
 {  74,   0 }, {  75,   0 }, {  76,   0 }, {  77,   0 }, {  78,   0 },
 {  79,   0 }, {  80,   0 }, {  81,   0 }, {  82,   0 }, {  83,   0 },
 {  84,   0 }, {  85,   0 }, {  86,   0 }, {  87,   0 }, {  88,   0 },
 {  89,   0 }, {  90,   0 }, {  91,   0 }, {  92,   0 }, {  93,   0 },
 {  94,   0 }, {  95,   0 }, {  96,   0 }, {  97,   0 }, {  98,   0 },
 {  99,   0 }, { 100,   0 }, { 101,   0 }, { 102,   0 }, { 103,   0 },

 { 104,   0 }, { 105,   0 }, { 106,   0 }, { 107,   0 }, { 108,   0 },
 { 109,   0 }, { 110,   0 }, { 111,   0 }, { 112,   0 }, { 113,   0 },
 { 114,   0 }, { 115,   0 }, { 116,   0 }, { 117,   0 }, { 118,   0 },
 { 119,   0 }, { 120,   0 }, { 121,   0 }, { 122,   0 }, { 123,   0 },
 { 124,   0 }, { 125,   0 }, { 126,   0 }, { 127,   0 }, { 128,   0 },
 { 129,   0 }, { 130,   0 }, { 131,   0 }, { 132,   0 }, { 133,   0 },
 { 134,   0 }, { 135,   0 }, { 136,   0 }, { 137,   0 }, { 138,   0 },
 { 139,   0 }, { 140,   0 }, { 141,   0 }, { 142,   0 }, { 143,   0 },
 { 144,   0 }, { 145,   0 }, { 146,   0 }, { 147,   0 }, { 148,   0 },
 { 149,   0 }, { 150,   0 }, { 151,   0 }, { 152,   0 }, { 153,   0 },

 { 154,   0 }, { 155,   0 }, { 156,   0 }, { 157,   0 }, { 158,   0 },
 { 159,   0 }, { 160,   0 }, { 161,   0 }, { 162,   0 }, { 163,   0 },
 { 164,   0 }, { 165,   0 }, { 166,   0 }, { 167,   0 }, { 168,   0 },
 { 169,   0 }, { 170,   0 }, { 171,   0 }, { 172,   0 }, { 173,   0 },
 { 174,   0 }, { 175,   0 }, { 176,   0 }, { 177,   0 }, { 178,   0 },
 { 179,   0 }, { 180,   0 }, { 181,   0 }, { 182,   0 }, { 183,   0 },
 { 184,   0 }, { 185,   0 }, { 186,   0 }, { 187,   0 }, { 188,   0 },
 { 189,   0 }, { 190,   0 }, { 191,   0 }, { 192,   0 }, { 193,   0 },
 { 194,   0 }, { 195,   0 }, { 196,   0 }, { 197,   0 }, { 198,   0 },
 { 199,   0 }, { 200,   0 }, { 201,   0 }, { 202,   0 }, { 203,   0 },

 { 204,   0 }, { 205,   0 }, { 206,   0 }, { 207,   0 }, { 208,   0 },
 { 209,   0 }, { 210,   0 }, { 211,   0 }, { 212,   0 }, { 213,   0 },
 { 214,   0 }, { 215,   0 }, { 216,   0 }, { 217,   0 }, { 218,   0 },
 { 219,   0 }, { 220,   0 }, { 221,   0 }, { 222,   0 }, { 223,   0 },
 { 224,   0 }, { 225,   0 }, { 226,   0 }, { 227,   0 }, { 228,   0 },
 { 229,   0 }, { 230,   0 }, { 231,   0 }, { 232,   0 }, { 233,   0 },
 { 234,   0 }, { 235,   0 }, { 236,   0 }, { 237,   0 }, { 238,   0 },
 { 239,   0 }, { 240,   0 }, { 241,   0 }, { 242,   0 }, { 243,   0 },
 { 244,   0 }, { 245,   0 }, { 246,   0 }, { 247,   0 }, { 248,   0 },
 { 249,   0 }, { 250,   0 }, { 251,   0 }, { 252,   0 }, { 253,   0 },

 { 254,   0 }, { 255,   0 }, { 256,   0 }, {   0,  24 }, {   0,17621 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   9,   0 }, {  10,   0 },
 {   0,   0 }, {  12,   0 }, {  13,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  32,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  39,-12241 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  45,-12109 },

 {   0,  24 }, {   0,17574 }, {   1,4124 }, {   2,4124 }, {   3,4124 },
 {   4,4124 }, {   5,4124 }, {   6,4124 }, {   7,4124 }, {   8,4124 },
 {   9,4382 }, {  10,-3025 }, {  11,4124 }, {  12,4382 }, {  13,-3025 },
 {  14,4124 }, {  15,4124 }, {  16,4124 }, {  17,4124 }, {  18,4124 },
 {  19,4124 }, {  20,4124 }, {  21,4124 }, {  22,4124 }, {  23,4124 },
 {  24,4124 }, {  25,4124 }, {  26,4124 }, {  27,4124 }, {  28,4124 },
 {  29,4124 }, {  30,4124 }, {  31,4124 }, {  32,4382 }, {  33,4124 },
 {  34,4124 }, {  35,4124 }, {  36,4124 }, {  37,4124 }, {  38,4124 },
 {  39,4124 }, {  40,4124 }, {  41,4124 }, {  42,4124 }, {  43,4124 },
 {  44,4124 }, {  45,4640 }, {  46,4124 }, {  47,4124 }, {  48,4124 },

 {  49,4124 }, {  50,4124 }, {  51,4124 }, {  52,4124 }, {  53,4124 },
 {  54,4124 }, {  55,4124 }, {  56,4124 }, {  57,4124 }, {  58,4124 },
 {  59,4124 }, {  60,4124 }, {  61,4124 }, {  62,4124 }, {  63,4124 },
 {  64,4124 }, {  65,4124 }, {  66,4124 }, {  67,4124 }, {  68,4124 },
 {  69,4124 }, {  70,4124 }, {  71,4124 }, {  72,4124 }, {  73,4124 },
 {  74,4124 }, {  75,4124 }, {  76,4124 }, {  77,4124 }, {  78,4124 },
 {  79,4124 }, {  80,4124 }, {  81,4124 }, {  82,4124 }, {  83,4124 },
 {  84,4124 }, {  85,4124 }, {  86,4124 }, {  87,4124 }, {  88,4124 },
 {  89,4124 }, {  90,4124 }, {  91,4124 }, {  92,4124 }, {  93,4124 },
 {  94,4124 }, {  95,4124 }, {  96,4124 }, {  97,4124 }, {  98,4124 },

 {  99,4124 }, { 100,4124 }, { 101,4124 }, { 102,4124 }, { 103,4124 },
 { 104,4124 }, { 105,4124 }, { 106,4124 }, { 107,4124 }, { 108,4124 },
 { 109,4124 }, { 110,4124 }, { 111,4124 }, { 112,4124 }, { 113,4124 },
 { 114,4124 }, { 115,4124 }, { 116,4124 }, { 117,4124 }, { 118,4124 },
 { 119,4124 }, { 120,4124 }, { 121,4124 }, { 122,4124 }, { 123,4124 },
 { 124,4124 }, { 125,4124 }, { 126,4124 }, { 127,4124 }, { 128,4124 },
 { 129,4124 }, { 130,4124 }, { 131,4124 }, { 132,4124 }, { 133,4124 },
 { 134,4124 }, { 135,4124 }, { 136,4124 }, { 137,4124 }, { 138,4124 },
 { 139,4124 }, { 140,4124 }, { 141,4124 }, { 142,4124 }, { 143,4124 },
 { 144,4124 }, { 145,4124 }, { 146,4124 }, { 147,4124 }, { 148,4124 },

 { 149,4124 }, { 150,4124 }, { 151,4124 }, { 152,4124 }, { 153,4124 },
 { 154,4124 }, { 155,4124 }, { 156,4124 }, { 157,4124 }, { 158,4124 },
 { 159,4124 }, { 160,4124 }, { 161,4124 }, { 162,4124 }, { 163,4124 },
 { 164,4124 }, { 165,4124 }, { 166,4124 }, { 167,4124 }, { 168,4124 },
 { 169,4124 }, { 170,4124 }, { 171,4124 }, { 172,4124 }, { 173,4124 },
 { 174,4124 }, { 175,4124 }, { 176,4124 }, { 177,4124 }, { 178,4124 },
 { 179,4124 }, { 180,4124 }, { 181,4124 }, { 182,4124 }, { 183,4124 },
 { 184,4124 }, { 185,4124 }, { 186,4124 }, { 187,4124 }, { 188,4124 },
 { 189,4124 }, { 190,4124 }, { 191,4124 }, { 192,4124 }, { 193,4124 },
 { 194,4124 }, { 195,4124 }, { 196,4124 }, { 197,4124 }, { 198,4124 },

 { 199,4124 }, { 200,4124 }, { 201,4124 }, { 202,4124 }, { 203,4124 },
 { 204,4124 }, { 205,4124 }, { 206,4124 }, { 207,4124 }, { 208,4124 },
 { 209,4124 }, { 210,4124 }, { 211,4124 }, { 212,4124 }, { 213,4124 },
 { 214,4124 }, { 215,4124 }, { 216,4124 }, { 217,4124 }, { 218,4124 },
 { 219,4124 }, { 220,4124 }, { 221,4124 }, { 222,4124 }, { 223,4124 },
 { 224,4124 }, { 225,4124 }, { 226,4124 }, { 227,4124 }, { 228,4124 },
 { 229,4124 }, { 230,4124 }, { 231,4124 }, { 232,4124 }, { 233,4124 },
 { 234,4124 }, { 235,4124 }, { 236,4124 }, { 237,4124 }, { 238,4124 },
 { 239,4124 }, { 240,4124 }, { 241,4124 }, { 242,4124 }, { 243,4124 },
 { 244,4124 }, { 245,4124 }, { 246,4124 }, { 247,4124 }, { 248,4124 },

 { 249,4124 }, { 250,4124 }, { 251,4124 }, { 252,4124 }, { 253,4124 },
 { 254,4124 }, { 255,4124 }, { 256,4124 }, {   0,  26 }, {   0,17316 },
 {   1,   0 }, {   2,   0 }, {   3,   0 }, {   4,   0 }, {   5,   0 },
 {   6,   0 }, {   7,   0 }, {   8,   0 }, {   9,   0 }, {   0,   0 },
 {  11,   0 }, {  12,   0 }, {   0,   0 }, {  14,   0 }, {  15,   0 },
 {  16,   0 }, {  17,   0 }, {  18,   0 }, {  19,   0 }, {  20,   0 },
 {  21,   0 }, {  22,   0 }, {  23,   0 }, {  24,   0 }, {  25,   0 },
 {  26,   0 }, {  27,   0 }, {  28,   0 }, {  29,   0 }, {  30,   0 },
 {  31,   0 }, {  32,   0 }, {  33,   0 }, {  34,   0 }, {  35,   0 },
 {  36,   0 }, {  37,   0 }, {  38,   0 }, {  39,   0 }, {  40,   0 },

 {  41,   0 }, {  42,   0 }, {  43,   0 }, {  44,   0 }, {  45,   0 },
 {  46,   0 }, {  47,   0 }, {  48,   0 }, {  49,   0 }, {  50,   0 },
 {  51,   0 }, {  52,   0 }, {  53,   0 }, {  54,   0 }, {  55,   0 },
 {  56,   0 }, {  57,   0 }, {  58,   0 }, {  59,   0 }, {  60,   0 },
 {  61,   0 }, {  62,   0 }, {  63,   0 }, {  64,   0 }, {  65,   0 },
 {  66,   0 }, {  67,   0 }, {  68,   0 }, {  69,   0 }, {  70,   0 },
 {  71,   0 }, {  72,   0 }, {  73,   0 }, {  74,   0 }, {  75,   0 },
 {  76,   0 }, {  77,   0 }, {  78,   0 }, {  79,   0 }, {  80,   0 },
 {  81,   0 }, {  82,   0 }, {  83,   0 }, {  84,   0 }, {  85,   0 },
 {  86,   0 }, {  87,   0 }, {  88,   0 }, {  89,   0 }, {  90,   0 },

 {  91,   0 }, {  92,   0 }, {  93,   0 }, {  94,   0 }, {  95,   0 },
 {  96,   0 }, {  97,   0 }, {  98,   0 }, {  99,   0 }, { 100,   0 },
 { 101,   0 }, { 102,   0 }, { 103,   0 }, { 104,   0 }, { 105,   0 },
 { 106,   0 }, { 107,   0 }, { 108,   0 }, { 109,   0 }, { 110,   0 },
 { 111,   0 }, { 112,   0 }, { 113,   0 }, { 114,   0 }, { 115,   0 },
 { 116,   0 }, { 117,   0 }, { 118,   0 }, { 119,   0 }, { 120,   0 },
 { 121,   0 }, { 122,   0 }, { 123,   0 }, { 124,   0 }, { 125,   0 },
 { 126,   0 }, { 127,   0 }, { 128,   0 }, { 129,   0 }, { 130,   0 },
 { 131,   0 }, { 132,   0 }, { 133,   0 }, { 134,   0 }, { 135,   0 },
 { 136,   0 }, { 137,   0 }, { 138,   0 }, { 139,   0 }, { 140,   0 },

 { 141,   0 }, { 142,   0 }, { 143,   0 }, { 144,   0 }, { 145,   0 },
 { 146,   0 }, { 147,   0 }, { 148,   0 }, { 149,   0 }, { 150,   0 },
 { 151,   0 }, { 152,   0 }, { 153,   0 }, { 154,   0 }, { 155,   0 },
 { 156,   0 }, { 157,   0 }, { 158,   0 }, { 159,   0 }, { 160,   0 },
 { 161,   0 }, { 162,   0 }, { 163,   0 }, { 164,   0 }, { 165,   0 },
 { 166,   0 }, { 167,   0 }, { 168,   0 }, { 169,   0 }, { 170,   0 },
 { 171,   0 }, { 172,   0 }, { 173,   0 }, { 174,   0 }, { 175,   0 },
 { 176,   0 }, { 177,   0 }, { 178,   0 }, { 179,   0 }, { 180,   0 },
 { 181,   0 }, { 182,   0 }, { 183,   0 }, { 184,   0 }, { 185,   0 },
 { 186,   0 }, { 187,   0 }, { 188,   0 }, { 189,   0 }, { 190,   0 },

 { 191,   0 }, { 192,   0 }, { 193,   0 }, { 194,   0 }, { 195,   0 },
 { 196,   0 }, { 197,   0 }, { 198,   0 }, { 199,   0 }, { 200,   0 },
 { 201,   0 }, { 202,   0 }, { 203,   0 }, { 204,   0 }, { 205,   0 },
 { 206,   0 }, { 207,   0 }, { 208,   0 }, { 209,   0 }, { 210,   0 },
 { 211,   0 }, { 212,   0 }, { 213,   0 }, { 214,   0 }, { 215,   0 },
 { 216,   0 }, { 217,   0 }, { 218,   0 }, { 219,   0 }, { 220,   0 },
 { 221,   0 }, { 222,   0 }, { 223,   0 }, { 224,   0 }, { 225,   0 },
 { 226,   0 }, { 227,   0 }, { 228,   0 }, { 229,   0 }, { 230,   0 },
 { 231,   0 }, { 232,   0 }, { 233,   0 }, { 234,   0 }, { 235,   0 },
 { 236,   0 }, { 237,   0 }, { 238,   0 }, { 239,   0 }, { 240,   0 },

 { 241,   0 }, { 242,   0 }, { 243,   0 }, { 244,   0 }, { 245,   0 },
 { 246,   0 }, { 247,   0 }, { 248,   0 }, { 249,   0 }, { 250,   0 },
 { 251,   0 }, { 252,   0 }, { 253,   0 }, { 254,   0 }, { 255,   0 },
 { 256,   0 }, {   0,  37 }, {   0,17058 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,  37 },
 {   0,17035 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  48,4382 }, {  49,4382 }, {  50,4382 }, {  51,4382 }, {  52,4382 },
 {  53,4382 }, {  54,4382 }, {  55,4382 }, {  56,4382 }, {  57,4382 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  65,4382 }, {  66,4382 }, {  67,4382 },
 {  68,4382 }, {  69,4382 }, {  70,4382 }, {  48,4382 }, {  49,4382 },
 {  50,4382 }, {  51,4382 }, {  52,4382 }, {  53,4382 }, {  54,4382 },
 {  55,4382 }, {  56,4382 }, {  57,4382 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  65,4382 }, {  66,4382 }, {  67,4382 }, {  68,4382 }, {  69,4382 },
 {  70,4382 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  97,4382 },
 {  98,4382 }, {  99,4382 }, { 100,4382 }, { 101,4382 }, { 102,4382 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  97,4382 }, {  98,4382 }, {  99,4382 },
 { 100,4382 }, { 101,4382 }, { 102,4382 }, {   0,   9 }, {   0,16931 },
 {   1,4382 }, {   2,4382 }, {   3,4382 }, {   4,4382 }, {   5,4382 },

 {   6,4382 }, {   7,4382 }, {   8,4382 }, {   9,4640 }, {  10,4898 },
 {  11,4382 }, {  12,4640 }, {  13,4898 }, {  14,4382 }, {  15,4382 },
 {  16,4382 }, {  17,4382 }, {  18,4382 }, {  19,4382 }, {  20,4382 },
 {  21,4382 }, {  22,4382 }, {  23,4382 }, {  24,4382 }, {  25,4382 },
 {  26,4382 }, {  27,4382 }, {  28,4382 }, {  29,4382 }, {  30,4382 },
 {  31,4382 }, {  32,4640 }, {  33,4382 }, {  34,4382 }, {  35,4382 },
 {  36,4382 }, {  37,4382 }, {  38,4382 }, {  39,4382 }, {  40,4382 },
 {  41,4382 }, {  42,4382 }, {  43,4382 }, {  44,4382 }, {  45,4945 },
 {  46,4382 }, {  47,4382 }, {  48,4382 }, {  49,4382 }, {  50,4382 },
 {  51,4382 }, {  52,4382 }, {  53,4382 }, {  54,4382 }, {  55,4382 },

 {  56,4382 }, {  57,4382 }, {  58,4382 }, {  59,4382 }, {  60,4382 },
 {  61,4382 }, {  62,4382 }, {  63,4382 }, {  64,4382 }, {  65,4382 },
 {  66,4382 }, {  67,4382 }, {  68,4382 }, {  69,4382 }, {  70,4382 },
 {  71,4382 }, {  72,4382 }, {  73,4382 }, {  74,4382 }, {  75,4382 },
 {  76,4382 }, {  77,4382 }, {  78,4382 }, {  79,4382 }, {  80,4382 },
 {  81,4382 }, {  82,4382 }, {  83,4382 }, {  84,4382 }, {  85,4382 },
 {  86,4382 }, {  87,4382 }, {  88,4382 }, {  89,4382 }, {  90,4382 },
 {  91,4382 }, {  92,4382 }, {  93,4382 }, {  94,4382 }, {  95,4382 },
 {  96,4382 }, {  97,4382 }, {  98,4382 }, {  99,4382 }, { 100,4382 },
 { 101,4382 }, { 102,4382 }, { 103,4382 }, { 104,4382 }, { 105,4382 },

 { 106,4382 }, { 107,4382 }, { 108,4382 }, { 109,4382 }, { 110,4382 },
 { 111,4382 }, { 112,4382 }, { 113,4382 }, { 114,4382 }, { 115,4382 },
 { 116,4382 }, { 117,4382 }, { 118,4382 }, { 119,4382 }, { 120,4382 },
 { 121,4382 }, { 122,4382 }, { 123,4382 }, { 124,4382 }, { 125,4382 },
 { 126,4382 }, { 127,4382 }, { 128,4382 }, { 129,4382 }, { 130,4382 },
 { 131,4382 }, { 132,4382 }, { 133,4382 }, { 134,4382 }, { 135,4382 },
 { 136,4382 }, { 137,4382 }, { 138,4382 }, { 139,4382 }, { 140,4382 },
 { 141,4382 }, { 142,4382 }, { 143,4382 }, { 144,4382 }, { 145,4382 },
 { 146,4382 }, { 147,4382 }, { 148,4382 }, { 149,4382 }, { 150,4382 },
 { 151,4382 }, { 152,4382 }, { 153,4382 }, { 154,4382 }, { 155,4382 },

 { 156,4382 }, { 157,4382 }, { 158,4382 }, { 159,4382 }, { 160,4382 },
 { 161,4382 }, { 162,4382 }, { 163,4382 }, { 164,4382 }, { 165,4382 },
 { 166,4382 }, { 167,4382 }, { 168,4382 }, { 169,4382 }, { 170,4382 },
 { 171,4382 }, { 172,4382 }, { 173,4382 }, { 174,4382 }, { 175,4382 },
 { 176,4382 }, { 177,4382 }, { 178,4382 }, { 179,4382 }, { 180,4382 },
 { 181,4382 }, { 182,4382 }, { 183,4382 }, { 184,4382 }, { 185,4382 },
 { 186,4382 }, { 187,4382 }, { 188,4382 }, { 189,4382 }, { 190,4382 },
 { 191,4382 }, { 192,4382 }, { 193,4382 }, { 194,4382 }, { 195,4382 },
 { 196,4382 }, { 197,4382 }, { 198,4382 }, { 199,4382 }, { 200,4382 },
 { 201,4382 }, { 202,4382 }, { 203,4382 }, { 204,4382 }, { 205,4382 },

 { 206,4382 }, { 207,4382 }, { 208,4382 }, { 209,4382 }, { 210,4382 },
 { 211,4382 }, { 212,4382 }, { 213,4382 }, { 214,4382 }, { 215,4382 },
 { 216,4382 }, { 217,4382 }, { 218,4382 }, { 219,4382 }, { 220,4382 },
 { 221,4382 }, { 222,4382 }, { 223,4382 }, { 224,4382 }, { 225,4382 },
 { 226,4382 }, { 227,4382 }, { 228,4382 }, { 229,4382 }, { 230,4382 },
 { 231,4382 }, { 232,4382 }, { 233,4382 }, { 234,4382 }, { 235,4382 },
 { 236,4382 }, { 237,4382 }, { 238,4382 }, { 239,4382 }, { 240,4382 },
 { 241,4382 }, { 242,4382 }, { 243,4382 }, { 244,4382 }, { 245,4382 },
 { 246,4382 }, { 247,4382 }, { 248,4382 }, { 249,4382 }, { 250,4382 },
 { 251,4382 }, { 252,4382 }, { 253,4382 }, { 254,4382 }, { 255,4382 },

 { 256,4382 }, {   0,   9 }, {   0,16673 }, {   1,   0 }, {   2,   0 },
 {   3,   0 }, {   4,   0 }, {   5,   0 }, {   6,   0 }, {   7,   0 },
 {   8,   0 }, {   9, 258 }, {  10,-6406 }, {  11,   0 }, {  12, 258 },
 {  13,-6406 }, {  14,   0 }, {  15,   0 }, {  16,   0 }, {  17,   0 },
 {  18,   0 }, {  19,   0 }, {  20,   0 }, {  21,   0 }, {  22,   0 },
 {  23,   0 }, {  24,   0 }, {  25,   0 }, {  26,   0 }, {  27,   0 },
 {  28,   0 }, {  29,   0 }, {  30,   0 }, {  31,   0 }, {  32, 258 },
 {  33,   0 }, {  34,   0 }, {  35,   0 }, {  36,   0 }, {  37,   0 },
 {  38,   0 }, {  39,   0 }, {  40,   0 }, {  41,   0 }, {  42,   0 },
 {  43,   0 }, {  44,   0 }, {  45, 516 }, {  46,   0 }, {  47,   0 },

 {  48,   0 }, {  49,   0 }, {  50,   0 }, {  51,   0 }, {  52,   0 },
 {  53,   0 }, {  54,   0 }, {  55,   0 }, {  56,   0 }, {  57,   0 },
 {  58,   0 }, {  59,   0 }, {  60,   0 }, {  61,   0 }, {  62,   0 },
 {  63,   0 }, {  64,   0 }, {  65,   0 }, {  66,   0 }, {  67,   0 },
 {  68,   0 }, {  69,   0 }, {  70,   0 }, {  71,   0 }, {  72,   0 },
 {  73,   0 }, {  74,   0 }, {  75,   0 }, {  76,   0 }, {  77,   0 },
 {  78,   0 }, {  79,   0 }, {  80,   0 }, {  81,   0 }, {  82,   0 },
 {  83,   0 }, {  84,   0 }, {  85,   0 }, {  86,   0 }, {  87,   0 },
 {  88,   0 }, {  89,   0 }, {  90,   0 }, {  91,   0 }, {  92,   0 },
 {  93,   0 }, {  94,   0 }, {  95,   0 }, {  96,   0 }, {  97,   0 },

 {  98,   0 }, {  99,   0 }, { 100,   0 }, { 101,   0 }, { 102,   0 },
 { 103,   0 }, { 104,   0 }, { 105,   0 }, { 106,   0 }, { 107,   0 },
 { 108,   0 }, { 109,   0 }, { 110,   0 }, { 111,   0 }, { 112,   0 },
 { 113,   0 }, { 114,   0 }, { 115,   0 }, { 116,   0 }, { 117,   0 },
 { 118,   0 }, { 119,   0 }, { 120,   0 }, { 121,   0 }, { 122,   0 },
 { 123,   0 }, { 124,   0 }, { 125,   0 }, { 126,   0 }, { 127,   0 },
 { 128,   0 }, { 129,   0 }, { 130,   0 }, { 131,   0 }, { 132,   0 },
 { 133,   0 }, { 134,   0 }, { 135,   0 }, { 136,   0 }, { 137,   0 },
 { 138,   0 }, { 139,   0 }, { 140,   0 }, { 141,   0 }, { 142,   0 },
 { 143,   0 }, { 144,   0 }, { 145,   0 }, { 146,   0 }, { 147,   0 },

 { 148,   0 }, { 149,   0 }, { 150,   0 }, { 151,   0 }, { 152,   0 },
 { 153,   0 }, { 154,   0 }, { 155,   0 }, { 156,   0 }, { 157,   0 },
 { 158,   0 }, { 159,   0 }, { 160,   0 }, { 161,   0 }, { 162,   0 },
 { 163,   0 }, { 164,   0 }, { 165,   0 }, { 166,   0 }, { 167,   0 },
 { 168,   0 }, { 169,   0 }, { 170,   0 }, { 171,   0 }, { 172,   0 },
 { 173,   0 }, { 174,   0 }, { 175,   0 }, { 176,   0 }, { 177,   0 },
 { 178,   0 }, { 179,   0 }, { 180,   0 }, { 181,   0 }, { 182,   0 },
 { 183,   0 }, { 184,   0 }, { 185,   0 }, { 186,   0 }, { 187,   0 },
 { 188,   0 }, { 189,   0 }, { 190,   0 }, { 191,   0 }, { 192,   0 },
 { 193,   0 }, { 194,   0 }, { 195,   0 }, { 196,   0 }, { 197,   0 },

 { 198,   0 }, { 199,   0 }, { 200,   0 }, { 201,   0 }, { 202,   0 },
 { 203,   0 }, { 204,   0 }, { 205,   0 }, { 206,   0 }, { 207,   0 },
 { 208,   0 }, { 209,   0 }, { 210,   0 }, { 211,   0 }, { 212,   0 },
 { 213,   0 }, { 214,   0 }, { 215,   0 }, { 216,   0 }, { 217,   0 },
 { 218,   0 }, { 219,   0 }, { 220,   0 }, { 221,   0 }, { 222,   0 },
 { 223,   0 }, { 224,   0 }, { 225,   0 }, { 226,   0 }, { 227,   0 },
 { 228,   0 }, { 229,   0 }, { 230,   0 }, { 231,   0 }, { 232,   0 },
 { 233,   0 }, { 234,   0 }, { 235,   0 }, { 236,   0 }, { 237,   0 },
 { 238,   0 }, { 239,   0 }, { 240,   0 }, { 241,   0 }, { 242,   0 },
 { 243,   0 }, { 244,   0 }, { 245,   0 }, { 246,   0 }, { 247,   0 },

 { 248,   0 }, { 249,   0 }, { 250,   0 }, { 251,   0 }, { 252,   0 },
 { 253,   0 }, { 254,   0 }, { 255,   0 }, { 256,   0 }, {   0,   9 },
 {   0,16415 }, {   1,-258 }, {   2,-258 }, {   3,-258 }, {   4,-258 },
 {   5,-258 }, {   6,-258 }, {   7,-258 }, {   8,-258 }, {   9,   0 },
 {  10,-6664 }, {  11,-258 }, {  12,   0 }, {  13,-6664 }, {  14,-258 },
 {  15,-258 }, {  16,-258 }, {  17,-258 }, {  18,-258 }, {  19,-258 },
 {  20,-258 }, {  21,-258 }, {  22,-258 }, {  23,-258 }, {  24,-258 },
 {  25,-258 }, {  26,-258 }, {  27,-258 }, {  28,-258 }, {  29,-258 },
 {  30,-258 }, {  31,-258 }, {  32,   0 }, {  33,-258 }, {  34,-258 },
 {  35,-258 }, {  36,-258 }, {  37,-258 }, {  38,-258 }, {  39,-258 },

 {  40,-258 }, {  41,-258 }, {  42,-258 }, {  43,-258 }, {  44,-258 },
 {  45, 258 }, {  46,-258 }, {  47,-258 }, {  48,-258 }, {  49,-258 },
 {  50,-258 }, {  51,-258 }, {  52,-258 }, {  53,-258 }, {  54,-258 },
 {  55,-258 }, {  56,-258 }, {  57,-258 }, {  58,-258 }, {  59,-258 },
 {  60,-258 }, {  61,-258 }, {  62,-258 }, {  63,-258 }, {  64,-258 },
 {  65,-258 }, {  66,-258 }, {  67,-258 }, {  68,-258 }, {  69,-258 },
 {  70,-258 }, {  71,-258 }, {  72,-258 }, {  73,-258 }, {  74,-258 },
 {  75,-258 }, {  76,-258 }, {  77,-258 }, {  78,-258 }, {  79,-258 },
 {  80,-258 }, {  81,-258 }, {  82,-258 }, {  83,-258 }, {  84,-258 },
 {  85,-258 }, {  86,-258 }, {  87,-258 }, {  88,-258 }, {  89,-258 },

 {  90,-258 }, {  91,-258 }, {  92,-258 }, {  93,-258 }, {  94,-258 },
 {  95,-258 }, {  96,-258 }, {  97,-258 }, {  98,-258 }, {  99,-258 },
 { 100,-258 }, { 101,-258 }, { 102,-258 }, { 103,-258 }, { 104,-258 },
 { 105,-258 }, { 106,-258 }, { 107,-258 }, { 108,-258 }, { 109,-258 },
 { 110,-258 }, { 111,-258 }, { 112,-258 }, { 113,-258 }, { 114,-258 },
 { 115,-258 }, { 116,-258 }, { 117,-258 }, { 118,-258 }, { 119,-258 },
 { 120,-258 }, { 121,-258 }, { 122,-258 }, { 123,-258 }, { 124,-258 },
 { 125,-258 }, { 126,-258 }, { 127,-258 }, { 128,-258 }, { 129,-258 },
 { 130,-258 }, { 131,-258 }, { 132,-258 }, { 133,-258 }, { 134,-258 },
 { 135,-258 }, { 136,-258 }, { 137,-258 }, { 138,-258 }, { 139,-258 },

 { 140,-258 }, { 141,-258 }, { 142,-258 }, { 143,-258 }, { 144,-258 },
 { 145,-258 }, { 146,-258 }, { 147,-258 }, { 148,-258 }, { 149,-258 },
 { 150,-258 }, { 151,-258 }, { 152,-258 }, { 153,-258 }, { 154,-258 },
 { 155,-258 }, { 156,-258 }, { 157,-258 }, { 158,-258 }, { 159,-258 },
 { 160,-258 }, { 161,-258 }, { 162,-258 }, { 163,-258 }, { 164,-258 },
 { 165,-258 }, { 166,-258 }, { 167,-258 }, { 168,-258 }, { 169,-258 },
 { 170,-258 }, { 171,-258 }, { 172,-258 }, { 173,-258 }, { 174,-258 },
 { 175,-258 }, { 176,-258 }, { 177,-258 }, { 178,-258 }, { 179,-258 },
 { 180,-258 }, { 181,-258 }, { 182,-258 }, { 183,-258 }, { 184,-258 },
 { 185,-258 }, { 186,-258 }, { 187,-258 }, { 188,-258 }, { 189,-258 },

 { 190,-258 }, { 191,-258 }, { 192,-258 }, { 193,-258 }, { 194,-258 },
 { 195,-258 }, { 196,-258 }, { 197,-258 }, { 198,-258 }, { 199,-258 },
 { 200,-258 }, { 201,-258 }, { 202,-258 }, { 203,-258 }, { 204,-258 },
 { 205,-258 }, { 206,-258 }, { 207,-258 }, { 208,-258 }, { 209,-258 },
 { 210,-258 }, { 211,-258 }, { 212,-258 }, { 213,-258 }, { 214,-258 },
 { 215,-258 }, { 216,-258 }, { 217,-258 }, { 218,-258 }, { 219,-258 },
 { 220,-258 }, { 221,-258 }, { 222,-258 }, { 223,-258 }, { 224,-258 },
 { 225,-258 }, { 226,-258 }, { 227,-258 }, { 228,-258 }, { 229,-258 },
 { 230,-258 }, { 231,-258 }, { 232,-258 }, { 233,-258 }, { 234,-258 },
 { 235,-258 }, { 236,-258 }, { 237,-258 }, { 238,-258 }, { 239,-258 },

 { 240,-258 }, { 241,-258 }, { 242,-258 }, { 243,-258 }, { 244,-258 },
 { 245,-258 }, { 246,-258 }, { 247,-258 }, { 248,-258 }, { 249,-258 },
 { 250,-258 }, { 251,-258 }, { 252,-258 }, { 253,-258 }, { 254,-258 },
 { 255,-258 }, { 256,-258 }, {   0,   9 }, {   0,16157 }, {   1,-516 },
 {   2,-516 }, {   3,-516 }, {   4,-516 }, {   5,-516 }, {   6,-516 },
 {   7,-516 }, {   8,-516 }, {   9,-258 }, {  10,-6922 }, {  11,-516 },
 {  12,-258 }, {  13,-6922 }, {  14,-516 }, {  15,-516 }, {  16,-516 },
 {  17,-516 }, {  18,-516 }, {  19,-516 }, {  20,-516 }, {  21,-516 },
 {  22,-516 }, {  23,-516 }, {  24,-516 }, {  25,-516 }, {  26,-516 },
 {  27,-516 }, {  28,-516 }, {  29,-516 }, {  30,-516 }, {  31,-516 },

 {  32,-258 }, {  33,-516 }, {  34,-516 }, {  35,-516 }, {  36,-516 },
 {  37,-516 }, {  38,-516 }, {  39,-516 }, {  40,-516 }, {  41,-516 },
 {  42,-516 }, {  43,-516 }, {  44,-516 }, {  45,4429 }, {  46,-516 },
 {  47,-516 }, {  48,-516 }, {  49,-516 }, {  50,-516 }, {  51,-516 },
 {  52,-516 }, {  53,-516 }, {  54,-516 }, {  55,-516 }, {  56,-516 },
 {  57,-516 }, {  58,-516 }, {  59,-516 }, {  60,-516 }, {  61,-516 },
 {  62,-516 }, {  63,-516 }, {  64,-516 }, {  65,-516 }, {  66,-516 },
 {  67,-516 }, {  68,-516 }, {  69,-516 }, {  70,-516 }, {  71,-516 },
 {  72,-516 }, {  73,-516 }, {  74,-516 }, {  75,-516 }, {  76,-516 },
 {  77,-516 }, {  78,-516 }, {  79,-516 }, {  80,-516 }, {  81,-516 },

 {  82,-516 }, {  83,-516 }, {  84,-516 }, {  85,-516 }, {  86,-516 },
 {  87,-516 }, {  88,-516 }, {  89,-516 }, {  90,-516 }, {  91,-516 },
 {  92,-516 }, {  93,-516 }, {  94,-516 }, {  95,-516 }, {  96,-516 },
 {  97,-516 }, {  98,-516 }, {  99,-516 }, { 100,-516 }, { 101,-516 },
 { 102,-516 }, { 103,-516 }, { 104,-516 }, { 105,-516 }, { 106,-516 },
 { 107,-516 }, { 108,-516 }, { 109,-516 }, { 110,-516 }, { 111,-516 },
 { 112,-516 }, { 113,-516 }, { 114,-516 }, { 115,-516 }, { 116,-516 },
 { 117,-516 }, { 118,-516 }, { 119,-516 }, { 120,-516 }, { 121,-516 },
 { 122,-516 }, { 123,-516 }, { 124,-516 }, { 125,-516 }, { 126,-516 },
 { 127,-516 }, { 128,-516 }, { 129,-516 }, { 130,-516 }, { 131,-516 },

 { 132,-516 }, { 133,-516 }, { 134,-516 }, { 135,-516 }, { 136,-516 },
 { 137,-516 }, { 138,-516 }, { 139,-516 }, { 140,-516 }, { 141,-516 },
 { 142,-516 }, { 143,-516 }, { 144,-516 }, { 145,-516 }, { 146,-516 },
 { 147,-516 }, { 148,-516 }, { 149,-516 }, { 150,-516 }, { 151,-516 },
 { 152,-516 }, { 153,-516 }, { 154,-516 }, { 155,-516 }, { 156,-516 },
 { 157,-516 }, { 158,-516 }, { 159,-516 }, { 160,-516 }, { 161,-516 },
 { 162,-516 }, { 163,-516 }, { 164,-516 }, { 165,-516 }, { 166,-516 },
 { 167,-516 }, { 168,-516 }, { 169,-516 }, { 170,-516 }, { 171,-516 },
 { 172,-516 }, { 173,-516 }, { 174,-516 }, { 175,-516 }, { 176,-516 },
 { 177,-516 }, { 178,-516 }, { 179,-516 }, { 180,-516 }, { 181,-516 },

 { 182,-516 }, { 183,-516 }, { 184,-516 }, { 185,-516 }, { 186,-516 },
 { 187,-516 }, { 188,-516 }, { 189,-516 }, { 190,-516 }, { 191,-516 },
 { 192,-516 }, { 193,-516 }, { 194,-516 }, { 195,-516 }, { 196,-516 },
 { 197,-516 }, { 198,-516 }, { 199,-516 }, { 200,-516 }, { 201,-516 },
 { 202,-516 }, { 203,-516 }, { 204,-516 }, { 205,-516 }, { 206,-516 },
 { 207,-516 }, { 208,-516 }, { 209,-516 }, { 210,-516 }, { 211,-516 },
 { 212,-516 }, { 213,-516 }, { 214,-516 }, { 215,-516 }, { 216,-516 },
 { 217,-516 }, { 218,-516 }, { 219,-516 }, { 220,-516 }, { 221,-516 },
 { 222,-516 }, { 223,-516 }, { 224,-516 }, { 225,-516 }, { 226,-516 },
 { 227,-516 }, { 228,-516 }, { 229,-516 }, { 230,-516 }, { 231,-516 },

 { 232,-516 }, { 233,-516 }, { 234,-516 }, { 235,-516 }, { 236,-516 },
 { 237,-516 }, { 238,-516 }, { 239,-516 }, { 240,-516 }, { 241,-516 },
 { 242,-516 }, { 243,-516 }, { 244,-516 }, { 245,-516 }, { 246,-516 },
 { 247,-516 }, { 248,-516 }, { 249,-516 }, { 250,-516 }, { 251,-516 },
 { 252,-516 }, { 253,-516 }, { 254,-516 }, { 255,-516 }, { 256,-516 },
 {   0,  16 }, {   0,15899 }, {   1,4429 }, {   2,4429 }, {   3,4429 },
 {   4,4429 }, {   5,4429 }, {   6,4429 }, {   7,4429 }, {   8,4429 },
 {   9,4687 }, {  10,4945 }, {  11,4429 }, {  12,4687 }, {  13,4945 },
 {  14,4429 }, {  15,4429 }, {  16,4429 }, {  17,4429 }, {  18,4429 },
 {  19,4429 }, {  20,4429 }, {  21,4429 }, {  22,4429 }, {  23,4429 },

 {  24,4429 }, {  25,4429 }, {  26,4429 }, {  27,4429 }, {  28,4429 },
 {  29,4429 }, {  30,4429 }, {  31,4429 }, {  32,4687 }, {  33,4429 },
 {  34,4429 }, {  35,4429 }, {  36,4429 }, {  37,4429 }, {  38,4429 },
 {  39,4429 }, {  40,4429 }, {  41,4429 }, {  42,4429 }, {  43,4429 },
 {  44,4429 }, {  45,4992 }, {  46,4429 }, {  47,4429 }, {  48,4429 },
 {  49,4429 }, {  50,4429 }, {  51,4429 }, {  52,4429 }, {  53,4429 },
 {  54,4429 }, {  55,4429 }, {  56,4429 }, {  57,4429 }, {  58,4429 },
 {  59,4429 }, {  60,4429 }, {  61,4429 }, {  62,4429 }, {  63,4429 },
 {  64,4429 }, {  65,4429 }, {  66,4429 }, {  67,4429 }, {  68,4429 },
 {  69,4429 }, {  70,4429 }, {  71,4429 }, {  72,4429 }, {  73,4429 },

 {  74,4429 }, {  75,4429 }, {  76,4429 }, {  77,4429 }, {  78,4429 },
 {  79,4429 }, {  80,4429 }, {  81,4429 }, {  82,4429 }, {  83,4429 },
 {  84,4429 }, {  85,4429 }, {  86,4429 }, {  87,4429 }, {  88,4429 },
 {  89,4429 }, {  90,4429 }, {  91,4429 }, {  92,4429 }, {  93,4429 },
 {  94,4429 }, {  95,4429 }, {  96,4429 }, {  97,4429 }, {  98,4429 },
 {  99,4429 }, { 100,4429 }, { 101,4429 }, { 102,4429 }, { 103,4429 },
 { 104,4429 }, { 105,4429 }, { 106,4429 }, { 107,4429 }, { 108,4429 },
 { 109,4429 }, { 110,4429 }, { 111,4429 }, { 112,4429 }, { 113,4429 },
 { 114,4429 }, { 115,4429 }, { 116,4429 }, { 117,4429 }, { 118,4429 },
 { 119,4429 }, { 120,4429 }, { 121,4429 }, { 122,4429 }, { 123,4429 },

 { 124,4429 }, { 125,4429 }, { 126,4429 }, { 127,4429 }, { 128,4429 },
 { 129,4429 }, { 130,4429 }, { 131,4429 }, { 132,4429 }, { 133,4429 },
 { 134,4429 }, { 135,4429 }, { 136,4429 }, { 137,4429 }, { 138,4429 },
 { 139,4429 }, { 140,4429 }, { 141,4429 }, { 142,4429 }, { 143,4429 },
 { 144,4429 }, { 145,4429 }, { 146,4429 }, { 147,4429 }, { 148,4429 },
 { 149,4429 }, { 150,4429 }, { 151,4429 }, { 152,4429 }, { 153,4429 },
 { 154,4429 }, { 155,4429 }, { 156,4429 }, { 157,4429 }, { 158,4429 },
 { 159,4429 }, { 160,4429 }, { 161,4429 }, { 162,4429 }, { 163,4429 },
 { 164,4429 }, { 165,4429 }, { 166,4429 }, { 167,4429 }, { 168,4429 },
 { 169,4429 }, { 170,4429 }, { 171,4429 }, { 172,4429 }, { 173,4429 },

 { 174,4429 }, { 175,4429 }, { 176,4429 }, { 177,4429 }, { 178,4429 },
 { 179,4429 }, { 180,4429 }, { 181,4429 }, { 182,4429 }, { 183,4429 },
 { 184,4429 }, { 185,4429 }, { 186,4429 }, { 187,4429 }, { 188,4429 },
 { 189,4429 }, { 190,4429 }, { 191,4429 }, { 192,4429 }, { 193,4429 },
 { 194,4429 }, { 195,4429 }, { 196,4429 }, { 197,4429 }, { 198,4429 },
 { 199,4429 }, { 200,4429 }, { 201,4429 }, { 202,4429 }, { 203,4429 },
 { 204,4429 }, { 205,4429 }, { 206,4429 }, { 207,4429 }, { 208,4429 },
 { 209,4429 }, { 210,4429 }, { 211,4429 }, { 212,4429 }, { 213,4429 },
 { 214,4429 }, { 215,4429 }, { 216,4429 }, { 217,4429 }, { 218,4429 },
 { 219,4429 }, { 220,4429 }, { 221,4429 }, { 222,4429 }, { 223,4429 },

 { 224,4429 }, { 225,4429 }, { 226,4429 }, { 227,4429 }, { 228,4429 },
 { 229,4429 }, { 230,4429 }, { 231,4429 }, { 232,4429 }, { 233,4429 },
 { 234,4429 }, { 235,4429 }, { 236,4429 }, { 237,4429 }, { 238,4429 },
 { 239,4429 }, { 240,4429 }, { 241,4429 }, { 242,4429 }, { 243,4429 },
 { 244,4429 }, { 245,4429 }, { 246,4429 }, { 247,4429 }, { 248,4429 },
 { 249,4429 }, { 250,4429 }, { 251,4429 }, { 252,4429 }, { 253,4429 },
 { 254,4429 }, { 255,4429 }, { 256,4429 }, {   0,  16 }, {   0,15641 },
 {   1,   0 }, {   2,   0 }, {   3,   0 }, {   4,   0 }, {   5,   0 },
 {   6,   0 }, {   7,   0 }, {   8,   0 }, {   9, 258 }, {  10,-6484 },
 {  11,   0 }, {  12, 258 }, {  13,-6484 }, {  14,   0 }, {  15,   0 },

 {  16,   0 }, {  17,   0 }, {  18,   0 }, {  19,   0 }, {  20,   0 },
 {  21,   0 }, {  22,   0 }, {  23,   0 }, {  24,   0 }, {  25,   0 },
 {  26,   0 }, {  27,   0 }, {  28,   0 }, {  29,   0 }, {  30,   0 },
 {  31,   0 }, {  32, 258 }, {  33,   0 }, {  34,   0 }, {  35,   0 },
 {  36,   0 }, {  37,   0 }, {  38,   0 }, {  39,   0 }, {  40,   0 },
 {  41,   0 }, {  42,   0 }, {  43,   0 }, {  44,   0 }, {  45, 516 },
 {  46,   0 }, {  47,   0 }, {  48,   0 }, {  49,   0 }, {  50,   0 },
 {  51,   0 }, {  52,   0 }, {  53,   0 }, {  54,   0 }, {  55,   0 },
 {  56,   0 }, {  57,   0 }, {  58,   0 }, {  59,   0 }, {  60,   0 },
 {  61,   0 }, {  62,   0 }, {  63,   0 }, {  64,   0 }, {  65,   0 },

 {  66,   0 }, {  67,   0 }, {  68,   0 }, {  69,   0 }, {  70,   0 },
 {  71,   0 }, {  72,   0 }, {  73,   0 }, {  74,   0 }, {  75,   0 },
 {  76,   0 }, {  77,   0 }, {  78,   0 }, {  79,   0 }, {  80,   0 },
 {  81,   0 }, {  82,   0 }, {  83,   0 }, {  84,   0 }, {  85,   0 },
 {  86,   0 }, {  87,   0 }, {  88,   0 }, {  89,   0 }, {  90,   0 },
 {  91,   0 }, {  92,   0 }, {  93,   0 }, {  94,   0 }, {  95,   0 },
 {  96,   0 }, {  97,   0 }, {  98,   0 }, {  99,   0 }, { 100,   0 },
 { 101,   0 }, { 102,   0 }, { 103,   0 }, { 104,   0 }, { 105,   0 },
 { 106,   0 }, { 107,   0 }, { 108,   0 }, { 109,   0 }, { 110,   0 },
 { 111,   0 }, { 112,   0 }, { 113,   0 }, { 114,   0 }, { 115,   0 },

 { 116,   0 }, { 117,   0 }, { 118,   0 }, { 119,   0 }, { 120,   0 },
 { 121,   0 }, { 122,   0 }, { 123,   0 }, { 124,   0 }, { 125,   0 },
 { 126,   0 }, { 127,   0 }, { 128,   0 }, { 129,   0 }, { 130,   0 },
 { 131,   0 }, { 132,   0 }, { 133,   0 }, { 134,   0 }, { 135,   0 },
 { 136,   0 }, { 137,   0 }, { 138,   0 }, { 139,   0 }, { 140,   0 },
 { 141,   0 }, { 142,   0 }, { 143,   0 }, { 144,   0 }, { 145,   0 },
 { 146,   0 }, { 147,   0 }, { 148,   0 }, { 149,   0 }, { 150,   0 },
 { 151,   0 }, { 152,   0 }, { 153,   0 }, { 154,   0 }, { 155,   0 },
 { 156,   0 }, { 157,   0 }, { 158,   0 }, { 159,   0 }, { 160,   0 },
 { 161,   0 }, { 162,   0 }, { 163,   0 }, { 164,   0 }, { 165,   0 },

 { 166,   0 }, { 167,   0 }, { 168,   0 }, { 169,   0 }, { 170,   0 },
 { 171,   0 }, { 172,   0 }, { 173,   0 }, { 174,   0 }, { 175,   0 },
 { 176,   0 }, { 177,   0 }, { 178,   0 }, { 179,   0 }, { 180,   0 },
 { 181,   0 }, { 182,   0 }, { 183,   0 }, { 184,   0 }, { 185,   0 },
 { 186,   0 }, { 187,   0 }, { 188,   0 }, { 189,   0 }, { 190,   0 },
 { 191,   0 }, { 192,   0 }, { 193,   0 }, { 194,   0 }, { 195,   0 },
 { 196,   0 }, { 197,   0 }, { 198,   0 }, { 199,   0 }, { 200,   0 },
 { 201,   0 }, { 202,   0 }, { 203,   0 }, { 204,   0 }, { 205,   0 },
 { 206,   0 }, { 207,   0 }, { 208,   0 }, { 209,   0 }, { 210,   0 },
 { 211,   0 }, { 212,   0 }, { 213,   0 }, { 214,   0 }, { 215,   0 },

 { 216,   0 }, { 217,   0 }, { 218,   0 }, { 219,   0 }, { 220,   0 },
 { 221,   0 }, { 222,   0 }, { 223,   0 }, { 224,   0 }, { 225,   0 },
 { 226,   0 }, { 227,   0 }, { 228,   0 }, { 229,   0 }, { 230,   0 },
 { 231,   0 }, { 232,   0 }, { 233,   0 }, { 234,   0 }, { 235,   0 },
 { 236,   0 }, { 237,   0 }, { 238,   0 }, { 239,   0 }, { 240,   0 },
 { 241,   0 }, { 242,   0 }, { 243,   0 }, { 244,   0 }, { 245,   0 },
 { 246,   0 }, { 247,   0 }, { 248,   0 }, { 249,   0 }, { 250,   0 },
 { 251,   0 }, { 252,   0 }, { 253,   0 }, { 254,   0 }, { 255,   0 },
 { 256,   0 }, {   0,  16 }, {   0,15383 }, {   1,-258 }, {   2,-258 },
 {   3,-258 }, {   4,-258 }, {   5,-258 }, {   6,-258 }, {   7,-258 },

 {   8,-258 }, {   9,   0 }, {  10,-6742 }, {  11,-258 }, {  12,   0 },
 {  13,-6742 }, {  14,-258 }, {  15,-258 }, {  16,-258 }, {  17,-258 },
 {  18,-258 }, {  19,-258 }, {  20,-258 }, {  21,-258 }, {  22,-258 },
 {  23,-258 }, {  24,-258 }, {  25,-258 }, {  26,-258 }, {  27,-258 },
 {  28,-258 }, {  29,-258 }, {  30,-258 }, {  31,-258 }, {  32,   0 },
 {  33,-258 }, {  34,-258 }, {  35,-258 }, {  36,-258 }, {  37,-258 },
 {  38,-258 }, {  39,-258 }, {  40,-258 }, {  41,-258 }, {  42,-258 },
 {  43,-258 }, {  44,-258 }, {  45, 258 }, {  46,-258 }, {  47,-258 },
 {  48,-258 }, {  49,-258 }, {  50,-258 }, {  51,-258 }, {  52,-258 },
 {  53,-258 }, {  54,-258 }, {  55,-258 }, {  56,-258 }, {  57,-258 },

 {  58,-258 }, {  59,-258 }, {  60,-258 }, {  61,-258 }, {  62,-258 },
 {  63,-258 }, {  64,-258 }, {  65,-258 }, {  66,-258 }, {  67,-258 },
 {  68,-258 }, {  69,-258 }, {  70,-258 }, {  71,-258 }, {  72,-258 },
 {  73,-258 }, {  74,-258 }, {  75,-258 }, {  76,-258 }, {  77,-258 },
 {  78,-258 }, {  79,-258 }, {  80,-258 }, {  81,-258 }, {  82,-258 },
 {  83,-258 }, {  84,-258 }, {  85,-258 }, {  86,-258 }, {  87,-258 },
 {  88,-258 }, {  89,-258 }, {  90,-258 }, {  91,-258 }, {  92,-258 },
 {  93,-258 }, {  94,-258 }, {  95,-258 }, {  96,-258 }, {  97,-258 },
 {  98,-258 }, {  99,-258 }, { 100,-258 }, { 101,-258 }, { 102,-258 },
 { 103,-258 }, { 104,-258 }, { 105,-258 }, { 106,-258 }, { 107,-258 },

 { 108,-258 }, { 109,-258 }, { 110,-258 }, { 111,-258 }, { 112,-258 },
 { 113,-258 }, { 114,-258 }, { 115,-258 }, { 116,-258 }, { 117,-258 },
 { 118,-258 }, { 119,-258 }, { 120,-258 }, { 121,-258 }, { 122,-258 },
 { 123,-258 }, { 124,-258 }, { 125,-258 }, { 126,-258 }, { 127,-258 },
 { 128,-258 }, { 129,-258 }, { 130,-258 }, { 131,-258 }, { 132,-258 },
 { 133,-258 }, { 134,-258 }, { 135,-258 }, { 136,-258 }, { 137,-258 },
 { 138,-258 }, { 139,-258 }, { 140,-258 }, { 141,-258 }, { 142,-258 },
 { 143,-258 }, { 144,-258 }, { 145,-258 }, { 146,-258 }, { 147,-258 },
 { 148,-258 }, { 149,-258 }, { 150,-258 }, { 151,-258 }, { 152,-258 },
 { 153,-258 }, { 154,-258 }, { 155,-258 }, { 156,-258 }, { 157,-258 },

 { 158,-258 }, { 159,-258 }, { 160,-258 }, { 161,-258 }, { 162,-258 },
 { 163,-258 }, { 164,-258 }, { 165,-258 }, { 166,-258 }, { 167,-258 },
 { 168,-258 }, { 169,-258 }, { 170,-258 }, { 171,-258 }, { 172,-258 },
 { 173,-258 }, { 174,-258 }, { 175,-258 }, { 176,-258 }, { 177,-258 },
 { 178,-258 }, { 179,-258 }, { 180,-258 }, { 181,-258 }, { 182,-258 },
 { 183,-258 }, { 184,-258 }, { 185,-258 }, { 186,-258 }, { 187,-258 },
 { 188,-258 }, { 189,-258 }, { 190,-258 }, { 191,-258 }, { 192,-258 },
 { 193,-258 }, { 194,-258 }, { 195,-258 }, { 196,-258 }, { 197,-258 },
 { 198,-258 }, { 199,-258 }, { 200,-258 }, { 201,-258 }, { 202,-258 },
 { 203,-258 }, { 204,-258 }, { 205,-258 }, { 206,-258 }, { 207,-258 },

 { 208,-258 }, { 209,-258 }, { 210,-258 }, { 211,-258 }, { 212,-258 },
 { 213,-258 }, { 214,-258 }, { 215,-258 }, { 216,-258 }, { 217,-258 },
 { 218,-258 }, { 219,-258 }, { 220,-258 }, { 221,-258 }, { 222,-258 },
 { 223,-258 }, { 224,-258 }, { 225,-258 }, { 226,-258 }, { 227,-258 },
 { 228,-258 }, { 229,-258 }, { 230,-258 }, { 231,-258 }, { 232,-258 },
 { 233,-258 }, { 234,-258 }, { 235,-258 }, { 236,-258 }, { 237,-258 },
 { 238,-258 }, { 239,-258 }, { 240,-258 }, { 241,-258 }, { 242,-258 },
 { 243,-258 }, { 244,-258 }, { 245,-258 }, { 246,-258 }, { 247,-258 },
 { 248,-258 }, { 249,-258 }, { 250,-258 }, { 251,-258 }, { 252,-258 },
 { 253,-258 }, { 254,-258 }, { 255,-258 }, { 256,-258 }, {   0,  16 },

 {   0,15125 }, {   1,-516 }, {   2,-516 }, {   3,-516 }, {   4,-516 },
 {   5,-516 }, {   6,-516 }, {   7,-516 }, {   8,-516 }, {   9,-258 },
 {  10,-7000 }, {  11,-516 }, {  12,-258 }, {  13,-7000 }, {  14,-516 },
 {  15,-516 }, {  16,-516 }, {  17,-516 }, {  18,-516 }, {  19,-516 },
 {  20,-516 }, {  21,-516 }, {  22,-516 }, {  23,-516 }, {  24,-516 },
 {  25,-516 }, {  26,-516 }, {  27,-516 }, {  28,-516 }, {  29,-516 },
 {  30,-516 }, {  31,-516 }, {  32,-258 }, {  33,-516 }, {  34,-516 },
 {  35,-516 }, {  36,-516 }, {  37,-516 }, {  38,-516 }, {  39,-516 },
 {  40,-516 }, {  41,-516 }, {  42,-516 }, {  43,-516 }, {  44,-516 },
 {  45,4476 }, {  46,-516 }, {  47,-516 }, {  48,-516 }, {  49,-516 },

 {  50,-516 }, {  51,-516 }, {  52,-516 }, {  53,-516 }, {  54,-516 },
 {  55,-516 }, {  56,-516 }, {  57,-516 }, {  58,-516 }, {  59,-516 },
 {  60,-516 }, {  61,-516 }, {  62,-516 }, {  63,-516 }, {  64,-516 },
 {  65,-516 }, {  66,-516 }, {  67,-516 }, {  68,-516 }, {  69,-516 },
 {  70,-516 }, {  71,-516 }, {  72,-516 }, {  73,-516 }, {  74,-516 },
 {  75,-516 }, {  76,-516 }, {  77,-516 }, {  78,-516 }, {  79,-516 },
 {  80,-516 }, {  81,-516 }, {  82,-516 }, {  83,-516 }, {  84,-516 },
 {  85,-516 }, {  86,-516 }, {  87,-516 }, {  88,-516 }, {  89,-516 },
 {  90,-516 }, {  91,-516 }, {  92,-516 }, {  93,-516 }, {  94,-516 },
 {  95,-516 }, {  96,-516 }, {  97,-516 }, {  98,-516 }, {  99,-516 },

 { 100,-516 }, { 101,-516 }, { 102,-516 }, { 103,-516 }, { 104,-516 },
 { 105,-516 }, { 106,-516 }, { 107,-516 }, { 108,-516 }, { 109,-516 },
 { 110,-516 }, { 111,-516 }, { 112,-516 }, { 113,-516 }, { 114,-516 },
 { 115,-516 }, { 116,-516 }, { 117,-516 }, { 118,-516 }, { 119,-516 },
 { 120,-516 }, { 121,-516 }, { 122,-516 }, { 123,-516 }, { 124,-516 },
 { 125,-516 }, { 126,-516 }, { 127,-516 }, { 128,-516 }, { 129,-516 },
 { 130,-516 }, { 131,-516 }, { 132,-516 }, { 133,-516 }, { 134,-516 },
 { 135,-516 }, { 136,-516 }, { 137,-516 }, { 138,-516 }, { 139,-516 },
 { 140,-516 }, { 141,-516 }, { 142,-516 }, { 143,-516 }, { 144,-516 },
 { 145,-516 }, { 146,-516 }, { 147,-516 }, { 148,-516 }, { 149,-516 },

 { 150,-516 }, { 151,-516 }, { 152,-516 }, { 153,-516 }, { 154,-516 },
 { 155,-516 }, { 156,-516 }, { 157,-516 }, { 158,-516 }, { 159,-516 },
 { 160,-516 }, { 161,-516 }, { 162,-516 }, { 163,-516 }, { 164,-516 },
 { 165,-516 }, { 166,-516 }, { 167,-516 }, { 168,-516 }, { 169,-516 },
 { 170,-516 }, { 171,-516 }, { 172,-516 }, { 173,-516 }, { 174,-516 },
 { 175,-516 }, { 176,-516 }, { 177,-516 }, { 178,-516 }, { 179,-516 },
 { 180,-516 }, { 181,-516 }, { 182,-516 }, { 183,-516 }, { 184,-516 },
 { 185,-516 }, { 186,-516 }, { 187,-516 }, { 188,-516 }, { 189,-516 },
 { 190,-516 }, { 191,-516 }, { 192,-516 }, { 193,-516 }, { 194,-516 },
 { 195,-516 }, { 196,-516 }, { 197,-516 }, { 198,-516 }, { 199,-516 },

 { 200,-516 }, { 201,-516 }, { 202,-516 }, { 203,-516 }, { 204,-516 },
 { 205,-516 }, { 206,-516 }, { 207,-516 }, { 208,-516 }, { 209,-516 },
 { 210,-516 }, { 211,-516 }, { 212,-516 }, { 213,-516 }, { 214,-516 },
 { 215,-516 }, { 216,-516 }, { 217,-516 }, { 218,-516 }, { 219,-516 },
 { 220,-516 }, { 221,-516 }, { 222,-516 }, { 223,-516 }, { 224,-516 },
 { 225,-516 }, { 226,-516 }, { 227,-516 }, { 228,-516 }, { 229,-516 },
 { 230,-516 }, { 231,-516 }, { 232,-516 }, { 233,-516 }, { 234,-516 },
 { 235,-516 }, { 236,-516 }, { 237,-516 }, { 238,-516 }, { 239,-516 },
 { 240,-516 }, { 241,-516 }, { 242,-516 }, { 243,-516 }, { 244,-516 },
 { 245,-516 }, { 246,-516 }, { 247,-516 }, { 248,-516 }, { 249,-516 },

 { 250,-516 }, { 251,-516 }, { 252,-516 }, { 253,-516 }, { 254,-516 },
 { 255,-516 }, { 256,-516 }, {   0,  22 }, {   0,14867 }, {   1,4476 },
 {   2,4476 }, {   3,4476 }, {   4,4476 }, {   5,4476 }, {   6,4476 },
 {   7,4476 }, {   8,4476 }, {   9,4734 }, {  10,4992 }, {  11,4476 },
 {  12,4734 }, {  13,4992 }, {  14,4476 }, {  15,4476 }, {  16,4476 },
 {  17,4476 }, {  18,4476 }, {  19,4476 }, {  20,4476 }, {  21,4476 },
 {  22,4476 }, {  23,4476 }, {  24,4476 }, {  25,4476 }, {  26,4476 },
 {  27,4476 }, {  28,4476 }, {  29,4476 }, {  30,4476 }, {  31,4476 },
 {  32,4734 }, {  33,4476 }, {  34,4476 }, {  35,4476 }, {  36,4476 },
 {  37,4476 }, {  38,4476 }, {  39,4476 }, {  40,4476 }, {  41,4476 },

 {  42,4476 }, {  43,4476 }, {  44,4476 }, {  45,5039 }, {  46,4476 },
 {  47,4476 }, {  48,4476 }, {  49,4476 }, {  50,4476 }, {  51,4476 },
 {  52,4476 }, {  53,4476 }, {  54,4476 }, {  55,4476 }, {  56,4476 },
 {  57,4476 }, {  58,4476 }, {  59,4476 }, {  60,4476 }, {  61,4476 },
 {  62,4476 }, {  63,4476 }, {  64,4476 }, {  65,4476 }, {  66,4476 },
 {  67,4476 }, {  68,4476 }, {  69,4476 }, {  70,4476 }, {  71,4476 },
 {  72,4476 }, {  73,4476 }, {  74,4476 }, {  75,4476 }, {  76,4476 },
 {  77,4476 }, {  78,4476 }, {  79,4476 }, {  80,4476 }, {  81,4476 },
 {  82,4476 }, {  83,4476 }, {  84,4476 }, {  85,4476 }, {  86,4476 },
 {  87,4476 }, {  88,4476 }, {  89,4476 }, {  90,4476 }, {  91,4476 },

 {  92,4476 }, {  93,4476 }, {  94,4476 }, {  95,4476 }, {  96,4476 },
 {  97,4476 }, {  98,4476 }, {  99,4476 }, { 100,4476 }, { 101,4476 },
 { 102,4476 }, { 103,4476 }, { 104,4476 }, { 105,4476 }, { 106,4476 },
 { 107,4476 }, { 108,4476 }, { 109,4476 }, { 110,4476 }, { 111,4476 },
 { 112,4476 }, { 113,4476 }, { 114,4476 }, { 115,4476 }, { 116,4476 },
 { 117,4476 }, { 118,4476 }, { 119,4476 }, { 120,4476 }, { 121,4476 },
 { 122,4476 }, { 123,4476 }, { 124,4476 }, { 125,4476 }, { 126,4476 },
 { 127,4476 }, { 128,4476 }, { 129,4476 }, { 130,4476 }, { 131,4476 },
 { 132,4476 }, { 133,4476 }, { 134,4476 }, { 135,4476 }, { 136,4476 },
 { 137,4476 }, { 138,4476 }, { 139,4476 }, { 140,4476 }, { 141,4476 },

 { 142,4476 }, { 143,4476 }, { 144,4476 }, { 145,4476 }, { 146,4476 },
 { 147,4476 }, { 148,4476 }, { 149,4476 }, { 150,4476 }, { 151,4476 },
 { 152,4476 }, { 153,4476 }, { 154,4476 }, { 155,4476 }, { 156,4476 },
 { 157,4476 }, { 158,4476 }, { 159,4476 }, { 160,4476 }, { 161,4476 },
 { 162,4476 }, { 163,4476 }, { 164,4476 }, { 165,4476 }, { 166,4476 },
 { 167,4476 }, { 168,4476 }, { 169,4476 }, { 170,4476 }, { 171,4476 },
 { 172,4476 }, { 173,4476 }, { 174,4476 }, { 175,4476 }, { 176,4476 },
 { 177,4476 }, { 178,4476 }, { 179,4476 }, { 180,4476 }, { 181,4476 },
 { 182,4476 }, { 183,4476 }, { 184,4476 }, { 185,4476 }, { 186,4476 },
 { 187,4476 }, { 188,4476 }, { 189,4476 }, { 190,4476 }, { 191,4476 },

 { 192,4476 }, { 193,4476 }, { 194,4476 }, { 195,4476 }, { 196,4476 },
 { 197,4476 }, { 198,4476 }, { 199,4476 }, { 200,4476 }, { 201,4476 },
 { 202,4476 }, { 203,4476 }, { 204,4476 }, { 205,4476 }, { 206,4476 },
 { 207,4476 }, { 208,4476 }, { 209,4476 }, { 210,4476 }, { 211,4476 },
 { 212,4476 }, { 213,4476 }, { 214,4476 }, { 215,4476 }, { 216,4476 },
 { 217,4476 }, { 218,4476 }, { 219,4476 }, { 220,4476 }, { 221,4476 },
 { 222,4476 }, { 223,4476 }, { 224,4476 }, { 225,4476 }, { 226,4476 },
 { 227,4476 }, { 228,4476 }, { 229,4476 }, { 230,4476 }, { 231,4476 },
 { 232,4476 }, { 233,4476 }, { 234,4476 }, { 235,4476 }, { 236,4476 },
 { 237,4476 }, { 238,4476 }, { 239,4476 }, { 240,4476 }, { 241,4476 },

 { 242,4476 }, { 243,4476 }, { 244,4476 }, { 245,4476 }, { 246,4476 },
 { 247,4476 }, { 248,4476 }, { 249,4476 }, { 250,4476 }, { 251,4476 },
 { 252,4476 }, { 253,4476 }, { 254,4476 }, { 255,4476 }, { 256,4476 },
 {   0,  22 }, {   0,14609 }, {   1,   0 }, {   2,   0 }, {   3,   0 },
 {   4,   0 }, {   5,   0 }, {   6,   0 }, {   7,   0 }, {   8,   0 },
 {   9, 258 }, {  10,-7206 }, {  11,   0 }, {  12, 258 }, {  13,-7206 },
 {  14,   0 }, {  15,   0 }, {  16,   0 }, {  17,   0 }, {  18,   0 },
 {  19,   0 }, {  20,   0 }, {  21,   0 }, {  22,   0 }, {  23,   0 },
 {  24,   0 }, {  25,   0 }, {  26,   0 }, {  27,   0 }, {  28,   0 },
 {  29,   0 }, {  30,   0 }, {  31,   0 }, {  32, 258 }, {  33,   0 },

 {  34,   0 }, {  35,   0 }, {  36,   0 }, {  37,   0 }, {  38,   0 },
 {  39,   0 }, {  40,   0 }, {  41,   0 }, {  42,   0 }, {  43,   0 },
 {  44,   0 }, {  45, 516 }, {  46,   0 }, {  47,   0 }, {  48,   0 },
 {  49,   0 }, {  50,   0 }, {  51,   0 }, {  52,   0 }, {  53,   0 },
 {  54,   0 }, {  55,   0 }, {  56,   0 }, {  57,   0 }, {  58,   0 },
 {  59,   0 }, {  60,   0 }, {  61,   0 }, {  62,   0 }, {  63,   0 },
 {  64,   0 }, {  65,   0 }, {  66,   0 }, {  67,   0 }, {  68,   0 },
 {  69,   0 }, {  70,   0 }, {  71,   0 }, {  72,   0 }, {  73,   0 },
 {  74,   0 }, {  75,   0 }, {  76,   0 }, {  77,   0 }, {  78,   0 },
 {  79,   0 }, {  80,   0 }, {  81,   0 }, {  82,   0 }, {  83,   0 },

 {  84,   0 }, {  85,   0 }, {  86,   0 }, {  87,   0 }, {  88,   0 },
 {  89,   0 }, {  90,   0 }, {  91,   0 }, {  92,   0 }, {  93,   0 },
 {  94,   0 }, {  95,   0 }, {  96,   0 }, {  97,   0 }, {  98,   0 },
 {  99,   0 }, { 100,   0 }, { 101,   0 }, { 102,   0 }, { 103,   0 },
 { 104,   0 }, { 105,   0 }, { 106,   0 }, { 107,   0 }, { 108,   0 },
 { 109,   0 }, { 110,   0 }, { 111,   0 }, { 112,   0 }, { 113,   0 },
 { 114,   0 }, { 115,   0 }, { 116,   0 }, { 117,   0 }, { 118,   0 },
 { 119,   0 }, { 120,   0 }, { 121,   0 }, { 122,   0 }, { 123,   0 },
 { 124,   0 }, { 125,   0 }, { 126,   0 }, { 127,   0 }, { 128,   0 },
 { 129,   0 }, { 130,   0 }, { 131,   0 }, { 132,   0 }, { 133,   0 },

 { 134,   0 }, { 135,   0 }, { 136,   0 }, { 137,   0 }, { 138,   0 },
 { 139,   0 }, { 140,   0 }, { 141,   0 }, { 142,   0 }, { 143,   0 },
 { 144,   0 }, { 145,   0 }, { 146,   0 }, { 147,   0 }, { 148,   0 },
 { 149,   0 }, { 150,   0 }, { 151,   0 }, { 152,   0 }, { 153,   0 },
 { 154,   0 }, { 155,   0 }, { 156,   0 }, { 157,   0 }, { 158,   0 },
 { 159,   0 }, { 160,   0 }, { 161,   0 }, { 162,   0 }, { 163,   0 },
 { 164,   0 }, { 165,   0 }, { 166,   0 }, { 167,   0 }, { 168,   0 },
 { 169,   0 }, { 170,   0 }, { 171,   0 }, { 172,   0 }, { 173,   0 },
 { 174,   0 }, { 175,   0 }, { 176,   0 }, { 177,   0 }, { 178,   0 },
 { 179,   0 }, { 180,   0 }, { 181,   0 }, { 182,   0 }, { 183,   0 },

 { 184,   0 }, { 185,   0 }, { 186,   0 }, { 187,   0 }, { 188,   0 },
 { 189,   0 }, { 190,   0 }, { 191,   0 }, { 192,   0 }, { 193,   0 },
 { 194,   0 }, { 195,   0 }, { 196,   0 }, { 197,   0 }, { 198,   0 },
 { 199,   0 }, { 200,   0 }, { 201,   0 }, { 202,   0 }, { 203,   0 },
 { 204,   0 }, { 205,   0 }, { 206,   0 }, { 207,   0 }, { 208,   0 },
 { 209,   0 }, { 210,   0 }, { 211,   0 }, { 212,   0 }, { 213,   0 },
 { 214,   0 }, { 215,   0 }, { 216,   0 }, { 217,   0 }, { 218,   0 },
 { 219,   0 }, { 220,   0 }, { 221,   0 }, { 222,   0 }, { 223,   0 },
 { 224,   0 }, { 225,   0 }, { 226,   0 }, { 227,   0 }, { 228,   0 },
 { 229,   0 }, { 230,   0 }, { 231,   0 }, { 232,   0 }, { 233,   0 },

 { 234,   0 }, { 235,   0 }, { 236,   0 }, { 237,   0 }, { 238,   0 },
 { 239,   0 }, { 240,   0 }, { 241,   0 }, { 242,   0 }, { 243,   0 },
 { 244,   0 }, { 245,   0 }, { 246,   0 }, { 247,   0 }, { 248,   0 },
 { 249,   0 }, { 250,   0 }, { 251,   0 }, { 252,   0 }, { 253,   0 },
 { 254,   0 }, { 255,   0 }, { 256,   0 }, {   0,  22 }, {   0,14351 },
 {   1,-258 }, {   2,-258 }, {   3,-258 }, {   4,-258 }, {   5,-258 },
 {   6,-258 }, {   7,-258 }, {   8,-258 }, {   9,   0 }, {  10,-7464 },
 {  11,-258 }, {  12,   0 }, {  13,-7464 }, {  14,-258 }, {  15,-258 },
 {  16,-258 }, {  17,-258 }, {  18,-258 }, {  19,-258 }, {  20,-258 },
 {  21,-258 }, {  22,-258 }, {  23,-258 }, {  24,-258 }, {  25,-258 },

 {  26,-258 }, {  27,-258 }, {  28,-258 }, {  29,-258 }, {  30,-258 },
 {  31,-258 }, {  32,   0 }, {  33,-258 }, {  34,-258 }, {  35,-258 },
 {  36,-258 }, {  37,-258 }, {  38,-258 }, {  39,-258 }, {  40,-258 },
 {  41,-258 }, {  42,-258 }, {  43,-258 }, {  44,-258 }, {  45, 258 },
 {  46,-258 }, {  47,-258 }, {  48,-258 }, {  49,-258 }, {  50,-258 },
 {  51,-258 }, {  52,-258 }, {  53,-258 }, {  54,-258 }, {  55,-258 },
 {  56,-258 }, {  57,-258 }, {  58,-258 }, {  59,-258 }, {  60,-258 },
 {  61,-258 }, {  62,-258 }, {  63,-258 }, {  64,-258 }, {  65,-258 },
 {  66,-258 }, {  67,-258 }, {  68,-258 }, {  69,-258 }, {  70,-258 },
 {  71,-258 }, {  72,-258 }, {  73,-258 }, {  74,-258 }, {  75,-258 },

 {  76,-258 }, {  77,-258 }, {  78,-258 }, {  79,-258 }, {  80,-258 },
 {  81,-258 }, {  82,-258 }, {  83,-258 }, {  84,-258 }, {  85,-258 },
 {  86,-258 }, {  87,-258 }, {  88,-258 }, {  89,-258 }, {  90,-258 },
 {  91,-258 }, {  92,-258 }, {  93,-258 }, {  94,-258 }, {  95,-258 },
 {  96,-258 }, {  97,-258 }, {  98,-258 }, {  99,-258 }, { 100,-258 },
 { 101,-258 }, { 102,-258 }, { 103,-258 }, { 104,-258 }, { 105,-258 },
 { 106,-258 }, { 107,-258 }, { 108,-258 }, { 109,-258 }, { 110,-258 },
 { 111,-258 }, { 112,-258 }, { 113,-258 }, { 114,-258 }, { 115,-258 },
 { 116,-258 }, { 117,-258 }, { 118,-258 }, { 119,-258 }, { 120,-258 },
 { 121,-258 }, { 122,-258 }, { 123,-258 }, { 124,-258 }, { 125,-258 },

 { 126,-258 }, { 127,-258 }, { 128,-258 }, { 129,-258 }, { 130,-258 },
 { 131,-258 }, { 132,-258 }, { 133,-258 }, { 134,-258 }, { 135,-258 },
 { 136,-258 }, { 137,-258 }, { 138,-258 }, { 139,-258 }, { 140,-258 },
 { 141,-258 }, { 142,-258 }, { 143,-258 }, { 144,-258 }, { 145,-258 },
 { 146,-258 }, { 147,-258 }, { 148,-258 }, { 149,-258 }, { 150,-258 },
 { 151,-258 }, { 152,-258 }, { 153,-258 }, { 154,-258 }, { 155,-258 },
 { 156,-258 }, { 157,-258 }, { 158,-258 }, { 159,-258 }, { 160,-258 },
 { 161,-258 }, { 162,-258 }, { 163,-258 }, { 164,-258 }, { 165,-258 },
 { 166,-258 }, { 167,-258 }, { 168,-258 }, { 169,-258 }, { 170,-258 },
 { 171,-258 }, { 172,-258 }, { 173,-258 }, { 174,-258 }, { 175,-258 },

 { 176,-258 }, { 177,-258 }, { 178,-258 }, { 179,-258 }, { 180,-258 },
 { 181,-258 }, { 182,-258 }, { 183,-258 }, { 184,-258 }, { 185,-258 },
 { 186,-258 }, { 187,-258 }, { 188,-258 }, { 189,-258 }, { 190,-258 },
 { 191,-258 }, { 192,-258 }, { 193,-258 }, { 194,-258 }, { 195,-258 },
 { 196,-258 }, { 197,-258 }, { 198,-258 }, { 199,-258 }, { 200,-258 },
 { 201,-258 }, { 202,-258 }, { 203,-258 }, { 204,-258 }, { 205,-258 },
 { 206,-258 }, { 207,-258 }, { 208,-258 }, { 209,-258 }, { 210,-258 },
 { 211,-258 }, { 212,-258 }, { 213,-258 }, { 214,-258 }, { 215,-258 },
 { 216,-258 }, { 217,-258 }, { 218,-258 }, { 219,-258 }, { 220,-258 },
 { 221,-258 }, { 222,-258 }, { 223,-258 }, { 224,-258 }, { 225,-258 },

 { 226,-258 }, { 227,-258 }, { 228,-258 }, { 229,-258 }, { 230,-258 },
 { 231,-258 }, { 232,-258 }, { 233,-258 }, { 234,-258 }, { 235,-258 },
 { 236,-258 }, { 237,-258 }, { 238,-258 }, { 239,-258 }, { 240,-258 },
 { 241,-258 }, { 242,-258 }, { 243,-258 }, { 244,-258 }, { 245,-258 },
 { 246,-258 }, { 247,-258 }, { 248,-258 }, { 249,-258 }, { 250,-258 },
 { 251,-258 }, { 252,-258 }, { 253,-258 }, { 254,-258 }, { 255,-258 },
 { 256,-258 }, {   0,  22 }, {   0,14093 }, {   1,-516 }, {   2,-516 },
 {   3,-516 }, {   4,-516 }, {   5,-516 }, {   6,-516 }, {   7,-516 },
 {   8,-516 }, {   9,-258 }, {  10,-7722 }, {  11,-516 }, {  12,-258 },
 {  13,-7722 }, {  14,-516 }, {  15,-516 }, {  16,-516 }, {  17,-516 },

 {  18,-516 }, {  19,-516 }, {  20,-516 }, {  21,-516 }, {  22,-516 },
 {  23,-516 }, {  24,-516 }, {  25,-516 }, {  26,-516 }, {  27,-516 },
 {  28,-516 }, {  29,-516 }, {  30,-516 }, {  31,-516 }, {  32,-258 },
 {  33,-516 }, {  34,-516 }, {  35,-516 }, {  36,-516 }, {  37,-516 },
 {  38,-516 }, {  39,-516 }, {  40,-516 }, {  41,-516 }, {  42,-516 },
 {  43,-516 }, {  44,-516 }, {  45,4523 }, {  46,-516 }, {  47,-516 },
 {  48,-516 }, {  49,-516 }, {  50,-516 }, {  51,-516 }, {  52,-516 },
 {  53,-516 }, {  54,-516 }, {  55,-516 }, {  56,-516 }, {  57,-516 },
 {  58,-516 }, {  59,-516 }, {  60,-516 }, {  61,-516 }, {  62,-516 },
 {  63,-516 }, {  64,-516 }, {  65,-516 }, {  66,-516 }, {  67,-516 },

 {  68,-516 }, {  69,-516 }, {  70,-516 }, {  71,-516 }, {  72,-516 },
 {  73,-516 }, {  74,-516 }, {  75,-516 }, {  76,-516 }, {  77,-516 },
 {  78,-516 }, {  79,-516 }, {  80,-516 }, {  81,-516 }, {  82,-516 },
 {  83,-516 }, {  84,-516 }, {  85,-516 }, {  86,-516 }, {  87,-516 },
 {  88,-516 }, {  89,-516 }, {  90,-516 }, {  91,-516 }, {  92,-516 },
 {  93,-516 }, {  94,-516 }, {  95,-516 }, {  96,-516 }, {  97,-516 },
 {  98,-516 }, {  99,-516 }, { 100,-516 }, { 101,-516 }, { 102,-516 },
 { 103,-516 }, { 104,-516 }, { 105,-516 }, { 106,-516 }, { 107,-516 },
 { 108,-516 }, { 109,-516 }, { 110,-516 }, { 111,-516 }, { 112,-516 },
 { 113,-516 }, { 114,-516 }, { 115,-516 }, { 116,-516 }, { 117,-516 },

 { 118,-516 }, { 119,-516 }, { 120,-516 }, { 121,-516 }, { 122,-516 },
 { 123,-516 }, { 124,-516 }, { 125,-516 }, { 126,-516 }, { 127,-516 },
 { 128,-516 }, { 129,-516 }, { 130,-516 }, { 131,-516 }, { 132,-516 },
 { 133,-516 }, { 134,-516 }, { 135,-516 }, { 136,-516 }, { 137,-516 },
 { 138,-516 }, { 139,-516 }, { 140,-516 }, { 141,-516 }, { 142,-516 },
 { 143,-516 }, { 144,-516 }, { 145,-516 }, { 146,-516 }, { 147,-516 },
 { 148,-516 }, { 149,-516 }, { 150,-516 }, { 151,-516 }, { 152,-516 },
 { 153,-516 }, { 154,-516 }, { 155,-516 }, { 156,-516 }, { 157,-516 },
 { 158,-516 }, { 159,-516 }, { 160,-516 }, { 161,-516 }, { 162,-516 },
 { 163,-516 }, { 164,-516 }, { 165,-516 }, { 166,-516 }, { 167,-516 },

 { 168,-516 }, { 169,-516 }, { 170,-516 }, { 171,-516 }, { 172,-516 },
 { 173,-516 }, { 174,-516 }, { 175,-516 }, { 176,-516 }, { 177,-516 },
 { 178,-516 }, { 179,-516 }, { 180,-516 }, { 181,-516 }, { 182,-516 },
 { 183,-516 }, { 184,-516 }, { 185,-516 }, { 186,-516 }, { 187,-516 },
 { 188,-516 }, { 189,-516 }, { 190,-516 }, { 191,-516 }, { 192,-516 },
 { 193,-516 }, { 194,-516 }, { 195,-516 }, { 196,-516 }, { 197,-516 },
 { 198,-516 }, { 199,-516 }, { 200,-516 }, { 201,-516 }, { 202,-516 },
 { 203,-516 }, { 204,-516 }, { 205,-516 }, { 206,-516 }, { 207,-516 },
 { 208,-516 }, { 209,-516 }, { 210,-516 }, { 211,-516 }, { 212,-516 },
 { 213,-516 }, { 214,-516 }, { 215,-516 }, { 216,-516 }, { 217,-516 },

 { 218,-516 }, { 219,-516 }, { 220,-516 }, { 221,-516 }, { 222,-516 },
 { 223,-516 }, { 224,-516 }, { 225,-516 }, { 226,-516 }, { 227,-516 },
 { 228,-516 }, { 229,-516 }, { 230,-516 }, { 231,-516 }, { 232,-516 },
 { 233,-516 }, { 234,-516 }, { 235,-516 }, { 236,-516 }, { 237,-516 },
 { 238,-516 }, { 239,-516 }, { 240,-516 }, { 241,-516 }, { 242,-516 },
 { 243,-516 }, { 244,-516 }, { 245,-516 }, { 246,-516 }, { 247,-516 },
 { 248,-516 }, { 249,-516 }, { 250,-516 }, { 251,-516 }, { 252,-516 },
 { 253,-516 }, { 254,-516 }, { 255,-516 }, { 256,-516 }, {   0,  37 },
 {   0,13835 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,  37 }, {   0,13812 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  48,4523 }, {  49,4523 },
 {  50,4523 }, {  51,4523 }, {  52,4523 }, {  53,4523 }, {  54,4523 },
 {  55,4523 }, {  56,4523 }, {  57,4523 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  65,4523 }, {  66,4523 }, {  67,4523 }, {  68,4523 }, {  69,4523 },
 {  70,4523 }, {  48,4523 }, {  49,4523 }, {  50,4523 }, {  51,4523 },
 {  52,4523 }, {  53,4523 }, {  54,4523 }, {  55,4523 }, {  56,4523 },
 {  57,4523 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65,4523 }, {  66,4523 },
 {  67,4523 }, {  68,4523 }, {  69,4523 }, {  70,4523 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  97,4523 }, {  98,4523 }, {  99,4523 },
 { 100,4523 }, { 101,4523 }, { 102,4523 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  97,4523 }, {  98,4523 }, {  99,4523 }, { 100,4523 }, { 101,4523 },
 { 102,4523 }, {   0,  24 }, {   0,13708 }, {   1,4523 }, {   2,4523 },
 {   3,4523 }, {   4,4523 }, {   5,4523 }, {   6,4523 }, {   7,4523 },
 {   8,4523 }, {   9,4781 }, {  10,5039 }, {  11,4523 }, {  12,4781 },
 {  13,5039 }, {  14,4523 }, {  15,4523 }, {  16,4523 }, {  17,4523 },
 {  18,4523 }, {  19,4523 }, {  20,4523 }, {  21,4523 }, {  22,4523 },
 {  23,4523 }, {  24,4523 }, {  25,4523 }, {  26,4523 }, {  27,4523 },
 {  28,4523 }, {  29,4523 }, {  30,4523 }, {  31,4523 }, {  32,4781 },

 {  33,4523 }, {  34,4523 }, {  35,4523 }, {  36,4523 }, {  37,4523 },
 {  38,4523 }, {  39,4523 }, {  40,4523 }, {  41,4523 }, {  42,4523 },
 {  43,4523 }, {  44,4523 }, {  45,5086 }, {  46,4523 }, {  47,4523 },
 {  48,4523 }, {  49,4523 }, {  50,4523 }, {  51,4523 }, {  52,4523 },
 {  53,4523 }, {  54,4523 }, {  55,4523 }, {  56,4523 }, {  57,4523 },
 {  58,4523 }, {  59,4523 }, {  60,4523 }, {  61,4523 }, {  62,4523 },
 {  63,4523 }, {  64,4523 }, {  65,4523 }, {  66,4523 }, {  67,4523 },
 {  68,4523 }, {  69,4523 }, {  70,4523 }, {  71,4523 }, {  72,4523 },
 {  73,4523 }, {  74,4523 }, {  75,4523 }, {  76,4523 }, {  77,4523 },
 {  78,4523 }, {  79,4523 }, {  80,4523 }, {  81,4523 }, {  82,4523 },

 {  83,4523 }, {  84,4523 }, {  85,4523 }, {  86,4523 }, {  87,4523 },
 {  88,4523 }, {  89,4523 }, {  90,4523 }, {  91,4523 }, {  92,4523 },
 {  93,4523 }, {  94,4523 }, {  95,4523 }, {  96,4523 }, {  97,4523 },
 {  98,4523 }, {  99,4523 }, { 100,4523 }, { 101,4523 }, { 102,4523 },
 { 103,4523 }, { 104,4523 }, { 105,4523 }, { 106,4523 }, { 107,4523 },
 { 108,4523 }, { 109,4523 }, { 110,4523 }, { 111,4523 }, { 112,4523 },
 { 113,4523 }, { 114,4523 }, { 115,4523 }, { 116,4523 }, { 117,4523 },
 { 118,4523 }, { 119,4523 }, { 120,4523 }, { 121,4523 }, { 122,4523 },
 { 123,4523 }, { 124,4523 }, { 125,4523 }, { 126,4523 }, { 127,4523 },
 { 128,4523 }, { 129,4523 }, { 130,4523 }, { 131,4523 }, { 132,4523 },

 { 133,4523 }, { 134,4523 }, { 135,4523 }, { 136,4523 }, { 137,4523 },
 { 138,4523 }, { 139,4523 }, { 140,4523 }, { 141,4523 }, { 142,4523 },
 { 143,4523 }, { 144,4523 }, { 145,4523 }, { 146,4523 }, { 147,4523 },
 { 148,4523 }, { 149,4523 }, { 150,4523 }, { 151,4523 }, { 152,4523 },
 { 153,4523 }, { 154,4523 }, { 155,4523 }, { 156,4523 }, { 157,4523 },
 { 158,4523 }, { 159,4523 }, { 160,4523 }, { 161,4523 }, { 162,4523 },
 { 163,4523 }, { 164,4523 }, { 165,4523 }, { 166,4523 }, { 167,4523 },
 { 168,4523 }, { 169,4523 }, { 170,4523 }, { 171,4523 }, { 172,4523 },
 { 173,4523 }, { 174,4523 }, { 175,4523 }, { 176,4523 }, { 177,4523 },
 { 178,4523 }, { 179,4523 }, { 180,4523 }, { 181,4523 }, { 182,4523 },

 { 183,4523 }, { 184,4523 }, { 185,4523 }, { 186,4523 }, { 187,4523 },
 { 188,4523 }, { 189,4523 }, { 190,4523 }, { 191,4523 }, { 192,4523 },
 { 193,4523 }, { 194,4523 }, { 195,4523 }, { 196,4523 }, { 197,4523 },
 { 198,4523 }, { 199,4523 }, { 200,4523 }, { 201,4523 }, { 202,4523 },
 { 203,4523 }, { 204,4523 }, { 205,4523 }, { 206,4523 }, { 207,4523 },
 { 208,4523 }, { 209,4523 }, { 210,4523 }, { 211,4523 }, { 212,4523 },
 { 213,4523 }, { 214,4523 }, { 215,4523 }, { 216,4523 }, { 217,4523 },
 { 218,4523 }, { 219,4523 }, { 220,4523 }, { 221,4523 }, { 222,4523 },
 { 223,4523 }, { 224,4523 }, { 225,4523 }, { 226,4523 }, { 227,4523 },
 { 228,4523 }, { 229,4523 }, { 230,4523 }, { 231,4523 }, { 232,4523 },

 { 233,4523 }, { 234,4523 }, { 235,4523 }, { 236,4523 }, { 237,4523 },
 { 238,4523 }, { 239,4523 }, { 240,4523 }, { 241,4523 }, { 242,4523 },
 { 243,4523 }, { 244,4523 }, { 245,4523 }, { 246,4523 }, { 247,4523 },
 { 248,4523 }, { 249,4523 }, { 250,4523 }, { 251,4523 }, { 252,4523 },
 { 253,4523 }, { 254,4523 }, { 255,4523 }, { 256,4523 }, {   0,  24 },
 {   0,13450 }, {   1,   0 }, {   2,   0 }, {   3,   0 }, {   4,   0 },
 {   5,   0 }, {   6,   0 }, {   7,   0 }, {   8,   0 }, {   9, 258 },
 {  10,-7149 }, {  11,   0 }, {  12, 258 }, {  13,-7149 }, {  14,   0 },
 {  15,   0 }, {  16,   0 }, {  17,   0 }, {  18,   0 }, {  19,   0 },
 {  20,   0 }, {  21,   0 }, {  22,   0 }, {  23,   0 }, {  24,   0 },

 {  25,   0 }, {  26,   0 }, {  27,   0 }, {  28,   0 }, {  29,   0 },
 {  30,   0 }, {  31,   0 }, {  32, 258 }, {  33,   0 }, {  34,   0 },
 {  35,   0 }, {  36,   0 }, {  37,   0 }, {  38,   0 }, {  39,   0 },
 {  40,   0 }, {  41,   0 }, {  42,   0 }, {  43,   0 }, {  44,   0 },
 {  45, 516 }, {  46,   0 }, {  47,   0 }, {  48,   0 }, {  49,   0 },
 {  50,   0 }, {  51,   0 }, {  52,   0 }, {  53,   0 }, {  54,   0 },
 {  55,   0 }, {  56,   0 }, {  57,   0 }, {  58,   0 }, {  59,   0 },
 {  60,   0 }, {  61,   0 }, {  62,   0 }, {  63,   0 }, {  64,   0 },
 {  65,   0 }, {  66,   0 }, {  67,   0 }, {  68,   0 }, {  69,   0 },
 {  70,   0 }, {  71,   0 }, {  72,   0 }, {  73,   0 }, {  74,   0 },

 {  75,   0 }, {  76,   0 }, {  77,   0 }, {  78,   0 }, {  79,   0 },
 {  80,   0 }, {  81,   0 }, {  82,   0 }, {  83,   0 }, {  84,   0 },
 {  85,   0 }, {  86,   0 }, {  87,   0 }, {  88,   0 }, {  89,   0 },
 {  90,   0 }, {  91,   0 }, {  92,   0 }, {  93,   0 }, {  94,   0 },
 {  95,   0 }, {  96,   0 }, {  97,   0 }, {  98,   0 }, {  99,   0 },
 { 100,   0 }, { 101,   0 }, { 102,   0 }, { 103,   0 }, { 104,   0 },
 { 105,   0 }, { 106,   0 }, { 107,   0 }, { 108,   0 }, { 109,   0 },
 { 110,   0 }, { 111,   0 }, { 112,   0 }, { 113,   0 }, { 114,   0 },
 { 115,   0 }, { 116,   0 }, { 117,   0 }, { 118,   0 }, { 119,   0 },
 { 120,   0 }, { 121,   0 }, { 122,   0 }, { 123,   0 }, { 124,   0 },

 { 125,   0 }, { 126,   0 }, { 127,   0 }, { 128,   0 }, { 129,   0 },
 { 130,   0 }, { 131,   0 }, { 132,   0 }, { 133,   0 }, { 134,   0 },
 { 135,   0 }, { 136,   0 }, { 137,   0 }, { 138,   0 }, { 139,   0 },
 { 140,   0 }, { 141,   0 }, { 142,   0 }, { 143,   0 }, { 144,   0 },
 { 145,   0 }, { 146,   0 }, { 147,   0 }, { 148,   0 }, { 149,   0 },
 { 150,   0 }, { 151,   0 }, { 152,   0 }, { 153,   0 }, { 154,   0 },
 { 155,   0 }, { 156,   0 }, { 157,   0 }, { 158,   0 }, { 159,   0 },
 { 160,   0 }, { 161,   0 }, { 162,   0 }, { 163,   0 }, { 164,   0 },
 { 165,   0 }, { 166,   0 }, { 167,   0 }, { 168,   0 }, { 169,   0 },
 { 170,   0 }, { 171,   0 }, { 172,   0 }, { 173,   0 }, { 174,   0 },

 { 175,   0 }, { 176,   0 }, { 177,   0 }, { 178,   0 }, { 179,   0 },
 { 180,   0 }, { 181,   0 }, { 182,   0 }, { 183,   0 }, { 184,   0 },
 { 185,   0 }, { 186,   0 }, { 187,   0 }, { 188,   0 }, { 189,   0 },
 { 190,   0 }, { 191,   0 }, { 192,   0 }, { 193,   0 }, { 194,   0 },
 { 195,   0 }, { 196,   0 }, { 197,   0 }, { 198,   0 }, { 199,   0 },
 { 200,   0 }, { 201,   0 }, { 202,   0 }, { 203,   0 }, { 204,   0 },
 { 205,   0 }, { 206,   0 }, { 207,   0 }, { 208,   0 }, { 209,   0 },
 { 210,   0 }, { 211,   0 }, { 212,   0 }, { 213,   0 }, { 214,   0 },
 { 215,   0 }, { 216,   0 }, { 217,   0 }, { 218,   0 }, { 219,   0 },
 { 220,   0 }, { 221,   0 }, { 222,   0 }, { 223,   0 }, { 224,   0 },

 { 225,   0 }, { 226,   0 }, { 227,   0 }, { 228,   0 }, { 229,   0 },
 { 230,   0 }, { 231,   0 }, { 232,   0 }, { 233,   0 }, { 234,   0 },
 { 235,   0 }, { 236,   0 }, { 237,   0 }, { 238,   0 }, { 239,   0 },
 { 240,   0 }, { 241,   0 }, { 242,   0 }, { 243,   0 }, { 244,   0 },
 { 245,   0 }, { 246,   0 }, { 247,   0 }, { 248,   0 }, { 249,   0 },
 { 250,   0 }, { 251,   0 }, { 252,   0 }, { 253,   0 }, { 254,   0 },
 { 255,   0 }, { 256,   0 }, {   0,  24 }, {   0,13192 }, {   1,-258 },
 {   2,-258 }, {   3,-258 }, {   4,-258 }, {   5,-258 }, {   6,-258 },
 {   7,-258 }, {   8,-258 }, {   9,   0 }, {  10,-7407 }, {  11,-258 },
 {  12,   0 }, {  13,-7407 }, {  14,-258 }, {  15,-258 }, {  16,-258 },

 {  17,-258 }, {  18,-258 }, {  19,-258 }, {  20,-258 }, {  21,-258 },
 {  22,-258 }, {  23,-258 }, {  24,-258 }, {  25,-258 }, {  26,-258 },
 {  27,-258 }, {  28,-258 }, {  29,-258 }, {  30,-258 }, {  31,-258 },
 {  32,   0 }, {  33,-258 }, {  34,-258 }, {  35,-258 }, {  36,-258 },
 {  37,-258 }, {  38,-258 }, {  39,-258 }, {  40,-258 }, {  41,-258 },
 {  42,-258 }, {  43,-258 }, {  44,-258 }, {  45, 258 }, {  46,-258 },
 {  47,-258 }, {  48,-258 }, {  49,-258 }, {  50,-258 }, {  51,-258 },
 {  52,-258 }, {  53,-258 }, {  54,-258 }, {  55,-258 }, {  56,-258 },
 {  57,-258 }, {  58,-258 }, {  59,-258 }, {  60,-258 }, {  61,-258 },
 {  62,-258 }, {  63,-258 }, {  64,-258 }, {  65,-258 }, {  66,-258 },

 {  67,-258 }, {  68,-258 }, {  69,-258 }, {  70,-258 }, {  71,-258 },
 {  72,-258 }, {  73,-258 }, {  74,-258 }, {  75,-258 }, {  76,-258 },
 {  77,-258 }, {  78,-258 }, {  79,-258 }, {  80,-258 }, {  81,-258 },
 {  82,-258 }, {  83,-258 }, {  84,-258 }, {  85,-258 }, {  86,-258 },
 {  87,-258 }, {  88,-258 }, {  89,-258 }, {  90,-258 }, {  91,-258 },
 {  92,-258 }, {  93,-258 }, {  94,-258 }, {  95,-258 }, {  96,-258 },
 {  97,-258 }, {  98,-258 }, {  99,-258 }, { 100,-258 }, { 101,-258 },
 { 102,-258 }, { 103,-258 }, { 104,-258 }, { 105,-258 }, { 106,-258 },
 { 107,-258 }, { 108,-258 }, { 109,-258 }, { 110,-258 }, { 111,-258 },
 { 112,-258 }, { 113,-258 }, { 114,-258 }, { 115,-258 }, { 116,-258 },

 { 117,-258 }, { 118,-258 }, { 119,-258 }, { 120,-258 }, { 121,-258 },
 { 122,-258 }, { 123,-258 }, { 124,-258 }, { 125,-258 }, { 126,-258 },
 { 127,-258 }, { 128,-258 }, { 129,-258 }, { 130,-258 }, { 131,-258 },
 { 132,-258 }, { 133,-258 }, { 134,-258 }, { 135,-258 }, { 136,-258 },
 { 137,-258 }, { 138,-258 }, { 139,-258 }, { 140,-258 }, { 141,-258 },
 { 142,-258 }, { 143,-258 }, { 144,-258 }, { 145,-258 }, { 146,-258 },
 { 147,-258 }, { 148,-258 }, { 149,-258 }, { 150,-258 }, { 151,-258 },
 { 152,-258 }, { 153,-258 }, { 154,-258 }, { 155,-258 }, { 156,-258 },
 { 157,-258 }, { 158,-258 }, { 159,-258 }, { 160,-258 }, { 161,-258 },
 { 162,-258 }, { 163,-258 }, { 164,-258 }, { 165,-258 }, { 166,-258 },

 { 167,-258 }, { 168,-258 }, { 169,-258 }, { 170,-258 }, { 171,-258 },
 { 172,-258 }, { 173,-258 }, { 174,-258 }, { 175,-258 }, { 176,-258 },
 { 177,-258 }, { 178,-258 }, { 179,-258 }, { 180,-258 }, { 181,-258 },
 { 182,-258 }, { 183,-258 }, { 184,-258 }, { 185,-258 }, { 186,-258 },
 { 187,-258 }, { 188,-258 }, { 189,-258 }, { 190,-258 }, { 191,-258 },
 { 192,-258 }, { 193,-258 }, { 194,-258 }, { 195,-258 }, { 196,-258 },
 { 197,-258 }, { 198,-258 }, { 199,-258 }, { 200,-258 }, { 201,-258 },
 { 202,-258 }, { 203,-258 }, { 204,-258 }, { 205,-258 }, { 206,-258 },
 { 207,-258 }, { 208,-258 }, { 209,-258 }, { 210,-258 }, { 211,-258 },
 { 212,-258 }, { 213,-258 }, { 214,-258 }, { 215,-258 }, { 216,-258 },

 { 217,-258 }, { 218,-258 }, { 219,-258 }, { 220,-258 }, { 221,-258 },
 { 222,-258 }, { 223,-258 }, { 224,-258 }, { 225,-258 }, { 226,-258 },
 { 227,-258 }, { 228,-258 }, { 229,-258 }, { 230,-258 }, { 231,-258 },
 { 232,-258 }, { 233,-258 }, { 234,-258 }, { 235,-258 }, { 236,-258 },
 { 237,-258 }, { 238,-258 }, { 239,-258 }, { 240,-258 }, { 241,-258 },
 { 242,-258 }, { 243,-258 }, { 244,-258 }, { 245,-258 }, { 246,-258 },
 { 247,-258 }, { 248,-258 }, { 249,-258 }, { 250,-258 }, { 251,-258 },
 { 252,-258 }, { 253,-258 }, { 254,-258 }, { 255,-258 }, { 256,-258 },
 {   0,  24 }, {   0,12934 }, {   1,-516 }, {   2,-516 }, {   3,-516 },
 {   4,-516 }, {   5,-516 }, {   6,-516 }, {   7,-516 }, {   8,-516 },

 {   9,-258 }, {  10,-7665 }, {  11,-516 }, {  12,-258 }, {  13,-7665 },
 {  14,-516 }, {  15,-516 }, {  16,-516 }, {  17,-516 }, {  18,-516 },
 {  19,-516 }, {  20,-516 }, {  21,-516 }, {  22,-516 }, {  23,-516 },
 {  24,-516 }, {  25,-516 }, {  26,-516 }, {  27,-516 }, {  28,-516 },
 {  29,-516 }, {  30,-516 }, {  31,-516 }, {  32,-258 }, {  33,-516 },
 {  34,-516 }, {  35,-516 }, {  36,-516 }, {  37,-516 }, {  38,-516 },
 {  39,-516 }, {  40,-516 }, {  41,-516 }, {  42,-516 }, {  43,-516 },
 {  44,-516 }, {  45,4570 }, {  46,-516 }, {  47,-516 }, {  48,-516 },
 {  49,-516 }, {  50,-516 }, {  51,-516 }, {  52,-516 }, {  53,-516 },
 {  54,-516 }, {  55,-516 }, {  56,-516 }, {  57,-516 }, {  58,-516 },

 {  59,-516 }, {  60,-516 }, {  61,-516 }, {  62,-516 }, {  63,-516 },
 {  64,-516 }, {  65,-516 }, {  66,-516 }, {  67,-516 }, {  68,-516 },
 {  69,-516 }, {  70,-516 }, {  71,-516 }, {  72,-516 }, {  73,-516 },
 {  74,-516 }, {  75,-516 }, {  76,-516 }, {  77,-516 }, {  78,-516 },
 {  79,-516 }, {  80,-516 }, {  81,-516 }, {  82,-516 }, {  83,-516 },
 {  84,-516 }, {  85,-516 }, {  86,-516 }, {  87,-516 }, {  88,-516 },
 {  89,-516 }, {  90,-516 }, {  91,-516 }, {  92,-516 }, {  93,-516 },
 {  94,-516 }, {  95,-516 }, {  96,-516 }, {  97,-516 }, {  98,-516 },
 {  99,-516 }, { 100,-516 }, { 101,-516 }, { 102,-516 }, { 103,-516 },
 { 104,-516 }, { 105,-516 }, { 106,-516 }, { 107,-516 }, { 108,-516 },

 { 109,-516 }, { 110,-516 }, { 111,-516 }, { 112,-516 }, { 113,-516 },
 { 114,-516 }, { 115,-516 }, { 116,-516 }, { 117,-516 }, { 118,-516 },
 { 119,-516 }, { 120,-516 }, { 121,-516 }, { 122,-516 }, { 123,-516 },
 { 124,-516 }, { 125,-516 }, { 126,-516 }, { 127,-516 }, { 128,-516 },
 { 129,-516 }, { 130,-516 }, { 131,-516 }, { 132,-516 }, { 133,-516 },
 { 134,-516 }, { 135,-516 }, { 136,-516 }, { 137,-516 }, { 138,-516 },
 { 139,-516 }, { 140,-516 }, { 141,-516 }, { 142,-516 }, { 143,-516 },
 { 144,-516 }, { 145,-516 }, { 146,-516 }, { 147,-516 }, { 148,-516 },
 { 149,-516 }, { 150,-516 }, { 151,-516 }, { 152,-516 }, { 153,-516 },
 { 154,-516 }, { 155,-516 }, { 156,-516 }, { 157,-516 }, { 158,-516 },

 { 159,-516 }, { 160,-516 }, { 161,-516 }, { 162,-516 }, { 163,-516 },
 { 164,-516 }, { 165,-516 }, { 166,-516 }, { 167,-516 }, { 168,-516 },
 { 169,-516 }, { 170,-516 }, { 171,-516 }, { 172,-516 }, { 173,-516 },
 { 174,-516 }, { 175,-516 }, { 176,-516 }, { 177,-516 }, { 178,-516 },
 { 179,-516 }, { 180,-516 }, { 181,-516 }, { 182,-516 }, { 183,-516 },
 { 184,-516 }, { 185,-516 }, { 186,-516 }, { 187,-516 }, { 188,-516 },
 { 189,-516 }, { 190,-516 }, { 191,-516 }, { 192,-516 }, { 193,-516 },
 { 194,-516 }, { 195,-516 }, { 196,-516 }, { 197,-516 }, { 198,-516 },
 { 199,-516 }, { 200,-516 }, { 201,-516 }, { 202,-516 }, { 203,-516 },
 { 204,-516 }, { 205,-516 }, { 206,-516 }, { 207,-516 }, { 208,-516 },

 { 209,-516 }, { 210,-516 }, { 211,-516 }, { 212,-516 }, { 213,-516 },
 { 214,-516 }, { 215,-516 }, { 216,-516 }, { 217,-516 }, { 218,-516 },
 { 219,-516 }, { 220,-516 }, { 221,-516 }, { 222,-516 }, { 223,-516 },
 { 224,-516 }, { 225,-516 }, { 226,-516 }, { 227,-516 }, { 228,-516 },
 { 229,-516 }, { 230,-516 }, { 231,-516 }, { 232,-516 }, { 233,-516 },
 { 234,-516 }, { 235,-516 }, { 236,-516 }, { 237,-516 }, { 238,-516 },
 { 239,-516 }, { 240,-516 }, { 241,-516 }, { 242,-516 }, { 243,-516 },
 { 244,-516 }, { 245,-516 }, { 246,-516 }, { 247,-516 }, { 248,-516 },
 { 249,-516 }, { 250,-516 }, { 251,-516 }, { 252,-516 }, { 253,-516 },
 { 254,-516 }, { 255,-516 }, { 256,-516 }, {   0,  37 }, {   0,12676 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,  37 }, {   0,12653 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  48,4570 }, {  49,4570 }, {  50,4570 },

 {  51,4570 }, {  52,4570 }, {  53,4570 }, {  54,4570 }, {  55,4570 },
 {  56,4570 }, {  57,4570 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65,4570 },
 {  66,4570 }, {  67,4570 }, {  68,4570 }, {  69,4570 }, {  70,4570 },
 {  48,4570 }, {  49,4570 }, {  50,4570 }, {  51,4570 }, {  52,4570 },
 {  53,4570 }, {  54,4570 }, {  55,4570 }, {  56,4570 }, {  57,4570 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  65,4570 }, {  66,4570 }, {  67,4570 },
 {  68,4570 }, {  69,4570 }, {  70,4570 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  97,4570 }, {  98,4570 }, {  99,4570 }, { 100,4570 },

 { 101,4570 }, { 102,4570 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  97,4570 },
 {  98,4570 }, {  99,4570 }, { 100,4570 }, { 101,4570 }, { 102,4570 },
 {   0,   9 }, {   0,12549 }, {   1,   0 }, {   2,   0 }, {   3,   0 },
 {   4,   0 }, {   5,   0 }, {   6,   0 }, {   7,   0 }, {   8,   0 },
 {   9, 258 }, {  10, 516 }, {  11,   0 }, {  12, 258 }, {  13, 516 },
 {  14,   0 }, {  15,   0 }, {  16,   0 }, {  17,   0 }, {  18,   0 },
 {  19,   0 }, {  20,   0 }, {  21,   0 }, {  22,   0 }, {  23,   0 },

 {  24,   0 }, {  25,   0 }, {  26,   0 }, {  27,   0 }, {  28,   0 },
 {  29,   0 }, {  30,   0 }, {  31,   0 }, {  32, 258 }, {  33,   0 },
 {  34,   0 }, {  35,   0 }, {  36,   0 }, {  37,   0 }, {  38,   0 },
 {  39,   0 }, {  40,   0 }, {  41,   0 }, {  42,   0 }, {  43,   0 },
 {  44,   0 }, {  45, 563 }, {  46,   0 }, {  47,   0 }, {  48,   0 },
 {  49,   0 }, {  50,   0 }, {  51,   0 }, {  52,   0 }, {  53,   0 },
 {  54,   0 }, {  55,   0 }, {  56,   0 }, {  57,   0 }, {  58,   0 },
 {  59,   0 }, {  60,   0 }, {  61,   0 }, {  62,   0 }, {  63,   0 },
 {  64,   0 }, {  65,   0 }, {  66,   0 }, {  67,   0 }, {  68,   0 },
 {  69,   0 }, {  70,   0 }, {  71,   0 }, {  72,   0 }, {  73,   0 },

 {  74,   0 }, {  75,   0 }, {  76,   0 }, {  77,   0 }, {  78,   0 },
 {  79,   0 }, {  80,   0 }, {  81,   0 }, {  82,   0 }, {  83,   0 },
 {  84,   0 }, {  85,   0 }, {  86,   0 }, {  87,   0 }, {  88,   0 },
 {  89,   0 }, {  90,   0 }, {  91,   0 }, {  92,   0 }, {  93,   0 },
 {  94,   0 }, {  95,   0 }, {  96,   0 }, {  97,   0 }, {  98,   0 },
 {  99,   0 }, { 100,   0 }, { 101,   0 }, { 102,   0 }, { 103,   0 },
 { 104,   0 }, { 105,   0 }, { 106,   0 }, { 107,   0 }, { 108,   0 },
 { 109,   0 }, { 110,   0 }, { 111,   0 }, { 112,   0 }, { 113,   0 },
 { 114,   0 }, { 115,   0 }, { 116,   0 }, { 117,   0 }, { 118,   0 },
 { 119,   0 }, { 120,   0 }, { 121,   0 }, { 122,   0 }, { 123,   0 },

 { 124,   0 }, { 125,   0 }, { 126,   0 }, { 127,   0 }, { 128,   0 },
 { 129,   0 }, { 130,   0 }, { 131,   0 }, { 132,   0 }, { 133,   0 },
 { 134,   0 }, { 135,   0 }, { 136,   0 }, { 137,   0 }, { 138,   0 },
 { 139,   0 }, { 140,   0 }, { 141,   0 }, { 142,   0 }, { 143,   0 },
 { 144,   0 }, { 145,   0 }, { 146,   0 }, { 147,   0 }, { 148,   0 },
 { 149,   0 }, { 150,   0 }, { 151,   0 }, { 152,   0 }, { 153,   0 },
 { 154,   0 }, { 155,   0 }, { 156,   0 }, { 157,   0 }, { 158,   0 },
 { 159,   0 }, { 160,   0 }, { 161,   0 }, { 162,   0 }, { 163,   0 },
 { 164,   0 }, { 165,   0 }, { 166,   0 }, { 167,   0 }, { 168,   0 },
 { 169,   0 }, { 170,   0 }, { 171,   0 }, { 172,   0 }, { 173,   0 },

 { 174,   0 }, { 175,   0 }, { 176,   0 }, { 177,   0 }, { 178,   0 },
 { 179,   0 }, { 180,   0 }, { 181,   0 }, { 182,   0 }, { 183,   0 },
 { 184,   0 }, { 185,   0 }, { 186,   0 }, { 187,   0 }, { 188,   0 },
 { 189,   0 }, { 190,   0 }, { 191,   0 }, { 192,   0 }, { 193,   0 },
 { 194,   0 }, { 195,   0 }, { 196,   0 }, { 197,   0 }, { 198,   0 },
 { 199,   0 }, { 200,   0 }, { 201,   0 }, { 202,   0 }, { 203,   0 },
 { 204,   0 }, { 205,   0 }, { 206,   0 }, { 207,   0 }, { 208,   0 },
 { 209,   0 }, { 210,   0 }, { 211,   0 }, { 212,   0 }, { 213,   0 },
 { 214,   0 }, { 215,   0 }, { 216,   0 }, { 217,   0 }, { 218,   0 },
 { 219,   0 }, { 220,   0 }, { 221,   0 }, { 222,   0 }, { 223,   0 },

 { 224,   0 }, { 225,   0 }, { 226,   0 }, { 227,   0 }, { 228,   0 },
 { 229,   0 }, { 230,   0 }, { 231,   0 }, { 232,   0 }, { 233,   0 },
 { 234,   0 }, { 235,   0 }, { 236,   0 }, { 237,   0 }, { 238,   0 },
 { 239,   0 }, { 240,   0 }, { 241,   0 }, { 242,   0 }, { 243,   0 },
 { 244,   0 }, { 245,   0 }, { 246,   0 }, { 247,   0 }, { 248,   0 },
 { 249,   0 }, { 250,   0 }, { 251,   0 }, { 252,   0 }, { 253,   0 },
 { 254,   0 }, { 255,   0 }, { 256,   0 }, {   0,   9 }, {   0,12291 },
 {   1,-258 }, {   2,-258 }, {   3,-258 }, {   4,-258 }, {   5,-258 },
 {   6,-258 }, {   7,-258 }, {   8,-258 }, {   9,   0 }, {  10, 258 },
 {  11,-258 }, {  12,   0 }, {  13, 258 }, {  14,-258 }, {  15,-258 },

 {  16,-258 }, {  17,-258 }, {  18,-258 }, {  19,-258 }, {  20,-258 },
 {  21,-258 }, {  22,-258 }, {  23,-258 }, {  24,-258 }, {  25,-258 },
 {  26,-258 }, {  27,-258 }, {  28,-258 }, {  29,-258 }, {  30,-258 },
 {  31,-258 }, {  32,   0 }, {  33,-258 }, {  34,-258 }, {  35,-258 },
 {  36,-258 }, {  37,-258 }, {  38,-258 }, {  39,-258 }, {  40,-258 },
 {  41,-258 }, {  42,-258 }, {  43,-258 }, {  44,-258 }, {  45, 305 },
 {  46,-258 }, {  47,-258 }, {  48,-258 }, {  49,-258 }, {  50,-258 },
 {  51,-258 }, {  52,-258 }, {  53,-258 }, {  54,-258 }, {  55,-258 },
 {  56,-258 }, {  57,-258 }, {  58,-258 }, {  59,-258 }, {  60,-258 },
 {  61,-258 }, {  62,-258 }, {  63,-258 }, {  64,-258 }, {  65,-258 },

 {  66,-258 }, {  67,-258 }, {  68,-258 }, {  69,-258 }, {  70,-258 },
 {  71,-258 }, {  72,-258 }, {  73,-258 }, {  74,-258 }, {  75,-258 },
 {  76,-258 }, {  77,-258 }, {  78,-258 }, {  79,-258 }, {  80,-258 },
 {  81,-258 }, {  82,-258 }, {  83,-258 }, {  84,-258 }, {  85,-258 },
 {  86,-258 }, {  87,-258 }, {  88,-258 }, {  89,-258 }, {  90,-258 },
 {  91,-258 }, {  92,-258 }, {  93,-258 }, {  94,-258 }, {  95,-258 },
 {  96,-258 }, {  97,-258 }, {  98,-258 }, {  99,-258 }, { 100,-258 },
 { 101,-258 }, { 102,-258 }, { 103,-258 }, { 104,-258 }, { 105,-258 },
 { 106,-258 }, { 107,-258 }, { 108,-258 }, { 109,-258 }, { 110,-258 },
 { 111,-258 }, { 112,-258 }, { 113,-258 }, { 114,-258 }, { 115,-258 },

 { 116,-258 }, { 117,-258 }, { 118,-258 }, { 119,-258 }, { 120,-258 },
 { 121,-258 }, { 122,-258 }, { 123,-258 }, { 124,-258 }, { 125,-258 },
 { 126,-258 }, { 127,-258 }, { 128,-258 }, { 129,-258 }, { 130,-258 },
 { 131,-258 }, { 132,-258 }, { 133,-258 }, { 134,-258 }, { 135,-258 },
 { 136,-258 }, { 137,-258 }, { 138,-258 }, { 139,-258 }, { 140,-258 },
 { 141,-258 }, { 142,-258 }, { 143,-258 }, { 144,-258 }, { 145,-258 },
 { 146,-258 }, { 147,-258 }, { 148,-258 }, { 149,-258 }, { 150,-258 },
 { 151,-258 }, { 152,-258 }, { 153,-258 }, { 154,-258 }, { 155,-258 },
 { 156,-258 }, { 157,-258 }, { 158,-258 }, { 159,-258 }, { 160,-258 },
 { 161,-258 }, { 162,-258 }, { 163,-258 }, { 164,-258 }, { 165,-258 },

 { 166,-258 }, { 167,-258 }, { 168,-258 }, { 169,-258 }, { 170,-258 },
 { 171,-258 }, { 172,-258 }, { 173,-258 }, { 174,-258 }, { 175,-258 },
 { 176,-258 }, { 177,-258 }, { 178,-258 }, { 179,-258 }, { 180,-258 },
 { 181,-258 }, { 182,-258 }, { 183,-258 }, { 184,-258 }, { 185,-258 },
 { 186,-258 }, { 187,-258 }, { 188,-258 }, { 189,-258 }, { 190,-258 },
 { 191,-258 }, { 192,-258 }, { 193,-258 }, { 194,-258 }, { 195,-258 },
 { 196,-258 }, { 197,-258 }, { 198,-258 }, { 199,-258 }, { 200,-258 },
 { 201,-258 }, { 202,-258 }, { 203,-258 }, { 204,-258 }, { 205,-258 },
 { 206,-258 }, { 207,-258 }, { 208,-258 }, { 209,-258 }, { 210,-258 },
 { 211,-258 }, { 212,-258 }, { 213,-258 }, { 214,-258 }, { 215,-258 },

 { 216,-258 }, { 217,-258 }, { 218,-258 }, { 219,-258 }, { 220,-258 },
 { 221,-258 }, { 222,-258 }, { 223,-258 }, { 224,-258 }, { 225,-258 },
 { 226,-258 }, { 227,-258 }, { 228,-258 }, { 229,-258 }, { 230,-258 },
 { 231,-258 }, { 232,-258 }, { 233,-258 }, { 234,-258 }, { 235,-258 },
 { 236,-258 }, { 237,-258 }, { 238,-258 }, { 239,-258 }, { 240,-258 },
 { 241,-258 }, { 242,-258 }, { 243,-258 }, { 244,-258 }, { 245,-258 },
 { 246,-258 }, { 247,-258 }, { 248,-258 }, { 249,-258 }, { 250,-258 },
 { 251,-258 }, { 252,-258 }, { 253,-258 }, { 254,-258 }, { 255,-258 },
 { 256,-258 }, {   0,   9 }, {   0,12033 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   9,-7240 }, {  10,-7240 }, {   0,   0 }, {  12,-7240 },
 {  13,-7240 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  32,-7240 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  39,-17840 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  45,-17838 }, {   0,   9 }, {   0,11986 },
 {   1,-563 }, {   2,-563 }, {   3,-563 }, {   4,-563 }, {   5,-563 },
 {   6,-563 }, {   7,-563 }, {   8,-563 }, {   9,-305 }, {  10, -47 },

 {  11,-563 }, {  12,-305 }, {  13, -47 }, {  14,-563 }, {  15,-563 },
 {  16,-563 }, {  17,-563 }, {  18,-563 }, {  19,-563 }, {  20,-563 },
 {  21,-563 }, {  22,-563 }, {  23,-563 }, {  24,-563 }, {  25,-563 },
 {  26,-563 }, {  27,-563 }, {  28,-563 }, {  29,-563 }, {  30,-563 },
 {  31,-563 }, {  32,-305 }, {  33,-563 }, {  34,-563 }, {  35,-563 },
 {  36,-563 }, {  37,-563 }, {  38,-563 }, {  39,-563 }, {  40,-563 },
 {  41,-563 }, {  42,-563 }, {  43,-563 }, {  44,-563 }, {  45,4007 },
 {  46,-563 }, {  47,-563 }, {  48,-563 }, {  49,-563 }, {  50,-563 },
 {  51,-563 }, {  52,-563 }, {  53,-563 }, {  54,-563 }, {  55,-563 },
 {  56,-563 }, {  57,-563 }, {  58,-563 }, {  59,-563 }, {  60,-563 },

 {  61,-563 }, {  62,-563 }, {  63,-563 }, {  64,-563 }, {  65,-563 },
 {  66,-563 }, {  67,-563 }, {  68,-563 }, {  69,-563 }, {  70,-563 },
 {  71,-563 }, {  72,-563 }, {  73,-563 }, {  74,-563 }, {  75,-563 },
 {  76,-563 }, {  77,-563 }, {  78,-563 }, {  79,-563 }, {  80,-563 },
 {  81,-563 }, {  82,-563 }, {  83,-563 }, {  84,-563 }, {  85,-563 },
 {  86,-563 }, {  87,-563 }, {  88,-563 }, {  89,-563 }, {  90,-563 },
 {  91,-563 }, {  92,-563 }, {  93,-563 }, {  94,-563 }, {  95,-563 },
 {  96,-563 }, {  97,-563 }, {  98,-563 }, {  99,-563 }, { 100,-563 },
 { 101,-563 }, { 102,-563 }, { 103,-563 }, { 104,-563 }, { 105,-563 },
 { 106,-563 }, { 107,-563 }, { 108,-563 }, { 109,-563 }, { 110,-563 },

 { 111,-563 }, { 112,-563 }, { 113,-563 }, { 114,-563 }, { 115,-563 },
 { 116,-563 }, { 117,-563 }, { 118,-563 }, { 119,-563 }, { 120,-563 },
 { 121,-563 }, { 122,-563 }, { 123,-563 }, { 124,-563 }, { 125,-563 },
 { 126,-563 }, { 127,-563 }, { 128,-563 }, { 129,-563 }, { 130,-563 },
 { 131,-563 }, { 132,-563 }, { 133,-563 }, { 134,-563 }, { 135,-563 },
 { 136,-563 }, { 137,-563 }, { 138,-563 }, { 139,-563 }, { 140,-563 },
 { 141,-563 }, { 142,-563 }, { 143,-563 }, { 144,-563 }, { 145,-563 },
 { 146,-563 }, { 147,-563 }, { 148,-563 }, { 149,-563 }, { 150,-563 },
 { 151,-563 }, { 152,-563 }, { 153,-563 }, { 154,-563 }, { 155,-563 },
 { 156,-563 }, { 157,-563 }, { 158,-563 }, { 159,-563 }, { 160,-563 },

 { 161,-563 }, { 162,-563 }, { 163,-563 }, { 164,-563 }, { 165,-563 },
 { 166,-563 }, { 167,-563 }, { 168,-563 }, { 169,-563 }, { 170,-563 },
 { 171,-563 }, { 172,-563 }, { 173,-563 }, { 174,-563 }, { 175,-563 },
 { 176,-563 }, { 177,-563 }, { 178,-563 }, { 179,-563 }, { 180,-563 },
 { 181,-563 }, { 182,-563 }, { 183,-563 }, { 184,-563 }, { 185,-563 },
 { 186,-563 }, { 187,-563 }, { 188,-563 }, { 189,-563 }, { 190,-563 },
 { 191,-563 }, { 192,-563 }, { 193,-563 }, { 194,-563 }, { 195,-563 },
 { 196,-563 }, { 197,-563 }, { 198,-563 }, { 199,-563 }, { 200,-563 },
 { 201,-563 }, { 202,-563 }, { 203,-563 }, { 204,-563 }, { 205,-563 },
 { 206,-563 }, { 207,-563 }, { 208,-563 }, { 209,-563 }, { 210,-563 },

 { 211,-563 }, { 212,-563 }, { 213,-563 }, { 214,-563 }, { 215,-563 },
 { 216,-563 }, { 217,-563 }, { 218,-563 }, { 219,-563 }, { 220,-563 },
 { 221,-563 }, { 222,-563 }, { 223,-563 }, { 224,-563 }, { 225,-563 },
 { 226,-563 }, { 227,-563 }, { 228,-563 }, { 229,-563 }, { 230,-563 },
 { 231,-563 }, { 232,-563 }, { 233,-563 }, { 234,-563 }, { 235,-563 },
 { 236,-563 }, { 237,-563 }, { 238,-563 }, { 239,-563 }, { 240,-563 },
 { 241,-563 }, { 242,-563 }, { 243,-563 }, { 244,-563 }, { 245,-563 },
 { 246,-563 }, { 247,-563 }, { 248,-563 }, { 249,-563 }, { 250,-563 },
 { 251,-563 }, { 252,-563 }, { 253,-563 }, { 254,-563 }, { 255,-563 },
 { 256,-563 }, {   0,   9 }, {   0,11728 }, {   1,-4945 }, {   2,-4945 },

 {   3,-4945 }, {   4,-4945 }, {   5,-4945 }, {   6,-4945 }, {   7,-4945 },
 {   8,-4945 }, {   9,-4687 }, {  10,-11351 }, {  11,-4945 }, {  12,-4687 },
 {  13,-11351 }, {  14,-4945 }, {  15,-4945 }, {  16,-4945 }, {  17,-4945 },
 {  18,-4945 }, {  19,-4945 }, {  20,-4945 }, {  21,-4945 }, {  22,-4945 },
 {  23,-4945 }, {  24,-4945 }, {  25,-4945 }, {  26,-4945 }, {  27,-4945 },
 {  28,-4945 }, {  29,-4945 }, {  30,-4945 }, {  31,-4945 }, {  32,-4687 },
 {  33,-4945 }, {  34,-4945 }, {  35,-4945 }, {  36,-4945 }, {  37,-4945 },
 {  38,-4945 }, {  39,-4945 }, {  40,-4945 }, {  41,-4945 }, {  42,-4945 },
 {  43,-4945 }, {  44,-4945 }, {  45,   0 }, {  46,-4945 }, {  47,-4945 },
 {  48,-4945 }, {  49,-4945 }, {  50,-4945 }, {  51,-4945 }, {  52,-4945 },

 {  53,-4945 }, {  54,-4945 }, {  55,-4945 }, {  56,-4945 }, {  57,-4945 },
 {  58,-4945 }, {  59,-4945 }, {  60,-4945 }, {  61,-4945 }, {  62,-4945 },
 {  63,-4945 }, {  64,-4945 }, {  65,-4945 }, {  66,-4945 }, {  67,-4945 },
 {  68,-4945 }, {  69,-4945 }, {  70,-4945 }, {  71,-4945 }, {  72,-4945 },
 {  73,-4945 }, {  74,-4945 }, {  75,-4945 }, {  76,-4945 }, {  77,-4945 },
 {  78,-4945 }, {  79,-4945 }, {  80,-4945 }, {  81,-4945 }, {  82,-4945 },
 {  83,-4945 }, {  84,-4945 }, {  85,-4945 }, {  86,-4945 }, {  87,-4945 },
 {  88,-4945 }, {  89,-4945 }, {  90,-4945 }, {  91,-4945 }, {  92,-4945 },
 {  93,-4945 }, {  94,-4945 }, {  95,-4945 }, {  96,-4945 }, {  97,-4945 },
 {  98,-4945 }, {  99,-4945 }, { 100,-4945 }, { 101,-4945 }, { 102,-4945 },

 { 103,-4945 }, { 104,-4945 }, { 105,-4945 }, { 106,-4945 }, { 107,-4945 },
 { 108,-4945 }, { 109,-4945 }, { 110,-4945 }, { 111,-4945 }, { 112,-4945 },
 { 113,-4945 }, { 114,-4945 }, { 115,-4945 }, { 116,-4945 }, { 117,-4945 },
 { 118,-4945 }, { 119,-4945 }, { 120,-4945 }, { 121,-4945 }, { 122,-4945 },
 { 123,-4945 }, { 124,-4945 }, { 125,-4945 }, { 126,-4945 }, { 127,-4945 },
 { 128,-4945 }, { 129,-4945 }, { 130,-4945 }, { 131,-4945 }, { 132,-4945 },
 { 133,-4945 }, { 134,-4945 }, { 135,-4945 }, { 136,-4945 }, { 137,-4945 },
 { 138,-4945 }, { 139,-4945 }, { 140,-4945 }, { 141,-4945 }, { 142,-4945 },
 { 143,-4945 }, { 144,-4945 }, { 145,-4945 }, { 146,-4945 }, { 147,-4945 },
 { 148,-4945 }, { 149,-4945 }, { 150,-4945 }, { 151,-4945 }, { 152,-4945 },

 { 153,-4945 }, { 154,-4945 }, { 155,-4945 }, { 156,-4945 }, { 157,-4945 },
 { 158,-4945 }, { 159,-4945 }, { 160,-4945 }, { 161,-4945 }, { 162,-4945 },
 { 163,-4945 }, { 164,-4945 }, { 165,-4945 }, { 166,-4945 }, { 167,-4945 },
 { 168,-4945 }, { 169,-4945 }, { 170,-4945 }, { 171,-4945 }, { 172,-4945 },
 { 173,-4945 }, { 174,-4945 }, { 175,-4945 }, { 176,-4945 }, { 177,-4945 },
 { 178,-4945 }, { 179,-4945 }, { 180,-4945 }, { 181,-4945 }, { 182,-4945 },
 { 183,-4945 }, { 184,-4945 }, { 185,-4945 }, { 186,-4945 }, { 187,-4945 },
 { 188,-4945 }, { 189,-4945 }, { 190,-4945 }, { 191,-4945 }, { 192,-4945 },
 { 193,-4945 }, { 194,-4945 }, { 195,-4945 }, { 196,-4945 }, { 197,-4945 },
 { 198,-4945 }, { 199,-4945 }, { 200,-4945 }, { 201,-4945 }, { 202,-4945 },

 { 203,-4945 }, { 204,-4945 }, { 205,-4945 }, { 206,-4945 }, { 207,-4945 },
 { 208,-4945 }, { 209,-4945 }, { 210,-4945 }, { 211,-4945 }, { 212,-4945 },
 { 213,-4945 }, { 214,-4945 }, { 215,-4945 }, { 216,-4945 }, { 217,-4945 },
 { 218,-4945 }, { 219,-4945 }, { 220,-4945 }, { 221,-4945 }, { 222,-4945 },
 { 223,-4945 }, { 224,-4945 }, { 225,-4945 }, { 226,-4945 }, { 227,-4945 },
 { 228,-4945 }, { 229,-4945 }, { 230,-4945 }, { 231,-4945 }, { 232,-4945 },
 { 233,-4945 }, { 234,-4945 }, { 235,-4945 }, { 236,-4945 }, { 237,-4945 },
 { 238,-4945 }, { 239,-4945 }, { 240,-4945 }, { 241,-4945 }, { 242,-4945 },
 { 243,-4945 }, { 244,-4945 }, { 245,-4945 }, { 246,-4945 }, { 247,-4945 },
 { 248,-4945 }, { 249,-4945 }, { 250,-4945 }, { 251,-4945 }, { 252,-4945 },

 { 253,-4945 }, { 254,-4945 }, { 255,-4945 }, { 256,-4945 }, {   0,  16 },
 {   0,11470 }, {   1,   0 }, {   2,   0 }, {   3,   0 }, {   4,   0 },
 {   5,   0 }, {   6,   0 }, {   7,   0 }, {   8,   0 }, {   9, 258 },
 {  10, 516 }, {  11,   0 }, {  12, 258 }, {  13, 516 }, {  14,   0 },
 {  15,   0 }, {  16,   0 }, {  17,   0 }, {  18,   0 }, {  19,   0 },
 {  20,   0 }, {  21,   0 }, {  22,   0 }, {  23,   0 }, {  24,   0 },
 {  25,   0 }, {  26,   0 }, {  27,   0 }, {  28,   0 }, {  29,   0 },
 {  30,   0 }, {  31,   0 }, {  32, 258 }, {  33,   0 }, {  34,   0 },
 {  35,   0 }, {  36,   0 }, {  37,   0 }, {  38,   0 }, {  39,   0 },
 {  40,   0 }, {  41,   0 }, {  42,   0 }, {  43,   0 }, {  44,   0 },

 {  45, 563 }, {  46,   0 }, {  47,   0 }, {  48,   0 }, {  49,   0 },
 {  50,   0 }, {  51,   0 }, {  52,   0 }, {  53,   0 }, {  54,   0 },
 {  55,   0 }, {  56,   0 }, {  57,   0 }, {  58,   0 }, {  59,   0 },
 {  60,   0 }, {  61,   0 }, {  62,   0 }, {  63,   0 }, {  64,   0 },
 {  65,   0 }, {  66,   0 }, {  67,   0 }, {  68,   0 }, {  69,   0 },
 {  70,   0 }, {  71,   0 }, {  72,   0 }, {  73,   0 }, {  74,   0 },
 {  75,   0 }, {  76,   0 }, {  77,   0 }, {  78,   0 }, {  79,   0 },
 {  80,   0 }, {  81,   0 }, {  82,   0 }, {  83,   0 }, {  84,   0 },
 {  85,   0 }, {  86,   0 }, {  87,   0 }, {  88,   0 }, {  89,   0 },
 {  90,   0 }, {  91,   0 }, {  92,   0 }, {  93,   0 }, {  94,   0 },

 {  95,   0 }, {  96,   0 }, {  97,   0 }, {  98,   0 }, {  99,   0 },
 { 100,   0 }, { 101,   0 }, { 102,   0 }, { 103,   0 }, { 104,   0 },
 { 105,   0 }, { 106,   0 }, { 107,   0 }, { 108,   0 }, { 109,   0 },
 { 110,   0 }, { 111,   0 }, { 112,   0 }, { 113,   0 }, { 114,   0 },
 { 115,   0 }, { 116,   0 }, { 117,   0 }, { 118,   0 }, { 119,   0 },
 { 120,   0 }, { 121,   0 }, { 122,   0 }, { 123,   0 }, { 124,   0 },
 { 125,   0 }, { 126,   0 }, { 127,   0 }, { 128,   0 }, { 129,   0 },
 { 130,   0 }, { 131,   0 }, { 132,   0 }, { 133,   0 }, { 134,   0 },
 { 135,   0 }, { 136,   0 }, { 137,   0 }, { 138,   0 }, { 139,   0 },
 { 140,   0 }, { 141,   0 }, { 142,   0 }, { 143,   0 }, { 144,   0 },

 { 145,   0 }, { 146,   0 }, { 147,   0 }, { 148,   0 }, { 149,   0 },
 { 150,   0 }, { 151,   0 }, { 152,   0 }, { 153,   0 }, { 154,   0 },
 { 155,   0 }, { 156,   0 }, { 157,   0 }, { 158,   0 }, { 159,   0 },
 { 160,   0 }, { 161,   0 }, { 162,   0 }, { 163,   0 }, { 164,   0 },
 { 165,   0 }, { 166,   0 }, { 167,   0 }, { 168,   0 }, { 169,   0 },
 { 170,   0 }, { 171,   0 }, { 172,   0 }, { 173,   0 }, { 174,   0 },
 { 175,   0 }, { 176,   0 }, { 177,   0 }, { 178,   0 }, { 179,   0 },
 { 180,   0 }, { 181,   0 }, { 182,   0 }, { 183,   0 }, { 184,   0 },
 { 185,   0 }, { 186,   0 }, { 187,   0 }, { 188,   0 }, { 189,   0 },
 { 190,   0 }, { 191,   0 }, { 192,   0 }, { 193,   0 }, { 194,   0 },

 { 195,   0 }, { 196,   0 }, { 197,   0 }, { 198,   0 }, { 199,   0 },
 { 200,   0 }, { 201,   0 }, { 202,   0 }, { 203,   0 }, { 204,   0 },
 { 205,   0 }, { 206,   0 }, { 207,   0 }, { 208,   0 }, { 209,   0 },
 { 210,   0 }, { 211,   0 }, { 212,   0 }, { 213,   0 }, { 214,   0 },
 { 215,   0 }, { 216,   0 }, { 217,   0 }, { 218,   0 }, { 219,   0 },
 { 220,   0 }, { 221,   0 }, { 222,   0 }, { 223,   0 }, { 224,   0 },
 { 225,   0 }, { 226,   0 }, { 227,   0 }, { 228,   0 }, { 229,   0 },
 { 230,   0 }, { 231,   0 }, { 232,   0 }, { 233,   0 }, { 234,   0 },
 { 235,   0 }, { 236,   0 }, { 237,   0 }, { 238,   0 }, { 239,   0 },
 { 240,   0 }, { 241,   0 }, { 242,   0 }, { 243,   0 }, { 244,   0 },

 { 245,   0 }, { 246,   0 }, { 247,   0 }, { 248,   0 }, { 249,   0 },
 { 250,   0 }, { 251,   0 }, { 252,   0 }, { 253,   0 }, { 254,   0 },
 { 255,   0 }, { 256,   0 }, {   0,  16 }, {   0,11212 }, {   1,-258 },
 {   2,-258 }, {   3,-258 }, {   4,-258 }, {   5,-258 }, {   6,-258 },
 {   7,-258 }, {   8,-258 }, {   9,   0 }, {  10, 258 }, {  11,-258 },
 {  12,   0 }, {  13, 258 }, {  14,-258 }, {  15,-258 }, {  16,-258 },
 {  17,-258 }, {  18,-258 }, {  19,-258 }, {  20,-258 }, {  21,-258 },
 {  22,-258 }, {  23,-258 }, {  24,-258 }, {  25,-258 }, {  26,-258 },
 {  27,-258 }, {  28,-258 }, {  29,-258 }, {  30,-258 }, {  31,-258 },
 {  32,   0 }, {  33,-258 }, {  34,-258 }, {  35,-258 }, {  36,-258 },

 {  37,-258 }, {  38,-258 }, {  39,-258 }, {  40,-258 }, {  41,-258 },
 {  42,-258 }, {  43,-258 }, {  44,-258 }, {  45, 305 }, {  46,-258 },
 {  47,-258 }, {  48,-258 }, {  49,-258 }, {  50,-258 }, {  51,-258 },
 {  52,-258 }, {  53,-258 }, {  54,-258 }, {  55,-258 }, {  56,-258 },
 {  57,-258 }, {  58,-258 }, {  59,-258 }, {  60,-258 }, {  61,-258 },
 {  62,-258 }, {  63,-258 }, {  64,-258 }, {  65,-258 }, {  66,-258 },
 {  67,-258 }, {  68,-258 }, {  69,-258 }, {  70,-258 }, {  71,-258 },
 {  72,-258 }, {  73,-258 }, {  74,-258 }, {  75,-258 }, {  76,-258 },
 {  77,-258 }, {  78,-258 }, {  79,-258 }, {  80,-258 }, {  81,-258 },
 {  82,-258 }, {  83,-258 }, {  84,-258 }, {  85,-258 }, {  86,-258 },

 {  87,-258 }, {  88,-258 }, {  89,-258 }, {  90,-258 }, {  91,-258 },
 {  92,-258 }, {  93,-258 }, {  94,-258 }, {  95,-258 }, {  96,-258 },
 {  97,-258 }, {  98,-258 }, {  99,-258 }, { 100,-258 }, { 101,-258 },
 { 102,-258 }, { 103,-258 }, { 104,-258 }, { 105,-258 }, { 106,-258 },
 { 107,-258 }, { 108,-258 }, { 109,-258 }, { 110,-258 }, { 111,-258 },
 { 112,-258 }, { 113,-258 }, { 114,-258 }, { 115,-258 }, { 116,-258 },
 { 117,-258 }, { 118,-258 }, { 119,-258 }, { 120,-258 }, { 121,-258 },
 { 122,-258 }, { 123,-258 }, { 124,-258 }, { 125,-258 }, { 126,-258 },
 { 127,-258 }, { 128,-258 }, { 129,-258 }, { 130,-258 }, { 131,-258 },
 { 132,-258 }, { 133,-258 }, { 134,-258 }, { 135,-258 }, { 136,-258 },

 { 137,-258 }, { 138,-258 }, { 139,-258 }, { 140,-258 }, { 141,-258 },
 { 142,-258 }, { 143,-258 }, { 144,-258 }, { 145,-258 }, { 146,-258 },
 { 147,-258 }, { 148,-258 }, { 149,-258 }, { 150,-258 }, { 151,-258 },
 { 152,-258 }, { 153,-258 }, { 154,-258 }, { 155,-258 }, { 156,-258 },
 { 157,-258 }, { 158,-258 }, { 159,-258 }, { 160,-258 }, { 161,-258 },
 { 162,-258 }, { 163,-258 }, { 164,-258 }, { 165,-258 }, { 166,-258 },
 { 167,-258 }, { 168,-258 }, { 169,-258 }, { 170,-258 }, { 171,-258 },
 { 172,-258 }, { 173,-258 }, { 174,-258 }, { 175,-258 }, { 176,-258 },
 { 177,-258 }, { 178,-258 }, { 179,-258 }, { 180,-258 }, { 181,-258 },
 { 182,-258 }, { 183,-258 }, { 184,-258 }, { 185,-258 }, { 186,-258 },

 { 187,-258 }, { 188,-258 }, { 189,-258 }, { 190,-258 }, { 191,-258 },
 { 192,-258 }, { 193,-258 }, { 194,-258 }, { 195,-258 }, { 196,-258 },
 { 197,-258 }, { 198,-258 }, { 199,-258 }, { 200,-258 }, { 201,-258 },
 { 202,-258 }, { 203,-258 }, { 204,-258 }, { 205,-258 }, { 206,-258 },
 { 207,-258 }, { 208,-258 }, { 209,-258 }, { 210,-258 }, { 211,-258 },
 { 212,-258 }, { 213,-258 }, { 214,-258 }, { 215,-258 }, { 216,-258 },
 { 217,-258 }, { 218,-258 }, { 219,-258 }, { 220,-258 }, { 221,-258 },
 { 222,-258 }, { 223,-258 }, { 224,-258 }, { 225,-258 }, { 226,-258 },
 { 227,-258 }, { 228,-258 }, { 229,-258 }, { 230,-258 }, { 231,-258 },
 { 232,-258 }, { 233,-258 }, { 234,-258 }, { 235,-258 }, { 236,-258 },

 { 237,-258 }, { 238,-258 }, { 239,-258 }, { 240,-258 }, { 241,-258 },
 { 242,-258 }, { 243,-258 }, { 244,-258 }, { 245,-258 }, { 246,-258 },
 { 247,-258 }, { 248,-258 }, { 249,-258 }, { 250,-258 }, { 251,-258 },
 { 252,-258 }, { 253,-258 }, { 254,-258 }, { 255,-258 }, { 256,-258 },
 {   0,  16 }, {   0,10954 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   9,-8007 }, {  10,-8007 }, {   0,   0 }, {  12,-8007 }, {  13,-8007 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  32,-8007 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  39,-18915 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  45,-18910 }, {   0,  16 }, {   0,10907 }, {   1,-563 },
 {   2,-563 }, {   3,-563 }, {   4,-563 }, {   5,-563 }, {   6,-563 },
 {   7,-563 }, {   8,-563 }, {   9,-305 }, {  10, -47 }, {  11,-563 },
 {  12,-305 }, {  13, -47 }, {  14,-563 }, {  15,-563 }, {  16,-563 },
 {  17,-563 }, {  18,-563 }, {  19,-563 }, {  20,-563 }, {  21,-563 },
 {  22,-563 }, {  23,-563 }, {  24,-563 }, {  25,-563 }, {  26,-563 },
 {  27,-563 }, {  28,-563 }, {  29,-563 }, {  30,-563 }, {  31,-563 },

 {  32,-305 }, {  33,-563 }, {  34,-563 }, {  35,-563 }, {  36,-563 },
 {  37,-563 }, {  38,-563 }, {  39,-563 }, {  40,-563 }, {  41,-563 },
 {  42,-563 }, {  43,-563 }, {  44,-563 }, {  45,3186 }, {  46,-563 },
 {  47,-563 }, {  48,-563 }, {  49,-563 }, {  50,-563 }, {  51,-563 },
 {  52,-563 }, {  53,-563 }, {  54,-563 }, {  55,-563 }, {  56,-563 },
 {  57,-563 }, {  58,-563 }, {  59,-563 }, {  60,-563 }, {  61,-563 },
 {  62,-563 }, {  63,-563 }, {  64,-563 }, {  65,-563 }, {  66,-563 },
 {  67,-563 }, {  68,-563 }, {  69,-563 }, {  70,-563 }, {  71,-563 },
 {  72,-563 }, {  73,-563 }, {  74,-563 }, {  75,-563 }, {  76,-563 },
 {  77,-563 }, {  78,-563 }, {  79,-563 }, {  80,-563 }, {  81,-563 },

 {  82,-563 }, {  83,-563 }, {  84,-563 }, {  85,-563 }, {  86,-563 },
 {  87,-563 }, {  88,-563 }, {  89,-563 }, {  90,-563 }, {  91,-563 },
 {  92,-563 }, {  93,-563 }, {  94,-563 }, {  95,-563 }, {  96,-563 },
 {  97,-563 }, {  98,-563 }, {  99,-563 }, { 100,-563 }, { 101,-563 },
 { 102,-563 }, { 103,-563 }, { 104,-563 }, { 105,-563 }, { 106,-563 },
 { 107,-563 }, { 108,-563 }, { 109,-563 }, { 110,-563 }, { 111,-563 },
 { 112,-563 }, { 113,-563 }, { 114,-563 }, { 115,-563 }, { 116,-563 },
 { 117,-563 }, { 118,-563 }, { 119,-563 }, { 120,-563 }, { 121,-563 },
 { 122,-563 }, { 123,-563 }, { 124,-563 }, { 125,-563 }, { 126,-563 },
 { 127,-563 }, { 128,-563 }, { 129,-563 }, { 130,-563 }, { 131,-563 },

 { 132,-563 }, { 133,-563 }, { 134,-563 }, { 135,-563 }, { 136,-563 },
 { 137,-563 }, { 138,-563 }, { 139,-563 }, { 140,-563 }, { 141,-563 },
 { 142,-563 }, { 143,-563 }, { 144,-563 }, { 145,-563 }, { 146,-563 },
 { 147,-563 }, { 148,-563 }, { 149,-563 }, { 150,-563 }, { 151,-563 },
 { 152,-563 }, { 153,-563 }, { 154,-563 }, { 155,-563 }, { 156,-563 },
 { 157,-563 }, { 158,-563 }, { 159,-563 }, { 160,-563 }, { 161,-563 },
 { 162,-563 }, { 163,-563 }, { 164,-563 }, { 165,-563 }, { 166,-563 },
 { 167,-563 }, { 168,-563 }, { 169,-563 }, { 170,-563 }, { 171,-563 },
 { 172,-563 }, { 173,-563 }, { 174,-563 }, { 175,-563 }, { 176,-563 },
 { 177,-563 }, { 178,-563 }, { 179,-563 }, { 180,-563 }, { 181,-563 },

 { 182,-563 }, { 183,-563 }, { 184,-563 }, { 185,-563 }, { 186,-563 },
 { 187,-563 }, { 188,-563 }, { 189,-563 }, { 190,-563 }, { 191,-563 },
 { 192,-563 }, { 193,-563 }, { 194,-563 }, { 195,-563 }, { 196,-563 },
 { 197,-563 }, { 198,-563 }, { 199,-563 }, { 200,-563 }, { 201,-563 },
 { 202,-563 }, { 203,-563 }, { 204,-563 }, { 205,-563 }, { 206,-563 },
 { 207,-563 }, { 208,-563 }, { 209,-563 }, { 210,-563 }, { 211,-563 },
 { 212,-563 }, { 213,-563 }, { 214,-563 }, { 215,-563 }, { 216,-563 },
 { 217,-563 }, { 218,-563 }, { 219,-563 }, { 220,-563 }, { 221,-563 },
 { 222,-563 }, { 223,-563 }, { 224,-563 }, { 225,-563 }, { 226,-563 },
 { 227,-563 }, { 228,-563 }, { 229,-563 }, { 230,-563 }, { 231,-563 },

 { 232,-563 }, { 233,-563 }, { 234,-563 }, { 235,-563 }, { 236,-563 },
 { 237,-563 }, { 238,-563 }, { 239,-563 }, { 240,-563 }, { 241,-563 },
 { 242,-563 }, { 243,-563 }, { 244,-563 }, { 245,-563 }, { 246,-563 },
 { 247,-563 }, { 248,-563 }, { 249,-563 }, { 250,-563 }, { 251,-563 },
 { 252,-563 }, { 253,-563 }, { 254,-563 }, { 255,-563 }, { 256,-563 },
 {   0,  16 }, {   0,10649 }, {   1,-4992 }, {   2,-4992 }, {   3,-4992 },
 {   4,-4992 }, {   5,-4992 }, {   6,-4992 }, {   7,-4992 }, {   8,-4992 },
 {   9,-4734 }, {  10,-11476 }, {  11,-4992 }, {  12,-4734 }, {  13,-11476 },
 {  14,-4992 }, {  15,-4992 }, {  16,-4992 }, {  17,-4992 }, {  18,-4992 },
 {  19,-4992 }, {  20,-4992 }, {  21,-4992 }, {  22,-4992 }, {  23,-4992 },

 {  24,-4992 }, {  25,-4992 }, {  26,-4992 }, {  27,-4992 }, {  28,-4992 },
 {  29,-4992 }, {  30,-4992 }, {  31,-4992 }, {  32,-4734 }, {  33,-4992 },
 {  34,-4992 }, {  35,-4992 }, {  36,-4992 }, {  37,-4992 }, {  38,-4992 },
 {  39,-4992 }, {  40,-4992 }, {  41,-4992 }, {  42,-4992 }, {  43,-4992 },
 {  44,-4992 }, {  45,   0 }, {  46,-4992 }, {  47,-4992 }, {  48,-4992 },
 {  49,-4992 }, {  50,-4992 }, {  51,-4992 }, {  52,-4992 }, {  53,-4992 },
 {  54,-4992 }, {  55,-4992 }, {  56,-4992 }, {  57,-4992 }, {  58,-4992 },
 {  59,-4992 }, {  60,-4992 }, {  61,-4992 }, {  62,-4992 }, {  63,-4992 },
 {  64,-4992 }, {  65,-4992 }, {  66,-4992 }, {  67,-4992 }, {  68,-4992 },
 {  69,-4992 }, {  70,-4992 }, {  71,-4992 }, {  72,-4992 }, {  73,-4992 },

 {  74,-4992 }, {  75,-4992 }, {  76,-4992 }, {  77,-4992 }, {  78,-4992 },
 {  79,-4992 }, {  80,-4992 }, {  81,-4992 }, {  82,-4992 }, {  83,-4992 },
 {  84,-4992 }, {  85,-4992 }, {  86,-4992 }, {  87,-4992 }, {  88,-4992 },
 {  89,-4992 }, {  90,-4992 }, {  91,-4992 }, {  92,-4992 }, {  93,-4992 },
 {  94,-4992 }, {  95,-4992 }, {  96,-4992 }, {  97,-4992 }, {  98,-4992 },
 {  99,-4992 }, { 100,-4992 }, { 101,-4992 }, { 102,-4992 }, { 103,-4992 },
 { 104,-4992 }, { 105,-4992 }, { 106,-4992 }, { 107,-4992 }, { 108,-4992 },
 { 109,-4992 }, { 110,-4992 }, { 111,-4992 }, { 112,-4992 }, { 113,-4992 },
 { 114,-4992 }, { 115,-4992 }, { 116,-4992 }, { 117,-4992 }, { 118,-4992 },
 { 119,-4992 }, { 120,-4992 }, { 121,-4992 }, { 122,-4992 }, { 123,-4992 },

 { 124,-4992 }, { 125,-4992 }, { 126,-4992 }, { 127,-4992 }, { 128,-4992 },
 { 129,-4992 }, { 130,-4992 }, { 131,-4992 }, { 132,-4992 }, { 133,-4992 },
 { 134,-4992 }, { 135,-4992 }, { 136,-4992 }, { 137,-4992 }, { 138,-4992 },
 { 139,-4992 }, { 140,-4992 }, { 141,-4992 }, { 142,-4992 }, { 143,-4992 },
 { 144,-4992 }, { 145,-4992 }, { 146,-4992 }, { 147,-4992 }, { 148,-4992 },
 { 149,-4992 }, { 150,-4992 }, { 151,-4992 }, { 152,-4992 }, { 153,-4992 },
 { 154,-4992 }, { 155,-4992 }, { 156,-4992 }, { 157,-4992 }, { 158,-4992 },
 { 159,-4992 }, { 160,-4992 }, { 161,-4992 }, { 162,-4992 }, { 163,-4992 },
 { 164,-4992 }, { 165,-4992 }, { 166,-4992 }, { 167,-4992 }, { 168,-4992 },
 { 169,-4992 }, { 170,-4992 }, { 171,-4992 }, { 172,-4992 }, { 173,-4992 },

 { 174,-4992 }, { 175,-4992 }, { 176,-4992 }, { 177,-4992 }, { 178,-4992 },
 { 179,-4992 }, { 180,-4992 }, { 181,-4992 }, { 182,-4992 }, { 183,-4992 },
 { 184,-4992 }, { 185,-4992 }, { 186,-4992 }, { 187,-4992 }, { 188,-4992 },
 { 189,-4992 }, { 190,-4992 }, { 191,-4992 }, { 192,-4992 }, { 193,-4992 },
 { 194,-4992 }, { 195,-4992 }, { 196,-4992 }, { 197,-4992 }, { 198,-4992 },
 { 199,-4992 }, { 200,-4992 }, { 201,-4992 }, { 202,-4992 }, { 203,-4992 },
 { 204,-4992 }, { 205,-4992 }, { 206,-4992 }, { 207,-4992 }, { 208,-4992 },
 { 209,-4992 }, { 210,-4992 }, { 211,-4992 }, { 212,-4992 }, { 213,-4992 },
 { 214,-4992 }, { 215,-4992 }, { 216,-4992 }, { 217,-4992 }, { 218,-4992 },
 { 219,-4992 }, { 220,-4992 }, { 221,-4992 }, { 222,-4992 }, { 223,-4992 },

 { 224,-4992 }, { 225,-4992 }, { 226,-4992 }, { 227,-4992 }, { 228,-4992 },
 { 229,-4992 }, { 230,-4992 }, { 231,-4992 }, { 232,-4992 }, { 233,-4992 },
 { 234,-4992 }, { 235,-4992 }, { 236,-4992 }, { 237,-4992 }, { 238,-4992 },
 { 239,-4992 }, { 240,-4992 }, { 241,-4992 }, { 242,-4992 }, { 243,-4992 },
 { 244,-4992 }, { 245,-4992 }, { 246,-4992 }, { 247,-4992 }, { 248,-4992 },
 { 249,-4992 }, { 250,-4992 }, { 251,-4992 }, { 252,-4992 }, { 253,-4992 },
 { 254,-4992 }, { 255,-4992 }, { 256,-4992 }, {   0,  22 }, {   0,10391 },
 {   1,   0 }, {   2,   0 }, {   3,   0 }, {   4,   0 }, {   5,   0 },
 {   6,   0 }, {   7,   0 }, {   8,   0 }, {   9, 258 }, {  10, 516 },
 {  11,   0 }, {  12, 258 }, {  13, 516 }, {  14,   0 }, {  15,   0 },

 {  16,   0 }, {  17,   0 }, {  18,   0 }, {  19,   0 }, {  20,   0 },
 {  21,   0 }, {  22,   0 }, {  23,   0 }, {  24,   0 }, {  25,   0 },
 {  26,   0 }, {  27,   0 }, {  28,   0 }, {  29,   0 }, {  30,   0 },
 {  31,   0 }, {  32, 258 }, {  33,   0 }, {  34,   0 }, {  35,   0 },
 {  36,   0 }, {  37,   0 }, {  38,   0 }, {  39,   0 }, {  40,   0 },
 {  41,   0 }, {  42,   0 }, {  43,   0 }, {  44,   0 }, {  45, 563 },
 {  46,   0 }, {  47,   0 }, {  48,   0 }, {  49,   0 }, {  50,   0 },
 {  51,   0 }, {  52,   0 }, {  53,   0 }, {  54,   0 }, {  55,   0 },
 {  56,   0 }, {  57,   0 }, {  58,   0 }, {  59,   0 }, {  60,   0 },
 {  61,   0 }, {  62,   0 }, {  63,   0 }, {  64,   0 }, {  65,   0 },

 {  66,   0 }, {  67,   0 }, {  68,   0 }, {  69,   0 }, {  70,   0 },
 {  71,   0 }, {  72,   0 }, {  73,   0 }, {  74,   0 }, {  75,   0 },
 {  76,   0 }, {  77,   0 }, {  78,   0 }, {  79,   0 }, {  80,   0 },
 {  81,   0 }, {  82,   0 }, {  83,   0 }, {  84,   0 }, {  85,   0 },
 {  86,   0 }, {  87,   0 }, {  88,   0 }, {  89,   0 }, {  90,   0 },
 {  91,   0 }, {  92,   0 }, {  93,   0 }, {  94,   0 }, {  95,   0 },
 {  96,   0 }, {  97,   0 }, {  98,   0 }, {  99,   0 }, { 100,   0 },
 { 101,   0 }, { 102,   0 }, { 103,   0 }, { 104,   0 }, { 105,   0 },
 { 106,   0 }, { 107,   0 }, { 108,   0 }, { 109,   0 }, { 110,   0 },
 { 111,   0 }, { 112,   0 }, { 113,   0 }, { 114,   0 }, { 115,   0 },

 { 116,   0 }, { 117,   0 }, { 118,   0 }, { 119,   0 }, { 120,   0 },
 { 121,   0 }, { 122,   0 }, { 123,   0 }, { 124,   0 }, { 125,   0 },
 { 126,   0 }, { 127,   0 }, { 128,   0 }, { 129,   0 }, { 130,   0 },
 { 131,   0 }, { 132,   0 }, { 133,   0 }, { 134,   0 }, { 135,   0 },
 { 136,   0 }, { 137,   0 }, { 138,   0 }, { 139,   0 }, { 140,   0 },
 { 141,   0 }, { 142,   0 }, { 143,   0 }, { 144,   0 }, { 145,   0 },
 { 146,   0 }, { 147,   0 }, { 148,   0 }, { 149,   0 }, { 150,   0 },
 { 151,   0 }, { 152,   0 }, { 153,   0 }, { 154,   0 }, { 155,   0 },
 { 156,   0 }, { 157,   0 }, { 158,   0 }, { 159,   0 }, { 160,   0 },
 { 161,   0 }, { 162,   0 }, { 163,   0 }, { 164,   0 }, { 165,   0 },

 { 166,   0 }, { 167,   0 }, { 168,   0 }, { 169,   0 }, { 170,   0 },
 { 171,   0 }, { 172,   0 }, { 173,   0 }, { 174,   0 }, { 175,   0 },
 { 176,   0 }, { 177,   0 }, { 178,   0 }, { 179,   0 }, { 180,   0 },
 { 181,   0 }, { 182,   0 }, { 183,   0 }, { 184,   0 }, { 185,   0 },
 { 186,   0 }, { 187,   0 }, { 188,   0 }, { 189,   0 }, { 190,   0 },
 { 191,   0 }, { 192,   0 }, { 193,   0 }, { 194,   0 }, { 195,   0 },
 { 196,   0 }, { 197,   0 }, { 198,   0 }, { 199,   0 }, { 200,   0 },
 { 201,   0 }, { 202,   0 }, { 203,   0 }, { 204,   0 }, { 205,   0 },
 { 206,   0 }, { 207,   0 }, { 208,   0 }, { 209,   0 }, { 210,   0 },
 { 211,   0 }, { 212,   0 }, { 213,   0 }, { 214,   0 }, { 215,   0 },

 { 216,   0 }, { 217,   0 }, { 218,   0 }, { 219,   0 }, { 220,   0 },
 { 221,   0 }, { 222,   0 }, { 223,   0 }, { 224,   0 }, { 225,   0 },
 { 226,   0 }, { 227,   0 }, { 228,   0 }, { 229,   0 }, { 230,   0 },
 { 231,   0 }, { 232,   0 }, { 233,   0 }, { 234,   0 }, { 235,   0 },
 { 236,   0 }, { 237,   0 }, { 238,   0 }, { 239,   0 }, { 240,   0 },
 { 241,   0 }, { 242,   0 }, { 243,   0 }, { 244,   0 }, { 245,   0 },
 { 246,   0 }, { 247,   0 }, { 248,   0 }, { 249,   0 }, { 250,   0 },
 { 251,   0 }, { 252,   0 }, { 253,   0 }, { 254,   0 }, { 255,   0 },
 { 256,   0 }, {   0,  22 }, {   0,10133 }, {   1,-258 }, {   2,-258 },
 {   3,-258 }, {   4,-258 }, {   5,-258 }, {   6,-258 }, {   7,-258 },

 {   8,-258 }, {   9,   0 }, {  10, 258 }, {  11,-258 }, {  12,   0 },
 {  13, 258 }, {  14,-258 }, {  15,-258 }, {  16,-258 }, {  17,-258 },
 {  18,-258 }, {  19,-258 }, {  20,-258 }, {  21,-258 }, {  22,-258 },
 {  23,-258 }, {  24,-258 }, {  25,-258 }, {  26,-258 }, {  27,-258 },
 {  28,-258 }, {  29,-258 }, {  30,-258 }, {  31,-258 }, {  32,   0 },
 {  33,-258 }, {  34,-258 }, {  35,-258 }, {  36,-258 }, {  37,-258 },
 {  38,-258 }, {  39,-258 }, {  40,-258 }, {  41,-258 }, {  42,-258 },
 {  43,-258 }, {  44,-258 }, {  45, 305 }, {  46,-258 }, {  47,-258 },
 {  48,-258 }, {  49,-258 }, {  50,-258 }, {  51,-258 }, {  52,-258 },
 {  53,-258 }, {  54,-258 }, {  55,-258 }, {  56,-258 }, {  57,-258 },

 {  58,-258 }, {  59,-258 }, {  60,-258 }, {  61,-258 }, {  62,-258 },
 {  63,-258 }, {  64,-258 }, {  65,-258 }, {  66,-258 }, {  67,-258 },
 {  68,-258 }, {  69,-258 }, {  70,-258 }, {  71,-258 }, {  72,-258 },
 {  73,-258 }, {  74,-258 }, {  75,-258 }, {  76,-258 }, {  77,-258 },
 {  78,-258 }, {  79,-258 }, {  80,-258 }, {  81,-258 }, {  82,-258 },
 {  83,-258 }, {  84,-258 }, {  85,-258 }, {  86,-258 }, {  87,-258 },
 {  88,-258 }, {  89,-258 }, {  90,-258 }, {  91,-258 }, {  92,-258 },
 {  93,-258 }, {  94,-258 }, {  95,-258 }, {  96,-258 }, {  97,-258 },
 {  98,-258 }, {  99,-258 }, { 100,-258 }, { 101,-258 }, { 102,-258 },
 { 103,-258 }, { 104,-258 }, { 105,-258 }, { 106,-258 }, { 107,-258 },

 { 108,-258 }, { 109,-258 }, { 110,-258 }, { 111,-258 }, { 112,-258 },
 { 113,-258 }, { 114,-258 }, { 115,-258 }, { 116,-258 }, { 117,-258 },
 { 118,-258 }, { 119,-258 }, { 120,-258 }, { 121,-258 }, { 122,-258 },
 { 123,-258 }, { 124,-258 }, { 125,-258 }, { 126,-258 }, { 127,-258 },
 { 128,-258 }, { 129,-258 }, { 130,-258 }, { 131,-258 }, { 132,-258 },
 { 133,-258 }, { 134,-258 }, { 135,-258 }, { 136,-258 }, { 137,-258 },
 { 138,-258 }, { 139,-258 }, { 140,-258 }, { 141,-258 }, { 142,-258 },
 { 143,-258 }, { 144,-258 }, { 145,-258 }, { 146,-258 }, { 147,-258 },
 { 148,-258 }, { 149,-258 }, { 150,-258 }, { 151,-258 }, { 152,-258 },
 { 153,-258 }, { 154,-258 }, { 155,-258 }, { 156,-258 }, { 157,-258 },

 { 158,-258 }, { 159,-258 }, { 160,-258 }, { 161,-258 }, { 162,-258 },
 { 163,-258 }, { 164,-258 }, { 165,-258 }, { 166,-258 }, { 167,-258 },
 { 168,-258 }, { 169,-258 }, { 170,-258 }, { 171,-258 }, { 172,-258 },
 { 173,-258 }, { 174,-258 }, { 175,-258 }, { 176,-258 }, { 177,-258 },
 { 178,-258 }, { 179,-258 }, { 180,-258 }, { 181,-258 }, { 182,-258 },
 { 183,-258 }, { 184,-258 }, { 185,-258 }, { 186,-258 }, { 187,-258 },
 { 188,-258 }, { 189,-258 }, { 190,-258 }, { 191,-258 }, { 192,-258 },
 { 193,-258 }, { 194,-258 }, { 195,-258 }, { 196,-258 }, { 197,-258 },
 { 198,-258 }, { 199,-258 }, { 200,-258 }, { 201,-258 }, { 202,-258 },
 { 203,-258 }, { 204,-258 }, { 205,-258 }, { 206,-258 }, { 207,-258 },

 { 208,-258 }, { 209,-258 }, { 210,-258 }, { 211,-258 }, { 212,-258 },
 { 213,-258 }, { 214,-258 }, { 215,-258 }, { 216,-258 }, { 217,-258 },
 { 218,-258 }, { 219,-258 }, { 220,-258 }, { 221,-258 }, { 222,-258 },
 { 223,-258 }, { 224,-258 }, { 225,-258 }, { 226,-258 }, { 227,-258 },
 { 228,-258 }, { 229,-258 }, { 230,-258 }, { 231,-258 }, { 232,-258 },
 { 233,-258 }, { 234,-258 }, { 235,-258 }, { 236,-258 }, { 237,-258 },
 { 238,-258 }, { 239,-258 }, { 240,-258 }, { 241,-258 }, { 242,-258 },
 { 243,-258 }, { 244,-258 }, { 245,-258 }, { 246,-258 }, { 247,-258 },
 { 248,-258 }, { 249,-258 }, { 250,-258 }, { 251,-258 }, { 252,-258 },
 { 253,-258 }, { 254,-258 }, { 255,-258 }, { 256,-258 }, {   0,  22 },

 {   0,9875 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   9,-8707 },
 {  10,-8707 }, {   0,   0 }, {  12,-8707 }, {  13,-8707 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  32,-8707 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  39,-19987 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  45,-19966 }, {   0,  22 }, {   0,9828 }, {   1,-563 }, {   2,-563 },

 {   3,-563 }, {   4,-563 }, {   5,-563 }, {   6,-563 }, {   7,-563 },
 {   8,-563 }, {   9,-305 }, {  10, -47 }, {  11,-563 }, {  12,-305 },
 {  13, -47 }, {  14,-563 }, {  15,-563 }, {  16,-563 }, {  17,-563 },
 {  18,-563 }, {  19,-563 }, {  20,-563 }, {  21,-563 }, {  22,-563 },
 {  23,-563 }, {  24,-563 }, {  25,-563 }, {  26,-563 }, {  27,-563 },
 {  28,-563 }, {  29,-563 }, {  30,-563 }, {  31,-563 }, {  32,-305 },
 {  33,-563 }, {  34,-563 }, {  35,-563 }, {  36,-563 }, {  37,-563 },
 {  38,-563 }, {  39,-563 }, {  40,-563 }, {  41,-563 }, {  42,-563 },
 {  43,-563 }, {  44,-563 }, {  45,2365 }, {  46,-563 }, {  47,-563 },
 {  48,-563 }, {  49,-563 }, {  50,-563 }, {  51,-563 }, {  52,-563 },

 {  53,-563 }, {  54,-563 }, {  55,-563 }, {  56,-563 }, {  57,-563 },
 {  58,-563 }, {  59,-563 }, {  60,-563 }, {  61,-563 }, {  62,-563 },
 {  63,-563 }, {  64,-563 }, {  65,-563 }, {  66,-563 }, {  67,-563 },
 {  68,-563 }, {  69,-563 }, {  70,-563 }, {  71,-563 }, {  72,-563 },
 {  73,-563 }, {  74,-563 }, {  75,-563 }, {  76,-563 }, {  77,-563 },
 {  78,-563 }, {  79,-563 }, {  80,-563 }, {  81,-563 }, {  82,-563 },
 {  83,-563 }, {  84,-563 }, {  85,-563 }, {  86,-563 }, {  87,-563 },
 {  88,-563 }, {  89,-563 }, {  90,-563 }, {  91,-563 }, {  92,-563 },
 {  93,-563 }, {  94,-563 }, {  95,-563 }, {  96,-563 }, {  97,-563 },
 {  98,-563 }, {  99,-563 }, { 100,-563 }, { 101,-563 }, { 102,-563 },

 { 103,-563 }, { 104,-563 }, { 105,-563 }, { 106,-563 }, { 107,-563 },
 { 108,-563 }, { 109,-563 }, { 110,-563 }, { 111,-563 }, { 112,-563 },
 { 113,-563 }, { 114,-563 }, { 115,-563 }, { 116,-563 }, { 117,-563 },
 { 118,-563 }, { 119,-563 }, { 120,-563 }, { 121,-563 }, { 122,-563 },
 { 123,-563 }, { 124,-563 }, { 125,-563 }, { 126,-563 }, { 127,-563 },
 { 128,-563 }, { 129,-563 }, { 130,-563 }, { 131,-563 }, { 132,-563 },
 { 133,-563 }, { 134,-563 }, { 135,-563 }, { 136,-563 }, { 137,-563 },
 { 138,-563 }, { 139,-563 }, { 140,-563 }, { 141,-563 }, { 142,-563 },
 { 143,-563 }, { 144,-563 }, { 145,-563 }, { 146,-563 }, { 147,-563 },
 { 148,-563 }, { 149,-563 }, { 150,-563 }, { 151,-563 }, { 152,-563 },

 { 153,-563 }, { 154,-563 }, { 155,-563 }, { 156,-563 }, { 157,-563 },
 { 158,-563 }, { 159,-563 }, { 160,-563 }, { 161,-563 }, { 162,-563 },
 { 163,-563 }, { 164,-563 }, { 165,-563 }, { 166,-563 }, { 167,-563 },
 { 168,-563 }, { 169,-563 }, { 170,-563 }, { 171,-563 }, { 172,-563 },
 { 173,-563 }, { 174,-563 }, { 175,-563 }, { 176,-563 }, { 177,-563 },
 { 178,-563 }, { 179,-563 }, { 180,-563 }, { 181,-563 }, { 182,-563 },
 { 183,-563 }, { 184,-563 }, { 185,-563 }, { 186,-563 }, { 187,-563 },
 { 188,-563 }, { 189,-563 }, { 190,-563 }, { 191,-563 }, { 192,-563 },
 { 193,-563 }, { 194,-563 }, { 195,-563 }, { 196,-563 }, { 197,-563 },
 { 198,-563 }, { 199,-563 }, { 200,-563 }, { 201,-563 }, { 202,-563 },

 { 203,-563 }, { 204,-563 }, { 205,-563 }, { 206,-563 }, { 207,-563 },
 { 208,-563 }, { 209,-563 }, { 210,-563 }, { 211,-563 }, { 212,-563 },
 { 213,-563 }, { 214,-563 }, { 215,-563 }, { 216,-563 }, { 217,-563 },
 { 218,-563 }, { 219,-563 }, { 220,-563 }, { 221,-563 }, { 222,-563 },
 { 223,-563 }, { 224,-563 }, { 225,-563 }, { 226,-563 }, { 227,-563 },
 { 228,-563 }, { 229,-563 }, { 230,-563 }, { 231,-563 }, { 232,-563 },
 { 233,-563 }, { 234,-563 }, { 235,-563 }, { 236,-563 }, { 237,-563 },
 { 238,-563 }, { 239,-563 }, { 240,-563 }, { 241,-563 }, { 242,-563 },
 { 243,-563 }, { 244,-563 }, { 245,-563 }, { 246,-563 }, { 247,-563 },
 { 248,-563 }, { 249,-563 }, { 250,-563 }, { 251,-563 }, { 252,-563 },

 { 253,-563 }, { 254,-563 }, { 255,-563 }, { 256,-563 }, {   0,  22 },
 {   0,9570 }, {   1,-5039 }, {   2,-5039 }, {   3,-5039 }, {   4,-5039 },
 {   5,-5039 }, {   6,-5039 }, {   7,-5039 }, {   8,-5039 }, {   9,-4781 },
 {  10,-12245 }, {  11,-5039 }, {  12,-4781 }, {  13,-12245 }, {  14,-5039 },
 {  15,-5039 }, {  16,-5039 }, {  17,-5039 }, {  18,-5039 }, {  19,-5039 },
 {  20,-5039 }, {  21,-5039 }, {  22,-5039 }, {  23,-5039 }, {  24,-5039 },
 {  25,-5039 }, {  26,-5039 }, {  27,-5039 }, {  28,-5039 }, {  29,-5039 },
 {  30,-5039 }, {  31,-5039 }, {  32,-4781 }, {  33,-5039 }, {  34,-5039 },
 {  35,-5039 }, {  36,-5039 }, {  37,-5039 }, {  38,-5039 }, {  39,-5039 },
 {  40,-5039 }, {  41,-5039 }, {  42,-5039 }, {  43,-5039 }, {  44,-5039 },

 {  45,   0 }, {  46,-5039 }, {  47,-5039 }, {  48,-5039 }, {  49,-5039 },
 {  50,-5039 }, {  51,-5039 }, {  52,-5039 }, {  53,-5039 }, {  54,-5039 },
 {  55,-5039 }, {  56,-5039 }, {  57,-5039 }, {  58,-5039 }, {  59,-5039 },
 {  60,-5039 }, {  61,-5039 }, {  62,-5039 }, {  63,-5039 }, {  64,-5039 },
 {  65,-5039 }, {  66,-5039 }, {  67,-5039 }, {  68,-5039 }, {  69,-5039 },
 {  70,-5039 }, {  71,-5039 }, {  72,-5039 }, {  73,-5039 }, {  74,-5039 },
 {  75,-5039 }, {  76,-5039 }, {  77,-5039 }, {  78,-5039 }, {  79,-5039 },
 {  80,-5039 }, {  81,-5039 }, {  82,-5039 }, {  83,-5039 }, {  84,-5039 },
 {  85,-5039 }, {  86,-5039 }, {  87,-5039 }, {  88,-5039 }, {  89,-5039 },
 {  90,-5039 }, {  91,-5039 }, {  92,-5039 }, {  93,-5039 }, {  94,-5039 },

 {  95,-5039 }, {  96,-5039 }, {  97,-5039 }, {  98,-5039 }, {  99,-5039 },
 { 100,-5039 }, { 101,-5039 }, { 102,-5039 }, { 103,-5039 }, { 104,-5039 },
 { 105,-5039 }, { 106,-5039 }, { 107,-5039 }, { 108,-5039 }, { 109,-5039 },
 { 110,-5039 }, { 111,-5039 }, { 112,-5039 }, { 113,-5039 }, { 114,-5039 },
 { 115,-5039 }, { 116,-5039 }, { 117,-5039 }, { 118,-5039 }, { 119,-5039 },
 { 120,-5039 }, { 121,-5039 }, { 122,-5039 }, { 123,-5039 }, { 124,-5039 },
 { 125,-5039 }, { 126,-5039 }, { 127,-5039 }, { 128,-5039 }, { 129,-5039 },
 { 130,-5039 }, { 131,-5039 }, { 132,-5039 }, { 133,-5039 }, { 134,-5039 },
 { 135,-5039 }, { 136,-5039 }, { 137,-5039 }, { 138,-5039 }, { 139,-5039 },
 { 140,-5039 }, { 141,-5039 }, { 142,-5039 }, { 143,-5039 }, { 144,-5039 },

 { 145,-5039 }, { 146,-5039 }, { 147,-5039 }, { 148,-5039 }, { 149,-5039 },
 { 150,-5039 }, { 151,-5039 }, { 152,-5039 }, { 153,-5039 }, { 154,-5039 },
 { 155,-5039 }, { 156,-5039 }, { 157,-5039 }, { 158,-5039 }, { 159,-5039 },
 { 160,-5039 }, { 161,-5039 }, { 162,-5039 }, { 163,-5039 }, { 164,-5039 },
 { 165,-5039 }, { 166,-5039 }, { 167,-5039 }, { 168,-5039 }, { 169,-5039 },
 { 170,-5039 }, { 171,-5039 }, { 172,-5039 }, { 173,-5039 }, { 174,-5039 },
 { 175,-5039 }, { 176,-5039 }, { 177,-5039 }, { 178,-5039 }, { 179,-5039 },
 { 180,-5039 }, { 181,-5039 }, { 182,-5039 }, { 183,-5039 }, { 184,-5039 },
 { 185,-5039 }, { 186,-5039 }, { 187,-5039 }, { 188,-5039 }, { 189,-5039 },
 { 190,-5039 }, { 191,-5039 }, { 192,-5039 }, { 193,-5039 }, { 194,-5039 },

 { 195,-5039 }, { 196,-5039 }, { 197,-5039 }, { 198,-5039 }, { 199,-5039 },
 { 200,-5039 }, { 201,-5039 }, { 202,-5039 }, { 203,-5039 }, { 204,-5039 },
 { 205,-5039 }, { 206,-5039 }, { 207,-5039 }, { 208,-5039 }, { 209,-5039 },
 { 210,-5039 }, { 211,-5039 }, { 212,-5039 }, { 213,-5039 }, { 214,-5039 },
 { 215,-5039 }, { 216,-5039 }, { 217,-5039 }, { 218,-5039 }, { 219,-5039 },
 { 220,-5039 }, { 221,-5039 }, { 222,-5039 }, { 223,-5039 }, { 224,-5039 },
 { 225,-5039 }, { 226,-5039 }, { 227,-5039 }, { 228,-5039 }, { 229,-5039 },
 { 230,-5039 }, { 231,-5039 }, { 232,-5039 }, { 233,-5039 }, { 234,-5039 },
 { 235,-5039 }, { 236,-5039 }, { 237,-5039 }, { 238,-5039 }, { 239,-5039 },
 { 240,-5039 }, { 241,-5039 }, { 242,-5039 }, { 243,-5039 }, { 244,-5039 },

 { 245,-5039 }, { 246,-5039 }, { 247,-5039 }, { 248,-5039 }, { 249,-5039 },
 { 250,-5039 }, { 251,-5039 }, { 252,-5039 }, { 253,-5039 }, { 254,-5039 },
 { 255,-5039 }, { 256,-5039 }, {   0,  37 }, {   0,9312 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,  37 }, {   0,9289 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  48,2107 }, {  49,2107 }, {  50,2107 }, {  51,2107 },
 {  52,2107 }, {  53,2107 }, {  54,2107 }, {  55,2107 }, {  56,2107 },
 {  57,2107 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65,2107 }, {  66,2107 },
 {  67,2107 }, {  68,2107 }, {  69,2107 }, {  70,2107 }, {  48,-20420 },
 {  49,-20420 }, {  50,-20420 }, {  51,-20420 }, {  52,-20420 }, {  53,-20420 },
 {  54,-20420 }, {  55,-20420 }, {  56,-20420 }, {  57,-20420 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {  65,-20420 }, {  66,-20420 }, {  67,-20420 }, {  68,-20420 },
 {  69,-20420 }, {  70,-20420 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  97,2107 }, {  98,2107 }, {  99,2107 }, { 100,2107 }, { 101,2107 },
 { 102,2107 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  97,-20420 }, {  98,-20420 },
 {  99,-20420 }, { 100,-20420 }, { 101,-20420 }, { 102,-20420 }, {   0,  24 },
 {   0,9185 }, {   1,   0 }, {   2,   0 }, {   3,   0 }, {   4,   0 },
 {   5,   0 }, {   6,   0 }, {   7,   0 }, {   8,   0 }, {   9, 258 },

 {  10, 516 }, {  11,   0 }, {  12, 258 }, {  13, 516 }, {  14,   0 },
 {  15,   0 }, {  16,   0 }, {  17,   0 }, {  18,   0 }, {  19,   0 },
 {  20,   0 }, {  21,   0 }, {  22,   0 }, {  23,   0 }, {  24,   0 },
 {  25,   0 }, {  26,   0 }, {  27,   0 }, {  28,   0 }, {  29,   0 },
 {  30,   0 }, {  31,   0 }, {  32, 258 }, {  33,   0 }, {  34,   0 },
 {  35,   0 }, {  36,   0 }, {  37,   0 }, {  38,   0 }, {  39,   0 },
 {  40,   0 }, {  41,   0 }, {  42,   0 }, {  43,   0 }, {  44,   0 },
 {  45, 563 }, {  46,   0 }, {  47,   0 }, {  48,   0 }, {  49,   0 },
 {  50,   0 }, {  51,   0 }, {  52,   0 }, {  53,   0 }, {  54,   0 },
 {  55,   0 }, {  56,   0 }, {  57,   0 }, {  58,   0 }, {  59,   0 },

 {  60,   0 }, {  61,   0 }, {  62,   0 }, {  63,   0 }, {  64,   0 },
 {  65,   0 }, {  66,   0 }, {  67,   0 }, {  68,   0 }, {  69,   0 },
 {  70,   0 }, {  71,   0 }, {  72,   0 }, {  73,   0 }, {  74,   0 },
 {  75,   0 }, {  76,   0 }, {  77,   0 }, {  78,   0 }, {  79,   0 },
 {  80,   0 }, {  81,   0 }, {  82,   0 }, {  83,   0 }, {  84,   0 },
 {  85,   0 }, {  86,   0 }, {  87,   0 }, {  88,   0 }, {  89,   0 },
 {  90,   0 }, {  91,   0 }, {  92,   0 }, {  93,   0 }, {  94,   0 },
 {  95,   0 }, {  96,   0 }, {  97,   0 }, {  98,   0 }, {  99,   0 },
 { 100,   0 }, { 101,   0 }, { 102,   0 }, { 103,   0 }, { 104,   0 },
 { 105,   0 }, { 106,   0 }, { 107,   0 }, { 108,   0 }, { 109,   0 },

 { 110,   0 }, { 111,   0 }, { 112,   0 }, { 113,   0 }, { 114,   0 },
 { 115,   0 }, { 116,   0 }, { 117,   0 }, { 118,   0 }, { 119,   0 },
 { 120,   0 }, { 121,   0 }, { 122,   0 }, { 123,   0 }, { 124,   0 },
 { 125,   0 }, { 126,   0 }, { 127,   0 }, { 128,   0 }, { 129,   0 },
 { 130,   0 }, { 131,   0 }, { 132,   0 }, { 133,   0 }, { 134,   0 },
 { 135,   0 }, { 136,   0 }, { 137,   0 }, { 138,   0 }, { 139,   0 },
 { 140,   0 }, { 141,   0 }, { 142,   0 }, { 143,   0 }, { 144,   0 },
 { 145,   0 }, { 146,   0 }, { 147,   0 }, { 148,   0 }, { 149,   0 },
 { 150,   0 }, { 151,   0 }, { 152,   0 }, { 153,   0 }, { 154,   0 },
 { 155,   0 }, { 156,   0 }, { 157,   0 }, { 158,   0 }, { 159,   0 },

 { 160,   0 }, { 161,   0 }, { 162,   0 }, { 163,   0 }, { 164,   0 },
 { 165,   0 }, { 166,   0 }, { 167,   0 }, { 168,   0 }, { 169,   0 },
 { 170,   0 }, { 171,   0 }, { 172,   0 }, { 173,   0 }, { 174,   0 },
 { 175,   0 }, { 176,   0 }, { 177,   0 }, { 178,   0 }, { 179,   0 },
 { 180,   0 }, { 181,   0 }, { 182,   0 }, { 183,   0 }, { 184,   0 },
 { 185,   0 }, { 186,   0 }, { 187,   0 }, { 188,   0 }, { 189,   0 },
 { 190,   0 }, { 191,   0 }, { 192,   0 }, { 193,   0 }, { 194,   0 },
 { 195,   0 }, { 196,   0 }, { 197,   0 }, { 198,   0 }, { 199,   0 },
 { 200,   0 }, { 201,   0 }, { 202,   0 }, { 203,   0 }, { 204,   0 },
 { 205,   0 }, { 206,   0 }, { 207,   0 }, { 208,   0 }, { 209,   0 },

 { 210,   0 }, { 211,   0 }, { 212,   0 }, { 213,   0 }, { 214,   0 },
 { 215,   0 }, { 216,   0 }, { 217,   0 }, { 218,   0 }, { 219,   0 },
 { 220,   0 }, { 221,   0 }, { 222,   0 }, { 223,   0 }, { 224,   0 },
 { 225,   0 }, { 226,   0 }, { 227,   0 }, { 228,   0 }, { 229,   0 },
 { 230,   0 }, { 231,   0 }, { 232,   0 }, { 233,   0 }, { 234,   0 },
 { 235,   0 }, { 236,   0 }, { 237,   0 }, { 238,   0 }, { 239,   0 },
 { 240,   0 }, { 241,   0 }, { 242,   0 }, { 243,   0 }, { 244,   0 },
 { 245,   0 }, { 246,   0 }, { 247,   0 }, { 248,   0 }, { 249,   0 },
 { 250,   0 }, { 251,   0 }, { 252,   0 }, { 253,   0 }, { 254,   0 },
 { 255,   0 }, { 256,   0 }, {   0,  24 }, {   0,8927 }, {   1,-258 },

 {   2,-258 }, {   3,-258 }, {   4,-258 }, {   5,-258 }, {   6,-258 },
 {   7,-258 }, {   8,-258 }, {   9,   0 }, {  10, 258 }, {  11,-258 },
 {  12,   0 }, {  13, 258 }, {  14,-258 }, {  15,-258 }, {  16,-258 },
 {  17,-258 }, {  18,-258 }, {  19,-258 }, {  20,-258 }, {  21,-258 },
 {  22,-258 }, {  23,-258 }, {  24,-258 }, {  25,-258 }, {  26,-258 },
 {  27,-258 }, {  28,-258 }, {  29,-258 }, {  30,-258 }, {  31,-258 },
 {  32,   0 }, {  33,-258 }, {  34,-258 }, {  35,-258 }, {  36,-258 },
 {  37,-258 }, {  38,-258 }, {  39,-258 }, {  40,-258 }, {  41,-258 },
 {  42,-258 }, {  43,-258 }, {  44,-258 }, {  45, 305 }, {  46,-258 },
 {  47,-258 }, {  48,-258 }, {  49,-258 }, {  50,-258 }, {  51,-258 },

 {  52,-258 }, {  53,-258 }, {  54,-258 }, {  55,-258 }, {  56,-258 },
 {  57,-258 }, {  58,-258 }, {  59,-258 }, {  60,-258 }, {  61,-258 },
 {  62,-258 }, {  63,-258 }, {  64,-258 }, {  65,-258 }, {  66,-258 },
 {  67,-258 }, {  68,-258 }, {  69,-258 }, {  70,-258 }, {  71,-258 },
 {  72,-258 }, {  73,-258 }, {  74,-258 }, {  75,-258 }, {  76,-258 },
 {  77,-258 }, {  78,-258 }, {  79,-258 }, {  80,-258 }, {  81,-258 },
 {  82,-258 }, {  83,-258 }, {  84,-258 }, {  85,-258 }, {  86,-258 },
 {  87,-258 }, {  88,-258 }, {  89,-258 }, {  90,-258 }, {  91,-258 },
 {  92,-258 }, {  93,-258 }, {  94,-258 }, {  95,-258 }, {  96,-258 },
 {  97,-258 }, {  98,-258 }, {  99,-258 }, { 100,-258 }, { 101,-258 },

 { 102,-258 }, { 103,-258 }, { 104,-258 }, { 105,-258 }, { 106,-258 },
 { 107,-258 }, { 108,-258 }, { 109,-258 }, { 110,-258 }, { 111,-258 },
 { 112,-258 }, { 113,-258 }, { 114,-258 }, { 115,-258 }, { 116,-258 },
 { 117,-258 }, { 118,-258 }, { 119,-258 }, { 120,-258 }, { 121,-258 },
 { 122,-258 }, { 123,-258 }, { 124,-258 }, { 125,-258 }, { 126,-258 },
 { 127,-258 }, { 128,-258 }, { 129,-258 }, { 130,-258 }, { 131,-258 },
 { 132,-258 }, { 133,-258 }, { 134,-258 }, { 135,-258 }, { 136,-258 },
 { 137,-258 }, { 138,-258 }, { 139,-258 }, { 140,-258 }, { 141,-258 },
 { 142,-258 }, { 143,-258 }, { 144,-258 }, { 145,-258 }, { 146,-258 },
 { 147,-258 }, { 148,-258 }, { 149,-258 }, { 150,-258 }, { 151,-258 },

 { 152,-258 }, { 153,-258 }, { 154,-258 }, { 155,-258 }, { 156,-258 },
 { 157,-258 }, { 158,-258 }, { 159,-258 }, { 160,-258 }, { 161,-258 },
 { 162,-258 }, { 163,-258 }, { 164,-258 }, { 165,-258 }, { 166,-258 },
 { 167,-258 }, { 168,-258 }, { 169,-258 }, { 170,-258 }, { 171,-258 },
 { 172,-258 }, { 173,-258 }, { 174,-258 }, { 175,-258 }, { 176,-258 },
 { 177,-258 }, { 178,-258 }, { 179,-258 }, { 180,-258 }, { 181,-258 },
 { 182,-258 }, { 183,-258 }, { 184,-258 }, { 185,-258 }, { 186,-258 },
 { 187,-258 }, { 188,-258 }, { 189,-258 }, { 190,-258 }, { 191,-258 },
 { 192,-258 }, { 193,-258 }, { 194,-258 }, { 195,-258 }, { 196,-258 },
 { 197,-258 }, { 198,-258 }, { 199,-258 }, { 200,-258 }, { 201,-258 },

 { 202,-258 }, { 203,-258 }, { 204,-258 }, { 205,-258 }, { 206,-258 },
 { 207,-258 }, { 208,-258 }, { 209,-258 }, { 210,-258 }, { 211,-258 },
 { 212,-258 }, { 213,-258 }, { 214,-258 }, { 215,-258 }, { 216,-258 },
 { 217,-258 }, { 218,-258 }, { 219,-258 }, { 220,-258 }, { 221,-258 },
 { 222,-258 }, { 223,-258 }, { 224,-258 }, { 225,-258 }, { 226,-258 },
 { 227,-258 }, { 228,-258 }, { 229,-258 }, { 230,-258 }, { 231,-258 },
 { 232,-258 }, { 233,-258 }, { 234,-258 }, { 235,-258 }, { 236,-258 },
 { 237,-258 }, { 238,-258 }, { 239,-258 }, { 240,-258 }, { 241,-258 },
 { 242,-258 }, { 243,-258 }, { 244,-258 }, { 245,-258 }, { 246,-258 },
 { 247,-258 }, { 248,-258 }, { 249,-258 }, { 250,-258 }, { 251,-258 },

 { 252,-258 }, { 253,-258 }, { 254,-258 }, { 255,-258 }, { 256,-258 },
 {   0,  24 }, {   0,8669 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   9,-8952 }, {  10,-8952 }, {   0,   0 }, {  12,-8952 }, {  13,-8952 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  32,-8952 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  39,-21193 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {  45,-21061 }, {   0,  24 }, {   0,8622 }, {   1,-563 },
 {   2,-563 }, {   3,-563 }, {   4,-563 }, {   5,-563 }, {   6,-563 },
 {   7,-563 }, {   8,-563 }, {   9,-305 }, {  10, -47 }, {  11,-563 },
 {  12,-305 }, {  13, -47 }, {  14,-563 }, {  15,-563 }, {  16,-563 },
 {  17,-563 }, {  18,-563 }, {  19,-563 }, {  20,-563 }, {  21,-563 },
 {  22,-563 }, {  23,-563 }, {  24,-563 }, {  25,-563 }, {  26,-563 },
 {  27,-563 }, {  28,-563 }, {  29,-563 }, {  30,-563 }, {  31,-563 },
 {  32,-305 }, {  33,-563 }, {  34,-563 }, {  35,-563 }, {  36,-563 },
 {  37,-563 }, {  38,-563 }, {  39,-563 }, {  40,-563 }, {  41,-563 },
 {  42,-563 }, {  43,-563 }, {  44,-563 }, {  45,1521 }, {  46,-563 },

 {  47,-563 }, {  48,-563 }, {  49,-563 }, {  50,-563 }, {  51,-563 },
 {  52,-563 }, {  53,-563 }, {  54,-563 }, {  55,-563 }, {  56,-563 },
 {  57,-563 }, {  58,-563 }, {  59,-563 }, {  60,-563 }, {  61,-563 },
 {  62,-563 }, {  63,-563 }, {  64,-563 }, {  65,-563 }, {  66,-563 },
 {  67,-563 }, {  68,-563 }, {  69,-563 }, {  70,-563 }, {  71,-563 },
 {  72,-563 }, {  73,-563 }, {  74,-563 }, {  75,-563 }, {  76,-563 },
 {  77,-563 }, {  78,-563 }, {  79,-563 }, {  80,-563 }, {  81,-563 },
 {  82,-563 }, {  83,-563 }, {  84,-563 }, {  85,-563 }, {  86,-563 },
 {  87,-563 }, {  88,-563 }, {  89,-563 }, {  90,-563 }, {  91,-563 },
 {  92,-563 }, {  93,-563 }, {  94,-563 }, {  95,-563 }, {  96,-563 },

 {  97,-563 }, {  98,-563 }, {  99,-563 }, { 100,-563 }, { 101,-563 },
 { 102,-563 }, { 103,-563 }, { 104,-563 }, { 105,-563 }, { 106,-563 },
 { 107,-563 }, { 108,-563 }, { 109,-563 }, { 110,-563 }, { 111,-563 },
 { 112,-563 }, { 113,-563 }, { 114,-563 }, { 115,-563 }, { 116,-563 },
 { 117,-563 }, { 118,-563 }, { 119,-563 }, { 120,-563 }, { 121,-563 },
 { 122,-563 }, { 123,-563 }, { 124,-563 }, { 125,-563 }, { 126,-563 },
 { 127,-563 }, { 128,-563 }, { 129,-563 }, { 130,-563 }, { 131,-563 },
 { 132,-563 }, { 133,-563 }, { 134,-563 }, { 135,-563 }, { 136,-563 },
 { 137,-563 }, { 138,-563 }, { 139,-563 }, { 140,-563 }, { 141,-563 },
 { 142,-563 }, { 143,-563 }, { 144,-563 }, { 145,-563 }, { 146,-563 },

 { 147,-563 }, { 148,-563 }, { 149,-563 }, { 150,-563 }, { 151,-563 },
 { 152,-563 }, { 153,-563 }, { 154,-563 }, { 155,-563 }, { 156,-563 },
 { 157,-563 }, { 158,-563 }, { 159,-563 }, { 160,-563 }, { 161,-563 },
 { 162,-563 }, { 163,-563 }, { 164,-563 }, { 165,-563 }, { 166,-563 },
 { 167,-563 }, { 168,-563 }, { 169,-563 }, { 170,-563 }, { 171,-563 },
 { 172,-563 }, { 173,-563 }, { 174,-563 }, { 175,-563 }, { 176,-563 },
 { 177,-563 }, { 178,-563 }, { 179,-563 }, { 180,-563 }, { 181,-563 },
 { 182,-563 }, { 183,-563 }, { 184,-563 }, { 185,-563 }, { 186,-563 },
 { 187,-563 }, { 188,-563 }, { 189,-563 }, { 190,-563 }, { 191,-563 },
 { 192,-563 }, { 193,-563 }, { 194,-563 }, { 195,-563 }, { 196,-563 },

 { 197,-563 }, { 198,-563 }, { 199,-563 }, { 200,-563 }, { 201,-563 },
 { 202,-563 }, { 203,-563 }, { 204,-563 }, { 205,-563 }, { 206,-563 },
 { 207,-563 }, { 208,-563 }, { 209,-563 }, { 210,-563 }, { 211,-563 },
 { 212,-563 }, { 213,-563 }, { 214,-563 }, { 215,-563 }, { 216,-563 },
 { 217,-563 }, { 218,-563 }, { 219,-563 }, { 220,-563 }, { 221,-563 },
 { 222,-563 }, { 223,-563 }, { 224,-563 }, { 225,-563 }, { 226,-563 },
 { 227,-563 }, { 228,-563 }, { 229,-563 }, { 230,-563 }, { 231,-563 },
 { 232,-563 }, { 233,-563 }, { 234,-563 }, { 235,-563 }, { 236,-563 },
 { 237,-563 }, { 238,-563 }, { 239,-563 }, { 240,-563 }, { 241,-563 },
 { 242,-563 }, { 243,-563 }, { 244,-563 }, { 245,-563 }, { 246,-563 },

 { 247,-563 }, { 248,-563 }, { 249,-563 }, { 250,-563 }, { 251,-563 },
 { 252,-563 }, { 253,-563 }, { 254,-563 }, { 255,-563 }, { 256,-563 },
 {   0,  24 }, {   0,8364 }, {   1,-5086 }, {   2,-5086 }, {   3,-5086 },
 {   4,-5086 }, {   5,-5086 }, {   6,-5086 }, {   7,-5086 }, {   8,-5086 },
 {   9,-4828 }, {  10,-12235 }, {  11,-5086 }, {  12,-4828 }, {  13,-12235 },
 {  14,-5086 }, {  15,-5086 }, {  16,-5086 }, {  17,-5086 }, {  18,-5086 },
 {  19,-5086 }, {  20,-5086 }, {  21,-5086 }, {  22,-5086 }, {  23,-5086 },
 {  24,-5086 }, {  25,-5086 }, {  26,-5086 }, {  27,-5086 }, {  28,-5086 },
 {  29,-5086 }, {  30,-5086 }, {  31,-5086 }, {  32,-4828 }, {  33,-5086 },
 {  34,-5086 }, {  35,-5086 }, {  36,-5086 }, {  37,-5086 }, {  38,-5086 },

 {  39,-5086 }, {  40,-5086 }, {  41,-5086 }, {  42,-5086 }, {  43,-5086 },
 {  44,-5086 }, {  45,   0 }, {  46,-5086 }, {  47,-5086 }, {  48,-5086 },
 {  49,-5086 }, {  50,-5086 }, {  51,-5086 }, {  52,-5086 }, {  53,-5086 },
 {  54,-5086 }, {  55,-5086 }, {  56,-5086 }, {  57,-5086 }, {  58,-5086 },
 {  59,-5086 }, {  60,-5086 }, {  61,-5086 }, {  62,-5086 }, {  63,-5086 },
 {  64,-5086 }, {  65,-5086 }, {  66,-5086 }, {  67,-5086 }, {  68,-5086 },
 {  69,-5086 }, {  70,-5086 }, {  71,-5086 }, {  72,-5086 }, {  73,-5086 },
 {  74,-5086 }, {  75,-5086 }, {  76,-5086 }, {  77,-5086 }, {  78,-5086 },
 {  79,-5086 }, {  80,-5086 }, {  81,-5086 }, {  82,-5086 }, {  83,-5086 },
 {  84,-5086 }, {  85,-5086 }, {  86,-5086 }, {  87,-5086 }, {  88,-5086 },

 {  89,-5086 }, {  90,-5086 }, {  91,-5086 }, {  92,-5086 }, {  93,-5086 },
 {  94,-5086 }, {  95,-5086 }, {  96,-5086 }, {  97,-5086 }, {  98,-5086 },
 {  99,-5086 }, { 100,-5086 }, { 101,-5086 }, { 102,-5086 }, { 103,-5086 },
 { 104,-5086 }, { 105,-5086 }, { 106,-5086 }, { 107,-5086 }, { 108,-5086 },
 { 109,-5086 }, { 110,-5086 }, { 111,-5086 }, { 112,-5086 }, { 113,-5086 },
 { 114,-5086 }, { 115,-5086 }, { 116,-5086 }, { 117,-5086 }, { 118,-5086 },
 { 119,-5086 }, { 120,-5086 }, { 121,-5086 }, { 122,-5086 }, { 123,-5086 },
 { 124,-5086 }, { 125,-5086 }, { 126,-5086 }, { 127,-5086 }, { 128,-5086 },
 { 129,-5086 }, { 130,-5086 }, { 131,-5086 }, { 132,-5086 }, { 133,-5086 },
 { 134,-5086 }, { 135,-5086 }, { 136,-5086 }, { 137,-5086 }, { 138,-5086 },

 { 139,-5086 }, { 140,-5086 }, { 141,-5086 }, { 142,-5086 }, { 143,-5086 },
 { 144,-5086 }, { 145,-5086 }, { 146,-5086 }, { 147,-5086 }, { 148,-5086 },
 { 149,-5086 }, { 150,-5086 }, { 151,-5086 }, { 152,-5086 }, { 153,-5086 },
 { 154,-5086 }, { 155,-5086 }, { 156,-5086 }, { 157,-5086 }, { 158,-5086 },
 { 159,-5086 }, { 160,-5086 }, { 161,-5086 }, { 162,-5086 }, { 163,-5086 },
 { 164,-5086 }, { 165,-5086 }, { 166,-5086 }, { 167,-5086 }, { 168,-5086 },
 { 169,-5086 }, { 170,-5086 }, { 171,-5086 }, { 172,-5086 }, { 173,-5086 },
 { 174,-5086 }, { 175,-5086 }, { 176,-5086 }, { 177,-5086 }, { 178,-5086 },
 { 179,-5086 }, { 180,-5086 }, { 181,-5086 }, { 182,-5086 }, { 183,-5086 },
 { 184,-5086 }, { 185,-5086 }, { 186,-5086 }, { 187,-5086 }, { 188,-5086 },

 { 189,-5086 }, { 190,-5086 }, { 191,-5086 }, { 192,-5086 }, { 193,-5086 },
 { 194,-5086 }, { 195,-5086 }, { 196,-5086 }, { 197,-5086 }, { 198,-5086 },
 { 199,-5086 }, { 200,-5086 }, { 201,-5086 }, { 202,-5086 }, { 203,-5086 },
 { 204,-5086 }, { 205,-5086 }, { 206,-5086 }, { 207,-5086 }, { 208,-5086 },
 { 209,-5086 }, { 210,-5086 }, { 211,-5086 }, { 212,-5086 }, { 213,-5086 },
 { 214,-5086 }, { 215,-5086 }, { 216,-5086 }, { 217,-5086 }, { 218,-5086 },
 { 219,-5086 }, { 220,-5086 }, { 221,-5086 }, { 222,-5086 }, { 223,-5086 },
 { 224,-5086 }, { 225,-5086 }, { 226,-5086 }, { 227,-5086 }, { 228,-5086 },
 { 229,-5086 }, { 230,-5086 }, { 231,-5086 }, { 232,-5086 }, { 233,-5086 },
 { 234,-5086 }, { 235,-5086 }, { 236,-5086 }, { 237,-5086 }, { 238,-5086 },

 { 239,-5086 }, { 240,-5086 }, { 241,-5086 }, { 242,-5086 }, { 243,-5086 },
 { 244,-5086 }, { 245,-5086 }, { 246,-5086 }, { 247,-5086 }, { 248,-5086 },
 { 249,-5086 }, { 250,-5086 }, { 251,-5086 }, { 252,-5086 }, { 253,-5086 },
 { 254,-5086 }, { 255,-5086 }, { 256,-5086 }, {   0,  37 }, {   0,8106 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,  37 }, {   0,8083 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  48,1263 }, {  49,1263 }, {  50,1263 },
 {  51,1263 }, {  52,1263 }, {  53,1263 }, {  54,1263 }, {  55,1263 },
 {  56,1263 }, {  57,1263 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65,1263 },
 {  66,1263 }, {  67,1263 }, {  68,1263 }, {  69,1263 }, {  70,1263 },
 {  48,-21620 }, {  49,-21620 }, {  50,-21620 }, {  51,-21620 }, {  52,-21620 },
 {  53,-21620 }, {  54,-21620 }, {  55,-21620 }, {  56,-21620 }, {  57,-21620 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  65,-21620 }, {  66,-21620 }, {  67,-21620 },
 {  68,-21620 }, {  69,-21620 }, {  70,-21620 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  97,1263 }, {  98,1263 }, {  99,1263 }, { 100,1263 },
 { 101,1263 }, { 102,1263 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  97,-21620 },
 {  98,-21620 }, {  99,-21620 }, { 100,-21620 }, { 101,-21620 }, { 102,-21620 },
 {   0,   9 }, {   0,7979 }, {   1,-4570 }, {   2,-4570 }, {   3,-4570 },

 {   4,-4570 }, {   5,-4570 }, {   6,-4570 }, {   7,-4570 }, {   8,-4570 },
 {   9,-4312 }, {  10,-4054 }, {  11,-4570 }, {  12,-4312 }, {  13,-4054 },
 {  14,-4570 }, {  15,-4570 }, {  16,-4570 }, {  17,-4570 }, {  18,-4570 },
 {  19,-4570 }, {  20,-4570 }, {  21,-4570 }, {  22,-4570 }, {  23,-4570 },
 {  24,-4570 }, {  25,-4570 }, {  26,-4570 }, {  27,-4570 }, {  28,-4570 },
 {  29,-4570 }, {  30,-4570 }, {  31,-4570 }, {  32,-4312 }, {  33,-4570 },
 {  34,-4570 }, {  35,-4570 }, {  36,-4570 }, {  37,-4570 }, {  38,-4570 },
 {  39,-4570 }, {  40,-4570 }, {  41,-4570 }, {  42,-4570 }, {  43,-4570 },
 {  44,-4570 }, {  45,   0 }, {  46,-4570 }, {  47,-4570 }, {  48,-4570 },
 {  49,-4570 }, {  50,-4570 }, {  51,-4570 }, {  52,-4570 }, {  53,-4570 },

 {  54,-4570 }, {  55,-4570 }, {  56,-4570 }, {  57,-4570 }, {  58,-4570 },
 {  59,-4570 }, {  60,-4570 }, {  61,-4570 }, {  62,-4570 }, {  63,-4570 },
 {  64,-4570 }, {  65,-4570 }, {  66,-4570 }, {  67,-4570 }, {  68,-4570 },
 {  69,-4570 }, {  70,-4570 }, {  71,-4570 }, {  72,-4570 }, {  73,-4570 },
 {  74,-4570 }, {  75,-4570 }, {  76,-4570 }, {  77,-4570 }, {  78,-4570 },
 {  79,-4570 }, {  80,-4570 }, {  81,-4570 }, {  82,-4570 }, {  83,-4570 },
 {  84,-4570 }, {  85,-4570 }, {  86,-4570 }, {  87,-4570 }, {  88,-4570 },
 {  89,-4570 }, {  90,-4570 }, {  91,-4570 }, {  92,-4570 }, {  93,-4570 },
 {  94,-4570 }, {  95,-4570 }, {  96,-4570 }, {  97,-4570 }, {  98,-4570 },
 {  99,-4570 }, { 100,-4570 }, { 101,-4570 }, { 102,-4570 }, { 103,-4570 },

 { 104,-4570 }, { 105,-4570 }, { 106,-4570 }, { 107,-4570 }, { 108,-4570 },
 { 109,-4570 }, { 110,-4570 }, { 111,-4570 }, { 112,-4570 }, { 113,-4570 },
 { 114,-4570 }, { 115,-4570 }, { 116,-4570 }, { 117,-4570 }, { 118,-4570 },
 { 119,-4570 }, { 120,-4570 }, { 121,-4570 }, { 122,-4570 }, { 123,-4570 },
 { 124,-4570 }, { 125,-4570 }, { 126,-4570 }, { 127,-4570 }, { 128,-4570 },
 { 129,-4570 }, { 130,-4570 }, { 131,-4570 }, { 132,-4570 }, { 133,-4570 },
 { 134,-4570 }, { 135,-4570 }, { 136,-4570 }, { 137,-4570 }, { 138,-4570 },
 { 139,-4570 }, { 140,-4570 }, { 141,-4570 }, { 142,-4570 }, { 143,-4570 },
 { 144,-4570 }, { 145,-4570 }, { 146,-4570 }, { 147,-4570 }, { 148,-4570 },
 { 149,-4570 }, { 150,-4570 }, { 151,-4570 }, { 152,-4570 }, { 153,-4570 },

 { 154,-4570 }, { 155,-4570 }, { 156,-4570 }, { 157,-4570 }, { 158,-4570 },
 { 159,-4570 }, { 160,-4570 }, { 161,-4570 }, { 162,-4570 }, { 163,-4570 },
 { 164,-4570 }, { 165,-4570 }, { 166,-4570 }, { 167,-4570 }, { 168,-4570 },
 { 169,-4570 }, { 170,-4570 }, { 171,-4570 }, { 172,-4570 }, { 173,-4570 },
 { 174,-4570 }, { 175,-4570 }, { 176,-4570 }, { 177,-4570 }, { 178,-4570 },
 { 179,-4570 }, { 180,-4570 }, { 181,-4570 }, { 182,-4570 }, { 183,-4570 },
 { 184,-4570 }, { 185,-4570 }, { 186,-4570 }, { 187,-4570 }, { 188,-4570 },
 { 189,-4570 }, { 190,-4570 }, { 191,-4570 }, { 192,-4570 }, { 193,-4570 },
 { 194,-4570 }, { 195,-4570 }, { 196,-4570 }, { 197,-4570 }, { 198,-4570 },
 { 199,-4570 }, { 200,-4570 }, { 201,-4570 }, { 202,-4570 }, { 203,-4570 },

 { 204,-4570 }, { 205,-4570 }, { 206,-4570 }, { 207,-4570 }, { 208,-4570 },
 { 209,-4570 }, { 210,-4570 }, { 211,-4570 }, { 212,-4570 }, { 213,-4570 },
 { 214,-4570 }, { 215,-4570 }, { 216,-4570 }, { 217,-4570 }, { 218,-4570 },
 { 219,-4570 }, { 220,-4570 }, { 221,-4570 }, { 222,-4570 }, { 223,-4570 },
 { 224,-4570 }, { 225,-4570 }, { 226,-4570 }, { 227,-4570 }, { 228,-4570 },
 { 229,-4570 }, { 230,-4570 }, { 231,-4570 }, { 232,-4570 }, { 233,-4570 },
 { 234,-4570 }, { 235,-4570 }, { 236,-4570 }, { 237,-4570 }, { 238,-4570 },
 { 239,-4570 }, { 240,-4570 }, { 241,-4570 }, { 242,-4570 }, { 243,-4570 },
 { 244,-4570 }, { 245,-4570 }, { 246,-4570 }, { 247,-4570 }, { 248,-4570 },
 { 249,-4570 }, { 250,-4570 }, { 251,-4570 }, { 252,-4570 }, { 253,-4570 },

 { 254,-4570 }, { 255,-4570 }, { 256,-4570 }, {   0,  16 }, {   0,7721 },
 {   1,-3749 }, {   2,-3749 }, {   3,-3749 }, {   4,-3749 }, {   5,-3749 },
 {   6,-3749 }, {   7,-3749 }, {   8,-3749 }, {   9,-3491 }, {  10,-3233 },
 {  11,-3749 }, {  12,-3491 }, {  13,-3233 }, {  14,-3749 }, {  15,-3749 },
 {  16,-3749 }, {  17,-3749 }, {  18,-3749 }, {  19,-3749 }, {  20,-3749 },
 {  21,-3749 }, {  22,-3749 }, {  23,-3749 }, {  24,-3749 }, {  25,-3749 },
 {  26,-3749 }, {  27,-3749 }, {  28,-3749 }, {  29,-3749 }, {  30,-3749 },
 {  31,-3749 }, {  32,-3491 }, {  33,-3749 }, {  34,-3749 }, {  35,-3749 },
 {  36,-3749 }, {  37,-3749 }, {  38,-3749 }, {  39,-3749 }, {  40,-3749 },
 {  41,-3749 }, {  42,-3749 }, {  43,-3749 }, {  44,-3749 }, {  45,   0 },

 {  46,-3749 }, {  47,-3749 }, {  48,-3749 }, {  49,-3749 }, {  50,-3749 },
 {  51,-3749 }, {  52,-3749 }, {  53,-3749 }, {  54,-3749 }, {  55,-3749 },
 {  56,-3749 }, {  57,-3749 }, {  58,-3749 }, {  59,-3749 }, {  60,-3749 },
 {  61,-3749 }, {  62,-3749 }, {  63,-3749 }, {  64,-3749 }, {  65,-3749 },
 {  66,-3749 }, {  67,-3749 }, {  68,-3749 }, {  69,-3749 }, {  70,-3749 },
 {  71,-3749 }, {  72,-3749 }, {  73,-3749 }, {  74,-3749 }, {  75,-3749 },
 {  76,-3749 }, {  77,-3749 }, {  78,-3749 }, {  79,-3749 }, {  80,-3749 },
 {  81,-3749 }, {  82,-3749 }, {  83,-3749 }, {  84,-3749 }, {  85,-3749 },
 {  86,-3749 }, {  87,-3749 }, {  88,-3749 }, {  89,-3749 }, {  90,-3749 },
 {  91,-3749 }, {  92,-3749 }, {  93,-3749 }, {  94,-3749 }, {  95,-3749 },

 {  96,-3749 }, {  97,-3749 }, {  98,-3749 }, {  99,-3749 }, { 100,-3749 },
 { 101,-3749 }, { 102,-3749 }, { 103,-3749 }, { 104,-3749 }, { 105,-3749 },
 { 106,-3749 }, { 107,-3749 }, { 108,-3749 }, { 109,-3749 }, { 110,-3749 },
 { 111,-3749 }, { 112,-3749 }, { 113,-3749 }, { 114,-3749 }, { 115,-3749 },
 { 116,-3749 }, { 117,-3749 }, { 118,-3749 }, { 119,-3749 }, { 120,-3749 },
 { 121,-3749 }, { 122,-3749 }, { 123,-3749 }, { 124,-3749 }, { 125,-3749 },
 { 126,-3749 }, { 127,-3749 }, { 128,-3749 }, { 129,-3749 }, { 130,-3749 },
 { 131,-3749 }, { 132,-3749 }, { 133,-3749 }, { 134,-3749 }, { 135,-3749 },
 { 136,-3749 }, { 137,-3749 }, { 138,-3749 }, { 139,-3749 }, { 140,-3749 },
 { 141,-3749 }, { 142,-3749 }, { 143,-3749 }, { 144,-3749 }, { 145,-3749 },

 { 146,-3749 }, { 147,-3749 }, { 148,-3749 }, { 149,-3749 }, { 150,-3749 },
 { 151,-3749 }, { 152,-3749 }, { 153,-3749 }, { 154,-3749 }, { 155,-3749 },
 { 156,-3749 }, { 157,-3749 }, { 158,-3749 }, { 159,-3749 }, { 160,-3749 },
 { 161,-3749 }, { 162,-3749 }, { 163,-3749 }, { 164,-3749 }, { 165,-3749 },
 { 166,-3749 }, { 167,-3749 }, { 168,-3749 }, { 169,-3749 }, { 170,-3749 },
 { 171,-3749 }, { 172,-3749 }, { 173,-3749 }, { 174,-3749 }, { 175,-3749 },
 { 176,-3749 }, { 177,-3749 }, { 178,-3749 }, { 179,-3749 }, { 180,-3749 },
 { 181,-3749 }, { 182,-3749 }, { 183,-3749 }, { 184,-3749 }, { 185,-3749 },
 { 186,-3749 }, { 187,-3749 }, { 188,-3749 }, { 189,-3749 }, { 190,-3749 },
 { 191,-3749 }, { 192,-3749 }, { 193,-3749 }, { 194,-3749 }, { 195,-3749 },

 { 196,-3749 }, { 197,-3749 }, { 198,-3749 }, { 199,-3749 }, { 200,-3749 },
 { 201,-3749 }, { 202,-3749 }, { 203,-3749 }, { 204,-3749 }, { 205,-3749 },
 { 206,-3749 }, { 207,-3749 }, { 208,-3749 }, { 209,-3749 }, { 210,-3749 },
 { 211,-3749 }, { 212,-3749 }, { 213,-3749 }, { 214,-3749 }, { 215,-3749 },
 { 216,-3749 }, { 217,-3749 }, { 218,-3749 }, { 219,-3749 }, { 220,-3749 },
 { 221,-3749 }, { 222,-3749 }, { 223,-3749 }, { 224,-3749 }, { 225,-3749 },
 { 226,-3749 }, { 227,-3749 }, { 228,-3749 }, { 229,-3749 }, { 230,-3749 },
 { 231,-3749 }, { 232,-3749 }, { 233,-3749 }, { 234,-3749 }, { 235,-3749 },
 { 236,-3749 }, { 237,-3749 }, { 238,-3749 }, { 239,-3749 }, { 240,-3749 },
 { 241,-3749 }, { 242,-3749 }, { 243,-3749 }, { 244,-3749 }, { 245,-3749 },

 { 246,-3749 }, { 247,-3749 }, { 248,-3749 }, { 249,-3749 }, { 250,-3749 },
 { 251,-3749 }, { 252,-3749 }, { 253,-3749 }, { 254,-3749 }, { 255,-3749 },
 { 256,-3749 }, {   0,  22 }, {   0,7463 }, {   1,-2928 }, {   2,-2928 },
 {   3,-2928 }, {   4,-2928 }, {   5,-2928 }, {   6,-2928 }, {   7,-2928 },
 {   8,-2928 }, {   9,-2670 }, {  10,-2412 }, {  11,-2928 }, {  12,-2670 },
 {  13,-2412 }, {  14,-2928 }, {  15,-2928 }, {  16,-2928 }, {  17,-2928 },
 {  18,-2928 }, {  19,-2928 }, {  20,-2928 }, {  21,-2928 }, {  22,-2928 },
 {  23,-2928 }, {  24,-2928 }, {  25,-2928 }, {  26,-2928 }, {  27,-2928 },
 {  28,-2928 }, {  29,-2928 }, {  30,-2928 }, {  31,-2928 }, {  32,-2670 },
 {  33,-2928 }, {  34,-2928 }, {  35,-2928 }, {  36,-2928 }, {  37,-2928 },

 {  38,-2928 }, {  39,-2928 }, {  40,-2928 }, {  41,-2928 }, {  42,-2928 },
 {  43,-2928 }, {  44,-2928 }, {  45,   0 }, {  46,-2928 }, {  47,-2928 },
 {  48,-2928 }, {  49,-2928 }, {  50,-2928 }, {  51,-2928 }, {  52,-2928 },
 {  53,-2928 }, {  54,-2928 }, {  55,-2928 }, {  56,-2928 }, {  57,-2928 },
 {  58,-2928 }, {  59,-2928 }, {  60,-2928 }, {  61,-2928 }, {  62,-2928 },
 {  63,-2928 }, {  64,-2928 }, {  65,-2928 }, {  66,-2928 }, {  67,-2928 },
 {  68,-2928 }, {  69,-2928 }, {  70,-2928 }, {  71,-2928 }, {  72,-2928 },
 {  73,-2928 }, {  74,-2928 }, {  75,-2928 }, {  76,-2928 }, {  77,-2928 },
 {  78,-2928 }, {  79,-2928 }, {  80,-2928 }, {  81,-2928 }, {  82,-2928 },
 {  83,-2928 }, {  84,-2928 }, {  85,-2928 }, {  86,-2928 }, {  87,-2928 },

 {  88,-2928 }, {  89,-2928 }, {  90,-2928 }, {  91,-2928 }, {  92,-2928 },
 {  93,-2928 }, {  94,-2928 }, {  95,-2928 }, {  96,-2928 }, {  97,-2928 },
 {  98,-2928 }, {  99,-2928 }, { 100,-2928 }, { 101,-2928 }, { 102,-2928 },
 { 103,-2928 }, { 104,-2928 }, { 105,-2928 }, { 106,-2928 }, { 107,-2928 },
 { 108,-2928 }, { 109,-2928 }, { 110,-2928 }, { 111,-2928 }, { 112,-2928 },
 { 113,-2928 }, { 114,-2928 }, { 115,-2928 }, { 116,-2928 }, { 117,-2928 },
 { 118,-2928 }, { 119,-2928 }, { 120,-2928 }, { 121,-2928 }, { 122,-2928 },
 { 123,-2928 }, { 124,-2928 }, { 125,-2928 }, { 126,-2928 }, { 127,-2928 },
 { 128,-2928 }, { 129,-2928 }, { 130,-2928 }, { 131,-2928 }, { 132,-2928 },
 { 133,-2928 }, { 134,-2928 }, { 135,-2928 }, { 136,-2928 }, { 137,-2928 },

 { 138,-2928 }, { 139,-2928 }, { 140,-2928 }, { 141,-2928 }, { 142,-2928 },
 { 143,-2928 }, { 144,-2928 }, { 145,-2928 }, { 146,-2928 }, { 147,-2928 },
 { 148,-2928 }, { 149,-2928 }, { 150,-2928 }, { 151,-2928 }, { 152,-2928 },
 { 153,-2928 }, { 154,-2928 }, { 155,-2928 }, { 156,-2928 }, { 157,-2928 },
 { 158,-2928 }, { 159,-2928 }, { 160,-2928 }, { 161,-2928 }, { 162,-2928 },
 { 163,-2928 }, { 164,-2928 }, { 165,-2928 }, { 166,-2928 }, { 167,-2928 },
 { 168,-2928 }, { 169,-2928 }, { 170,-2928 }, { 171,-2928 }, { 172,-2928 },
 { 173,-2928 }, { 174,-2928 }, { 175,-2928 }, { 176,-2928 }, { 177,-2928 },
 { 178,-2928 }, { 179,-2928 }, { 180,-2928 }, { 181,-2928 }, { 182,-2928 },
 { 183,-2928 }, { 184,-2928 }, { 185,-2928 }, { 186,-2928 }, { 187,-2928 },

 { 188,-2928 }, { 189,-2928 }, { 190,-2928 }, { 191,-2928 }, { 192,-2928 },
 { 193,-2928 }, { 194,-2928 }, { 195,-2928 }, { 196,-2928 }, { 197,-2928 },
 { 198,-2928 }, { 199,-2928 }, { 200,-2928 }, { 201,-2928 }, { 202,-2928 },
 { 203,-2928 }, { 204,-2928 }, { 205,-2928 }, { 206,-2928 }, { 207,-2928 },
 { 208,-2928 }, { 209,-2928 }, { 210,-2928 }, { 211,-2928 }, { 212,-2928 },
 { 213,-2928 }, { 214,-2928 }, { 215,-2928 }, { 216,-2928 }, { 217,-2928 },
 { 218,-2928 }, { 219,-2928 }, { 220,-2928 }, { 221,-2928 }, { 222,-2928 },
 { 223,-2928 }, { 224,-2928 }, { 225,-2928 }, { 226,-2928 }, { 227,-2928 },
 { 228,-2928 }, { 229,-2928 }, { 230,-2928 }, { 231,-2928 }, { 232,-2928 },
 { 233,-2928 }, { 234,-2928 }, { 235,-2928 }, { 236,-2928 }, { 237,-2928 },

 { 238,-2928 }, { 239,-2928 }, { 240,-2928 }, { 241,-2928 }, { 242,-2928 },
 { 243,-2928 }, { 244,-2928 }, { 245,-2928 }, { 246,-2928 }, { 247,-2928 },
 { 248,-2928 }, { 249,-2928 }, { 250,-2928 }, { 251,-2928 }, { 252,-2928 },
 { 253,-2928 }, { 254,-2928 }, { 255,-2928 }, { 256,-2928 }, {   0,  37 },
 {   0,7205 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  48, 385 }, {  49, 385 },
 {  50, 385 }, {  51, 385 }, {  52, 385 }, {  53, 385 }, {  54, 385 },
 {  55, 385 }, {  56, 385 }, {  57, 385 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  65, 385 }, {  66, 385 }, {  67, 385 }, {  68, 385 }, {  69, 385 },
 {  70, 385 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  97, 385 }, {  98, 385 }, {  99, 385 },
 { 100, 385 }, { 101, 385 }, { 102, 385 }, {   0,  24 }, {   0,7101 },
 {   1,-2084 }, {   2,-2084 }, {   3,-2084 }, {   4,-2084 }, {   5,-2084 },
 {   6,-2084 }, {   7,-2084 }, {   8,-2084 }, {   9,-1826 }, {  10,-1568 },
 {  11,-2084 }, {  12,-1826 }, {  13,-1568 }, {  14,-2084 }, {  15,-2084 },
 {  16,-2084 }, {  17,-2084 }, {  18,-2084 }, {  19,-2084 }, {  20,-2084 },
 {  21,-2084 }, {  22,-2084 }, {  23,-2084 }, {  24,-2084 }, {  25,-2084 },

 {  26,-2084 }, {  27,-2084 }, {  28,-2084 }, {  29,-2084 }, {  30,-2084 },
 {  31,-2084 }, {  32,-1826 }, {  33,-2084 }, {  34,-2084 }, {  35,-2084 },
 {  36,-2084 }, {  37,-2084 }, {  38,-2084 }, {  39,-2084 }, {  40,-2084 },
 {  41,-2084 }, {  42,-2084 }, {  43,-2084 }, {  44,-2084 }, {  45,   0 },
 {  46,-2084 }, {  47,-2084 }, {  48,-2084 }, {  49,-2084 }, {  50,-2084 },
 {  51,-2084 }, {  52,-2084 }, {  53,-2084 }, {  54,-2084 }, {  55,-2084 },
 {  56,-2084 }, {  57,-2084 }, {  58,-2084 }, {  59,-2084 }, {  60,-2084 },
 {  61,-2084 }, {  62,-2084 }, {  63,-2084 }, {  64,-2084 }, {  65,-2084 },
 {  66,-2084 }, {  67,-2084 }, {  68,-2084 }, {  69,-2084 }, {  70,-2084 },
 {  71,-2084 }, {  72,-2084 }, {  73,-2084 }, {  74,-2084 }, {  75,-2084 },

 {  76,-2084 }, {  77,-2084 }, {  78,-2084 }, {  79,-2084 }, {  80,-2084 },
 {  81,-2084 }, {  82,-2084 }, {  83,-2084 }, {  84,-2084 }, {  85,-2084 },
 {  86,-2084 }, {  87,-2084 }, {  88,-2084 }, {  89,-2084 }, {  90,-2084 },
 {  91,-2084 }, {  92,-2084 }, {  93,-2084 }, {  94,-2084 }, {  95,-2084 },
 {  96,-2084 }, {  97,-2084 }, {  98,-2084 }, {  99,-2084 }, { 100,-2084 },
 { 101,-2084 }, { 102,-2084 }, { 103,-2084 }, { 104,-2084 }, { 105,-2084 },
 { 106,-2084 }, { 107,-2084 }, { 108,-2084 }, { 109,-2084 }, { 110,-2084 },
 { 111,-2084 }, { 112,-2084 }, { 113,-2084 }, { 114,-2084 }, { 115,-2084 },
 { 116,-2084 }, { 117,-2084 }, { 118,-2084 }, { 119,-2084 }, { 120,-2084 },
 { 121,-2084 }, { 122,-2084 }, { 123,-2084 }, { 124,-2084 }, { 125,-2084 },

 { 126,-2084 }, { 127,-2084 }, { 128,-2084 }, { 129,-2084 }, { 130,-2084 },
 { 131,-2084 }, { 132,-2084 }, { 133,-2084 }, { 134,-2084 }, { 135,-2084 },
 { 136,-2084 }, { 137,-2084 }, { 138,-2084 }, { 139,-2084 }, { 140,-2084 },
 { 141,-2084 }, { 142,-2084 }, { 143,-2084 }, { 144,-2084 }, { 145,-2084 },
 { 146,-2084 }, { 147,-2084 }, { 148,-2084 }, { 149,-2084 }, { 150,-2084 },
 { 151,-2084 }, { 152,-2084 }, { 153,-2084 }, { 154,-2084 }, { 155,-2084 },
 { 156,-2084 }, { 157,-2084 }, { 158,-2084 }, { 159,-2084 }, { 160,-2084 },
 { 161,-2084 }, { 162,-2084 }, { 163,-2084 }, { 164,-2084 }, { 165,-2084 },
 { 166,-2084 }, { 167,-2084 }, { 168,-2084 }, { 169,-2084 }, { 170,-2084 },
 { 171,-2084 }, { 172,-2084 }, { 173,-2084 }, { 174,-2084 }, { 175,-2084 },

 { 176,-2084 }, { 177,-2084 }, { 178,-2084 }, { 179,-2084 }, { 180,-2084 },
 { 181,-2084 }, { 182,-2084 }, { 183,-2084 }, { 184,-2084 }, { 185,-2084 },
 { 186,-2084 }, { 187,-2084 }, { 188,-2084 }, { 189,-2084 }, { 190,-2084 },
 { 191,-2084 }, { 192,-2084 }, { 193,-2084 }, { 194,-2084 }, { 195,-2084 },
 { 196,-2084 }, { 197,-2084 }, { 198,-2084 }, { 199,-2084 }, { 200,-2084 },
 { 201,-2084 }, { 202,-2084 }, { 203,-2084 }, { 204,-2084 }, { 205,-2084 },
 { 206,-2084 }, { 207,-2084 }, { 208,-2084 }, { 209,-2084 }, { 210,-2084 },
 { 211,-2084 }, { 212,-2084 }, { 213,-2084 }, { 214,-2084 }, { 215,-2084 },
 { 216,-2084 }, { 217,-2084 }, { 218,-2084 }, { 219,-2084 }, { 220,-2084 },
 { 221,-2084 }, { 222,-2084 }, { 223,-2084 }, { 224,-2084 }, { 225,-2084 },

 { 226,-2084 }, { 227,-2084 }, { 228,-2084 }, { 229,-2084 }, { 230,-2084 },
 { 231,-2084 }, { 232,-2084 }, { 233,-2084 }, { 234,-2084 }, { 235,-2084 },
 { 236,-2084 }, { 237,-2084 }, { 238,-2084 }, { 239,-2084 }, { 240,-2084 },
 { 241,-2084 }, { 242,-2084 }, { 243,-2084 }, { 244,-2084 }, { 245,-2084 },
 { 246,-2084 }, { 247,-2084 }, { 248,-2084 }, { 249,-2084 }, { 250,-2084 },
 { 251,-2084 }, { 252,-2084 }, { 253,-2084 }, { 254,-2084 }, { 255,-2084 },
 { 256,-2084 }, {   0,  37 }, {   0,6843 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,  37 },
 {   0,6820 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  48, 136 }, {  49, 136 }, {  50, 136 }, {  51, 136 }, {  52, 136 },
 {  53, 136 }, {  54, 136 }, {  55, 136 }, {  56, 136 }, {  57, 136 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  65, 136 }, {  66, 136 }, {  67, 136 },

 {  68, 136 }, {  69, 136 }, {  70, 136 }, {  48, 137 }, {  49, 137 },
 {  50, 137 }, {  51, 137 }, {  52, 137 }, {  53, 137 }, {  54, 137 },
 {  55, 137 }, {  56, 137 }, {  57, 137 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  65, 137 }, {  66, 137 }, {  67, 137 }, {  68, 137 }, {  69, 137 },
 {  70, 137 }, {   0,  55 }, {   0,6748 }, {   0,   0 }, {  97, 136 },
 {  98, 136 }, {  99, 136 }, { 100, 136 }, { 101, 136 }, { 102, 136 },
 {   0,   0 }, {   9, 137 }, {  10, 137 }, {   0,   0 }, {  12, 137 },
 {  13, 137 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,  28 }, {   0,6724 }, {  97, 137 }, {  98, 137 }, {  99, 137 },
 { 100, 137 }, { 101, 137 }, { 102, 137 }, {   0,   0 }, {  32, 137 },
 {   9, 418 }, {  10, 418 }, {   0,   0 }, {  12, 418 }, {  13, 418 },
 {   0,   0 }, {  39, 184 }, {   0,  37 }, {   0,6707 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  45,-22946 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  32, 418 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  39, 465 }, {   0,  37 }, {   0,6683 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  45,-22944 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  48, 706 }, {  49, 706 }, {  50, 706 }, {  51, 706 },
 {  52, 706 }, {  53, 706 }, {  54, 706 }, {  55, 706 }, {  56, 706 },
 {  57, 706 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65, 706 }, {  66, 706 },
 {  67, 706 }, {  68, 706 }, {  69, 706 }, {  70, 706 }, {   0,   0 },
 {  48, 705 }, {  49, 705 }, {  50, 705 }, {  51, 705 }, {  52, 705 },
 {  53, 705 }, {  54, 705 }, {  55, 705 }, {  56, 705 }, {  57, 705 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  65, 705 }, {  66, 705 }, {  67, 705 },
 {  68, 705 }, {  69, 705 }, {  70, 705 }, {   0,  55 }, {   0,6611 },
 {  97, 706 }, {  98, 706 }, {  99, 706 }, { 100, 706 }, { 101, 706 },
 { 102, 706 }, {   0,   0 }, {   0,   0 }, {   9,   0 }, {  10,   0 },
 {   0,   0 }, {  12,   0 }, {  13,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  97, 705 },
 {  98, 705 }, {  99, 705 }, { 100, 705 }, { 101, 705 }, { 102, 705 },
 {   0,   0 }, {  32,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  39,  47 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  45,-23083 },
 {   0,  55 }, {   0,6564 }, {   1,-23091 }, {   2,-23091 }, {   3,-23091 },
 {   4,-23091 }, {   5,-23091 }, {   6,-23091 }, {   7,-23091 }, {   8,-23091 },
 {   9,-23091 }, {  10,-23091 }, {  11,-23091 }, {  12,-23091 }, {  13,-23091 },
 {  14,-23091 }, {  15,-23091 }, {  16,-23091 }, {  17,-23091 }, {  18,-23091 },
 {  19,-23091 }, {  20,-23091 }, {  21,-23091 }, {  22,-23091 }, {  23,-23091 },
 {  24,-23091 }, {  25,-23091 }, {  26,-23091 }, {  27,-23091 }, {  28,-23091 },
 {  29,-23091 }, {  30,-23091 }, {  31,-23091 }, {  32,-23091 }, {  33,-23091 },
 {  34,-23091 }, {  35,-23091 }, {  36,-23091 }, {  37,-23091 }, {  38,-23091 },

 {   0,   0 }, {  40,-23091 }, {  41,-23091 }, {  42,-23091 }, {  43,-23091 },
 {  44,-23091 }, {  45,-23091 }, {  46,-23091 }, {  47,-23091 }, {  48,-23091 },
 {  49,-23091 }, {  50,-23091 }, {  51,-23091 }, {  52,-23091 }, {  53,-23091 },
 {  54,-23091 }, {  55,-23091 }, {  56,-23091 }, {  57,-23091 }, {  58,-23091 },
 {  59,-23091 }, {  60,-23091 }, {  61,-23091 }, {  62,-23091 }, {  63,-23091 },
 {  64,-23091 }, {  65,-23091 }, {  66,-23091 }, {  67,-23091 }, {  68,-23091 },
 {  69,-23091 }, {  70,-23091 }, {  71,-23091 }, {  72,-23091 }, {  73,-23091 },
 {  74,-23091 }, {  75,-23091 }, {  76,-23091 }, {  77,-23091 }, {  78,-23091 },
 {  79,-23091 }, {  80,-23091 }, {  81,-23091 }, {  82,-23091 }, {  83,-23091 },
 {  84,-23091 }, {  85,-23091 }, {  86,-23091 }, {  87,-23091 }, {  88,-23091 },

 {  89,-23091 }, {  90,-23091 }, {  91,-23091 }, {  92,-23091 }, {  93,-23091 },
 {  94,-23091 }, {  95,-23091 }, {  96,-23091 }, {  97,-23091 }, {  98,-23091 },
 {  99,-23091 }, { 100,-23091 }, { 101,-23091 }, { 102,-23091 }, { 103,-23091 },
 { 104,-23091 }, { 105,-23091 }, { 106,-23091 }, { 107,-23091 }, { 108,-23091 },
 { 109,-23091 }, { 110,-23091 }, { 111,-23091 }, { 112,-23091 }, { 113,-23091 },
 { 114,-23091 }, { 115,-23091 }, { 116,-23091 }, { 117,-23091 }, { 118,-23091 },
 { 119,-23091 }, { 120,-23091 }, { 121,-23091 }, { 122,-23091 }, { 123,-23091 },
 { 124,-23091 }, { 125,-23091 }, { 126,-23091 }, { 127,-23091 }, { 128,-23091 },
 { 129,-23091 }, { 130,-23091 }, { 131,-23091 }, { 132,-23091 }, { 133,-23091 },
 { 134,-23091 }, { 135,-23091 }, { 136,-23091 }, { 137,-23091 }, { 138,-23091 },

 { 139,-23091 }, { 140,-23091 }, { 141,-23091 }, { 142,-23091 }, { 143,-23091 },
 { 144,-23091 }, { 145,-23091 }, { 146,-23091 }, { 147,-23091 }, { 148,-23091 },
 { 149,-23091 }, { 150,-23091 }, { 151,-23091 }, { 152,-23091 }, { 153,-23091 },
 { 154,-23091 }, { 155,-23091 }, { 156,-23091 }, { 157,-23091 }, { 158,-23091 },
 { 159,-23091 }, { 160,-23091 }, { 161,-23091 }, { 162,-23091 }, { 163,-23091 },
 { 164,-23091 }, { 165,-23091 }, { 166,-23091 }, { 167,-23091 }, { 168,-23091 },
 { 169,-23091 }, { 170,-23091 }, { 171,-23091 }, { 172,-23091 }, { 173,-23091 },
 { 174,-23091 }, { 175,-23091 }, { 176,-23091 }, { 177,-23091 }, { 178,-23091 },
 { 179,-23091 }, { 180,-23091 }, { 181,-23091 }, { 182,-23091 }, { 183,-23091 },
 { 184,-23091 }, { 185,-23091 }, { 186,-23091 }, { 187,-23091 }, { 188,-23091 },

 { 189,-23091 }, { 190,-23091 }, { 191,-23091 }, { 192,-23091 }, { 193,-23091 },
 { 194,-23091 }, { 195,-23091 }, { 196,-23091 }, { 197,-23091 }, { 198,-23091 },
 { 199,-23091 }, { 200,-23091 }, { 201,-23091 }, { 202,-23091 }, { 203,-23091 },
 { 204,-23091 }, { 205,-23091 }, { 206,-23091 }, { 207,-23091 }, { 208,-23091 },
 { 209,-23091 }, { 210,-23091 }, { 211,-23091 }, { 212,-23091 }, { 213,-23091 },
 { 214,-23091 }, { 215,-23091 }, { 216,-23091 }, { 217,-23091 }, { 218,-23091 },
 { 219,-23091 }, { 220,-23091 }, { 221,-23091 }, { 222,-23091 }, { 223,-23091 },
 { 224,-23091 }, { 225,-23091 }, { 226,-23091 }, { 227,-23091 }, { 228,-23091 },
 { 229,-23091 }, { 230,-23091 }, { 231,-23091 }, { 232,-23091 }, { 233,-23091 },
 { 234,-23091 }, { 235,-23091 }, { 236,-23091 }, { 237,-23091 }, { 238,-23091 },

 { 239,-23091 }, { 240,-23091 }, { 241,-23091 }, { 242,-23091 }, { 243,-23091 },
 { 244,-23091 }, { 245,-23091 }, { 246,-23091 }, { 247,-23091 }, { 248,-23091 },
 { 249,-23091 }, { 250,-23091 }, { 251,-23091 }, { 252,-23091 }, { 253,-23091 },
 { 254,-23091 }, { 255,-23091 }, { 256,-23091 }, {   0,  28 }, {   0,6306 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   9,   0 }, {  10,   0 },
 {   0,   0 }, {  12,   0 }, {  13,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {  32,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  39,  47 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  45,-23362 },
 {   0,  28 }, {   0,6259 }, {   1,-23388 }, {   2,-23388 }, {   3,-23388 },
 {   4,-23388 }, {   5,-23388 }, {   6,-23388 }, {   7,-23388 }, {   8,-23388 },
 {   9,-23388 }, {  10,-23388 }, {  11,-23388 }, {  12,-23388 }, {  13,-23388 },
 {  14,-23388 }, {  15,-23388 }, {  16,-23388 }, {  17,-23388 }, {  18,-23388 },
 {  19,-23388 }, {  20,-23388 }, {  21,-23388 }, {  22,-23388 }, {  23,-23388 },
 {  24,-23388 }, {  25,-23388 }, {  26,-23388 }, {  27,-23388 }, {  28,-23388 },
 {  29,-23388 }, {  30,-23388 }, {  31,-23388 }, {  32,-23388 }, {  33,-23388 },

 {  34,-23388 }, {  35,-23388 }, {  36,-23388 }, {  37,-23388 }, {  38,-23388 },
 {   0,   0 }, {  40,-23388 }, {  41,-23388 }, {  42,-23388 }, {  43,-23388 },
 {  44,-23388 }, {  45,-23388 }, {  46,-23388 }, {  47,-23388 }, {  48,-23388 },
 {  49,-23388 }, {  50,-23388 }, {  51,-23388 }, {  52,-23388 }, {  53,-23388 },
 {  54,-23388 }, {  55,-23388 }, {  56,-23388 }, {  57,-23388 }, {  58,-23388 },
 {  59,-23388 }, {  60,-23388 }, {  61,-23388 }, {  62,-23388 }, {  63,-23388 },
 {  64,-23388 }, {  65,-23388 }, {  66,-23388 }, {  67,-23388 }, {  68,-23388 },
 {  69,-23388 }, {  70,-23388 }, {  71,-23388 }, {  72,-23388 }, {  73,-23388 },
 {  74,-23388 }, {  75,-23388 }, {  76,-23388 }, {  77,-23388 }, {  78,-23388 },
 {  79,-23388 }, {  80,-23388 }, {  81,-23388 }, {  82,-23388 }, {  83,-23388 },

 {  84,-23388 }, {  85,-23388 }, {  86,-23388 }, {  87,-23388 }, {  88,-23388 },
 {  89,-23388 }, {  90,-23388 }, {  91,-23388 }, {  92,-23388 }, {  93,-23388 },
 {  94,-23388 }, {  95,-23388 }, {  96,-23388 }, {  97,-23388 }, {  98,-23388 },
 {  99,-23388 }, { 100,-23388 }, { 101,-23388 }, { 102,-23388 }, { 103,-23388 },
 { 104,-23388 }, { 105,-23388 }, { 106,-23388 }, { 107,-23388 }, { 108,-23388 },
 { 109,-23388 }, { 110,-23388 }, { 111,-23388 }, { 112,-23388 }, { 113,-23388 },
 { 114,-23388 }, { 115,-23388 }, { 116,-23388 }, { 117,-23388 }, { 118,-23388 },
 { 119,-23388 }, { 120,-23388 }, { 121,-23388 }, { 122,-23388 }, { 123,-23388 },
 { 124,-23388 }, { 125,-23388 }, { 126,-23388 }, { 127,-23388 }, { 128,-23388 },
 { 129,-23388 }, { 130,-23388 }, { 131,-23388 }, { 132,-23388 }, { 133,-23388 },

 { 134,-23388 }, { 135,-23388 }, { 136,-23388 }, { 137,-23388 }, { 138,-23388 },
 { 139,-23388 }, { 140,-23388 }, { 141,-23388 }, { 142,-23388 }, { 143,-23388 },
 { 144,-23388 }, { 145,-23388 }, { 146,-23388 }, { 147,-23388 }, { 148,-23388 },
 { 149,-23388 }, { 150,-23388 }, { 151,-23388 }, { 152,-23388 }, { 153,-23388 },
 { 154,-23388 }, { 155,-23388 }, { 156,-23388 }, { 157,-23388 }, { 158,-23388 },
 { 159,-23388 }, { 160,-23388 }, { 161,-23388 }, { 162,-23388 }, { 163,-23388 },
 { 164,-23388 }, { 165,-23388 }, { 166,-23388 }, { 167,-23388 }, { 168,-23388 },
 { 169,-23388 }, { 170,-23388 }, { 171,-23388 }, { 172,-23388 }, { 173,-23388 },
 { 174,-23388 }, { 175,-23388 }, { 176,-23388 }, { 177,-23388 }, { 178,-23388 },
 { 179,-23388 }, { 180,-23388 }, { 181,-23388 }, { 182,-23388 }, { 183,-23388 },

 { 184,-23388 }, { 185,-23388 }, { 186,-23388 }, { 187,-23388 }, { 188,-23388 },
 { 189,-23388 }, { 190,-23388 }, { 191,-23388 }, { 192,-23388 }, { 193,-23388 },
 { 194,-23388 }, { 195,-23388 }, { 196,-23388 }, { 197,-23388 }, { 198,-23388 },
 { 199,-23388 }, { 200,-23388 }, { 201,-23388 }, { 202,-23388 }, { 203,-23388 },
 { 204,-23388 }, { 205,-23388 }, { 206,-23388 }, { 207,-23388 }, { 208,-23388 },
 { 209,-23388 }, { 210,-23388 }, { 211,-23388 }, { 212,-23388 }, { 213,-23388 },
 { 214,-23388 }, { 215,-23388 }, { 216,-23388 }, { 217,-23388 }, { 218,-23388 },
 { 219,-23388 }, { 220,-23388 }, { 221,-23388 }, { 222,-23388 }, { 223,-23388 },
 { 224,-23388 }, { 225,-23388 }, { 226,-23388 }, { 227,-23388 }, { 228,-23388 },
 { 229,-23388 }, { 230,-23388 }, { 231,-23388 }, { 232,-23388 }, { 233,-23388 },

 { 234,-23388 }, { 235,-23388 }, { 236,-23388 }, { 237,-23388 }, { 238,-23388 },
 { 239,-23388 }, { 240,-23388 }, { 241,-23388 }, { 242,-23388 }, { 243,-23388 },
 { 244,-23388 }, { 245,-23388 }, { 246,-23388 }, { 247,-23388 }, { 248,-23388 },
 { 249,-23388 }, { 250,-23388 }, { 251,-23388 }, { 252,-23388 }, { 253,-23388 },
 { 254,-23388 }, { 255,-23388 }, { 256,-23388 }, {   0,  37 }, {   0,6001 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,  37 }, {   0,5978 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  48, 643 }, {  49, 643 }, {  50, 643 },
 {  51, 643 }, {  52, 643 }, {  53, 643 }, {  54, 643 }, {  55, 643 },
 {  56, 643 }, {  57, 643 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  65, 643 },
 {  66, 643 }, {  67, 643 }, {  68, 643 }, {  69, 643 }, {  70, 643 },
 {  48,-23731 }, {  49,-23731 }, {  50,-23731 }, {  51,-23731 }, {  52,-23731 },

 {  53,-23731 }, {  54,-23731 }, {  55,-23731 }, {  56,-23731 }, {  57,-23731 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  65,-23731 }, {  66,-23731 }, {  67,-23731 },
 {  68,-23731 }, {  69,-23731 }, {  70,-23731 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  97, 643 }, {  98, 643 }, {  99, 643 }, { 100, 643 },
 { 101, 643 }, { 102, 643 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  97,-23731 },
 {  98,-23731 }, {  99,-23731 }, { 100,-23731 }, { 101,-23731 }, { 102,-23731 },

 {   0,  55 }, {   0,5874 }, {   1, 620 }, {   2, 620 }, {   3, 620 },
 {   4, 620 }, {   5, 620 }, {   6, 620 }, {   7, 620 }, {   8, 620 },
 {   9, 878 }, {  10,-737 }, {  11, 620 }, {  12, 878 }, {  13,-737 },
 {  14, 620 }, {  15, 620 }, {  16, 620 }, {  17, 620 }, {  18, 620 },
 {  19, 620 }, {  20, 620 }, {  21, 620 }, {  22, 620 }, {  23, 620 },
 {  24, 620 }, {  25, 620 }, {  26, 620 }, {  27, 620 }, {  28, 620 },
 {  29, 620 }, {  30, 620 }, {  31, 620 }, {  32, 878 }, {  33, 620 },
 {  34, 620 }, {  35, 620 }, {  36, 620 }, {  37, 620 }, {  38, 620 },
 {  39,1136 }, {  40, 620 }, {  41, 620 }, {  42, 620 }, {  43, 620 },
 {  44, 620 }, {  45,1394 }, {  46, 620 }, {  47, 620 }, {  48, 620 },

 {  49, 620 }, {  50, 620 }, {  51, 620 }, {  52, 620 }, {  53, 620 },
 {  54, 620 }, {  55, 620 }, {  56, 620 }, {  57, 620 }, {  58, 620 },
 {  59, 620 }, {  60, 620 }, {  61, 620 }, {  62, 620 }, {  63, 620 },
 {  64, 620 }, {  65, 620 }, {  66, 620 }, {  67, 620 }, {  68, 620 },
 {  69, 620 }, {  70, 620 }, {  71, 620 }, {  72, 620 }, {  73, 620 },
 {  74, 620 }, {  75, 620 }, {  76, 620 }, {  77, 620 }, {  78, 620 },
 {  79, 620 }, {  80, 620 }, {  81, 620 }, {  82, 620 }, {  83, 620 },
 {  84, 620 }, {  85, 620 }, {  86, 620 }, {  87, 620 }, {  88, 620 },
 {  89, 620 }, {  90, 620 }, {  91, 620 }, {  92, 620 }, {  93, 620 },
 {  94, 620 }, {  95, 620 }, {  96, 620 }, {  97, 620 }, {  98, 620 },

 {  99, 620 }, { 100, 620 }, { 101, 620 }, { 102, 620 }, { 103, 620 },
 { 104, 620 }, { 105, 620 }, { 106, 620 }, { 107, 620 }, { 108, 620 },
 { 109, 620 }, { 110, 620 }, { 111, 620 }, { 112, 620 }, { 113, 620 },
 { 114, 620 }, { 115, 620 }, { 116, 620 }, { 117, 620 }, { 118, 620 },
 { 119, 620 }, { 120, 620 }, { 121, 620 }, { 122, 620 }, { 123, 620 },
 { 124, 620 }, { 125, 620 }, { 126, 620 }, { 127, 620 }, { 128, 620 },
 { 129, 620 }, { 130, 620 }, { 131, 620 }, { 132, 620 }, { 133, 620 },
 { 134, 620 }, { 135, 620 }, { 136, 620 }, { 137, 620 }, { 138, 620 },
 { 139, 620 }, { 140, 620 }, { 141, 620 }, { 142, 620 }, { 143, 620 },
 { 144, 620 }, { 145, 620 }, { 146, 620 }, { 147, 620 }, { 148, 620 },

 { 149, 620 }, { 150, 620 }, { 151, 620 }, { 152, 620 }, { 153, 620 },
 { 154, 620 }, { 155, 620 }, { 156, 620 }, { 157, 620 }, { 158, 620 },
 { 159, 620 }, { 160, 620 }, { 161, 620 }, { 162, 620 }, { 163, 620 },
 { 164, 620 }, { 165, 620 }, { 166, 620 }, { 167, 620 }, { 168, 620 },
 { 169, 620 }, { 170, 620 }, { 171, 620 }, { 172, 620 }, { 173, 620 },
 { 174, 620 }, { 175, 620 }, { 176, 620 }, { 177, 620 }, { 178, 620 },
 { 179, 620 }, { 180, 620 }, { 181, 620 }, { 182, 620 }, { 183, 620 },
 { 184, 620 }, { 185, 620 }, { 186, 620 }, { 187, 620 }, { 188, 620 },
 { 189, 620 }, { 190, 620 }, { 191, 620 }, { 192, 620 }, { 193, 620 },
 { 194, 620 }, { 195, 620 }, { 196, 620 }, { 197, 620 }, { 198, 620 },

 { 199, 620 }, { 200, 620 }, { 201, 620 }, { 202, 620 }, { 203, 620 },
 { 204, 620 }, { 205, 620 }, { 206, 620 }, { 207, 620 }, { 208, 620 },
 { 209, 620 }, { 210, 620 }, { 211, 620 }, { 212, 620 }, { 213, 620 },
 { 214, 620 }, { 215, 620 }, { 216, 620 }, { 217, 620 }, { 218, 620 },
 { 219, 620 }, { 220, 620 }, { 221, 620 }, { 222, 620 }, { 223, 620 },
 { 224, 620 }, { 225, 620 }, { 226, 620 }, { 227, 620 }, { 228, 620 },
 { 229, 620 }, { 230, 620 }, { 231, 620 }, { 232, 620 }, { 233, 620 },
 { 234, 620 }, { 235, 620 }, { 236, 620 }, { 237, 620 }, { 238, 620 },
 { 239, 620 }, { 240, 620 }, { 241, 620 }, { 242, 620 }, { 243, 620 },
 { 244, 620 }, { 245, 620 }, { 246, 620 }, { 247, 620 }, { 248, 620 },

 { 249, 620 }, { 250, 620 }, { 251, 620 }, { 252, 620 }, { 253, 620 },
 { 254, 620 }, { 255, 620 }, { 256, 620 }, {   0,  28 }, {   0,5616 },
 {   1,1394 }, {   2,1394 }, {   3,1394 }, {   4,1394 }, {   5,1394 },
 {   6,1394 }, {   7,1394 }, {   8,1394 }, {   9,1652 }, {  10,-690 },
 {  11,1394 }, {  12,1652 }, {  13,-690 }, {  14,1394 }, {  15,1394 },
 {  16,1394 }, {  17,1394 }, {  18,1394 }, {  19,1394 }, {  20,1394 },
 {  21,1394 }, {  22,1394 }, {  23,1394 }, {  24,1394 }, {  25,1394 },
 {  26,1394 }, {  27,1394 }, {  28,1394 }, {  29,1394 }, {  30,1394 },
 {  31,1394 }, {  32,1652 }, {  33,1394 }, {  34,1394 }, {  35,1394 },
 {  36,1394 }, {  37,1394 }, {  38,1394 }, {  39,1910 }, {  40,1394 },

 {  41,1394 }, {  42,1394 }, {  43,1394 }, {  44,1394 }, {  45,2168 },
 {  46,1394 }, {  47,1394 }, {  48,1394 }, {  49,1394 }, {  50,1394 },
 {  51,1394 }, {  52,1394 }, {  53,1394 }, {  54,1394 }, {  55,1394 },
 {  56,1394 }, {  57,1394 }, {  58,1394 }, {  59,1394 }, {  60,1394 },
 {  61,1394 }, {  62,1394 }, {  63,1394 }, {  64,1394 }, {  65,1394 },
 {  66,1394 }, {  67,1394 }, {  68,1394 }, {  69,1394 }, {  70,1394 },
 {  71,1394 }, {  72,1394 }, {  73,1394 }, {  74,1394 }, {  75,1394 },
 {  76,1394 }, {  77,1394 }, {  78,1394 }, {  79,1394 }, {  80,1394 },
 {  81,1394 }, {  82,1394 }, {  83,1394 }, {  84,1394 }, {  85,1394 },
 {  86,1394 }, {  87,1394 }, {  88,1394 }, {  89,1394 }, {  90,1394 },

 {  91,1394 }, {  92,1394 }, {  93,1394 }, {  94,1394 }, {  95,1394 },
 {  96,1394 }, {  97,1394 }, {  98,1394 }, {  99,1394 }, { 100,1394 },
 { 101,1394 }, { 102,1394 }, { 103,1394 }, { 104,1394 }, { 105,1394 },
 { 106,1394 }, { 107,1394 }, { 108,1394 }, { 109,1394 }, { 110,1394 },
 { 111,1394 }, { 112,1394 }, { 113,1394 }, { 114,1394 }, { 115,1394 },
 { 116,1394 }, { 117,1394 }, { 118,1394 }, { 119,1394 }, { 120,1394 },
 { 121,1394 }, { 122,1394 }, { 123,1394 }, { 124,1394 }, { 125,1394 },
 { 126,1394 }, { 127,1394 }, { 128,1394 }, { 129,1394 }, { 130,1394 },
 { 131,1394 }, { 132,1394 }, { 133,1394 }, { 134,1394 }, { 135,1394 },
 { 136,1394 }, { 137,1394 }, { 138,1394 }, { 139,1394 }, { 140,1394 },

 { 141,1394 }, { 142,1394 }, { 143,1394 }, { 144,1394 }, { 145,1394 },
 { 146,1394 }, { 147,1394 }, { 148,1394 }, { 149,1394 }, { 150,1394 },
 { 151,1394 }, { 152,1394 }, { 153,1394 }, { 154,1394 }, { 155,1394 },
 { 156,1394 }, { 157,1394 }, { 158,1394 }, { 159,1394 }, { 160,1394 },
 { 161,1394 }, { 162,1394 }, { 163,1394 }, { 164,1394 }, { 165,1394 },
 { 166,1394 }, { 167,1394 }, { 168,1394 }, { 169,1394 }, { 170,1394 },
 { 171,1394 }, { 172,1394 }, { 173,1394 }, { 174,1394 }, { 175,1394 },
 { 176,1394 }, { 177,1394 }, { 178,1394 }, { 179,1394 }, { 180,1394 },
 { 181,1394 }, { 182,1394 }, { 183,1394 }, { 184,1394 }, { 185,1394 },
 { 186,1394 }, { 187,1394 }, { 188,1394 }, { 189,1394 }, { 190,1394 },

 { 191,1394 }, { 192,1394 }, { 193,1394 }, { 194,1394 }, { 195,1394 },
 { 196,1394 }, { 197,1394 }, { 198,1394 }, { 199,1394 }, { 200,1394 },
 { 201,1394 }, { 202,1394 }, { 203,1394 }, { 204,1394 }, { 205,1394 },
 { 206,1394 }, { 207,1394 }, { 208,1394 }, { 209,1394 }, { 210,1394 },
 { 211,1394 }, { 212,1394 }, { 213,1394 }, { 214,1394 }, { 215,1394 },
 { 216,1394 }, { 217,1394 }, { 218,1394 }, { 219,1394 }, { 220,1394 },
 { 221,1394 }, { 222,1394 }, { 223,1394 }, { 224,1394 }, { 225,1394 },
 { 226,1394 }, { 227,1394 }, { 228,1394 }, { 229,1394 }, { 230,1394 },
 { 231,1394 }, { 232,1394 }, { 233,1394 }, { 234,1394 }, { 235,1394 },
 { 236,1394 }, { 237,1394 }, { 238,1394 }, { 239,1394 }, { 240,1394 },

 { 241,1394 }, { 242,1394 }, { 243,1394 }, { 244,1394 }, { 245,1394 },
 { 246,1394 }, { 247,1394 }, { 248,1394 }, { 249,1394 }, { 250,1394 },
 { 251,1394 }, { 252,1394 }, { 253,1394 }, { 254,1394 }, { 255,1394 },
 { 256,1394 }, {   0,  37 }, {   0,5358 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  48,-24345 }, {  49,-24345 }, {  50,-24345 }, {  51,-24345 }, {  52,-24345 },
 {  53,-24345 }, {  54,-24345 }, {  55,-24345 }, {  56,-24345 }, {  57,-24345 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  65,-24345 }, {  66,-24345 }, {  67,-24345 },
 {  68,-24345 }, {  69,-24345 }, {  70,-24345 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  97,-24345 },
 {  98,-24345 }, {  99,-24345 }, { 100,-24345 }, { 101,-24345 }, { 102,-24345 },
 {   0,  55 }, {   0,5254 }, {   1,   0 }, {   2,   0 }, {   3,   0 },
 {   4,   0 }, {   5,   0 }, {   6,   0 }, {   7,   0 }, {   8,   0 },
 {   9, 258 }, {  10,-1357 }, {  11,   0 }, {  12, 258 }, {  13,-1357 },
 {  14,   0 }, {  15,   0 }, {  16,   0 }, {  17,   0 }, {  18,   0 },
 {  19,   0 }, {  20,   0 }, {  21,   0 }, {  22,   0 }, {  23,   0 },
 {  24,   0 }, {  25,   0 }, {  26,   0 }, {  27,   0 }, {  28,   0 },

 {  29,   0 }, {  30,   0 }, {  31,   0 }, {  32, 258 }, {  33,   0 },
 {  34,   0 }, {  35,   0 }, {  36,   0 }, {  37,   0 }, {  38,   0 },
 {  39, 516 }, {  40,   0 }, {  41,   0 }, {  42,   0 }, {  43,   0 },
 {  44,   0 }, {  45, 774 }, {  46,   0 }, {  47,   0 }, {  48,   0 },
 {  49,   0 }, {  50,   0 }, {  51,   0 }, {  52,   0 }, {  53,   0 },
 {  54,   0 }, {  55,   0 }, {  56,   0 }, {  57,   0 }, {  58,   0 },
 {  59,   0 }, {  60,   0 }, {  61,   0 }, {  62,   0 }, {  63,   0 },
 {  64,   0 }, {  65,   0 }, {  66,   0 }, {  67,   0 }, {  68,   0 },
 {  69,   0 }, {  70,   0 }, {  71,   0 }, {  72,   0 }, {  73,   0 },
 {  74,   0 }, {  75,   0 }, {  76,   0 }, {  77,   0 }, {  78,   0 },

 {  79,   0 }, {  80,   0 }, {  81,   0 }, {  82,   0 }, {  83,   0 },
 {  84,   0 }, {  85,   0 }, {  86,   0 }, {  87,   0 }, {  88,   0 },
 {  89,   0 }, {  90,   0 }, {  91,   0 }, {  92,   0 }, {  93,   0 },
 {  94,   0 }, {  95,   0 }, {  96,   0 }, {  97,   0 }, {  98,   0 },
 {  99,   0 }, { 100,   0 }, { 101,   0 }, { 102,   0 }, { 103,   0 },
 { 104,   0 }, { 105,   0 }, { 106,   0 }, { 107,   0 }, { 108,   0 },
 { 109,   0 }, { 110,   0 }, { 111,   0 }, { 112,   0 }, { 113,   0 },
 { 114,   0 }, { 115,   0 }, { 116,   0 }, { 117,   0 }, { 118,   0 },
 { 119,   0 }, { 120,   0 }, { 121,   0 }, { 122,   0 }, { 123,   0 },
 { 124,   0 }, { 125,   0 }, { 126,   0 }, { 127,   0 }, { 128,   0 },

 { 129,   0 }, { 130,   0 }, { 131,   0 }, { 132,   0 }, { 133,   0 },
 { 134,   0 }, { 135,   0 }, { 136,   0 }, { 137,   0 }, { 138,   0 },
 { 139,   0 }, { 140,   0 }, { 141,   0 }, { 142,   0 }, { 143,   0 },
 { 144,   0 }, { 145,   0 }, { 146,   0 }, { 147,   0 }, { 148,   0 },
 { 149,   0 }, { 150,   0 }, { 151,   0 }, { 152,   0 }, { 153,   0 },
 { 154,   0 }, { 155,   0 }, { 156,   0 }, { 157,   0 }, { 158,   0 },
 { 159,   0 }, { 160,   0 }, { 161,   0 }, { 162,   0 }, { 163,   0 },
 { 164,   0 }, { 165,   0 }, { 166,   0 }, { 167,   0 }, { 168,   0 },
 { 169,   0 }, { 170,   0 }, { 171,   0 }, { 172,   0 }, { 173,   0 },
 { 174,   0 }, { 175,   0 }, { 176,   0 }, { 177,   0 }, { 178,   0 },

 { 179,   0 }, { 180,   0 }, { 181,   0 }, { 182,   0 }, { 183,   0 },
 { 184,   0 }, { 185,   0 }, { 186,   0 }, { 187,   0 }, { 188,   0 },
 { 189,   0 }, { 190,   0 }, { 191,   0 }, { 192,   0 }, { 193,   0 },
 { 194,   0 }, { 195,   0 }, { 196,   0 }, { 197,   0 }, { 198,   0 },
 { 199,   0 }, { 200,   0 }, { 201,   0 }, { 202,   0 }, { 203,   0 },
 { 204,   0 }, { 205,   0 }, { 206,   0 }, { 207,   0 }, { 208,   0 },
 { 209,   0 }, { 210,   0 }, { 211,   0 }, { 212,   0 }, { 213,   0 },
 { 214,   0 }, { 215,   0 }, { 216,   0 }, { 217,   0 }, { 218,   0 },
 { 219,   0 }, { 220,   0 }, { 221,   0 }, { 222,   0 }, { 223,   0 },
 { 224,   0 }, { 225,   0 }, { 226,   0 }, { 227,   0 }, { 228,   0 },

 { 229,   0 }, { 230,   0 }, { 231,   0 }, { 232,   0 }, { 233,   0 },
 { 234,   0 }, { 235,   0 }, { 236,   0 }, { 237,   0 }, { 238,   0 },
 { 239,   0 }, { 240,   0 }, { 241,   0 }, { 242,   0 }, { 243,   0 },
 { 244,   0 }, { 245,   0 }, { 246,   0 }, { 247,   0 }, { 248,   0 },
 { 249,   0 }, { 250,   0 }, { 251,   0 }, { 252,   0 }, { 253,   0 },
 { 254,   0 }, { 255,   0 }, { 256,   0 }, {   0,  55 }, {   0,4996 },
 {   1,-258 }, {   2,-258 }, {   3,-258 }, {   4,-258 }, {   5,-258 },
 {   6,-258 }, {   7,-258 }, {   8,-258 }, {   9,   0 }, {  10,-1615 },
 {  11,-258 }, {  12,   0 }, {  13,-1615 }, {  14,-258 }, {  15,-258 },
 {  16,-258 }, {  17,-258 }, {  18,-258 }, {  19,-258 }, {  20,-258 },

 {  21,-258 }, {  22,-258 }, {  23,-258 }, {  24,-258 }, {  25,-258 },
 {  26,-258 }, {  27,-258 }, {  28,-258 }, {  29,-258 }, {  30,-258 },
 {  31,-258 }, {  32,   0 }, {  33,-258 }, {  34,-258 }, {  35,-258 },
 {  36,-258 }, {  37,-258 }, {  38,-258 }, {  39, 258 }, {  40,-258 },
 {  41,-258 }, {  42,-258 }, {  43,-258 }, {  44,-258 }, {  45, 516 },
 {  46,-258 }, {  47,-258 }, {  48,-258 }, {  49,-258 }, {  50,-258 },
 {  51,-258 }, {  52,-258 }, {  53,-258 }, {  54,-258 }, {  55,-258 },
 {  56,-258 }, {  57,-258 }, {  58,-258 }, {  59,-258 }, {  60,-258 },
 {  61,-258 }, {  62,-258 }, {  63,-258 }, {  64,-258 }, {  65,-258 },
 {  66,-258 }, {  67,-258 }, {  68,-258 }, {  69,-258 }, {  70,-258 },

 {  71,-258 }, {  72,-258 }, {  73,-258 }, {  74,-258 }, {  75,-258 },
 {  76,-258 }, {  77,-258 }, {  78,-258 }, {  79,-258 }, {  80,-258 },
 {  81,-258 }, {  82,-258 }, {  83,-258 }, {  84,-258 }, {  85,-258 },
 {  86,-258 }, {  87,-258 }, {  88,-258 }, {  89,-258 }, {  90,-258 },
 {  91,-258 }, {  92,-258 }, {  93,-258 }, {  94,-258 }, {  95,-258 },
 {  96,-258 }, {  97,-258 }, {  98,-258 }, {  99,-258 }, { 100,-258 },
 { 101,-258 }, { 102,-258 }, { 103,-258 }, { 104,-258 }, { 105,-258 },
 { 106,-258 }, { 107,-258 }, { 108,-258 }, { 109,-258 }, { 110,-258 },
 { 111,-258 }, { 112,-258 }, { 113,-258 }, { 114,-258 }, { 115,-258 },
 { 116,-258 }, { 117,-258 }, { 118,-258 }, { 119,-258 }, { 120,-258 },

 { 121,-258 }, { 122,-258 }, { 123,-258 }, { 124,-258 }, { 125,-258 },
 { 126,-258 }, { 127,-258 }, { 128,-258 }, { 129,-258 }, { 130,-258 },
 { 131,-258 }, { 132,-258 }, { 133,-258 }, { 134,-258 }, { 135,-258 },
 { 136,-258 }, { 137,-258 }, { 138,-258 }, { 139,-258 }, { 140,-258 },
 { 141,-258 }, { 142,-258 }, { 143,-258 }, { 144,-258 }, { 145,-258 },
 { 146,-258 }, { 147,-258 }, { 148,-258 }, { 149,-258 }, { 150,-258 },
 { 151,-258 }, { 152,-258 }, { 153,-258 }, { 154,-258 }, { 155,-258 },
 { 156,-258 }, { 157,-258 }, { 158,-258 }, { 159,-258 }, { 160,-258 },
 { 161,-258 }, { 162,-258 }, { 163,-258 }, { 164,-258 }, { 165,-258 },
 { 166,-258 }, { 167,-258 }, { 168,-258 }, { 169,-258 }, { 170,-258 },

 { 171,-258 }, { 172,-258 }, { 173,-258 }, { 174,-258 }, { 175,-258 },
 { 176,-258 }, { 177,-258 }, { 178,-258 }, { 179,-258 }, { 180,-258 },
 { 181,-258 }, { 182,-258 }, { 183,-258 }, { 184,-258 }, { 185,-258 },
 { 186,-258 }, { 187,-258 }, { 188,-258 }, { 189,-258 }, { 190,-258 },
 { 191,-258 }, { 192,-258 }, { 193,-258 }, { 194,-258 }, { 195,-258 },
 { 196,-258 }, { 197,-258 }, { 198,-258 }, { 199,-258 }, { 200,-258 },
 { 201,-258 }, { 202,-258 }, { 203,-258 }, { 204,-258 }, { 205,-258 },
 { 206,-258 }, { 207,-258 }, { 208,-258 }, { 209,-258 }, { 210,-258 },
 { 211,-258 }, { 212,-258 }, { 213,-258 }, { 214,-258 }, { 215,-258 },
 { 216,-258 }, { 217,-258 }, { 218,-258 }, { 219,-258 }, { 220,-258 },

 { 221,-258 }, { 222,-258 }, { 223,-258 }, { 224,-258 }, { 225,-258 },
 { 226,-258 }, { 227,-258 }, { 228,-258 }, { 229,-258 }, { 230,-258 },
 { 231,-258 }, { 232,-258 }, { 233,-258 }, { 234,-258 }, { 235,-258 },
 { 236,-258 }, { 237,-258 }, { 238,-258 }, { 239,-258 }, { 240,-258 },
 { 241,-258 }, { 242,-258 }, { 243,-258 }, { 244,-258 }, { 245,-258 },
 { 246,-258 }, { 247,-258 }, { 248,-258 }, { 249,-258 }, { 250,-258 },
 { 251,-258 }, { 252,-258 }, { 253,-258 }, { 254,-258 }, { 255,-258 },
 { 256,-258 }, {   0,  55 }, {   0,4738 }, {   1,1548 }, {   2,1548 },
 {   3,1548 }, {   4,1548 }, {   5,1548 }, {   6,1548 }, {   7,1548 },
 {   8,1548 }, {   9,1806 }, {  10,2064 }, {  11,1548 }, {  12,1806 },

 {  13,2064 }, {  14,1548 }, {  15,1548 }, {  16,1548 }, {  17,1548 },
 {  18,1548 }, {  19,1548 }, {  20,1548 }, {  21,1548 }, {  22,1548 },
 {  23,1548 }, {  24,1548 }, {  25,1548 }, {  26,1548 }, {  27,1548 },
 {  28,1548 }, {  29,1548 }, {  30,1548 }, {  31,1548 }, {  32,1806 },
 {  33,1548 }, {  34,1548 }, {  35,1548 }, {  36,1548 }, {  37,1548 },
 {  38,1548 }, {  39,   0 }, {  40,1548 }, {  41,1548 }, {  42,1548 },
 {  43,1548 }, {  44,1548 }, {  45,2111 }, {  46,1548 }, {  47,1548 },
 {  48,1548 }, {  49,1548 }, {  50,1548 }, {  51,1548 }, {  52,1548 },
 {  53,1548 }, {  54,1548 }, {  55,1548 }, {  56,1548 }, {  57,1548 },
 {  58,1548 }, {  59,1548 }, {  60,1548 }, {  61,1548 }, {  62,1548 },

 {  63,1548 }, {  64,1548 }, {  65,1548 }, {  66,1548 }, {  67,1548 },
 {  68,1548 }, {  69,1548 }, {  70,1548 }, {  71,1548 }, {  72,1548 },
 {  73,1548 }, {  74,1548 }, {  75,1548 }, {  76,1548 }, {  77,1548 },
 {  78,1548 }, {  79,1548 }, {  80,1548 }, {  81,1548 }, {  82,1548 },
 {  83,1548 }, {  84,1548 }, {  85,1548 }, {  86,1548 }, {  87,1548 },
 {  88,1548 }, {  89,1548 }, {  90,1548 }, {  91,1548 }, {  92,1548 },
 {  93,1548 }, {  94,1548 }, {  95,1548 }, {  96,1548 }, {  97,1548 },
 {  98,1548 }, {  99,1548 }, { 100,1548 }, { 101,1548 }, { 102,1548 },
 { 103,1548 }, { 104,1548 }, { 105,1548 }, { 106,1548 }, { 107,1548 },
 { 108,1548 }, { 109,1548 }, { 110,1548 }, { 111,1548 }, { 112,1548 },

 { 113,1548 }, { 114,1548 }, { 115,1548 }, { 116,1548 }, { 117,1548 },
 { 118,1548 }, { 119,1548 }, { 120,1548 }, { 121,1548 }, { 122,1548 },
 { 123,1548 }, { 124,1548 }, { 125,1548 }, { 126,1548 }, { 127,1548 },
 { 128,1548 }, { 129,1548 }, { 130,1548 }, { 131,1548 }, { 132,1548 },
 { 133,1548 }, { 134,1548 }, { 135,1548 }, { 136,1548 }, { 137,1548 },
 { 138,1548 }, { 139,1548 }, { 140,1548 }, { 141,1548 }, { 142,1548 },
 { 143,1548 }, { 144,1548 }, { 145,1548 }, { 146,1548 }, { 147,1548 },
 { 148,1548 }, { 149,1548 }, { 150,1548 }, { 151,1548 }, { 152,1548 },
 { 153,1548 }, { 154,1548 }, { 155,1548 }, { 156,1548 }, { 157,1548 },
 { 158,1548 }, { 159,1548 }, { 160,1548 }, { 161,1548 }, { 162,1548 },

 { 163,1548 }, { 164,1548 }, { 165,1548 }, { 166,1548 }, { 167,1548 },
 { 168,1548 }, { 169,1548 }, { 170,1548 }, { 171,1548 }, { 172,1548 },
 { 173,1548 }, { 174,1548 }, { 175,1548 }, { 176,1548 }, { 177,1548 },
 { 178,1548 }, { 179,1548 }, { 180,1548 }, { 181,1548 }, { 182,1548 },
 { 183,1548 }, { 184,1548 }, { 185,1548 }, { 186,1548 }, { 187,1548 },
 { 188,1548 }, { 189,1548 }, { 190,1548 }, { 191,1548 }, { 192,1548 },
 { 193,1548 }, { 194,1548 }, { 195,1548 }, { 196,1548 }, { 197,1548 },
 { 198,1548 }, { 199,1548 }, { 200,1548 }, { 201,1548 }, { 202,1548 },
 { 203,1548 }, { 204,1548 }, { 205,1548 }, { 206,1548 }, { 207,1548 },
 { 208,1548 }, { 209,1548 }, { 210,1548 }, { 211,1548 }, { 212,1548 },

 { 213,1548 }, { 214,1548 }, { 215,1548 }, { 216,1548 }, { 217,1548 },
 { 218,1548 }, { 219,1548 }, { 220,1548 }, { 221,1548 }, { 222,1548 },
 { 223,1548 }, { 224,1548 }, { 225,1548 }, { 226,1548 }, { 227,1548 },
 { 228,1548 }, { 229,1548 }, { 230,1548 }, { 231,1548 }, { 232,1548 },
 { 233,1548 }, { 234,1548 }, { 235,1548 }, { 236,1548 }, { 237,1548 },
 { 238,1548 }, { 239,1548 }, { 240,1548 }, { 241,1548 }, { 242,1548 },
 { 243,1548 }, { 244,1548 }, { 245,1548 }, { 246,1548 }, { 247,1548 },
 { 248,1548 }, { 249,1548 }, { 250,1548 }, { 251,1548 }, { 252,1548 },
 { 253,1548 }, { 254,1548 }, { 255,1548 }, { 256,1548 }, {   0,  55 },
 {   0,4480 }, {   1,-774 }, {   2,-774 }, {   3,-774 }, {   4,-774 },

 {   5,-774 }, {   6,-774 }, {   7,-774 }, {   8,-774 }, {   9,-516 },
 {  10,-2131 }, {  11,-774 }, {  12,-516 }, {  13,-2131 }, {  14,-774 },
 {  15,-774 }, {  16,-774 }, {  17,-774 }, {  18,-774 }, {  19,-774 },
 {  20,-774 }, {  21,-774 }, {  22,-774 }, {  23,-774 }, {  24,-774 },
 {  25,-774 }, {  26,-774 }, {  27,-774 }, {  28,-774 }, {  29,-774 },
 {  30,-774 }, {  31,-774 }, {  32,-516 }, {  33,-774 }, {  34,-774 },
 {  35,-774 }, {  36,-774 }, {  37,-774 }, {  38,-774 }, {  39,-258 },
 {  40,-774 }, {  41,-774 }, {  42,-774 }, {  43,-774 }, {  44,-774 },
 {  45,2111 }, {  46,-774 }, {  47,-774 }, {  48,-774 }, {  49,-774 },
 {  50,-774 }, {  51,-774 }, {  52,-774 }, {  53,-774 }, {  54,-774 },

 {  55,-774 }, {  56,-774 }, {  57,-774 }, {  58,-774 }, {  59,-774 },
 {  60,-774 }, {  61,-774 }, {  62,-774 }, {  63,-774 }, {  64,-774 },
 {  65,-774 }, {  66,-774 }, {  67,-774 }, {  68,-774 }, {  69,-774 },
 {  70,-774 }, {  71,-774 }, {  72,-774 }, {  73,-774 }, {  74,-774 },
 {  75,-774 }, {  76,-774 }, {  77,-774 }, {  78,-774 }, {  79,-774 },
 {  80,-774 }, {  81,-774 }, {  82,-774 }, {  83,-774 }, {  84,-774 },
 {  85,-774 }, {  86,-774 }, {  87,-774 }, {  88,-774 }, {  89,-774 },
 {  90,-774 }, {  91,-774 }, {  92,-774 }, {  93,-774 }, {  94,-774 },
 {  95,-774 }, {  96,-774 }, {  97,-774 }, {  98,-774 }, {  99,-774 },
 { 100,-774 }, { 101,-774 }, { 102,-774 }, { 103,-774 }, { 104,-774 },

 { 105,-774 }, { 106,-774 }, { 107,-774 }, { 108,-774 }, { 109,-774 },
 { 110,-774 }, { 111,-774 }, { 112,-774 }, { 113,-774 }, { 114,-774 },
 { 115,-774 }, { 116,-774 }, { 117,-774 }, { 118,-774 }, { 119,-774 },
 { 120,-774 }, { 121,-774 }, { 122,-774 }, { 123,-774 }, { 124,-774 },
 { 125,-774 }, { 126,-774 }, { 127,-774 }, { 128,-774 }, { 129,-774 },
 { 130,-774 }, { 131,-774 }, { 132,-774 }, { 133,-774 }, { 134,-774 },
 { 135,-774 }, { 136,-774 }, { 137,-774 }, { 138,-774 }, { 139,-774 },
 { 140,-774 }, { 141,-774 }, { 142,-774 }, { 143,-774 }, { 144,-774 },
 { 145,-774 }, { 146,-774 }, { 147,-774 }, { 148,-774 }, { 149,-774 },
 { 150,-774 }, { 151,-774 }, { 152,-774 }, { 153,-774 }, { 154,-774 },

 { 155,-774 }, { 156,-774 }, { 157,-774 }, { 158,-774 }, { 159,-774 },
 { 160,-774 }, { 161,-774 }, { 162,-774 }, { 163,-774 }, { 164,-774 },
 { 165,-774 }, { 166,-774 }, { 167,-774 }, { 168,-774 }, { 169,-774 },
 { 170,-774 }, { 171,-774 }, { 172,-774 }, { 173,-774 }, { 174,-774 },
 { 175,-774 }, { 176,-774 }, { 177,-774 }, { 178,-774 }, { 179,-774 },
 { 180,-774 }, { 181,-774 }, { 182,-774 }, { 183,-774 }, { 184,-774 },
 { 185,-774 }, { 186,-774 }, { 187,-774 }, { 188,-774 }, { 189,-774 },
 { 190,-774 }, { 191,-774 }, { 192,-774 }, { 193,-774 }, { 194,-774 },
 { 195,-774 }, { 196,-774 }, { 197,-774 }, { 198,-774 }, { 199,-774 },
 { 200,-774 }, { 201,-774 }, { 202,-774 }, { 203,-774 }, { 204,-774 },

 { 205,-774 }, { 206,-774 }, { 207,-774 }, { 208,-774 }, { 209,-774 },
 { 210,-774 }, { 211,-774 }, { 212,-774 }, { 213,-774 }, { 214,-774 },
 { 215,-774 }, { 216,-774 }, { 217,-774 }, { 218,-774 }, { 219,-774 },
 { 220,-774 }, { 221,-774 }, { 222,-774 }, { 223,-774 }, { 224,-774 },
 { 225,-774 }, { 226,-774 }, { 227,-774 }, { 228,-774 }, { 229,-774 },
 { 230,-774 }, { 231,-774 }, { 232,-774 }, { 233,-774 }, { 234,-774 },
 { 235,-774 }, { 236,-774 }, { 237,-774 }, { 238,-774 }, { 239,-774 },
 { 240,-774 }, { 241,-774 }, { 242,-774 }, { 243,-774 }, { 244,-774 },
 { 245,-774 }, { 246,-774 }, { 247,-774 }, { 248,-774 }, { 249,-774 },
 { 250,-774 }, { 251,-774 }, { 252,-774 }, { 253,-774 }, { 254,-774 },

 { 255,-774 }, { 256,-774 }, {   0,  28 }, {   0,4222 }, {   1,   0 },
 {   2,   0 }, {   3,   0 }, {   4,   0 }, {   5,   0 }, {   6,   0 },
 {   7,   0 }, {   8,   0 }, {   9, 258 }, {  10,-2084 }, {  11,   0 },
 {  12, 258 }, {  13,-2084 }, {  14,   0 }, {  15,   0 }, {  16,   0 },
 {  17,   0 }, {  18,   0 }, {  19,   0 }, {  20,   0 }, {  21,   0 },
 {  22,   0 }, {  23,   0 }, {  24,   0 }, {  25,   0 }, {  26,   0 },
 {  27,   0 }, {  28,   0 }, {  29,   0 }, {  30,   0 }, {  31,   0 },
 {  32, 258 }, {  33,   0 }, {  34,   0 }, {  35,   0 }, {  36,   0 },
 {  37,   0 }, {  38,   0 }, {  39, 516 }, {  40,   0 }, {  41,   0 },
 {  42,   0 }, {  43,   0 }, {  44,   0 }, {  45, 774 }, {  46,   0 },

 {  47,   0 }, {  48,   0 }, {  49,   0 }, {  50,   0 }, {  51,   0 },
 {  52,   0 }, {  53,   0 }, {  54,   0 }, {  55,   0 }, {  56,   0 },
 {  57,   0 }, {  58,   0 }, {  59,   0 }, {  60,   0 }, {  61,   0 },
 {  62,   0 }, {  63,   0 }, {  64,   0 }, {  65,   0 }, {  66,   0 },
 {  67,   0 }, {  68,   0 }, {  69,   0 }, {  70,   0 }, {  71,   0 },
 {  72,   0 }, {  73,   0 }, {  74,   0 }, {  75,   0 }, {  76,   0 },
 {  77,   0 }, {  78,   0 }, {  79,   0 }, {  80,   0 }, {  81,   0 },
 {  82,   0 }, {  83,   0 }, {  84,   0 }, {  85,   0 }, {  86,   0 },
 {  87,   0 }, {  88,   0 }, {  89,   0 }, {  90,   0 }, {  91,   0 },
 {  92,   0 }, {  93,   0 }, {  94,   0 }, {  95,   0 }, {  96,   0 },

 {  97,   0 }, {  98,   0 }, {  99,   0 }, { 100,   0 }, { 101,   0 },
 { 102,   0 }, { 103,   0 }, { 104,   0 }, { 105,   0 }, { 106,   0 },
 { 107,   0 }, { 108,   0 }, { 109,   0 }, { 110,   0 }, { 111,   0 },
 { 112,   0 }, { 113,   0 }, { 114,   0 }, { 115,   0 }, { 116,   0 },
 { 117,   0 }, { 118,   0 }, { 119,   0 }, { 120,   0 }, { 121,   0 },
 { 122,   0 }, { 123,   0 }, { 124,   0 }, { 125,   0 }, { 126,   0 },
 { 127,   0 }, { 128,   0 }, { 129,   0 }, { 130,   0 }, { 131,   0 },
 { 132,   0 }, { 133,   0 }, { 134,   0 }, { 135,   0 }, { 136,   0 },
 { 137,   0 }, { 138,   0 }, { 139,   0 }, { 140,   0 }, { 141,   0 },
 { 142,   0 }, { 143,   0 }, { 144,   0 }, { 145,   0 }, { 146,   0 },

 { 147,   0 }, { 148,   0 }, { 149,   0 }, { 150,   0 }, { 151,   0 },
 { 152,   0 }, { 153,   0 }, { 154,   0 }, { 155,   0 }, { 156,   0 },
 { 157,   0 }, { 158,   0 }, { 159,   0 }, { 160,   0 }, { 161,   0 },
 { 162,   0 }, { 163,   0 }, { 164,   0 }, { 165,   0 }, { 166,   0 },
 { 167,   0 }, { 168,   0 }, { 169,   0 }, { 170,   0 }, { 171,   0 },
 { 172,   0 }, { 173,   0 }, { 174,   0 }, { 175,   0 }, { 176,   0 },
 { 177,   0 }, { 178,   0 }, { 179,   0 }, { 180,   0 }, { 181,   0 },
 { 182,   0 }, { 183,   0 }, { 184,   0 }, { 185,   0 }, { 186,   0 },
 { 187,   0 }, { 188,   0 }, { 189,   0 }, { 190,   0 }, { 191,   0 },
 { 192,   0 }, { 193,   0 }, { 194,   0 }, { 195,   0 }, { 196,   0 },

 { 197,   0 }, { 198,   0 }, { 199,   0 }, { 200,   0 }, { 201,   0 },
 { 202,   0 }, { 203,   0 }, { 204,   0 }, { 205,   0 }, { 206,   0 },
 { 207,   0 }, { 208,   0 }, { 209,   0 }, { 210,   0 }, { 211,   0 },
 { 212,   0 }, { 213,   0 }, { 214,   0 }, { 215,   0 }, { 216,   0 },
 { 217,   0 }, { 218,   0 }, { 219,   0 }, { 220,   0 }, { 221,   0 },
 { 222,   0 }, { 223,   0 }, { 224,   0 }, { 225,   0 }, { 226,   0 },
 { 227,   0 }, { 228,   0 }, { 229,   0 }, { 230,   0 }, { 231,   0 },
 { 232,   0 }, { 233,   0 }, { 234,   0 }, { 235,   0 }, { 236,   0 },
 { 237,   0 }, { 238,   0 }, { 239,   0 }, { 240,   0 }, { 241,   0 },
 { 242,   0 }, { 243,   0 }, { 244,   0 }, { 245,   0 }, { 246,   0 },

 { 247,   0 }, { 248,   0 }, { 249,   0 }, { 250,   0 }, { 251,   0 },
 { 252,   0 }, { 253,   0 }, { 254,   0 }, { 255,   0 }, { 256,   0 },
 {   0,  28 }, {   0,3964 }, {   1,-258 }, {   2,-258 }, {   3,-258 },
 {   4,-258 }, {   5,-258 }, {   6,-258 }, {   7,-258 }, {   8,-258 },
 {   9,   0 }, {  10,-2342 }, {  11,-258 }, {  12,   0 }, {  13,-2342 },
 {  14,-258 }, {  15,-258 }, {  16,-258 }, {  17,-258 }, {  18,-258 },
 {  19,-258 }, {  20,-258 }, {  21,-258 }, {  22,-258 }, {  23,-258 },
 {  24,-258 }, {  25,-258 }, {  26,-258 }, {  27,-258 }, {  28,-258 },
 {  29,-258 }, {  30,-258 }, {  31,-258 }, {  32,   0 }, {  33,-258 },
 {  34,-258 }, {  35,-258 }, {  36,-258 }, {  37,-258 }, {  38,-258 },

 {  39, 258 }, {  40,-258 }, {  41,-258 }, {  42,-258 }, {  43,-258 },
 {  44,-258 }, {  45, 516 }, {  46,-258 }, {  47,-258 }, {  48,-258 },
 {  49,-258 }, {  50,-258 }, {  51,-258 }, {  52,-258 }, {  53,-258 },
 {  54,-258 }, {  55,-258 }, {  56,-258 }, {  57,-258 }, {  58,-258 },
 {  59,-258 }, {  60,-258 }, {  61,-258 }, {  62,-258 }, {  63,-258 },
 {  64,-258 }, {  65,-258 }, {  66,-258 }, {  67,-258 }, {  68,-258 },
 {  69,-258 }, {  70,-258 }, {  71,-258 }, {  72,-258 }, {  73,-258 },
 {  74,-258 }, {  75,-258 }, {  76,-258 }, {  77,-258 }, {  78,-258 },
 {  79,-258 }, {  80,-258 }, {  81,-258 }, {  82,-258 }, {  83,-258 },
 {  84,-258 }, {  85,-258 }, {  86,-258 }, {  87,-258 }, {  88,-258 },

 {  89,-258 }, {  90,-258 }, {  91,-258 }, {  92,-258 }, {  93,-258 },
 {  94,-258 }, {  95,-258 }, {  96,-258 }, {  97,-258 }, {  98,-258 },
 {  99,-258 }, { 100,-258 }, { 101,-258 }, { 102,-258 }, { 103,-258 },
 { 104,-258 }, { 105,-258 }, { 106,-258 }, { 107,-258 }, { 108,-258 },
 { 109,-258 }, { 110,-258 }, { 111,-258 }, { 112,-258 }, { 113,-258 },
 { 114,-258 }, { 115,-258 }, { 116,-258 }, { 117,-258 }, { 118,-258 },
 { 119,-258 }, { 120,-258 }, { 121,-258 }, { 122,-258 }, { 123,-258 },
 { 124,-258 }, { 125,-258 }, { 126,-258 }, { 127,-258 }, { 128,-258 },
 { 129,-258 }, { 130,-258 }, { 131,-258 }, { 132,-258 }, { 133,-258 },
 { 134,-258 }, { 135,-258 }, { 136,-258 }, { 137,-258 }, { 138,-258 },

 { 139,-258 }, { 140,-258 }, { 141,-258 }, { 142,-258 }, { 143,-258 },
 { 144,-258 }, { 145,-258 }, { 146,-258 }, { 147,-258 }, { 148,-258 },
 { 149,-258 }, { 150,-258 }, { 151,-258 }, { 152,-258 }, { 153,-258 },
 { 154,-258 }, { 155,-258 }, { 156,-258 }, { 157,-258 }, { 158,-258 },
 { 159,-258 }, { 160,-258 }, { 161,-258 }, { 162,-258 }, { 163,-258 },
 { 164,-258 }, { 165,-258 }, { 166,-258 }, { 167,-258 }, { 168,-258 },
 { 169,-258 }, { 170,-258 }, { 171,-258 }, { 172,-258 }, { 173,-258 },
 { 174,-258 }, { 175,-258 }, { 176,-258 }, { 177,-258 }, { 178,-258 },
 { 179,-258 }, { 180,-258 }, { 181,-258 }, { 182,-258 }, { 183,-258 },
 { 184,-258 }, { 185,-258 }, { 186,-258 }, { 187,-258 }, { 188,-258 },

 { 189,-258 }, { 190,-258 }, { 191,-258 }, { 192,-258 }, { 193,-258 },
 { 194,-258 }, { 195,-258 }, { 196,-258 }, { 197,-258 }, { 198,-258 },
 { 199,-258 }, { 200,-258 }, { 201,-258 }, { 202,-258 }, { 203,-258 },
 { 204,-258 }, { 205,-258 }, { 206,-258 }, { 207,-258 }, { 208,-258 },
 { 209,-258 }, { 210,-258 }, { 211,-258 }, { 212,-258 }, { 213,-258 },
 { 214,-258 }, { 215,-258 }, { 216,-258 }, { 217,-258 }, { 218,-258 },
 { 219,-258 }, { 220,-258 }, { 221,-258 }, { 222,-258 }, { 223,-258 },
 { 224,-258 }, { 225,-258 }, { 226,-258 }, { 227,-258 }, { 228,-258 },
 { 229,-258 }, { 230,-258 }, { 231,-258 }, { 232,-258 }, { 233,-258 },
 { 234,-258 }, { 235,-258 }, { 236,-258 }, { 237,-258 }, { 238,-258 },

 { 239,-258 }, { 240,-258 }, { 241,-258 }, { 242,-258 }, { 243,-258 },
 { 244,-258 }, { 245,-258 }, { 246,-258 }, { 247,-258 }, { 248,-258 },
 { 249,-258 }, { 250,-258 }, { 251,-258 }, { 252,-258 }, { 253,-258 },
 { 254,-258 }, { 255,-258 }, { 256,-258 }, {   0,  28 }, {   0,3706 },
 {   1,1595 }, {   2,1595 }, {   3,1595 }, {   4,1595 }, {   5,1595 },
 {   6,1595 }, {   7,1595 }, {   8,1595 }, {   9,1853 }, {  10,2111 },
 {  11,1595 }, {  12,1853 }, {  13,2111 }, {  14,1595 }, {  15,1595 },
 {  16,1595 }, {  17,1595 }, {  18,1595 }, {  19,1595 }, {  20,1595 },
 {  21,1595 }, {  22,1595 }, {  23,1595 }, {  24,1595 }, {  25,1595 },
 {  26,1595 }, {  27,1595 }, {  28,1595 }, {  29,1595 }, {  30,1595 },

 {  31,1595 }, {  32,1853 }, {  33,1595 }, {  34,1595 }, {  35,1595 },
 {  36,1595 }, {  37,1595 }, {  38,1595 }, {  39,   0 }, {  40,1595 },
 {  41,1595 }, {  42,1595 }, {  43,1595 }, {  44,1595 }, {  45,2158 },
 {  46,1595 }, {  47,1595 }, {  48,1595 }, {  49,1595 }, {  50,1595 },
 {  51,1595 }, {  52,1595 }, {  53,1595 }, {  54,1595 }, {  55,1595 },
 {  56,1595 }, {  57,1595 }, {  58,1595 }, {  59,1595 }, {  60,1595 },
 {  61,1595 }, {  62,1595 }, {  63,1595 }, {  64,1595 }, {  65,1595 },
 {  66,1595 }, {  67,1595 }, {  68,1595 }, {  69,1595 }, {  70,1595 },
 {  71,1595 }, {  72,1595 }, {  73,1595 }, {  74,1595 }, {  75,1595 },
 {  76,1595 }, {  77,1595 }, {  78,1595 }, {  79,1595 }, {  80,1595 },

 {  81,1595 }, {  82,1595 }, {  83,1595 }, {  84,1595 }, {  85,1595 },
 {  86,1595 }, {  87,1595 }, {  88,1595 }, {  89,1595 }, {  90,1595 },
 {  91,1595 }, {  92,1595 }, {  93,1595 }, {  94,1595 }, {  95,1595 },
 {  96,1595 }, {  97,1595 }, {  98,1595 }, {  99,1595 }, { 100,1595 },
 { 101,1595 }, { 102,1595 }, { 103,1595 }, { 104,1595 }, { 105,1595 },
 { 106,1595 }, { 107,1595 }, { 108,1595 }, { 109,1595 }, { 110,1595 },
 { 111,1595 }, { 112,1595 }, { 113,1595 }, { 114,1595 }, { 115,1595 },
 { 116,1595 }, { 117,1595 }, { 118,1595 }, { 119,1595 }, { 120,1595 },
 { 121,1595 }, { 122,1595 }, { 123,1595 }, { 124,1595 }, { 125,1595 },
 { 126,1595 }, { 127,1595 }, { 128,1595 }, { 129,1595 }, { 130,1595 },

 { 131,1595 }, { 132,1595 }, { 133,1595 }, { 134,1595 }, { 135,1595 },
 { 136,1595 }, { 137,1595 }, { 138,1595 }, { 139,1595 }, { 140,1595 },
 { 141,1595 }, { 142,1595 }, { 143,1595 }, { 144,1595 }, { 145,1595 },
 { 146,1595 }, { 147,1595 }, { 148,1595 }, { 149,1595 }, { 150,1595 },
 { 151,1595 }, { 152,1595 }, { 153,1595 }, { 154,1595 }, { 155,1595 },
 { 156,1595 }, { 157,1595 }, { 158,1595 }, { 159,1595 }, { 160,1595 },
 { 161,1595 }, { 162,1595 }, { 163,1595 }, { 164,1595 }, { 165,1595 },
 { 166,1595 }, { 167,1595 }, { 168,1595 }, { 169,1595 }, { 170,1595 },
 { 171,1595 }, { 172,1595 }, { 173,1595 }, { 174,1595 }, { 175,1595 },
 { 176,1595 }, { 177,1595 }, { 178,1595 }, { 179,1595 }, { 180,1595 },

 { 181,1595 }, { 182,1595 }, { 183,1595 }, { 184,1595 }, { 185,1595 },
 { 186,1595 }, { 187,1595 }, { 188,1595 }, { 189,1595 }, { 190,1595 },
 { 191,1595 }, { 192,1595 }, { 193,1595 }, { 194,1595 }, { 195,1595 },
 { 196,1595 }, { 197,1595 }, { 198,1595 }, { 199,1595 }, { 200,1595 },
 { 201,1595 }, { 202,1595 }, { 203,1595 }, { 204,1595 }, { 205,1595 },
 { 206,1595 }, { 207,1595 }, { 208,1595 }, { 209,1595 }, { 210,1595 },
 { 211,1595 }, { 212,1595 }, { 213,1595 }, { 214,1595 }, { 215,1595 },
 { 216,1595 }, { 217,1595 }, { 218,1595 }, { 219,1595 }, { 220,1595 },
 { 221,1595 }, { 222,1595 }, { 223,1595 }, { 224,1595 }, { 225,1595 },
 { 226,1595 }, { 227,1595 }, { 228,1595 }, { 229,1595 }, { 230,1595 },

 { 231,1595 }, { 232,1595 }, { 233,1595 }, { 234,1595 }, { 235,1595 },
 { 236,1595 }, { 237,1595 }, { 238,1595 }, { 239,1595 }, { 240,1595 },
 { 241,1595 }, { 242,1595 }, { 243,1595 }, { 244,1595 }, { 245,1595 },
 { 246,1595 }, { 247,1595 }, { 248,1595 }, { 249,1595 }, { 250,1595 },
 { 251,1595 }, { 252,1595 }, { 253,1595 }, { 254,1595 }, { 255,1595 },
 { 256,1595 }, {   0,  28 }, {   0,3448 }, {   1,-774 }, {   2,-774 },
 {   3,-774 }, {   4,-774 }, {   5,-774 }, {   6,-774 }, {   7,-774 },
 {   8,-774 }, {   9,-516 }, {  10,-2858 }, {  11,-774 }, {  12,-516 },
 {  13,-2858 }, {  14,-774 }, {  15,-774 }, {  16,-774 }, {  17,-774 },
 {  18,-774 }, {  19,-774 }, {  20,-774 }, {  21,-774 }, {  22,-774 },

 {  23,-774 }, {  24,-774 }, {  25,-774 }, {  26,-774 }, {  27,-774 },
 {  28,-774 }, {  29,-774 }, {  30,-774 }, {  31,-774 }, {  32,-516 },
 {  33,-774 }, {  34,-774 }, {  35,-774 }, {  36,-774 }, {  37,-774 },
 {  38,-774 }, {  39,-258 }, {  40,-774 }, {  41,-774 }, {  42,-774 },
 {  43,-774 }, {  44,-774 }, {  45,2158 }, {  46,-774 }, {  47,-774 },
 {  48,-774 }, {  49,-774 }, {  50,-774 }, {  51,-774 }, {  52,-774 },
 {  53,-774 }, {  54,-774 }, {  55,-774 }, {  56,-774 }, {  57,-774 },
 {  58,-774 }, {  59,-774 }, {  60,-774 }, {  61,-774 }, {  62,-774 },
 {  63,-774 }, {  64,-774 }, {  65,-774 }, {  66,-774 }, {  67,-774 },
 {  68,-774 }, {  69,-774 }, {  70,-774 }, {  71,-774 }, {  72,-774 },

 {  73,-774 }, {  74,-774 }, {  75,-774 }, {  76,-774 }, {  77,-774 },
 {  78,-774 }, {  79,-774 }, {  80,-774 }, {  81,-774 }, {  82,-774 },
 {  83,-774 }, {  84,-774 }, {  85,-774 }, {  86,-774 }, {  87,-774 },
 {  88,-774 }, {  89,-774 }, {  90,-774 }, {  91,-774 }, {  92,-774 },
 {  93,-774 }, {  94,-774 }, {  95,-774 }, {  96,-774 }, {  97,-774 },
 {  98,-774 }, {  99,-774 }, { 100,-774 }, { 101,-774 }, { 102,-774 },
 { 103,-774 }, { 104,-774 }, { 105,-774 }, { 106,-774 }, { 107,-774 },
 { 108,-774 }, { 109,-774 }, { 110,-774 }, { 111,-774 }, { 112,-774 },
 { 113,-774 }, { 114,-774 }, { 115,-774 }, { 116,-774 }, { 117,-774 },
 { 118,-774 }, { 119,-774 }, { 120,-774 }, { 121,-774 }, { 122,-774 },

 { 123,-774 }, { 124,-774 }, { 125,-774 }, { 126,-774 }, { 127,-774 },
 { 128,-774 }, { 129,-774 }, { 130,-774 }, { 131,-774 }, { 132,-774 },
 { 133,-774 }, { 134,-774 }, { 135,-774 }, { 136,-774 }, { 137,-774 },
 { 138,-774 }, { 139,-774 }, { 140,-774 }, { 141,-774 }, { 142,-774 },
 { 143,-774 }, { 144,-774 }, { 145,-774 }, { 146,-774 }, { 147,-774 },
 { 148,-774 }, { 149,-774 }, { 150,-774 }, { 151,-774 }, { 152,-774 },
 { 153,-774 }, { 154,-774 }, { 155,-774 }, { 156,-774 }, { 157,-774 },
 { 158,-774 }, { 159,-774 }, { 160,-774 }, { 161,-774 }, { 162,-774 },
 { 163,-774 }, { 164,-774 }, { 165,-774 }, { 166,-774 }, { 167,-774 },
 { 168,-774 }, { 169,-774 }, { 170,-774 }, { 171,-774 }, { 172,-774 },

 { 173,-774 }, { 174,-774 }, { 175,-774 }, { 176,-774 }, { 177,-774 },
 { 178,-774 }, { 179,-774 }, { 180,-774 }, { 181,-774 }, { 182,-774 },
 { 183,-774 }, { 184,-774 }, { 185,-774 }, { 186,-774 }, { 187,-774 },
 { 188,-774 }, { 189,-774 }, { 190,-774 }, { 191,-774 }, { 192,-774 },
 { 193,-774 }, { 194,-774 }, { 195,-774 }, { 196,-774 }, { 197,-774 },
 { 198,-774 }, { 199,-774 }, { 200,-774 }, { 201,-774 }, { 202,-774 },
 { 203,-774 }, { 204,-774 }, { 205,-774 }, { 206,-774 }, { 207,-774 },
 { 208,-774 }, { 209,-774 }, { 210,-774 }, { 211,-774 }, { 212,-774 },
 { 213,-774 }, { 214,-774 }, { 215,-774 }, { 216,-774 }, { 217,-774 },
 { 218,-774 }, { 219,-774 }, { 220,-774 }, { 221,-774 }, { 222,-774 },

 { 223,-774 }, { 224,-774 }, { 225,-774 }, { 226,-774 }, { 227,-774 },
 { 228,-774 }, { 229,-774 }, { 230,-774 }, { 231,-774 }, { 232,-774 },
 { 233,-774 }, { 234,-774 }, { 235,-774 }, { 236,-774 }, { 237,-774 },
 { 238,-774 }, { 239,-774 }, { 240,-774 }, { 241,-774 }, { 242,-774 },
 { 243,-774 }, { 244,-774 }, { 245,-774 }, { 246,-774 }, { 247,-774 },
 { 248,-774 }, { 249,-774 }, { 250,-774 }, { 251,-774 }, { 252,-774 },
 { 253,-774 }, { 254,-774 }, { 255,-774 }, { 256,-774 }, {   0,  55 },
 {   0,3190 }, {   1,-2064 }, {   2,-2064 }, {   3,-2064 }, {   4,-2064 },
 {   5,-2064 }, {   6,-2064 }, {   7,-2064 }, {   8,-2064 }, {   9,-1806 },
 {  10,-3421 }, {  11,-2064 }, {  12,-1806 }, {  13,-3421 }, {  14,-2064 },

 {  15,-2064 }, {  16,-2064 }, {  17,-2064 }, {  18,-2064 }, {  19,-2064 },
 {  20,-2064 }, {  21,-2064 }, {  22,-2064 }, {  23,-2064 }, {  24,-2064 },
 {  25,-2064 }, {  26,-2064 }, {  27,-2064 }, {  28,-2064 }, {  29,-2064 },
 {  30,-2064 }, {  31,-2064 }, {  32,-1806 }, {  33,-2064 }, {  34,-2064 },
 {  35,-2064 }, {  36,-2064 }, {  37,-2064 }, {  38,-2064 }, {  39,2158 },
 {  40,-2064 }, {  41,-2064 }, {  42,-2064 }, {  43,-2064 }, {  44,-2064 },
 {  45,-1290 }, {  46,-2064 }, {  47,-2064 }, {  48,-2064 }, {  49,-2064 },
 {  50,-2064 }, {  51,-2064 }, {  52,-2064 }, {  53,-2064 }, {  54,-2064 },
 {  55,-2064 }, {  56,-2064 }, {  57,-2064 }, {  58,-2064 }, {  59,-2064 },
 {  60,-2064 }, {  61,-2064 }, {  62,-2064 }, {  63,-2064 }, {  64,-2064 },

 {  65,-2064 }, {  66,-2064 }, {  67,-2064 }, {  68,-2064 }, {  69,-2064 },
 {  70,-2064 }, {  71,-2064 }, {  72,-2064 }, {  73,-2064 }, {  74,-2064 },
 {  75,-2064 }, {  76,-2064 }, {  77,-2064 }, {  78,-2064 }, {  79,-2064 },
 {  80,-2064 }, {  81,-2064 }, {  82,-2064 }, {  83,-2064 }, {  84,-2064 },
 {  85,-2064 }, {  86,-2064 }, {  87,-2064 }, {  88,-2064 }, {  89,-2064 },
 {  90,-2064 }, {  91,-2064 }, {  92,-2064 }, {  93,-2064 }, {  94,-2064 },
 {  95,-2064 }, {  96,-2064 }, {  97,-2064 }, {  98,-2064 }, {  99,-2064 },
 { 100,-2064 }, { 101,-2064 }, { 102,-2064 }, { 103,-2064 }, { 104,-2064 },
 { 105,-2064 }, { 106,-2064 }, { 107,-2064 }, { 108,-2064 }, { 109,-2064 },
 { 110,-2064 }, { 111,-2064 }, { 112,-2064 }, { 113,-2064 }, { 114,-2064 },

 { 115,-2064 }, { 116,-2064 }, { 117,-2064 }, { 118,-2064 }, { 119,-2064 },
 { 120,-2064 }, { 121,-2064 }, { 122,-2064 }, { 123,-2064 }, { 124,-2064 },
 { 125,-2064 }, { 126,-2064 }, { 127,-2064 }, { 128,-2064 }, { 129,-2064 },
 { 130,-2064 }, { 131,-2064 }, { 132,-2064 }, { 133,-2064 }, { 134,-2064 },
 { 135,-2064 }, { 136,-2064 }, { 137,-2064 }, { 138,-2064 }, { 139,-2064 },
 { 140,-2064 }, { 141,-2064 }, { 142,-2064 }, { 143,-2064 }, { 144,-2064 },
 { 145,-2064 }, { 146,-2064 }, { 147,-2064 }, { 148,-2064 }, { 149,-2064 },
 { 150,-2064 }, { 151,-2064 }, { 152,-2064 }, { 153,-2064 }, { 154,-2064 },
 { 155,-2064 }, { 156,-2064 }, { 157,-2064 }, { 158,-2064 }, { 159,-2064 },
 { 160,-2064 }, { 161,-2064 }, { 162,-2064 }, { 163,-2064 }, { 164,-2064 },

 { 165,-2064 }, { 166,-2064 }, { 167,-2064 }, { 168,-2064 }, { 169,-2064 },
 { 170,-2064 }, { 171,-2064 }, { 172,-2064 }, { 173,-2064 }, { 174,-2064 },
 { 175,-2064 }, { 176,-2064 }, { 177,-2064 }, { 178,-2064 }, { 179,-2064 },
 { 180,-2064 }, { 181,-2064 }, { 182,-2064 }, { 183,-2064 }, { 184,-2064 },
 { 185,-2064 }, { 186,-2064 }, { 187,-2064 }, { 188,-2064 }, { 189,-2064 },
 { 190,-2064 }, { 191,-2064 }, { 192,-2064 }, { 193,-2064 }, { 194,-2064 },
 { 195,-2064 }, { 196,-2064 }, { 197,-2064 }, { 198,-2064 }, { 199,-2064 },
 { 200,-2064 }, { 201,-2064 }, { 202,-2064 }, { 203,-2064 }, { 204,-2064 },
 { 205,-2064 }, { 206,-2064 }, { 207,-2064 }, { 208,-2064 }, { 209,-2064 },
 { 210,-2064 }, { 211,-2064 }, { 212,-2064 }, { 213,-2064 }, { 214,-2064 },

 { 215,-2064 }, { 216,-2064 }, { 217,-2064 }, { 218,-2064 }, { 219,-2064 },
 { 220,-2064 }, { 221,-2064 }, { 222,-2064 }, { 223,-2064 }, { 224,-2064 },
 { 225,-2064 }, { 226,-2064 }, { 227,-2064 }, { 228,-2064 }, { 229,-2064 },
 { 230,-2064 }, { 231,-2064 }, { 232,-2064 }, { 233,-2064 }, { 234,-2064 },
 { 235,-2064 }, { 236,-2064 }, { 237,-2064 }, { 238,-2064 }, { 239,-2064 },
 { 240,-2064 }, { 241,-2064 }, { 242,-2064 }, { 243,-2064 }, { 244,-2064 },
 { 245,-2064 }, { 246,-2064 }, { 247,-2064 }, { 248,-2064 }, { 249,-2064 },
 { 250,-2064 }, { 251,-2064 }, { 252,-2064 }, { 253,-2064 }, { 254,-2064 },
 { 255,-2064 }, { 256,-2064 }, {   0,  55 }, {   0,2932 }, {   1,-2322 },
 {   2,-2322 }, {   3,-2322 }, {   4,-2322 }, {   5,-2322 }, {   6,-2322 },

 {   7,-2322 }, {   8,-2322 }, {   9,-2064 }, {  10,-3679 }, {  11,-2322 },
 {  12,-2064 }, {  13,-3679 }, {  14,-2322 }, {  15,-2322 }, {  16,-2322 },
 {  17,-2322 }, {  18,-2322 }, {  19,-2322 }, {  20,-2322 }, {  21,-2322 },
 {  22,-2322 }, {  23,-2322 }, {  24,-2322 }, {  25,-2322 }, {  26,-2322 },
 {  27,-2322 }, {  28,-2322 }, {  29,-2322 }, {  30,-2322 }, {  31,-2322 },
 {  32,-2064 }, {  33,-2322 }, {  34,-2322 }, {  35,-2322 }, {  36,-2322 },
 {  37,-2322 }, {  38,-2322 }, {  39,1900 }, {  40,-2322 }, {  41,-2322 },
 {  42,-2322 }, {  43,-2322 }, {  44,-2322 }, {  45,-1548 }, {  46,-2322 },
 {  47,-2322 }, {  48,-2322 }, {  49,-2322 }, {  50,-2322 }, {  51,-2322 },
 {  52,-2322 }, {  53,-2322 }, {  54,-2322 }, {  55,-2322 }, {  56,-2322 },

 {  57,-2322 }, {  58,-2322 }, {  59,-2322 }, {  60,-2322 }, {  61,-2322 },
 {  62,-2322 }, {  63,-2322 }, {  64,-2322 }, {  65,-2322 }, {  66,-2322 },
 {  67,-2322 }, {  68,-2322 }, {  69,-2322 }, {  70,-2322 }, {  71,-2322 },
 {  72,-2322 }, {  73,-2322 }, {  74,-2322 }, {  75,-2322 }, {  76,-2322 },
 {  77,-2322 }, {  78,-2322 }, {  79,-2322 }, {  80,-2322 }, {  81,-2322 },
 {  82,-2322 }, {  83,-2322 }, {  84,-2322 }, {  85,-2322 }, {  86,-2322 },
 {  87,-2322 }, {  88,-2322 }, {  89,-2322 }, {  90,-2322 }, {  91,-2322 },
 {  92,-2322 }, {  93,-2322 }, {  94,-2322 }, {  95,-2322 }, {  96,-2322 },
 {  97,-2322 }, {  98,-2322 }, {  99,-2322 }, { 100,-2322 }, { 101,-2322 },
 { 102,-2322 }, { 103,-2322 }, { 104,-2322 }, { 105,-2322 }, { 106,-2322 },

 { 107,-2322 }, { 108,-2322 }, { 109,-2322 }, { 110,-2322 }, { 111,-2322 },
 { 112,-2322 }, { 113,-2322 }, { 114,-2322 }, { 115,-2322 }, { 116,-2322 },
 { 117,-2322 }, { 118,-2322 }, { 119,-2322 }, { 120,-2322 }, { 121,-2322 },
 { 122,-2322 }, { 123,-2322 }, { 124,-2322 }, { 125,-2322 }, { 126,-2322 },
 { 127,-2322 }, { 128,-2322 }, { 129,-2322 }, { 130,-2322 }, { 131,-2322 },
 { 132,-2322 }, { 133,-2322 }, { 134,-2322 }, { 135,-2322 }, { 136,-2322 },
 { 137,-2322 }, { 138,-2322 }, { 139,-2322 }, { 140,-2322 }, { 141,-2322 },
 { 142,-2322 }, { 143,-2322 }, { 144,-2322 }, { 145,-2322 }, { 146,-2322 },
 { 147,-2322 }, { 148,-2322 }, { 149,-2322 }, { 150,-2322 }, { 151,-2322 },
 { 152,-2322 }, { 153,-2322 }, { 154,-2322 }, { 155,-2322 }, { 156,-2322 },

 { 157,-2322 }, { 158,-2322 }, { 159,-2322 }, { 160,-2322 }, { 161,-2322 },
 { 162,-2322 }, { 163,-2322 }, { 164,-2322 }, { 165,-2322 }, { 166,-2322 },
 { 167,-2322 }, { 168,-2322 }, { 169,-2322 }, { 170,-2322 }, { 171,-2322 },
 { 172,-2322 }, { 173,-2322 }, { 174,-2322 }, { 175,-2322 }, { 176,-2322 },
 { 177,-2322 }, { 178,-2322 }, { 179,-2322 }, { 180,-2322 }, { 181,-2322 },
 { 182,-2322 }, { 183,-2322 }, { 184,-2322 }, { 185,-2322 }, { 186,-2322 },
 { 187,-2322 }, { 188,-2322 }, { 189,-2322 }, { 190,-2322 }, { 191,-2322 },
 { 192,-2322 }, { 193,-2322 }, { 194,-2322 }, { 195,-2322 }, { 196,-2322 },
 { 197,-2322 }, { 198,-2322 }, { 199,-2322 }, { 200,-2322 }, { 201,-2322 },
 { 202,-2322 }, { 203,-2322 }, { 204,-2322 }, { 205,-2322 }, { 206,-2322 },

 { 207,-2322 }, { 208,-2322 }, { 209,-2322 }, { 210,-2322 }, { 211,-2322 },
 { 212,-2322 }, { 213,-2322 }, { 214,-2322 }, { 215,-2322 }, { 216,-2322 },
 { 217,-2322 }, { 218,-2322 }, { 219,-2322 }, { 220,-2322 }, { 221,-2322 },
 { 222,-2322 }, { 223,-2322 }, { 224,-2322 }, { 225,-2322 }, { 226,-2322 },
 { 227,-2322 }, { 228,-2322 }, { 229,-2322 }, { 230,-2322 }, { 231,-2322 },
 { 232,-2322 }, { 233,-2322 }, { 234,-2322 }, { 235,-2322 }, { 236,-2322 },
 { 237,-2322 }, { 238,-2322 }, { 239,-2322 }, { 240,-2322 }, { 241,-2322 },
 { 242,-2322 }, { 243,-2322 }, { 244,-2322 }, { 245,-2322 }, { 246,-2322 },
 { 247,-2322 }, { 248,-2322 }, { 249,-2322 }, { 250,-2322 }, { 251,-2322 },
 { 252,-2322 }, { 253,-2322 }, { 254,-2322 }, { 255,-2322 }, { 256,-2322 },

 {   0,  55 }, {   0,2674 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   9,-3937 }, {  10,-3937 }, {   0,   0 }, {  12,-3937 }, {  13,-3937 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {  32,-3937 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  39,1900 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {  45,-27020 }, {   0,  55 }, {   0,2627 }, {   1,-2627 },

 {   2,-2627 }, {   3,-2627 }, {   4,-2627 }, {   5,-2627 }, {   6,-2627 },
 {   7,-2627 }, {   8,-2627 }, {   9,-2369 }, {  10,-3984 }, {  11,-2627 },
 {  12,-2369 }, {  13,-3984 }, {  14,-2627 }, {  15,-2627 }, {  16,-2627 },
 {  17,-2627 }, {  18,-2627 }, {  19,-2627 }, {  20,-2627 }, {  21,-2627 },
 {  22,-2627 }, {  23,-2627 }, {  24,-2627 }, {  25,-2627 }, {  26,-2627 },
 {  27,-2627 }, {  28,-2627 }, {  29,-2627 }, {  30,-2627 }, {  31,-2627 },
 {  32,-2369 }, {  33,-2627 }, {  34,-2627 }, {  35,-2627 }, {  36,-2627 },
 {  37,-2627 }, {  38,-2627 }, {  39,1595 }, {  40,-2627 }, {  41,-2627 },
 {  42,-2627 }, {  43,-2627 }, {  44,-2627 }, {  45, 258 }, {  46,-2627 },
 {  47,-2627 }, {  48,-2627 }, {  49,-2627 }, {  50,-2627 }, {  51,-2627 },

 {  52,-2627 }, {  53,-2627 }, {  54,-2627 }, {  55,-2627 }, {  56,-2627 },
 {  57,-2627 }, {  58,-2627 }, {  59,-2627 }, {  60,-2627 }, {  61,-2627 },
 {  62,-2627 }, {  63,-2627 }, {  64,-2627 }, {  65,-2627 }, {  66,-2627 },
 {  67,-2627 }, {  68,-2627 }, {  69,-2627 }, {  70,-2627 }, {  71,-2627 },
 {  72,-2627 }, {  73,-2627 }, {  74,-2627 }, {  75,-2627 }, {  76,-2627 },
 {  77,-2627 }, {  78,-2627 }, {  79,-2627 }, {  80,-2627 }, {  81,-2627 },
 {  82,-2627 }, {  83,-2627 }, {  84,-2627 }, {  85,-2627 }, {  86,-2627 },
 {  87,-2627 }, {  88,-2627 }, {  89,-2627 }, {  90,-2627 }, {  91,-2627 },
 {  92,-2627 }, {  93,-2627 }, {  94,-2627 }, {  95,-2627 }, {  96,-2627 },
 {  97,-2627 }, {  98,-2627 }, {  99,-2627 }, { 100,-2627 }, { 101,-2627 },

 { 102,-2627 }, { 103,-2627 }, { 104,-2627 }, { 105,-2627 }, { 106,-2627 },
 { 107,-2627 }, { 108,-2627 }, { 109,-2627 }, { 110,-2627 }, { 111,-2627 },
 { 112,-2627 }, { 113,-2627 }, { 114,-2627 }, { 115,-2627 }, { 116,-2627 },
 { 117,-2627 }, { 118,-2627 }, { 119,-2627 }, { 120,-2627 }, { 121,-2627 },
 { 122,-2627 }, { 123,-2627 }, { 124,-2627 }, { 125,-2627 }, { 126,-2627 },
 { 127,-2627 }, { 128,-2627 }, { 129,-2627 }, { 130,-2627 }, { 131,-2627 },
 { 132,-2627 }, { 133,-2627 }, { 134,-2627 }, { 135,-2627 }, { 136,-2627 },
 { 137,-2627 }, { 138,-2627 }, { 139,-2627 }, { 140,-2627 }, { 141,-2627 },
 { 142,-2627 }, { 143,-2627 }, { 144,-2627 }, { 145,-2627 }, { 146,-2627 },
 { 147,-2627 }, { 148,-2627 }, { 149,-2627 }, { 150,-2627 }, { 151,-2627 },

 { 152,-2627 }, { 153,-2627 }, { 154,-2627 }, { 155,-2627 }, { 156,-2627 },
 { 157,-2627 }, { 158,-2627 }, { 159,-2627 }, { 160,-2627 }, { 161,-2627 },
 { 162,-2627 }, { 163,-2627 }, { 164,-2627 }, { 165,-2627 }, { 166,-2627 },
 { 167,-2627 }, { 168,-2627 }, { 169,-2627 }, { 170,-2627 }, { 171,-2627 },
 { 172,-2627 }, { 173,-2627 }, { 174,-2627 }, { 175,-2627 }, { 176,-2627 },
 { 177,-2627 }, { 178,-2627 }, { 179,-2627 }, { 180,-2627 }, { 181,-2627 },
 { 182,-2627 }, { 183,-2627 }, { 184,-2627 }, { 185,-2627 }, { 186,-2627 },
 { 187,-2627 }, { 188,-2627 }, { 189,-2627 }, { 190,-2627 }, { 191,-2627 },
 { 192,-2627 }, { 193,-2627 }, { 194,-2627 }, { 195,-2627 }, { 196,-2627 },
 { 197,-2627 }, { 198,-2627 }, { 199,-2627 }, { 200,-2627 }, { 201,-2627 },

 { 202,-2627 }, { 203,-2627 }, { 204,-2627 }, { 205,-2627 }, { 206,-2627 },
 { 207,-2627 }, { 208,-2627 }, { 209,-2627 }, { 210,-2627 }, { 211,-2627 },
 { 212,-2627 }, { 213,-2627 }, { 214,-2627 }, { 215,-2627 }, { 216,-2627 },
 { 217,-2627 }, { 218,-2627 }, { 219,-2627 }, { 220,-2627 }, { 221,-2627 },
 { 222,-2627 }, { 223,-2627 }, { 224,-2627 }, { 225,-2627 }, { 226,-2627 },
 { 227,-2627 }, { 228,-2627 }, { 229,-2627 }, { 230,-2627 }, { 231,-2627 },
 { 232,-2627 }, { 233,-2627 }, { 234,-2627 }, { 235,-2627 }, { 236,-2627 },
 { 237,-2627 }, { 238,-2627 }, { 239,-2627 }, { 240,-2627 }, { 241,-2627 },
 { 242,-2627 }, { 243,-2627 }, { 244,-2627 }, { 245,-2627 }, { 246,-2627 },
 { 247,-2627 }, { 248,-2627 }, { 249,-2627 }, { 250,-2627 }, { 251,-2627 },

 { 252,-2627 }, { 253,-2627 }, { 254,-2627 }, { 255,-2627 }, { 256,-2627 },
 {   0,  55 }, {   0,2369 }, {   1,-2885 }, {   2,-2885 }, {   3,-2885 },
 {   4,-2885 }, {   5,-2885 }, {   6,-2885 }, {   7,-2885 }, {   8,-2885 },
 {   9,-2627 }, {  10,-4242 }, {  11,-2885 }, {  12,-2627 }, {  13,-4242 },
 {  14,-2885 }, {  15,-2885 }, {  16,-2885 }, {  17,-2885 }, {  18,-2885 },
 {  19,-2885 }, {  20,-2885 }, {  21,-2885 }, {  22,-2885 }, {  23,-2885 },
 {  24,-2885 }, {  25,-2885 }, {  26,-2885 }, {  27,-2885 }, {  28,-2885 },
 {  29,-2885 }, {  30,-2885 }, {  31,-2885 }, {  32,-2627 }, {  33,-2885 },
 {  34,-2885 }, {  35,-2885 }, {  36,-2885 }, {  37,-2885 }, {  38,-2885 },
 {  39,-2369 }, {  40,-2885 }, {  41,-2885 }, {  42,-2885 }, {  43,-2885 },

 {  44,-2885 }, {  45,   0 }, {  46,-2885 }, {  47,-2885 }, {  48,-2885 },
 {  49,-2885 }, {  50,-2885 }, {  51,-2885 }, {  52,-2885 }, {  53,-2885 },
 {  54,-2885 }, {  55,-2885 }, {  56,-2885 }, {  57,-2885 }, {  58,-2885 },
 {  59,-2885 }, {  60,-2885 }, {  61,-2885 }, {  62,-2885 }, {  63,-2885 },
 {  64,-2885 }, {  65,-2885 }, {  66,-2885 }, {  67,-2885 }, {  68,-2885 },
 {  69,-2885 }, {  70,-2885 }, {  71,-2885 }, {  72,-2885 }, {  73,-2885 },
 {  74,-2885 }, {  75,-2885 }, {  76,-2885 }, {  77,-2885 }, {  78,-2885 },
 {  79,-2885 }, {  80,-2885 }, {  81,-2885 }, {  82,-2885 }, {  83,-2885 },
 {  84,-2885 }, {  85,-2885 }, {  86,-2885 }, {  87,-2885 }, {  88,-2885 },
 {  89,-2885 }, {  90,-2885 }, {  91,-2885 }, {  92,-2885 }, {  93,-2885 },

 {  94,-2885 }, {  95,-2885 }, {  96,-2885 }, {  97,-2885 }, {  98,-2885 },
 {  99,-2885 }, { 100,-2885 }, { 101,-2885 }, { 102,-2885 }, { 103,-2885 },
 { 104,-2885 }, { 105,-2885 }, { 106,-2885 }, { 107,-2885 }, { 108,-2885 },
 { 109,-2885 }, { 110,-2885 }, { 111,-2885 }, { 112,-2885 }, { 113,-2885 },
 { 114,-2885 }, { 115,-2885 }, { 116,-2885 }, { 117,-2885 }, { 118,-2885 },
 { 119,-2885 }, { 120,-2885 }, { 121,-2885 }, { 122,-2885 }, { 123,-2885 },
 { 124,-2885 }, { 125,-2885 }, { 126,-2885 }, { 127,-2885 }, { 128,-2885 },
 { 129,-2885 }, { 130,-2885 }, { 131,-2885 }, { 132,-2885 }, { 133,-2885 },
 { 134,-2885 }, { 135,-2885 }, { 136,-2885 }, { 137,-2885 }, { 138,-2885 },
 { 139,-2885 }, { 140,-2885 }, { 141,-2885 }, { 142,-2885 }, { 143,-2885 },

 { 144,-2885 }, { 145,-2885 }, { 146,-2885 }, { 147,-2885 }, { 148,-2885 },
 { 149,-2885 }, { 150,-2885 }, { 151,-2885 }, { 152,-2885 }, { 153,-2885 },
 { 154,-2885 }, { 155,-2885 }, { 156,-2885 }, { 157,-2885 }, { 158,-2885 },
 { 159,-2885 }, { 160,-2885 }, { 161,-2885 }, { 162,-2885 }, { 163,-2885 },
 { 164,-2885 }, { 165,-2885 }, { 166,-2885 }, { 167,-2885 }, { 168,-2885 },
 { 169,-2885 }, { 170,-2885 }, { 171,-2885 }, { 172,-2885 }, { 173,-2885 },
 { 174,-2885 }, { 175,-2885 }, { 176,-2885 }, { 177,-2885 }, { 178,-2885 },
 { 179,-2885 }, { 180,-2885 }, { 181,-2885 }, { 182,-2885 }, { 183,-2885 },
 { 184,-2885 }, { 185,-2885 }, { 186,-2885 }, { 187,-2885 }, { 188,-2885 },
 { 189,-2885 }, { 190,-2885 }, { 191,-2885 }, { 192,-2885 }, { 193,-2885 },

 { 194,-2885 }, { 195,-2885 }, { 196,-2885 }, { 197,-2885 }, { 198,-2885 },
 { 199,-2885 }, { 200,-2885 }, { 201,-2885 }, { 202,-2885 }, { 203,-2885 },
 { 204,-2885 }, { 205,-2885 }, { 206,-2885 }, { 207,-2885 }, { 208,-2885 },
 { 209,-2885 }, { 210,-2885 }, { 211,-2885 }, { 212,-2885 }, { 213,-2885 },
 { 214,-2885 }, { 215,-2885 }, { 216,-2885 }, { 217,-2885 }, { 218,-2885 },
 { 219,-2885 }, { 220,-2885 }, { 221,-2885 }, { 222,-2885 }, { 223,-2885 },
 { 224,-2885 }, { 225,-2885 }, { 226,-2885 }, { 227,-2885 }, { 228,-2885 },
 { 229,-2885 }, { 230,-2885 }, { 231,-2885 }, { 232,-2885 }, { 233,-2885 },
 { 234,-2885 }, { 235,-2885 }, { 236,-2885 }, { 237,-2885 }, { 238,-2885 },
 { 239,-2885 }, { 240,-2885 }, { 241,-2885 }, { 242,-2885 }, { 243,-2885 },

 { 244,-2885 }, { 245,-2885 }, { 246,-2885 }, { 247,-2885 }, { 248,-2885 },
 { 249,-2885 }, { 250,-2885 }, { 251,-2885 }, { 252,-2885 }, { 253,-2885 },
 { 254,-2885 }, { 255,-2885 }, { 256,-2885 }, {   0,  28 }, {   0,2111 },
 {   1,-2111 }, {   2,-2111 }, {   3,-2111 }, {   4,-2111 }, {   5,-2111 },
 {   6,-2111 }, {   7,-2111 }, {   8,-2111 }, {   9,-1853 }, {  10,-4195 },
 {  11,-2111 }, {  12,-1853 }, {  13,-4195 }, {  14,-2111 }, {  15,-2111 },
 {  16,-2111 }, {  17,-2111 }, {  18,-2111 }, {  19,-2111 }, {  20,-2111 },
 {  21,-2111 }, {  22,-2111 }, {  23,-2111 }, {  24,-2111 }, {  25,-2111 },
 {  26,-2111 }, {  27,-2111 }, {  28,-2111 }, {  29,-2111 }, {  30,-2111 },
 {  31,-2111 }, {  32,-1853 }, {  33,-2111 }, {  34,-2111 }, {  35,-2111 },

 {  36,-2111 }, {  37,-2111 }, {  38,-2111 }, {  39,1595 }, {  40,-2111 },
 {  41,-2111 }, {  42,-2111 }, {  43,-2111 }, {  44,-2111 }, {  45,-1337 },
 {  46,-2111 }, {  47,-2111 }, {  48,-2111 }, {  49,-2111 }, {  50,-2111 },
 {  51,-2111 }, {  52,-2111 }, {  53,-2111 }, {  54,-2111 }, {  55,-2111 },
 {  56,-2111 }, {  57,-2111 }, {  58,-2111 }, {  59,-2111 }, {  60,-2111 },
 {  61,-2111 }, {  62,-2111 }, {  63,-2111 }, {  64,-2111 }, {  65,-2111 },
 {  66,-2111 }, {  67,-2111 }, {  68,-2111 }, {  69,-2111 }, {  70,-2111 },
 {  71,-2111 }, {  72,-2111 }, {  73,-2111 }, {  74,-2111 }, {  75,-2111 },
 {  76,-2111 }, {  77,-2111 }, {  78,-2111 }, {  79,-2111 }, {  80,-2111 },
 {  81,-2111 }, {  82,-2111 }, {  83,-2111 }, {  84,-2111 }, {  85,-2111 },

 {  86,-2111 }, {  87,-2111 }, {  88,-2111 }, {  89,-2111 }, {  90,-2111 },
 {  91,-2111 }, {  92,-2111 }, {  93,-2111 }, {  94,-2111 }, {  95,-2111 },
 {  96,-2111 }, {  97,-2111 }, {  98,-2111 }, {  99,-2111 }, { 100,-2111 },
 { 101,-2111 }, { 102,-2111 }, { 103,-2111 }, { 104,-2111 }, { 105,-2111 },
 { 106,-2111 }, { 107,-2111 }, { 108,-2111 }, { 109,-2111 }, { 110,-2111 },
 { 111,-2111 }, { 112,-2111 }, { 113,-2111 }, { 114,-2111 }, { 115,-2111 },
 { 116,-2111 }, { 117,-2111 }, { 118,-2111 }, { 119,-2111 }, { 120,-2111 },
 { 121,-2111 }, { 122,-2111 }, { 123,-2111 }, { 124,-2111 }, { 125,-2111 },
 { 126,-2111 }, { 127,-2111 }, { 128,-2111 }, { 129,-2111 }, { 130,-2111 },
 { 131,-2111 }, { 132,-2111 }, { 133,-2111 }, { 134,-2111 }, { 135,-2111 },

 { 136,-2111 }, { 137,-2111 }, { 138,-2111 }, { 139,-2111 }, { 140,-2111 },
 { 141,-2111 }, { 142,-2111 }, { 143,-2111 }, { 144,-2111 }, { 145,-2111 },
 { 146,-2111 }, { 147,-2111 }, { 148,-2111 }, { 149,-2111 }, { 150,-2111 },
 { 151,-2111 }, { 152,-2111 }, { 153,-2111 }, { 154,-2111 }, { 155,-2111 },
 { 156,-2111 }, { 157,-2111 }, { 158,-2111 }, { 159,-2111 }, { 160,-2111 },
 { 161,-2111 }, { 162,-2111 }, { 163,-2111 }, { 164,-2111 }, { 165,-2111 },
 { 166,-2111 }, { 167,-2111 }, { 168,-2111 }, { 169,-2111 }, { 170,-2111 },
 { 171,-2111 }, { 172,-2111 }, { 173,-2111 }, { 174,-2111 }, { 175,-2111 },
 { 176,-2111 }, { 177,-2111 }, { 178,-2111 }, { 179,-2111 }, { 180,-2111 },
 { 181,-2111 }, { 182,-2111 }, { 183,-2111 }, { 184,-2111 }, { 185,-2111 },

 { 186,-2111 }, { 187,-2111 }, { 188,-2111 }, { 189,-2111 }, { 190,-2111 },
 { 191,-2111 }, { 192,-2111 }, { 193,-2111 }, { 194,-2111 }, { 195,-2111 },
 { 196,-2111 }, { 197,-2111 }, { 198,-2111 }, { 199,-2111 }, { 200,-2111 },
 { 201,-2111 }, { 202,-2111 }, { 203,-2111 }, { 204,-2111 }, { 205,-2111 },
 { 206,-2111 }, { 207,-2111 }, { 208,-2111 }, { 209,-2111 }, { 210,-2111 },
 { 211,-2111 }, { 212,-2111 }, { 213,-2111 }, { 214,-2111 }, { 215,-2111 },
 { 216,-2111 }, { 217,-2111 }, { 218,-2111 }, { 219,-2111 }, { 220,-2111 },
 { 221,-2111 }, { 222,-2111 }, { 223,-2111 }, { 224,-2111 }, { 225,-2111 },
 { 226,-2111 }, { 227,-2111 }, { 228,-2111 }, { 229,-2111 }, { 230,-2111 },
 { 231,-2111 }, { 232,-2111 }, { 233,-2111 }, { 234,-2111 }, { 235,-2111 },

 { 236,-2111 }, { 237,-2111 }, { 238,-2111 }, { 239,-2111 }, { 240,-2111 },
 { 241,-2111 }, { 242,-2111 }, { 243,-2111 }, { 244,-2111 }, { 245,-2111 },
 { 246,-2111 }, { 247,-2111 }, { 248,-2111 }, { 249,-2111 }, { 250,-2111 },
 { 251,-2111 }, { 252,-2111 }, { 253,-2111 }, { 254,-2111 }, { 255,-2111 },
 { 256,-2111 }, {   0,  28 }, {   0,1853 }, {   1,-2369 }, {   2,-2369 },
 {   3,-2369 }, {   4,-2369 }, {   5,-2369 }, {   6,-2369 }, {   7,-2369 },
 {   8,-2369 }, {   9,-2111 }, {  10,-4453 }, {  11,-2369 }, {  12,-2111 },
 {  13,-4453 }, {  14,-2369 }, {  15,-2369 }, {  16,-2369 }, {  17,-2369 },
 {  18,-2369 }, {  19,-2369 }, {  20,-2369 }, {  21,-2369 }, {  22,-2369 },
 {  23,-2369 }, {  24,-2369 }, {  25,-2369 }, {  26,-2369 }, {  27,-2369 },

 {  28,-2369 }, {  29,-2369 }, {  30,-2369 }, {  31,-2369 }, {  32,-2111 },
 {  33,-2369 }, {  34,-2369 }, {  35,-2369 }, {  36,-2369 }, {  37,-2369 },
 {  38,-2369 }, {  39,1337 }, {  40,-2369 }, {  41,-2369 }, {  42,-2369 },
 {  43,-2369 }, {  44,-2369 }, {  45,-1595 }, {  46,-2369 }, {  47,-2369 },
 {  48,-2369 }, {  49,-2369 }, {  50,-2369 }, {  51,-2369 }, {  52,-2369 },
 {  53,-2369 }, {  54,-2369 }, {  55,-2369 }, {  56,-2369 }, {  57,-2369 },
 {  58,-2369 }, {  59,-2369 }, {  60,-2369 }, {  61,-2369 }, {  62,-2369 },
 {  63,-2369 }, {  64,-2369 }, {  65,-2369 }, {  66,-2369 }, {  67,-2369 },
 {  68,-2369 }, {  69,-2369 }, {  70,-2369 }, {  71,-2369 }, {  72,-2369 },
 {  73,-2369 }, {  74,-2369 }, {  75,-2369 }, {  76,-2369 }, {  77,-2369 },

 {  78,-2369 }, {  79,-2369 }, {  80,-2369 }, {  81,-2369 }, {  82,-2369 },
 {  83,-2369 }, {  84,-2369 }, {  85,-2369 }, {  86,-2369 }, {  87,-2369 },
 {  88,-2369 }, {  89,-2369 }, {  90,-2369 }, {  91,-2369 }, {  92,-2369 },
 {  93,-2369 }, {  94,-2369 }, {  95,-2369 }, {  96,-2369 }, {  97,-2369 },
 {  98,-2369 }, {  99,-2369 }, { 100,-2369 }, { 101,-2369 }, { 102,-2369 },
 { 103,-2369 }, { 104,-2369 }, { 105,-2369 }, { 106,-2369 }, { 107,-2369 },
 { 108,-2369 }, { 109,-2369 }, { 110,-2369 }, { 111,-2369 }, { 112,-2369 },
 { 113,-2369 }, { 114,-2369 }, { 115,-2369 }, { 116,-2369 }, { 117,-2369 },
 { 118,-2369 }, { 119,-2369 }, { 120,-2369 }, { 121,-2369 }, { 122,-2369 },
 { 123,-2369 }, { 124,-2369 }, { 125,-2369 }, { 126,-2369 }, { 127,-2369 },

 { 128,-2369 }, { 129,-2369 }, { 130,-2369 }, { 131,-2369 }, { 132,-2369 },
 { 133,-2369 }, { 134,-2369 }, { 135,-2369 }, { 136,-2369 }, { 137,-2369 },
 { 138,-2369 }, { 139,-2369 }, { 140,-2369 }, { 141,-2369 }, { 142,-2369 },
 { 143,-2369 }, { 144,-2369 }, { 145,-2369 }, { 146,-2369 }, { 147,-2369 },
 { 148,-2369 }, { 149,-2369 }, { 150,-2369 }, { 151,-2369 }, { 152,-2369 },
 { 153,-2369 }, { 154,-2369 }, { 155,-2369 }, { 156,-2369 }, { 157,-2369 },
 { 158,-2369 }, { 159,-2369 }, { 160,-2369 }, { 161,-2369 }, { 162,-2369 },
 { 163,-2369 }, { 164,-2369 }, { 165,-2369 }, { 166,-2369 }, { 167,-2369 },
 { 168,-2369 }, { 169,-2369 }, { 170,-2369 }, { 171,-2369 }, { 172,-2369 },
 { 173,-2369 }, { 174,-2369 }, { 175,-2369 }, { 176,-2369 }, { 177,-2369 },

 { 178,-2369 }, { 179,-2369 }, { 180,-2369 }, { 181,-2369 }, { 182,-2369 },
 { 183,-2369 }, { 184,-2369 }, { 185,-2369 }, { 186,-2369 }, { 187,-2369 },
 { 188,-2369 }, { 189,-2369 }, { 190,-2369 }, { 191,-2369 }, { 192,-2369 },
 { 193,-2369 }, { 194,-2369 }, { 195,-2369 }, { 196,-2369 }, { 197,-2369 },
 { 198,-2369 }, { 199,-2369 }, { 200,-2369 }, { 201,-2369 }, { 202,-2369 },
 { 203,-2369 }, { 204,-2369 }, { 205,-2369 }, { 206,-2369 }, { 207,-2369 },
 { 208,-2369 }, { 209,-2369 }, { 210,-2369 }, { 211,-2369 }, { 212,-2369 },
 { 213,-2369 }, { 214,-2369 }, { 215,-2369 }, { 216,-2369 }, { 217,-2369 },
 { 218,-2369 }, { 219,-2369 }, { 220,-2369 }, { 221,-2369 }, { 222,-2369 },
 { 223,-2369 }, { 224,-2369 }, { 225,-2369 }, { 226,-2369 }, { 227,-2369 },

 { 228,-2369 }, { 229,-2369 }, { 230,-2369 }, { 231,-2369 }, { 232,-2369 },
 { 233,-2369 }, { 234,-2369 }, { 235,-2369 }, { 236,-2369 }, { 237,-2369 },
 { 238,-2369 }, { 239,-2369 }, { 240,-2369 }, { 241,-2369 }, { 242,-2369 },
 { 243,-2369 }, { 244,-2369 }, { 245,-2369 }, { 246,-2369 }, { 247,-2369 },
 { 248,-2369 }, { 249,-2369 }, { 250,-2369 }, { 251,-2369 }, { 252,-2369 },
 { 253,-2369 }, { 254,-2369 }, { 255,-2369 }, { 256,-2369 }, {   0,  28 },
 {   0,1595 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   9,-4711 },
 {  10,-4711 }, {   0,   0 }, {  12,-4711 }, {  13,-4711 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },

 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {  32,-4711 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  39,1337 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {  45,-28073 }, {   0,  28 }, {   0,1548 }, {   1,-2674 }, {   2,-2674 },
 {   3,-2674 }, {   4,-2674 }, {   5,-2674 }, {   6,-2674 }, {   7,-2674 },
 {   8,-2674 }, {   9,-2416 }, {  10,-4758 }, {  11,-2674 }, {  12,-2416 },
 {  13,-4758 }, {  14,-2674 }, {  15,-2674 }, {  16,-2674 }, {  17,-2674 },
 {  18,-2674 }, {  19,-2674 }, {  20,-2674 }, {  21,-2674 }, {  22,-2674 },

 {  23,-2674 }, {  24,-2674 }, {  25,-2674 }, {  26,-2674 }, {  27,-2674 },
 {  28,-2674 }, {  29,-2674 }, {  30,-2674 }, {  31,-2674 }, {  32,-2416 },
 {  33,-2674 }, {  34,-2674 }, {  35,-2674 }, {  36,-2674 }, {  37,-2674 },
 {  38,-2674 }, {  39,1032 }, {  40,-2674 }, {  41,-2674 }, {  42,-2674 },
 {  43,-2674 }, {  44,-2674 }, {  45, 258 }, {  46,-2674 }, {  47,-2674 },
 {  48,-2674 }, {  49,-2674 }, {  50,-2674 }, {  51,-2674 }, {  52,-2674 },
 {  53,-2674 }, {  54,-2674 }, {  55,-2674 }, {  56,-2674 }, {  57,-2674 },
 {  58,-2674 }, {  59,-2674 }, {  60,-2674 }, {  61,-2674 }, {  62,-2674 },
 {  63,-2674 }, {  64,-2674 }, {  65,-2674 }, {  66,-2674 }, {  67,-2674 },
 {  68,-2674 }, {  69,-2674 }, {  70,-2674 }, {  71,-2674 }, {  72,-2674 },

 {  73,-2674 }, {  74,-2674 }, {  75,-2674 }, {  76,-2674 }, {  77,-2674 },
 {  78,-2674 }, {  79,-2674 }, {  80,-2674 }, {  81,-2674 }, {  82,-2674 },
 {  83,-2674 }, {  84,-2674 }, {  85,-2674 }, {  86,-2674 }, {  87,-2674 },
 {  88,-2674 }, {  89,-2674 }, {  90,-2674 }, {  91,-2674 }, {  92,-2674 },
 {  93,-2674 }, {  94,-2674 }, {  95,-2674 }, {  96,-2674 }, {  97,-2674 },
 {  98,-2674 }, {  99,-2674 }, { 100,-2674 }, { 101,-2674 }, { 102,-2674 },
 { 103,-2674 }, { 104,-2674 }, { 105,-2674 }, { 106,-2674 }, { 107,-2674 },
 { 108,-2674 }, { 109,-2674 }, { 110,-2674 }, { 111,-2674 }, { 112,-2674 },
 { 113,-2674 }, { 114,-2674 }, { 115,-2674 }, { 116,-2674 }, { 117,-2674 },
 { 118,-2674 }, { 119,-2674 }, { 120,-2674 }, { 121,-2674 }, { 122,-2674 },

 { 123,-2674 }, { 124,-2674 }, { 125,-2674 }, { 126,-2674 }, { 127,-2674 },
 { 128,-2674 }, { 129,-2674 }, { 130,-2674 }, { 131,-2674 }, { 132,-2674 },
 { 133,-2674 }, { 134,-2674 }, { 135,-2674 }, { 136,-2674 }, { 137,-2674 },
 { 138,-2674 }, { 139,-2674 }, { 140,-2674 }, { 141,-2674 }, { 142,-2674 },
 { 143,-2674 }, { 144,-2674 }, { 145,-2674 }, { 146,-2674 }, { 147,-2674 },
 { 148,-2674 }, { 149,-2674 }, { 150,-2674 }, { 151,-2674 }, { 152,-2674 },
 { 153,-2674 }, { 154,-2674 }, { 155,-2674 }, { 156,-2674 }, { 157,-2674 },
 { 158,-2674 }, { 159,-2674 }, { 160,-2674 }, { 161,-2674 }, { 162,-2674 },
 { 163,-2674 }, { 164,-2674 }, { 165,-2674 }, { 166,-2674 }, { 167,-2674 },
 { 168,-2674 }, { 169,-2674 }, { 170,-2674 }, { 171,-2674 }, { 172,-2674 },

 { 173,-2674 }, { 174,-2674 }, { 175,-2674 }, { 176,-2674 }, { 177,-2674 },
 { 178,-2674 }, { 179,-2674 }, { 180,-2674 }, { 181,-2674 }, { 182,-2674 },
 { 183,-2674 }, { 184,-2674 }, { 185,-2674 }, { 186,-2674 }, { 187,-2674 },
 { 188,-2674 }, { 189,-2674 }, { 190,-2674 }, { 191,-2674 }, { 192,-2674 },
 { 193,-2674 }, { 194,-2674 }, { 195,-2674 }, { 196,-2674 }, { 197,-2674 },
 { 198,-2674 }, { 199,-2674 }, { 200,-2674 }, { 201,-2674 }, { 202,-2674 },
 { 203,-2674 }, { 204,-2674 }, { 205,-2674 }, { 206,-2674 }, { 207,-2674 },
 { 208,-2674 }, { 209,-2674 }, { 210,-2674 }, { 211,-2674 }, { 212,-2674 },
 { 213,-2674 }, { 214,-2674 }, { 215,-2674 }, { 216,-2674 }, { 217,-2674 },
 { 218,-2674 }, { 219,-2674 }, { 220,-2674 }, { 221,-2674 }, { 222,-2674 },

 { 223,-2674 }, { 224,-2674 }, { 225,-2674 }, { 226,-2674 }, { 227,-2674 },
 { 228,-2674 }, { 229,-2674 }, { 230,-2674 }, { 231,-2674 }, { 232,-2674 },
 { 233,-2674 }, { 234,-2674 }, { 235,-2674 }, { 236,-2674 }, { 237,-2674 },
 { 238,-2674 }, { 239,-2674 }, { 240,-2674 }, { 241,-2674 }, { 242,-2674 },
 { 243,-2674 }, { 244,-2674 }, { 245,-2674 }, { 246,-2674 }, { 247,-2674 },
 { 248,-2674 }, { 249,-2674 }, { 250,-2674 }, { 251,-2674 }, { 252,-2674 },
 { 253,-2674 }, { 254,-2674 }, { 255,-2674 }, { 256,-2674 }, {   0,  28 },
 {   0,1290 }, {   1,-2932 }, {   2,-2932 }, {   3,-2932 }, {   4,-2932 },
 {   5,-2932 }, {   6,-2932 }, {   7,-2932 }, {   8,-2932 }, {   9,-2674 },
 {  10,-5016 }, {  11,-2932 }, {  12,-2674 }, {  13,-5016 }, {  14,-2932 },

 {  15,-2932 }, {  16,-2932 }, {  17,-2932 }, {  18,-2932 }, {  19,-2932 },
 {  20,-2932 }, {  21,-2932 }, {  22,-2932 }, {  23,-2932 }, {  24,-2932 },
 {  25,-2932 }, {  26,-2932 }, {  27,-2932 }, {  28,-2932 }, {  29,-2932 },
 {  30,-2932 }, {  31,-2932 }, {  32,-2674 }, {  33,-2932 }, {  34,-2932 },
 {  35,-2932 }, {  36,-2932 }, {  37,-2932 }, {  38,-2932 }, {  39,-2416 },
 {  40,-2932 }, {  41,-2932 }, {  42,-2932 }, {  43,-2932 }, {  44,-2932 },
 {  45,   0 }, {  46,-2932 }, {  47,-2932 }, {  48,-2932 }, {  49,-2932 },
 {  50,-2932 }, {  51,-2932 }, {  52,-2932 }, {  53,-2932 }, {  54,-2932 },
 {  55,-2932 }, {  56,-2932 }, {  57,-2932 }, {  58,-2932 }, {  59,-2932 },
 {  60,-2932 }, {  61,-2932 }, {  62,-2932 }, {  63,-2932 }, {  64,-2932 },

 {  65,-2932 }, {  66,-2932 }, {  67,-2932 }, {  68,-2932 }, {  69,-2932 },
 {  70,-2932 }, {  71,-2932 }, {  72,-2932 }, {  73,-2932 }, {  74,-2932 },
 {  75,-2932 }, {  76,-2932 }, {  77,-2932 }, {  78,-2932 }, {  79,-2932 },
 {  80,-2932 }, {  81,-2932 }, {  82,-2932 }, {  83,-2932 }, {  84,-2932 },
 {  85,-2932 }, {  86,-2932 }, {  87,-2932 }, {  88,-2932 }, {  89,-2932 },
 {  90,-2932 }, {  91,-2932 }, {  92,-2932 }, {  93,-2932 }, {  94,-2932 },
 {  95,-2932 }, {  96,-2932 }, {  97,-2932 }, {  98,-2932 }, {  99,-2932 },
 { 100,-2932 }, { 101,-2932 }, { 102,-2932 }, { 103,-2932 }, { 104,-2932 },
 { 105,-2932 }, { 106,-2932 }, { 107,-2932 }, { 108,-2932 }, { 109,-2932 },
 { 110,-2932 }, { 111,-2932 }, { 112,-2932 }, { 113,-2932 }, { 114,-2932 },

 { 115,-2932 }, { 116,-2932 }, { 117,-2932 }, { 118,-2932 }, { 119,-2932 },
 { 120,-2932 }, { 121,-2932 }, { 122,-2932 }, { 123,-2932 }, { 124,-2932 },
 { 125,-2932 }, { 126,-2932 }, { 127,-2932 }, { 128,-2932 }, { 129,-2932 },
 { 130,-2932 }, { 131,-2932 }, { 132,-2932 }, { 133,-2932 }, { 134,-2932 },
 { 135,-2932 }, { 136,-2932 }, { 137,-2932 }, { 138,-2932 }, { 139,-2932 },
 { 140,-2932 }, { 141,-2932 }, { 142,-2932 }, { 143,-2932 }, { 144,-2932 },
 { 145,-2932 }, { 146,-2932 }, { 147,-2932 }, { 148,-2932 }, { 149,-2932 },
 { 150,-2932 }, { 151,-2932 }, { 152,-2932 }, { 153,-2932 }, { 154,-2932 },
 { 155,-2932 }, { 156,-2932 }, { 157,-2932 }, { 158,-2932 }, { 159,-2932 },
 { 160,-2932 }, { 161,-2932 }, { 162,-2932 }, { 163,-2932 }, { 164,-2932 },

 { 165,-2932 }, { 166,-2932 }, { 167,-2932 }, { 168,-2932 }, { 169,-2932 },
 { 170,-2932 }, { 171,-2932 }, { 172,-2932 }, { 173,-2932 }, { 174,-2932 },
 { 175,-2932 }, { 176,-2932 }, { 177,-2932 }, { 178,-2932 }, { 179,-2932 },
 { 180,-2932 }, { 181,-2932 }, { 182,-2932 }, { 183,-2932 }, { 184,-2932 },
 { 185,-2932 }, { 186,-2932 }, { 187,-2932 }, { 188,-2932 }, { 189,-2932 },
 { 190,-2932 }, { 191,-2932 }, { 192,-2932 }, { 193,-2932 }, { 194,-2932 },
 { 195,-2932 }, { 196,-2932 }, { 197,-2932 }, { 198,-2932 }, { 199,-2932 },
 { 200,-2932 }, { 201,-2932 }, { 202,-2932 }, { 203,-2932 }, { 204,-2932 },
 { 205,-2932 }, { 206,-2932 }, { 207,-2932 }, { 208,-2932 }, { 209,-2932 },
 { 210,-2932 }, { 211,-2932 }, { 212,-2932 }, { 213,-2932 }, { 214,-2932 },

 { 215,-2932 }, { 216,-2932 }, { 217,-2932 }, { 218,-2932 }, { 219,-2932 },
 { 220,-2932 }, { 221,-2932 }, { 222,-2932 }, { 223,-2932 }, { 224,-2932 },
 { 225,-2932 }, { 226,-2932 }, { 227,-2932 }, { 228,-2932 }, { 229,-2932 },
 { 230,-2932 }, { 231,-2932 }, { 232,-2932 }, { 233,-2932 }, { 234,-2932 },
 { 235,-2932 }, { 236,-2932 }, { 237,-2932 }, { 238,-2932 }, { 239,-2932 },
 { 240,-2932 }, { 241,-2932 }, { 242,-2932 }, { 243,-2932 }, { 244,-2932 },
 { 245,-2932 }, { 246,-2932 }, { 247,-2932 }, { 248,-2932 }, { 249,-2932 },
 { 250,-2932 }, { 251,-2932 }, { 252,-2932 }, { 253,-2932 }, { 254,-2932 },
 { 255,-2932 }, { 256,-2932 }, {   0,  55 }, {   0,1032 }, {   1,-2158 },
 {   2,-2158 }, {   3,-2158 }, {   4,-2158 }, {   5,-2158 }, {   6,-2158 },

 {   7,-2158 }, {   8,-2158 }, {   9,-1900 }, {  10,-1642 }, {  11,-2158 },
 {  12,-1900 }, {  13,-1642 }, {  14,-2158 }, {  15,-2158 }, {  16,-2158 },
 {  17,-2158 }, {  18,-2158 }, {  19,-2158 }, {  20,-2158 }, {  21,-2158 },
 {  22,-2158 }, {  23,-2158 }, {  24,-2158 }, {  25,-2158 }, {  26,-2158 },
 {  27,-2158 }, {  28,-2158 }, {  29,-2158 }, {  30,-2158 }, {  31,-2158 },
 {  32,-1900 }, {  33,-2158 }, {  34,-2158 }, {  35,-2158 }, {  36,-2158 },
 {  37,-2158 }, {  38,-2158 }, {  39,-3706 }, {  40,-2158 }, {  41,-2158 },
 {  42,-2158 }, {  43,-2158 }, {  44,-2158 }, {  45,-1595 }, {  46,-2158 },
 {  47,-2158 }, {  48,-2158 }, {  49,-2158 }, {  50,-2158 }, {  51,-2158 },
 {  52,-2158 }, {  53,-2158 }, {  54,-2158 }, {  55,-2158 }, {  56,-2158 },

 {  57,-2158 }, {  58,-2158 }, {  59,-2158 }, {  60,-2158 }, {  61,-2158 },
 {  62,-2158 }, {  63,-2158 }, {  64,-2158 }, {  65,-2158 }, {  66,-2158 },
 {  67,-2158 }, {  68,-2158 }, {  69,-2158 }, {  70,-2158 }, {  71,-2158 },
 {  72,-2158 }, {  73,-2158 }, {  74,-2158 }, {  75,-2158 }, {  76,-2158 },
 {  77,-2158 }, {  78,-2158 }, {  79,-2158 }, {  80,-2158 }, {  81,-2158 },
 {  82,-2158 }, {  83,-2158 }, {  84,-2158 }, {  85,-2158 }, {  86,-2158 },
 {  87,-2158 }, {  88,-2158 }, {  89,-2158 }, {  90,-2158 }, {  91,-2158 },
 {  92,-2158 }, {  93,-2158 }, {  94,-2158 }, {  95,-2158 }, {  96,-2158 },
 {  97,-2158 }, {  98,-2158 }, {  99,-2158 }, { 100,-2158 }, { 101,-2158 },
 { 102,-2158 }, { 103,-2158 }, { 104,-2158 }, { 105,-2158 }, { 106,-2158 },

 { 107,-2158 }, { 108,-2158 }, { 109,-2158 }, { 110,-2158 }, { 111,-2158 },
 { 112,-2158 }, { 113,-2158 }, { 114,-2158 }, { 115,-2158 }, { 116,-2158 },
 { 117,-2158 }, { 118,-2158 }, { 119,-2158 }, { 120,-2158 }, { 121,-2158 },
 { 122,-2158 }, { 123,-2158 }, { 124,-2158 }, { 125,-2158 }, { 126,-2158 },
 { 127,-2158 }, { 128,-2158 }, { 129,-2158 }, { 130,-2158 }, { 131,-2158 },
 { 132,-2158 }, { 133,-2158 }, { 134,-2158 }, { 135,-2158 }, { 136,-2158 },
 { 137,-2158 }, { 138,-2158 }, { 139,-2158 }, { 140,-2158 }, { 141,-2158 },
 { 142,-2158 }, { 143,-2158 }, { 144,-2158 }, { 145,-2158 }, { 146,-2158 },
 { 147,-2158 }, { 148,-2158 }, { 149,-2158 }, { 150,-2158 }, { 151,-2158 },
 { 152,-2158 }, { 153,-2158 }, { 154,-2158 }, { 155,-2158 }, { 156,-2158 },

 { 157,-2158 }, { 158,-2158 }, { 159,-2158 }, { 160,-2158 }, { 161,-2158 },
 { 162,-2158 }, { 163,-2158 }, { 164,-2158 }, { 165,-2158 }, { 166,-2158 },
 { 167,-2158 }, { 168,-2158 }, { 169,-2158 }, { 170,-2158 }, { 171,-2158 },
 { 172,-2158 }, { 173,-2158 }, { 174,-2158 }, { 175,-2158 }, { 176,-2158 },
 { 177,-2158 }, { 178,-2158 }, { 179,-2158 }, { 180,-2158 }, { 181,-2158 },
 { 182,-2158 }, { 183,-2158 }, { 184,-2158 }, { 185,-2158 }, { 186,-2158 },
 { 187,-2158 }, { 188,-2158 }, { 189,-2158 }, { 190,-2158 }, { 191,-2158 },
 { 192,-2158 }, { 193,-2158 }, { 194,-2158 }, { 195,-2158 }, { 196,-2158 },
 { 197,-2158 }, { 198,-2158 }, { 199,-2158 }, { 200,-2158 }, { 201,-2158 },
 { 202,-2158 }, { 203,-2158 }, { 204,-2158 }, { 205,-2158 }, { 206,-2158 },

 { 207,-2158 }, { 208,-2158 }, { 209,-2158 }, { 210,-2158 }, { 211,-2158 },
 { 212,-2158 }, { 213,-2158 }, { 214,-2158 }, { 215,-2158 }, { 216,-2158 },
 { 217,-2158 }, { 218,-2158 }, { 219,-2158 }, { 220,-2158 }, { 221,-2158 },
 { 222,-2158 }, { 223,-2158 }, { 224,-2158 }, { 225,-2158 }, { 226,-2158 },
 { 227,-2158 }, { 228,-2158 }, { 229,-2158 }, { 230,-2158 }, { 231,-2158 },
 { 232,-2158 }, { 233,-2158 }, { 234,-2158 }, { 235,-2158 }, { 236,-2158 },
 { 237,-2158 }, { 238,-2158 }, { 239,-2158 }, { 240,-2158 }, { 241,-2158 },
 { 242,-2158 }, { 243,-2158 }, { 244,-2158 }, { 245,-2158 }, { 246,-2158 },
 { 247,-2158 }, { 248,-2158 }, { 249,-2158 }, { 250,-2158 }, { 251,-2158 },
 { 252,-2158 }, { 253,-2158 }, { 254,-2158 }, { 255,-2158 }, { 256,-2158 },

 {   0,  55 }, {   0, 774 }, {   1,-28881 }, {   2,-28881 }, {   3,-28881 },
 {   4,-28881 }, {   5,-28881 }, {   6,-28881 }, {   7,-28881 }, {   8,-28881 },
 {   9,-28881 }, {  10,-28881 }, {  11,-28881 }, {  12,-28881 }, {  13,-28881 },
 {  14,-28881 }, {  15,-28881 }, {  16,-28881 }, {  17,-28881 }, {  18,-28881 },
 {  19,-28881 }, {  20,-28881 }, {  21,-28881 }, {  22,-28881 }, {  23,-28881 },
 {  24,-28881 }, {  25,-28881 }, {  26,-28881 }, {  27,-28881 }, {  28,-28881 },
 {  29,-28881 }, {  30,-28881 }, {  31,-28881 }, {  32,-28881 }, {  33,-28881 },
 {  34,-28881 }, {  35,-28881 }, {  36,-28881 }, {  37,-28881 }, {  38,-28881 },
 {   0,   0 }, {  40,-28881 }, {  41,-28881 }, {  42,-28881 }, {  43,-28881 },
 {  44,-28881 }, {  45,-28881 }, {  46,-28881 }, {  47,-28881 }, {  48,-28881 },

 {  49,-28881 }, {  50,-28881 }, {  51,-28881 }, {  52,-28881 }, {  53,-28881 },
 {  54,-28881 }, {  55,-28881 }, {  56,-28881 }, {  57,-28881 }, {  58,-28881 },
 {  59,-28881 }, {  60,-28881 }, {  61,-28881 }, {  62,-28881 }, {  63,-28881 },
 {  64,-28881 }, {  65,-28881 }, {  66,-28881 }, {  67,-28881 }, {  68,-28881 },
 {  69,-28881 }, {  70,-28881 }, {  71,-28881 }, {  72,-28881 }, {  73,-28881 },
 {  74,-28881 }, {  75,-28881 }, {  76,-28881 }, {  77,-28881 }, {  78,-28881 },
 {  79,-28881 }, {  80,-28881 }, {  81,-28881 }, {  82,-28881 }, {  83,-28881 },
 {  84,-28881 }, {  85,-28881 }, {  86,-28881 }, {  87,-28881 }, {  88,-28881 },
 {  89,-28881 }, {  90,-28881 }, {  91,-28881 }, {  92,-28881 }, {  93,-28881 },
 {  94,-28881 }, {  95,-28881 }, {  96,-28881 }, {  97,-28881 }, {  98,-28881 },

 {  99,-28881 }, { 100,-28881 }, { 101,-28881 }, { 102,-28881 }, { 103,-28881 },
 { 104,-28881 }, { 105,-28881 }, { 106,-28881 }, { 107,-28881 }, { 108,-28881 },
 { 109,-28881 }, { 110,-28881 }, { 111,-28881 }, { 112,-28881 }, { 113,-28881 },
 { 114,-28881 }, { 115,-28881 }, { 116,-28881 }, { 117,-28881 }, { 118,-28881 },
 { 119,-28881 }, { 120,-28881 }, { 121,-28881 }, { 122,-28881 }, { 123,-28881 },
 { 124,-28881 }, { 125,-28881 }, { 126,-28881 }, { 127,-28881 }, { 128,-28881 },
 { 129,-28881 }, { 130,-28881 }, { 131,-28881 }, { 132,-28881 }, { 133,-28881 },
 { 134,-28881 }, { 135,-28881 }, { 136,-28881 }, { 137,-28881 }, { 138,-28881 },
 { 139,-28881 }, { 140,-28881 }, { 141,-28881 }, { 142,-28881 }, { 143,-28881 },
 { 144,-28881 }, { 145,-28881 }, { 146,-28881 }, { 147,-28881 }, { 148,-28881 },

 { 149,-28881 }, { 150,-28881 }, { 151,-28881 }, { 152,-28881 }, { 153,-28881 },
 { 154,-28881 }, { 155,-28881 }, { 156,-28881 }, { 157,-28881 }, { 158,-28881 },
 { 159,-28881 }, { 160,-28881 }, { 161,-28881 }, { 162,-28881 }, { 163,-28881 },
 { 164,-28881 }, { 165,-28881 }, { 166,-28881 }, { 167,-28881 }, { 168,-28881 },
 { 169,-28881 }, { 170,-28881 }, { 171,-28881 }, { 172,-28881 }, { 173,-28881 },
 { 174,-28881 }, { 175,-28881 }, { 176,-28881 }, { 177,-28881 }, { 178,-28881 },
 { 179,-28881 }, { 180,-28881 }, { 181,-28881 }, { 182,-28881 }, { 183,-28881 },
 { 184,-28881 }, { 185,-28881 }, { 186,-28881 }, { 187,-28881 }, { 188,-28881 },
 { 189,-28881 }, { 190,-28881 }, { 191,-28881 }, { 192,-28881 }, { 193,-28881 },
 { 194,-28881 }, { 195,-28881 }, { 196,-28881 }, { 197,-28881 }, { 198,-28881 },

 { 199,-28881 }, { 200,-28881 }, { 201,-28881 }, { 202,-28881 }, { 203,-28881 },
 { 204,-28881 }, { 205,-28881 }, { 206,-28881 }, { 207,-28881 }, { 208,-28881 },
 { 209,-28881 }, { 210,-28881 }, { 211,-28881 }, { 212,-28881 }, { 213,-28881 },
 { 214,-28881 }, { 215,-28881 }, { 216,-28881 }, { 217,-28881 }, { 218,-28881 },
 { 219,-28881 }, { 220,-28881 }, { 221,-28881 }, { 222,-28881 }, { 223,-28881 },
 { 224,-28881 }, { 225,-28881 }, { 226,-28881 }, { 227,-28881 }, { 228,-28881 },
 { 229,-28881 }, { 230,-28881 }, { 231,-28881 }, { 232,-28881 }, { 233,-28881 },
 { 234,-28881 }, { 235,-28881 }, { 236,-28881 }, { 237,-28881 }, { 238,-28881 },
 { 239,-28881 }, { 240,-28881 }, { 241,-28881 }, { 242,-28881 }, { 243,-28881 },
 { 244,-28881 }, { 245,-28881 }, { 246,-28881 }, { 247,-28881 }, { 248,-28881 },

 { 249,-28881 }, { 250,-28881 }, { 251,-28881 }, { 252,-28881 }, { 253,-28881 },
 { 254,-28881 }, { 255,-28881 }, { 256,-28881 }, {   0,  28 }, {   0, 516 },
 {   1,-1595 }, {   2,-1595 }, {   3,-1595 }, {   4,-1595 }, {   5,-1595 },
 {   6,-1595 }, {   7,-1595 }, {   8,-1595 }, {   9,-1337 }, {  10,-1079 },
 {  11,-1595 }, {  12,-1337 }, {  13,-1079 }, {  14,-1595 }, {  15,-1595 },
 {  16,-1595 }, {  17,-1595 }, {  18,-1595 }, {  19,-1595 }, {  20,-1595 },
 {  21,-1595 }, {  22,-1595 }, {  23,-1595 }, {  24,-1595 }, {  25,-1595 },
 {  26,-1595 }, {  27,-1595 }, {  28,-1595 }, {  29,-1595 }, {  30,-1595 },
 {  31,-1595 }, {  32,-1337 }, {  33,-1595 }, {  34,-1595 }, {  35,-1595 },
 {  36,-1595 }, {  37,-1595 }, {  38,-1595 }, {  39,-3190 }, {  40,-1595 },

 {  41,-1595 }, {  42,-1595 }, {  43,-1595 }, {  44,-1595 }, {  45,-1032 },
 {  46,-1595 }, {  47,-1595 }, {  48,-1595 }, {  49,-1595 }, {  50,-1595 },
 {  51,-1595 }, {  52,-1595 }, {  53,-1595 }, {  54,-1595 }, {  55,-1595 },
 {  56,-1595 }, {  57,-1595 }, {  58,-1595 }, {  59,-1595 }, {  60,-1595 },
 {  61,-1595 }, {  62,-1595 }, {  63,-1595 }, {  64,-1595 }, {  65,-1595 },
 {  66,-1595 }, {  67,-1595 }, {  68,-1595 }, {  69,-1595 }, {  70,-1595 },
 {  71,-1595 }, {  72,-1595 }, {  73,-1595 }, {  74,-1595 }, {  75,-1595 },
 {  76,-1595 }, {  77,-1595 }, {  78,-1595 }, {  79,-1595 }, {  80,-1595 },
 {  81,-1595 }, {  82,-1595 }, {  83,-1595 }, {  84,-1595 }, {  85,-1595 },
 {  86,-1595 }, {  87,-1595 }, {  88,-1595 }, {  89,-1595 }, {  90,-1595 },

 {  91,-1595 }, {  92,-1595 }, {  93,-1595 }, {  94,-1595 }, {  95,-1595 },
 {  96,-1595 }, {  97,-1595 }, {  98,-1595 }, {  99,-1595 }, { 100,-1595 },
 { 101,-1595 }, { 102,-1595 }, { 103,-1595 }, { 104,-1595 }, { 105,-1595 },
 { 106,-1595 }, { 107,-1595 }, { 108,-1595 }, { 109,-1595 }, { 110,-1595 },
 { 111,-1595 }, { 112,-1595 }, { 113,-1595 }, { 114,-1595 }, { 115,-1595 },
 { 116,-1595 }, { 117,-1595 }, { 118,-1595 }, { 119,-1595 }, { 120,-1595 },
 { 121,-1595 }, { 122,-1595 }, { 123,-1595 }, { 124,-1595 }, { 125,-1595 },
 { 126,-1595 }, { 127,-1595 }, { 128,-1595 }, { 129,-1595 }, { 130,-1595 },
 { 131,-1595 }, { 132,-1595 }, { 133,-1595 }, { 134,-1595 }, { 135,-1595 },
 { 136,-1595 }, { 137,-1595 }, { 138,-1595 }, { 139,-1595 }, { 140,-1595 },

 { 141,-1595 }, { 142,-1595 }, { 143,-1595 }, { 144,-1595 }, { 145,-1595 },
 { 146,-1595 }, { 147,-1595 }, { 148,-1595 }, { 149,-1595 }, { 150,-1595 },
 { 151,-1595 }, { 152,-1595 }, { 153,-1595 }, { 154,-1595 }, { 155,-1595 },
 { 156,-1595 }, { 157,-1595 }, { 158,-1595 }, { 159,-1595 }, { 160,-1595 },
 { 161,-1595 }, { 162,-1595 }, { 163,-1595 }, { 164,-1595 }, { 165,-1595 },
 { 166,-1595 }, { 167,-1595 }, { 168,-1595 }, { 169,-1595 }, { 170,-1595 },
 { 171,-1595 }, { 172,-1595 }, { 173,-1595 }, { 174,-1595 }, { 175,-1595 },
 { 176,-1595 }, { 177,-1595 }, { 178,-1595 }, { 179,-1595 }, { 180,-1595 },
 { 181,-1595 }, { 182,-1595 }, { 183,-1595 }, { 184,-1595 }, { 185,-1595 },
 { 186,-1595 }, { 187,-1595 }, { 188,-1595 }, { 189,-1595 }, { 190,-1595 },

 { 191,-1595 }, { 192,-1595 }, { 193,-1595 }, { 194,-1595 }, { 195,-1595 },
 { 196,-1595 }, { 197,-1595 }, { 198,-1595 }, { 199,-1595 }, { 200,-1595 },
 { 201,-1595 }, { 202,-1595 }, { 203,-1595 }, { 204,-1595 }, { 205,-1595 },
 { 206,-1595 }, { 207,-1595 }, { 208,-1595 }, { 209,-1595 }, { 210,-1595 },
 { 211,-1595 }, { 212,-1595 }, { 213,-1595 }, { 214,-1595 }, { 215,-1595 },
 { 216,-1595 }, { 217,-1595 }, { 218,-1595 }, { 219,-1595 }, { 220,-1595 },
 { 221,-1595 }, { 222,-1595 }, { 223,-1595 }, { 224,-1595 }, { 225,-1595 },
 { 226,-1595 }, { 227,-1595 }, { 228,-1595 }, { 229,-1595 }, { 230,-1595 },
 { 231,-1595 }, { 232,-1595 }, { 233,-1595 }, { 234,-1595 }, { 235,-1595 },
 { 236,-1595 }, { 237,-1595 }, { 238,-1595 }, { 239,-1595 }, { 240,-1595 },

 { 241,-1595 }, { 242,-1595 }, { 243,-1595 }, { 244,-1595 }, { 245,-1595 },
 { 246,-1595 }, { 247,-1595 }, { 248,-1595 }, { 249,-1595 }, { 250,-1595 },
 { 251,-1595 }, { 252,-1595 }, { 253,-1595 }, { 254,-1595 }, { 255,-1595 },
 { 256,-1595 }, {   0,  28 }, {   0, 258 }, {   1,-29389 }, {   2,-29389 },
 {   3,-29389 }, {   4,-29389 }, {   5,-29389 }, {   6,-29389 }, {   7,-29389 },
 {   8,-29389 }, {   9,-29389 }, {  10,-29389 }, {  11,-29389 }, {  12,-29389 },
 {  13,-29389 }, {  14,-29389 }, {  15,-29389 }, {  16,-29389 }, {  17,-29389 },
 {  18,-29389 }, {  19,-29389 }, {  20,-29389 }, {  21,-29389 }, {  22,-29389 },
 {  23,-29389 }, {  24,-29389 }, {  25,-29389 }, {  26,-29389 }, {  27,-29389 },
 {  28,-29389 }, {  29,-29389 }, {  30,-29389 }, {  31,-29389 }, {  32,-29389 },

 {  33,-29389 }, {  34,-29389 }, {  35,-29389 }, {  36,-29389 }, {  37,-29389 },
 {  38,-29389 }, {   0,   0 }, {  40,-29389 }, {  41,-29389 }, {  42,-29389 },
 {  43,-29389 }, {  44,-29389 }, {  45,-29389 }, {  46,-29389 }, {  47,-29389 },
 {  48,-29389 }, {  49,-29389 }, {  50,-29389 }, {  51,-29389 }, {  52,-29389 },
 {  53,-29389 }, {  54,-29389 }, {  55,-29389 }, {  56,-29389 }, {  57,-29389 },
 {  58,-29389 }, {  59,-29389 }, {  60,-29389 }, {  61,-29389 }, {  62,-29389 },
 {  63,-29389 }, {  64,-29389 }, {  65,-29389 }, {  66,-29389 }, {  67,-29389 },
 {  68,-29389 }, {  69,-29389 }, {  70,-29389 }, {  71,-29389 }, {  72,-29389 },
 {  73,-29389 }, {  74,-29389 }, {  75,-29389 }, {  76,-29389 }, {  77,-29389 },
 {  78,-29389 }, {  79,-29389 }, {  80,-29389 }, {  81,-29389 }, {  82,-29389 },

 {  83,-29389 }, {  84,-29389 }, {  85,-29389 }, {  86,-29389 }, {  87,-29389 },
 {  88,-29389 }, {  89,-29389 }, {  90,-29389 }, {  91,-29389 }, {  92,-29389 },
 {  93,-29389 }, {  94,-29389 }, {  95,-29389 }, {  96,-29389 }, {  97,-29389 },
 {  98,-29389 }, {  99,-29389 }, { 100,-29389 }, { 101,-29389 }, { 102,-29389 },
 { 103,-29389 }, { 104,-29389 }, { 105,-29389 }, { 106,-29389 }, { 107,-29389 },
 { 108,-29389 }, { 109,-29389 }, { 110,-29389 }, { 111,-29389 }, { 112,-29389 },
 { 113,-29389 }, { 114,-29389 }, { 115,-29389 }, { 116,-29389 }, { 117,-29389 },
 { 118,-29389 }, { 119,-29389 }, { 120,-29389 }, { 121,-29389 }, { 122,-29389 },
 { 123,-29389 }, { 124,-29389 }, { 125,-29389 }, { 126,-29389 }, { 127,-29389 },
 { 128,-29389 }, { 129,-29389 }, { 130,-29389 }, { 131,-29389 }, { 132,-29389 },

 { 133,-29389 }, { 134,-29389 }, { 135,-29389 }, { 136,-29389 }, { 137,-29389 },
 { 138,-29389 }, { 139,-29389 }, { 140,-29389 }, { 141,-29389 }, { 142,-29389 },
 { 143,-29389 }, { 144,-29389 }, { 145,-29389 }, { 146,-29389 }, { 147,-29389 },
 { 148,-29389 }, { 149,-29389 }, { 150,-29389 }, { 151,-29389 }, { 152,-29389 },
 { 153,-29389 }, { 154,-29389 }, { 155,-29389 }, { 156,-29389 }, { 157,-29389 },
 { 158,-29389 }, { 159,-29389 }, { 160,-29389 }, { 161,-29389 }, { 162,-29389 },
 { 163,-29389 }, { 164,-29389 }, { 165,-29389 }, { 166,-29389 }, { 167,-29389 },
 { 168,-29389 }, { 169,-29389 }, { 170,-29389 }, { 171,-29389 }, { 172,-29389 },
 { 173,-29389 }, { 174,-29389 }, { 175,-29389 }, { 176,-29389 }, { 177,-29389 },
 { 178,-29389 }, { 179,-29389 }, { 180,-29389 }, { 181,-29389 }, { 182,-29389 },

 { 183,-29389 }, { 184,-29389 }, { 185,-29389 }, { 186,-29389 }, { 187,-29389 },
 { 188,-29389 }, { 189,-29389 }, { 190,-29389 }, { 191,-29389 }, { 192,-29389 },
 { 193,-29389 }, { 194,-29389 }, { 195,-29389 }, { 196,-29389 }, { 197,-29389 },
 { 198,-29389 }, { 199,-29389 }, { 200,-29389 }, { 201,-29389 }, { 202,-29389 },
 { 203,-29389 }, { 204,-29389 }, { 205,-29389 }, { 206,-29389 }, { 207,-29389 },
 { 208,-29389 }, { 209,-29389 }, { 210,-29389 }, { 211,-29389 }, { 212,-29389 },
 { 213,-29389 }, { 214,-29389 }, { 215,-29389 }, { 216,-29389 }, { 217,-29389 },
 { 218,-29389 }, { 219,-29389 }, { 220,-29389 }, { 221,-29389 }, { 222,-29389 },
 { 223,-29389 }, { 224,-29389 }, { 225,-29389 }, { 226,-29389 }, { 227,-29389 },
 { 228,-29389 }, { 229,-29389 }, { 230,-29389 }, { 231,-29389 }, { 232,-29389 },

 { 233,-29389 }, { 234,-29389 }, { 235,-29389 }, { 236,-29389 }, { 237,-29389 },
 { 238,-29389 }, { 239,-29389 }, { 240,-29389 }, { 241,-29389 }, { 242,-29389 },
 { 243,-29389 }, { 244,-29389 }, { 245,-29389 }, { 246,-29389 }, { 247,-29389 },
 { 248,-29389 }, { 249,-29389 }, { 250,-29389 }, { 251,-29389 }, { 252,-29389 },
 { 253,-29389 }, { 254,-29389 }, { 255,-29389 }, { 256,-29389 }, { 257,  81 },
 {   1,   0 },    };

static yyconst struct yy_trans_info *yy_start_state_list[27] =
    {
    &yy_transition[1],
    &yy_transition[3],
    &yy_transition[261],
    &yy_transition[519],
    &yy_transition[777],
    &yy_transition[1035],
    &yy_transition[1293],
    &yy_transition[1551],
    &yy_transition[1809],
    &yy_transition[2067],
    &yy_transition[2325],
    &yy_transition[2583],
    &yy_transition[2841],
    &yy_transition[3099],
    &yy_transition[3357],
    &yy_transition[3615],
    &yy_transition[3873],
    &yy_transition[4131],
    &yy_transition[4389],
    &yy_transition[4647],
    &yy_transition[4905],
    &yy_transition[5163],
    &yy_transition[5421],
    &yy_transition[5679],
    &yy_transition[5937],
    &yy_transition[6195],
    &yy_transition[6453],

    } ;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
#line 1 "scan.l"

#line 48 "scan.l"
/* Avoid exit() on fatal scanner errors (a bit ugly -- see yy_fatal_error) */
#undef fprintf
#define fprintf(file, fmt, msg)  fprintf_to_ereport(fmt, msg)

static void
fprintf_to_ereport(const char *fmt, const char *msg)
{
	ereport(ERROR, (errmsg_internal("%s", msg)));
}

/*
 * GUC variables.  This is a DIRECT violation of the warning given at the
 * head of gram.y, ie flex/bison code must not depend on any GUC variables;
 * as such, changing their values can induce very unintuitive behavior.
 * But we shall have to live with it until we can remove these variables.
 */
int			backslash_quote = BACKSLASH_QUOTE_SAFE_ENCODING;
bool		escape_string_warning = true;
bool		standard_conforming_strings = true;

/*
 * Set the type of YYSTYPE.
 */
#define YYSTYPE core_YYSTYPE

/*
 * Set the type of yyextra.  All state variables used by the scanner should
 * be in yyextra, *not* statically allocated.
 */
#define YY_EXTRA_TYPE core_yy_extra_type *

/*
 * Each call to core_yylex must set yylloc to the location of the found token
 * (expressed as a byte offset from the start of the input text).
 * When we parse a token that requires multiple lexer rules to process,
 * this should be done in the first such rule, else yylloc will point
 * into the middle of the token.
 */
#define SET_YYLLOC()  (*(yylloc) = yytext - yyextra->scanbuf)

/*
 * Advance yylloc by the given number of bytes.
 */
#define ADVANCE_YYLLOC(delta)  ( *(yylloc) += (delta) )

#define startlit()	( yyextra->literallen = 0 )
static void addlit(char *ytext, int yleng, core_yyscan_t yyscanner);
static void addlitchar(unsigned char ychar, core_yyscan_t yyscanner);
static char *litbufdup(core_yyscan_t yyscanner);
static char *litbuf_udeescape(unsigned char escape, core_yyscan_t yyscanner);
static unsigned char unescape_single_char(unsigned char c, core_yyscan_t yyscanner);
static int	process_integer_literal(const char *token, YYSTYPE *lval);
static bool is_utf16_surrogate_first(pg_wchar c);
static bool is_utf16_surrogate_second(pg_wchar c);
static pg_wchar surrogate_pair_to_codepoint(pg_wchar first, pg_wchar second);
static void addunicode(pg_wchar c, yyscan_t yyscanner);
static bool check_uescapechar(unsigned char escape);

#define yyerror(msg)  scanner_yyerror(msg, yyscanner)

#define lexer_errposition()  scanner_errposition(*(yylloc), yyscanner)

static void check_string_escape_warning(unsigned char ychar, core_yyscan_t yyscanner);
static void check_escape_warning(core_yyscan_t yyscanner);

/*
 * Work around a bug in flex 2.5.35: it emits a couple of functions that
 * it forgets to emit declarations for.  Since we use -Wmissing-prototypes,
 * this would cause warnings.  Providing our own declarations should be
 * harmless even when the bug gets fixed.
 */
extern int	core_yyget_column(yyscan_t yyscanner);
extern void core_yyset_column(int column_no, yyscan_t yyscanner);

#define YY_NO_INPUT 1
/*
 * OK, here is a short description of lex/flex rules behavior.
 * The longest pattern which matches an input string is always chosen.
 * For equal-length patterns, the first occurring in the rules list is chosen.
 * INITIAL is the starting state, to which all non-conditional rules apply.
 * Exclusive states change parsing rules while the state is active.  When in
 * an exclusive state, only those rules defined for that state apply.
 *
 * We use exclusive states for quoted strings, extended comments,
 * and to eliminate parsing troubles for numeric strings.
 * Exclusive states:
 *  <xb> bit string literal
 *  <xc> extended C-style comments
 *  <xd> delimited identifiers (double-quoted identifiers)
 *  <xh> hexadecimal numeric string
 *  <xq> standard quoted strings
 *  <xe> extended quoted strings (support backslash escape sequences)
 *  <xdolq> $foo$ quoted strings
 *  <xui> quoted identifier with Unicode escapes
 *  <xuiend> end of a quoted identifier with Unicode escapes, UESCAPE can follow
 *  <xus> quoted string with Unicode escapes
 *  <xusend> end of a quoted string with Unicode escapes, UESCAPE can follow
 *  <xeu> Unicode surrogate pair in extended quoted string
 *
 * Remember to add an <<EOF>> case whenever you add a new exclusive state!
 * The default one is probably not the right thing.
 */












/*
 * In order to make the world safe for Windows and Mac clients as well as
 * Unix ones, we accept either \n or \r as a newline.  A DOS-style \r\n
 * sequence will be seen as two successive newlines, but that doesn't cause
 * any problems.  Comments that start with -- and extend to the next
 * newline are treated as equivalent to a single whitespace character.
 *
 * NOTE a fine point: if there is no newline following --, we will absorb
 * everything to the end of the input as a comment.  This is correct.  Older
 * versions of Postgres failed to recognize -- as a comment if the input
 * did not end with a newline.
 *
 * XXX perhaps \f (formfeed) should be treated as a newline as well?
 *
 * XXX if you change the set of whitespace characters, fix scanner_isspace()
 * to agree, and see also the plpgsql lexer.
 */
/*
 * SQL requires at least one newline in the whitespace separating
 * string literals that are to be concatenated.  Silly, but who are we
 * to argue?  Note that {whitespace_with_newline} should not have * after
 * it, whereas {whitespace} should generally have a * after it...
 */
/*
 * To ensure that {quotecontinue} can be scanned without having to back up
 * if the full pattern isn't matched, we include trailing whitespace in
 * {quotestop}.  This matches all cases where {quotecontinue} fails to match,
 * except for {quote} followed by whitespace and just one "-" (not two,
 * which would start a {comment}).  To cover that we have {quotefail}.
 * The actions for {quotestop} and {quotefail} must throw back characters
 * beyond the quote proper.
 */
/* Bit string
 * It is tempting to scan the string for only those characters
 * which are allowed. However, this leads to silently swallowed
 * characters if illegal characters are included in the string.
 * For example, if xbinside is [01] then B'ABCD' is interpreted
 * as a zero-length string, and the ABCD' is lost!
 * Better to pass the string forward and let the input routines
 * validate the contents.
 */
/* Hexadecimal number */
/* National character */
/* Quoted string that allows backslash escapes */
/* Extended quote
 * xqdouble implements embedded quote, ''''
 */
/* $foo$ style quotes ("dollar quoting")
 * The quoted string starts with $foo$ where "foo" is an optional string
 * in the form of an identifier, except that it may not contain "$",
 * and extends to the first occurrence of an identical string.
 * There is *no* processing of the quoted text.
 *
 * {dolqfailed} is an error rule to avoid scanner backup when {dolqdelim}
 * fails to match its trailing "$".
 */
/* Double quote
 * Allows embedded spaces and other special characters into identifiers.
 */
/* Unicode escapes */
/* error rule to avoid backup */
/* Quoted identifier with Unicode escapes */
/* Quoted string with Unicode escapes */
/* Optional UESCAPE after a quoted string or identifier with Unicode escapes. */
/* error rule to avoid backup */
/* C-style comments
 *
 * The "extended comment" syntax closely resembles allowable operator syntax.
 * The tricky part here is to get lex to recognize a string starting with
 * slash-star as a comment, when interpreting it as an operator would produce
 * a longer match --- remember lex will prefer a longer match!  Also, if we
 * have something like plus-slash-star, lex will think this is a 3-character
 * operator whereas we want to see it as a + operator and a comment start.
 * The solution is two-fold:
 * 1. append {op_chars}* to xcstart so that it matches as much text as
 *    {operator} would. Then the tie-breaker (first matching rule of same
 *    length) ensures xcstart wins.  We put back the extra stuff with yyless()
 *    in case it contains a star-slash that should terminate the comment.
 * 2. In the operator rule, check for slash-star within the operator, and
 *    if found throw it back with yyless().  This handles the plus-slash-star
 *    problem.
 * Dash-dash comments have similar interactions with the operator rule.
 */
/* Assorted special-case operators and operator-like tokens */
/*
 * "self" is the set of chars that should be returned as single-character
 * tokens.  "op_chars" is the set of chars that can make up "Op" tokens,
 * which can be one or more characters long (but if a single-char token
 * appears in the "self" set, it is not to be returned as an Op).  Note
 * that the sets overlap, but each has some chars that are not in the other.
 *
 * If you change either set, adjust the character lists appearing in the
 * rule for "operator"!
 */
/* we no longer allow unary minus in numbers.
 * instead we pass it separately to parser. there it gets
 * coerced via doNegate() -- Leon aug 20 1999
 *
 * {decimalfail} is used because we would like "1..10" to lex as 1, dot_dot, 10.
 *
 * {realfail1} and {realfail2} are added to prevent the need for scanner
 * backup when the {real} rule fails to match completely.
 */
/*
 * Dollar quoted strings are totally opaque, and no escaping is done on them.
 * Other quoted strings must allow some special characters such as single-quote
 *  and newline.
 * Embedded single-quotes are implemented both in the SQL standard
 *  style of two adjacent single quotes "''" and in the Postgres/Java style
 *  of escaped-quote "\'".
 * Other embedded escaped characters are matched explicitly and the leading
 *  backslash is dropped from the string.
 * Note that xcstart must appear before operator, as explained above!
 *  Also whitespace (comment) must appear before operator.
 */
#line 8832 "scan.c"

#define INITIAL 0
#define xb 1
#define xc 2
#define xd 3
#define xh 4
#define xe 5
#define xq 6
#define xdolq 7
#define xui 8
#define xuiend 9
#define xus 10
#define xusend 11
#define xeu 12

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

/* Holds the entire state of the reentrant scanner. */
struct yyguts_t
    {

    /* User-defined. Not touched by flex. */
    YY_EXTRA_TYPE yyextra_r;

    /* The rest are the same as the globals declared in the non-reentrant scanner. */
    FILE *yyin_r, *yyout_r;
    size_t yy_buffer_stack_top; /**< index of top of stack. */
    size_t yy_buffer_stack_max; /**< capacity of stack. */
    YY_BUFFER_STATE * yy_buffer_stack; /**< Stack as an array. */
    char yy_hold_char;
    int yy_n_chars;
    int yyleng_r;
    char *yy_c_buf_p;
    int yy_init;
    int yy_start;
    int yy_did_buffer_switch_on_eof;
    int yy_start_stack_ptr;
    int yy_start_stack_depth;
    int *yy_start_stack;
    yy_state_type yy_last_accepting_state;
    char* yy_last_accepting_cpos;

    int yylineno_r;
    int yy_flex_debug_r;

    char *yytext_r;
    int yy_more_flag;
    int yy_more_len;

    YYSTYPE * yylval_r;

    YYLTYPE * yylloc_r;

    }; /* end struct yyguts_t */

static int yy_init_globals (yyscan_t yyscanner );

    /* This must go here because YYSTYPE and YYLTYPE are included
     * from bison output in section 1.*/
    #    define yylval yyg->yylval_r
    
    #    define yylloc yyg->yylloc_r
    
int core_yylex_init (yyscan_t* scanner);

int core_yylex_init_extra (YY_EXTRA_TYPE user_defined,yyscan_t* scanner);

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int core_yylex_destroy (yyscan_t yyscanner );

int core_yyget_debug (yyscan_t yyscanner );

void core_yyset_debug (int debug_flag ,yyscan_t yyscanner );

YY_EXTRA_TYPE core_yyget_extra (yyscan_t yyscanner );

void core_yyset_extra (YY_EXTRA_TYPE user_defined ,yyscan_t yyscanner );

FILE *core_yyget_in (yyscan_t yyscanner );

void core_yyset_in  (FILE * _in_str ,yyscan_t yyscanner );

FILE *core_yyget_out (yyscan_t yyscanner );

void core_yyset_out  (FILE * _out_str ,yyscan_t yyscanner );

			int core_yyget_leng (yyscan_t yyscanner );

char *core_yyget_text (yyscan_t yyscanner );

int core_yyget_lineno (yyscan_t yyscanner );

void core_yyset_lineno (int _line_number ,yyscan_t yyscanner );

int core_yyget_column  (yyscan_t yyscanner );

void core_yyset_column (int _column_no ,yyscan_t yyscanner );

YYSTYPE * core_yyget_lval (yyscan_t yyscanner );

void core_yyset_lval (YYSTYPE * yylval_param ,yyscan_t yyscanner );

       YYLTYPE *core_yyget_lloc (yyscan_t yyscanner );
    
        void core_yyset_lloc (YYLTYPE * yylloc_param ,yyscan_t yyscanner );
    
/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int core_yywrap (yyscan_t yyscanner );
#else
extern int core_yywrap (yyscan_t yyscanner );
#endif
#endif

#ifndef YY_NO_UNPUT
    
#endif

#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int ,yyscan_t yyscanner);
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * ,yyscan_t yyscanner);
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (yyscan_t yyscanner );
#else
static int input (yyscan_t yyscanner );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( yytext, (size_t) yyleng, 1, yyout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		int n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = (int) fread(buf, 1, (yy_size_t) max_size, yyin)) == 0 && ferror(yyin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(yyin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg , yyscanner)
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int core_yylex \
               (YYSTYPE * yylval_param,YYLTYPE * yylloc_param ,yyscan_t yyscanner);

#define YY_DECL int core_yylex \
               (YYSTYPE * yylval_param, YYLTYPE * yylloc_param , yyscan_t yyscanner)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK /*LINTED*/break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	yy_state_type yy_current_state;
	char *yy_cp, *yy_bp;
	int yy_act;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    yylval = yylval_param;

    yylloc = yylloc_param;

	if ( !yyg->yy_init )
		{
		yyg->yy_init = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! yyg->yy_start )
			yyg->yy_start = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			core_yyensure_buffer_stack (yyscanner);
			YY_CURRENT_BUFFER_LVALUE =
				core_yy_create_buffer(yyin,YY_BUF_SIZE ,yyscanner);
		}

		core_yy_load_buffer_state(yyscanner );
		}

	{
#line 400 "scan.l"


#line 9130 "scan.c"

	while ( /*CONSTCOND*/1 )		/* loops until end-of-file is reached */
		{
		yy_cp = yyg->yy_c_buf_p;

		/* Support of yytext. */
		*yy_cp = yyg->yy_hold_char;

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = yy_start_state_list[yyg->yy_start];
yy_match:
		{
		yyconst struct yy_trans_info *yy_trans_info;

		YY_CHAR yy_c;

		for ( yy_c = YY_SC_TO_UI(*yy_cp);
		      (yy_trans_info = &yy_current_state[(unsigned int) yy_c])->
		yy_verify == yy_c;
		      yy_c = YY_SC_TO_UI(*++yy_cp) )
			yy_current_state += yy_trans_info->yy_nxt;
		}

yy_find_action:
		yy_act = yy_current_state[-1].yy_nxt;

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
case 1:
/* rule 1 can match eol */
YY_RULE_SETUP
#line 402 "scan.l"
{
					/* ignore */
				}
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 406 "scan.l"
{
					/* Set location in case of syntax error in comment */
					SET_YYLLOC();
					yyextra->xcdepth = 0;
					BEGIN(xc);
					/* Put back any characters past slash-star; see above */
					yyless(2);
				}
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 415 "scan.l"
{
					(yyextra->xcdepth)++;
					/* Put back any characters past slash-star; see above */
					yyless(2);
				}
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 421 "scan.l"
{
					if (yyextra->xcdepth <= 0)
						BEGIN(INITIAL);
					else
						(yyextra->xcdepth)--;
				}
	YY_BREAK
case 5:
/* rule 5 can match eol */
YY_RULE_SETUP
#line 428 "scan.l"
{
					/* ignore */
				}
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 432 "scan.l"
{
					/* ignore */
				}
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 436 "scan.l"
{
					/* ignore */
				}
	YY_BREAK
case YY_STATE_EOF(xc):
#line 440 "scan.l"
{ yyerror("unterminated /* comment"); }
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 442 "scan.l"
{
					/* Binary bit type.
					 * At some point we should simply pass the string
					 * forward to the parser and label it there.
					 * In the meantime, place a leading "b" on the string
					 * to mark it for the input routine as a binary string.
					 */
					SET_YYLLOC();
					BEGIN(xb);
					startlit();
					addlitchar('b', yyscanner);
				}
	YY_BREAK
case 9:
/* rule 9 can match eol */
#line 455 "scan.l"
case 10:
/* rule 10 can match eol */
YY_RULE_SETUP
#line 455 "scan.l"
{
					yyless(1);
					BEGIN(INITIAL);
					yylval->str = litbufdup(yyscanner);
					return BCONST;
				}
	YY_BREAK
case 11:
/* rule 11 can match eol */
#line 462 "scan.l"
case 12:
/* rule 12 can match eol */
YY_RULE_SETUP
#line 462 "scan.l"
{
					addlit(yytext, yyleng, yyscanner);
				}
	YY_BREAK
case 13:
/* rule 13 can match eol */
#line 466 "scan.l"
case 14:
/* rule 14 can match eol */
YY_RULE_SETUP
#line 466 "scan.l"
{
					/* ignore */
				}
	YY_BREAK
case YY_STATE_EOF(xb):
#line 469 "scan.l"
{ yyerror("unterminated bit string literal"); }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 471 "scan.l"
{
					/* Hexadecimal bit type.
					 * At some point we should simply pass the string
					 * forward to the parser and label it there.
					 * In the meantime, place a leading "x" on the string
					 * to mark it for the input routine as a hex string.
					 */
					SET_YYLLOC();
					BEGIN(xh);
					startlit();
					addlitchar('x', yyscanner);
				}
	YY_BREAK
case 16:
/* rule 16 can match eol */
#line 484 "scan.l"
case 17:
/* rule 17 can match eol */
YY_RULE_SETUP
#line 484 "scan.l"
{
					yyless(1);
					BEGIN(INITIAL);
					yylval->str = litbufdup(yyscanner);
					return XCONST;
				}
	YY_BREAK
case YY_STATE_EOF(xh):
#line 490 "scan.l"
{ yyerror("unterminated hexadecimal string literal"); }
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 492 "scan.l"
{
					/* National character.
					 * We will pass this along as a normal character string,
					 * but preceded with an internally-generated "NCHAR".
					 */
					const ScanKeyword *keyword;

					SET_YYLLOC();
					yyless(1);	/* eat only 'n' this time */

					keyword = ScanKeywordLookup("nchar",
												yyextra->keywords,
												yyextra->num_keywords);
					if (keyword != NULL)
					{
						yylval->keyword = keyword->name;
						return keyword->value;
					}
					else
					{
						/* If NCHAR isn't a keyword, just return "n" */
						yylval->str = pstrdup("n");
						return IDENT;
					}
				}
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 518 "scan.l"
{
					yyextra->warn_on_first_escape = true;
					yyextra->saw_non_ascii = false;
					SET_YYLLOC();
					if (yyextra->standard_conforming_strings)
						BEGIN(xq);
					else
						BEGIN(xe);
					startlit();
				}
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 528 "scan.l"
{
					yyextra->warn_on_first_escape = false;
					yyextra->saw_non_ascii = false;
					SET_YYLLOC();
					BEGIN(xe);
					startlit();
				}
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 535 "scan.l"
{
					SET_YYLLOC();
					if (!yyextra->standard_conforming_strings)
						ereport(ERROR,
								(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
								 errmsg("unsafe use of string constant with Unicode escapes"),
								 errdetail("String constants with Unicode escapes cannot be used when standard_conforming_strings is off."),
								 lexer_errposition()));
					BEGIN(xus);
					startlit();
				}
	YY_BREAK
case 22:
/* rule 22 can match eol */
#line 547 "scan.l"
case 23:
/* rule 23 can match eol */
YY_RULE_SETUP
#line 547 "scan.l"
{
					yyless(1);
					BEGIN(INITIAL);
					/*
					 * check that the data remains valid if it might have been
					 * made invalid by unescaping any chars.
					 */
					if (yyextra->saw_non_ascii)
						pg_verifymbstr(yyextra->literalbuf,
									   yyextra->literallen,
									   false);
					yylval->str = litbufdup(yyscanner);
					return SCONST;
				}
	YY_BREAK
case 24:
/* rule 24 can match eol */
#line 562 "scan.l"
case 25:
/* rule 25 can match eol */
YY_RULE_SETUP
#line 562 "scan.l"
{
					/* throw back all but the quote */
					yyless(1);
					/* xusend state looks for possible UESCAPE */
					BEGIN(xusend);
				}
	YY_BREAK
case 26:
/* rule 26 can match eol */
YY_RULE_SETUP
#line 568 "scan.l"
{
					/* stay in xusend state over whitespace */
				}
	YY_BREAK
case YY_STATE_EOF(xusend):
#line 571 "scan.l"
case 27:
/* rule 27 can match eol */
#line 573 "scan.l"
case 28:
/* rule 28 can match eol */
YY_RULE_SETUP
#line 573 "scan.l"
{
					/* no UESCAPE after the quote, throw back everything */
					yyless(0);
					BEGIN(INITIAL);
					yylval->str = litbuf_udeescape('\\', yyscanner);
					return SCONST;
				}
	YY_BREAK
case 29:
/* rule 29 can match eol */
YY_RULE_SETUP
#line 580 "scan.l"
{
					/* found UESCAPE after the end quote */
					BEGIN(INITIAL);
					if (!check_uescapechar(yytext[yyleng - 2]))
					{
						SET_YYLLOC();
						ADVANCE_YYLLOC(yyleng - 2);
						yyerror("invalid Unicode escape character");
					}
					yylval->str = litbuf_udeescape(yytext[yyleng - 2],
												   yyscanner);
					return SCONST;
				}
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 593 "scan.l"
{
					addlitchar('\'', yyscanner);
				}
	YY_BREAK
case 31:
/* rule 31 can match eol */
YY_RULE_SETUP
#line 596 "scan.l"
{
					addlit(yytext, yyleng, yyscanner);
				}
	YY_BREAK
case 32:
/* rule 32 can match eol */
YY_RULE_SETUP
#line 599 "scan.l"
{
					addlit(yytext, yyleng, yyscanner);
				}
	YY_BREAK
case 33:
YY_RULE_SETUP
#line 602 "scan.l"
{
					pg_wchar	c = strtoul(yytext + 2, NULL, 16);

					check_escape_warning(yyscanner);

					if (is_utf16_surrogate_first(c))
					{
						yyextra->utf16_first_part = c;
						BEGIN(xeu);
					}
					else if (is_utf16_surrogate_second(c))
						yyerror("invalid Unicode surrogate pair");
					else
						addunicode(c, yyscanner);
				}
	YY_BREAK
case 34:
YY_RULE_SETUP
#line 617 "scan.l"
{
					pg_wchar	c = strtoul(yytext + 2, NULL, 16);

					if (!is_utf16_surrogate_second(c))
						yyerror("invalid Unicode surrogate pair");

					c = surrogate_pair_to_codepoint(yyextra->utf16_first_part, c);

					addunicode(c, yyscanner);

					BEGIN(xe);
				}
	YY_BREAK
case 35:
YY_RULE_SETUP
#line 629 "scan.l"
{ yyerror("invalid Unicode surrogate pair"); }
	YY_BREAK
case 36:
/* rule 36 can match eol */
YY_RULE_SETUP
#line 630 "scan.l"
{ yyerror("invalid Unicode surrogate pair"); }
	YY_BREAK
case YY_STATE_EOF(xeu):
#line 631 "scan.l"
{ yyerror("invalid Unicode surrogate pair"); }
	YY_BREAK
case 37:
YY_RULE_SETUP
#line 632 "scan.l"
{
					ereport(ERROR,
							(errcode(ERRCODE_INVALID_ESCAPE_SEQUENCE),
							 errmsg("invalid Unicode escape"),
							 errhint("Unicode escapes must be \\uXXXX or \\UXXXXXXXX."),
							 lexer_errposition()));
				}
	YY_BREAK
case 38:
/* rule 38 can match eol */
YY_RULE_SETUP
#line 639 "scan.l"
{
					if (yytext[1] == '\'')
					{
						if (yyextra->backslash_quote == BACKSLASH_QUOTE_OFF ||
							(yyextra->backslash_quote == BACKSLASH_QUOTE_SAFE_ENCODING &&
							 PG_ENCODING_IS_CLIENT_ONLY(pg_get_client_encoding())))
							ereport(ERROR,
									(errcode(ERRCODE_NONSTANDARD_USE_OF_ESCAPE_CHARACTER),
									 errmsg("unsafe use of \\' in a string literal"),
									 errhint("Use '' to write quotes in strings. \\' is insecure in client-only encodings."),
									 lexer_errposition()));
					}
					check_string_escape_warning(yytext[1], yyscanner);
					addlitchar(unescape_single_char(yytext[1], yyscanner),
							   yyscanner);
				}
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 655 "scan.l"
{
					unsigned char c = strtoul(yytext + 1, NULL, 8);

					check_escape_warning(yyscanner);
					addlitchar(c, yyscanner);
					if (c == '\0' || IS_HIGHBIT_SET(c))
						yyextra->saw_non_ascii = true;
				}
	YY_BREAK
case 40:
YY_RULE_SETUP
#line 663 "scan.l"
{
					unsigned char c = strtoul(yytext + 2, NULL, 16);

					check_escape_warning(yyscanner);
					addlitchar(c, yyscanner);
					if (c == '\0' || IS_HIGHBIT_SET(c))
						yyextra->saw_non_ascii = true;
				}
	YY_BREAK
case 41:
/* rule 41 can match eol */
YY_RULE_SETUP
#line 671 "scan.l"
{
					/* ignore */
				}
	YY_BREAK
case 42:
YY_RULE_SETUP
#line 674 "scan.l"
{
					/* This is only needed for \ just before EOF */
					addlitchar(yytext[0], yyscanner);
				}
	YY_BREAK
case YY_STATE_EOF(xq):
case YY_STATE_EOF(xe):
case YY_STATE_EOF(xus):
#line 678 "scan.l"
{ yyerror("unterminated quoted string"); }
	YY_BREAK
case 43:
YY_RULE_SETUP
#line 680 "scan.l"
{
					SET_YYLLOC();
					yyextra->dolqstart = pstrdup(yytext);
					BEGIN(xdolq);
					startlit();
				}
	YY_BREAK
case 44:
YY_RULE_SETUP
#line 686 "scan.l"
{
					SET_YYLLOC();
					/* throw back all but the initial "$" */
					yyless(1);
					/* and treat it as {other} */
					return yytext[0];
				}
	YY_BREAK
case 45:
YY_RULE_SETUP
#line 693 "scan.l"
{
					if (strcmp(yytext, yyextra->dolqstart) == 0)
					{
						pfree(yyextra->dolqstart);
						yyextra->dolqstart = NULL;
						BEGIN(INITIAL);
						yylval->str = litbufdup(yyscanner);
						return SCONST;
					}
					else
					{
						/*
						 * When we fail to match $...$ to dolqstart, transfer
						 * the $... part to the output, but put back the final
						 * $ for rescanning.  Consider $delim$...$junk$delim$
						 */
						addlit(yytext, yyleng - 1, yyscanner);
						yyless(yyleng - 1);
					}
				}
	YY_BREAK
case 46:
/* rule 46 can match eol */
YY_RULE_SETUP
#line 713 "scan.l"
{
					addlit(yytext, yyleng, yyscanner);
				}
	YY_BREAK
case 47:
YY_RULE_SETUP
#line 716 "scan.l"
{
					addlit(yytext, yyleng, yyscanner);
				}
	YY_BREAK
case 48:
YY_RULE_SETUP
#line 719 "scan.l"
{
					/* This is only needed for $ inside the quoted text */
					addlitchar(yytext[0], yyscanner);
				}
	YY_BREAK
case YY_STATE_EOF(xdolq):
#line 723 "scan.l"
{ yyerror("unterminated dollar-quoted string"); }
	YY_BREAK
case 49:
YY_RULE_SETUP
#line 725 "scan.l"
{
					SET_YYLLOC();
					BEGIN(xd);
					startlit();
				}
	YY_BREAK
case 50:
YY_RULE_SETUP
#line 730 "scan.l"
{
					SET_YYLLOC();
					BEGIN(xui);
					startlit();
				}
	YY_BREAK
case 51:
YY_RULE_SETUP
#line 735 "scan.l"
{
					char	   *ident;

					BEGIN(INITIAL);
					if (yyextra->literallen == 0)
						yyerror("zero-length delimited identifier");
					ident = litbufdup(yyscanner);
					if (yyextra->literallen >= NAMEDATALEN)
						truncate_identifier(ident, yyextra->literallen, true);
					yylval->str = ident;
					return IDENT;
				}
	YY_BREAK
case 52:
YY_RULE_SETUP
#line 747 "scan.l"
{
					yyless(1);
					/* xuiend state looks for possible UESCAPE */
					BEGIN(xuiend);
				}
	YY_BREAK
case 53:
/* rule 53 can match eol */
YY_RULE_SETUP
#line 752 "scan.l"
{
					/* stay in xuiend state over whitespace */
				}
	YY_BREAK
case YY_STATE_EOF(xuiend):
#line 755 "scan.l"
case 54:
/* rule 54 can match eol */
#line 757 "scan.l"
case 55:
/* rule 55 can match eol */
YY_RULE_SETUP
#line 757 "scan.l"
{
					/* no UESCAPE after the quote, throw back everything */
					char	   *ident;
					int			identlen;

					yyless(0);

					BEGIN(INITIAL);
					if (yyextra->literallen == 0)
						yyerror("zero-length delimited identifier");
					ident = litbuf_udeescape('\\', yyscanner);
					identlen = strlen(ident);
					if (identlen >= NAMEDATALEN)
						truncate_identifier(ident, identlen, true);
					yylval->str = ident;
					return IDENT;
				}
	YY_BREAK
case 56:
/* rule 56 can match eol */
YY_RULE_SETUP
#line 774 "scan.l"
{
					/* found UESCAPE after the end quote */
					char	   *ident;
					int			identlen;

					BEGIN(INITIAL);
					if (yyextra->literallen == 0)
						yyerror("zero-length delimited identifier");
					if (!check_uescapechar(yytext[yyleng - 2]))
					{
						SET_YYLLOC();
						ADVANCE_YYLLOC(yyleng - 2);
						yyerror("invalid Unicode escape character");
					}
					ident = litbuf_udeescape(yytext[yyleng - 2], yyscanner);
					identlen = strlen(ident);
					if (identlen >= NAMEDATALEN)
						truncate_identifier(ident, identlen, true);
					yylval->str = ident;
					return IDENT;
				}
	YY_BREAK
case 57:
YY_RULE_SETUP
#line 795 "scan.l"
{
					addlitchar('"', yyscanner);
				}
	YY_BREAK
case 58:
/* rule 58 can match eol */
YY_RULE_SETUP
#line 798 "scan.l"
{
					addlit(yytext, yyleng, yyscanner);
				}
	YY_BREAK
case YY_STATE_EOF(xd):
case YY_STATE_EOF(xui):
#line 801 "scan.l"
{ yyerror("unterminated quoted identifier"); }
	YY_BREAK
case 59:
YY_RULE_SETUP
#line 803 "scan.l"
{
					char	   *ident;

					SET_YYLLOC();
					/* throw back all but the initial u/U */
					yyless(1);
					/* and treat it as {identifier} */
					ident = downcase_truncate_identifier(yytext, yyleng, true);
					yylval->str = ident;
					return IDENT;
				}
	YY_BREAK
case 60:
YY_RULE_SETUP
#line 815 "scan.l"
{
					SET_YYLLOC();
					return TYPECAST;
				}
	YY_BREAK
case 61:
YY_RULE_SETUP
#line 820 "scan.l"
{
					SET_YYLLOC();
					return DOT_DOT;
				}
	YY_BREAK
case 62:
YY_RULE_SETUP
#line 825 "scan.l"
{
					SET_YYLLOC();
					return COLON_EQUALS;
				}
	YY_BREAK
case 63:
YY_RULE_SETUP
#line 830 "scan.l"
{
					SET_YYLLOC();
					return EQUALS_GREATER;
				}
	YY_BREAK
case 64:
YY_RULE_SETUP
#line 835 "scan.l"
{
					SET_YYLLOC();
					return LESS_EQUALS;
				}
	YY_BREAK
case 65:
YY_RULE_SETUP
#line 840 "scan.l"
{
					SET_YYLLOC();
					return GREATER_EQUALS;
				}
	YY_BREAK
case 66:
YY_RULE_SETUP
#line 845 "scan.l"
{
					/* We accept both "<>" and "!=" as meaning NOT_EQUALS */
					SET_YYLLOC();
					return NOT_EQUALS;
				}
	YY_BREAK
case 67:
YY_RULE_SETUP
#line 851 "scan.l"
{
					/* We accept both "<>" and "!=" as meaning NOT_EQUALS */
					SET_YYLLOC();
					return NOT_EQUALS;
				}
	YY_BREAK
case 68:
YY_RULE_SETUP
#line 857 "scan.l"
{
					SET_YYLLOC();
					return yytext[0];
				}
	YY_BREAK
case 69:
YY_RULE_SETUP
#line 862 "scan.l"
{
					/*
					 * Check for embedded slash-star or dash-dash; those
					 * are comment starts, so operator must stop there.
					 * Note that slash-star or dash-dash at the first
					 * character will match a prior rule, not this one.
					 */
					int			nchars = yyleng;
					char	   *slashstar = strstr(yytext, "/*");
					char	   *dashdash = strstr(yytext, "--");
#ifdef _PG_ORCL_
					int			need_replace_ne = 0;
#endif
					if (slashstar && dashdash)
					{
						/* if both appear, take the first one */
						if (slashstar > dashdash)
							slashstar = dashdash;
					}
					else if (!slashstar)
						slashstar = dashdash;
					if (slashstar)
						nchars = slashstar - yytext;

					/*
					 * For SQL compatibility, '+' and '-' cannot be the
					 * last char of a multi-char operator unless the operator
					 * contains chars that are not in SQL operators.
					 * The idea is to lex '=-' as two operators, but not
					 * to forbid operator names like '?-' that could not be
					 * sequences of SQL operators.
					 */
#ifdef _PG_ORCL_
					/* Convert "!=" operator to "<>" for compatibility and let !=- can also work */
					if (enable_oracle_compatible && nchars > 1 &&
					    yytext[0] == '!' && yytext[1] == '=')
					{
						yytext[0] = '<';
						yytext[1] = '>';
						need_replace_ne = 1;
					}
#endif
					while (nchars > 1 &&
						   (yytext[nchars - 1] == '+' ||
							yytext[nchars - 1] == '-'))
					{
						int			ic;

						for (ic = nchars - 2; ic >= 0; ic--)
						{
							if (strchr("~!@#^&|`?%", yytext[ic]))
								break;
						}
						if (ic >= 0)
							break; /* found a char that makes it OK */
						nchars--; /* else remove the +/-, and check again */
					}

					SET_YYLLOC();

					if (nchars < yyleng)
					{
						/* Strip the unwanted chars from the token */
						yyless(nchars);
						/*
						 * If what we have left is only one char, and it's
						 * one of the characters matching "self", then
						 * return it as a character token the same way
						 * that the "self" rule would have.
						 */
						if (nchars == 1 &&
							strchr(",()[].;:+-*/%^<>=", yytext[0]))
							return yytext[0];
					}

					/*
					 * Complain if operator is too long.  Unlike the case
					 * for identifiers, we make this an error not a notice-
					 * and-truncate, because the odds are we are looking at
					 * a syntactic mistake anyway.
					 */
					if (nchars >= NAMEDATALEN)
						yyerror("operator too long");

					yylval->str = pstrdup(yytext);
#ifdef _PG_ORCL_
					if (enable_oracle_compatible && 
						need_replace_ne)
					{
						yytext[0] = '!';
						yytext[1] = '=';
					}
#endif
					return Op;
				}
	YY_BREAK
case 70:
YY_RULE_SETUP
#line 958 "scan.l"
{
					SET_YYLLOC();
					yylval->ival = atol(yytext + 1);
					return PARAM;
				}
	YY_BREAK
case 71:
YY_RULE_SETUP
#line 963 "scan.l"
{
                    SET_YYLLOC();
                    yyerror("invalid input variable");
                }
	YY_BREAK
case 72:
YY_RULE_SETUP
#line 968 "scan.l"
{
					SET_YYLLOC();
					return process_integer_literal(yytext, yylval);
				}
	YY_BREAK
case 73:
YY_RULE_SETUP
#line 972 "scan.l"
{
					SET_YYLLOC();
					yylval->str = pstrdup(yytext);
					return FCONST;
				}
	YY_BREAK
case 74:
YY_RULE_SETUP
#line 977 "scan.l"
{
					/* throw back the .., and treat as integer */
					yyless(yyleng - 2);
					SET_YYLLOC();
					return process_integer_literal(yytext, yylval);
				}
	YY_BREAK
case 75:
YY_RULE_SETUP
#line 983 "scan.l"
{
					SET_YYLLOC();
					yylval->str = pstrdup(yytext);
					return FCONST;
				}
	YY_BREAK
case 76:
YY_RULE_SETUP
#line 988 "scan.l"
{
					/*
					 * throw back the [Ee], and treat as {decimal}.  Note
					 * that it is possible the input is actually {integer},
					 * but since this case will almost certainly lead to a
					 * syntax error anyway, we don't bother to distinguish.
					 */
					yyless(yyleng - 1);
					SET_YYLLOC();
					yylval->str = pstrdup(yytext);
					return FCONST;
				}
	YY_BREAK
case 77:
YY_RULE_SETUP
#line 1000 "scan.l"
{
					/* throw back the [Ee][+-], and proceed as above */
					yyless(yyleng - 2);
					SET_YYLLOC();
					yylval->str = pstrdup(yytext);
					return FCONST;
				}
	YY_BREAK
case 78:
YY_RULE_SETUP
#line 1009 "scan.l"
{
					const ScanKeyword *keyword;
					char	   *ident;

					SET_YYLLOC();

					/* Is it a keyword? */
					keyword = ScanKeywordLookup(yytext,
												yyextra->keywords,
												yyextra->num_keywords);
					if (keyword != NULL)
					{
						yylval->keyword = keyword->name;
						return keyword->value;
					}

					/*
					 * No.  Convert the identifier to lower case, and truncate
					 * if necessary.
					 */
					ident = downcase_truncate_identifier(yytext, yyleng, true);
					yylval->str = ident;
					return IDENT;
				}
	YY_BREAK
case 79:
YY_RULE_SETUP
#line 1034 "scan.l"
{
					SET_YYLLOC();
					return yytext[0];
				}
	YY_BREAK
case YY_STATE_EOF(INITIAL):
#line 1039 "scan.l"
{
					SET_YYLLOC();
					yyterminate();
				}
	YY_BREAK
case 80:
YY_RULE_SETUP
#line 1044 "scan.l"
YY_FATAL_ERROR( "flex scanner jammed" );
	YY_BREAK
#line 10133 "scan.c"

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - yyg->yytext_ptr) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = yyg->yy_hold_char;
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * core_yylex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( yyg->yy_c_buf_p <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			yyg->yy_c_buf_p = yyg->yytext_ptr + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state( yyscanner );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state , yyscanner);

			yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++yyg->yy_c_buf_p;
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = yyg->yy_c_buf_p;
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer( yyscanner ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				yyg->yy_did_buffer_switch_on_eof = 0;

				if ( core_yywrap(yyscanner ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					yyg->yy_c_buf_p = yyg->yytext_ptr + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! yyg->yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				yyg->yy_c_buf_p =
					yyg->yytext_ptr + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state( yyscanner );

				yy_cp = yyg->yy_c_buf_p;
				yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				yyg->yy_c_buf_p =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars];

				yy_current_state = yy_get_previous_state( yyscanner );

				yy_cp = yyg->yy_c_buf_p;
				yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of user's declarations */
} /* end of core_yylex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	char *source = yyg->yytext_ptr;
	yy_size_t number_to_move, i;
	int ret_val;

	if ( yyg->yy_c_buf_p > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yyg->yy_c_buf_p - yyg->yytext_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (yy_size_t) (yyg->yy_c_buf_p - yyg->yytext_ptr) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER_LVALUE;

			int yy_c_buf_p_offset =
				(int) (yyg->yy_c_buf_p - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					core_yyrealloc((void *) b->yy_ch_buf,(yy_size_t) (b->yy_buf_size + 2) ,yyscanner );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = NULL;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			yyg->yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			yyg->yy_n_chars, num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	if ( yyg->yy_n_chars == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			core_yyrestart(yyin  ,yyscanner);
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((int) (yyg->yy_n_chars + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		int new_size = yyg->yy_n_chars + number_to_move + (yyg->yy_n_chars >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) core_yyrealloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,(yy_size_t) new_size ,yyscanner );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
	}

	yyg->yy_n_chars += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	yyg->yytext_ptr = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (yyscan_t yyscanner)
{
	yy_state_type yy_current_state;
	char *yy_cp;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	yy_current_state = yy_start_state_list[yyg->yy_start];

	for ( yy_cp = yyg->yytext_ptr + YY_MORE_ADJ; yy_cp < yyg->yy_c_buf_p; ++yy_cp )
		{
		yy_current_state += yy_current_state[(*yy_cp ? YY_SC_TO_UI(*yy_cp) : 256)].yy_nxt;
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state , yyscan_t yyscanner)
{
	int yy_is_jam;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner; /* This var may be unused depending upon options. */

	int yy_c = 256;
	yyconst struct yy_trans_info *yy_trans_info;

	yy_trans_info = &yy_current_state[(unsigned int) yy_c];
	yy_current_state += yy_trans_info->yy_nxt;
	yy_is_jam = (yy_trans_info->yy_verify != yy_c);

	(void)yyg;
	return yy_is_jam ? 0 : yy_current_state;
}

#ifndef YY_NO_UNPUT

#endif

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (yyscan_t yyscanner)
#else
    static int input  (yyscan_t yyscanner)
#endif

{
	int c;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	*yyg->yy_c_buf_p = yyg->yy_hold_char;

	if ( *yyg->yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yyg->yy_c_buf_p < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
			/* This was really a NUL. */
			*yyg->yy_c_buf_p = '\0';

		else
			{ /* need more input */
			int offset = yyg->yy_c_buf_p - yyg->yytext_ptr;
			++yyg->yy_c_buf_p;

			switch ( yy_get_next_buffer( yyscanner ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					core_yyrestart(yyin ,yyscanner);

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( core_yywrap(yyscanner ) )
						return 0;

					if ( ! yyg->yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput(yyscanner);
#else
					return input(yyscanner);
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					yyg->yy_c_buf_p = yyg->yytext_ptr + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) yyg->yy_c_buf_p;	/* cast for 8-bit char's */
	*yyg->yy_c_buf_p = '\0';	/* preserve yytext */
	yyg->yy_hold_char = *++yyg->yy_c_buf_p;

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * @param yyscanner The scanner object.
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void core_yyrestart  (FILE * input_file , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if ( ! YY_CURRENT_BUFFER ){
        core_yyensure_buffer_stack (yyscanner);
		YY_CURRENT_BUFFER_LVALUE =
            core_yy_create_buffer(yyin,YY_BUF_SIZE ,yyscanner);
	}

	core_yy_init_buffer(YY_CURRENT_BUFFER,input_file ,yyscanner);
	core_yy_load_buffer_state(yyscanner );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * @param yyscanner The scanner object.
 */
    void core_yy_switch_to_buffer  (YY_BUFFER_STATE  new_buffer , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	/* TODO. We should be able to replace this entire function body
	 * with
	 *		core_yypop_buffer_state();
	 *		core_yypush_buffer_state(new_buffer);
     */
	core_yyensure_buffer_stack (yyscanner);
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*yyg->yy_c_buf_p = yyg->yy_hold_char;
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	core_yy_load_buffer_state(yyscanner );

	/* We don't actually know whether we did this switch during
	 * EOF (core_yywrap()) processing, but the only time this flag
	 * is looked at is after core_yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yyg->yy_did_buffer_switch_on_eof = 1;
}

static void core_yy_load_buffer_state  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	yyg->yytext_ptr = yyg->yy_c_buf_p = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	yyg->yy_hold_char = *yyg->yy_c_buf_p;
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * @param yyscanner The scanner object.
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE core_yy_create_buffer  (FILE * file, int  size , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) core_yyalloc(sizeof( struct yy_buffer_state ) ,yyscanner );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in core_yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) core_yyalloc((yy_size_t) (b->yy_buf_size + 2) ,yyscanner );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in core_yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	core_yy_init_buffer(b,file ,yyscanner);

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with core_yy_create_buffer()
 * @param yyscanner The scanner object.
 */
    void core_yy_delete_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		core_yyfree((void *) b->yy_ch_buf ,yyscanner );

	core_yyfree((void *) b ,yyscanner );
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a core_yyrestart() or at EOF.
 */
    static void core_yy_init_buffer  (YY_BUFFER_STATE  b, FILE * file , yyscan_t yyscanner)

{
	int oerrno = errno;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	core_yy_flush_buffer(b ,yyscanner);

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then core_yy_init_buffer was _probably_
     * called from core_yyrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * @param yyscanner The scanner object.
 */
    void core_yy_flush_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		core_yy_load_buffer_state(yyscanner );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  @param yyscanner The scanner object.
 */
void core_yypush_buffer_state (YY_BUFFER_STATE new_buffer , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if (new_buffer == NULL)
		return;

	core_yyensure_buffer_stack(yyscanner);

	/* This block is copied from core_yy_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*yyg->yy_c_buf_p = yyg->yy_hold_char;
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		yyg->yy_buffer_stack_top++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from core_yy_switch_to_buffer. */
	core_yy_load_buffer_state(yyscanner );
	yyg->yy_did_buffer_switch_on_eof = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  @param yyscanner The scanner object.
 */
void core_yypop_buffer_state (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if (!YY_CURRENT_BUFFER)
		return;

	core_yy_delete_buffer(YY_CURRENT_BUFFER ,yyscanner);
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if (yyg->yy_buffer_stack_top > 0)
		--yyg->yy_buffer_stack_top;

	if (YY_CURRENT_BUFFER) {
		core_yy_load_buffer_state(yyscanner );
		yyg->yy_did_buffer_switch_on_eof = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void core_yyensure_buffer_stack (yyscan_t yyscanner)
{
	int num_to_alloc;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if (!yyg->yy_buffer_stack) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
      num_to_alloc = 1; /* After all that talk, this was set to 1 anyways... */
		yyg->yy_buffer_stack = (struct yy_buffer_state**)core_yyalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								, yyscanner);
		if ( ! yyg->yy_buffer_stack )
			YY_FATAL_ERROR( "out of dynamic memory in core_yyensure_buffer_stack()" );
								  
		memset(yyg->yy_buffer_stack, 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		yyg->yy_buffer_stack_max = num_to_alloc;
		yyg->yy_buffer_stack_top = 0;
		return;
	}

	if (yyg->yy_buffer_stack_top >= (yyg->yy_buffer_stack_max) - 1){

		/* Increase the buffer to prepare for a possible push. */
		yy_size_t grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = yyg->yy_buffer_stack_max + grow_size;
		yyg->yy_buffer_stack = (struct yy_buffer_state**)core_yyrealloc
								(yyg->yy_buffer_stack,
								num_to_alloc * sizeof(struct yy_buffer_state*)
								, yyscanner);
		if ( ! yyg->yy_buffer_stack )
			YY_FATAL_ERROR( "out of dynamic memory in core_yyensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset(yyg->yy_buffer_stack + yyg->yy_buffer_stack_max, 0, grow_size * sizeof(struct yy_buffer_state*));
		yyg->yy_buffer_stack_max = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE core_yy_scan_buffer  (char * base, yy_size_t  size , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return NULL;

	b = (YY_BUFFER_STATE) core_yyalloc(sizeof( struct yy_buffer_state ) ,yyscanner );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in core_yy_scan_buffer()" );

	b->yy_buf_size = (int) (size - 2);	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = NULL;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	core_yy_switch_to_buffer(b ,yyscanner );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to core_yylex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       core_yy_scan_bytes() instead.
 */
YY_BUFFER_STATE core_yy_scan_string (yyconst char * yystr , yyscan_t yyscanner)
{
    
	return core_yy_scan_bytes(yystr,(int) strlen(yystr) ,yyscanner);
}

/** Setup the input buffer state to scan the given bytes. The next call to core_yylex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE core_yy_scan_bytes  (yyconst char * yybytes, int  _yybytes_len , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = (yy_size_t) (_yybytes_len + 2);
	buf = (char *) core_yyalloc(n ,yyscanner );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in core_yy_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = core_yy_scan_buffer(buf,n ,yyscanner);
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in core_yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yynoreturn yy_fatal_error (yyconst char* msg , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;
	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        yy_size_t yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		yytext[yyleng] = yyg->yy_hold_char; \
		yyg->yy_c_buf_p = yytext + yyless_macro_arg; \
		yyg->yy_hold_char = *yyg->yy_c_buf_p; \
		*yyg->yy_c_buf_p = '\0'; \
		yyleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the user-defined data for this scanner.
 * @param yyscanner The scanner object.
 */
YY_EXTRA_TYPE core_yyget_extra  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyextra;
}

/** Get the current line number.
 * @param yyscanner The scanner object.
 */
int core_yyget_lineno  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    
        if (! YY_CURRENT_BUFFER)
            return 0;
    
    return yylineno;
}

/** Get the current column number.
 * @param yyscanner The scanner object.
 */
int core_yyget_column  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    
        if (! YY_CURRENT_BUFFER)
            return 0;
    
    return yycolumn;
}

/** Get the input stream.
 * @param yyscanner The scanner object.
 */
FILE *core_yyget_in  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyin;
}

/** Get the output stream.
 * @param yyscanner The scanner object.
 */
FILE *core_yyget_out  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyout;
}

/** Get the length of the current token.
 * @param yyscanner The scanner object.
 */
int core_yyget_leng  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyleng;
}

/** Get the current token.
 * @param yyscanner The scanner object.
 */

char *core_yyget_text  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yytext;
}

/** Set the user-defined data. This data is never touched by the scanner.
 * @param user_defined The data to be associated with this scanner.
 * @param yyscanner The scanner object.
 */
void core_yyset_extra (YY_EXTRA_TYPE  user_defined , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyextra = user_defined ;
}

/** Set the current line number.
 * @param _line_number line number
 * @param yyscanner The scanner object.
 */
void core_yyset_lineno (int  _line_number , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* lineno is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           YY_FATAL_ERROR( "core_yyset_lineno called with no buffer" );
    
    yylineno = _line_number;
}

/** Set the current column.
 * @param _column_no column number
 * @param yyscanner The scanner object.
 */
void core_yyset_column (int  _column_no , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* column is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           YY_FATAL_ERROR( "core_yyset_column called with no buffer" );
    
    yycolumn = _column_no;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param _in_str A readable stream.
 * @param yyscanner The scanner object.
 * @see core_yy_switch_to_buffer
 */
void core_yyset_in (FILE *  _in_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyin = _in_str ;
}

void core_yyset_out (FILE *  _out_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyout = _out_str ;
}

int core_yyget_debug  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yy_flex_debug;
}

void core_yyset_debug (int  _bdebug , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yy_flex_debug = _bdebug ;
}

/* Accessor methods for yylval and yylloc */

YYSTYPE * core_yyget_lval  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yylval;
}

void core_yyset_lval (YYSTYPE *  yylval_param , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yylval = yylval_param;
}

YYLTYPE *core_yyget_lloc  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yylloc;
}
    
void core_yyset_lloc (YYLTYPE *  yylloc_param , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yylloc = yylloc_param;
}
    
/* User-visible API */

/* core_yylex_init is special because it creates the scanner itself, so it is
 * the ONLY reentrant function that doesn't take the scanner as the last argument.
 * That's why we explicitly handle the declaration, instead of using our macros.
 */

int core_yylex_init(yyscan_t* ptr_yy_globals)

{
    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) core_yyalloc ( sizeof( struct yyguts_t ), NULL );

    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));

    return yy_init_globals ( *ptr_yy_globals );
}

/* core_yylex_init_extra has the same functionality as core_yylex_init, but follows the
 * convention of taking the scanner as the last argument. Note however, that
 * this is a *pointer* to a scanner, as it will be allocated by this call (and
 * is the reason, too, why this function also must handle its own declaration).
 * The user defined value in the first argument will be available to core_yyalloc in
 * the yyextra field.
 */

int core_yylex_init_extra(YY_EXTRA_TYPE yy_user_defined,yyscan_t* ptr_yy_globals )

{
    struct yyguts_t dummy_yyguts;

    core_yyset_extra (yy_user_defined, &dummy_yyguts);

    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }
	
    *ptr_yy_globals = (yyscan_t) core_yyalloc ( sizeof( struct yyguts_t ), &dummy_yyguts );
	
    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }
    
    /* By setting to 0xAA, we expose bugs in
    yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));
    
    core_yyset_extra (yy_user_defined, *ptr_yy_globals);
    
    return yy_init_globals ( *ptr_yy_globals );
}

static int yy_init_globals (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from core_yylex_destroy(), so don't allocate here.
     */

    yyg->yy_buffer_stack = NULL;
    yyg->yy_buffer_stack_top = 0;
    yyg->yy_buffer_stack_max = 0;
    yyg->yy_c_buf_p = NULL;
    yyg->yy_init = 0;
    yyg->yy_start = 0;

    yyg->yy_start_stack_ptr = 0;
    yyg->yy_start_stack_depth = 0;
    yyg->yy_start_stack =  NULL;

/* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = NULL;
    yyout = NULL;
#endif

    /* For future reference: Set errno on error, since we are called by
     * core_yylex_init()
     */
    return 0;
}

/* core_yylex_destroy is for both reentrant and non-reentrant scanners. */
int core_yylex_destroy  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		core_yy_delete_buffer(YY_CURRENT_BUFFER ,yyscanner );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		core_yypop_buffer_state(yyscanner);
	}

	/* Destroy the stack itself. */
	core_yyfree(yyg->yy_buffer_stack ,yyscanner);
	yyg->yy_buffer_stack = NULL;

    /* Destroy the start condition stack. */
        core_yyfree(yyg->yy_start_stack ,yyscanner );
        yyg->yy_start_stack = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * core_yylex() is called, initialization will occur. */
    yy_init_globals( yyscanner);

    /* Destroy the main struct (reentrant only). */
    core_yyfree ( yyscanner , yyscanner );
    yyscanner = NULL;
    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;

	int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s , yyscan_t yyscanner)
{
	int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

#define YYTABLES_NAME "yytables"

#line 1044 "scan.l"



/*
 * Arrange access to yyextra for subroutines of the main core_yylex() function.
 * We expect each subroutine to have a yyscanner parameter.  Rather than
 * use the yyget_xxx functions, which might or might not get inlined by the
 * compiler, we cheat just a bit and cast yyscanner to the right type.
 */
#undef yyextra
#define yyextra  (((struct yyguts_t *) yyscanner)->yyextra_r)

/* Likewise for a couple of other things we need. */
#undef yylloc
#define yylloc	(((struct yyguts_t *) yyscanner)->yylloc_r)
#undef yyleng
#define yyleng	(((struct yyguts_t *) yyscanner)->yyleng_r)


/*
 * scanner_errposition
 *		Report a lexer or grammar error cursor position, if possible.
 *
 * This is expected to be used within an ereport() call.  The return value
 * is a dummy (always 0, in fact).
 *
 * Note that this can only be used for messages emitted during raw parsing
 * (essentially, scan.l and gram.y), since it requires the yyscanner struct
 * to still be available.
 */
int
scanner_errposition(int location, core_yyscan_t yyscanner)
{
	int			pos;

	if (location < 0)
		return 0;				/* no-op if location is unknown */

	/* Convert byte offset to character number */
	pos = pg_mbstrlen_with_len(yyextra->scanbuf, location) + 1;
	/* And pass it to the ereport mechanism */
	return errposition(pos);
}

/*
 * scanner_yyerror
 *		Report a lexer or grammar error.
 *
 * The message's cursor position is whatever YYLLOC was last set to,
 * ie, the start of the current token if called within core_yylex(), or the
 * most recently lexed token if called from the grammar.
 * This is OK for syntax error messages from the Bison parser, because Bison
 * parsers report error as soon as the first unparsable token is reached.
 * Beware of using yyerror for other purposes, as the cursor position might
 * be misleading!
 */
void
scanner_yyerror(const char *message, core_yyscan_t yyscanner)
{
	const char *loc = yyextra->scanbuf + *yylloc;

	if (*loc == YY_END_OF_BUFFER_CHAR)
	{
		ereport(ERROR,
				(errcode(ERRCODE_SYNTAX_ERROR),
		/* translator: %s is typically the translation of "syntax error" */
				 errmsg("%s at end of input", _(message)),
				 lexer_errposition()));
	}
	else
	{
		ereport(ERROR,
				(errcode(ERRCODE_SYNTAX_ERROR),
		/* translator: first %s is typically the translation of "syntax error" */
				 errmsg("%s at or near \"%s\"", _(message), loc),
				 lexer_errposition()));
	}
}


/*
 * Called before any actual parsing is done
 */
core_yyscan_t
scanner_init(const char *str,
			 core_yy_extra_type *yyext,
			 const ScanKeyword *keywords,
			 int num_keywords)
{
	Size		slen = strlen(str);
	yyscan_t	scanner;

	if (core_yylex_init(&scanner) != 0)
		elog(ERROR, "core_yylex_init() failed: %m");

	core_yyset_extra(yyext, scanner);

	yyext->keywords = keywords;
	yyext->num_keywords = num_keywords;

	yyext->backslash_quote = backslash_quote;
	yyext->escape_string_warning = escape_string_warning;
	yyext->standard_conforming_strings = standard_conforming_strings;

	/*
	 * Make a scan buffer with special termination needed by flex.
	 */
	yyext->scanbuf = (char *) palloc(slen + 2);
	yyext->scanbuflen = slen;
	memcpy(yyext->scanbuf, str, slen);
	yyext->scanbuf[slen] = yyext->scanbuf[slen + 1] = YY_END_OF_BUFFER_CHAR;
	core_yy_scan_buffer(yyext->scanbuf,slen + 2,scanner);

	/* initialize literal buffer to a reasonable but expansible size */
	yyext->literalalloc = 1024;
	yyext->literalbuf = (char *) palloc(yyext->literalalloc);
	yyext->literallen = 0;

	return scanner;
}


/*
 * Called after parsing is done to clean up after scanner_init()
 */
void
scanner_finish(core_yyscan_t yyscanner)
{
	/*
	 * We don't bother to call core_yylex_destroy(), because all it would do is
	 * pfree a small amount of control storage.  It's cheaper to leak the
	 * storage until the parsing context is destroyed.  The amount of space
	 * involved is usually negligible compared to the output parse tree
	 * anyway.
	 *
	 * We do bother to pfree the scanbuf and literal buffer, but only if they
	 * represent a nontrivial amount of space.  The 8K cutoff is arbitrary.
	 */
	if (yyextra->scanbuflen >= 8192)
		pfree(yyextra->scanbuf);
	if (yyextra->literalalloc >= 8192)
		pfree(yyextra->literalbuf);
}


static void
addlit(char *ytext, int yleng, core_yyscan_t yyscanner)
{
	/* enlarge buffer if needed */
	if ((yyextra->literallen + yleng) >= yyextra->literalalloc)
	{
		do
		{
			yyextra->literalalloc *= 2;
		} while ((yyextra->literallen + yleng) >= yyextra->literalalloc);
		yyextra->literalbuf = (char *) repalloc(yyextra->literalbuf,
												yyextra->literalalloc);
	}
	/* append new data */
	memcpy(yyextra->literalbuf + yyextra->literallen, ytext, yleng);
	yyextra->literallen += yleng;
}


static void
addlitchar(unsigned char ychar, core_yyscan_t yyscanner)
{
	/* enlarge buffer if needed */
	if ((yyextra->literallen + 1) >= yyextra->literalalloc)
	{
		yyextra->literalalloc *= 2;
		yyextra->literalbuf = (char *) repalloc(yyextra->literalbuf,
												yyextra->literalalloc);
	}
	/* append new data */
	yyextra->literalbuf[yyextra->literallen] = ychar;
	yyextra->literallen += 1;
}


/*
 * Create a palloc'd copy of literalbuf, adding a trailing null.
 */
static char *
litbufdup(core_yyscan_t yyscanner)
{
	int			llen = yyextra->literallen;
	char	   *new;

	new = palloc(llen + 1);
	memcpy(new, yyextra->literalbuf, llen);
	new[llen] = '\0';
	return new;
}

static int
process_integer_literal(const char *token, YYSTYPE *lval)
{
	long		val;
	char	   *endptr;

	errno = 0;
	val = strtol(token, &endptr, 10);
	if (*endptr != '\0' || errno == ERANGE
#ifdef HAVE_LONG_INT_64
	/* if long > 32 bits, check for overflow of int4 */
		|| val != (long) ((int32) val)
#endif
		)
	{
		/* integer too large, treat it as a float */
		lval->str = pstrdup(token);
		return FCONST;
	}
	lval->ival = val;
	return ICONST;
}

static unsigned int
hexval(unsigned char c)
{
	if (c >= '0' && c <= '9')
		return c - '0';
	if (c >= 'a' && c <= 'f')
		return c - 'a' + 0xA;
	if (c >= 'A' && c <= 'F')
		return c - 'A' + 0xA;
	elog(ERROR, "invalid hexadecimal digit");
	return 0;					/* not reached */
}

static void
check_unicode_value(pg_wchar c, char *loc, core_yyscan_t yyscanner)
{
	if (GetDatabaseEncoding() == PG_UTF8)
		return;

	if (c > 0x7F)
	{
		ADVANCE_YYLLOC(loc - yyextra->literalbuf + 3);	/* 3 for U&" */
		yyerror("Unicode escape values cannot be used for code point values above 007F when the server encoding is not UTF8");
	}
}

static bool
is_utf16_surrogate_first(pg_wchar c)
{
	return (c >= 0xD800 && c <= 0xDBFF);
}

static bool
is_utf16_surrogate_second(pg_wchar c)
{
	return (c >= 0xDC00 && c <= 0xDFFF);
}

static pg_wchar
surrogate_pair_to_codepoint(pg_wchar first, pg_wchar second)
{
	return ((first & 0x3FF) << 10) + 0x10000 + (second & 0x3FF);
}

static void
addunicode(pg_wchar c, core_yyscan_t yyscanner)
{
	char		buf[8];

	if (c == 0 || c > 0x10FFFF)
		yyerror("invalid Unicode escape value");
	if (c > 0x7F)
	{
		if (GetDatabaseEncoding() != PG_UTF8)
			yyerror("Unicode escape values cannot be used for code point values above 007F when the server encoding is not UTF8");
		yyextra->saw_non_ascii = true;
	}
	unicode_to_utf8(c, (unsigned char *) buf);
	addlit(buf, pg_mblen(buf), yyscanner);
}

/* is 'escape' acceptable as Unicode escape character (UESCAPE syntax) ? */
static bool
check_uescapechar(unsigned char escape)
{
	if (isxdigit(escape)
		|| escape == '+'
		|| escape == '\''
		|| escape == '"'
		|| scanner_isspace(escape))
	{
		return false;
	}
	else
		return true;
}

/* like litbufdup, but handle unicode escapes */
static char *
litbuf_udeescape(unsigned char escape, core_yyscan_t yyscanner)
{
	char	   *new;
	char	   *litbuf,
			   *in,
			   *out;
	pg_wchar	pair_first = 0;

	/* Make literalbuf null-terminated to simplify the scanning loop */
	litbuf = yyextra->literalbuf;
	litbuf[yyextra->literallen] = '\0';

	/*
	 * This relies on the subtle assumption that a UTF-8 expansion cannot be
	 * longer than its escaped representation.
	 */
	new = palloc(yyextra->literallen + 1);

	in = litbuf;
	out = new;
	while (*in)
	{
		if (in[0] == escape)
		{
			if (in[1] == escape)
			{
				if (pair_first)
				{
					ADVANCE_YYLLOC(in - litbuf + 3);	/* 3 for U&" */
					yyerror("invalid Unicode surrogate pair");
				}
				*out++ = escape;
				in += 2;
			}
			else if (isxdigit((unsigned char) in[1]) &&
					 isxdigit((unsigned char) in[2]) &&
					 isxdigit((unsigned char) in[3]) &&
					 isxdigit((unsigned char) in[4]))
			{
				pg_wchar	unicode;

				unicode = (hexval(in[1]) << 12) +
					(hexval(in[2]) << 8) +
					(hexval(in[3]) << 4) +
					hexval(in[4]);
				check_unicode_value(unicode, in, yyscanner);
				if (pair_first)
				{
					if (is_utf16_surrogate_second(unicode))
					{
						unicode = surrogate_pair_to_codepoint(pair_first, unicode);
						pair_first = 0;
					}
					else
					{
						ADVANCE_YYLLOC(in - litbuf + 3);		/* 3 for U&" */
						yyerror("invalid Unicode surrogate pair");
					}
				}
				else if (is_utf16_surrogate_second(unicode))
					yyerror("invalid Unicode surrogate pair");

				if (is_utf16_surrogate_first(unicode))
					pair_first = unicode;
				else
				{
					unicode_to_utf8(unicode, (unsigned char *) out);
					out += pg_mblen(out);
				}
				in += 5;
			}
			else if (in[1] == '+' &&
					 isxdigit((unsigned char) in[2]) &&
					 isxdigit((unsigned char) in[3]) &&
					 isxdigit((unsigned char) in[4]) &&
					 isxdigit((unsigned char) in[5]) &&
					 isxdigit((unsigned char) in[6]) &&
					 isxdigit((unsigned char) in[7]))
			{
				pg_wchar	unicode;

				unicode = (hexval(in[2]) << 20) +
					(hexval(in[3]) << 16) +
					(hexval(in[4]) << 12) +
					(hexval(in[5]) << 8) +
					(hexval(in[6]) << 4) +
					hexval(in[7]);
				check_unicode_value(unicode, in, yyscanner);
				if (pair_first)
				{
					if (is_utf16_surrogate_second(unicode))
					{
						unicode = surrogate_pair_to_codepoint(pair_first, unicode);
						pair_first = 0;
					}
					else
					{
						ADVANCE_YYLLOC(in - litbuf + 3);		/* 3 for U&" */
						yyerror("invalid Unicode surrogate pair");
					}
				}
				else if (is_utf16_surrogate_second(unicode))
					yyerror("invalid Unicode surrogate pair");

				if (is_utf16_surrogate_first(unicode))
					pair_first = unicode;
				else
				{
					unicode_to_utf8(unicode, (unsigned char *) out);
					out += pg_mblen(out);
				}
				in += 8;
			}
			else
			{
				ADVANCE_YYLLOC(in - litbuf + 3);		/* 3 for U&" */
				yyerror("invalid Unicode escape value");
			}
		}
		else
		{
			if (pair_first)
			{
				ADVANCE_YYLLOC(in - litbuf + 3);		/* 3 for U&" */
				yyerror("invalid Unicode surrogate pair");
			}
			*out++ = *in++;
		}
	}

	/* unfinished surrogate pair? */
	if (pair_first)
	{
		ADVANCE_YYLLOC(in - litbuf + 3);				/* 3 for U&" */
		yyerror("invalid Unicode surrogate pair");
	}

	*out = '\0';

	/*
	 * We could skip pg_verifymbstr if we didn't process any non-7-bit-ASCII
	 * codes; but it's probably not worth the trouble, since this isn't likely
	 * to be a performance-critical path.
	 */
	pg_verifymbstr(new, out - new, false);
	return new;
}

static unsigned char
unescape_single_char(unsigned char c, core_yyscan_t yyscanner)
{
	switch (c)
	{
		case 'b':
			return '\b';
		case 'f':
			return '\f';
		case 'n':
			return '\n';
		case 'r':
			return '\r';
		case 't':
			return '\t';
		default:
			/* check for backslash followed by non-7-bit-ASCII */
			if (c == '\0' || IS_HIGHBIT_SET(c))
				yyextra->saw_non_ascii = true;

			return c;
	}
}

static void
check_string_escape_warning(unsigned char ychar, core_yyscan_t yyscanner)
{
	if (ychar == '\'')
	{
		if (yyextra->warn_on_first_escape && yyextra->escape_string_warning)
			ereport(WARNING,
					(errcode(ERRCODE_NONSTANDARD_USE_OF_ESCAPE_CHARACTER),
					 errmsg("nonstandard use of \\' in a string literal"),
					 errhint("Use '' to write quotes in strings, or use the escape string syntax (E'...')."),
					 lexer_errposition()));
		yyextra->warn_on_first_escape = false;	/* warn only once per string */
	}
	else if (ychar == '\\')
	{
		if (yyextra->warn_on_first_escape && yyextra->escape_string_warning)
			ereport(WARNING,
					(errcode(ERRCODE_NONSTANDARD_USE_OF_ESCAPE_CHARACTER),
					 errmsg("nonstandard use of \\\\ in a string literal"),
					 errhint("Use the escape string syntax for backslashes, e.g., E'\\\\'."),
					 lexer_errposition()));
		yyextra->warn_on_first_escape = false;	/* warn only once per string */
	}
	else
		check_escape_warning(yyscanner);
}

static void
check_escape_warning(core_yyscan_t yyscanner)
{
	if (yyextra->warn_on_first_escape && yyextra->escape_string_warning)
		ereport(WARNING,
				(errcode(ERRCODE_NONSTANDARD_USE_OF_ESCAPE_CHARACTER),
				 errmsg("nonstandard use of escape in a string literal"),
		errhint("Use the escape string syntax for escapes, e.g., E'\\r\\n'."),
				 lexer_errposition()));
	yyextra->warn_on_first_escape = false;		/* warn only once per string */
}

/*
 * Interface functions to make flex use palloc() instead of malloc().
 * It'd be better to make these static, but flex insists otherwise.
 */

void *
core_yyalloc(yy_size_t bytes, core_yyscan_t yyscanner)
{
	return palloc(bytes);
}

void *
core_yyrealloc(void *ptr, yy_size_t bytes, core_yyscan_t yyscanner)
{
	if (ptr)
		return repalloc(ptr, bytes);
	else
		return palloc(bytes);
}

void
core_yyfree(void *ptr, core_yyscan_t yyscanner)
{
	if (ptr)
		pfree(ptr);
}

